Package: ForceChoice
Title: Forced-Choice Modeling Based on Item Response Theory and
        Cognitive Diagnostic Models
Version: 1.0.0
Authors@R: c(person(given = "Haijiang", 
                    family = "Qin", 
                    role = c("aut", "cre", "cph"), 
                    email = "haijiang133@outlook.com", 
                    comment = c(ORCID = "0009-0000-6721-5653")),
             person(given = "Lei",
                    family = "Guo", 
                    role = c("aut", "cph"), 
                    email = "happygl1229@swu.edu.cn", 
                    comment = c(ORCID = "0000-0002-8273-3587")))
Description: Fits, simulates, and evaluates forced-choice and traditional
    item response theory (IRT) models for noncognitive assessment. Eight model
    families are supported, spanning dominance (multidimensional IRT (MIRT)
    1PL--4PL; multidimensional generalized partial credit model (MGPCM)),
    ideal-point unfolding (multidimensional generalized graded unfolding
    model (MGGUM)), and forced-choice designs (forced-choice multidimensional
    IRT (FCMIRT), forced-choice generalized graded unfolding model (FCGGUM),
    Thurstonian IRT (TIRT), forced-choice diagnostic classification model
    (FCDCM), forced-choice generalized deterministic inputs, noisy "and" gate
    model (FCGDINA)) that mitigate response biases such as acquiescence and
    social desirability. Core estimation backends include full Bayesian
    inference via Hamiltonian Monte Carlo (Stan) and a fast improved
    stochastic expectation-maximization (iStEM) algorithm suitable for
    large-scale data; FCGDINA also provides a deterministic
    expectation-maximization (EM) estimator. Comprehensive model evaluation
    uses the limited-information M2 family of goodness-of-fit statistics
    (Maydeu-Olivares and Joe, 2005 <doi:10.1198/016214504000002069>; 2006
    <doi:10.1007/s11336-005-1295-9>) together with root mean square error of
    approximation (RMSEA), comparative fit index (CFI), Tucker-Lewis index
    (TLI), and standardized root mean square residual (SRMSR).
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: yes
Config/ForceChoice/skip-rstantools-config: true
Biarch: true
Depends: R (>= 4.1.0)
Imports: coda, GPArotation, MASS, methods, numDeriv, parallel, Rcpp (>=
        0.12.0), RcppParallel (>= 5.0.1), rstan (>= 2.18.1), rstantools
        (>= 2.6.0)
LinkingTo: BH (>= 1.66.0), Rcpp (>= 0.12.0), RcppArmadillo, RcppEigen
        (>= 0.3.3.3.0), RcppParallel (>= 5.0.1), rstan (>= 2.18.1),
        StanHeaders (>= 2.18.0)
SystemRequirements: GNU make
Suggests: knitr, loo, rmarkdown
VignetteBuilder: knitr
Config/testthat/edition: 3
Packaged: 2026-07-22 03:06:49 UTC; Haijiang
Author: Haijiang Qin [aut, cre, cph] (ORCID:
    <https://orcid.org/0009-0000-6721-5653>),
  Lei Guo [aut, cph] (ORCID: <https://orcid.org/0000-0002-8273-3587>)
Maintainer: Haijiang Qin <haijiang133@outlook.com>
Repository: CRAN
Date/Publication: 2026-07-30 17:20:30 UTC

# Generated by roxygen2: do not edit by hand

S3method(coef,FCDCM)
S3method(coef,FCGDINA)
S3method(coef,FCGGUM)
S3method(coef,FCMIRT)
S3method(coef,MGGUM)
S3method(coef,MGPCM)
S3method(coef,MIRT)
S3method(coef,TIRT)
S3method(confint,FCDCM)
S3method(confint,FCGDINA)
S3method(confint,FCGGUM)
S3method(confint,FCMIRT)
S3method(confint,MGGUM)
S3method(confint,MGPCM)
S3method(confint,MIRT)
S3method(confint,TIRT)
S3method(deviance,FCDCM)
S3method(deviance,FCGDINA)
S3method(deviance,FCGGUM)
S3method(deviance,FCMIRT)
S3method(deviance,MGGUM)
S3method(deviance,MGPCM)
S3method(deviance,MIRT)
S3method(deviance,TIRT)
S3method(extract,FCDCM)
S3method(extract,FCGDINA)
S3method(extract,FCGGUM)
S3method(extract,FCMIRT)
S3method(extract,MGGUM)
S3method(extract,MGPCM)
S3method(extract,MIRT)
S3method(extract,TIRT)
S3method(fitted,FCDCM)
S3method(fitted,FCGDINA)
S3method(fitted,FCGGUM)
S3method(fitted,FCMIRT)
S3method(fitted,MGGUM)
S3method(fitted,MGPCM)
S3method(fitted,MIRT)
S3method(fitted,TIRT)
S3method(get.fit.index,FCDCM)
S3method(get.fit.index,FCGDINA)
S3method(get.fit.index,FCGGUM)
S3method(get.fit.index,FCMIRT)
S3method(get.fit.index,MGGUM)
S3method(get.fit.index,MGPCM)
S3method(get.fit.index,MIRT)
S3method(get.fit.index,TIRT)
S3method(get.log_lik,default)
S3method(logLik,FCDCM)
S3method(logLik,FCGDINA)
S3method(logLik,FCGGUM)
S3method(logLik,FCMIRT)
S3method(logLik,MGGUM)
S3method(logLik,MGPCM)
S3method(logLik,MIRT)
S3method(logLik,TIRT)
S3method(nobs,FCDCM)
S3method(nobs,FCGDINA)
S3method(nobs,FCGGUM)
S3method(nobs,FCMIRT)
S3method(nobs,MGGUM)
S3method(nobs,MGPCM)
S3method(nobs,MIRT)
S3method(nobs,TIRT)
S3method(plot,FCDCM)
S3method(plot,FCGDINA)
S3method(plot,FCGGUM)
S3method(plot,FCMIRT)
S3method(plot,MGGUM)
S3method(plot,MGPCM)
S3method(plot,MIRT)
S3method(plot,TIRT)
S3method(predict,FCDCM)
S3method(predict,FCGDINA)
S3method(predict,FCGGUM)
S3method(predict,FCMIRT)
S3method(predict,MGGUM)
S3method(predict,MGPCM)
S3method(predict,MIRT)
S3method(predict,TIRT)
S3method(print,FCDCM)
S3method(print,FCGDINA)
S3method(print,FCGGUM)
S3method(print,FCMIRT)
S3method(print,MGGUM)
S3method(print,MGPCM)
S3method(print,MIRT)
S3method(print,TIRT)
S3method(print,data.FCDCM)
S3method(print,data.FCGDINA)
S3method(print,data.FCGGUM)
S3method(print,data.FCMIRT)
S3method(print,data.MGGUM)
S3method(print,data.MGPCM)
S3method(print,data.MIRT)
S3method(print,data.TIRT)
S3method(print,good.of.fit)
S3method(print,summary.FCDCM)
S3method(print,summary.FCGDINA)
S3method(print,summary.FCGGUM)
S3method(print,summary.FCMIRT)
S3method(print,summary.MGGUM)
S3method(print,summary.MGPCM)
S3method(print,summary.MIRT)
S3method(print,summary.TIRT)
S3method(print,summary.good.of.fit)
S3method(residuals,FCDCM)
S3method(residuals,FCGDINA)
S3method(residuals,FCGGUM)
S3method(residuals,FCMIRT)
S3method(residuals,MGGUM)
S3method(residuals,MGPCM)
S3method(residuals,MIRT)
S3method(residuals,TIRT)
S3method(rotate,MGPCM)
S3method(rotate,MIRT)
S3method(summary,FCDCM)
S3method(summary,FCGDINA)
S3method(summary,FCGGUM)
S3method(summary,FCMIRT)
S3method(summary,MGGUM)
S3method(summary,MGPCM)
S3method(summary,MIRT)
S3method(summary,TIRT)
S3method(summary,good.of.fit)
S3method(update,FCDCM)
S3method(update,FCGDINA)
S3method(update,FCGGUM)
S3method(update,FCMIRT)
S3method(update,MGGUM)
S3method(update,MGPCM)
S3method(update,MIRT)
S3method(update,TIRT)
S3method(vcov,FCDCM)
S3method(vcov,FCGDINA)
S3method(vcov,FCGGUM)
S3method(vcov,FCMIRT)
S3method(vcov,MGGUM)
S3method(vcov,MGPCM)
S3method(vcov,MIRT)
S3method(vcov,TIRT)
export(extract)
export(fit.FCDCM)
export(fit.FCGDINA)
export(fit.FCGGUM)
export(fit.FCMIRT)
export(fit.MGGUM)
export(fit.MGPCM)
export(fit.MIRT)
export(fit.TIRT)
export(get.block.items.from.data)
export(get.block.items.from.data.FCDCM)
export(get.data.from.response)
export(get.data.from.response.FCDCM)
export(get.data.from.response.TIRT)
export(get.fit.index)
export(get.log_lik)
export(get.response.from.data)
export(get.response.from.data.FCDCM)
export(get.response.from.data.TIRT)
export(good.of.fit)
export(model.FCGDINA)
export(rotate)
export(sim.data.FCDCM)
export(sim.data.FCGDINA)
export(sim.data.FCGGUM)
export(sim.data.FCMIRT)
export(sim.data.MGGUM)
export(sim.data.MGPCM)
export(sim.data.MIRT)
export(sim.data.TIRT)
export(summary.good.of.fit)
import(Rcpp)
import(methods)
import(rstantools)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(rstan,sampling)
importFrom(stats,fitted)
importFrom(stats,logLik)
importFrom(stats,nobs)
importFrom(stats,plogis)
importFrom(stats,pnorm)
importFrom(stats,qlogis)
importFrom(stats,qnorm)
importFrom(stats,rlnorm)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,update)
importFrom(utils,combn)
importFrom(utils,flush.console)
importFrom(utils,tail)
useDynLib(ForceChoice, .registration = TRUE)

# ForceChoice

## Forced-Choice Modeling Based on Item Response Theory and Cognitive Diagnosis

The **ForceChoice** package provides a unified framework for fitting,
simulating, and evaluating forced-choice and traditional item response
models. It supports **eight model families** spanning dominance,
ideal-point/unfolding, and diagnostic classification paradigms, with both
full Bayesian estimation (Stan/HMC) and a fast iterative stochastic EM
(iStEM) algorithm. FCGDINA also provides a deterministic EM estimator.

The model descriptions below distinguish the package's implemented
parameterizations from the broader source literature. Core references are
given with DOI links where available so that the modeling assumptions,
response-process assumptions, and goodness-of-fit framework can be traced
back to peer-reviewed sources.

## Features

- **Eight model families**: MIRT, MGPCM, MGGUM, FCMIRT, FCGGUM, TIRT,
  FCDCM, FCGDINA
- **Estimation backends**: Stan (full Bayesian HMC), iStEM (fast stochastic
  EM for large data), and deterministic EM for FCGDINA
- **Comprehensive goodness-of-fit**: Limited-information M2, RMSEA, CFI, TLI,
  SRMSR, pseudo-*R*^2^, Yen's Q3, posterior diagnostics
- **Forced-choice response types**: Full ranking (RANK), most-least (MOLE),
  and best-only (PICK)
- **Consistent S3 interface**: `coef()`, `fitted()`, `logLik()`,
  `plot()`, `predict()`, `print()`, `residuals()`, `summary()`, `update()`,
  `vcov()`, `confint()`

## Installation

You can install the development version from GitHub:

```r
# install.packages("remotes")
remotes::install_github("Naidantu/ForceChoice")
```

## Quick Start

### Example 1: Multidimensional IRT (MIRT)

```r
library(ForceChoice)

# Simulate binary response data from a 2-dimensional 2PL model
sim <- sim.data.MIRT(N = 20, I = 6, D = 2, model = "2PL")

# Fit via iStEM (fast, scalable to large datasets)
fit <- fit.MIRT(sim$data, model = "2PL", D = 2, method = "iStEM",
                control.method = list(
                  vis = FALSE, seed = 123,
                  M = 2, B = 2, burnin.maxitr = 2,
                  maxitr = 3, eps1 = 10, eps2 = 10,
                  estimate.se = FALSE))

# Examine results
print(fit)
summary(fit)

# Item parameter estimates
head(coef(fit))

# Person trait estimates
head(fit$theta$est)

# Factor correlation
fit$Corr$est

# Goodness-of-fit evaluation
gof <- get.fit.index(fit)
summary(gof)
```

### Example 2: Forced-Choice MIRT (FCMIRT)

```r
# Simulate forced-choice ranking data
sim <- sim.data.FCMIRT(N.person = 20, N.block = 3, I.block = 2,
                       D = 2, model = "2PL", fc.type = "RANK")

# The block.items and fc.type are auto-detected from the data
fit <- fit.FCMIRT(sim$data, model = "2PL", D = 2,
                  method = "iStEM",
                  control.method = list(
                    vis = FALSE, seed = 123,
                    M = 2, B = 2, burnin.maxitr = 2,
                    maxitr = 3, eps1 = 10, eps2 = 10,
                    estimate.se = FALSE))

# Trait recovery: correlation between estimates and true values
diag(cor(fit$theta$est, sim$theta))

# Goodness-of-fit for forced-choice data
gof <- get.fit.index(fit)
summary(gof)
```

### Example 3: Thurstonian IRT (TIRT)

```r
# Simulate forced-choice data for TIRT
sim <- sim.data.TIRT(N.person = 20, N.block = 3, I.block = 2,
                     D = 2, fc.type = "RANK")

fit <- fit.TIRT(sim$data, Q.matrix = sim$Q.matrix,
                block.items = sim$block.items,
                method = "iStEM",
                control.method = list(
                  vis = FALSE, seed = 123,
                  M = 2, B = 2, burnin.maxitr = 2,
                  maxitr = 3, eps1 = 10, eps2 = 10,
                  estimate.se = FALSE))

# Trait recovery
cor(fit$theta$est, sim$theta)
```

### Example 4: Forced-Choice DCM (FCDCM)

```r
# Simulate paired-comparison data with DINA condensation rule
sim <- sim.data.FCDCM(N.person = 20, N.block = 3, D = 2,
                      dcm.type = "DINA")

# Fit
fit <- fit.FCDCM(sim$data, Q.matrix = sim$Q.matrix,
                 block.items = sim$block.items,
                 method = "iStEM",
                 control.method = list(
                   vis = FALSE, seed = 123,
                   M = 2, B = 2, burnin.maxitr = 2,
                   maxitr = 3, eps1 = 10, eps2 = 10,
                   estimate.se = FALSE))

# Posterior attribute mastery profiles
head(fit$alpha$est)

# Attribute mastery proportions
colMeans(fit$alpha$est > 0.5)
```

### Example 5: Forced-Choice GDINA (FCGDINA)

```r
# Simulate forced-choice diagnostic data under a CDM item model
sim <- sim.data.FCGDINA(N.person = 20, N.block = 2, I.block = 2,
                        D = 2, model = "GDINA", fc.type = "RANK")

# Fit with deterministic EM for a reproducible baseline
fit <- fit.FCGDINA(sim$data, Q.matrix = sim$Q.matrix,
                   block.items = sim$block.items, model = "GDINA",
                   fc.type = sim$fc.type, method = "EM",
                   control.method = list(vis = FALSE, seed = 123,
                                         maxitr = 2,
                                         estimate.se = FALSE))

head(fit$alpha$est)
```

## Model Overview

| Function | Model | Response Type | Key Reference |
|---|---|---|---|
| `fit.MIRT()` | Multidimensional IRT (1PL--4PL) | Binary | Reckase (2009) |
| `fit.MGPCM()` | Generalized Partial Credit | Polytomous | Muraki (1992) |
| `fit.MGGUM()` | Generalized Graded Unfolding | Polytomous | Roberts et al. (2000) |
| `fit.FCMIRT()` | Forced-Choice MIRT | Ranking/MOLE/PICK | Zheng et al. (2024); Luce (1959); Plackett (1975) |
| `fit.FCGGUM()` | Forced-Choice GGUM | Ranking/MOLE/PICK | Lee et al. (2018); Roberts et al. (2000) |
| `fit.TIRT()` | Thurstonian IRT | Ranking/MOLE/PICK | Brown & Maydeu-Olivares (2011) |
| `fit.FCDCM()` | Forced-Choice DCM | Paired comparison | Huang (2022) |
| `fit.FCGDINA()` | Forced-Choice GDINA | Ranking/MOLE/PICK | de la Torre (2011); Luce (1959); Plackett (1975) |

## Estimation Methods

### Stan (method = "stan")

Full Bayesian inference via Hamiltonian Monte Carlo (NUTS). Provides posterior
means, standard deviations, and R-hat convergence diagnostics. Recommended for
final inference and small-to-moderate datasets.

```r
# stan code, long time
fit <- fit.MIRT(data, model = "2PL", D = 2, method = "stan",
                control.method = list(chains = 1, iter = 200,
                                      warmup = 100, cores = 1,
                                      seed = 123))
```

### iStEM (method = "iStEM")

Iterative Stochastic EM with Metropolis-within-Gibbs person sampling and
L-BFGS-B item optimization. Scales to large datasets. Convergence monitored
via Geweke diagnostics and batch-means Monte Carlo error.

```r
fit <- fit.MIRT(data, model = "2PL", D = 2, method = "iStEM",
                control.method = list(
                  vis = FALSE, seed = 123,
                  M = 2, B = 2, burnin.maxitr = 2,
                  maxitr = 3, eps1 = 10, eps2 = 10,
                  estimate.se = FALSE))
```

### EM (method = "EM")

Deterministic posterior-weight EM is available for FCGDINA. It provides a
fast, reproducible point-estimation baseline without MCMC sampling.

## Goodness-of-Fit

All fitted models support comprehensive fit evaluation via `get.fit.index()`:

```r
gof <- get.fit.index(fit)
summary(gof)
```

Fit indices reported:
- **Information criteria**: AIC, AICc, BIC, CAIC, SABIC, HQIC
- **Absolute fit**: M2, RMSEA (with 90% CI), SRMSR
- **Comparative fit**: CFI, TLI, IFI
- **Pseudo-R2**: McFadden, Cox--Snell, Nagelkerke, and others
- **Local dependence**: Yen's Q3
- **Classification**: Posterior entropy, mean max posterior

## References

Brown, A., & Maydeu-Olivares, A. (2011). Item response modeling of
forced-choice questionnaires. *Educational and Psychological Measurement*,
*71*(3), 460--502. <https://doi.org/10.1177/0013164410375112>

de la Torre, J. (2011). The generalized DINA model framework.
*Psychometrika*, *76*(2), 179--199. <https://doi.org/10.1007/s11336-011-9207-7>

Huang, H.-Y. (2022). Diagnostic classification model for forced-choice
items and noncognitive tests. *Educational and Psychological Measurement*,
*83*(1), 146--180. <https://doi.org/10.1177/00131644211069906>

Lee, P., Joo, S.-H., Stark, S., & Chernyshenko, O. S. (2018). GGUM-RANK
statement and person parameter estimation with multidimensional forced
choice triplets. *Applied Psychological Measurement*, *43*(3), 226--240.
<https://doi.org/10.1177/0146621618768294>

Luce, R. D. (1959). *Individual choice behavior: A theoretical analysis*.
Wiley.

Maydeu-Olivares, A., & Joe, H. (2005). Limited- and full-information
estimation and goodness-of-fit testing in 2^n contingency tables: A unified
framework. *Journal of the American Statistical Association*, *100*(471),
1009--1020. <https://doi.org/10.1198/016214504000002069>

Maydeu-Olivares, A., & Joe, H. (2006). Limited information goodness-of-fit
testing in multidimensional contingency tables. *Psychometrika*, *71*(4),
713--732. <https://doi.org/10.1007/s11336-005-1295-9>

Muraki, E. (1992). A generalized partial credit model: Application of an EM
algorithm. *Applied Psychological Measurement*, *16*(2), 159--176.
<https://doi.org/10.1177/014662169201600206>

Plackett, R. L. (1975). The analysis of permutations. *Journal of the Royal
Statistical Society: Series C (Applied Statistics)*, *24*(2), 193--202.
<https://doi.org/10.2307/2346567>

Reckase, M. D. (2009). *Multidimensional Item Response Theory*. Springer.
<https://doi.org/10.1007/978-0-387-89976-3>

Roberts, J. S., Donoghue, J. R., & Laughlin, J. E. (2000). A general item
response theory model for unfolding unidimensional polytomous responses.
*Applied Psychological Measurement*, *24*(1), 3--32.
<https://doi.org/10.1177/01466216000241001>

Tu, N., Zhang, B., Angrave, L., Sun, T., & Neuman, M. (2023). Estimating the
multidimensional generalized graded unfolding model with covariates using a
Bayesian approach. *Journal of Intelligence*, *11*(8), 163.
<https://doi.org/10.3390/jintelligence11080163>

Zheng, C., Liu, J., Li, Y., Xu, P., Zhang, B., Wei, R., Zhang, W.,
Liu, B., & Huang, J. (2024). A 2PLM-RANK multidimensional forced-choice
model and its fast estimation algorithm. *Behavior Research Methods*,
*56*(6), 6363--6388. <https://doi.org/10.3758/s13428-023-02315-x>

Zhu, Y.-A., Xu, J., Wang, D., Li, X., Cai, Y., & Tu, D. (2024). A ranking
forced choice diagnostic classification model for psychological assessment
using forced choice questionnaires. *British Journal of Mathematical and
Statistical Psychology*, *78*(2), 617--646.
<https://doi.org/10.1111/bmsp.12376>

## License

GPL (>= 3)

#' Fit the Multidimensional Generalized Partial Credit Model (MGPCM)
#'
#' @description
#' Fits a multidimensional generalization of the partial credit model to
#' polytomous (multi-category) response data. Two estimation backends are
#' provided: full Bayesian inference via Hamiltonian Monte Carlo (Stan) and
#' a fast stochastic-EM algorithm (iStEM).
#'
#' @section Model Specification:
#'
#' Let \eqn{Y_{ij} \in \{0, 1, \dots, K_i - 1\}} denote the categorical
#' response of person \eqn{j = 1, \dots, N} to item \eqn{i = 1, \dots, I},
#' where \eqn{K_i \ge 2} is the number of response categories for item
#' \eqn{i}. Let \eqn{\boldsymbol{\theta}_j} be the \eqn{D}-dimensional
#' latent trait vector.
#'
#' The category response probability implemented in both the C++ and Stan
#' backends is a softmax over category scores:
#' \deqn{
#'   P(Y_{ij} = k \mid \boldsymbol{\theta}_j) =
#'   \frac{
#'     \exp\{k\,\eta_{ij} + d_{ik}\}
#'   }{
#'     \sum_{r=0}^{K_i-1} \exp\{r\,\eta_{ij} + d_{ir}\}
#'   },
#'   \qquad k = 0, 1, \dots, K_i - 1,
#' }
#' where
#' \deqn{
#'   \eta_{ij} = \sum_{d=1}^{D} a_{id}\,\theta_{jd}.
#' }
#' The discrimination parameters satisfy \eqn{a_{id} > 0} when
#' \eqn{q_{id} = 1} and \eqn{a_{id} = 0} when \eqn{q_{id} = 0}. The
#' \eqn{d_{ik}} values are category intercepts, not cumulative step
#' difficulties; \eqn{d_{i0} = 0} is fixed for identification and
#' \eqn{d_{i1}, \dots, d_{i,K_i-1}} are free.
#'
#' This formulation nests the standard (unidimensional) generalized partial
#' credit model (Muraki, 1992) when \eqn{D = 1} and all \eqn{q_{i1} = 1}.
#'
#' \strong{Prior distributions:}
#' \describe{
#'   \item{\eqn{a_{id}} (free)}{Log-normal: \eqn{\log a_{id} \sim N(\mu_a, \sigma_a^2)}}
#'   \item{\eqn{d_{ik}} (free)}{Normal:
#'         \eqn{d_{ik} \sim N(\mu_d, \sigma_d^2)} for \eqn{k \ge 1};
#'         \eqn{d_{i0}=0} is fixed. The same normal penalty is used by
#'         Stan and by the iStEM item update when \code{use.prior = TRUE}.}
#'   \item{\eqn{\boldsymbol{\theta}_j}}{Multivariate normal:
#'         \eqn{\boldsymbol{\theta}_j \sim N_D(\mathbf{0}, \boldsymbol{\Sigma})}}
#' }
#'
#' @section Estimation Methods:
#'
#' \describe{
#'   \item{\strong{Stan} (\code{method = "stan"}):}{
#'     Full Bayesian inference via HMC. The joint posterior is sampled
#'     with multiple chains, providing posterior means, standard deviations,
#'     and \eqn{\hat{R}} diagnostics.}
#'   \item{\strong{iStEM} (\code{method = "iStEM"}):}{
#'     Improved Stochastic EM alternating finite-grid block Gibbs
#'     person-sampling and item-parameter optimization (L-BFGS-B).
#'     Convergence monitored via Geweke diagnostics and batch-means MC error.}
#' }
#'
#' @param data An \eqn{N \times I} matrix of integer responses coded
#'   \eqn{0, 1, \dots, K_i - 1}. Rows index persons, columns index items.
#' @param D Integer; number of latent dimensions (\eqn{D \ge 1}).
#'   Default is \code{2}.
#' @param Q.matrix An optional \eqn{I \times D} binary matrix. Entry
#'   \eqn{q_{id} = 1} frees \eqn{a_{id}}; \eqn{q_{id} = 0} fixes it to 0.
#'   Default uses a triangular identification structure.
#' @param length.poly Optional integer scalar or length-\eqn{I} vector giving
#'   the number of categories per item. If \code{NULL}, counts are inferred
#'   from observed maxima; supply this argument when a valid category is
#'   unobserved in the sample.
#' @param method Estimation method: \code{"iStEM"} (default) or
#'   \code{"stan"}.
#' @param control.model A named list of model-level hyperparameters.
#'   Supported entries:
#'   \describe{
#'     \item{\code{a.mu}, \code{a.sigma}}{Prior location and scale for
#'           \eqn{\log a_{id}} (log-normal). Defaults: 0.25, 0.25.
#'           Controls the prior mean and spread of the discrimination
#'           parameters across dimensions.}
#'     \item{\code{d.mu}, \code{d.sigma}}{Prior mean and SD for free
#'           category intercepts \eqn{d_{ik}} (\eqn{k \ge 1}; normal).
#'           Defaults: 0, 1. The first category intercept
#'           \eqn{d_{i0}=0} is fixed for identification.}
#'     \item{\code{theta.mu}}{Prior mean vector for
#'           \eqn{\boldsymbol{\theta}_j}. Default: \code{rep(0, D)}.
#'           A vector of length \eqn{D} specifying the prior mean for
#'           each latent dimension.}
#'     \item{\code{L}}{Theta grid size per dimension for marginal
#'           log-likelihood computation and iStEM block Gibbs sampling.
#'           Default adapts to \eqn{D}
#'           (e.g., 61 for D = 1, 31 for D = 2, 15 for D = 3).
#'           Larger grids increase numerical precision at the cost of
#'           exponential growth in computation (\eqn{L^D} total nodes).}
#'     \item{\code{theta.lower}, \code{theta.upper}}{Bounds for the
#'           theta grid used by marginal log-likelihood computation and
#'           iStEM block Gibbs sampling. Defaults: -6, 6.}
#'   }
#' @param control.method A named list of method-specific tuning parameters.
#'   Common entries (used by both Stan and iStEM):
#'   \describe{
#'     \item{\code{cores}}{Number of CPU cores for parallel chains
#'           (Stan) or ignored (iStEM). Default: the number of
#'           \code{chains}.}
#'     \item{\code{vis}}{Logical; if \code{TRUE} (default), prints progress
#'           information to the console.}
#'     \item{\code{seed}}{Random seed for reproducibility.
#'           Default: a random integer.}
#'   }
#'   Stan-specific entries:
#'   \describe{
#'     \item{\code{chains}}{Number of MCMC chains (default: 2).}
#'     \item{\code{iter}}{Total iterations per chain (default: 5000).}
#'     \item{\code{warmup}}{Warmup/burn-in iterations per chain
#'           (default: \code{iter / 2}).}
#'     \item{\code{thin}}{Thinning interval (default: 1).}
#'     \item{\code{init}}{Initial values: \code{"random"} (default)
#'           for uniform(-2, 2) initialization, or a list of initial
#'           values per chain.}
#'     \item{\code{algorithm}}{MCMC algorithm: \code{"HMC"} (default),
#'           \code{"HMC"}, or \code{"Fixed_param"}.}
#'     \item{\code{adapt_delta}}{Target average acceptance probability
#'           (NUTS; default: 0.95). Values closer to 1 reduce step size
#'           and improve sampling for difficult posteriors.}
#'     \item{\code{max_treedepth}}{Maximum tree depth (NUTS; default: 10).
#'           Increase if "max treedepth exceeded" warnings appear.}
#'     \item{\code{stepsize}}{Initial step size for the leapfrog
#'           integrator (auto-tuned by Stan if not set).}
#'     \item{\code{int_time}}{Total integration time for HMC trajectories
#'           (only when \code{algorithm = "HMC"}).}
#'     \item{\code{metric}}{Mass matrix type: \code{"unit_e"},
#'           \code{"diag_e"} (default), or \code{"dense_e"}.}
#'     \item{\code{adapt_engaged}}{Logical; if \code{TRUE} (default),
#'           warmup adaptation is enabled.}
#'     \item{\code{adapt_init_buffer}, \code{adapt_term_buffer},
#'           \code{adapt_window}}{Warmup adaptation scheduling parameters
#'           (defaults: 25, 50, 25).}
#'   }
#'   iStEM-specific entries:
#'   \describe{
#'     \item{\code{M}}{Number of burn-in batches retained for Geweke
#'           convergence diagnosis (default: 10; must be \eqn{\ge 2}).}
#'     \item{\code{B}}{Batch size: MCMC iterations per batch (default: 20).}
#'     \item{\code{burnin.maxitr}}{Maximum burn-in batches (default: 100).}
#'     \item{\code{maxitr}}{Maximum total batches (default: 2000).}
#'     \item{\code{eps1}}{Geweke z-score convergence threshold
#'           (default: 1.5).}
#'     \item{\code{eps2}}{Monte Carlo error tolerance (default: 0.4).}
#'     \item{\code{frac1}, \code{frac2}}{Fractions for the Geweke
#'           diagnostic (defaults: 0.1, 0.5).}
#'     \item{\code{corr.optim.maxit}}{Maximum L-BFGS-B iterations for the
#'           constrained unit-diagonal correlation update (default: 50).}
#'     \item{\code{optim.maxit}}{Maximum L-BFGS-B iterations per item
#'           (default: 50).}
#'     \item{\code{fix.corr}}{Logical; fix correlations to identity
#'           (default: \code{FALSE}).}
#'     \item{\code{estimate.se}}{Logical; compute standard errors from
#'           final MC chain (default: \code{TRUE}).}
#'     \item{\code{a.lower}, \code{a.upper}}{Bounds on \eqn{a_{id}}
#'           (defaults: 1e-4, 6).}
#'     \item{\code{d.lower}, \code{d.upper}}{Bounds on free category
#'           intercepts \eqn{d_{ik}} for \eqn{k \ge 1}. Defaults are -8
#'           and 8 in the iStEM backend.}
#'   }
#'
#' @return An object of class \code{"MGPCM"} with components:
#' \describe{
#'   \item{\code{npar}}{Number of free parameters.}
#'   \item{\code{method}}{\code{"stan"} or \code{"iStEM"}.}
#'   \item{\code{theta}}{List with \code{est}, \code{se}, \code{Rhat}
#'         (\eqn{N \times D}).}
#'   \item{\code{par}}{List with \code{est}, \code{se}, \code{Rhat},
#'         \code{free} (\eqn{I \times (D + K_{max})}). Columns are
#'         \code{a1..aD, d0, d1, ..., d_{K_{max}-1}}.}
#'   \item{\code{Corr}}{List with \code{est}, \code{se}, \code{Rhat}
#'         (\eqn{D \times D}).}
#'   \item{\code{length.poly}}{Integer vector of per-item category counts.}
#'   \item{\code{logLik}}{Marginal log-likelihood (class \code{"logLik"}).}
#'   \item{\code{call}, \code{arguments}}{Call and argument records.}
#' }
#'
#' @references
#' Muraki, E. (1992). A generalized partial credit model: Application of an
#'   EM algorithm. \emph{Applied Psychological Measurement}, 16(2), 159--176.
#'   \doi{10.1177/014662169201600206}
#'
#' Yao, L., & Schwarz, R. D. (2006). A multidimensional partial credit model
#'   for polytomous data. \emph{Applied Psychological Measurement}, 30(4),
#'   295--318.
#'
#' @seealso
#' \code{\link{sim.data.MGPCM}}, \code{\link{get.fit.index.MGPCM}},
#' \code{\link{logLik.MGPCM}}, \code{\link{rotate}}
#'
#' @examples
#' sim <- sim.data.MGPCM(N = 20, I = 6, D = 2, length.poly = 4)
#' fit <- fit.MGPCM(sim$response, D = 2, method = "iStEM",
#'                  control.method = list(
#'                    vis = FALSE, seed = 123,
#'                    M = 2, B = 2, burnin.maxitr = 2,
#'                    maxitr = 3, eps1 = 10, eps2 = 10,
#'                    estimate.se = FALSE))
#' head(fit$theta$est)
#' fit$par$est[1:5, ]
#' gof <- get.fit.index(fit)
#' summary(gof)
#'
#' @export
fit.MGPCM <- function(data, D = NULL, Q.matrix = NULL, length.poly = NULL,
                      method = c("iStEM", "stan"),
                      control.model = NULL,
                      control.method = NULL) {

  call <- match.call()
  method <- match.arg(method)
  data <- istem_prepare_response(data, binary = FALSE)
  length.poly <- istem_prepare_length_poly(data, length.poly)
  D <- resolve_D(D, Q.matrix = Q.matrix, I = ncol(data))
  Q.matrix <- istem_prepare_01_q(
    I = ncol(data), D = D, Q.matrix = Q.matrix, triangular = TRUE
  )
  control.model  <- fc_as_control_list(control.model, "control.model")
  control.method <- fc_as_control_list(control.method, "control.method")

  if (method == "stan") {
    return(fit.MGPCM.stan(
      response = data,
      D = D,
      Q.matrix = Q.matrix,
      length.poly = length.poly,
      control.model = control.model,
      control.method = control.method,
      .call = call
    ))
  }

  fit.MGPCM.iStEM(
    response = data,
    D = D,
    Q.matrix = Q.matrix,
    length.poly = length.poly,
    control.model = control.model,
    control.method = control.method,
    .call = call
  )
}

#' @describeIn get.fit.index FCDCM model: binary block responses with
#'   quadrature over higher-order \eqn{\theta} and exact marginalization
#'   over \eqn{2^D} attribute patterns.
#' @export
get.fit.index.FCDCM <- function(object, ...) {
  response <- object$response
  D <- object$arguments$D
  B <- ncol(response)
  npar <- object$npar

  logLik.obj <- object$logLik
  pi <- as.vector(attr(logLik.obj, "pi"))
  theta.norm <- attr(logLik.obj, "theta.norm")
  pi.theta <- as.vector(attr(logLik.obj, "pi.theta"))
  if (length(pi.theta) == 0L) {
    pi.theta <- stats::dnorm(theta.norm[, 1L])
    pi.theta <- pi.theta / sum(pi.theta)
  }

  par.vec <- fcdcm_free_par_vec(
    delta1 = object$delta$est[, "delta1"],
    delta0 = object$delta$est[, "delta0"],
    par = object$par$est
  )

  loglik.fun <- function(par.vec, ...) {
    pars <- fcdcm_expand_free_par(
      par.vec, D = D, B = B,
      dim.names = rownames(object$delta$est),
      block.names = rownames(object$par$est),
      par.names = colnames(object$par$est)
    )
    support <- fcdcm_latent_support(
      theta = theta.norm,
      pi = pi.theta,
      delta1 = pars$delta1,
      delta0 = pars$delta0,
      par = pars$par,
      alpha.patterns = object$alpha.patterns,
      zeta.patterns = object$zeta.patterns,
      patterns = object$patterns
    )
    cpp_loglik_binary(
      prob = support$prob,
      response = matrix(as.integer(response), nrow = nrow(response), ncol = B),
      pi = support$pi
    )$logLik
  }

  prob.fun <- function(par.vec, ...) {
    pars <- fcdcm_expand_free_par(
      par.vec, D = D, B = B,
      dim.names = rownames(object$delta$est),
      block.names = rownames(object$par$est),
      par.names = colnames(object$par$est)
    )
    fcdcm_latent_support(
      theta = theta.norm,
      pi = pi.theta,
      delta1 = pars$delta1,
      delta0 = pars$delta0,
      par = pars$par,
      alpha.patterns = object$alpha.patterns,
      zeta.patterns = object$zeta.patterns,
      patterns = object$patterns
    )$prob
  }

  pi.fun <- function(par.vec, ...) {
    pars <- fcdcm_expand_free_par(
      par.vec, D = D, B = B,
      dim.names = rownames(object$delta$est),
      block.names = rownames(object$par$est),
      par.names = colnames(object$par$est)
    )
    fcdcm_latent_support(
      theta = theta.norm,
      pi = pi.theta,
      delta1 = pars$delta1,
      delta0 = pars$delta0,
      par = pars$par,
      alpha.patterns = object$alpha.patterns,
      zeta.patterns = object$zeta.patterns,
      patterns = object$patterns
    )$pi
  }

  good.of.fit(
    par.vec = par.vec,
    loglik.fun = loglik.fun,
    prob.fun = prob.fun,
    response = response,
    npar = npar,
    pi = pi,
    pi.fun = pi.fun,
    response.type = "binary",
    ...
  )
}

# ---- Control helpers ----------------------------------------------------------

get_ctrl <- function(name, default, control) {
  if (!is.null(control[[name]])) control[[name]] else default
}


# ---- Array helpers ------------------------------------------------------------

ensure_3d <- function(x) {
  if (is.null(x)) return(NULL)
  nd <- length(dim(x))
  if (nd == 0L) {
    # 1D vector -> (iter, 1, 1)
    dim(x) <- c(length(x), 1L, 1L)
  } else if (nd == 2L) {
    # 2D matrix  -> (iter, rows, 1)
    dim(x) <- c(dim(x), 1L)
  }
  # nd == 3L: already correct, return as-is
  return(x)
}


# ---- FC type normalisation ----------------------------------------------------

#' Normalise and recycle forced-choice type vector
#'
#' @param fc.type Character vector (or \code{NULL}) describing the FC format of
#'   each block.  Recycled to length \code{N.block} and uppercased.
#' @param N.block Number of blocks.
#' @param default Default FC type used when \code{fc.type} is \code{NULL}.
#' @return A character vector of length \code{N.block}.
#' @keywords internal
normalize_fc_type <- function(fc.type, N.block, default = "RANK") {
  N.block <- check_integer_scalar(N.block, "N.block")
  if (is.null(fc.type)) fc.type <- default
  if (length(fc.type) == 1L) {
    fc.type <- rep(fc.type, N.block)
  } else if (length(fc.type) != N.block) {
    stop("'fc.type' must have length 1 or the number of blocks (",
         N.block, ").", call. = FALSE)
  }
  fc.type <- toupper(trimws(as.character(fc.type)))
  valid <- c("RANK", "MOLE", "PICK")
  if (anyNA(fc.type) || any(!fc.type %in% valid)) {
    stop("'fc.type' must contain only RANK, MOLE, or PICK.",
         call. = FALSE)
  }
  fc.type
}


# ---- Permutations  ------------------------------------------------------------

get_permutations <- function(vector, r) {
  n <- length(vector)
  if (r > n) stop("Selected element count 'r' cannot exceed the vector length.", call. = FALSE)
  vector <- as.integer(vector)
  if (anyNA(vector)) stop("'vector' must not contain NA values.", call. = FALSE)
  cpp_get_permutations(vector, as.integer(r))
}


# ---- Correlation matrix validation -------------------------------------------

#' Validate a user-supplied correlation matrix
#'
#' Checks dimensions, symmetry, unit diagonal, and positive semi-definiteness.
#' Returns a valid correlation matrix (or \code{diag(D)} when \code{NULL}).
#'
#' @param Corr A D x D matrix or \code{NULL}.
#' @param D Expected number of dimensions.
#' @param label Character label for error messages.
#' @return A valid D x D correlation matrix.
#' @keywords internal
validate_corr_matrix <- function(Corr, D, label = "'Corr'") {
  if (is.null(Corr)) return(diag(D))
  if (!is.matrix(Corr) || nrow(Corr) != D || ncol(Corr) != D)
    stop(label, " must be a ", D, "x", D, " matrix", call. = FALSE)
  if (!isSymmetric(unname(Corr), tol = 1e-8))
    stop(label, " must be symmetric", call. = FALSE)
  if (any(abs(diag(Corr) - 1) > 1e-8))
    stop(label, " must be a correlation matrix with diagonal 1", call. = FALSE)
  eig <- eigen(Corr, symmetric = TRUE, only.values = TRUE)$values
  if (any(eig < -1e-10))
    stop(label, " is not positive semi-definite", call. = FALSE)
  Corr
}


# ---- Rhat extraction ----------------------------------------------------------

#' Extract R-hat values for a matrix parameter from a Stan summary
#'
#' @param stan.sum Stan summary matrix (rownames encode parameter indices).
#' @param param_name Parameter name prefix (e.g. \code{"theta"}).
#' @param nrow,ncol Dimensions of the output matrix.
#' @return A numeric matrix of R-hat values.
#' @keywords internal
extract_rhat_matrix <- function(stan.sum, param_name, nrow, ncol) {
  cpp_extract_rhat_matrix(
    rhat       = stan.sum[, "Rhat"],
    row_names  = rownames(stan.sum),
    param_name = param_name,
    nrow_out   = as.integer(nrow),
    ncol_out   = as.integer(ncol)
  )
}

#' Extract R-hat values for a vector parameter from a Stan summary
#'
#' @param stan.sum Stan summary matrix.
#' @param param_name Parameter name prefix (e.g. \code{"delta1"}).
#' @param n Expected length of the output vector.
#' @return A numeric vector of R-hat values (NA where not found).
#' @keywords internal
extract_rhat_vector <- function(stan.sum, param_name, n) {
  cpp_extract_rhat_vector(
    rhat       = stan.sum[, "Rhat"],
    row_names  = rownames(stan.sum),
    param_name = param_name,
    n_out      = as.integer(n)
  )
}


# ---- Parameter-counting helpers -----------------------------------------------

#' Degrees of freedom for a correlation matrix
#'
#' @param D Number of dimensions.
#' @param include_corr Logical; if \code{FALSE} returns 0.
#' @return Integer, the number of free parameters in a D x D correlation matrix.
#' @keywords internal
free_corr_npar <- function(D, include_corr = TRUE) {
  if (isTRUE(include_corr) && D > 1L) D * (D - 1L) / 2L else 0L
}

#' Free-parameter mask for MIRT models
#'
#' @param model_type Integer 2, 3, or 4.
#' @param Q.matrix I x D Q-matrix.
#' @return An I x (D+3) logical matrix.
#' @keywords internal
mirt_par_free_mask <- function(model_type, Q.matrix) {
  I <- nrow(Q.matrix)
  D <- ncol(Q.matrix)
  free <- matrix(FALSE, I, D + 3L)
  if (model_type >= 2L) {
    free[, seq_len(D)] <- Q.matrix == 1
  }
  free[, D + 1L] <- TRUE                                # b always free
  if (model_type >= 3L) {
    free[, D + 2L] <- TRUE                              # c
  }
  if (model_type == 4L) {
    free[, D + 3L] <- TRUE                              # d
  }
  colnames(free) <- c(paste0("a", seq_len(D)), "b", "c", "d")
  free
}

#' Free-parameter mask for MGPCM models
#'
#' @param Q.matrix I x D Q-matrix.
#' @param length.poly Integer vector of category counts per item.
#' @param max_poly Maximum number of categories (default \code{max(length.poly)}).
#' @return An I x (D + max_poly) logical matrix.
#' @keywords internal
mgpcm_par_free_mask <- function(Q.matrix, length.poly,
                                max_poly = max(length.poly)) {
  I <- nrow(Q.matrix)
  D <- ncol(Q.matrix)
  free <- matrix(FALSE, I, D + max_poly)
  free[, seq_len(D)] <- Q.matrix == 1
  for (i in seq_len(I)) {
    Ki <- length.poly[i]
    if (Ki > 1L) {
      free[i, D + 2:Ki] <- TRUE
    }
  }
  colnames(free) <- c(paste0("a", seq_len(D)),
                      paste0("d", 0:(max_poly - 1L)))
  free
}

#' Free-parameter mask for MGGUM models
#'
#' @param Q.matrix I x D Q-matrix.
#' @param length.poly Integer vector of category counts per item.
#' @param max_poly Maximum number of categories.
#' @return An I x (2D + max_poly) logical matrix.
#' @keywords internal
mggum_par_free_mask <- function(Q.matrix, length.poly,
                                max_poly = max(length.poly)) {
  I <- nrow(Q.matrix)
  D <- ncol(Q.matrix)
  free <- matrix(FALSE, I, D + D + max_poly)
  active <- Q.matrix != 0
  free[, seq_len(D)]         <- active
  free[, D + seq_len(D)]     <- active
  for (i in seq_len(I)) {
    Ki <- length.poly[i]
    if (Ki > 1L) {
      free[i, D + D + 2:Ki] <- TRUE
    }
  }
  colnames(free) <- c(
    paste0("a",     seq_len(D)),
    paste0("delta", seq_len(D)),
    paste0("tau",   0:(max_poly - 1L))
  )
  free
}

#' Free-parameter mask for TIRT models
#'
#' @param Q.matrix I x D Q-matrix.
#' @param lambda.free Logical vector of length I.
#' @param psi.free Logical vector of length I.
#' @return An I x (D+1) logical matrix.
#' @keywords internal
tirt_par_free_mask <- function(Q.matrix, lambda.free, psi.free) {
  I <- nrow(Q.matrix)
  D <- ncol(Q.matrix)
  free <- matrix(FALSE, I, D + 1L)
  for (i in seq_len(I)) {
    if (isTRUE(lambda.free[i])) {
      active <- which(Q.matrix[i, ] != 0)
      if (length(active) > 0L) {
        free[i, active[1L]] <- TRUE
      }
    }
  }
  free[, D + 1L] <- psi.free
  colnames(free) <- c(paste0("Dim.", seq_len(D)), "psi2")
  free
}

#' Fit MGPCM Model via Stan
#'
#' @param response An N x I matrix of integer responses (0, 1, ..., K_i - 1)
#' @param D Number of latent dimensions
#' @param Q.matrix An I x D 0/1 matrix indicating which discrimination parameters
#'   are freely estimated (1) and which are fixed to 0 (default: NULL, builds
#'   a triangular identification structure).
#' @param length.poly Optional category counts for the items.
#' @param control.model Optional named list of model-level controls and
#'   hyperparameters.
#' @param control.method Optional named list of method-specific controls. For
#'   Stan this includes \code{cores}, \code{vis}, \code{seed}, \code{chains},
#'   \code{iter}, \code{warmup}, \code{thin}, \code{init}, \code{algorithm},
#'   and Stan sampler controls.
#'
#' @noRd
fit.MGPCM.stan <- function(response, D = 2, Q.matrix = NULL,
                       length.poly = NULL,
                       control.model = NULL,
                       control.method = NULL,
                       .call = NULL) {
  call <- if (is.null(.call)) match.call() else .call
  control <- fc_as_control_list(control.model, "control.model")
  control.method <- fc_as_control_list(control.method, "control.method")
  common.method <- fit_common_method_control(control.method)
  stan.method <- fc_stan_method_control(control.method)
  chains <- stan.method$chains
  iter <- stan.method$iter
  warmup <- stan.method$warmup
  thin <- stan.method$thin
  init <- stan.method$init
  algorithm <- stan.method$algorithm

  sm <- stanmodels[["MGPCM"]]

  if (is.null(sm)) {
    stop("Stan model MGPCM not found !", call. = FALSE)
  }

  mcmc_env <- setup_mcmc_env(common.method$cores)
  on.exit(restore_mcmc_env(mcmc_env), add = TRUE)

  stan_control <- build_stan_control(algorithm, control.method)

  N <- nrow(response)
  I <- ncol(response)

  response <- istem_prepare_response(response, binary = FALSE)
  length.poly <- istem_prepare_length_poly(response, length.poly)
  max_poly <- max(length.poly)

  Q.matrix <- istem_prepare_01_q(I, D, Q.matrix, triangular = TRUE)

  corr_df <- free_corr_npar(D)

  # Convert response to long format
  data.long <- data.frame(
    y   = as.vector(response),
    pid = rep(1:N, times = I),
    iid = rep(1:I, each = N)
  )

  theta_mu_raw <- get_ctrl("theta.mu", rep(0, D), control)
  theta_mu     <- array(theta_mu_raw, dim = D)

  stan.data <- list(
    N_obs       = nrow(data.long),
    N           = N,
    I           = I,
    D           = D,
    max_poly    = max_poly,
    y           = data.long$y,
    pid         = data.long$pid,
    iid         = data.long$iid,
    length_poly = as.integer(length.poly),
    Q_matrix    = Q.matrix,
    a_mu        = get_ctrl("a.mu", 0.25, control),
    a_sigma     = get_ctrl("a.sigma", 0.25, control),
    d_mu        = get_ctrl("d.mu", 0.0, control),
    d_sigma     = get_ctrl("d.sigma", 1.0, control),
    theta_mu    = theta_mu
  )

  set.seed(common.method$seed)

  stan.obj <- suppressWarnings(
    rstan::sampling(
      object        = sm,
      data          = stan.data,
      chains        = chains,
      iter          = iter,
      warmup        = warmup,
      thin          = thin,
      init          = init,
      algorithm     = algorithm,
      cores         = mcmc_env$cores,
      control       = stan_control,
      verbose       = common.method$vis,
      show_messages = common.method$vis
    )
  )

  MCMC.obj <- rstan::extract(stan.obj, permuted = TRUE)
  stan.sum <- rstan::summary(stan.obj)$summary

  # ---- Extract theta ----
  theta_est <- extract_theta_stan(MCMC.obj, stan.sum, N, D, rownames(response))

  # ---- Build par matrix: I x (D + max_poly) ----
  # Columns: a1 ... aD, d0, d1, ..., d_{max_poly-1}
  par      <- matrix(NA_real_, nrow = I, ncol = D + max_poly)
  par.se   <- matrix(NA_real_, nrow = I, ncol = D + max_poly)
  par.Rhat <- matrix(NA_real_, nrow = I, ncol = D + max_poly)

  # Extract a parameters
  a_arr  <- ensure_3d(MCMC.obj$a)
  a      <- apply(a_arr, c(2, 3), mean)
  a.se   <- apply(a_arr, c(2, 3), sd)

  par[, 1:D]      <- a
  par.se[, 1:D]   <- a.se
  par.Rhat[, 1:D] <- extract_rhat_matrix(stan.sum, "a", I, D)

  # d0 = 0 (fixed for all items)
  par[, D + 1]      <- 0
  par.se[, D + 1]   <- NA_real_
  par.Rhat[, D + 1] <- NA_real_

  # Extract free d parameters and map back to par matrix
  d_free      <- apply(MCMC.obj$d_free, 2, mean)
  d_free.se   <- apply(MCMC.obj$d_free, 2, sd)
  d_free.idx  <- grep("^d_free\\[", rownames(stan.sum))

  d_pos <- 1  # position in d_free vector
  for (i in 1:I) {
    Ki <- length.poly[i]
    if (Ki > 1) {
      n_d_i <- Ki - 1
      cols <- (D + 2):(D + Ki)
      rows_idx <- d_pos:(d_pos + n_d_i - 1)
      par[i, cols]      <- d_free[rows_idx]
      par.se[i, cols]   <- d_free.se[rows_idx]
      par.Rhat[i, cols] <- stan.sum[d_free.idx[rows_idx], "Rhat"]
      d_pos <- d_pos + n_d_i
    }
  }

  par.free <- mgpcm_par_free_mask(Q.matrix, length.poly, max_poly)
  colnames(par) <- colnames(par.se) <- colnames(par.Rhat) <- colnames(par.free) <-
    c(paste0("a", 1:D), paste0("d", 0:(max_poly - 1)))
  rownames(par) <- rownames(par.se) <- rownames(par.Rhat) <- rownames(par.free) <-
    colnames(response)
  par.se[!par.free] <- NA_real_
  par.Rhat[!par.free] <- NA_real_
  npar <- sum(par.free) + corr_df

  # ---- Extract Corr ----
  Corr_est <- extract_corr_stan(MCMC.obj, stan.sum, D)

  # ---- Assemble results ----
  par_est   <- list(est = par, se = par.se, Rhat = par.Rhat, free = par.free)

  results <- list(
    npar     = npar,
    method   = "stan",
    theta    = theta_est,
    par      = par_est,
    Corr     = Corr_est,
    stan.obj = stan.obj,
    MCMC.obj = MCMC.obj,
    log_lik = MCMC.obj$log_lik,
    Q.matrix    = Q.matrix,
    length.poly = length.poly,
    call        = call,
    arguments = list(
      response  = response,
      D         = D,
      Q.matrix  = Q.matrix,
      length.poly = length.poly,
      method    = "stan",
      chains    = chains,
      iter      = iter,
      warmup    = warmup,
      thin      = thin,
      init      = init,
      algorithm = algorithm,
      cores     = common.method$cores,
      vis       = common.method$vis,
      seed      = common.method$seed,
      control.model = control,
      control.method = fit_effective_method_control(
        control.method, method = stan.method, common = common.method
      )
    )
  )

  class(results) <- "MGPCM"

  L <- get_ctrl("L", NULL, control)
  results$logLik <- logLik.MGPCM(results, L=L)

  return(results)
}

#' Simulate Data from the Thurstonian IRT (TIRT) Model
#'
#' @description
#' Generates forced-choice pairwise comparison data under the Thurstonian
#' IRT framework. Each statement loads on exactly one latent dimension;
#' pairwise latent differences with probit link determine binary outcomes.
#' Supports RANK, MOLE, and PICK response types. For each pair \eqn{(i,k)},
#' the simulated comparison uses
#' \eqn{-\gamma_{ik} + \lambda_i q_i'\theta_j -
#' \lambda_k q_k'\theta_j + \epsilon_i - \epsilon_k}, with
#' \eqn{\epsilon_i \sim N(0,\psi_i^2)}.
#'
#' @param N.person Integer; number of persons (default: 1000).
#' @param N.block Integer; number of blocks (default: 10).
#' @param I.block Integer; items per block (default: 2).
#' @param D Integer; number of latent dimensions (default: 3, >= 2).
#' @param fc.type Character; \code{"RANK"} (default), \code{"MOLE"}, or
#'   \code{"PICK"}.
#' @param control Optional list with entries \code{Q.matrix}, \code{block.items},
#'   \code{lambda}, \code{psi2}, \code{gamma.matrix}, \code{gamma.item},
#'   \code{Corr}.
#'
#' @return An object of class \code{"data.TIRT"} containing \code{data},
#'   \code{response}, \code{theta}, \code{par}, \code{Q.matrix},
#'   \code{block.items}, \code{Corr}, \code{gamma.matrix}, \code{lambda},
#'   \code{psi2}, \code{pairs.value}, and data-generating arguments.
#'
#' @seealso \code{\link{fit.TIRT}}, \code{\link{model.TIRT}}
#'
#' @examples
#' set.seed(123)
#' sim <- sim.data.TIRT(N.person = 20, N.block = 3, I.block = 2,
#'                      D = 2, fc.type = "RANK")
#' str(sim$data)
#' head(sim$response)
#' dim(sim$Q.matrix)
#'
#' @export
sim.data.TIRT <- function(N.person = 1000, N.block = 10, I.block = 2, D = 3,
                          fc.type = "RANK", control = NULL) {

  call <- match.call()

  if (D < 2) {
    stop("TIRT requires at least 2 latent dimensions (D >= 2).", call. = FALSE)
  }

  if (is.null(control)) control <- list()

  # ---- Scalar integer validation ----
  N.person <- check_integer_scalar(N.person, "N.person", 1L)
  D        <- check_integer_scalar(D,        "D",        2L)
  I.block  <- check_integer_scalar(I.block,  "I.block",  2L)
  N.block  <- check_integer_scalar(N.block,  "N.block",  1L)

  # ---- Q-matrix and block configuration ----
  Q.matrix    <- control$Q.matrix
  block.items <- control$block.items

  res <- resolve_fc_blocks(
    Q.matrix       = Q.matrix,
    block.items    = block.items,
    I.block        = I.block,
    N.block        = N.block,
    D              = D,
    q.valid.values = c(-1, 0, 1),
    q.row.check    = function(r) sum(abs(r)) == 1L
  )
  I.states   <- res$I.states
  N.block    <- res$N.block
  I.block    <- res$I.block
  block.items <- res$block.items
  Q.matrix   <- res$Q.matrix
  D          <- res$D

  if (D < 2) {
    stop("TIRT requires at least 2 latent dimensions (D >= 2).", call. = FALSE)
  }

  # ---- fc.type validation ----
  fc.type <- normalize_fc_type(fc.type, N.block)

  # ---- Corr and theta ----
  Corr  <- validate_corr_matrix(control$Corr, D)
  theta <- generate_theta_mvn(N.person, D, Corr)

  # ---- Auto-generate Q-matrix if not supplied ----
  if (is.null(Q.matrix)) {
    Q.matrix <- sim.Q.matrix.FC(I.states = I.states, D = D,
                                block.items = block.items,
                                allow.negative = TRUE)
  }

  # ---- TIRT-specific: lambda, psi2, gamma ----
  lambda <- control$lambda
  psi2   <- control$psi2
  gamma.matrix <- control$gamma.matrix

  last_item_in_block <- as.integer(vapply(block.items, function(x) x[length(x)], numeric(1L)))

  if (is.null(lambda)) {
    lambda <- runif(I.states, 0.40, 0.90)
    if (D == 2 && length(I.block) == 1L && I.block == 2) {
      first_pair_items <- block.items[[1]]
      lambda[first_pair_items] <- rep(0.8, length(first_pair_items))
    }
  } else {
    if (length(lambda) != I.states) stop("Length of 'lambda' must be ", I.states, call. = FALSE)
    if (D == 2 && length(I.block) == 1L && I.block == 2) {
      first_pair_items <- block.items[[1]]
      lambda[first_pair_items] <- rep(0.8, length(first_pair_items))
    }
  }

  is_case_a <- (length(I.block) == 1L && I.block == 2 && D > 2)
  is_case_b <- (D == 2 && length(I.block) == 1L && I.block == 2)

  if (is.null(psi2)) {
    psi2 <- 1 - lambda^2
    if (!is_case_a && !is_case_b) {
      psi2[last_item_in_block] <- 1.0
    }
    if (is_case_a || is_case_b) {
      psi2[] <- 0.5
    }
  } else {
    if (length(psi2) != I.states) stop("Length of 'psi2' must be ", I.states, call. = FALSE)
    if (!is_case_a && !is_case_b) {
      psi2[last_item_in_block] <- 1.0
    }
    if (is_case_a || is_case_b) {
      psi2[] <- 0.5
    }
  }

  if (is.null(gamma.matrix)) {
    gamma.matrix <- matrix(NA, I.states, I.states)
  } else {
    if (!is.matrix(gamma.matrix) || nrow(gamma.matrix) != I.states ||
        ncol(gamma.matrix) != I.states)
      stop("'gamma.matrix' must be an ", I.states, "x", I.states, " matrix", call. = FALSE)
  }

  # ---- TIRT-specific: gamma generation ----
  # Generate item-level gamma utilities to ensure transitive pairwise comparisons
  if (all(is.na(gamma.matrix))) {
    gamma.item <- get_ctrl("gamma.item", runif(I.states, -1, 1), control)
    if (length(gamma.item) != I.states)
      stop("'gamma.item' must have length ", I.states, call. = FALSE)
    for (i in seq_len(I.states)) {
      for (k in seq_len(I.states)) {
        if (i != k) {
          gamma.matrix[i, k] <- gamma.item[i] - gamma.item[k]
        }
      }
    }
  } else {
    # User-provided gamma.matrix: check for intransitivities and warn
    for (i in seq_len(I.states)) {
      for (j in seq_len(I.states)) {
        for (k in seq_len(I.states)) {
          if (i != j && j != k && i != k) {
            if (!is.na(gamma.matrix[i, j]) && !is.na(gamma.matrix[j, k]) &&
                !is.na(gamma.matrix[i, k])) {
              sum_ij_jk <- gamma.matrix[i, j] + gamma.matrix[j, k]
              if (abs(sum_ij_jk - gamma.matrix[i, k]) > 1e-6) {
                warning(sprintf(
                  "Intransitivity detected in gamma.matrix: gamma[%d,%d]=%.4f + gamma[%d,%d]=%.4f = %.4f != gamma[%d,%d]=%.4f.",
                  i, j, gamma.matrix[i, j], j, k, gamma.matrix[j, k],
                  sum_ij_jk, i, k, gamma.matrix[i, k]
                ))
              }
            }
          }
        }
      }
    }
  }

  # ---- TIRT-specific: pairwise response simulation ----
  pairs.total <- sum(vapply(block.items, function(x) choose(length(x), 2), numeric(1L)))
  response.total <- matrix(NA, N.person, pairs.total)
  pairs <- c()

  gamma <- rep(NA, pairs.total)
  idx <- 1L
  for (b in seq_len(N.block)) {
    block.items.cur <- block.items[[b]]
    pairs.value.total <- t(combn(block.items.cur, 2))
    I.pairs.cur <- nrow(pairs.value.total)
    for (pp in seq_len(I.pairs.cur)) {
      i <- pairs.value.total[pp, 1]
      k <- pairs.value.total[pp, 2]
      if (is.na(gamma.matrix[i, k])) {
        if (!exists("gamma.item", inherits = FALSE)) {
          gamma.item <- runif(I.states, -1.0, 1.0)
        }
        gamma_val <- gamma.item[i] - gamma.item[k]
        gamma.matrix[i, k] <- gamma_val
        gamma.matrix[k, i] <- -gamma_val
      }
      gamma[idx] <- gamma.matrix[i, k]
      idx <- idx + 1L
    }

    if (fc.type[b] == "RANK") {
      pairs <- c(pairs, I.pairs.cur)
    } else if (fc.type[b] == "MOLE") {
      pairs <- c(pairs, length(block.items.cur) * 2 - 3)
    } else if (fc.type[b] == "PICK") {
      pairs <- c(pairs, length(block.items.cur) - 1)
    }
  }

  response <- matrix(NA, N.person, sum(pairs))
  pairs.value <- vector("list", N.person)
  for (p in seq_len(N.person)) {
    item_errors <- rnorm(I.states, mean = 0, sd = sqrt(psi2))
    idx.total <- 0L
    idx <- 0L
    for (b in seq_len(N.block)) {
      block.items.cur <- block.items[[b]]
      pairs.value.total <- t(combn(block.items.cur, 2))

      I.pairs.total.cur <- nrow(pairs.value.total)
      scores.b <- rep(0, length(block.items.cur))
      for (pp in seq_len(I.pairs.total.cur)) {
        i <- pairs.value.total[pp, 1]
        k <- pairs.value.total[pp, 2]

        lambda.i <- lambda[i]
        lambda.k <- lambda[k]
        gamma.current <- gamma.matrix[i, k]

        mu <- -gamma.current +
          lambda.i * sum(Q.matrix[i, ] * theta[p, ]) -
          lambda.k * sum(Q.matrix[k, ] * theta[p, ])
        latent_y <- mu + item_errors[i] - item_errors[k]
        response.total[p, idx.total + pp] <- as.integer(latent_y >= 0)

        if (response.total[p, idx.total + pp] > 0) {
          scores.b[match(i, block.items.cur)] <- scores.b[match(i, block.items.cur)] + 1L
        } else {
          scores.b[match(k, block.items.cur)] <- scores.b[match(k, block.items.cur)] + 1L
        }
      }

      if (fc.type[b] == "RANK") {
        pairs.value[[p]][[b]] <- pairs.value.total
        response[p, (idx + 1):(idx + pairs[b])] <-
          response.total[p, (idx.total + 1):(idx.total + I.pairs.total.cur)]
      } else if (fc.type[b] == "MOLE") {
        pairs.max <- which.max(scores.b)[1]
        pairs.min <- which.min(scores.b)[1]

        pairs.value.posi <- sort(unique(c(
          seq_len(I.pairs.total.cur) * (pairs.value.total[, 1] == block.items.cur[pairs.max]),
          seq_len(I.pairs.total.cur) * (pairs.value.total[, 1] == block.items.cur[pairs.min]),
          seq_len(I.pairs.total.cur) * (pairs.value.total[, 2] == block.items.cur[pairs.max]),
          seq_len(I.pairs.total.cur) * (pairs.value.total[, 2] == block.items.cur[pairs.min])
        )))
        pairs.value.posi <- pairs.value.posi[pairs.value.posi != 0]
        pairs.value[[p]][[b]] <- pairs.value.total[pairs.value.posi, , drop = FALSE]

        response[p, (idx + 1):(idx + pairs[b])] <-
          response.total[p, (idx.total + seq_len(I.pairs.total.cur))[pairs.value.posi]]
      } else if (fc.type[b] == "PICK") {
        pairs.max <- which.max(scores.b)[1]
        pairs.value.posi <- sort(unique(c(
          seq_len(I.pairs.total.cur) * (pairs.value.total[, 1] == block.items.cur[pairs.max]),
          seq_len(I.pairs.total.cur) * (pairs.value.total[, 2] == block.items.cur[pairs.max])
        )))
        pairs.value.posi <- pairs.value.posi[pairs.value.posi != 0]
        pairs.value[[p]][[b]] <- pairs.value.total[pairs.value.posi, , drop = FALSE]

        response[p, (idx + 1):(idx + pairs[b])] <-
          response.total[p, (idx.total + seq_len(I.pairs.total.cur))[pairs.value.posi]]
      }
      idx.total <- idx.total + I.pairs.total.cur
      idx <- idx + pairs[b]
    }
  }

  # ---- Row/column names ----
  colnames(response) <- paste0("pairs", seq_len(sum(pairs)))
  rownames(Q.matrix) <- rownames(gamma.matrix) <-
    colnames(gamma.matrix) <- paste0("S", seq_len(I.states))
  colnames(Q.matrix) <- colnames(theta) <- paste0("Dim.", seq_len(D))

  lambda_sign <- apply(Q.matrix, 1, function(x) {
    nz <- which(x != 0)
    if (length(nz) == 0) return(1)
    sign(x[nz[1]])
  })
  lambda_signed <- lambda * lambda_sign
  par <- cbind(lambda_signed * abs(Q.matrix), psi2)

  data <- get.data.from.response.TIRT(response, block.items, fc.type, pairs.value)

  # ---- Output assembly ----
  results <- list(
    data         = data,
    response     = response,
    theta        = theta,
    par          = par,
    Q.matrix     = Q.matrix,
    block.items  = block.items,
    Corr         = Corr,
    pairs.value  = pairs.value,
    gamma.matrix = gamma.matrix,
    gamma        = gamma,
    gamma.item   = if (exists("gamma.item", inherits = FALSE)) gamma.item else NULL,
    lambda       = lambda_signed,
    psi2         = psi2,
    N.person     = N.person,
    N.block      = N.block,
    I.block      = I.block,
    D            = D,
    I.states     = I.states,
    fc.type      = fc.type,
    call         = call,
    arguments    = list(
      N.person = N.person,
      N.block  = N.block,
      I.block  = I.block,
      D        = D,
      fc.type  = fc.type,
      control  = control
    )
  )

  class(results) <- "data.TIRT"
  return(results)
}

#' Compute TIRT Pairwise Comparison Probabilities
#' @param theta Latent trait matrix.
#' @param lambda Factor loading magnitudes per statement. If signed values
#'   are supplied from a fitted object, the function uses \code{abs(lambda)}
#'   and the sign in \code{Q.matrix} carries the direction.
#' @param psi2 Uniqueness variances per statement.
#' @param gamma.matrix Pairwise intercept matrix.
#' @param Q.matrix Design Q-matrix.
#' @param pairs.matrix Matrix of statement-index pairs.
#' @return Matrix of pairwise choice probabilities:
#'   \eqn{\Phi\{[-\gamma_{ik} + (|\lambda_i|q_i -
#'   |\lambda_k|q_k)'\theta_j] / \sqrt{\psi_i^2+\psi_k^2}\}}.
#' @keywords internal
model.TIRT <- function(theta, lambda, psi2, gamma.matrix, Q.matrix, pairs.matrix) {
  lambda.matrix <- t(abs(lambda) * Q.matrix)
  N.person <- nrow(theta)
  I.pairs <- nrow(pairs.matrix)
  prob <- matrix(0, N.person, I.pairs)

  for (i in seq_len(I.pairs)) {
    mu <- -gamma.matrix[pairs.matrix[i, 1], pairs.matrix[i, 2]] +
      theta %*% lambda.matrix[, pairs.matrix[i, 1]] -
      theta %*% lambda.matrix[, pairs.matrix[i, 2]]
    pis2l <- sqrt(max(psi2[pairs.matrix[i, 1]] + psi2[pairs.matrix[i, 2]],
                      .Machine$double.eps))
    prob[, i] <- pnorm(mu / pis2l)
  }
  return(prob)
}

#' Fit FCDCM Model via Stan
#'
#' @noRd
fit.FCDCM.stan <- function(data, Q.matrix, block.items = NULL,
                           dcm.type = "DINA",
                           control.model = NULL,
                           control.method = NULL,
                           .call = NULL) {
  call <- if (is.null(.call)) match.call() else .call
  control <- fc_as_control_list(control.model, "control.model")
  control.method <- fc_as_control_list(control.method, "control.method")
  common.method <- fit_common_method_control(control.method)
  stan.method <- fc_stan_method_control(control.method)
  prior <- list(
    delta1.mu = istem_scalar(get_ctrl("delta1.mu", 0.0, control), "delta1.mu"),
    delta1.sigma = istem_scalar(get_ctrl("delta1.sigma", 0.50, control), "delta1.sigma", positive = TRUE),
    delta0.mu = istem_scalar(get_ctrl("delta0.mu", 0.00, control), "delta0.mu"),
    delta0.sigma = istem_scalar(get_ctrl("delta0.sigma", 1.00, control), "delta0.sigma", positive = TRUE),
    eta0.mu = istem_scalar(
      get_ctrl("eta0.mu", 0.10, control),
      "eta0.mu", positive = TRUE),
    eta0.sigma = istem_scalar(
      get_ctrl("eta0.sigma", 0.10, control),
      "eta0.sigma", positive = TRUE),
    etaAB.mu = istem_scalar(
      get_ctrl("etaAB.mu", 0.30, control),
      "etaAB.mu", positive = TRUE),
    etaAB.sigma = istem_scalar(
      get_ctrl("etaAB.sigma", 0.10, control),
      "etaAB.sigma", positive = TRUE),
    eta_equal0.mu = istem_scalar(
      get_ctrl("eta_equal0.mu", 0.50, control),
      "eta_equal0.mu", positive = TRUE),
    eta_equal0.sigma = istem_scalar(
      get_ctrl("eta_equal0.sigma", 0.25, control),
      "eta_equal0.sigma", positive = TRUE),
    eta_equal1.mu = istem_scalar(
      get_ctrl("eta_equal1.mu", 0.50, control),
      "eta_equal1.mu", positive = TRUE),
    eta_equal1.sigma = istem_scalar(
      get_ctrl("eta_equal1.sigma", 0.25, control),
      "eta_equal1.sigma", positive = TRUE)
  )

  sm <- stanmodels[["FCDCM"]]
  if (is.null(sm)) {
    stop("Stan model FCDCM not found !", call. = FALSE)
  }

  data.matrix <- as.matrix(data)
  N <- nrow(data.matrix)
  B <- ncol(data.matrix)
  if (is.null(N) || is.null(B) || N < 1L || B < 1L) {
    stop("'data' must be an N x B FCDCM data matrix.", call. = FALSE)
  }
  if (is.numeric(data.matrix) || is.integer(data.matrix) ||
      is.logical(data.matrix)) {
    stop("'data' must contain FCDCM ranking strings such as '1>3'. ",
         "Use get.data.from.response.FCDCM() to convert a response matrix.",
         call. = FALSE)
  }

  if (is.null(block.items)) {
    block.items <- get.block.items.from.data.FCDCM(data)
  }
  if (!is.list(block.items) || length(block.items) != B) {
    stop("'block.items' must be a list with one two-statement entry per block.",
         call. = FALSE)
  }
  block.items <- lapply(block.items, as.integer)
  if (any(vapply(block.items, length, integer(1L)) != 2L)) {
    stop("Each FCDCM block must contain exactly two statement indices.",
         call. = FALSE)
  }
  all.items <- unlist(block.items, use.names = FALSE)
  if (anyNA(all.items) || any(all.items < 1L)) {
    stop("FCDCM statement indices in 'block.items' must be positive integers.",
         call. = FALSE)
  }
  if (anyDuplicated(all.items)) {
    stop("'block.items' contains duplicate statement indices.", call. = FALSE)
  }
  I.states <- length(all.items)
  if (!setequal(all.items, seq_len(I.states))) {
    stop("'block.items' must contain exactly the statement indices 1 through ",
         I.states, ".", call. = FALSE)
  }

  patterns <- matrix(all.items, ncol = 2L, byrow = TRUE)
  storage.mode(patterns) <- "integer"

  response <- get.response.from.data.FCDCM(data.matrix, block.items)$response
  if (anyNA(response) || !all(response %in% c(0L, 1L))) {
    stop("FCDCM responses must contain only 0/1 after parsing.",
         call. = FALSE)
  }
  storage.mode(response) <- "integer"

  Q.matrix <- as.matrix(Q.matrix)
  if (anyNA(Q.matrix) || !all(Q.matrix %in% c(0, 1))) {
    stop("'Q.matrix' must be a 0/1 matrix.", call. = FALSE)
  }
  if (nrow(Q.matrix) != I.states) {
    stop("'Q.matrix' must have ", I.states,
         " rows, one for each statement.", call. = FALSE)
  }
  if (any(rowSums(Q.matrix) < 1L)) {
    stop("Each statement must require at least one attribute.",
         call. = FALSE)
  }
  D <- ncol(Q.matrix)
  storage.mode(Q.matrix) <- "numeric"

  dcm.type <- toupper(dcm.type)
  if (length(dcm.type) != I.states) {
    dcm.type <- rep(dcm.type[1L], I.states)
  }
  if (!all(dcm.type %in% c("DINA", "DINO"))) {
    stop("'dcm.type' must contain only 'DINA' or 'DINO'.",
         call. = FALSE)
  }

  alpha.patterns <- as.matrix(do.call(expand.grid, rep(list(0:1), D)))
  storage.mode(alpha.patterns) <- "numeric"
  zeta.patterns <- get.zeta(alpha.patterns, Q.matrix, dcm.type)
  storage.mode(zeta.patterns) <- "integer"

  block.names <- colnames(response)
  if (is.null(block.names)) block.names <- paste0("B", seq_len(B))
  dim.names <- colnames(Q.matrix)
  if (is.null(dim.names)) dim.names <- paste0("Dim.", seq_len(D))
  item.names <- rownames(Q.matrix)
  if (is.null(item.names)) item.names <- paste0("S", seq_len(I.states))

  colnames(response) <- block.names
  rownames(patterns) <- block.names
  colnames(patterns) <- c("SA", "SB")
  rownames(Q.matrix) <- item.names
  colnames(Q.matrix) <- dim.names
  colnames(alpha.patterns) <- dim.names
  colnames(zeta.patterns) <- item.names

  prep <- list(
    response = response,
    Q.matrix = Q.matrix,
    block.items = block.items,
    patterns = patterns,
    dcm.type = dcm.type,
    alpha.patterns = alpha.patterns,
    zeta.patterns = zeta.patterns,
    N = N,
    B = B,
    D = D,
    I.states = I.states,
    block.names = block.names,
    dim.names = dim.names,
    item.names = item.names
  )
  mcmc_env <- setup_mcmc_env(common.method$cores)
  on.exit(restore_mcmc_env(mcmc_env), add = TRUE)

  stan.control <- build_stan_control(stan.method$algorithm, control.method)

  stan.data <- list(
    N = prep$N,
    B = prep$B,
    I = prep$I.states,
    D = prep$D,
    C = nrow(prep$alpha.patterns),
    y = as.integer(prep$response),
    alpha_patterns = as.integer(t(prep$alpha.patterns)),
    zeta_patterns = as.integer(t(prep$zeta.patterns)),
    patterns = as.integer(t(prep$patterns)),
    delta1_mu = prior$delta1.mu,
    delta1_sigma = prior$delta1.sigma,
    delta0_mu = prior$delta0.mu,
    delta0_sigma = prior$delta0.sigma,
    eta0_mu = prior$eta0.mu,
    eta0_sigma = prior$eta0.sigma,
    etaAB_mu = prior$etaAB.mu,
    etaAB_sigma = prior$etaAB.sigma,
    eta_equal0_mu = prior$eta_equal0.mu,
    eta_equal0_sigma = prior$eta_equal0.sigma,
    eta_equal1_mu = prior$eta_equal1.mu,
    eta_equal1_sigma = prior$eta_equal1.sigma,
    free_equal = 0L
  )

  set.seed(common.method$seed)
  stan.obj <- suppressWarnings(
    rstan::sampling(
      object = sm,
      data = stan.data,
      chains = stan.method$chains,
      iter = stan.method$iter,
      warmup = stan.method$warmup,
      thin = stan.method$thin,
      init = stan.method$init,
      algorithm = stan.method$algorithm,
      cores = mcmc_env$cores,
      control = stan.control,
      verbose = common.method$vis,
      show_messages = common.method$vis,
      seed = common.method$seed
    )
  )

  MCMC.obj <- rstan::extract(stan.obj, permuted = TRUE)
  stan.sum <- rstan::summary(stan.obj)$summary

  theta.draws <- as.matrix(MCMC.obj$theta)
  theta <- matrix(colMeans(theta.draws), ncol = 1L)
  theta.se <- matrix(apply(theta.draws, 2L, stats::sd), ncol = 1L)
  theta.Rhat <- matrix(extract_rhat_vector(stan.sum, "theta", prep$N),
                       ncol = 1L)
  colnames(theta) <- colnames(theta.se) <- colnames(theta.Rhat) <- "theta"
  rownames(theta) <- rownames(theta.se) <- rownames(theta.Rhat) <-
    rownames(prep$response)

  delta1.draws <- as.matrix(MCMC.obj$delta1)
  delta0.draws <- as.matrix(MCMC.obj$delta0)
  delta <- cbind(
    delta1 = colMeans(delta1.draws),
    delta0 = colMeans(delta0.draws)
  )
  delta.se <- cbind(
    delta1 = apply(delta1.draws, 2L, stats::sd),
    delta0 = apply(delta0.draws, 2L, stats::sd)
  )
  delta.Rhat <- cbind(
    delta1 = extract_rhat_vector(stan.sum, "delta1", prep$D),
    delta0 = extract_rhat_vector(stan.sum, "delta0", prep$D)
  )
  rownames(delta) <- rownames(delta.se) <- rownames(delta.Rhat) <-
    prep$dim.names

  eta0.draws <- as.matrix(MCMC.obj$eta0)
  etaAB.draws <- as.matrix(MCMC.obj$etaAB)
  par <- cbind(eta0 = colMeans(eta0.draws),
               etaAB = colMeans(etaAB.draws))
  par.se <- cbind(eta0 = apply(eta0.draws, 2L, stats::sd),
                  etaAB = apply(etaAB.draws, 2L, stats::sd))
  par.Rhat <- cbind(eta0 = extract_rhat_vector(stan.sum, "eta0", prep$B),
                    etaAB = extract_rhat_vector(stan.sum, "etaAB", prep$B))
  rownames(par) <- rownames(par.se) <- rownames(par.Rhat) <-
    prep$block.names
  par.free <- matrix(TRUE, prep$B, ncol(par),
                     dimnames = list(prep$block.names,
                                     colnames(par)))

  class.post <- fcdcm_stan_class_posterior(
    theta.draws = theta.draws,
    delta1.draws = delta1.draws,
    delta0.draws = delta0.draws,
    eta0.draws = eta0.draws,
    etaAB.draws = etaAB.draws,
    response = prep$response,
    alpha.patterns = prep$alpha.patterns,
    zeta.patterns = prep$zeta.patterns,
    patterns = prep$patterns
  )
  colnames(class.post) <- pattern_key(prep$alpha.patterns)
  rownames(class.post) <- rownames(prep$response)
  alpha.prob <- class.post %*% prep$alpha.patterns
  colnames(alpha.prob) <- prep$dim.names
  rownames(alpha.prob) <- rownames(prep$response)
  alpha.est <- fcdcm_alpha_map(class.post, prep$alpha.patterns)
  storage.mode(alpha.est) <- "integer"
  alpha.se <- matrix(NA_real_, prep$N, prep$D,
                     dimnames = dimnames(alpha.est))
  alpha.Rhat <- alpha.se

  npar <- ncol(par) * prep$B + 2L * prep$D
  results <- list(
    npar = npar,
    method = "stan",
    theta = list(est = theta, se = theta.se, Rhat = theta.Rhat),
    alpha = list(est = alpha.est, se = alpha.se, Rhat = alpha.Rhat,
                 prob = alpha.prob),
    class.post = class.post,
    delta = list(est = delta, se = delta.se, Rhat = delta.Rhat),
    par = list(est = par, se = par.se, Rhat = par.Rhat, free = par.free),
    Corr = list(
      est = matrix(1, 1L, 1L, dimnames = list("theta", "theta")),
      se = matrix(0, 1L, 1L, dimnames = list("theta", "theta")),
      Rhat = matrix(NA_real_, 1L, 1L, dimnames = list("theta", "theta"))
    ),
    stan.obj = stan.obj,
    MCMC.obj = MCMC.obj,
    log_lik = MCMC.obj$log_lik,
    Q.matrix = prep$Q.matrix,
    block.items = prep$block.items,
    patterns = prep$patterns,
    alpha.patterns = prep$alpha.patterns,
    zeta.patterns = prep$zeta.patterns,
    dcm.type = prep$dcm.type,
    response = prep$response,
    call = call,
    arguments = list(
      data = data,
      Q.matrix = prep$Q.matrix,
      block.items = prep$block.items,
      D = prep$D,
      dcm.type = prep$dcm.type,
      method = "stan",
      chains = stan.method$chains,
      iter = stan.method$iter,
      warmup = stan.method$warmup,
      thin = stan.method$thin,
      init = stan.method$init,
      algorithm = stan.method$algorithm,
      cores = common.method$cores,
      vis = common.method$vis,
      seed = common.method$seed,
      control.model = control,
      control.method = fit_effective_method_control(
        control.method, method = stan.method, common = common.method
      )
    )
  )

  class(results) <- "FCDCM"
  L <- get_ctrl("L", NULL, control)
  results$logLik <- logLik.FCDCM(results, L = L)
  results
}

#' Fit the Forced-Choice Generalized Graded Unfolding Model (FCGGUM)
#'
#' @description
#' Fits a forced-choice variant of the multidimensional generalized graded
#' unfolding model (GGUM) to comparative ranking data. Unlike dominance-based
#' FC models, the FCGGUM uses an ideal-point (unfolding) item response
#' process: endorsement probability peaks when the person and item locations
#' coincide, and the block-level ranking follows the Luce--Plackett model.
#'
#' @section Model Specification:
#'
#' \strong{Item-level unfolding process.}
#' For statement \eqn{i} with Q-vector \eqn{\mathbf{q}_i \in \{-1, 0, 1\}^D},
#' the category response function is the GGUM (see \code{\link{fit.MGGUM}}
#' for the full polytomous specification). For the forced-choice context,
#' each item is binary (\eqn{K_i=2}). Define
#' \deqn{
#'   r_{ij} =
#'   \left[
#'     \sum_{d=1}^{D} a_{id}^2
#'     \left(\theta_{jd} - \delta_{id}\right)^2
#'   \right]^{1/2},
#'   \qquad
#'   S_i = \sum_{d=1}^{D} a_{id},
#'   \qquad
#'   \psi_{i1} = \tau_{i1}S_i .
#' }
#' The binary GGUM endorsement probability used by the code is:
#' \deqn{
#'   P_i(\boldsymbol{\theta}_j) =
#'   \frac{
#'     \exp\{r_{ij}-\psi_{i1}\} +
#'     \exp\{2r_{ij}-\psi_{i1}\}
#'   }{
#'     1 + \exp\{3r_{ij}\} +
#'     \exp\{r_{ij}-\psi_{i1}\} +
#'     \exp\{2r_{ij}-\psi_{i1}\}
#'   },
#' }
#' where \eqn{\tau_{i1} < 0} is the first (and only free) threshold.
#'
#' \strong{Block-level forced-choice ranking.}
#' Within each block \eqn{b}, the probability of the observed ranking is
#' computed by applying the sequential Luce/Plackett rule to
#' \eqn{u_i(\boldsymbol{\theta}_j)=\mathrm{logit}
#' \{P_i(\boldsymbol{\theta}_j)\}}:
#' \deqn{
#'   P(\text{ranking} \mid \boldsymbol{\theta}_j) =
#'   \prod_{m=1}^{K_b-1}
#'   \frac{\exp\{u_{i_{(m)}}(\boldsymbol{\theta}_j)\}}
#'        {\sum_{r=m}^{K_b}\exp\{u_{i_{(r)}}(\boldsymbol{\theta}_j)\}}.
#' }
#' MOLE and PICK probabilities are sums over compatible full rankings,
#' normalized over the observed block patterns by the C++ probability helper.
#'
#' \strong{Q-matrix for unfolding.} The Q-matrix serves a dual role:
#' entries of \eqn{\pm 1} activate both the slope \eqn{a_{id}} and the
#' location \eqn{\delta_{id}}; the sign of \eqn{q_{id}} determines the
#' sign of \eqn{\delta_{id}} (positive vs. negative side of dimension
#' \eqn{d}).
#'
#' @section Estimation Methods:
#'
#' \describe{
#'   \item{\strong{Stan} (\code{method = "stan"}):}{
#'     Full Bayesian inference via HMC. The GGUM likelihood involves
#'     summation terms that are handled in the Stan model block.}
#'   \item{\strong{iStEM} (\code{method = "iStEM"}):}{
#'     Improved Stochastic EM. Person sampling uses finite-grid block Gibbs;
#'     GGUM item parameters updated via constrained L-BFGS-B with
#'     monotonicity constraints on \eqn{\tau} and sign constraints on
#'     \eqn{\delta}.}
#' }
#'
#' @param data An \eqn{N \times B} forced-choice data matrix with ranking
#'   strings or 1-based pattern indices.
#' @param Q.matrix An optional \eqn{I \times D} matrix with entries
#'   \eqn{-1, 0, 1}. See \code{\link{fit.MGGUM}} for the sign convention.
#' @param block.items A list of length \eqn{B} giving global item indices
#'   in each block. Parsed from \code{data} if \code{NULL} and data contains
#'   ranking strings.
#' @param D Integer; number of latent dimensions. If \code{NULL}, inferred
#'   from \code{Q.matrix} when available, otherwise from the block structure.
#' @param fc.type Forced-choice response type: \code{"RANK"} (default),
#'   \code{"MOLE"}, or \code{"PICK"}.
#' @param method Estimation method: \code{"iStEM"} (default) or
#'   \code{"stan"}.
#' @param control.model A named list of model-level hyperparameters for the
#'   GGUM unfolding model in a forced-choice framework. Supported entries:
#'   \describe{
#'     \item{\code{a.mu}, \code{a.sigma}}{Prior location and scale for
#'           \eqn{\log a_{id}} (log-normal). Defaults: 0, 0.5.
#'           The log-normal prior ensures positivity of discrimination
#'           parameters in the unfolding model. The zero mean-log
#'           encourages slopes near 1 unless the data strongly indicate
#'           otherwise.}
#'     \item{\code{delta.pos.mu}, \code{delta.pos.sigma}}{Prior mean and
#'           SD for \eqn{\delta_{id}} when \eqn{q_{id} = 1} (positive-side
#'           item locations; normal). Defaults: 1, 0.5. In the FC context,
#'           these priors regularize the location of statements on the
#'           positive side of each latent dimension.}
#'     \item{\code{delta.neg.mu}, \code{delta.neg.sigma}}{Prior mean and
#'           SD for \eqn{\delta_{id}} when \eqn{q_{id} = -1} (negative-side
#'           item locations; normal). Defaults: -1, 0.5. These priors
#'           regularize statement locations on the negative side of each
#'           dimension.}
#'     \item{\code{tau.mu}, \code{tau.sigma}}{Location and scale for the
#'           log-normal prior on the positive threshold magnitude
#'           \eqn{-\tau_{i1}}. Defaults: 0, 0.5. In the binary FCGGUM,
#'           each statement has a single free threshold
#'           \eqn{\tau_{i1}<0}.}
#'     \item{\code{theta.mu}}{Prior mean vector for
#'           \eqn{\boldsymbol{\theta}_j}. Default: \code{rep(0, D)}.}
#'     \item{\code{L}}{Theta grid size per dimension for marginal
#'           log-likelihood computation and iStEM block Gibbs sampling.
#'           Default adapts to \eqn{D}.}
#'     \item{\code{theta.lower}, \code{theta.upper}}{Bounds for the
#'           theta grid used by marginal log-likelihood computation and
#'           iStEM block Gibbs sampling. Defaults: -6, 6.}
#'   }
#' @param control.method A named list of method-specific tuning parameters.
#'   Common entries (used by both Stan and iStEM):
#'   \describe{
#'     \item{\code{cores}}{Number of CPU cores for parallel chains
#'           (Stan) or ignored (iStEM). Default: the number of
#'           \code{chains}.}
#'     \item{\code{vis}}{Logical; if \code{TRUE} (default), prints progress
#'           information to the console.}
#'     \item{\code{seed}}{Random seed for reproducibility.
#'           Default: a random integer.}
#'   }
#'   Stan-specific entries:
#'   \describe{
#'     \item{\code{chains}}{Number of MCMC chains (default: 2).}
#'     \item{\code{iter}}{Total iterations per chain (default: 5000).}
#'     \item{\code{warmup}}{Warmup/burn-in iterations per chain
#'           (default: \code{iter / 2}).}
#'     \item{\code{thin}}{Thinning interval (default: 1).}
#'     \item{\code{init}}{Initial values: \code{"random"} (default)
#'           for uniform(-2, 2) initialization, or a list of initial
#'           values per chain.}
#'     \item{\code{algorithm}}{MCMC algorithm: \code{"HMC"} (default),
#'           \code{"HMC"}, or \code{"Fixed_param"}.}
#'     \item{\code{adapt_delta}}{Target average acceptance probability
#'           (NUTS; default: 0.95). The GGUM likelihood involves summation
#'           terms that can create challenging posterior geometries;
#'           consider increasing to 0.9--0.95 if divergences occur.}
#'     \item{\code{max_treedepth}}{Maximum tree depth (NUTS; default: 10).
#'           Increase if "max treedepth exceeded" warnings appear.}
#'     \item{\code{stepsize}}{Initial step size for the leapfrog
#'           integrator (auto-tuned by Stan if not set).}
#'     \item{\code{int_time}}{Total integration time for HMC trajectories
#'           (only when \code{algorithm = "HMC"}).}
#'     \item{\code{metric}}{Mass matrix type: \code{"unit_e"},
#'           \code{"diag_e"} (default), or \code{"dense_e"}.}
#'     \item{\code{adapt_engaged}}{Logical; if \code{TRUE} (default),
#'           warmup adaptation is enabled.}
#'     \item{\code{adapt_init_buffer}, \code{adapt_term_buffer},
#'           \code{adapt_window}}{Warmup adaptation scheduling parameters
#'           (defaults: 25, 50, 25).}
#'   }
#'   iStEM-specific entries:
#'   \describe{
#'     \item{\code{M}}{Number of burn-in batches retained for Geweke
#'           convergence diagnosis (default: 10; must be \eqn{\ge 2}).}
#'     \item{\code{B}}{Batch size: MCMC iterations per batch (default: 20).}
#'     \item{\code{burnin.maxitr}}{Maximum burn-in batches (default: 100).}
#'     \item{\code{maxitr}}{Maximum total batches (default: 2000).}
#'     \item{\code{eps1}}{Geweke z-score convergence threshold for
#'           burn-in (default: 1.5).}
#'     \item{\code{eps2}}{Monte Carlo error tolerance for the final
#'           chain (default: 0.4).}
#'     \item{\code{frac1}, \code{frac2}}{Fractions for the Geweke
#'           diagnostic (defaults: 0.1, 0.5).}
#'     \item{\code{corr.optim.maxit}}{Maximum L-BFGS-B iterations for the
#'           constrained unit-diagonal correlation update (default: 50).}
#'     \item{\code{optim.maxit}}{Maximum L-BFGS-B iterations per item
#'           (default: 50). GGUM item parameters (\eqn{a}, \eqn{\delta},
#'           \eqn{\tau}) are optimized jointly under monotonicity and
#'           sign constraints. Increase if optimization warnings appear.}
#'     \item{\code{fix.corr}}{Logical; if \code{TRUE}, the inter-trait
#'           correlation matrix is fixed to the identity (default:
#'           \code{FALSE}).}
#'     \item{\code{estimate.se}}{Logical; if \code{TRUE} (default),
#'           standard errors are computed from the final Monte Carlo
#'           chain.}
#'     \item{\code{delta.lower}}{Lower bound on \eqn{|\delta_{id}|}
#'           (default: \code{1e-4}). Ensures item locations are bounded
#'           away from zero for active dimensions.}
#'     \item{\code{tau.gap.lower}, \code{tau.gap.upper}}{Lower and upper
#'           bounds on the positive gap used to construct
#'           \eqn{\tau_{i1}<0}. Defaults are 1e-4 and 6.}
#'   }
#'
#' @return An object of class \code{"FCGGUM"} containing:
#' \describe{
#'   \item{\code{npar}}{Number of free parameters.}
#'   \item{\code{method}}{\code{"stan"} or \code{"iStEM"}.}
#'   \item{\code{theta}}{List with \code{est}, \code{se}, \code{Rhat}
#'         (\eqn{N \times D}).}
#'   \item{\code{par}}{List with \code{est}, \code{se}, \code{Rhat},
#'         \code{free} (\eqn{I \times (2D + 2)}).}
#'   \item{\code{Corr}}{List with \code{est}, \code{se}, \code{Rhat}
#'         (\eqn{D \times D}).}
#'   \item{\code{block.items}, \code{patterns}, \code{patterns.total},
#'         \code{response}}{Block structure.}
#'   \item{\code{logLik}}{Marginal log-likelihood (class \code{"logLik"}).}
#' }
#'
#' @references
#' Roberts, J. S., Donoghue, J. R., & Laughlin, J. E. (2000). A general item
#'   response theory model for unfolding unidimensional polytomous responses.
#'   \emph{Applied Psychological Measurement}, 24(1), 3--32.
#'
#' @seealso
#' \code{\link{sim.data.FCGGUM}}, \code{\link{get.fit.index.FCGGUM}},
#' \code{\link{logLik.FCGGUM}}, \code{\link{fit.MGGUM}}
#'
#' @examples
#' sim <- sim.data.FCGGUM(N.person = 20, N.block = 3, I.block = 2,
#'                        D = 2, fc.type = "RANK")
#' fit <- fit.FCGGUM(sim$data, Q.matrix = sim$Q.matrix,
#'                   block.items = sim$block.items,
#'                   D = 2, fc.type = "RANK", method = "iStEM",
#'                   control.method = list(
#'                     vis = FALSE, seed = 123,
#'                     M = 2, B = 2, burnin.maxitr = 2,
#'                     maxitr = 3, eps1 = 10, eps2 = 10,
#'                     estimate.se = FALSE))
#' head(fit$theta$est)
#' gof <- get.fit.index(fit)
#' summary(gof)
#'
#' @export
fit.FCGGUM <- function(data, Q.matrix = NULL, block.items = NULL,
                       D = NULL, fc.type = NULL,
                       method = c("iStEM", "stan"),
                       control.model = NULL,
                       control.method = NULL) {

  call <- match.call()
  method <- match.arg(method)

  block.items <- resolve_block_items(block.items, data)
  fc.type     <- resolve_fc_type(fc.type, data, block.items)
  D           <- resolve_D(D, Q.matrix = Q.matrix, block.items = block.items)

  control.model  <- fc_as_control_list(control.model, "control.model")
  control.method <- fc_as_control_list(control.method, "control.method")

  if (method == "stan") {
    return(fit.FCGGUM.stan(
      data = data,
      Q.matrix = Q.matrix,
      block.items = block.items,
      D = D,
      fc.type = fc.type,
      control.model = control.model,
      control.method = control.method,
      .call = call
    ))
  }

  fit.FCGGUM.iStEM(
    data = data,
    Q.matrix = Q.matrix,
    block.items = block.items,
    D = D,
    fc.type = fc.type,
    control.model = control.model,
    control.method = control.method,
    .call = call
  )
}

################################# FCGDINA Stan #################################

#' @noRd
fit.FCGDINA.stan <- function(data, Q.matrix, model, block.items, fc.type,
                             control.model, control.method, .call) {
  call <- if (is.null(.call)) match.call() else .call
  control <- fc_as_control_list(control.model, "control.model")
  control.method <- fc_as_control_list(control.method, "control.method")
  common.method <- fit_common_method_control(control.method)
  stan.method <- fc_stan_method_control(control.method)

  prep <- fcgdina_prepare(data, Q.matrix, model, block.items, fc.type)

  prior <- list(
    delta.mu = istem_scalar(
      get_ctrl("delta.mu", 0.0, control),
      "delta.mu", positive = FALSE),
    delta.sigma = istem_scalar(
      get_ctrl("delta.sigma", 1.0, control),
      "delta.sigma", positive = TRUE),
    pi.alpha = istem_scalar(
      get_ctrl("pi.prior.alpha", 1.0, control),
      "pi.prior.alpha", positive = TRUE)
  )
  identification <- fcgdina_identifiable_basis(prep)

  sm <- stanmodels[["FCGDINA"]]
  if (is.null(sm)) stop("Stan model FCGDINA not found.", call. = FALSE)

  stan.data <- list(
    N = prep$N,
    G = prep$G,
    B = prep$B,
    I = prep$I.states,
    D = prep$D,
    C = prep$C,
    y_unique = prep$response.unique,
    y_count = prep$response.count,
    alpha_patterns = prep$alpha.patterns,
    total_design_entries = prep$total_design_entries,
    flat_design = prep$flat_design,
    design_rows = prep$design.rows,
    design_cols = prep$design.cols,
    design_offset = prep$design_offset,
    total_delta_len = prep$total_delta_len,
    total_delta_free = identification$rank,
    delta_basis = identification$basis,
    delta_offset = prep$delta_offset,
    class_map = prep$class_map_mat,
    n_items = prep$n.items,
    total_block_items = length(prep$block.items.flat),
    block_items = prep$block.items.flat,
    item_start = prep$item.start,
    n_total = prep$n.total,
    total_cells_total = length(prep$patterns.total.flat),
    patterns_total = prep$patterns.total.flat,
    total_start = prep$total.start,
    n_obs = prep$n.obs,
    total_obs_patterns = sum(prep$n.obs),
    obs_pattern_start = prep$obs.pattern.start,
    total_obs_full_links = length(prep$obs.full.index),
    obs_full_start = prep$obs.full.start,
    obs_full_index = prep$obs.full.index,
    delta_prior_mu = drop(crossprod(
      identification$basis, rep(prior$delta.mu, prep$total_delta_len))),
    delta_prior_sigma = prior$delta.sigma,
    pi_prior_alpha = prior$pi.alpha
  )

  mcmc_env <- setup_mcmc_env(common.method$cores)
  on.exit(restore_mcmc_env(mcmc_env), add = TRUE)
  stan.control <- build_stan_control(stan.method$algorithm, control.method)

  set.seed(common.method$seed)
  stan.obj <- suppressWarnings(
    rstan::sampling(
      object = sm,
      data = stan.data,
      chains = stan.method$chains,
      iter = stan.method$iter,
      warmup = stan.method$warmup,
      thin = stan.method$thin,
      init = stan.method$init,
      algorithm = stan.method$algorithm,
      cores = mcmc_env$cores,
      control = stan.control,
      verbose = common.method$vis,
      show_messages = common.method$vis,
      seed = common.method$seed
    )
  )

  MCMC.obj <- rstan::extract(stan.obj, permuted = TRUE)
  stan.sum <- rstan::summary(stan.obj)$summary

  class.prob.group <- ensure_3d(MCMC.obj$class_prob_group)
  alignment <- fcgdina_align_draws(
    as.matrix(MCMC.obj$delta), as.matrix(MCMC.obj$pi),
    class.prob.group, prep, identification
  )
  MCMC.obj$delta <- alignment$delta
  MCMC.obj$pi <- alignment$pi
  MCMC.obj$class_prob_group <- alignment$class.prob
  class.prob.group <- alignment$class.prob

  # ---- Delta parameters ----
  # Stan samples delta on the logit scale.  For each stable-chain draw we
  # convert to the probability (identity) scale so that the default delta
  # output is comparable with EM / iStEM.  Logit-scale summaries are also
  # returned as delta.logit.
  delta.draws.logit <- as.matrix(MCMC.obj$delta)          # n_iter x total_delta_len

  # Logit-scale summaries (as sampled by Stan)
  delta.logit.est <- colMeans(delta.draws.logit)
  delta.logit.se <- apply(delta.draws.logit, 2L, stats::sd)

  # Probability-scale draws and summaries
  delta.draws.prob <- cpp_fcgdina_delta_logit_to_prob(
    delta.draws.logit,
    prep$flat_design,
    prep$design.rows,
    prep$design.cols,
    prep$design_offset,
    prep$delta_offset,
    prep$I.states
  )

  delta.est <- colMeans(delta.draws.prob)
  delta.se <- apply(delta.draws.prob, 2L, stats::sd)
  aligned.rhat <- fcgdina_aligned_rhat(stan.obj, prep, identification)
  delta.Rhat <- aligned.rhat$delta

  delta.list.est <- delta.list.se <- delta.list.Rhat <-
    delta.list.logit.est <- delta.list.logit.se <-
    vector("list", prep$I.states)
  for (i in seq_len(prep$I.states)) {
    idx <- prep$delta_offset[i] + seq_len(prep$delta.len[i])
    delta.list.est[[i]]       <- delta.est[idx]
    delta.list.se[[i]]        <- delta.se[idx]
    delta.list.Rhat[[i]]      <- delta.Rhat[idx]
    delta.list.logit.est[[i]] <- delta.logit.est[idx]
    delta.list.logit.se[[i]]  <- delta.logit.se[idx]
  }

  # ---- Structural parameters pi (jointly estimated by Stan) ----
  pi.draws <- as.matrix(MCMC.obj$pi)
  pi.est <- colMeans(pi.draws)
  pi.se <- apply(pi.draws, 2L, stats::sd)
  pi.Rhat <- aligned.rhat$pi
  names(pi.est) <- pattern_key(prep$alpha.patterns)

  # ---- Class posterior probabilities (already pi-weighted by Stan) ----
  class.post.group <- apply(class.prob.group, c(2, 3), mean)
  class.post <- t(class.post.group[, prep$response.group, drop = FALSE])
  log_lik.group <- as.matrix(MCMC.obj$log_lik_group)
  log_lik <- log_lik.group[, prep$response.group, drop = FALSE]

  # ---- Alpha estimates from pi-weighted posteriors ----
  alpha.prob <- class.post %*% prep$alpha.patterns
  colnames(alpha.prob) <- paste0("alpha", seq_len(prep$D))
  rownames(alpha.prob) <- rownames(prep$response)
  alpha.est <- (alpha.prob > 0.5) * 1L
  storage.mode(alpha.est) <- "integer"

  colnames(class.post) <- pattern_key(prep$alpha.patterns)
  rownames(class.post) <- rownames(prep$response)

  results <- list(
    npar = identification$rank + prep$C - 1L,
    method = "stan",
    alpha = list(est = alpha.est, se = matrix(NA_real_, prep$N, prep$D),
                 Rhat = matrix(NA_real_, prep$N, prep$D),
                 prob = alpha.prob),
    class.post = class.post,
    delta = list(est = delta.list.est, se = delta.list.se,
                 Rhat = delta.list.Rhat),
    delta.logit = list(est = delta.list.logit.est,
                       se  = delta.list.logit.se,
                       Rhat = delta.list.Rhat),
    delta.identification = identification[c(
      "rank", "nullity", "block.rank", "block.nullity", "orientation.rank",
      "convention"
    )],
    delta.link = "identity",
    delta.store = fcgdina_delta_store(t(delta.draws.prob), prep),
    pi = pi.est,
    pi.se = pi.se,
    pi.Rhat = pi.Rhat,
    stan.obj = stan.obj,
    MCMC.obj = MCMC.obj,
    log_lik = log_lik,
    EM = NULL,
    iStEM = NULL,
    Q.matrix = prep$Q.matrix,
    model = prep$model,
    block.items = prep$block.items,
    patterns = prep$patterns,
    patterns.total = prep$patterns.total,
    design.matrix.list = prep$design.matrix.list,
    alpha.patterns = prep$alpha.patterns,
    fc.type = prep$fc.type,
    response = prep$response,
    call = call,
    arguments = list(
      data = data,
      Q.matrix = prep$Q.matrix,
      block.items = prep$block.items,
      model = prep$model,
      fc.type = prep$fc.type,
      method = "stan",
      cores = common.method$cores,
      vis = common.method$vis,
      seed = common.method$seed,
      control.model = control,
      control.method = fit_effective_method_control(
        control.method, method = stan.method, common = common.method)
    )
  )
  class(results) <- "FCGDINA"
  results$logLik <- logLik.FCGDINA(results)
  results
}


#' Construct an identifiable FCGDINA coefficient basis
#'
#' Forced-choice probabilities depend only on within-block utility contrasts.
#' This helper removes every coefficient direction that leaves all such
#' contrasts unchanged. The resulting orthonormal row-space basis defines the
#' unique minimum-norm representative used by the Stan model.
#'
#' @param prep Prepared FCGDINA data returned by code{fcgdina_prepare()}.
#' @return A list containing the global basis and blockwise rank diagnostics.
#' @noRd
fcgdina_identifiable_basis <- function(prep) {
  block.basis <- vector("list", prep$B)
  block.index <- vector("list", prep$B)
  block.rank <- block.nullity <- integer(prep$B)

  for (b in seq_len(prep$B)) {
    items <- prep$block.items[[b]]
    item.len <- prep$delta.len[items]
    item.start <- cumsum(c(1L, item.len))
    n.par <- sum(item.len)
    contrast <- matrix(0, prep$C * (length(items) - 1L), n.par)
    design <- lapply(items, function(i) {
      prep$design.matrix.list[[i]][prep$class.map.list[[i]], , drop = FALSE]
    })

    for (k in 2:length(items)) {
      rows <- (k - 2L) * prep$C + seq_len(prep$C)
      contrast[rows, item.start[1L]:(item.start[2L] - 1L)] <- -design[[1L]]
      contrast[rows, item.start[k]:(item.start[k + 1L] - 1L)] <- design[[k]]
    }

    decomp <- svd(contrast, nu = 0L, nv = min(dim(contrast)))
    tol <- max(dim(contrast)) * max(decomp$d) * .Machine$double.eps
    rank <- sum(decomp$d > tol)
    if (rank < 1L) {
      stop("FCGDINA block ", b, " has no estimable utility contrast.",
           call. = FALSE)
    }

    block.basis[[b]] <- decomp$v[, seq_len(rank), drop = FALSE]
    block.index[[b]] <- unlist(lapply(items, function(i) {
      prep$delta_offset[i] + seq_len(prep$delta.len[i])
    }), use.names = FALSE)
    block.rank[b] <- rank
    block.nullity[b] <- n.par - rank
  }

  total.rank <- sum(block.rank)
  basis <- matrix(0, prep$total_delta_len, total.rank)
  col.start <- cumsum(c(1L, block.rank))
  for (b in seq_len(prep$B)) {
    cols <- col.start[b]:(col.start[b + 1L] - 1L)
    basis[block.index[[b]], cols] <- block.basis[[b]]
  }

  orientation <- matrix(0, prep$D, prep$total_delta_len)
  for (d in seq_len(prep$D)) {
    items <- which(prep$Q.matrix[, d] == 1)
    if (length(items) == 0L) {
      stop("FCGDINA attribute ", d, " is not measured by any statement.",
           call. = FALSE)
    }
    high <- prep$alpha.patterns[, d] == 1L
    for (i in items) {
      design <- prep$design.matrix.list[[i]][prep$class.map.list[[i]], ,
                                                    drop = FALSE]
      contrast <- colMeans(design[high, , drop = FALSE]) -
        colMeans(design[!high, , drop = FALSE])
      idx <- prep$delta_offset[i] + seq_len(prep$delta.len[i])
      orientation[d, idx] <- orientation[d, idx] + contrast / length(items)
    }
  }

  orientation.free <- orientation %*% basis
  orientation.svd <- svd(orientation.free, nu = 0L,
                          nv = ncol(orientation.free))
  orientation.tol <- max(dim(orientation.free)) * max(orientation.svd$d) *
    .Machine$double.eps
  orientation.rank <- sum(orientation.svd$d > orientation.tol)
  if (orientation.rank < prep$D) {
    stop("FCGDINA cannot orient all attributes: aggregate discrimination ",
         "constraints have rank ", orientation.rank, " but D = ", prep$D,
         ".", call. = FALSE)
  }

  list(
    basis = basis,
    orientation = orientation,
    rank = total.rank,
    nullity = prep$total_delta_len - total.rank,
    block.rank = block.rank,
    block.nullity = block.nullity,
    orientation.rank = orientation.rank,
    convention = paste(
      "orthonormal within-block contrast row space (minimum norm);",
      "GDINA/ACDM draws aligned to positive aggregate discrimination"
    )
  )
}


#' Align FCGDINA posterior draws to a common attribute orientation
#'
#' @param delta Draw-by-parameter logit coefficient matrix.
#' @param pi Draw-by-class structural probability matrix.
#' @param class.prob Optional draw-by-class-by-group posterior array.
#' @param prep,identification Prepared model and identification information.
#' @return Aligned draws and the applied class permutations.
#' @noRd
fcgdina_align_draws <- function(delta, pi, class.prob = NULL,
                                prep, identification) {
  delta <- as.matrix(delta)
  pi <- as.matrix(pi)
  permutations <- matrix(rep(seq_len(prep$C), each = nrow(delta)),
                         nrow(delta), prep$C)
  if (!prep$model %in% c("GDINA", "ACDM")) {
    return(list(delta = delta, pi = pi, class.prob = class.prob,
                permutations = permutations))
  }

  alpha.key <- pattern_key(prep$alpha.patterns)
  for (s in seq_len(nrow(delta))) {
    flip <- drop(identification$orientation %*% delta[s, ]) < 0
    if (!any(flip)) next

    alpha.flipped <- prep$alpha.patterns
    for (d in which(flip)) {
      alpha.flipped[, d] <- 1L - alpha.flipped[, d]
    }
    permutation <- match(pattern_key(alpha.flipped), alpha.key)
    delta.new <- delta[s, ]

    for (i in seq_len(prep$I.states)) {
      idx <- prep$delta_offset[i] + seq_len(prep$delta.len[i])
      design <- prep$design.matrix.list[[i]]
      map <- prep$class.map.list[[i]]
      eta <- drop(design[map, , drop = FALSE] %*% delta[s, idx])
      eta <- eta[permutation]
      eta.local <- vapply(seq_len(nrow(design)), function(r) {
        eta[which(map == r)[1L]]
      }, numeric(1L))
      delta.new[idx] <- qr.solve(design, eta.local)
    }

    delta[s, ] <- drop(identification$basis %*%
                          crossprod(identification$basis, delta.new))
    pi[s, ] <- pi[s, permutation]
    if (!is.null(class.prob)) {
      class.prob[s, , ] <- class.prob[s, permutation, ]
    }
    permutations[s, ] <- permutation
  }

  list(delta = delta, pi = pi, class.prob = class.prob,
       permutations = permutations)
}


#' Compute R-hat after FCGDINA attribute-orientation alignment
#'
#' @param stan.obj Fitted Stan model.
#' @param prep,identification Prepared model and identification information.
#' @return Lists of aligned delta and pi R-hat values.
#' @noRd
fcgdina_aligned_rhat <- function(stan.obj, prep, identification) {
  raw <- rstan::extract(
    stan.obj, pars = c("delta", "pi"), permuted = FALSE,
    inc_warmup = FALSE
  )
  n.iter <- dim(raw)[1L]
  n.chain <- dim(raw)[2L]
  parameter.names <- dimnames(raw)[[3L]]
  delta.idx <- grep("^delta\\[", parameter.names)
  pi.idx <- grep("^pi\\[", parameter.names)
  delta.chain <- array(NA_real_, c(n.iter, n.chain, prep$total_delta_len))
  pi.chain <- array(NA_real_, c(n.iter, n.chain, prep$C))

  for (chain in seq_len(n.chain)) {
    aligned <- fcgdina_align_draws(
      raw[, chain, delta.idx, drop = FALSE][, 1L, ],
      raw[, chain, pi.idx, drop = FALSE][, 1L, ],
      prep = prep, identification = identification
    )
    delta.chain[, chain, ] <- aligned$delta
    pi.chain[, chain, ] <- aligned$pi
  }

  if (n.chain < 2L) {
    return(list(delta = rep(NA_real_, prep$total_delta_len),
                pi = rep(NA_real_, prep$C)))
  }
  list(
    delta = rstan::monitor(delta.chain, print = FALSE)[, "Rhat"],
    pi = rstan::monitor(pi.chain, print = FALSE)[, "Rhat"]
  )
}

#' @title S3 Methods: vcov
#'
#' @description
#' Returns the variance--covariance matrix of the estimated model
#' parameters from a fitted \pkg{ForceChoice} model object. The package stores
#' marginal standard errors rather than a joint covariance estimate, so the
#' returned matrix is \eqn{\mathrm{diag}(\mathrm{SE}^2)} for every backend.
#'
#' @param object A fitted model object of class \code{"MIRT"},
#'   \code{"MGPCM"}, \code{"MGGUM"}, \code{"FCMIRT"}, \code{"FCDCM"},
#'   \code{"FCGDINA"}, \code{"FCGGUM"}, or \code{"TIRT"}.
#' @param ... Additional arguments (currently ignored).
#'
#' @return A square numeric matrix. Rows and columns correspond to the
#'   free model parameters in the package's canonical parameter map.
#'
#' @details
#' Standard errors reflect the active estimation backend. Off-diagonal
#' covariances are not retained in fitted objects and are therefore reported
#' as zero. Parameters whose standard errors were not estimated have
#' \code{NA} diagonal entries.
#'
#' @name vcov
NULL


#' @title S3 Methods: confint
#'
#' @description
#' Computes Wald-type confidence intervals for the model parameters of a
#' fitted \pkg{ForceChoice} model object. Intervals are constructed as
#' \eqn{\hat{\psi}_k \pm z_{1-\alpha/2}\,\mathrm{SE}_k}, where
#' \eqn{\hat{\psi}_k} is the point estimate and \eqn{\mathrm{SE}_k} is
#' the standard error of the \eqn{k}-th free parameter.
#'
#' @param object A fitted model object.
#' @param parm Optional numeric indices or character names selecting free
#'   parameters. By default, intervals are returned for all free parameters.
#' @param level Numeric; confidence level (default: \code{0.95} for 95%
#'   intervals).
#' @param ... Additional arguments (currently ignored).
#'
#' @return A numeric matrix with two columns, \code{2.5%} and
#'   \code{97.5%} (column names adjust automatically for non-default
#'   \code{level}). Each row corresponds to one free parameter.
#'
#' @name confint
NULL


#' @title S3 Methods: nobs
#'
#' @description
#' Extracts the number of observations (persons) from a fitted
#' \pkg{ForceChoice} model object. Returns the number of rows in the
#' response matrix used for model fitting.
#'
#' @param object A fitted model object.
#' @param ... Additional arguments (currently ignored).
#'
#' @return An integer: the sample size \eqn{N}.
#'
#' @name nobs
#' @importFrom stats nobs
NULL


#' @title S3 Methods: deviance
#'
#' @description
#' Returns the model deviance, defined as \eqn{-2 \ell} where \eqn{\ell}
#' is the marginal log-likelihood evaluated at the parameter estimates.
#'
#' @param object A fitted model object.
#' @param ... Additional arguments (currently ignored).
#'
#' @return A numeric scalar: \eqn{-2 \log L}.
#'
#' @name deviance
NULL


# ---- Internal helpers ----

.fc_matrix_parameter_map <- function(est, se, free = NULL, prefix = "par") {
  est <- as.matrix(est)
  if (is.null(se)) {
    se <- matrix(NA_real_, nrow(est), ncol(est), dimnames = dimnames(est))
  } else {
    se <- as.matrix(se)
  }
  if (!identical(dim(est), dim(se))) {
    stop("Estimate and standard-error matrices have different dimensions.",
         call. = FALSE)
  }
  if (is.null(free)) free <- matrix(TRUE, nrow(est), ncol(est))
  free <- as.matrix(free)
  if (!identical(dim(est), dim(free))) {
    stop("Estimate and free-parameter matrices have different dimensions.",
         call. = FALSE)
  }

  idx <- which(free, arr.ind = TRUE)
  if (nrow(idx) == 0L) {
    return(data.frame(name = character(), estimate = numeric(), se = numeric()))
  }
  row_labels <- rownames(est)
  if (is.null(row_labels)) row_labels <- as.character(seq_len(nrow(est)))
  col_labels <- colnames(est)
  if (is.null(col_labels)) col_labels <- as.character(seq_len(ncol(est)))
  data.frame(
    name = paste0(prefix, ".", col_labels[idx[, 2L]], "[",
                  row_labels[idx[, 1L]], "]"),
    estimate = as.numeric(est[free]),
    se = as.numeric(se[free]),
    stringsAsFactors = FALSE
  )
}

.fc_vector_parameter_map <- function(est, se = NULL, free = NULL,
                                     prefix) {
  labels <- names(est)
  est <- as.numeric(est)
  if (is.null(se)) se <- rep(NA_real_, length(est))
  se <- as.numeric(se)
  if (length(se) != length(est)) {
    stop("Estimate and standard-error vectors have different lengths.",
         call. = FALSE)
  }
  if (is.null(free)) free <- rep(TRUE, length(est))
  if (length(free) != length(est)) {
    stop("Estimate and free-parameter vectors have different lengths.",
         call. = FALSE)
  }
  if (is.null(labels)) labels <- as.character(seq_along(est))
  data.frame(
    name = paste0(prefix, "[", labels[free], "]"),
    estimate = est[free],
    se = se[free],
    stringsAsFactors = FALSE
  )
}

.fc_corr_parameter_map <- function(object) {
  Corr <- object$Corr$est
  if (is.null(Corr) || nrow(as.matrix(Corr)) < 2L) {
    return(data.frame(name = character(), estimate = numeric(), se = numeric()))
  }
  Corr <- as.matrix(Corr)
  Corr.se <- object$Corr$se
  if (is.null(Corr.se)) {
    Corr.se <- matrix(NA_real_, nrow(Corr), ncol(Corr),
                      dimnames = dimnames(Corr))
  }
  .fc_matrix_parameter_map(
    Corr, Corr.se, free = lower.tri(Corr), prefix = "Corr"
  )
}

.fc_fcgdina_stan_delta_free_map <- function(object) {
  rank <- object$delta.identification$rank
  if (!is.numeric(rank) || length(rank) != 1L ||
      !is.finite(rank) || rank < 1L) {
    stop("FCGDINA Stan object does not contain a valid delta rank.",
         call. = FALSE)
  }
  rank <- as.integer(rank)

  draws <- object$MCMC.obj$delta_free
  if (!is.null(draws)) {
    draws <- as.matrix(draws)
    if (ncol(draws) != rank && nrow(draws) == rank) {
      draws <- t(draws)
    }
    if (ncol(draws) != rank) {
      stop("FCGDINA Stan 'delta_free' draws are inconsistent with rank.",
           call. = FALSE)
    }
    est <- colMeans(draws)
    se <- if (nrow(draws) > 1L) apply(draws, 2L, stats::sd) else
      rep(NA_real_, rank)
  } else if (!is.null(object$stan.obj)) {
    stan.sum <- rstan::summary(object$stan.obj, pars = "delta_free")$summary
    if (nrow(stan.sum) != rank) {
      stop("FCGDINA Stan 'delta_free' summary is inconsistent with rank.",
           call. = FALSE)
    }
    est <- stan.sum[, "mean"]
    se <- stan.sum[, "sd"]
  } else {
    stop("FCGDINA Stan object does not contain delta_free draws.",
         call. = FALSE)
  }

  data.frame(
    name = paste0("delta_free[", seq_len(rank), "]"),
    estimate = as.numeric(est),
    se = as.numeric(se),
    stringsAsFactors = FALSE
  )
}

.fc_parameter_map <- function(object) {
  cls <- class(object)[1L]
  if (is.null(object$npar) || length(object$npar) != 1L ||
      !is.finite(object$npar)) {
    stop("Fitted object does not contain a valid 'npar'.", call. = FALSE)
  }

  map <- switch(cls,
    MIRT =, MGPCM =, MGGUM =, FCMIRT =, FCGGUM = {
      .fc_matrix_parameter_map(
        object$par$est, object$par$se, object$par$free, prefix = "par"
      )
    },
    FCDCM = {
      rbind(
        .fc_matrix_parameter_map(
          object$delta$est, object$delta$se, prefix = "delta"
        ),
        .fc_matrix_parameter_map(
          object$par$est, object$par$se, object$par$free, prefix = "par"
        )
      )
    },
    FCGDINA = {
      delta_map <- if (!is.null(object$delta.identification)) {
        .fc_fcgdina_stan_delta_free_map(object)
      } else {
        delta_maps <- Map(
          function(est, se, i) {
            .fc_vector_parameter_map(est, se, prefix = paste0("delta", i))
          },
          object$delta$est, object$delta$se, seq_along(object$delta$est)
        )
        do.call(rbind, delta_maps)
      }
      C <- length(object$pi)
      pi_map <- .fc_vector_parameter_map(
        object$pi, object$pi.se, free = seq_len(C) < C, prefix = "pi"
      )
      rbind(delta_map, pi_map)
    },
    TIRT = {
      rbind(
        .fc_matrix_parameter_map(
          object$par$est, object$par$se, object$par$free, prefix = "par"
        ),
        .fc_vector_parameter_map(
          object$gamma$est, object$gamma$se, object$gamma$free,
          prefix = "gamma"
        )
      )
    },
    stop("Unsupported fitted-object class: ", cls, call. = FALSE)
  )

  remaining <- as.integer(object$npar) - nrow(map)
  if (remaining > 0L && cls %in% c(
    "MIRT", "MGPCM", "MGGUM", "FCMIRT", "FCGGUM", "TIRT"
  )) {
    corr_map <- .fc_corr_parameter_map(object)
    if (nrow(corr_map) != remaining) {
      stop("Free correlation-parameter count is inconsistent with 'npar'.",
           call. = FALSE)
    }
    map <- rbind(map, corr_map)
  }
  if (nrow(map) != as.integer(object$npar)) {
    stop("Canonical free-parameter map is inconsistent with 'npar'.",
         call. = FALSE)
  }
  rownames(map) <- NULL
  map
}

.fc_vcov_impl <- function(object, ...) {
  map <- .fc_parameter_map(object)
  out <- diag(map$se^2, nrow = nrow(map))
  dimnames(out) <- list(map$name, map$name)
  out
}

.fc_confint_impl <- function(object, parm = NULL, level = 0.95, ...) {
  if (length(level) != 1L || !is.numeric(level) || is.na(level) ||
      level <= 0 || level >= 1) {
    stop("'level' must be a numeric scalar strictly between 0 and 1.",
         call. = FALSE)
  }
  map <- .fc_parameter_map(object)
  if (!is.null(parm)) {
    if (is.character(parm)) {
      idx <- match(parm, map$name)
      if (anyNA(idx)) {
        stop("Unknown parameter name(s): ",
             paste(parm[is.na(idx)], collapse = ", "), call. = FALSE)
      }
    } else if (is.numeric(parm) && all(is.finite(parm)) &&
               all(parm == floor(parm)) && all(parm >= 1L) &&
               all(parm <= nrow(map))) {
      idx <- as.integer(parm)
    } else {
      stop("'parm' must contain valid parameter names or indices.",
           call. = FALSE)
    }
    map <- map[idx, , drop = FALSE]
  }
  alpha <- (1 - level) / 2
  z <- stats::qnorm(1 - alpha)
  probs <- c(alpha, 1 - alpha)
  out <- cbind(map$estimate - z * map$se, map$estimate + z * map$se)
  colnames(out) <- paste0(format(100 * probs, trim = TRUE), " %")
  rownames(out) <- map$name
  out
}

.fc_nobs_impl <- function(object, ...) {
  response <- object$response
  if (is.null(response)) response <- object$arguments$data
  if (is.null(response)) response <- object$arguments$response
  if (is.null(response)) {
    stop("Fitted object does not contain response data.", call. = FALSE)
  }
  nrow(response)
}

.fc_deviance_impl <- function(object, ...) {
  value <- object$logLik
  if (is.null(value)) value <- logLik(object)
  -2 * as.numeric(value)
}


# ===========================================================================
# vcov
# ===========================================================================

#' @describeIn vcov Variance--covariance matrix of parameter estimates
#'   for \code{MIRT} objects.
#' @method vcov MIRT
#' @export
vcov.MIRT    <- .fc_vcov_impl

#' @describeIn vcov Variance--covariance matrix for \code{MGPCM} objects.
#' @method vcov MGPCM
#' @export
vcov.MGPCM   <- .fc_vcov_impl

#' @describeIn vcov Variance--covariance matrix for \code{MGGUM} objects.
#' @method vcov MGGUM
#' @export
vcov.MGGUM   <- .fc_vcov_impl

#' @describeIn vcov Variance--covariance matrix for \code{FCMIRT} objects.
#' @method vcov FCMIRT
#' @export
vcov.FCMIRT  <- .fc_vcov_impl

#' @describeIn vcov Variance--covariance matrix for \code{FCDCM} objects.
#' @method vcov FCDCM
#' @export
vcov.FCDCM   <- .fc_vcov_impl

#' @describeIn vcov Variance--covariance matrix for \code{FCGDINA} objects.
#' @method vcov FCGDINA
#' @export
vcov.FCGDINA <- .fc_vcov_impl

#' @describeIn vcov Variance--covariance matrix for \code{FCGGUM} objects.
#' @method vcov FCGGUM
#' @export
vcov.FCGGUM  <- .fc_vcov_impl

#' @describeIn vcov Variance--covariance matrix for \code{TIRT} objects.
#' @method vcov TIRT
#' @export
vcov.TIRT    <- .fc_vcov_impl


# ===========================================================================
# confint
# ===========================================================================

#' @describeIn confint Confidence intervals for \code{MIRT} parameters.
#' @method confint MIRT
#' @export
confint.MIRT   <- .fc_confint_impl

#' @describeIn confint Confidence intervals for \code{MGPCM} parameters.
#' @method confint MGPCM
#' @export
confint.MGPCM  <- .fc_confint_impl

#' @describeIn confint Confidence intervals for \code{MGGUM} parameters.
#' @method confint MGGUM
#' @export
confint.MGGUM  <- .fc_confint_impl

#' @describeIn confint Confidence intervals for \code{FCMIRT} parameters.
#' @method confint FCMIRT
#' @export
confint.FCMIRT <- .fc_confint_impl

#' @describeIn confint Confidence intervals for \code{FCDCM} parameters.
#' @method confint FCDCM
#' @export
confint.FCDCM  <- .fc_confint_impl

#' @describeIn confint Confidence intervals for \code{FCGDINA} parameters.
#' @method confint FCGDINA
#' @export
confint.FCGDINA <- .fc_confint_impl

#' @describeIn confint Confidence intervals for \code{FCGGUM} parameters.
#' @method confint FCGGUM
#' @export
confint.FCGGUM <- .fc_confint_impl

#' @describeIn confint Confidence intervals for \code{TIRT} parameters.
#' @method confint TIRT
#' @export
confint.TIRT   <- .fc_confint_impl


# ===========================================================================
# nobs
# ===========================================================================

#' @describeIn nobs Number of observations (persons) for \code{MIRT} objects.
#' @method nobs MIRT
#' @export
nobs.MIRT    <- .fc_nobs_impl

#' @describeIn nobs Number of observations for \code{MGPCM} objects.
#' @method nobs MGPCM
#' @export
nobs.MGPCM   <- .fc_nobs_impl

#' @describeIn nobs Number of observations for \code{MGGUM} objects.
#' @method nobs MGGUM
#' @export
nobs.MGGUM   <- .fc_nobs_impl

#' @describeIn nobs Number of observations for \code{FCMIRT} objects.
#' @method nobs FCMIRT
#' @export
nobs.FCMIRT  <- .fc_nobs_impl

#' @describeIn nobs Number of observations for \code{FCDCM} objects.
#' @method nobs FCDCM
#' @export
nobs.FCDCM   <- .fc_nobs_impl

#' @describeIn nobs Number of observations for \code{FCGDINA} objects.
#' @method nobs FCGDINA
#' @export
nobs.FCGDINA <- .fc_nobs_impl

#' @describeIn nobs Number of observations for \code{FCGGUM} objects.
#' @method nobs FCGGUM
#' @export
nobs.FCGGUM  <- .fc_nobs_impl

#' @describeIn nobs Number of observations for \code{TIRT} objects.
#' @method nobs TIRT
#' @export
nobs.TIRT    <- .fc_nobs_impl


# ===========================================================================
# deviance
# ===========================================================================

#' @describeIn deviance Model deviance (\eqn{-2 \ell}) for \code{MIRT} objects.
#' @method deviance MIRT
#' @export
deviance.MIRT   <- .fc_deviance_impl

#' @describeIn deviance Model deviance for \code{MGPCM} objects.
#' @method deviance MGPCM
#' @export
deviance.MGPCM  <- .fc_deviance_impl

#' @describeIn deviance Model deviance for \code{MGGUM} objects.
#' @method deviance MGGUM
#' @export
deviance.MGGUM  <- .fc_deviance_impl

#' @describeIn deviance Model deviance for \code{FCMIRT} objects.
#' @method deviance FCMIRT
#' @export
deviance.FCMIRT <- .fc_deviance_impl

#' @describeIn deviance Model deviance for \code{FCDCM} objects.
#' @method deviance FCDCM
#' @export
deviance.FCDCM  <- .fc_deviance_impl

#' @describeIn deviance Model deviance for \code{FCGDINA} objects.
#' @method deviance FCGDINA
#' @export
deviance.FCGDINA <- .fc_deviance_impl

#' @describeIn deviance Model deviance for \code{FCGGUM} objects.
#' @method deviance FCGGUM
#' @export
deviance.FCGGUM <- .fc_deviance_impl

#' @describeIn deviance Model deviance for \code{TIRT} objects.
#' @method deviance TIRT
#' @export
deviance.TIRT   <- .fc_deviance_impl

################################# FCGDINA EM ####################################

fit.FCGDINA.EM <- function(data, Q.matrix, model, block.items, fc.type,
                           control.model, control.method, .call) {
  call <- if (is.null(.call)) match.call() else .call
  control <- fc_as_control_list(control.model, "control.model")
  control.method <- fc_as_control_list(control.method, "control.method")
  common.method <- fit_common_method_control(control.method)
  method <- fcgdina_em_method_control(control.method)
  fcgdina_em_check_method_control(method)

  prep <- fcgdina_prepare(data, Q.matrix, model, block.items, fc.type)

  bounds <- list(
    delta.lower = get_ctrl("delta.lower", -4, control.method),
    delta.upper = get_ctrl("delta.upper", 4, control.method)
  )
  if (!is.finite(bounds$delta.lower) || !is.finite(bounds$delta.upper) ||
      bounds$delta.lower >= bounds$delta.upper) {
    stop("'delta.lower' must be smaller than 'delta.upper'.", call. = FALSE)
  }

  prior.init <- rep(1.0 / prep$C, prep$C)
  set.seed(common.method$seed)

  fit <- fcgdina_em_fit_core(
    prep = prep,
    delta.init = fcgdina_initial_delta(prep),
    prior.init = prior.init,
    method = method,
    bounds = bounds,
    vis = common.method$vis
  )

  pi.est <- fit$prior

  delta.vec.est <- unlist(fit$delta, use.names = FALSE)
  delta.vec.se <- rep(NA_real_, length(delta.vec.est))
  delta.store <- NULL
  bootstrap <- NULL

  if (method$estimate.se) {
    bootstrap <- fcgdina_em_bootstrap_se(
      prep = prep,
      delta.est = fit$delta,
      prior.init = pi.est,
      method = method,
      bounds = bounds,
      vis = common.method$vis
    )
    delta.vec.se <- bootstrap$se
    if (!is.null(bootstrap$delta.mat)) {
      ok <- colSums(is.finite(bootstrap$delta.mat)) == nrow(bootstrap$delta.mat)
      if (any(ok)) {
        delta.store <- fcgdina_delta_store(bootstrap$delta.mat[, ok, drop = FALSE], prep)
      }
    }
  }

  delta.list.est <- fit$delta
  delta.list.se <- fcgdina_unpack_delta(delta.vec.se, prep)
  delta.list.Rhat <- fcgdina_unpack_delta(rep(NA_real_, length(delta.vec.est)), prep)

  prob.class <- fcgdina_prob_class(prep, delta.list.est)
  class.post.group <- fcgdina_class_posterior(prob.class, prep, pi.est,
                                              grouped = TRUE)
  class.post <- fcgdina_expand_group_matrix(class.post.group, prep)
  alpha.prob <- class.post %*% prep$alpha.patterns
  colnames(alpha.prob) <- paste0("alpha", seq_len(prep$D))
  rownames(alpha.prob) <- rownames(prep$response)
  alpha.est <- (alpha.prob > 0.5) * 1L
  storage.mode(alpha.est) <- "integer"
  colnames(class.post) <- pattern_key(prep$alpha.patterns)
  rownames(class.post) <- rownames(prep$response)

  results <- list(
    npar = prep$total_delta_len + prep$C - 1L,
    method = "EM",
    alpha = list(est = alpha.est, se = matrix(NA_real_, prep$N, prep$D),
                 Rhat = matrix(NA_real_, prep$N, prep$D),
                 prob = alpha.prob),
    class.post = class.post,
    delta = list(est = delta.list.est, se = delta.list.se,
                 Rhat = delta.list.Rhat),
    pi = pi.est,
    delta.store = delta.store,
    Q.matrix = prep$Q.matrix,
    model = prep$model,
    block.items = prep$block.items,
    patterns = prep$patterns,
    patterns.total = prep$patterns.total,
    design.matrix.list = prep$design.matrix.list,
    alpha.patterns = prep$alpha.patterns,
    fc.type = prep$fc.type,
    response = prep$response,
    stan.obj = NULL,
    MCMC.obj = NULL,
    iStEM = NULL,
    EM = list(
      iter = fit$iter,
      maxitr = method$maxitr,
      converged = fit$converged,
      tol = method$tol,
      par.tol = method$par.tol,
      logLik.trace = fit$trace,
      final.logLik = fit$logLik,
      final.ll.change = fit$ll.change,
      final.max.delta.change = fit$max.delta.change,
      bootstrap = bootstrap,
      control = method
    ),
    call = call,
    arguments = list(
      data = data,
      Q.matrix = prep$Q.matrix,
      block.items = prep$block.items,
      model = prep$model,
      fc.type = prep$fc.type,
      method = "EM",
      cores = common.method$cores,
      vis = common.method$vis,
      seed = common.method$seed,
      control.model = control,
      control.method = fit_effective_method_control(
        control.method, method = utils::modifyList(method, bounds),
        common = common.method)
    )
  )
  class(results) <- "FCGDINA"
  results$logLik <- logLik.FCGDINA(results)
  results
}

fcgdina_em_method_control <- function(control.method) {
  control.method <- fc_as_control_list(control.method, "control.method")
  estimate.se <- isTRUE(get_ctrl("estimate.se", FALSE, control.method))
  bootstrap <- as.integer(
    get_ctrl("bootstrap", if (estimate.se) 100L else 0L, control.method)
  )
  list(
    maxitr = as.integer(get_ctrl("maxitr", 500L, control.method)),
    minitr = as.integer(get_ctrl("minitr", 2L, control.method)),
    tol = get_ctrl("tol", 1e-6, control.method),
    par.tol = get_ctrl("par.tol", 1e-4, control.method),
    optim.maxit = as.integer(get_ctrl("optim.maxit", 200L, control.method)),
    estimate.se = estimate.se,
    bootstrap = bootstrap
  )
}

fcgdina_em_check_method_control <- function(method) {
  if (length(method$maxitr) != 1L || is.na(method$maxitr) ||
      method$maxitr < 1L) {
    stop("'maxitr' in 'control.method' must be a positive integer.",
         call. = FALSE)
  }
  if (length(method$minitr) != 1L || is.na(method$minitr) ||
      method$minitr < 1L || method$minitr > method$maxitr) {
    stop("'minitr' in 'control.method' must be between 1 and 'maxitr'.",
         call. = FALSE)
  }
  if (!is.numeric(method$tol) || length(method$tol) != 1L ||
      !is.finite(method$tol) || method$tol <= 0) {
    stop("'tol' in 'control.method' must be positive.", call. = FALSE)
  }
  if (!is.numeric(method$par.tol) || length(method$par.tol) != 1L ||
      !is.finite(method$par.tol) || method$par.tol <= 0) {
    stop("'par.tol' in 'control.method' must be positive.", call. = FALSE)
  }
  if (length(method$optim.maxit) != 1L || is.na(method$optim.maxit) ||
      method$optim.maxit < 1L) {
    stop("'optim.maxit' in 'control.method' must be a positive integer.",
         call. = FALSE)
  }
  if (length(method$bootstrap) != 1L || is.na(method$bootstrap) ||
      method$bootstrap < 0L) {
    stop("'bootstrap' in 'control.method' must be a non-negative integer.",
         call. = FALSE)
  }
  if (method$estimate.se && method$bootstrap < 2L) {
    stop("'bootstrap' in 'control.method' must be at least 2 when ",
         "'estimate.se = TRUE'.", call. = FALSE)
  }
  invisible(NULL)
}

fcgdina_em_fit_core <- function(prep, delta.init, prior.init, method, bounds,
                                vis = FALSE) {
  delta.current <- delta.init
  prior.current <- prior.init
  par.current <- unlist(delta.current, use.names = FALSE)
  ll.prev <- -Inf
  trace <- data.frame(
    iter = integer(0L),
    logLik = numeric(0L),
    ll.change = numeric(0L),
    max.delta.change = numeric(0L)
  )
  converged <- FALSE
  iter.done <- 0L
  ll.change <- Inf
  max.delta.change <- Inf

  if (vis) cat("\nFCGDINA EM iterations:\n")
  for (iter in seq_len(method$maxitr)) {
    # E-step: posterior class probabilities given current (delta, pi)
    prob.class <- fcgdina_prob_class(prep, delta.current)
    class.post <- fcgdina_class_posterior(prob.class, prep, prior.current,
                                          grouped = TRUE)
    class.weight <- class.post * prep$response.count

    # M-step for delta (ECM: block-wise conditional maximization)
    delta.next <- fcgdina_fit_delta(
      prep = prep,
      delta.current = delta.current,
      class.weight = class.weight,
      method = method,
      bounds = bounds
    )

    # M-step for structural parameters pi
    prior.next <- colSums(class.weight) / prep$N

    # Convergence assessment with updated parameters
    par.next <- unlist(delta.next, use.names = FALSE)
    prob.next <- fcgdina_prob_class(prep, delta.next)
    ll <- fcgdina_marginal_loglik(prob.next, prep, prior.next, grouped = TRUE)

    ll.change <- if (is.finite(ll.prev)) abs(ll - ll.prev) else Inf
    max.delta.change <- max(abs(par.next - par.current))
    trace[iter, ] <- list(iter, ll, ll.change, max.delta.change)

    if (vis) {
      istem_progress(
        "  iter %04d/%04d | logLik %12.4f | dLL %10.6f | max dpar %10.6f",
        iter, method$maxitr, ll, ll.change, max.delta.change)
    }

    delta.current <- delta.next
    prior.current <- prior.next
    par.current <- par.next
    ll.prev <- ll
    iter.done <- iter

    if (iter >= method$minitr &&
        (ll.change < method$tol || max.delta.change < method$par.tol)) {
      converged <- TRUE
      break
    }
  }
  if (vis) {
    istem_progress(
      "  iter %04d/%04d | logLik %12.4f | dLL %10.6f | max dpar %10.6f",
      iter.done, method$maxitr, ll.prev, ll.change, max.delta.change,
      newline = TRUE)
  }

  list(
    delta = delta.current,
    prior = prior.current,
    iter = iter.done,
    converged = converged,
    trace = trace,
    logLik = ll.prev,
    ll.change = ll.change,
    max.delta.change = max.delta.change
  )
}

fcgdina_em_bootstrap_se <- function(prep, delta.est, prior.init, method, bounds,
                                    vis = FALSE) {
  R <- method$bootstrap
  delta.mat <- matrix(NA_real_, nrow = prep$total_delta_len, ncol = R)
  error <- rep(NA_character_, R)

  if (vis) cat("\nFCGDINA EM bootstrap SE:\n")
  for (r in seq_len(R)) {
    idx <- sample.int(prep$N, prep$N, replace = TRUE)
    response.r <- prep$response[idx, , drop = FALSE]
    prep.r <- fcgdina_prepare(
      data = response.r,
      Q.matrix = prep$Q.matrix,
      model = prep$model,
      block.items = prep$block.items,
      fc.type = prep$fc.type
    )
    fit.r <- tryCatch(
      fcgdina_em_fit_core(
        prep = prep.r,
        delta.init = delta.est,
        prior.init = prior.init,
        method = method,
        bounds = bounds,
        vis = FALSE
      ),
      error = function(e) e
    )
    if (inherits(fit.r, "error")) {
      error[r] <- conditionMessage(fit.r)
    } else {
      delta.mat[, r] <- unlist(fit.r$delta, use.names = FALSE)
    }

    if (vis) {
      ok <- sum(colSums(is.finite(delta.mat)) == nrow(delta.mat))
      istem_progress("  bootstrap %04d/%04d | successful %04d",
                     r, R, ok)
    }
  }
  if (vis) {
    ok <- sum(colSums(is.finite(delta.mat)) == nrow(delta.mat))
    istem_progress("  bootstrap %04d/%04d | successful %04d",
                   R, R, ok, newline = TRUE)
  }

  ok <- colSums(is.finite(delta.mat)) == nrow(delta.mat)
  se <- if (sum(ok) >= 2L) {
    apply(delta.mat[, ok, drop = FALSE], 1L, stats::sd)
  } else {
    rep(NA_real_, prep$total_delta_len)
  }

  list(
    R = R,
    successful = sum(ok),
    failed = R - sum(ok),
    se = se,
    delta.mat = delta.mat,
    error = error[!is.na(error)]
  )
}

#' @title S3 Methods: extract
#'
#' @description
#' A generic S3 extractor function designed to retrieve internal components
#' from fitted model objects produced by the \pkg{ForceChoice} package. This
#' function provides a consistent interface across all eight model classes,
#' allowing users to access estimated parameters, fit statistics, latent
#' trait estimates, convergence diagnostics, and model configuration data
#' without directly manipulating the internal list structure.
#'
#' @param object An object of one of the following classes:
#'   \itemize{
#'     \item \code{"MIRT"} --- Multidimensional IRT model.
#'     \item \code{"MGPCM"} --- Multidimensional Generalized Partial Credit Model.
#'     \item \code{"MGGUM"} --- Multidimensional Generalized Graded Unfolding Model.
#'     \item \code{"FCMIRT"} --- Forced-Choice Multidimensional IRT model.
#'     \item \code{"FCDCM"} --- Forced-Choice Diagnostic Classification Model.
#'     \item \code{"FCGDINA"} --- Forced-Choice GDINA model.
#'     \item \code{"FCGGUM"} --- Forced-Choice Generalized Graded Unfolding Model.
#'     \item \code{"TIRT"} --- Thurstonian IRT for Forced-Choice.
#'   }
#' @param what A character string specifying the name of the component to
#'   extract. Valid choices depend on the class of \code{object}. See Details
#'   section for full listings.
#' @param ... Additional arguments passed to methods (currently ignored).
#'
#' @return The requested component. Return type varies depending on
#'   \code{what} and the class of \code{object}. If an invalid \code{what}
#'   is provided, an informative error is thrown listing valid options.
#'
#' @details
#' This function supports extraction from the eight ForceChoice model classes.
#' Below are the available components for each:
#'
#' \describe{
#'   \item{\strong{\code{MIRT}, \code{MGPCM}, \code{MGGUM}}}{
#'     Traditional (single-stimulus) models. Available components:
#'     \describe{
#'       \item{\code{par}}{List (\code{est}, \code{se}, \code{Rhat}, \code{free})
#'         of item parameter arrays.}
#'       \item{\code{theta}}{List (\code{est}, \code{se}, \code{Rhat}) of
#'         \eqn{N \times D} person parameter matrices.}
#'       \item{\code{Corr}}{List (\code{est}, \code{se}, \code{Rhat}) of
#'         \eqn{D \times D} inter-trait correlation matrices.}
#'       \item{\code{logLik}}{Marginal log-likelihood (class \code{"logLik"}).}
#'       \item{\code{npar}}{Number of free parameters.}
#'       \item{\code{method}}{Estimation method (\code{"stan"} or \code{"iStEM"}).}
#'       \item{\code{Q.matrix}}{The \eqn{I \times D} Q-matrix.}
#'       \item{\code{length.poly}}{Vector of category counts per item
#'         (MGPCM, MGGUM only).}
#'       \item{\code{stan.obj}}{The \code{stanfit} object (Stan only).}
#'       \item{\code{MCMC.obj}}{The MCMC extract list (Stan only).}
#'       \item{\code{iStEM}}{List of iStEM diagnostics (iStEM only).}
#'       \item{\code{call}}{The matched call.}
#'       \item{\code{arguments}}{List of arguments used for fitting.}
#'     }
#'   }
#'
#'   \item{\strong{\code{FCMIRT}, \code{FCGGUM}}}{
#'     Forced-choice models. In addition to the traditional-model components
#'     listed above, the following are available:
#'     \describe{
#'       \item{\code{block.items}}{List of item indices in each block.}
#'       \item{\code{response}}{The \eqn{N \times B} forced-choice response matrix.}
#'       \item{\code{patterns}}{List of permissible ranking patterns per block.}
#'       \item{\code{patterns.total}}{List of all possible ranking patterns per block.}
#'       \item{\code{fc.type}}{Character vector of forced-choice formats
#'         (\code{"RANK"}, \code{"MOLE"}, \code{"PICK"}).}
#'     }
#'   }
#'
#'   \item{\strong{\code{FCDCM}}}{
#'     Forced-Choice DCM. Available components:
#'     \describe{
#'       \item{\code{par}}{List (\code{est}, \code{se}, \code{Rhat}, \code{free})
#'         of block \eqn{\eta} parameter matrices (\eqn{B \times 2}).}
#'       \item{\code{delta}}{List (\code{est}, \code{se}, \code{Rhat}) of
#'         higher-order \eqn{\delta} parameters (\eqn{D \times 2}).}
#'       \item{\code{theta}}{List (\code{est}, \code{se}, \code{Rhat}) of
#'         \eqn{N \times 1} higher-order trait estimates.}
#'       \item{\code{alpha}}{Posterior mean attribute profile
#'         (\eqn{N \times D}).}
#'       \item{\code{alpha.patterns}}{Full enumeration of \eqn{2^D}
#'         attribute mastery patterns.}
#'       \item{\code{zeta.patterns}}{Condensation outputs for each pattern.}
#'       \item{\code{dcm.type}}{DCM condensation rule (\code{"DINA"} or
#'         \code{"DINO"}).}
#'       \item{\code{response}}{The \eqn{N \times B} binary block response matrix.}
#'       \item{\code{block.items}}{List of two-statement blocks.}
#'       \item{\code{patterns}}{List of binary response patterns per block.}
#'       \item{\code{logLik}}{Marginal log-likelihood (class \code{"logLik"}).}
#'       \item{\code{npar}}{Number of free parameters.}
#'       \item{\code{method}}{Estimation method.}
#'       \item{\code{Q.matrix}}{The \eqn{I \times D} Q-matrix.}
#'       \item{\code{stan.obj}, \code{MCMC.obj}, \code{iStEM}, \code{EM}}{
#'         Estimation backend objects.}
#'       \item{\code{call}}{The matched call.}
#'       \item{\code{arguments}}{List of arguments used for fitting.}
#'     }
#'   }
#'
#'   \item{\strong{\code{FCGDINA}}}{
#'     Forced-Choice GDINA. Available components:
#'     \describe{
#'       \item{\code{delta}}{List (\code{est}, \code{se}, \code{Rhat}) of
#'         item-level CDM delta parameters.}
#'       \item{\code{alpha}}{Posterior attribute summaries, including
#'         posterior class probabilities when available.}
#'       \item{\code{alpha.patterns}}{Full enumeration of \eqn{2^D}
#'         attribute mastery patterns.}
#'       \item{\code{design.matrix.list}}{Per-item CDM design matrices.}
#'       \item{\code{response}, \code{block.items}, \code{patterns},
#'         \code{patterns.total}, \code{fc.type}}{Forced-choice data
#'         structures.}
#'     }
#'   }
#'
#'   \item{\strong{\code{TIRT}}}{
#'     Thurstonian IRT. Available components:
#'     \describe{
#'       \item{\code{par}}{List (\code{est}, \code{se}, \code{Rhat}, \code{free})
#'         of statement-level parameter matrices (\eqn{I \times (D+1)}).}
#'       \item{\code{theta}}{List (\code{est}, \code{se}, \code{Rhat}) of
#'         \eqn{N \times D} person parameter matrices.}
#'       \item{\code{gamma}}{List (\code{est}, \code{se}, \code{Rhat},
#'         \code{free}) of pairwise \eqn{\gamma} parameters.}
#'       \item{\code{gamma.matrix}}{List (\code{est}, \code{se}, \code{Rhat})
#'         of \eqn{I \times I} skew-symmetric \eqn{\gamma} matrices.}
#'       \item{\code{Corr}}{List (\code{est}, \code{se}, \code{Rhat}) of
#'         \eqn{D \times D} inter-trait correlation matrices.}
#'       \item{\code{pairs.matrix}}{Matrix of pairwise comparison indices.}
#'       \item{\code{pairs.value}}{Person-specific pair data (MOLE/PICK).}
#'       \item{\code{response}}{The \eqn{N \times I_{pairs}} pairwise
#'         response matrix.}
#'       \item{\code{block.items}}{List of item indices in each block.}
#'       \item{\code{fc.type}}{Character vector of forced-choice formats.}
#'       \item{\code{Q.matrix}}{The statement-level Q-matrix.}
#'       \item{\code{logLik}}{Marginal log-likelihood (class \code{"logLik"}).}
#'       \item{\code{npar}}{Number of free parameters.}
#'       \item{\code{method}}{Estimation method.}
#'       \item{\code{stan.obj}, \code{MCMC.obj}, \code{iStEM}}{Estimation
#'         backend objects.}
#'       \item{\code{call}}{The matched call.}
#'       \item{\code{arguments}}{List of arguments used for fitting.}
#'     }
#'   }
#' }
#'
#' @examples
#' sim <- sim.data.MIRT(N = 20, I = 6, D = 2, model = "m2pl")
#' fit <- fit.MIRT(
#'   sim$response, model = "m2pl", D = 2, method = "iStEM",
#'   control.method = list(
#'     vis = FALSE, seed = 123,
#'     M = 2, B = 2, burnin.maxitr = 2,
#'     maxitr = 3, eps1 = 10, eps2 = 10,
#'     estimate.se = FALSE)
#' )
#'
#' extract(fit, "par")       # item parameter estimates
#' extract(fit, "theta")     # person trait estimates
#' extract(fit, "Corr")      # factor correlation matrix
#' extract(fit, "npar")      # number of free parameters
#' extract(fit, "logLik")    # marginal log-likelihood
#' extract(fit, "iStEM")     # iStEM convergence diagnostics
#'
#' @name extract
NULL

#' @export
extract <- function(object, what, ...) {
  UseMethod("extract")
}


# ===========================================================================
# extract.MIRT
# ===========================================================================

#' @describeIn extract Extract components from \code{MIRT} objects.
#' @method extract MIRT
#' @export
extract.MIRT <- function(object, what, ...) {
  choices <- c("par", "theta", "Corr", "logLik", "npar", "method",
               "Q.matrix", "stan.obj", "MCMC.obj", "iStEM",
               "call", "arguments")
  what <- match.arg(what, choices)
  switch(what,
    par       = object$par,
    theta     = object$theta,
    Corr      = object$Corr,
    logLik    = object$logLik,
    npar      = object$npar,
    method    = object$method,
    Q.matrix  = object$Q.matrix,
    stan.obj  = object$stan.obj,
    MCMC.obj  = object$MCMC.obj,
    iStEM     = object$iStEM,
    call      = object$call,
    arguments = object$arguments
  )
}

# ===========================================================================
# extract.MGPCM
# ===========================================================================

#' @describeIn extract Extract components from \code{MGPCM} objects.
#' @method extract MGPCM
#' @export
extract.MGPCM <- function(object, what, ...) {
  choices <- c("par", "theta", "Corr", "logLik", "npar", "method",
               "Q.matrix", "length.poly",
               "stan.obj", "MCMC.obj", "iStEM",
               "call", "arguments")
  what <- match.arg(what, choices)
  switch(what,
    par         = object$par,
    theta       = object$theta,
    Corr        = object$Corr,
    logLik      = object$logLik,
    npar        = object$npar,
    method      = object$method,
    Q.matrix    = object$Q.matrix,
    length.poly = object$length.poly,
    stan.obj    = object$stan.obj,
    MCMC.obj    = object$MCMC.obj,
    iStEM       = object$iStEM,
    call        = object$call,
    arguments   = object$arguments
  )
}

# ===========================================================================
# extract.MGGUM
# ===========================================================================

#' @describeIn extract Extract components from \code{MGGUM} objects.
#' @method extract MGGUM
#' @export
extract.MGGUM <- function(object, what, ...) {
  choices <- c("par", "theta", "Corr", "logLik", "npar", "method",
               "Q.matrix", "length.poly",
               "stan.obj", "MCMC.obj", "iStEM",
               "call", "arguments")
  what <- match.arg(what, choices)
  switch(what,
    par         = object$par,
    theta       = object$theta,
    Corr        = object$Corr,
    logLik      = object$logLik,
    npar        = object$npar,
    method      = object$method,
    Q.matrix    = object$Q.matrix,
    length.poly = object$length.poly,
    stan.obj    = object$stan.obj,
    MCMC.obj    = object$MCMC.obj,
    iStEM       = object$iStEM,
    call        = object$call,
    arguments   = object$arguments
  )
}

# ===========================================================================
# extract.FCMIRT
# ===========================================================================

#' @describeIn extract Extract components from \code{FCMIRT} objects.
#' @method extract FCMIRT
#' @export
extract.FCMIRT <- function(object, what, ...) {
  choices <- c("par", "theta", "Corr", "logLik", "npar", "method",
               "Q.matrix", "block.items", "response",
               "patterns", "patterns.total", "fc.type",
               "stan.obj", "MCMC.obj", "iStEM",
               "call", "arguments")
  what <- match.arg(what, choices)
  switch(what,
    par            = object$par,
    theta          = object$theta,
    Corr           = object$Corr,
    logLik         = object$logLik,
    npar           = object$npar,
    method         = object$method,
    Q.matrix       = object$Q.matrix,
    block.items    = object$block.items,
    response       = object$response,
    patterns       = object$patterns,
    patterns.total = object$patterns.total,
    fc.type        = object$fc.type,
    stan.obj       = object$stan.obj,
    MCMC.obj       = object$MCMC.obj,
    iStEM          = object$iStEM,
    call           = object$call,
    arguments      = object$arguments
  )
}

# ===========================================================================
# extract.FCDCM
# ===========================================================================

#' @describeIn extract Extract components from \code{FCDCM} objects.
#' @method extract FCDCM
#' @export
extract.FCDCM <- function(object, what, ...) {
  choices <- c("par", "delta", "theta", "alpha", "class.post",
               "alpha.patterns", "zeta.patterns", "dcm.type",
               "logLik", "npar", "method",
               "Q.matrix", "block.items", "response", "patterns",
               "stan.obj", "MCMC.obj", "iStEM",
               "call", "arguments")
  what <- match.arg(what, choices)
  switch(what,
    par             = object$par,
    delta           = object$delta,
    theta           = object$theta,
    alpha           = object$alpha,
    class.post      = object$class.post,
    alpha.patterns  = object$alpha.patterns,
    zeta.patterns   = object$zeta.patterns,
    dcm.type        = object$dcm.type,
    logLik          = object$logLik,
    npar            = object$npar,
    method          = object$method,
    Q.matrix        = object$Q.matrix,
    block.items     = object$block.items,
    response        = object$response,
    patterns        = object$patterns,
    stan.obj        = object$stan.obj,
    MCMC.obj        = object$MCMC.obj,
    iStEM           = object$iStEM,
    call            = object$call,
    arguments       = object$arguments
  )
}

# ===========================================================================
# extract.FCGDINA
# ===========================================================================

#' @describeIn extract Extract components from \code{FCGDINA} objects.
#' @method extract FCGDINA
#' @export
extract.FCGDINA <- function(object, what, ...) {
  choices <- c("delta", "alpha", "alpha.patterns", "design.matrix.list",
               "logLik", "npar", "method", "Q.matrix", "block.items",
               "response", "patterns", "patterns.total", "fc.type",
               "stan.obj", "MCMC.obj", "iStEM", "EM", "call", "arguments")
  what <- match.arg(what, choices)
  switch(what,
    delta             = object$delta,
    alpha             = object$alpha,
    alpha.patterns    = object$alpha.patterns,
    design.matrix.list = object$design.matrix.list,
    logLik            = object$logLik,
    npar              = object$npar,
    method            = object$method,
    Q.matrix          = object$Q.matrix,
    block.items       = object$block.items,
    response          = object$response,
    patterns          = object$patterns,
    patterns.total    = object$patterns.total,
    fc.type           = object$fc.type,
    stan.obj          = object$stan.obj,
    MCMC.obj          = object$MCMC.obj,
    iStEM             = object$iStEM,
    EM                = object$EM,
    call              = object$call,
    arguments         = object$arguments
  )
}

# ===========================================================================
# extract.FCGGUM
# ===========================================================================

#' @describeIn extract Extract components from \code{FCGGUM} objects.
#' @method extract FCGGUM
#' @export
extract.FCGGUM <- function(object, what, ...) {
  choices <- c("par", "theta", "Corr", "logLik", "npar", "method",
               "Q.matrix", "block.items", "response", "length.poly",
               "patterns", "patterns.total", "fc.type",
               "stan.obj", "MCMC.obj", "iStEM",
               "call", "arguments")
  what <- match.arg(what, choices)
  switch(what,
    par            = object$par,
    theta          = object$theta,
    Corr           = object$Corr,
    logLik         = object$logLik,
    npar           = object$npar,
    method         = object$method,
    Q.matrix       = object$Q.matrix,
    block.items    = object$block.items,
    response       = object$response,
    length.poly    = object$length.poly,
    patterns       = object$patterns,
    patterns.total = object$patterns.total,
    fc.type        = object$fc.type,
    stan.obj       = object$stan.obj,
    MCMC.obj       = object$MCMC.obj,
    iStEM          = object$iStEM,
    call           = object$call,
    arguments      = object$arguments
  )
}

# ===========================================================================
# extract.TIRT
# ===========================================================================

#' @describeIn extract Extract components from \code{TIRT} objects.
#' @method extract TIRT
#' @export
extract.TIRT <- function(object, what, ...) {
  choices <- c("par", "theta", "gamma", "gamma.matrix", "Corr",
               "logLik", "npar", "method",
               "Q.matrix", "pairs.matrix", "pairs.value",
               "block.items", "response", "fc.type",
               "stan.obj", "MCMC.obj", "iStEM",
               "call", "arguments")
  what <- match.arg(what, choices)
  switch(what,
    par          = object$par,
    theta        = object$theta,
    gamma        = object$gamma,
    gamma.matrix = object$gamma.matrix,
    Corr         = object$Corr,
    logLik       = object$logLik,
    npar         = object$npar,
    method       = object$method,
    Q.matrix     = object$Q.matrix,
    pairs.matrix = object$pairs.matrix,
    pairs.value  = object$pairs.value,
    block.items  = object$block.items,
    response     = object$response,
    fc.type      = object$fc.type,
    stan.obj     = object$stan.obj,
    MCMC.obj     = object$MCMC.obj,
    iStEM        = object$iStEM,
    call         = object$call,
    arguments    = object$arguments
  )
}

# library(ForceChoice)
#
# ############################### data ###############################
# N.person <- 1000
# N.block <- 30
# D <- 4
# dcm.type <- "DINA"
#
# seed <- 23548
# set.seed(seed)
# data.obj <- sim.data.FCDCM(N.person = N.person, N.block = N.block,
#                            D = D, dcm.type = dcm.type)
# response <- data.obj$response
# data <- data.obj$data
# Q.matrix <- data.obj$Q.matrix
# block.items <- data.obj$block.items
#
# data.from.response <- get.data.from.response.FCDCM(response,
#                                                                  block.items)
# response.obj <- get.response.from.data.FCDCM(data, block.items)
# stopifnot(all(as.matrix(data) == as.matrix(data.from.response)))
# stopifnot(all(response == response.obj$response))
# block.items.detected <- get.block.items.from.data.FCDCM(data)
# stopifnot(all(vapply(seq_along(block.items), function(b) {
#   setequal(block.items[[b]], block.items.detected[[b]])
# }, logical(1L))))
#
# # ############################### Stan MCMC ###############################
# stan.control.method <- list(
#   init = "random",
#   algorithm = "HMC",
#   seed = seed,
#   vis = TRUE
# )
# object.FCDCM.stan <- fit.FCDCM(data, Q.matrix = Q.matrix,
#                                block.items = block.items,
#                                dcm.type = dcm.type,
#                                method = "stan",
#                                control.method = stan.control.method)
# cbind(data.obj$par, object.FCDCM.stan$par$est)
# cbind(data.obj$delta1, data.obj$delta0)
# object.FCDCM.stan$delta$est
# object.FCDCM.stan$par$Rhat
# get.fit.index(object.FCDCM.stan)
# mean(abs((object.FCDCM.stan$alpha$est - data.obj$alpha)))
#
# ############################## S3 ###############################
# class(object.FCDCM.stan)
# methods(class = class(object.FCDCM.stan))
#
# coef(object.FCDCM.stan, "par")
# confint(object.FCDCM.stan)
# deviance(object.FCDCM.stan)
# extract(object.FCDCM.stan, "par")
# fitted(object.FCDCM.stan)
# get.fit.index(object.FCDCM.stan)
# logLik(object.FCDCM.stan)
# nobs(object.FCDCM.stan)
# plot(object.FCDCM.stan)
# predict(object.FCDCM.stan)
# residuals(object.FCDCM.stan)
# temp <- summary(object.FCDCM.stan)
# update(object.FCDCM.stan, control.model=list(L=31))
# vcov(object.FCDCM.stan)
#
# ############################### iStEM ###############################
# iStEM.control.method <- list(
#   burnin.maxitr = 100,
#   maxitr = 500,
#   optim.maxit = 100,
#   seed = seed,
#   vis = TRUE
# )
#
# object.FCDCM.iStEM <- fit.FCDCM(data, Q.matrix = Q.matrix,
#                                 block.items = block.items,
#                                 dcm.type = dcm.type,
#                                 method = "iStEM",
#                                 control.method = iStEM.control.method)
# cbind(data.obj$par, object.FCDCM.iStEM$par$est)
# cbind(data.obj$delta1, data.obj$delta0)
# object.FCDCM.iStEM$delta$est
# get.fit.index(object.FCDCM.iStEM)
#
# sqrt(mean((data.obj$par - object.FCDCM.iStEM$par$est)**2))
# 1-mean(abs((object.FCDCM.iStEM$alpha$est - data.obj$alpha)))
#
# # ############################## S3 ###############################
# class(object.FCDCM.iStEM)
# methods(class = class(object.FCDCM.iStEM))
#
# coef(object.FCDCM.iStEM, "par")
# confint(object.FCDCM.iStEM)
# deviance(object.FCDCM.iStEM)
# extract(object.FCDCM.iStEM, "par")
# fitted(object.FCDCM.iStEM)
# get.fit.index(object.FCDCM.iStEM)
# logLik(object.FCDCM.iStEM)
# nobs(object.FCDCM.iStEM)
# plot(object.FCDCM.iStEM)
# predict(object.FCDCM.iStEM)
# residuals(object.FCDCM.iStEM)
# temp <- summary(object.FCDCM.iStEM)
# update(object.FCDCM.iStEM, control.model=list(L=31))
# vcov(object.FCDCM.iStEM)

fit.FCDCM.iStEM <- function(data, Q.matrix, block.items = NULL,
                            dcm.type = "DINA",
                            control.model = NULL,
                            control.method = NULL,
                            .call = NULL) {
  call <- if (is.null(.call)) match.call() else .call
  control.model <- fc_as_control_list(control.model, "control.model")
  control.method <- fc_as_control_list(control.method, "control.method")
  common.method <- fit_common_method_control(control.method)
  method <- istem_method_control(control.method)
  set.seed(common.method$seed)

  data.matrix <- as.matrix(data)
  N <- nrow(data.matrix)
  B <- ncol(data.matrix)
  if (is.null(N) || is.null(B) || N < 1L || B < 1L) {
    stop("'data' must be an N x B FCDCM data matrix.", call. = FALSE)
  }
  if (is.numeric(data.matrix) || is.integer(data.matrix) ||
      is.logical(data.matrix)) {
    stop("'data' must contain FCDCM ranking strings such as '1>3'. ",
         "Use get.data.from.response.FCDCM() to convert a response matrix.",
         call. = FALSE)
  }

  if (is.null(block.items)) {
    block.items <- get.block.items.from.data.FCDCM(data)
  }
  if (!is.list(block.items) || length(block.items) != B) {
    stop("'block.items' must be a list with one two-statement entry per block.",
         call. = FALSE)
  }
  block.items <- lapply(block.items, as.integer)
  if (any(vapply(block.items, length, integer(1L)) != 2L)) {
    stop("Each FCDCM block must contain exactly two statement indices.",
         call. = FALSE)
  }
  all.items <- unlist(block.items, use.names = FALSE)
  if (anyNA(all.items) || any(all.items < 1L)) {
    stop("FCDCM statement indices in 'block.items' must be positive integers.",
         call. = FALSE)
  }
  if (anyDuplicated(all.items)) {
    stop("'block.items' contains duplicate statement indices.", call. = FALSE)
  }
  I.states <- length(all.items)
  if (!setequal(all.items, seq_len(I.states))) {
    stop("'block.items' must contain exactly the statement indices 1 through ",
         I.states, ".", call. = FALSE)
  }

  patterns <- matrix(all.items, ncol = 2L, byrow = TRUE)
  storage.mode(patterns) <- "integer"

  response <- get.response.from.data.FCDCM(data.matrix, block.items)$response
  if (anyNA(response) || !all(response %in% c(0L, 1L))) {
    stop("FCDCM responses must contain only 0/1 after parsing.",
         call. = FALSE)
  }
  storage.mode(response) <- "integer"
  response.group <- istem_response_groups(response)
  response.fit <- response.group$response
  count.fit <- response.group$count

  Q.matrix <- as.matrix(Q.matrix)
  if (anyNA(Q.matrix) || !all(Q.matrix %in% c(0, 1))) {
    stop("'Q.matrix' must be a 0/1 matrix.", call. = FALSE)
  }
  if (nrow(Q.matrix) != I.states) {
    stop("'Q.matrix' must have ", I.states,
         " rows, one for each statement.", call. = FALSE)
  }
  if (any(rowSums(Q.matrix) < 1L)) {
    stop("Each statement must require at least one attribute.",
         call. = FALSE)
  }
  D <- ncol(Q.matrix)
  storage.mode(Q.matrix) <- "numeric"
  method <- istem_apply_grid_control(method, control.model, 1L)
  istem_check_method_control(method)
  control.model <- istem_effective_grid_model_control(control.model, method)

  dcm.type <- toupper(dcm.type)
  if (length(dcm.type) != I.states) {
    dcm.type <- rep(dcm.type[1L], I.states)
  }
  if (!all(dcm.type %in% c("DINA", "DINO"))) {
    stop("'dcm.type' must contain only 'DINA' or 'DINO'.",
         call. = FALSE)
  }

  alpha.patterns <- as.matrix(do.call(expand.grid, rep(list(0:1), D)))
  storage.mode(alpha.patterns) <- "numeric"
  zeta.patterns <- get.zeta(alpha.patterns, Q.matrix, dcm.type)
  storage.mode(zeta.patterns) <- "integer"

  block.names <- colnames(response)
  if (is.null(block.names)) block.names <- paste0("B", seq_len(B))
  dim.names <- colnames(Q.matrix)
  if (is.null(dim.names)) dim.names <- paste0("Dim.", seq_len(D))
  item.names <- rownames(Q.matrix)
  if (is.null(item.names)) item.names <- paste0("S", seq_len(I.states))

  colnames(response) <- block.names
  rownames(patterns) <- block.names
  colnames(patterns) <- c("SA", "SB")
  rownames(Q.matrix) <- item.names
  colnames(Q.matrix) <- dim.names
  colnames(alpha.patterns) <- dim.names
  colnames(zeta.patterns) <- item.names

  prep <- list(
    response = response,
    response.unique = response.fit,
    response.count = count.fit,
    response.group = response.group$group,
    G = response.group$G,
    Q.matrix = Q.matrix,
    block.items = block.items,
    patterns = patterns,
    dcm.type = dcm.type,
    alpha.patterns = alpha.patterns,
    zeta.patterns = zeta.patterns,
    N = N,
    B = B,
    D = D,
    I.states = I.states,
    block.names = block.names,
    dim.names = dim.names,
    item.names = item.names
  )
  par.init <- get_ctrl("par", NULL, control.model)

  prior <- list(
    use.prior = get_ctrl("use.prior", TRUE, control.model),
    delta1.mu = istem_scalar(get_ctrl("delta1.mu", 0.0, control.model),
                             "delta1.mu"),
    delta1.sigma = istem_scalar(get_ctrl("delta1.sigma", 0.50, control.model),
                                "delta1.sigma", positive = TRUE),
    delta0.mu = istem_scalar(get_ctrl("delta0.mu", 0.00, control.model),
                             "delta0.mu"),
    delta0.sigma = istem_scalar(get_ctrl("delta0.sigma", 1.00, control.model),
                                "delta0.sigma", positive = TRUE),
    eta0.mu = istem_scalar(
      get_ctrl("eta0.mu", 0.10, control.model),
      "eta0.mu", positive = TRUE),
    eta0.sigma = istem_scalar(
      get_ctrl("eta0.sigma", 0.10, control.model),
      "eta0.sigma", positive = TRUE),
    etaAB.mu = istem_scalar(
      get_ctrl("etaAB.mu", 0.30, control.model),
      "etaAB.mu", positive = TRUE),
    etaAB.sigma = istem_scalar(
      get_ctrl("etaAB.sigma", 0.10, control.model),
      "etaAB.sigma", positive = TRUE)
  )
  if (!is.logical(prior$use.prior) || length(prior$use.prior) != 1L ||
      is.na(prior$use.prior)) {
    stop("'use.prior' in 'control.model' must be TRUE or FALSE.",
         call. = FALSE)
  }

  if (is.null(par.init)) {
    par.init <- cbind(
      eta0  = rep(prior$eta0.mu, prep$B),
      etaAB = rep(prior$etaAB.mu, prep$B)
    )
  }
  par.init <- fcdcm_normalize_par(par.init, B = prep$B, arg = "par")
  rownames(par.init) <- prep$block.names

  delta1.init <- as.numeric(get_ctrl(
    "delta1",
    stats::rlnorm(prep$D, meanlog = prior$delta1.mu,
                  sdlog = prior$delta1.sigma),
    control.model
  ))
  delta0.init <- as.numeric(get_ctrl(
    "delta0",
    stats::rnorm(prep$D, mean = prior$delta0.mu, sd = prior$delta0.sigma),
    control.model
  ))
  if (length(delta1.init) != prep$D || length(delta0.init) != prep$D ||
      anyNA(delta1.init) || anyNA(delta0.init) ||
      any(!is.finite(delta1.init)) || any(!is.finite(delta0.init)) ||
      any(delta1.init <= 0)) {
    stop("'delta1' and 'delta0' initial values must be finite vectors of length D, with delta1 > 0.",
         call. = FALSE)
  }
  names(delta1.init) <- names(delta0.init) <- prep$dim.names

  theta.init <- get_ctrl(
    "theta",
    drop(istem_random_theta(prep$G, 1L, 0, method$theta.lower,
                            method$theta.upper)),
    control.model
  )
  theta.init <- as.numeric(theta.init)
  if (length(theta.init) != prep$G || anyNA(theta.init) ||
      any(!is.finite(theta.init))) {
    stop("'theta' initial values must be a finite vector of length G.",
         call. = FALSE)
  }

  class.prob.init <- model.FCDCM(prep$zeta.patterns, par.init,
                                 prep$patterns)
  alpha.init <- cpp_fcdcm_sample_alpha(
    theta = matrix(theta.init, ncol = 1L),
    response = prep$response.unique,
    delta1 = delta1.init,
    delta0 = delta0.init,
    alpha_patterns = prep$alpha.patterns,
    zeta_patterns = prep$zeta.patterns,
    class_prob = class.prob.init
  )

  state <- list(
    response = prep$response.unique,
    response.count = prep$response.count,
    theta = matrix(theta.init, ncol = 1L),
    alpha = alpha.init$alpha,
    zeta = alpha.init$zeta,
    class.index = alpha.init$class.index,
    Corr = matrix(1, 1L, 1L),
    delta1 = delta1.init,
    delta0 = delta0.init,
    par = par.init,
    alpha.patterns = prep$alpha.patterns,
    zeta.patterns = prep$zeta.patterns,
    patterns = prep$patterns,
    class.prob = class.prob.init,
    optim.maxit = method$optim.maxit,
    prior = prior
  )

  run <- istem_run(
    state = state,
    method = method,
    N = prep$N,
    vis = common.method$vis,
    label = "FCDCM",
    update_parameters = fcdcm_istem_update_parameters,
    param_vec = fcdcm_istem_param_vec,
    sample_theta = fcdcm_istem_sample_theta,
    logLik_fun = function(state) {
      fcdcm_istem_loglik_trace(
        state, method$L, method$theta.lower, method$theta.upper
      )
    }
  )

  final <- fcdcm_istem_param_unpack(
    pv = run$chain.mean,
    pv_se = run$chain.sd,
    pv_rhat = NULL,
    D = prep$D,
    B = prep$B,
    dim.names = prep$dim.names,
    block.names = prep$block.names,
    par.names = colnames(par.init)
  )

  posterior <- fcdcm_marginal_posterior(
    response = prep$response,
    delta1 = final$delta[, "delta1"],
    delta0 = final$delta[, "delta0"],
    par = final$par,
    alpha.patterns = prep$alpha.patterns,
    zeta.patterns = prep$zeta.patterns,
    patterns = prep$patterns,
    L = method$L,
    theta.lower = method$theta.lower,
    theta.upper = method$theta.upper
  )
  class.post <- posterior$class.post
  theta <- posterior$theta
  colnames(theta) <- "theta"
  rownames(theta) <- rownames(prep$response)
  theta.se <- posterior$theta.se
  colnames(theta.se) <- "theta"
  rownames(theta.se) <- rownames(prep$response)
  theta.Rhat <- matrix(NA_real_, prep$N, 1L, dimnames = dimnames(theta))

  alpha.prob <- class.post %*% prep$alpha.patterns
  colnames(alpha.prob) <- prep$dim.names
  rownames(alpha.prob) <- rownames(prep$response)
  alpha.est <- fcdcm_alpha_map(class.post, prep$alpha.patterns)
  storage.mode(alpha.est) <- "integer"
  alpha.se <- matrix(NA_real_, prep$N, prep$D,
                     dimnames = dimnames(alpha.est))
  alpha.Rhat <- alpha.se

  par.free <- matrix(TRUE, prep$B, ncol(final$par),
                     dimnames = list(prep$block.names,
                                     colnames(final$par)))
  npar <- ncol(final$par) * prep$B + 2L * prep$D

  results <- list(
    npar = npar,
    method = "iStEM",
    theta = list(est = theta, se = theta.se, Rhat = theta.Rhat),
    alpha = list(est = alpha.est, se = alpha.se, Rhat = alpha.Rhat,
                 prob = alpha.prob),
    class.post = class.post,
    delta = list(est = final$delta, se = final$delta.se,
                 Rhat = final$delta.Rhat),
    par = list(est = final$par, se = final$par.se,
               Rhat = final$par.Rhat, free = par.free),
    Corr = list(
      est = matrix(1, 1L, 1L, dimnames = list("theta", "theta")),
      se = matrix(0, 1L, 1L, dimnames = list("theta", "theta")),
      Rhat = matrix(NA_real_, 1L, 1L, dimnames = list("theta", "theta"))
    ),
    stan.obj = NULL,
    MCMC.obj = NULL,
    Q.matrix = prep$Q.matrix,
    block.items = prep$block.items,
    patterns = prep$patterns,
    alpha.patterns = prep$alpha.patterns,
    zeta.patterns = prep$zeta.patterns,
    dcm.type = prep$dcm.type,
    response = prep$response,
    call = call,
    arguments = list(
      data = data,
      Q.matrix = prep$Q.matrix,
      block.items = prep$block.items,
      D = prep$D,
      dcm.type = prep$dcm.type,
      method = "iStEM",
      cores = common.method$cores,
      vis = common.method$vis,
      seed = common.method$seed,
      control.model = control.model,
      control.method = fit_effective_method_control(
        control.method, method = method, common = common.method
      )
    ),
    iStEM = utils::modifyList(run$iStEM, list(prior = prior))
  )

  class(results) <- "FCDCM"
  results$logLik <- logLik.FCDCM(results, L = method$L,
                                  theta.low = method$theta.lower,
                                  theta.up = method$theta.upper)
  results
}

fcdcm_istem_sample_theta <- function(state, theta_grid_length,
                                     theta_lower, theta_upper) {
  cpp_gibbs_fcdcm_theta(
    theta = state$theta,
    response = state$response,
    delta1 = state$delta1,
    delta0 = state$delta0,
    par = state$par,
    alpha_patterns = state$alpha.patterns,
    zeta_patterns = state$zeta.patterns,
    patterns = state$patterns,
    step = theta_grid_length,
    lower = theta_lower,
    upper = theta_upper
  )
}

fcdcm_istem_update_parameters <- function(state) {
  sampled <- cpp_fcdcm_sample_alpha(
    theta = state$theta,
    response = state$response,
    delta1 = state$delta1,
    delta0 = state$delta0,
    alpha_patterns = state$alpha.patterns,
    zeta_patterns = state$zeta.patterns,
    class_prob = state$class.prob
  )
  state$alpha <- sampled$alpha
  state$zeta <- sampled$zeta
  state$class.index <- sampled$class.index

  delta <- fcdcm_istem_update_delta(
    theta = state$theta[, 1L],
    alpha = state$alpha,
    weight = istem_state_weight(state),
    delta1 = state$delta1,
    delta0 = state$delta0,
    prior = state$prior,
    optim.maxit = state$optim.maxit
  )
  state$delta1 <- delta$delta1
  state$delta0 <- delta$delta0

  state$par <- fcdcm_istem_update_par(
    response = state$response,
    weight = istem_state_weight(state),
    zeta = state$zeta,
    par = state$par,
    patterns = state$patterns,
    prior = state$prior
  )
  state$class.prob <- model.FCDCM(state$zeta.patterns, state$par,
                                  state$patterns)
  state
}

fcdcm_istem_update_delta <- function(theta, alpha, weight, delta1, delta0,
                                     prior, optim.maxit) {
  theta <- as.numeric(theta)
  alpha <- as.matrix(alpha)
  storage.mode(alpha) <- "integer"
  D <- ncol(alpha)

  for (d in seq_len(D)) {
    y <- alpha[, d]
    x0 <- c(log(pmax(delta1[d], 1e-8)), delta0[d])
    obj <- function(x) {
      if (any(!is.finite(x)) || any(abs(x) > 30)) return(Inf)
      slope <- exp(x[1L])
      location <- x[2L]
      eta <- slope * (theta - location)
      ll <- sum(weight * (y * stats::plogis(eta, log.p = TRUE) +
                            (1L - y) * stats::plogis(-eta, log.p = TRUE)))
      lp <- if (isTRUE(prior$use.prior)) {
        stats::dlnorm(slope, meanlog = prior$delta1.mu,
                      sdlog = prior$delta1.sigma, log = TRUE) +
          stats::dnorm(location, mean = prior$delta0.mu,
                       sd = prior$delta0.sigma, log = TRUE)
      } else {
        0
      }
      if (!is.finite(lp)) return(Inf)
      -ll - lp
    }
    opt <- tryCatch(
      stats::optim(
        par = x0,
        fn = obj,
        method = "BFGS",
        control = list(maxit = optim.maxit)
      ),
      error = function(e) NULL
    )
    if (!is.null(opt) && is.finite(opt$value)) {
      delta1[d] <- exp(opt$par[1L])
      delta0[d] <- opt$par[2L]
    }
  }

  list(delta1 = delta1, delta0 = delta0)
}

fcdcm_istem_update_par <- function(response, weight, zeta, par, patterns,
                                   prior) {
  response <- as.matrix(response)
  zeta <- as.matrix(zeta)
  storage.mode(response) <- "integer"
  storage.mode(zeta) <- "integer"
  patterns <- as.matrix(patterns)
  storage.mode(patterns) <- "integer"
  par <- fcdcm_normalize_par(par, B = nrow(patterns), arg = "par")

  for (b in seq_len(nrow(patterns))) {
    za <- zeta[, patterns[b, 1L]]
    zb <- zeta[, patterns[b, 2L]]
    y <- response[, b]
    low <- za < zb
    high <- za > zb
    par[b, "eta0"] <- fcdcm_istem_fit_eta(
      y = y[low],
      weight = weight[low],
      shift = 0,
      init = par[b, "eta0"],
      prior.mu = prior$eta0.mu,
      prior.sigma = prior$eta0.sigma,
      use.prior = prior$use.prior
    )
    par[b, "etaAB"] <- fcdcm_istem_fit_eta(
      y = y[high],
      weight = weight[high],
      shift = 0.5,
      init = par[b, "etaAB"],
      prior.mu = prior$etaAB.mu,
      prior.sigma = prior$etaAB.sigma,
      use.prior = prior$use.prior
    )
  }

  par
}

fcdcm_istem_fit_eta <- function(y, weight, shift, init, prior.mu,
                                prior.sigma, use.prior, upper = 0.5) {
  y <- as.integer(y)
  weight <- as.numeric(weight)
  eps <- 1e-6
  if (length(y) == 0L) {
    if (isTRUE(use.prior)) {
      return(max(eps, min(upper - eps,
               stats::rnorm(1L, mean = prior.mu, sd = prior.sigma))))
    }
    return(pmin(pmax(init, eps), upper - eps))
  }

  obj <- function(eta) {
    p <- pmin(pmax(shift + eta, eps), 1 - eps)
    ll <- sum(weight * (y * log(p) + (1L - y) * log1p(-p)))
    lp <- if (isTRUE(use.prior)) {
      stats::dnorm(eta, mean = prior.mu, sd = prior.sigma, log = TRUE)
    } else {
      0
    }
    if (!is.finite(lp)) return(Inf)
    -ll - lp
  }

  opt <- tryCatch(
    stats::optimize(obj, interval = c(eps, upper - eps)),
    error = function(e) NULL
  )
  if (is.null(opt) || !is.finite(opt$objective)) {
    return(pmin(pmax(init, eps), upper - eps))
  }
  opt$minimum
}

fcdcm_istem_param_vec <- function(state) {
  c(
    as.numeric(state$delta1),
    as.numeric(state$delta0),
    as.numeric(state$par)
  )
}

fcdcm_istem_loglik_trace <- function(state, L, theta.lower, theta.upper) {
  theta.grid <- matrix(seq(theta.lower, theta.upper, length.out = L),
                       ncol = 1L)
  pi <- stats::dnorm(theta.grid[, 1L])
  pi <- pi / sum(pi)
  support <- fcdcm_latent_support(
    theta = theta.grid,
    pi = pi,
    delta1 = state$delta1,
    delta0 = state$delta0,
    par = state$par,
    alpha.patterns = state$alpha.patterns,
    zeta.patterns = state$zeta.patterns,
    patterns = state$patterns
  )
  istem_loglik_binary_sum(
    support$prob, state$response, state$response.count, support$pi
  )
}

fcdcm_istem_param_unpack <- function(pv, pv_se, pv_rhat = NULL, D, B,
                                     dim.names = NULL,
                                     block.names = NULL,
                                     par.names = c("eta0", "etaAB")) {
  idx <- 1L
  delta1 <- pv[idx:(idx + D - 1L)]
  delta1.se <- pv_se[idx:(idx + D - 1L)]
  delta1.rhat <- if (!is.null(pv_rhat)) pv_rhat[idx:(idx + D - 1L)] else rep(NA_real_, D)
  idx <- idx + D
  delta0 <- pv[idx:(idx + D - 1L)]
  delta0.se <- pv_se[idx:(idx + D - 1L)]
  delta0.rhat <- if (!is.null(pv_rhat)) pv_rhat[idx:(idx + D - 1L)] else rep(NA_real_, D)
  idx <- idx + D
  K <- length(par.names)
  par <- matrix(pv[idx:(idx + B * K - 1L)], nrow = B, ncol = K,
                dimnames = list(block.names, par.names))
  par.se <- matrix(pv_se[idx:(idx + B * K - 1L)], nrow = B, ncol = K,
                   dimnames = list(block.names, par.names))
  par.rhat <- if (!is.null(pv_rhat)) {
    matrix(pv_rhat[idx:(idx + B * K - 1L)], nrow = B, ncol = K,
           dimnames = list(block.names, par.names))
  } else {
    matrix(NA_real_, B, K, dimnames = list(block.names, par.names))
  }
  names(delta1) <- names(delta0) <- dim.names
  names(delta1.se) <- names(delta0.se) <- dim.names
  names(delta1.rhat) <- names(delta0.rhat) <- dim.names

  delta <- cbind(delta1 = delta1, delta0 = delta0)
  delta.se <- cbind(delta1 = delta1.se, delta0 = delta0.se)
  delta.Rhat <- cbind(delta1 = delta1.rhat, delta0 = delta0.rhat)
  list(
    delta = delta,
    delta.se = delta.se,
    delta.Rhat = delta.Rhat,
    par = par,
    par.se = par.se,
    par.Rhat = par.rhat
  )
}

fcdcm_posterior_alpha <- function(theta, response, delta1, delta0, par,
                                  alpha.patterns, zeta.patterns, patterns) {
  class.post <- fcdcm_class_posterior(
    theta = theta,
    response = response,
    delta1 = delta1,
    delta0 = delta0,
    par = par,
    alpha.patterns = alpha.patterns,
    zeta.patterns = zeta.patterns,
    patterns = patterns
  )
  alpha.prob <- class.post %*% as.matrix(alpha.patterns)
  colnames(alpha.prob) <- colnames(alpha.patterns)
  rownames(alpha.prob) <- rownames(response)
  alpha.prob
}

fcdcm_alpha_map <- function(class.post, alpha.patterns) {
  alpha.patterns <- as.matrix(alpha.patterns)
  alpha.est <- alpha.patterns[max.col(as.matrix(class.post),
                                      ties.method = "first"), , drop = FALSE]
  storage.mode(alpha.est) <- "integer"
  colnames(alpha.est) <- colnames(alpha.patterns)
  rownames(alpha.est) <- rownames(class.post)
  alpha.est
}

fcdcm_marginal_posterior <- function(response, delta1, delta0, par,
                                     alpha.patterns, zeta.patterns, patterns,
                                     L = 61, theta.lower = -6,
                                     theta.upper = 6, eps = 1e-12) {
  response <- as.matrix(response)
  storage.mode(response) <- "numeric"
  response.group <- istem_response_groups(response)
  y <- response.group$response
  count <- response.group$count
  G <- nrow(y)
  C <- nrow(alpha.patterns)

  theta.grid <- seq(theta.lower, theta.upper, length.out = L)
  pi.theta <- stats::dnorm(theta.grid)
  pi.theta <- pi.theta / sum(pi.theta)

  class.prob <- model.FCDCM(zeta.patterns, par, patterns)
  class.prob <- pmin(pmax(class.prob, eps), 1 - eps)
  log.p1 <- log(class.prob)
  log.p0 <- log1p(-class.prob)
  log.alpha <- fcdcm_alpha_profile_logprob(
    theta.grid, delta1, delta0, alpha.patterns, eps = eps
  )
  log.prior <- sweep(log.alpha, 1L, log(pi.theta), "+")

  class.post.group <- matrix(NA_real_, G, C)
  theta.group <- theta2.group <- numeric(G)

  for (g in seq_len(G)) {
    response.lp <- as.numeric(y[g, ] %*% t(log.p1) +
                                (1 - y[g, ]) %*% t(log.p0))
    log.post <- sweep(log.prior, 2L, response.lp, "+")
    max.log <- max(log.post)
    post <- exp(log.post - max.log)
    post <- post / sum(post)
    class.post.group[g, ] <- colSums(post)
    theta.weight <- rowSums(post)
    theta.group[g] <- sum(theta.grid * theta.weight)
    theta2.group[g] <- sum(theta.grid^2 * theta.weight)
  }

  class.post <- class.post.group[response.group$group, , drop = FALSE]
  colnames(class.post) <- pattern_key(alpha.patterns)
  rownames(class.post) <- rownames(response)

  theta <- matrix(theta.group[response.group$group], ncol = 1L)
  theta.se <- sqrt(pmax(theta2.group - theta.group^2, 0))
  theta.se <- matrix(theta.se[response.group$group], ncol = 1L)

  list(class.post = class.post, theta = theta, theta.se = theta.se,
       response.group = response.group, count = count)
}

fcdcm_class_posterior <- function(theta, response, delta1, delta0, par,
                                  alpha.patterns, zeta.patterns, patterns,
                                  eps = 1e-12) {
  class.prob <- model.FCDCM(zeta.patterns, par, patterns)
  fcdcm_class_posterior_class_prob(
    theta = theta,
    response = response,
    delta1 = delta1,
    delta0 = delta0,
    alpha.patterns = alpha.patterns,
    class.prob = class.prob,
    eps = eps
  )
}

fcdcm_class_posterior_class_prob <- function(theta, response, delta1, delta0,
                                             alpha.patterns, class.prob,
                                             eps = 1e-12) {
  theta <- as.numeric(theta)
  response <- as.matrix(response)
  storage.mode(response) <- "numeric"
  alpha.patterns <- as.matrix(alpha.patterns)
  class.prob <- as.matrix(class.prob)
  class.prob <- pmin(pmax(class.prob, eps), 1 - eps)

  log.alpha <- fcdcm_alpha_profile_logprob(
    theta, delta1, delta0, alpha.patterns, eps = eps
  )
  log.p1 <- log(class.prob)
  log.p0 <- log1p(-class.prob)
  log.response <- response %*% t(log.p1) + (1 - response) %*% t(log.p0)
  log.post <- log.alpha + log.response
  max.log <- apply(log.post, 1L, max)
  post <- exp(sweep(log.post, 1L, max.log, "-"))
  post <- post / rowSums(post)

  colnames(post) <- pattern_key(alpha.patterns)
  rownames(post) <- rownames(response)
  post
}

fcdcm_stan_class_posterior <- function(theta.draws, delta1.draws, delta0.draws,
                                       eta0.draws, etaAB.draws,
                                       response,
                                       alpha.patterns, zeta.patterns,
                                       patterns, eps = 1e-12) {
  theta.draws <- as.matrix(theta.draws)
  delta1.draws <- as.matrix(delta1.draws)
  delta0.draws <- as.matrix(delta0.draws)
  eta0.draws <- as.matrix(eta0.draws)
  etaAB.draws <- as.matrix(etaAB.draws)

  S <- nrow(theta.draws)
  response <- as.matrix(response)
  N <- nrow(response)
  C <- nrow(alpha.patterns)
  class.post <- matrix(0, N, C)

  for (s in seq_len(S)) {
    par.s <- cbind(eta0 = eta0.draws[s, ], etaAB = etaAB.draws[s, ])
    class.post <- class.post + fcdcm_class_posterior(
      theta = theta.draws[s, ],
      response = response,
      delta1 = delta1.draws[s, ],
      delta0 = delta0.draws[s, ],
      par = par.s,
      alpha.patterns = alpha.patterns,
      zeta.patterns = zeta.patterns,
      patterns = patterns,
      eps = eps
    )
  }

  class.post <- class.post / S
  colnames(class.post) <- pattern_key(alpha.patterns)
  rownames(class.post) <- rownames(response)
  class.post
}

# ===========================================================================
# rotate.MIRT
# ===========================================================================

#' @describeIn rotate Applies rotation to the posterior mean of loadings
#'   only from \code{MIRT} objects. Loadings (a) and factor correlation
#'   matrix (Corr) are updated to preserve model-implied covariances.
#'   For GPArotation methods, theta is re-standardized to a standard
#'   normal scale after rotation; for Promax, no post-rotation
#'   standardization is applied.
#' @method rotate MIRT
#' @export
rotate.MIRT <- function(object,
                        method,
                        vis          = TRUE,
                        target.a     = NULL,
                        target.theta = NULL,
                        pst.W        = NULL,
                        lp.p         = 1,
                        lp.gpaiter   = 5,
                        promax_m     = 4,
                        ...) {

  call <- match.call()
  user_args <- list(...)

  if (!is.character(method) || length(method) != 1L ||
      is.na(method) || !nzchar(method)) {
    stop("'method' must be a non-empty character scalar.", call. = FALSE)
  }

  D <- ncol(object$par$est) - 3L
  I <- nrow(object$par$est)

  rotation_arguments <- c(
    list(
      object = object,
      method = method,
      vis = vis,
      target.a = target.a,
      target.theta = target.theta,
      pst.W = pst.W,
      lp.p = lp.p,
      lp.gpaiter = lp.gpaiter,
      promax_m = promax_m
    ),
    user_args
  )
  new_rotation_result <- function(out) {
    structure(
      list(object = out, call = call, arguments = rotation_arguments),
      class = "RotateMIRT"
    )
  }

  if (D == 1L) {
    if (vis) message("D = 1: rotation is not applicable. Returning original fit object.")
    object$rotated         <- FALSE
    object$rotation_method <- method
    return(new_rotation_result(object))
  }

  model <- tolower(object$arguments$model)
  if (identical(model, "m1pl")) {
    stop("Rotation is not defined for multidimensional M1PL models with fixed loadings.",
         call. = FALSE)
  }

  Q.matrix <- object$Q.matrix
  if (!is.null(Q.matrix) && any(Q.matrix == 0)) {
    stop("Rotation is not allowed when Q.matrix contains zeros (confirmatory MIRT). ",
         "The Q.matrix specifies a confirmatory structure that rotation would destroy.",
         call. = FALSE)
  }

  use_target_a     <- !is.null(target.a)
  use_target_theta <- !is.null(target.theta)
  is_target_method <- method %in% c("targetT", "targetQ", "pstT", "pstQ")

  if (is_target_method) {
    if (use_target_a && use_target_theta) {
      if (vis) warning("Both 'target.a' and 'target.theta' provided. 'target.a' takes precedence; 'target.theta' will be ignored.")
      use_target_theta <- FALSE
    }
    if (!use_target_a && !use_target_theta) {
      stop(sprintf("Method '%s' requires either 'target.a' or 'target.theta'.", method),
           call. = FALSE)
    }
    if (use_target_theta && method %in% c("pstT", "pstQ")) {
      stop(sprintf("Method '%s' requires 'target.a'. 'target.theta' is not supported for PST.", method),
           call. = FALSE)
    }
    if (method %in% c("pstT", "pstQ") && is.null(pst.W)) {
      stop(sprintf("Method '%s' requires a weight matrix 'pst.W'.", method),
           call. = FALSE)
    }
    if (use_target_a && ncol(target.a) != D) {
      stop(sprintf("target.a has %d columns but model D = %d.", ncol(target.a), D),
           call. = FALSE)
    }
    if (use_target_theta && ncol(target.theta) != D) {
      stop(sprintf("target.theta has %d columns but model D = %d.", ncol(target.theta), D),
           call. = FALSE)
    }
  } else {
    if (use_target_a || use_target_theta) {
      if (vis) warning(sprintf("target.a/target.theta provided but method '%s' does not use targets. Ignoring.", method))
      use_target_a <- FALSE
      use_target_theta <- FALSE
    }
  }

  A_mean     <- object$par$est[, 1:D, drop = FALSE]
  theta_mean <- object$theta$est
  Corr_mean  <- object$Corr$est

  if (tolower(method) == "promax") {
    if (use_target_a || use_target_theta) {
      if (vis) warning("target.a/target.theta provided but method 'promax' does not use targets. Ignoring.")
    }

    if (vis) message(sprintf("Rotating posterior mean using 'promax' (m = %d)...", promax_m))

    rot_result_raw <- tryCatch(
      stats::promax(A_mean, m = promax_m),
      error = function(e) {
        stop(sprintf("Rotation of posterior mean failed: %s", conditionMessage(e)),
             call. = FALSE)
      }
    )

    T_mat <- rot_result_raw$rotmat
    standardize <- FALSE

  } else {
    base_args <- list(
      Tmat         = diag(D),
      normalize    = FALSE,
      eps          = 1e-5,
      maxit        = 1000L,
      randomStarts = 0L
    )

    method_specific <- switch(method,
                              "targetT" = , "targetQ" = list(Target = NULL, L = NULL),
                              "pstT"    = , "pstQ"    = list(W = pst.W, Target = NULL, L = NULL),
                              "oblimin"               = list(gam = 0),
                              "quartimin"             = list(),
                              "Varimax"               = list(),
                              "quartimax"             = list(),
                              "simplimax"             = list(k = I),
                              "geominT" = , "geominQ" = ,
                              "bigeominT" = , "bigeominQ" = list(delta = 0.01),
                              "cfT" = , "cfQ"         = list(kappa = 0),
                              "equamax"               = list(kappa = D / (2 * I)),
                              "parsimax"              = list(kappa = (D - 1) / (D + I - 2)),
                              "lpT" = , "lpQ"         = list(p = lp.p, gpaiter = lp.gpaiter),
                              "entropy" = , "oblimax" = ,
                              "bentlerT" = , "bentlerQ" = ,
                              "tandemI" = , "tandemII" = ,
                              "infomaxT" = , "infomaxQ" = ,
                              "mccammon" = , "varimin" = ,
                              "bifactorT" = , "bifactorQ" = list(),
                              list()
    )

    call_template <- utils::modifyList(base_args, method_specific)
    call_template <- utils::modifyList(call_template, user_args)
    call_template <- call_template[!sapply(call_template, is.null)]

    rot_fn <- tryCatch(
      get(method, mode = "function", envir = asNamespace("GPArotation")),
      error = function(e) {
        stop(sprintf("Rotation method '%s' not found in GPArotation package.", method),
             call. = FALSE)
      }
    )

    iter_call <- call_template

    if (is_target_method) {
      if (use_target_a) {
        iter_call$Target <- target.a
      } else if (use_target_theta) {
        svd_res <- svd(crossprod(target.theta, theta_mean))
        bridge  <- svd_res$v %*% t(svd_res$u)
        iter_call$Target <- A_mean %*% bridge
      }
    }

    iter_call$A <- A_mean

    if (vis) message(sprintf("Rotating posterior mean using '%s'...", method))

    rot_result <- tryCatch(
      do.call(rot_fn, iter_call),
      error = function(e) {
        stop(sprintf("Rotation of posterior mean failed: %s", conditionMessage(e)),
             call. = FALSE)
      }
    )

    if (is.null(rot_result) || is.null(rot_result$Th)) {
      stop("Rotation of posterior mean failed: no transformation matrix returned.",
           call. = FALSE)
    }

    T_mat <- if (isTRUE(rot_result$orthogonal)) {
      rot_result$Th
    } else {
      solve(t(rot_result$Th))
    }
    standardize <- TRUE
  }

  rotated <- .fc_transform_rotation(
    a = A_mean,
    theta = theta_mean,
    Corr = Corr_mean,
    Tmat = T_mat,
    standardize = standardize
  )

  par   <- object$par$est
  par[, seq_len(D)] <- rotated$a
  dimnames(par) <- dimnames(object$par$est)

  out <- object
  out$par$est   <- par
  out$theta$est <- rotated$theta
  out$Corr$est  <- rotated$Corr

  out$rotated         <- TRUE
  out$rotation_method <- method

  new_rotation_result(out)
}

# library(ForceChoice)
#
# ############################### data ###############################
# seed <- 4688567
# N <- 500
# I <- 10
# D <- 2
# Q.matrix <- NULL
# model <- "m2pl"
# Corr <- NULL
# rotate <- NULL
#
# set.seed(seed)
# data.obj <- sim.data.MIRT(N = N, I = I, D = D, model = model,
#                           Q.matrix = Q.matrix, Corr = Corr, rotate = rotate)
#
# response <- data.obj$response
# head(response)
#
# # ############################### Stan MCMC ###############################
# # object.MIRT.stan <- fit.MIRT(response, model = model, D = D,
# #                              Q.matrix = Q.matrix,
# #                              method = "stan",
# #                              control.method = c(list(
# #                                init = "random",
# #                                algorithm = "HMC"), list(seed = seed, vis = TRUE)))
# # cbind(data.obj$par, object.MIRT.stan$par$est)
# # get.fit.index(object.MIRT.stan)
#
# ############################### iStEM ###############################
# iStEM.control.method <- list(
#   burnin.maxitr = 40,
#   maxitr = 500,
#   theta.proposal.sd = 0.35,
#   optim.maxit = 50
# )
#
# object.MIRT.iStEM <- fit.MIRT(response, model = model, D = D,
#                               Q.matrix = Q.matrix,
#                               method = "iStEM",
#                               control.method = c(iStEM.control.method, list(seed = seed, vis = TRUE)))
# cbind(data.obj$par, object.MIRT.iStEM$par$est)
#
# # ---- Validate logLik with new quadrature helpers ----
# ll <- logLik(object.MIRT.iStEM)
# stopifnot(inherits(ll, "logLik"))
# stopifnot(is.finite(as.numeric(ll)))
# stopifnot(!is.null(attr(ll, "theta.norm")))
# cat(sprintf("  MIRT logLik: %.2f (df=%d)\n", as.numeric(ll), attr(ll, "df")))
#
# get.fit.index(object.MIRT.iStEM)

#' Fit FCGGUM Model via Stan
#'
#' Fits the Forced-Choice Generalized Graded Unfolding Model (FCGGUM) using
#' Bayesian MCMC via Stan.  The FCGGUM is a forced-choice wrapper around the
#' MGGUM: item-level endorsement probabilities are obtained from the MGGUM
#' binary endorsement probability, transformed to logits, and full rankings
#' are modeled as sequential Luce/PICK decisions over the remaining
#' statements in each block.
#'
#' @param data An N x B data frame where each column is a block and each cell
#'   contains the ranking order as a character string (e.g., "3>1>2"), or
#'   alternatively an N x B integer matrix of pattern indices (1-based) as
#'   returned by \code{sim.data.FCGGUM()$response}.
#' @param Q.matrix An I x D matrix with values -1, 0, or 1.
#'   \itemize{
#'     \item \code{1}: active dimension with \code{delta} constrained/regularized on the positive side
#'     \item \code{-1}: active dimension with \code{delta} constrained/regularized on the negative side
#'     \item \code{0}: \code{a} and \code{delta} fixed to 0 (inactive dimension)
#'   }
#'   The sign controls the side of the item-location prior/support, not the
#'   sign of discrimination; \code{a} is nonnegative whenever \code{|Q| = 1}.
#'   If \code{NULL}, a default matrix of all 1's is used.
#' @param block.items A list of length B; each element is an integer vector of
#'   (global) item indices belonging to that block.  If \code{NULL}, the block
#'   structure is auto-detected only from full-rank \code{"RANK"} string
#'   \code{data}; it is required when \code{data} is an integer matrix of
#'   pattern indices or when \code{fc.type} contains \code{"MOLE"} or
#'   \code{"PICK"}.
#' @param D Number of latent dimensions (must be >= 2).  If \code{NULL},
#'   \code{D} is derived from \code{ncol(Q.matrix)} when \code{Q.matrix}
#'   is provided.
#' @param control.model Optional named list of model-level controls and
#'   hyperparameters. Supported prior entries include \code{a.mu},
#'   \code{a.sigma}, \code{delta.pos.mu}, \code{delta.pos.sigma},
#'   \code{delta.neg.mu}, \code{delta.neg.sigma}, \code{tau.mu},
#'   \code{tau.sigma}, and \code{theta.mu}.
#' @param control.method Optional named list of method-specific controls. For
#'   Stan this includes \code{cores}, \code{vis}, \code{seed}, \code{chains},
#'   \code{iter}, \code{warmup}, \code{thin}, \code{init}, \code{algorithm},
#'   and Stan sampler controls.
#'
#' @noRd
fit.FCGGUM.stan <- function(data, Q.matrix = NULL, block.items = NULL, D = NULL, fc.type = "RANK",
                       control.model = NULL,
                       control.method = NULL,
                       .call = NULL) {
  call <- if (is.null(.call)) match.call() else .call
  control <- fc_as_control_list(control.model, "control.model")
  control.method <- fc_as_control_list(control.method, "control.method")
  common.method <- fit_common_method_control(control.method)
  stan.method <- fc_stan_method_control(control.method)
  chains <- stan.method$chains
  iter <- stan.method$iter
  warmup <- stan.method$warmup
  thin <- stan.method$thin
  init <- stan.method$init
  algorithm <- stan.method$algorithm

  sm <- stanmodels[["FCGGUM"]]

  if (is.null(sm)) {
    stop("Stan model FCGGUM not found !", call. = FALSE)
  }

  mcmc_env <- setup_mcmc_env(common.method$cores)
  on.exit(restore_mcmc_env(mcmc_env), add = TRUE)

  stan_control <- build_stan_control(algorithm, control.method)

  # ---- Validate fc.type ----
  fc.type.valid <- c("RANK", "MOLE", "PICK")
  if (!all(fc.type %in% fc.type.valid)) {
    stop("'fc.type' must be one of: ", paste(fc.type.valid, collapse = ", "), call. = FALSE)
  }

  data.matrix <- as.matrix(data)
  N.block.data <- ncol(data.matrix)
  if (is.null(N.block.data) || N.block.data < 1L) {
    stop("'data' must have at least one column/block.", call. = FALSE)
  }
  if (length(fc.type) != N.block.data) {
    fc.type <- rep(fc.type[1], N.block.data)
  }

  # ---- Auto-detect block items from full-rank string data if not provided ----
  is.pattern.index.data <- is.numeric(data.matrix) || is.integer(data.matrix)

  if (is.null(block.items)) {
    if (is.pattern.index.data) {
      stop("'block.items' is required when 'data' is an integer matrix of pattern indices.", call. = FALSE)
    }
    if (any(fc.type != "RANK")) {
      stop("'block.items' is required for MOLE/PICK data because partial rankings do not identify all block items.", call. = FALSE)
    }
    block.items <- get.block.items.from.data(data)
  }

  # ---- Convert ranking string data to response (pattern indices) ----
  if (is.pattern.index.data) {
    response <- matrix(as.integer(data.matrix), nrow = nrow(data.matrix), ncol = ncol(data.matrix),
                       dimnames = dimnames(data.matrix))
  } else {
    response <- get.response.from.data(data, block.items, fc.type = fc.type)
  }

  # ---- Validate block.items ----
  if (!is.list(block.items)) {
    stop("'block.items' must be a list.", call. = FALSE)
  }
  N.block <- length(block.items)

  all_items <- unlist(block.items)
  if (anyDuplicated(all_items)) {
    stop("'block.items' contains duplicate items.", call. = FALSE)
  }
  I <- length(all_items)
  if (!setequal(all_items, seq_len(I))) {
    stop("'block.items' must contain exactly the item indices 1 through ", I, ".", call. = FALSE)
  }
  if (any(vapply(block.items, length, integer(1L)) < 2L)) {
    stop("Each block in 'block.items' must contain at least two items.", call. = FALSE)
  }

  # ---- Validate response ----
  response <- as.matrix(response)
  N <- nrow(response)
  if (ncol(response) != N.block) {
    stop("'response' must have ", N.block, " columns (one per block).", call. = FALSE)
  }

  if (length(fc.type) != N.block) {
    fc.type <- rep(fc.type[1], N.block)
  }

  # ---- Map fc.type to integer codes for Stan ----
  fc.type.int <- match(fc.type, fc.type.valid)

  # ---- Item categories (binary agree/disagree for FCGGUM) ----
  length.poly <- rep(2L, I)
  max_poly <- 2L

  # ---- Validate Q.matrix and derive D ----
  if (!is.null(Q.matrix)) {
    if (!is.matrix(Q.matrix) || anyNA(Q.matrix) || !all(Q.matrix %in% c(-1, 0, 1))) {
      stop("'Q.matrix' must be a matrix containing only -1, 0, or 1", call. = FALSE)
    }
    D.q <- ncol(Q.matrix)
    if (!is.null(D) && D != D.q) {
      warning("'D' (", D, ") does not match ncol(Q.matrix) (", D.q, "); using ncol(Q.matrix).")
    }
    D <- D.q
    if (nrow(Q.matrix) != I) {
      stop("'Q.matrix' dimensions (", nrow(Q.matrix), "x", ncol(Q.matrix),
           ") do not match data (", I, "x", D, ").", call. = FALSE)
    } else if (any(rowSums(abs(Q.matrix)) < 1)) {
      stop("Each item must measure at least one trait (each row of 'Q.matrix' needs a non-zero entry)", call. = FALSE)
    }
  }

  # ---- Validate D ----
  if (is.null(D) || is.na(D)) {
    stop("Cannot determine 'D'. Please provide either 'Q.matrix' or 'D'.", call. = FALSE)
  }
  if (D < 2) {
    stop("FCGGUM requires at least 2 latent dimensions (D >= 2).", call. = FALSE)
  }

  if (is.null(Q.matrix)) {
    Q.matrix <- matrix(1, I, D)
  }

  corr_df <- free_corr_npar(D)

  # ---- Precompute patterns for each block ----
  patterns.total <- vector("list", N.block)
  for (b in 1:N.block) {
    patterns.total[[b]] <- get_permutations(block.items[[b]],
                                            length(block.items[[b]]))
  }

  patterns.obs <- vector("list", N.block)
  for (b in 1:N.block) {
    patterns.obs[[b]] <- get_permutations(
      block.items[[b]],
      switch(fc.type[b],
             RANK = length(block.items[[b]]),
             MOLE = 2,
             PICK = 1)
    )
  }

  for (b in 1:N.block) {
    if (anyNA(response[, b]) ||
        any(response[, b] < 1L | response[, b] > nrow(patterns.obs[[b]]))) {
      stop("'data' contains invalid pattern indices for block ", b,
           "; valid values are 1 through ", nrow(patterns.obs[[b]]), ".", call. = FALSE)
    }
  }

  # ---- Build flattened arrays for Stan ----
  n_items     <- integer(N.block)
  n_total     <- integer(N.block)
  n_obs       <- integer(N.block)
  fc_len      <- integer(N.block)

  block_items_flat   <- integer(0)
  patterns_total_flat <- integer(0)
  patterns_obs_flat   <- integer(0)

  item_start  <- integer(N.block + 1L)
  total_start <- integer(N.block + 1L)
  obs_start   <- integer(N.block + 1L)

  item_start[1]  <- 1L
  total_start[1] <- 1L
  obs_start[1]   <- 1L

  for (b in 1:N.block) {
    items.b <- block.items[[b]]
    K <- length(items.b)
    n_items[b] <- K

    fc_len[b] <- switch(fc.type[b],
                        RANK = K,
                        MOLE = 2L,
                        PICK = 1L)

    # Full ranking patterns to LOCAL indices 1..K
    pat.total <- patterns.total[[b]]
    n_total[b] <- nrow(pat.total)
    pat.total.local <- matrix(match(pat.total, items.b), nrow = n_total[b])
    patterns_total_flat <- c(patterns_total_flat, as.integer(t(pat.total.local)))

    # FC-type patterns to LOCAL indices 1..K
    pat.obs <- patterns.obs[[b]]
    n_obs[b] <- nrow(pat.obs)
    pat.obs.local <- matrix(match(pat.obs, items.b), nrow = n_obs[b])
    patterns_obs_flat <- c(patterns_obs_flat, as.integer(t(pat.obs.local)))

    # Global block items
    block_items_flat <- c(block_items_flat, items.b)

    item_start[b + 1]  <- item_start[b] + K
    total_start[b + 1] <- total_start[b] + n_total[b] * K
    obs_start[b + 1]   <- obs_start[b] + n_obs[b] * fc_len[b]
  }

  total_block_items <- length(block_items_flat)
  total_cells_total <- length(patterns_total_flat)
  total_cells_obs   <- length(patterns_obs_flat)

  # ---- Convert response to long format ----
  N_obs    <- N * N.block
  pid_long <- rep(1:N, times = N.block)
  bid_long <- rep(1:N.block, each = N)
  y_long   <- as.integer(response)

  # ---- Stan data ----
  theta_mu_raw <- get_ctrl("theta.mu", rep(0, D), control)
  theta_mu     <- array(theta_mu_raw, dim = D)

  stan.data <- list(
    N_obs              = N_obs,
    N                  = N,
    B                  = N.block,
    I                  = I,
    D                  = D,
    max_poly           = max_poly,
    pid                = pid_long,
    bid                = bid_long,
    y                  = y_long,
    length_poly        = as.integer(length.poly),
    Q_matrix           = Q.matrix,
    n_items            = n_items,
    fc_type            = fc.type.int,
    total_block_items  = total_block_items,
    block_items        = block_items_flat,
    item_start         = item_start,
    n_total            = n_total,
    total_cells_total  = total_cells_total,
    patterns_total     = patterns_total_flat,
    total_start        = total_start,
    n_obs              = n_obs,
    total_cells_obs    = total_cells_obs,
    patterns_obs       = patterns_obs_flat,
    obs_start          = obs_start,
    fc_len             = fc_len,
    a_mu               = get_ctrl("a.mu", 0, control),
    a_sigma            = get_ctrl("a.sigma", 0.5, control),
    delta_pos_mu       = get_ctrl("delta.pos.mu", 1, control),
    delta_pos_sigma    = get_ctrl("delta.pos.sigma", 0.5, control),
    delta_neg_mu       = get_ctrl("delta.neg.mu", -1, control),
    delta_neg_sigma    = get_ctrl("delta.neg.sigma", 0.5, control),
    tau_mu             = get_ctrl("tau.mu", 0, control),
    tau_sigma          = get_ctrl("tau.sigma", 0.5, control),
    theta_mu           = theta_mu
  )

  set.seed(common.method$seed)

  # ---- Run Stan ----
  stan.obj <- suppressWarnings(
    rstan::sampling(
      object        = sm,
      data          = stan.data,
      chains        = chains,
      iter          = iter,
      warmup        = warmup,
      thin          = thin,
      init          = init,
      algorithm     = algorithm,
      cores         = mcmc_env$cores,
      control       = stan_control,
      verbose       = common.method$vis,
      show_messages = common.method$vis
    )
  )

  MCMC.obj <- rstan::extract(stan.obj, permuted = TRUE)
  stan.sum <- rstan::summary(stan.obj)$summary

  # ---- Extract theta ----
  theta_est <- extract_theta_stan(MCMC.obj, stan.sum, N, D, rownames(response))

  # ---- Build par matrix: I x (D + D + max_poly) ----
  par      <- matrix(NA_real_, nrow = I, ncol = D + D + max_poly)
  par.se   <- matrix(NA_real_, nrow = I, ncol = D + D + max_poly)
  par.Rhat <- matrix(NA_real_, nrow = I, ncol = D + D + max_poly)

  a_arr <- ensure_3d(MCMC.obj$a)
  a     <- apply(a_arr, c(2, 3), mean)
  a.se  <- apply(a_arr, c(2, 3), sd)

  par[, 1:D]      <- a
  par.se[, 1:D]   <- a.se
  par.Rhat[, 1:D] <- extract_rhat_matrix(stan.sum, "a", I, D)

  delta_arr <- ensure_3d(MCMC.obj$delta)
  delta     <- apply(delta_arr, c(2, 3), mean)
  delta.se  <- apply(delta_arr, c(2, 3), sd)

  par[, (D + 1):(D + D)]      <- delta
  par.se[, (D + 1):(D + D)]   <- delta.se
  par.Rhat[, (D + 1):(D + D)] <- extract_rhat_matrix(stan.sum, "delta", I, D)

  par[, D + D + 1]      <- 0
  par.se[, D + D + 1]   <- NA_real_
  par.Rhat[, D + D + 1] <- NA_real_

  tau_arr <- ensure_3d(MCMC.obj$tau)
  tau     <- apply(tau_arr, c(2, 3), mean)
  tau.se  <- apply(tau_arr, c(2, 3), sd)
  for (i in 1:I) {
    Ki <- length.poly[i]
    if (Ki > 1) {
      tau_cols <- (D + D + 2):(D + D + Ki)
      par[i, tau_cols]      <- tau[i, 2:Ki]
      par.se[i, tau_cols]   <- tau.se[i, 2:Ki]
      tau_idx_i <- grep(paste0("^tau\\[", i, ","), rownames(stan.sum))
      tau_rhat_i <- stan.sum[tau_idx_i, "Rhat"]
      par.Rhat[i, tau_cols] <- tau_rhat_i[2:Ki]
    }
  }

  item_names <- paste0("item", all_items)
  par.free <- mggum_par_free_mask(Q.matrix, length.poly, max_poly)
  colnames(par) <- colnames(par.se) <- colnames(par.Rhat) <- colnames(par.free) <-
    c(paste0("a", 1:D),
      paste0("delta", 1:D),
      paste0("tau", 0:(max_poly - 1)))
  rownames(par) <- rownames(par.se) <- rownames(par.Rhat) <- rownames(par.free) <-
    item_names
  par.se[!par.free] <- NA_real_
  par.Rhat[!par.free] <- NA_real_
  npar <- sum(par.free) + corr_df

  # ---- Extract Corr ----
  Corr_est <- extract_corr_stan(MCMC.obj, stan.sum, D)

  # ---- Assemble results ----
  par_est   <- list(est = par, se = par.se, Rhat = par.Rhat, free = par.free)

  results <- list(
    npar           = npar,
    method         = "stan",
    theta          = theta_est,
    par            = par_est,
    Corr           = Corr_est,
    stan.obj       = stan.obj,
    MCMC.obj       = MCMC.obj,
    log_lik         = MCMC.obj$log_lik,
    Q.matrix       = Q.matrix,
    block.items    = block.items,
    fc.type        = fc.type,
    response       = response,
    length.poly    = length.poly,
    patterns       = patterns.obs,
    patterns.total = patterns.total,
    call           = call,
    arguments      = list(
      data        = data,
      Q.matrix    = Q.matrix,
      method      = "stan",
      block.items = block.items,
      D           = D,
      fc.type     = fc.type,
      chains      = chains,
      iter        = iter,
      warmup      = warmup,
      thin        = thin,
      init        = init,
      algorithm   = algorithm,
      cores       = common.method$cores,
      vis         = common.method$vis,
      seed        = common.method$seed,
      control.model = control,
      control.method = fit_effective_method_control(
        control.method, method = stan.method, common = common.method
      )
    )
  )

  class(results) <- "FCGGUM"

  L <- get_ctrl("L", NULL, control)
  results$logLik <- logLik.FCGGUM(results, L = L)

  return(results)
}

#' Fit TIRT Model via Stan
#'
#' @param data An N x B data frame where each column is a block and each cell
#'   contains the ranking order as a character string (e.g., "3>1>2").
#' @param Q.matrix An I x D matrix with values -1, 0, or 1. Each row has exactly
#'   one nonzero element: 1 = positive loading, -1 = reverse scoring.
#' @param block.items A list of length B, each element an integer vector of
#'   item indices belonging to that block.
#' @param fc.type Character vector of forced-choice types: \code{"RANK"} (default),
#'   \code{"MOLE"}, or \code{"PICK"}.
#' @param control.model Optional named list of model-level controls and
#'   hyperparameters. Supported entries include \code{lambda.alpha},
#'   \code{lambda.beta}, \code{psi.mu}, \code{psi.sigma}, \code{gamma.mu},
#'   \code{gamma.sigma}, and \code{theta.mu}.
#' @param control.method Optional named list of method-specific controls. For
#'   Stan this includes \code{cores}, \code{vis}, \code{seed}, \code{chains},
#'   \code{iter}, \code{warmup}, \code{thin}, \code{init}, \code{algorithm},
#'   and Stan sampler controls.
#'
#' @noRd
fit.TIRT.stan <- function(data, Q.matrix, block.items = NULL, fc.type = "RANK",
                     control.model = NULL,
                     control.method = NULL,
                       .call = NULL) {
  call <- if (is.null(.call)) match.call() else .call
  control <- fc_as_control_list(control.model, "control.model")
  control.method <- fc_as_control_list(control.method, "control.method")
  common.method <- fit_common_method_control(control.method)
  stan.method <- fc_stan_method_control(control.method)
  chains <- stan.method$chains
  iter <- stan.method$iter
  warmup <- stan.method$warmup
  thin <- stan.method$thin
  init <- stan.method$init
  algorithm <- stan.method$algorithm

  mcmc_env <- setup_mcmc_env(common.method$cores)
  on.exit(restore_mcmc_env(mcmc_env), add = TRUE)

  stan_control <- build_stan_control(algorithm, control.method)

  N.block.guess <- if (is.list(block.items)) length(block.items) else ncol(data)
  if (length(fc.type) != N.block.guess) fc.type <- rep(fc.type[1], N.block.guess)

  if (is.null(block.items)) block.items <- get.block.items.from.data(data)

  result <- get.response.from.data.TIRT(data, block.items, fc.type = fc.type)
  response <- result$response
  pairs.value <- result$pairs.value

  if (any(is.na(response))) {
    first_missing <- which(is.na(response), arr.ind = TRUE)[1L, ]
    stop(
      sprintf(
        "TIRT responses must not contain missing values (row %d, column %d).",
        first_missing[1L], first_missing[2L]
      ),
      call. = FALSE
    )
  }
  if (!all(response %in% c(0, 1))) {
    stop("TIRT responses must contain only 0 and 1.", call. = FALSE)
  }

  if (!is.matrix(Q.matrix)) {
    stop("'Q.matrix' must be a matrix.", call. = FALSE)
  }
  if (!all(Q.matrix %in% c(-1, 0, 1))) {
    stop("'Q.matrix' must contain only -1, 0, or 1.", call. = FALSE)
  }

  I.states <- nrow(Q.matrix)
  D <- ncol(Q.matrix)
  if (I.states < 1) {
    stop("'Q.matrix' must have at least one row (item).", call. = FALSE)
  }
  if (D < 2) {
    stop("At least two trait dimensions must be specified in Q.matrix (TIRT requires D >= 2).", call. = FALSE)
  }
  if (any(rowSums(abs(Q.matrix)) < 1)) {
    stop("Each item must measure at least one trait (row sums of abs('Q.matrix') must be >= 1).", call. = FALSE)
  }

  Idx_lambda_est_raw <- which(rowSums(abs(Q.matrix)) > 0)

  sm <- stanmodels[["TIRT"]]
  if (is.null(sm)) stop("Stan model 'TIRT' not found in the package namespace!", call. = FALSE)

  N.person <- nrow(response)
  if (is.null(N.person) || N.person < 1) {
    stop("Response matrix must have at least one row (person).", call. = FALSE)
  }

  if (!is.list(block.items)) {
    stop("'block.items' must be a list.", call. = FALSE)
  }
  if (length(block.items) < 1) {
    stop("'block.items' must contain at least one block.", call. = FALSE)
  }
  if (!all(sapply(block.items, function(x) length(x) >= 2))) {
    stop("Each block in 'block.items' must contain at least 2 items to form pairs.", call. = FALSE)
  }
  if (!all(sapply(block.items, is.integer) | sapply(block.items, is.numeric))) {
    stop("Elements inside 'block.items' must be integer or numeric item indices.", call. = FALSE)
  }
  all_items_in_blocks <- unlist(block.items)
  if (!all(all_items_in_blocks %in% 1:I.states)) {
    stop("Item indices in 'block.items' must be within the range of 1 to I.states (number of items in Q.matrix).", call. = FALSE)
  }
  if (length(all_items_in_blocks) != length(unique(all_items_in_blocks))) {
    stop("Item indices in 'block.items' must be unique across all blocks. An item cannot appear in multiple blocks.", call. = FALSE)
  }
  if (length(all_items_in_blocks) != I.states) {
    stop("The total number of items in 'block.items' must exactly match the number of items in Q.matrix (I.states).", call. = FALSE)
  }

  I.block  <- length(block.items[[1]])
  if (!all(sapply(block.items, length) == I.block)) {
    stop("All blocks in 'block.items' must have the same number of items.", call. = FALSE)
  }
  N.block    <- length(block.items)

  # Pre-compute per-block pair counts and total
  I.block.vec  <- sapply(block.items, length)
  block.pair.counts <- integer(N.block)
  for (b in 1:N.block) {
    k <- I.block.vec[b]
    if (fc.type[b] == "RANK") {
      block.pair.counts[b] <- choose(k, 2)
    } else if (fc.type[b] == "MOLE") {
      block.pair.counts[b] <- 2L * k - 3L
    } else if (fc.type[b] == "PICK") {
      block.pair.counts[b] <- k - 1L
    }
  }
  N.pairs.theoretical <- sum(block.pair.counts)

  N.pairs      <- ncol(response)
  if (N.pairs != N.pairs.theoretical) {
    stop(sprintf(
      "Column dimension of response matrix (%d) does not match the theoretical number of pairs derived from block.items and fc.type (%d). Please ensure response is in wide format (N.person x N.pairs) matrix.",
      N.pairs, N.pairs.theoretical
    ), call. = FALSE)
  }

  # Build pairs.matrix of ALL possible pairs (for gamma indexing)
  N.pairs.all <- sum(choose(I.block.vec, 2))
  pairs.matrix <- matrix(0L, N.pairs.all, 2)
  idx <- 0L
  for (b in 1:N.block) {
    items.b   <- block.items[[b]]
    pirs.cur  <- t(combn(items.b, 2))
    pairs.cur.length <- nrow(pirs.cur)
    pairs.matrix[(idx+1):(idx + pairs.cur.length), ] <- pirs.cur
    idx <- idx + pairs.cur.length
  }

  # Fast pair-to-gamma-index lookup (string key -> row in pairs.matrix)
  pair.keys <- paste0(pairs.matrix[,1L], "_", pairs.matrix[,2L])

  N.obs  <- N.person * N.pairs

  Y          <- integer(N.obs)
  Idx_person <- integer(N.obs)
  Idx_item_i <- integer(N.obs)
  Idx_item_k <- integer(N.obs)
  Idx_pair   <- integer(N.obs)

  idx <- 0L
  col_offset <- 0L
  for (b in 1:N.block) {
    n_pairs.b <- block.pair.counts[b]
    if (fc.type[b] == "RANK") {
      # RANK: all pairs observed, same set for every person
      items.b <- block.items[[b]]
      pairs.b <- t(combn(items.b, 2))
      for (p in 1:N.person) {
        for (pp in 1:n_pairs.b) {
          idx             <- idx + 1L
          Y[idx]          <- response[p, col_offset + pp]
          Idx_person[idx] <- p
          Idx_item_i[idx] <- pairs.b[pp, 1L]
          Idx_item_k[idx] <- pairs.b[pp, 2L]
          Idx_pair[idx]   <- match(paste0(pairs.b[pp, 1L], "_", pairs.b[pp, 2L]), pair.keys)
        }
      }
    } else {
      # MOLE / PICK: person-specific pairs
      for (p in 1:N.person) {
        pairs.cur <- pairs.value[[p]][[b]]
        if (!is.matrix(pairs.cur)) pairs.cur <- matrix(pairs.cur, ncol = 2)
        for (pp in 1:n_pairs.b) {
          idx             <- idx + 1L
          Y[idx]          <- response[p, col_offset + pp]
          Idx_person[idx] <- p
          Idx_item_i[idx] <- pairs.cur[pp, 1L]
          Idx_item_k[idx] <- pairs.cur[pp, 2L]
          Idx_pair[idx]   <- match(paste0(pairs.cur[pp, 1L], "_", pairs.cur[pp, 2L]), pair.keys)
        }
      }
    }
    col_offset <- col_offset + n_pairs.b
  }

  stopifnot(idx == N.obs, max(Idx_pair) <= N.pairs.all, min(Idx_pair) >= 1L)

  Idx_gamma_est <- sort(unique(Idx_pair))
  Idx_gamma_fix <- setdiff(seq_len(N.pairs.all), Idx_gamma_est)

  is_case_b <- (D == 2 && I.block == 2)
  if (is_case_b) {
    first_pair_items     <- block.items[[1]]
    Idx_lambda_fix       <- first_pair_items
    lambda_fix_val       <- rep(0.80, length(first_pair_items))

    Idx_lambda_est       <- setdiff(Idx_lambda_est_raw, Idx_lambda_fix)
  } else {
    Idx_lambda_est       <- Idx_lambda_est_raw
    Idx_lambda_fix       <- integer(0)
    lambda_fix_val       <- integer(0)
  }

  last_item_iblock <- unname(sapply(block.items, function(x) x[length(x)]))
  is_case_a        <- (I.block == 2 && D > 2)

  if (is_case_a || is_case_b) {
    Idx_psi_fix   <- 1:I.states
    psi_fix_val   <- rep(sqrt(0.5), I.states)
    Idx_psi_est   <- integer(0)
  } else {
    Idx_psi_fix   <- last_item_iblock
    psi_fix_val   <- rep(1.0, length(last_item_iblock))
    Idx_psi_est   <- setdiff(1:I.states, Idx_psi_fix)
  }

  Idx_psi_equal <- integer(0)
  Idx_psi_orig  <- integer(0)

  corr_df <- free_corr_npar(D)
  npar <- length(Idx_psi_est) + length(Idx_lambda_est) +
    length(Idx_gamma_est) + corr_df

  gamma_mu <- get_ctrl("gamma.mu", 0.0, control)
  gamma_fix_val <- rep(gamma_mu, length(Idx_gamma_fix))

  if (length(Idx_gamma_fix) > 0) {
    warning(sprintf(
      "TIRT: %d of %d possible pairwise comparisons were never observed (fc.type includes PICK/MOLE). The corresponding gamma parameters are fixed to the prior mean (%.2f) to avoid convergence issues from unobserved comparisons.",
      length(Idx_gamma_fix), N.pairs.all, gamma_mu
    ))
  }

  stan.data <- list(
    N_obs          = N.obs,
    N_pair         = N.pairs.all,
    N_person       = N.person,
    I_states       = I.states,
    D              = D,
    response       = Y,
    Idx_person     = Idx_person,
    Idx_item_i     = Idx_item_i,
    Idx_item_k     = Idx_item_k,
    Idx_pair       = Idx_pair,
    Q_matrix       = Q.matrix,
    N_lambda_est   = length(Idx_lambda_est),
    Idx_lambda_est = Idx_lambda_est,
    N_lambda_fix   = length(Idx_lambda_fix),
    Idx_lambda_fix = Idx_lambda_fix,
    lambda_fix_val = lambda_fix_val,
    N_psi_fix      = length(Idx_psi_fix),
    Idx_psi_fix    = Idx_psi_fix,
    psi_fix_val    = psi_fix_val,
    N_psi_est      = length(Idx_psi_est),
    Idx_psi_est    = Idx_psi_est,
    N_psi_equal    = length(Idx_psi_equal),
    Idx_psi_equal  = Idx_psi_equal,
    Idx_psi_orig   = Idx_psi_orig,
    N_gamma_fix    = length(Idx_gamma_fix),
    Idx_gamma_fix  = Idx_gamma_fix,
    gamma_fix_val  = gamma_fix_val,
    N_gamma_est    = length(Idx_gamma_est),
    Idx_gamma_est  = Idx_gamma_est,
    lambda_alpha   = get_ctrl("lambda.alpha", 0.40, control),
    lambda_beta    = get_ctrl("lambda.beta", 0.90, control),
    psi_mu         = get_ctrl("psi.mu", 0.80, control),
    psi_sigma      = get_ctrl("psi.sigma", 0.30, control),
    gamma_mu       = gamma_mu,
    gamma_sigma    = get_ctrl("gamma.sigma", 0.8, control),
    theta_mu       = get_ctrl("theta.mu", rep(0, D), control)
  )

  set.seed(common.method$seed)
  stan.obj <- suppressWarnings(
    rstan::sampling(
      object        = sm,
      data          = stan.data,
      chains        = chains,
      iter          = iter,
      warmup        = warmup,
      thin          = thin,
      init          = init,
      algorithm     = algorithm,
      cores         = mcmc_env$cores,
      control       = stan_control,
      verbose       = common.method$vis,
      show_messages = common.method$vis,
      open_progress = FALSE
    )
  )

  MCMC.obj <- rstan::extract(stan.obj, permuted = TRUE)
  stan.sum <- rstan::summary(stan.obj)$summary

  theta_est <- extract_theta_stan(MCMC.obj, stan.sum, N.person, D, rownames(response))

  lambda_raw  <- apply(MCMC.obj$lambda, 2, mean)
  lambda.se   <- apply(MCMC.obj$lambda, 2, sd)
  lambda.Rhat <- rep(NA_real_, I.states)
  lam.idx     <- grep("^lambda\\[", rownames(stan.sum))
  if (length(lam.idx) > 0) lambda.Rhat <- stan.sum[lam.idx, "Rhat"]

  lambda.free <- rep(FALSE, I.states)
  lambda.free[Idx_lambda_est] <- TRUE
  lambda.se[!lambda.free] <- NA_real_
  lambda.Rhat[!lambda.free] <- NA_real_

  lambda_sign <- apply(Q.matrix, 1, function(x) {
    nz <- which(x != 0)
    if (length(nz) == 0) return(1)
    sign(x[nz[1]])
  })
  lambda <- lambda_raw * lambda_sign

  psi_sd      <- apply(MCMC.obj$psi, 2, mean)
  psi_sd.se   <- apply(MCMC.obj$psi, 2, sd)
  psi2        <- psi_sd^2
  psi2.se     <- 2 * psi_sd * psi_sd.se
  psi2.Rhat   <- rep(NA_real_, I.states)
  psi.idx     <- grep("^psi\\[", rownames(stan.sum))
  if (length(psi.idx) > 0) psi2.Rhat <- stan.sum[psi.idx, "Rhat"]

  psi.free <- rep(FALSE, I.states)
  psi.free[Idx_psi_est] <- TRUE
  psi_sd.se[!psi.free] <- NA_real_
  psi2.se[!psi.free] <- NA_real_
  psi2.Rhat[!psi.free] <- NA_real_

  gamma      <- apply(MCMC.obj$gamma, 2, mean)
  gamma.se   <- apply(MCMC.obj$gamma, 2, sd)
  gamma.Rhat <- rep(NA_real_, N.pairs.all)
  gam.idx    <- grep("^gamma\\[", rownames(stan.sum))
  if (length(gam.idx) > 0) gamma.Rhat <- stan.sum[gam.idx, "Rhat"]

  gamma.free <- rep(FALSE, N.pairs.all)
  gamma.free[Idx_gamma_est] <- TRUE
  gamma.se[!gamma.free] <- NA_real_
  gamma.Rhat[!gamma.free] <- NA_real_

  gamma.matrix      <- matrix(NA_real_, nrow = I.states, ncol = I.states)
  gamma.matrix.se   <- matrix(NA_real_, nrow = I.states, ncol = I.states)
  gamma.matrix.Rhat <- matrix(NA_real_, nrow = I.states, ncol = I.states)
  for (r in seq_len(N.pairs.all)) {
    i <- pairs.matrix[r, 1L]; k <- pairs.matrix[r, 2L]
    gamma.matrix[i, k]      <- gamma[r]
    gamma.matrix.se[i, k]   <- gamma.se[r]
    gamma.matrix.Rhat[i, k] <- gamma.Rhat[r]
    gamma.matrix[k, i]      <- -gamma[r]
    gamma.matrix.se[k, i]   <- gamma.se[r]
    gamma.matrix.Rhat[k, i] <- gamma.Rhat[r]
  }

  Corr_est <- extract_corr_stan(MCMC.obj, stan.sum, D)
  par_est <- list(
    est   = cbind(lambda * abs(Q.matrix), psi2),
    se    = cbind(lambda.se * abs(Q.matrix), psi2.se),
    Rhat  = cbind(lambda.Rhat * abs(Q.matrix), psi2.Rhat),
    free  = tirt_par_free_mask(Q.matrix, lambda.free, psi.free)
  )

  colnames(par_est$est) <- colnames(par_est$se) <- colnames(par_est$Rhat) <-
    colnames(par_est$free) <- c(paste0("Dim.", 1:D), "psi2")
  rownames(par_est$est) <- rownames(par_est$se) <- rownames(par_est$Rhat) <-
    rownames(par_est$free) <- paste0("item", seq_len(I.states))
  par_est$se[!par_est$free] <- NA_real_
  par_est$Rhat[!par_est$free] <- NA_real_

  gamma_est <- list(est=gamma, se=gamma.se, Rhat=gamma.Rhat, free=gamma.free)
  gamma.matrix_est <- list(est = gamma.matrix, se = gamma.matrix.se, Rhat = gamma.matrix.Rhat)

  results <- list(
    npar         = npar,
    method       = "stan",
    theta        = theta_est,
    par          = par_est,
    gamma        = gamma_est,
    Corr         = Corr_est,
    gamma.matrix = gamma.matrix_est,
    Q.matrix     = Q.matrix,
    block.items  = block.items,
    pairs.matrix = pairs.matrix,
    pairs.value  = pairs.value,
    response     = response,
    fc.type      = fc.type,
    stan.obj     = stan.obj,
    MCMC.obj     = MCMC.obj,
    log_lik      = MCMC.obj$log_lik,
    call = call,
    arguments = list(
      data        = data,
      D           = D,
      Q.matrix    = Q.matrix,
      method      = "stan",
      block.items = block.items,
      fc.type     = fc.type,
      chains      = chains,
      iter        = iter,
      warmup      = warmup,
      thin        = thin,
      init        = init,
      algorithm   = algorithm,
      cores       = common.method$cores,
      vis         = common.method$vis,
      seed        = common.method$seed,
      control.model = control,
      control.method = fit_effective_method_control(
        control.method, method = stan.method, common = common.method
      )
    )
  )
  class(results) <- "TIRT"

  L <- get_ctrl("L", NULL, control)
  results$logLik <- logLik(results, L=L)

  return(results)
}

# library(ForceChoice)
#
# ############################### data ###############################
# N.person <- 500
# N.block <- 10
# I.block <- 3
# D <- 3
# fc.type <- "RANK"
# control <- NULL
#
# seed <- 3548875
# set.seed(seed)
# data.obj <- sim.data.TIRT(N.person = N.person, N.block = N.block,
#                           I.block = I.block, D = D, fc.type = fc.type,
#                           control = control)
# data.obj$par
# Q.matrix <- data.obj$Q.matrix
# response <- data.obj$response
# head(response)
# block.items <- data.obj$block.items
# pairs.value <- data.obj$pairs.value
# data <- get.data.from.response.TIRT(response, block.items,
#                                                   fc.type = fc.type,
#                                                   pairs.value = pairs.value)
# response.obj <- get.response.from.data.TIRT(data, block.items,
#                                                           fc.type = fc.type)
# head(response - response.obj$response)
# stopifnot(all(data == get.data.from.response.TIRT(response.obj$response,
#                                                       block.items,
#                                                       fc.type = fc.type,
#                                                       pairs.value = response.obj$pairs.value)))
#
# # ---- Verify unified block-item extraction ----
# block.items.from.data <- get.block.items.from.data(data)
# stopifnot(length(block.items.from.data) == N.block)
# stopifnot(all(mapply(setequal, block.items, block.items.from.data)))
#
# # ############################### Stan MCMC ###############################
# # object.TIRT.stan <- fit.TIRT(data, Q.matrix, block.items = block.items,
# #                              fc.type = fc.type,
# #                              method = "stan",
# #                              control.method = c(list(
# #                                init = "random",
# #                                algorithm = "HMC"), list(seed = seed, vis = TRUE)))
# # object.TIRT.stan
# # object.TIRT.stan$npar
# # cbind(data.obj$par, object.TIRT.stan$par$est)
# # object.TIRT.stan$par$Rhat
# # cbind(data.obj$gamma, object.TIRT.stan$gamma$est)
# # object.TIRT.stan$gamma$Rhat
# # get.fit.index(object.TIRT.stan)
#
# ############################### iStEM ###############################
# iStEM.control.method <- list(
#   burnin.maxitr = 40,
#   maxitr = 500,
#   theta.proposal.sd = 0.35,
#   optim.maxit = 50
# )
#
# object.TIRT.iStEM <- fit.TIRT(data, Q.matrix, block.items = block.items,
#                               fc.type = fc.type,
#                               method = "iStEM",
#                               control.method = c(iStEM.control.method, list(seed = seed, vis = TRUE)))
# object.TIRT.iStEM
# object.TIRT.iStEM$npar
# cbind(data.obj$par, object.TIRT.iStEM$par$est)
# cbind(data.obj$gamma, object.TIRT.iStEM$gamma$est)
#
# # ---- Validate logLik with new quadrature helpers ----
# ll <- logLik(object.TIRT.iStEM)
# stopifnot(inherits(ll, "logLik"))
# stopifnot(is.finite(as.numeric(ll)))
# cat(sprintf("  TIRT logLik: %.2f (df=%d)\n", as.numeric(ll), attr(ll, "df")))
#
# get.fit.index(object.TIRT.iStEM)

#' @title S3 Methods: print
#'
#' @description
#' Provides user-friendly, formatted console output for objects generated by
#' the \pkg{ForceChoice} package. This generic function dispatches to
#' class-specific methods that display concise summaries of fitted models,
#' simulation datasets, goodness-of-fit indices, and rotated solutions.
#' Designed for interactive use and quick diagnostic inspection.
#'
#' @param x An object of one of the following classes:
#'   \itemize{
#'     \item Fitted model objects: \code{\link[ForceChoice]{fit.MIRT}}
#'       (\code{"MIRT"}), \code{\link[ForceChoice]{fit.MGPCM}}
#'       (\code{"MGPCM"}), \code{\link[ForceChoice]{fit.MGGUM}}
#'       (\code{"MGGUM"}), \code{\link[ForceChoice]{fit.FCMIRT}}
#'       (\code{"FCMIRT"}), \code{\link[ForceChoice]{fit.FCDCM}}
#'       (\code{"FCDCM"}), \code{\link[ForceChoice]{fit.FCGGUM}}
#'       (\code{"FCGGUM"}), \code{\link[ForceChoice]{fit.TIRT}}
#'       (\code{"TIRT"})
#'     \item Goodness-of-fit objects: \code{\link[ForceChoice]{good.of.fit}}
#'       (\code{"good.of.fit"})
#'     \item Summary objects: \code{summary.MIRT}, \code{summary.MGPCM},
#'       \code{summary.MGGUM}, \code{summary.FCMIRT}, \code{summary.FCDCM},
#'       \code{summary.FCGGUM}, \code{summary.TIRT},
#'       \code{summary.good.of.fit}
#'   }
#' @param ... Additional arguments passed to methods (currently ignored in
#'   most cases).
#' @param digits Number of decimal places for numeric output (default: 4).
#'   Used by \code{print.good.of.fit}.
#'
#' @return Invisibly returns the input object \code{x}. No data is modified.
#'
#' @details
#' Each method produces a structured, human-readable summary optimized for
#' its object type:
#'
#' \describe{
#'   \item{\strong{Fitted Model Objects}}{
#'     Invokes \code{summary()} internally and prints comprehensive output
#'     including:
#'     \itemize{
#'       \item Model call and configuration (family, method, dimensions)
#'       \item Data characteristics (N, I or B, response type)
#'       \item Fit statistics (LogLik, npar, AIC, BIC)
#'       \item Factor correlation matrix
#'       \item Person parameter summary (N x D)
#'       \item Item / statement / block parameter summary
#'       \item For FCDCM: higher-order delta parameters and classification
#'             diagnostics
#'       \item For TIRT: loading (lambda) parameter summary
#'       \item Convergence diagnostics (algorithm, batches, latent grid length L)
#'     }
#'   }
#'
#'   \item{\strong{Goodness-of-Fit Objects (\code{good.of.fit})}}{
#'     Delegates to \code{summary.good.of.fit} and prints:
#'     \itemize{
#'       \item Overview table (sample size, items, parameters, moments)
#'       \item Limited-information absolute fit (M2, RMSEA, SRMSR)
#'       \item Incremental / comparative fit (CFI, TLI, IFI)
#'       \item Likelihood and information criteria (AIC, BIC, SABIC, CAIC)
#'       \item Pseudo-\eqn{R^2} measures
#'     }
#'   }
#' }
#'
#' @section Output Conventions:
#' \itemize{
#'   \item Numeric values are rounded to the precision specified by
#'         \code{digits} (default 4).
#'   \item Parameter summary tables show Min, Q1, Median, Mean, Q3, Max
#'         across items/statements for each parameter type.
#'   \item Convergence diagnostics adapt to the estimation backend
#'         (iStEM batch diagnostics vs. Stan HMC notes).
#' }
#'
#' @name print
NULL

# ===========================================================================
# Internal display helpers
# ===========================================================================

.fc_print_header <- function(title) {
  cat("\n==============================================\n")
  cat(title, "\n", sep = "")
  cat("==============================================\n\n")
}

.fc_print_matrix <- function(mat, digits, indent = "  ") {
  if (is.null(mat) || length(mat) == 0L) return(invisible(NULL))
  m <- as.matrix(mat)
  if (nrow(m) == 0L || ncol(m) == 0L) return(invisible(NULL))
  fm <- format(round(m, digits), nsmall = digits, justify = "right")
  rn <- rownames(m)
  cn <- colnames(m)

  # ---- compute column widths for proper alignment ----
  row_label_width <- if (!is.null(rn)) max(nchar(rn), na.rm = TRUE) else 0L
  if (row_label_width < 5L) row_label_width <- 5L

  col_widths <- integer(ncol(m))
  if (!is.null(cn)) {
    col_widths <- pmax(col_widths, nchar(cn))
  }
  for (j in seq_len(ncol(m))) {
    col_widths[j] <- max(col_widths[j], max(nchar(fm[, j]), na.rm = TRUE))
  }
  # ensure minimum width of 5 per column
  col_widths <- pmax(col_widths, 5L)

  # ---- print column header ----
  if (!is.null(cn)) {
    header <- sprintf("%-*s", row_label_width, "")
    for (j in seq_len(ncol(m))) {
      header <- paste0(header, "  ", sprintf("%*s", col_widths[j], cn[j]))
    }
    cat(indent, header, "\n", sep = "")
  }

  # ---- print separator line ----
  sep_line <- paste0(indent, paste(rep(" ", row_label_width), collapse = ""))
  for (j in seq_len(ncol(m))) {
    sep_line <- paste0(sep_line, "  ", paste(rep("-", col_widths[j]), collapse = ""))
  }
  cat(sep_line, "\n", sep = "")

  # ---- print data rows ----
  for (i in seq_len(nrow(m))) {
    row_label <- if (!is.null(rn)) rn[i] else ""
    line <- sprintf("%s%-*s", indent, row_label_width, row_label)
    for (j in seq_len(ncol(m))) {
      line <- paste0(line, "  ", sprintf("%*s", col_widths[j], fm[i, j]))
    }
    cat(line, "\n", sep = "")
  }
}

.fc_print_convergence <- function(conv) {
  if (is.null(conv) || length(conv) == 0L) return(invisible(NULL))
  cat("\nConvergence:\n")
  cat(sprintf("  Algorithm: %s\n", conv$algorithm))
  if (!is.null(conv$note)) {
    cat(sprintf("  Note: %s\n", conv$note))
  }
  if (!is.null(conv$burn_in_batches)) {
    cat(sprintf("  Burn-in Batches: %s\n", conv$burn_in_batches))
  }
  if (!is.null(conv$total_batches)) {
    cat(sprintf("  Total Batches: %s\n", conv$total_batches))
  }
  if (!is.null(conv$iterations)) {
    cat(sprintf("  Iterations: %s\n", conv$iterations))
  }
  if (!is.null(conv$burnin_converged)) {
    cat(sprintf("  Burn-in Converged: %s\n", as.character(conv$burnin_converged)))
  }
  if (!is.null(conv$final_converged)) {
    cat(sprintf("  Final Converged: %s\n", as.character(conv$final_converged)))
  }
  if (!is.null(conv$final_logLik) && is.finite(conv$final_logLik)) {
    cat(sprintf("  Final LogLik: %.4f\n", conv$final_logLik))
  }
  if (!is.null(conv$final_ll_change) && is.finite(conv$final_ll_change)) {
    cat(sprintf("  Final LogLik Change: %.6f\n", conv$final_ll_change))
  }
  if (!is.null(conv$final_max_delta_change) &&
      is.finite(conv$final_max_delta_change)) {
    cat(sprintf("  Final Max Delta Change: %.6f\n",
                conv$final_max_delta_change))
  }
  if (!is.null(conv$bootstrap_R) && conv$bootstrap_R > 0L) {
    cat(sprintf("  Bootstrap SE: %s / %s successful\n",
                conv$bootstrap_successful, conv$bootstrap_R))
  }
  if (!is.null(conv$L) && is.finite(conv$L)) {
    cat(sprintf("  Theta Grid L: %d\n", as.integer(conv$L)))
  }
}

# ===========================================================================
# Model print methods
#
# Convention: print.<Class>(x, ...) delegates to
# print.summary.<Class>(summary(x, ...)). The actual formatting is in the
# print.summary.<Class> methods below.
# ===========================================================================

# ---- good.of.fit ----

#' @describeIn print Print method for \code{good.of.fit} objects.
#'   Delegates to \code{summary.good.of.fit} and prints formatted tables.
#' @export
print.good.of.fit <- function(x, digits = 4, ...) {
  summ <- summary(x, digits = digits, ...)
  print(summ, ...)
  invisible(x)
}

#' @describeIn print Print method for \code{summary.good.of.fit} objects.
#'   Displays overview, absolute-fit, comparative-fit, likelihood/IC,
#'   and pseudo-\eqn{R^2} tables.
#' @export
print.summary.good.of.fit <- function(x, ...) {
  cat("\n")
  cat("Goodness-of-Fit Summary\n")
  cat("=======================\n")

  if (is.null(x$tables)) {
    return(.gof_print_legacy_summary(x))
  }

  if (!is.null(x$tables$Overview)) {
    .gof_print_overview(x$tables$Overview)
  }

  section.labels <- c(
    Absolute_Fit_M2        = "1. Limited-information Absolute Fit",
    Comparative_Fit        = "2. Incremental / Comparative Fit",
    Likelihood_and_IC      = "3. Likelihood & Information Criteria",
    Pseudo_R2              = "4. Pseudo-R2"
  )

  for (nm in names(section.labels)) {
    tab <- x$tables[[nm]]
    if (!is.null(tab) && nrow(tab) > 0L) {
      .gof_print_metric_table(section.labels[[nm]], tab)
    }
  }

  cat("Note: guide values are descriptive heuristics, not decision rules.\n")
  invisible(x)
}

# ---- MIRT ----

#' @describeIn print Print method for \code{MIRT} objects.
#'   Multidimensional IRT (1PL--4PL) model summary.
#' @export
print.MIRT <- function(x, ...) {
  print.summary.MIRT(summary(x, ...))
  invisible(x)
}

#' @describeIn print Print method for \code{summary.MIRT} objects.
#'   Shows model config, data info, fit stats, factor correlations,
#'   person/item parameter summaries, and convergence.
#' @export
print.summary.MIRT <- function(x, ...) {
  d <- x$digits
  .fc_print_header("MULTIDIMENSIONAL IRT (MIRT) MODEL SUMMARY")

  cat("Call:\n  ", paste(deparse(x$call), collapse = "\n   "), "\n", sep = "")

  cat("\nModel Configuration:\n")
  cat(sprintf("  Family: %s\n", x$model.info$family))
  cat(sprintf("  Model: %s\n", x$model.info$model))
  cat(sprintf("  Dimensions (D): %d\n", x$model.info$D))
  cat(sprintf("  Estimation Method: %s\n", x$model.info$method))

  cat("\nData Characteristics:\n")
  cat(sprintf("  Sample Size (N): %d\n", x$data.info$N))
  cat(sprintf("  Items (I): %d\n", x$data.info$I))
  cat(sprintf("  Response Type: %s\n", x$data.info$response.type))

  cat("\nFit Statistics:\n")
  cat(sprintf("  Log-likelihood: %.*f\n", d, x$fit.stats$LogLik))
  cat(sprintf("  Free Parameters: %d\n", x$fit.stats$npar))
  cat(sprintf("  AIC: %.*f\n", d, x$fit.stats$AIC))
  cat(sprintf("  BIC: %.*f\n", d, x$fit.stats$BIC))

  if (!is.null(x$Corr)) {
    cat("\nFactor Correlation Matrix:\n")
    .fc_print_matrix(x$Corr, d)
  }

  if (!is.null(x$theta.summary)) {
    cat("\nPerson Parameter Summary (N x D):\n")
    .fc_print_matrix(x$theta.summary, d)
  }

  if (!is.null(x$par.summary)) {
    cat("\nItem Parameter Summary:\n")
    .fc_print_matrix(x$par.summary, d)
  }

  if (!is.null(x$convergence)) {
    .fc_print_convergence(x$convergence)
  }

  cat("\nUse get.fit.index() to compute goodness-of-fit indices.\n\n")
  invisible(x)
}

# ---- MGPCM ----

#' @describeIn print Print method for \code{MGPCM} objects.
#'   Multidimensional Generalized Partial Credit Model summary.
#' @export
print.MGPCM <- function(x, ...) {
  print.summary.MGPCM(summary(x, ...))
  invisible(x)
}

#' @describeIn print Print method for \code{summary.MGPCM} objects.
#'   Shows polytomous model config, fit stats, factor correlations,
#'   person/item parameter summaries, and convergence.
#' @export
print.summary.MGPCM <- function(x, ...) {
  d <- x$digits
  .fc_print_header("MULTIDIMENSIONAL GENERALIZED PARTIAL CREDIT MODEL (MGPCM) SUMMARY")

  cat("Call:\n  ", paste(deparse(x$call), collapse = "\n   "), "\n", sep = "")

  cat("\nModel Configuration:\n")
  cat(sprintf("  Family: %s\n", x$model.info$family))
  cat(sprintf("  Dimensions (D): %d\n", x$model.info$D))
  cat(sprintf("  Estimation Method: %s\n", x$model.info$method))

  cat("\nData Characteristics:\n")
  cat(sprintf("  Sample Size (N): %d\n", x$data.info$N))
  cat(sprintf("  Items (I): %d\n", x$data.info$I))
  cat(sprintf("  Response Type: %s\n", x$data.info$response.type))

  cat("\nFit Statistics:\n")
  cat(sprintf("  Log-likelihood: %.*f\n", d, x$fit.stats$LogLik))
  cat(sprintf("  Free Parameters: %d\n", x$fit.stats$npar))
  cat(sprintf("  AIC: %.*f\n", d, x$fit.stats$AIC))
  cat(sprintf("  BIC: %.*f\n", d, x$fit.stats$BIC))

  if (!is.null(x$Corr)) {
    cat("\nFactor Correlation Matrix:\n")
    .fc_print_matrix(x$Corr, d)
  }

  if (!is.null(x$theta.summary)) {
    cat("\nPerson Parameter Summary (N x D):\n")
    .fc_print_matrix(x$theta.summary, d)
  }

  if (!is.null(x$par.summary)) {
    cat("\nItem Parameter Summary:\n")
    .fc_print_matrix(x$par.summary, d)
  }

  if (!is.null(x$convergence)) {
    .fc_print_convergence(x$convergence)
  }

  cat("\nUse get.fit.index() to compute goodness-of-fit indices.\n\n")
  invisible(x)
}

# ---- MGGUM ----

#' @describeIn print Print method for \code{MGGUM} objects.
#'   Multidimensional Generalized Graded Unfolding Model summary.
#' @export
print.MGGUM <- function(x, ...) {
  print.summary.MGGUM(summary(x, ...))
  invisible(x)
}

#' @describeIn print Print method for \code{summary.MGGUM} objects.
#'   Shows unfolding model config, fit stats, factor correlations,
#'   person/item parameter summaries, and convergence.
#' @export
print.summary.MGGUM <- function(x, ...) {
  d <- x$digits
  .fc_print_header("MULTIDIMENSIONAL GENERALIZED GRADED UNFOLDING MODEL (MGGUM) SUMMARY")

  cat("Call:\n  ", paste(deparse(x$call), collapse = "\n   "), "\n", sep = "")

  cat("\nModel Configuration:\n")
  cat(sprintf("  Family: %s\n", x$model.info$family))
  cat(sprintf("  Dimensions (D): %d\n", x$model.info$D))
  cat(sprintf("  Estimation Method: %s\n", x$model.info$method))

  cat("\nData Characteristics:\n")
  cat(sprintf("  Sample Size (N): %d\n", x$data.info$N))
  cat(sprintf("  Items (I): %d\n", x$data.info$I))
  cat(sprintf("  Response Type: %s\n", x$data.info$response.type))

  cat("\nFit Statistics:\n")
  cat(sprintf("  Log-likelihood: %.*f\n", d, x$fit.stats$LogLik))
  cat(sprintf("  Free Parameters: %d\n", x$fit.stats$npar))
  cat(sprintf("  AIC: %.*f\n", d, x$fit.stats$AIC))
  cat(sprintf("  BIC: %.*f\n", d, x$fit.stats$BIC))

  if (!is.null(x$Corr)) {
    cat("\nFactor Correlation Matrix:\n")
    .fc_print_matrix(x$Corr, d)
  }

  if (!is.null(x$theta.summary)) {
    cat("\nPerson Parameter Summary (N x D):\n")
    .fc_print_matrix(x$theta.summary, d)
  }

  if (!is.null(x$par.summary)) {
    cat("\nItem Parameter Summary:\n")
    .fc_print_matrix(x$par.summary, d)
  }

  if (!is.null(x$convergence)) {
    .fc_print_convergence(x$convergence)
  }

  cat("\nUse get.fit.index() to compute goodness-of-fit indices.\n\n")
  invisible(x)
}

# ---- FCMIRT ----

#' @describeIn print Print method for \code{FCMIRT} objects.
#'   Forced-Choice Multidimensional IRT model summary.
#' @export
print.FCMIRT <- function(x, ...) {
  print.summary.FCMIRT(summary(x, ...))
  invisible(x)
}

#' @describeIn print Print method for \code{summary.FCMIRT} objects.
#'   Shows forced-choice model config with block info, fit stats,
#'   factor correlations, person/statement parameter summaries.
#' @export
print.summary.FCMIRT <- function(x, ...) {
  d <- x$digits
  .fc_print_header("FORCED-CHOICE MULTIDIMENSIONAL IRT (FCMIRT) MODEL SUMMARY")

  cat("Call:\n  ", paste(deparse(x$call), collapse = "\n   "), "\n", sep = "")

  cat("\nModel Configuration:\n")
  cat(sprintf("  Family: %s\n", x$model.info$family))
  cat(sprintf("  Model: %s\n", x$model.info$model))
  cat(sprintf("  Dimensions (D): %d\n", x$model.info$D))
  cat(sprintf("  Estimation Method: %s\n", x$model.info$method))
  cat(sprintf("  FC Type: %s\n", x$model.info$fc.type))

  cat("\nData Characteristics:\n")
  cat(sprintf("  Sample Size (N): %d\n", x$data.info$N))
  cat(sprintf("  Blocks (B): %d\n", x$data.info$B))
  cat(sprintf("  Total Statements (I): %d\n", x$data.info$I))
  cat(sprintf("  Response Type: %s\n", x$data.info$response.type))

  cat("\nFit Statistics:\n")
  cat(sprintf("  Log-likelihood: %.*f\n", d, x$fit.stats$LogLik))
  cat(sprintf("  Free Parameters: %d\n", x$fit.stats$npar))
  cat(sprintf("  AIC: %.*f\n", d, x$fit.stats$AIC))
  cat(sprintf("  BIC: %.*f\n", d, x$fit.stats$BIC))

  if (!is.null(x$Corr)) {
    cat("\nFactor Correlation Matrix:\n")
    .fc_print_matrix(x$Corr, d)
  }

  if (!is.null(x$theta.summary)) {
    cat("\nPerson Parameter Summary (N x D):\n")
    .fc_print_matrix(x$theta.summary, d)
  }

  if (!is.null(x$par.summary)) {
    cat("\nStatement Parameter Summary:\n")
    .fc_print_matrix(x$par.summary, d)
  }

  if (!is.null(x$convergence)) {
    .fc_print_convergence(x$convergence)
  }

  cat("\nUse get.fit.index() to compute goodness-of-fit indices.\n\n")
  invisible(x)
}

# ---- FCDCM ----

#' @describeIn print Print method for \code{FCDCM} objects.
#'   Forced-Choice Diagnostic Classification Model summary with
#'   higher-order trait, attribute profiles, and classification diagnostics.
#' @export
print.FCDCM <- function(x, ...) {
  print.summary.FCDCM(summary(x, ...))
  invisible(x)
}

#' @describeIn print Print method for \code{summary.FCDCM} objects.
#'   Shows higher-order delta parameters, block eta parameters,
#'   attribute classification summary, and convergence.
#' @export
print.summary.FCDCM <- function(x, ...) {
  d <- x$digits
  .fc_print_header("FORCED-CHOICE DIAGNOSTIC CLASSIFICATION MODEL (FCDCM) SUMMARY")

  cat("Call:\n  ", paste(deparse(x$call), collapse = "\n   "), "\n", sep = "")

  cat("\nModel Configuration:\n")
  cat(sprintf("  Family: %s\n", x$model.info$family))
  cat(sprintf("  Attributes (D): %d\n", x$model.info$D))
  cat(sprintf("  DCM Type: %s\n", x$model.info$model))
  cat(sprintf("  Estimation Method: %s\n", x$model.info$method))

  cat("\nData Characteristics:\n")
  cat(sprintf("  Sample Size (N): %d\n", x$data.info$N))
  cat(sprintf("  Blocks (B): %d\n", x$data.info$B))
  cat(sprintf("  Response Type: %s\n", x$data.info$response.type))

  cat("\nFit Statistics:\n")
  cat(sprintf("  Log-likelihood: %.*f\n", d, x$fit.stats$LogLik))
  cat(sprintf("  Free Parameters: %d\n", x$fit.stats$npar))
  cat(sprintf("  AIC: %.*f\n", d, x$fit.stats$AIC))
  cat(sprintf("  BIC: %.*f\n", d, x$fit.stats$BIC))

  if (!is.null(x$delta.summary)) {
    cat("\nHigher-Order Parameters (delta):\n")
    .fc_print_matrix(x$delta.summary, d)
  }

  if (!is.null(x$par.summary)) {
    cat("\nBlock Parameters (eta):\n")
    .fc_print_matrix(x$par.summary, d)
  }

  if (!is.null(x$theta.summary)) {
    cat("\nHigher-Order Trait Summary (N x 1):\n")
    .fc_print_matrix(x$theta.summary, d)
  }

  if (!is.null(x$classification)) {
    cat("\nClassification:\n")
    cat(sprintf("  Observed Attribute Patterns: %s / %d possible\n",
                x$classification$n_patterns_observed,
                x$classification$n_patterns_possible))
    if (is.finite(x$classification$mean_max_posterior)) {
      cat(sprintf("  Mean Max Posterior: %.*f\n", d, x$classification$mean_max_posterior))
    }
  }

  if (!is.null(x$convergence)) {
    .fc_print_convergence(x$convergence)
  }

  cat("\nUse get.fit.index() to compute goodness-of-fit indices.\n\n")
  invisible(x)
}

# ---- FCGDINA ----

#' @describeIn print Print method for \code{FCGDINA} objects.
#'   Forced-Choice GDINA model summary.
#' @export
print.FCGDINA <- function(x, ...) {
  print.summary.FCGDINA(summary(x, ...))
  invisible(x)
}

#' @describeIn print Print method for \code{summary.FCGDINA} objects.
#'   Shows CDM type, forced-choice block info, fit stats, delta summaries,
#'   posterior attribute probabilities, and convergence.
#' @export
print.summary.FCGDINA <- function(x, ...) {
  d <- x$digits
  .fc_print_header("FORCED-CHOICE GDINA (FCGDINA) MODEL SUMMARY")

  cat("Call:\n  ", paste(deparse(x$call), collapse = "\n   "), "\n", sep = "")

  cat("\nModel Configuration:\n")
  cat(sprintf("  Family: %s\n", x$model.info$family))
  cat(sprintf("  CDM Model: %s\n", x$model.info$model))
  cat(sprintf("  Attributes (D): %d\n", x$model.info$D))
  cat(sprintf("  Estimation Method: %s\n", x$model.info$method))
  cat(sprintf("  FC Type: %s\n", x$model.info$fc.type))

  cat("\nData Characteristics:\n")
  cat(sprintf("  Sample Size (N): %d\n", x$data.info$N))
  cat(sprintf("  Blocks (B): %d\n", x$data.info$B))
  cat(sprintf("  Total Statements (I): %d\n", x$data.info$I))
  cat(sprintf("  Response Type: %s\n", x$data.info$response.type))

  cat("\nFit Statistics:\n")
  cat(sprintf("  Log-likelihood: %.*f\n", d, x$fit.stats$LogLik))
  cat(sprintf("  Free Parameters: %d\n", x$fit.stats$npar))
  cat(sprintf("  AIC: %.*f\n", d, x$fit.stats$AIC))
  cat(sprintf("  BIC: %.*f\n", d, x$fit.stats$BIC))

  if (!is.null(x$delta.summary)) {
    cat("\nCDM Delta Parameter Summary:\n")
    .fc_print_matrix(x$delta.summary, d)
  }

  if (!is.null(x$alpha.summary)) {
    cat("\nPosterior Attribute Probability Summary (N x D):\n")
    .fc_print_matrix(x$alpha.summary, d)
  }

  if (!is.null(x$classification)) {
    cat("\nClassification:\n")
    cat(sprintf("  Observed Attribute Patterns: %s / %d possible\n",
                x$classification$n_patterns_observed,
                x$classification$n_patterns_possible))
    if (is.finite(x$classification$mean_max_posterior)) {
      cat(sprintf("  Mean Max Posterior Probability: %.*f\n",
                  d, x$classification$mean_max_posterior))
    }
  }

  if (!is.null(x$pi.stats)) {
    cat(sprintf("\nStructural Parameters (class proportions):\n"))
    cat(sprintf("  Non-zero classes: %d / %d\n",
                x$pi.stats$n_nonzero,
                x$classification$n_patterns_possible))
    cat(sprintf("  Range: [%.*f, %.*f]\n",
                d, x$pi.stats$min, d, x$pi.stats$max))
    cat(sprintf("  Entropy: %.*f\n", d, x$pi.stats$entropy))
  }

  if (!is.null(x$convergence)) {
    .fc_print_convergence(x$convergence)
  }

  cat("\nUse get.fit.index() to compute goodness-of-fit indices.\n\n")
  invisible(x)
}

# ---- FCGGUM ----

#' @describeIn print Print method for \code{FCGGUM} objects.
#'   Forced-Choice Generalized Graded Unfolding Model summary.
#' @export
print.FCGGUM <- function(x, ...) {
  print.summary.FCGGUM(summary(x, ...))
  invisible(x)
}

#' @describeIn print Print method for \code{summary.FCGGUM} objects.
#'   Shows forced-choice unfolding model config, fit stats,
#'   factor correlations, person/statement parameter summaries.
#' @export
print.summary.FCGGUM <- function(x, ...) {
  d <- x$digits
  .fc_print_header("FORCED-CHOICE GENERALIZED GRADED UNFOLDING MODEL (FCGGUM) SUMMARY")

  cat("Call:\n  ", paste(deparse(x$call), collapse = "\n   "), "\n", sep = "")

  cat("\nModel Configuration:\n")
  cat(sprintf("  Family: %s\n", x$model.info$family))
  cat(sprintf("  Dimensions (D): %d\n", x$model.info$D))
  cat(sprintf("  Estimation Method: %s\n", x$model.info$method))
  cat(sprintf("  FC Type: %s\n", x$model.info$fc.type))

  cat("\nData Characteristics:\n")
  cat(sprintf("  Sample Size (N): %d\n", x$data.info$N))
  cat(sprintf("  Blocks (B): %d\n", x$data.info$B))
  cat(sprintf("  Total Statements (I): %d\n", x$data.info$I))
  cat(sprintf("  Response Type: %s\n", x$data.info$response.type))

  cat("\nFit Statistics:\n")
  cat(sprintf("  Log-likelihood: %.*f\n", d, x$fit.stats$LogLik))
  cat(sprintf("  Free Parameters: %d\n", x$fit.stats$npar))
  cat(sprintf("  AIC: %.*f\n", d, x$fit.stats$AIC))
  cat(sprintf("  BIC: %.*f\n", d, x$fit.stats$BIC))

  if (!is.null(x$Corr)) {
    cat("\nFactor Correlation Matrix:\n")
    .fc_print_matrix(x$Corr, d)
  }

  if (!is.null(x$theta.summary)) {
    cat("\nPerson Parameter Summary (N x D):\n")
    .fc_print_matrix(x$theta.summary, d)
  }

  if (!is.null(x$par.summary)) {
    cat("\nStatement Parameter Summary:\n")
    .fc_print_matrix(x$par.summary, d)
  }

  if (!is.null(x$convergence)) {
    .fc_print_convergence(x$convergence)
  }

  cat("\nUse get.fit.index() to compute goodness-of-fit indices.\n\n")
  invisible(x)
}

# ---- TIRT ----

#' @describeIn print Print method for \code{TIRT} objects.
#'   Thurstonian IRT for Forced-Choice model summary.
#' @export
print.TIRT <- function(x, ...) {
  print.summary.TIRT(summary(x, ...))
  invisible(x)
}

#' @describeIn print Print method for \code{summary.TIRT} objects.
#'   Shows pairwise-comparison model config, fit stats,
#'   factor correlations, loading/person parameter summaries.
#' @export
print.summary.TIRT <- function(x, ...) {
  d <- x$digits
  .fc_print_header("THURSTONIAN IRT FOR FORCED-CHOICE (TIRT) MODEL SUMMARY")

  cat("Call:\n  ", paste(deparse(x$call), collapse = "\n   "), "\n", sep = "")

  cat("\nModel Configuration:\n")
  cat(sprintf("  Family: %s\n", x$model.info$family))
  cat(sprintf("  Dimensions (D): %d\n", x$model.info$D))
  cat(sprintf("  Statements (I): %d\n", x$model.info$I.states))
  cat(sprintf("  Blocks: %d\n", x$model.info$N.block))
  cat(sprintf("  Estimation Method: %s\n", x$model.info$method))
  cat(sprintf("  FC Type: %s\n", x$model.info$fc.type))

  cat("\nData Characteristics:\n")
  cat(sprintf("  Sample Size (N): %d\n", x$data.info$N))
  cat(sprintf("  Pairwise Responses: %d\n", x$data.info$I.pairs))
  cat(sprintf("  Statements: %d\n", x$data.info$I.states))
  cat(sprintf("  Response Type: %s\n", x$data.info$response.type))

  cat("\nFit Statistics:\n")
  cat(sprintf("  Log-likelihood: %.*f\n", d, x$fit.stats$LogLik))
  cat(sprintf("  Free Parameters: %d\n", x$fit.stats$npar))
  cat(sprintf("  AIC: %.*f\n", d, x$fit.stats$AIC))
  cat(sprintf("  BIC: %.*f\n", d, x$fit.stats$BIC))

  if (!is.null(x$Corr)) {
    cat("\nFactor Correlation Matrix:\n")
    .fc_print_matrix(x$Corr, d)
  }

  if (!is.null(x$lambda.summary)) {
    cat("\nLoading (Lambda) Summary:\n")
    .fc_print_matrix(x$lambda.summary, d)
  }

  if (!is.null(x$theta.summary)) {
    cat("\nPerson Parameter Summary (N x D):\n")
    .fc_print_matrix(x$theta.summary, d)
  }

  if (!is.null(x$convergence)) {
    .fc_print_convergence(x$convergence)
  }

  cat("\nUse get.fit.index() to compute goodness-of-fit indices.\n\n")
  invisible(x)
}


# ===========================================================================
# Internal gof table printers (used by print.summary.good.of.fit)
# ===========================================================================

.gof_print_overview <- function(tab) {
  cat("\nOverview:\n")
  width <- max(nchar(tab$Field), na.rm = TRUE)
  for (i in seq_len(nrow(tab))) {
    cat(sprintf("  %-*s  %s\n", width, tab$Field[i], tab$Value[i]))
  }
}

.gof_print_metric_table <- function(title, tab) {
  cat("\n")
  cat(title, ":\n", sep = "")

  cols <- c("Metric", "Estimate", "Recommended")
  is_sub <- grepl("^SUB:", tab$Metric, perl = TRUE)
  tab$Metric <- sub("^SUB:\\s*", "", tab$Metric, perl = TRUE)

  metric_rows <- tab[!is_sub, , drop = FALSE]
  if (nrow(metric_rows) == 0L) return(invisible(NULL))

  widths <- vapply(cols, function(col) {
    header <- if (col == "Recommended") "Guide" else col
    max(nchar(c(header, metric_rows[[col]])), na.rm = TRUE)
  }, numeric(1L))
  widths <- pmax(widths, nchar(c("Metric", "Estimate", "Guide")))

  header <- sprintf("  %-*s  %*s  %-*s",
                    widths["Metric"], "Metric",
                    widths["Estimate"], "Estimate",
                    widths["Recommended"], "Guide")
  cat(header, "\n", sep = "")
  cat("  ", paste(
    vapply(widths, function(w) paste(rep("-", w), collapse = ""), character(1L)),
    collapse = "  "
  ), "\n", sep = "")

  mi <- 1L
  for (i in seq_len(nrow(tab))) {
    if (is_sub[i]) {
      cat("\n  ", tab$Metric[i], ":\n", sep = "")
    } else {
      r <- metric_rows[mi, , drop = FALSE]
      est <- if (nchar(r$Estimate) == 0L || r$Estimate == "NA") "NA" else r$Estimate
      rec <- if (nchar(r$Recommended) == 0L) "" else r$Recommended
      cat(sprintf("  %-*s  %*s  %-*s\n",
                  widths["Metric"], r$Metric,
                  widths["Estimate"], est,
                  widths["Recommended"], rec))
      mi <- mi + 1L
    }
  }
}

.gof_print_legacy_summary <- function(x) {
  digits <- attr(x, "digits")
  if (is.null(digits)) digits <- 4

  fmt <- function(val, d = digits, int = FALSE) {
    if (is.null(val) || length(val) == 0) return(character(0))
    res <- sapply(val, function(v) {
      if (is.na(v)) return("NA")
      if (int || abs(v - round(v)) < .Machine$double.eps^0.5) {
        formatC(round(v), format = "d", big.mark = ",")
      } else {
        formatC(v, format = "f", digits = d)
      }
    })
    names(res) <- names(val)
    return(res)
  }

  print_table <- function(labels, values) {
    if (length(labels) == 0) return(invisible(NULL))
    keep <- !(is.na(values) | values == "NA")
    labels <- labels[keep]
    values <- values[keep]
    if (length(labels) == 0) return(invisible(NULL))
    width <- max(nchar(labels), na.rm = TRUE)
    for (i in seq_along(labels)) {
      cat(sprintf("  %-*s %s\n", width, labels[i], values[i]))
    }
  }

  cat(sprintf("\nSample Size (N):       %s\n", fmt(x$Sample_Size, int = TRUE)))
  cat(sprintf("Items/Indicators:      %s\n", fmt(x$Items, int = TRUE)))
  cat(sprintf("Response Type:         %s\n", x$Response_Type))
  cat(sprintf("Parameters:            %s\n", fmt(x$Parameters, int = TRUE)))
  cat(sprintf("Moments:               %s total, %s bivariate\n",
              fmt(x$Moments, int = TRUE), fmt(x$Bivariate_Moments, int = TRUE)))
  cat(sprintf("Log-Likelihood:        %s\n", fmt(x$LogLik)))
  cat(sprintf("Deviance (-2LL):       %s\n\n", fmt(x$Deviance)))

  if (length(x$IC) > 0 && !all(is.na(x$IC))) {
    cat("Information Criteria:\n")
    print_table(names(x$IC), fmt(x$IC))
    cat("\n")
  }
  invisible(x)
}


# ===========================================================================
# Simulated data print methods
# ===========================================================================

.fc_print_data_header <- function(title) {
  cat(sprintf("\n%s\n", title))
  cat(paste(rep("=", nchar(title)), collapse = ""), "\n")
}

.print_data_common <- function(x, label) {
  .fc_print_data_header(paste0("SIMULATED ", label, " DATA"))
  cat(sprintf("Sample Size (N)     : %d\n", x$N))
  cat(sprintf("Items/Indicators (I): %d\n", x$I))
  cat(sprintf("Dimensions (D)      : %d\n", x$D))
  if (!is.null(x$model)) cat(sprintf("Model               : %s\n", x$model))
  cat(sprintf("Call: %s\n", paste(deparse(x$call), collapse = " ")))
  invisible(x)
}

#' @describeIn print Print method for \code{data.MIRT} objects.
#' @export
print.data.MIRT <- function(x, ...) {
  .print_data_common(x, "MIRT")
}

#' @describeIn print Print method for \code{data.MGPCM} objects.
#' @export
print.data.MGPCM <- function(x, ...) {
  .print_data_common(x, "MGPCM")
}

#' @describeIn print Print method for \code{data.MGGUM} objects.
#' @export
print.data.MGGUM <- function(x, ...) {
  .print_data_common(x, "MGGUM")
}

#' @describeIn print Print method for \code{data.FCMIRT} objects.
#' @export
print.data.FCMIRT <- function(x, ...) {
  .fc_print_data_header("SIMULATED FCMIRT DATA")
  cat(sprintf("Sample Size (N)      : %d\n", x$N.person))
  cat(sprintf("Blocks (B)           : %d\n", x$N.block))
  cat(sprintf("Statements (I)       : %d\n", x$I.states))
  cat(sprintf("Dimensions (D)       : %d\n", x$D))
  cat(sprintf("Model                : %s\n", x$model))
  cat(sprintf("FC Type              : %s\n",
              paste(x$fc.type, collapse = ", ")))
  cat(sprintf("Call: %s\n", paste(deparse(x$call), collapse = " ")))
  invisible(x)
}

#' @describeIn print Print method for \code{data.FCDCM} objects.
#' @export
print.data.FCDCM <- function(x, ...) {
  .fc_print_data_header("SIMULATED FCDCM DATA")
  cat(sprintf("Sample Size (N)      : %d\n", x$N.person))
  cat(sprintf("Blocks (B)           : %d\n", x$N.block))
  cat(sprintf("Attributes (D)       : %d\n", x$D))
  cat(sprintf("DCM Type             : %s\n",
              paste(unique(x$dcm.type), collapse = ", ")))
  cat(sprintf("Call: %s\n", paste(deparse(x$call), collapse = " ")))
  invisible(x)
}

#' @describeIn print Print method for \code{data.FCGDINA} objects.
#' @export
print.data.FCGDINA <- function(x, ...) {
  .fc_print_data_header("SIMULATED FCGDINA DATA")
  cat(sprintf("Sample Size (N)      : %d\n", x$N.person))
  cat(sprintf("Blocks (B)           : %d\n", x$N.block))
  cat(sprintf("Statements (I)       : %d\n", x$I.states))
  cat(sprintf("Attributes (D)       : %d\n", x$D))
  cat(sprintf("CDM Model            : %s\n", x$model))
  if (length(unique(x$fc.type)) == 1L) {
    cat(sprintf("FC Type              : %s\n", x$fc.type[1L]))
  } else {
    cat(sprintf("FC Type              : %s\n", paste(x$fc.type, collapse = ", ")))
  }
  cat(sprintf("Call: %s\n", paste(deparse(x$call), collapse = " ")))
  invisible(x)
}

#' @describeIn print Print method for \code{data.FCGGUM} objects.
#' @export
print.data.FCGGUM <- function(x, ...) {
  .fc_print_data_header("SIMULATED FCGGUM DATA")
  cat(sprintf("Sample Size (N)      : %d\n", x$N.person))
  cat(sprintf("Blocks (B)           : %d\n", x$N.block))
  cat(sprintf("Statements (I)       : %d\n", x$I.states))
  cat(sprintf("Dimensions (D)       : %d\n", x$D))
  cat(sprintf("FC Type              : %s\n",
              paste(x$fc.type, collapse = ", ")))
  cat(sprintf("Call: %s\n", paste(deparse(x$call), collapse = " ")))
  invisible(x)
}

#' @describeIn print Print method for \code{data.TIRT} objects.
#' @export
print.data.TIRT <- function(x, ...) {
  .fc_print_data_header("SIMULATED TIRT DATA")
  cat(sprintf("Sample Size (N)      : %d\n", x$N.person))
  cat(sprintf("Items (I)            : %d\n", x$I.states))
  cat(sprintf("Dimensions (D)       : %d\n", x$D))
  cat(sprintf("FC Type              : %s\n",
              paste(x$fc.type, collapse = ", ")))
  cat(sprintf("Call: %s\n", paste(deparse(x$call), collapse = " ")))
  invisible(x)
}

# ---- MCMC environment & algorithm helpers -------------------------------------

#' Setup single-threaded MCMC environment
#'
#' Ensures each MCMC chain runs strictly on a single OS thread by setting
#' environment variables that control Stan-internal threading and BLAS-level
#' parallelism.  The validated core count is returned inside the result list.
#'
#' @param cores Desired number of parallel chains (one chain per core).
#' @return A named list with old environment values and the validated
#'   \code{cores} value.  Pass this to \code{restore_mcmc_env()} to restore
#'   the previous thread-control state.
#' @keywords internal
setup_mcmc_env <- function(cores) {
  cores <- as.integer(cores)
  if (is.na(cores) || cores < 1L) {
    stop("'cores' must be a positive integer, got ", cores, call. = FALSE)
  }

  max_cores <- parallel::detectCores(logical = TRUE)
  if (cores > max_cores) {
    warning(sprintf(
      "'cores' (%d) exceeds available logical cores (%d). Reducing to %d.",
      cores, max_cores, max_cores
    ))
    cores <- max_cores
  }

  old <- list(
    STAN_NUM_THREADS     = Sys.getenv("STAN_NUM_THREADS",     unset = NA_character_),
    OMP_NUM_THREADS      = Sys.getenv("OMP_NUM_THREADS",      unset = NA_character_),
    MKL_NUM_THREADS      = Sys.getenv("MKL_NUM_THREADS",      unset = NA_character_),
    OPENBLAS_NUM_THREADS = Sys.getenv("OPENBLAS_NUM_THREADS", unset = NA_character_),
    cores                = cores
  )

  Sys.setenv(STAN_NUM_THREADS     = "1")
  Sys.setenv(OMP_NUM_THREADS      = "1")
  Sys.setenv(MKL_NUM_THREADS      = "1")
  Sys.setenv(OPENBLAS_NUM_THREADS = "1")

  return(old)
}


#' Restore MCMC threading environment to its previous state
#'
#' @param old A list returned by \code{setup_mcmc_env()}.
#' @return Invisibly \code{NULL}.
#' @keywords internal
restore_mcmc_env <- function(old) {
  restore_one <- function(name) {
    val <- old[[name]]
    if (is.na(val)) {
      Sys.unsetenv(name)
    } else {
      args <- list(val)
      names(args) <- name
      do.call(Sys.setenv, args)
    }
  }
  restore_one("STAN_NUM_THREADS")
  restore_one("OMP_NUM_THREADS")
  restore_one("MKL_NUM_THREADS")
  restore_one("OPENBLAS_NUM_THREADS")
  invisible(NULL)
}


#' Build Stan sampling-control list from user-provided \code{control.method}
#'
#' This helper extracts Stan sampler tuning knobs (e.g. \code{adapt_delta},
#' \code{max_treedepth}) from method-specific controls so they can be passed
#' safely to \code{rstan::sampling()}.
#'
#' @param algorithm Character scalar: \code{"NUTS"}, \code{"HMC"}, or
#'   \code{"Fixed_param"}.
#' @param control.method A named list of Stan method controls.
#' @return A named list of Stan sampling-control parameters (possibly empty).
#' @keywords internal
build_stan_control <- function(algorithm, control.method) {
  if (is.null(control.method)) control.method <- list()

  stan_ctrl_names <- c(
    "adapt_delta", "stepsize", "max_treedepth",
    "int_time", "metric", "adapt_engaged",
    "adapt_init_buffer", "adapt_term_buffer", "adapt_window"
  )

  stan_ctrl <- list(
    adapt_delta   = 0.95,
    max_treedepth = 12L
  )
  for (nm in stan_ctrl_names) {
    if (!is.null(control.method[[nm]])) {
      stan_ctrl[[nm]] <- control.method[[nm]]
    }
  }

  if (algorithm == "Fixed_param") {
    stan_ctrl$adapt_engaged <- FALSE
  }

  return(stan_ctrl)
}


# ---- Stan MCMC post-processing helpers ---------------------------------------

#' Extract theta posterior summary from Stan MCMC output
#'
#' @param MCMC.obj Output of \code{rstan::extract()}.
#' @param stan.sum Output of \code{rstan::summary()$summary}.
#' @param N Number of persons.
#' @param D Number of dimensions.
#' @param row_names Row names for the output matrices (typically person IDs).
#' @return A list with components \code{est}, \code{se}, and \code{Rhat}.
#' @keywords internal
extract_theta_stan <- function(MCMC.obj, stan.sum, N, D, row_names = NULL) {
  theta_arr  <- ensure_3d(MCMC.obj$theta)
  theta      <- apply(theta_arr, c(2, 3), mean)
  theta.se   <- apply(theta_arr, c(2, 3), sd)
  theta.Rhat <- extract_rhat_matrix(stan.sum, "theta", N, D)
  dn <- list(row_names, paste0("Dim.", seq_len(D)))
  dimnames(theta) <- dimnames(theta.se) <- dimnames(theta.Rhat) <- dn
  list(est = theta, se = theta.se, Rhat = theta.Rhat)
}

#' Extract correlation posterior summary from Stan MCMC output
#'
#' @param MCMC.obj Output of \code{rstan::extract()}.
#' @param stan.sum Output of \code{rstan::summary()$summary}.
#' @param D Number of dimensions.
#' @return A list with components \code{est}, \code{se}, and \code{Rhat}.
#' @keywords internal
extract_corr_stan <- function(MCMC.obj, stan.sum, D) {
  Corr_arr  <- ensure_3d(MCMC.obj$Corr)
  Corr      <- apply(Corr_arr, c(2, 3), mean)
  Corr.se   <- apply(Corr_arr, c(2, 3), sd)
  Corr.Rhat <- extract_rhat_matrix(stan.sum, "Corr", D, D)
  diag(Corr.Rhat) <- NA_real_
  dn <- list(paste0("Dim.", seq_len(D)), paste0("Dim.", seq_len(D)))
  dimnames(Corr) <- dimnames(Corr.se) <- dimnames(Corr.Rhat) <- dn
  list(est = Corr, se = Corr.se, Rhat = Corr.Rhat)
}


# ---- Quadrature grid helpers for logLik ---------------------------------------

#' Multivariate normal quadrature grid with Cholesky-stabilised weights
#'
#' Used by all continuous-trait \code{logLik()} methods.
#'
#' @param Corr D x D correlation matrix (can be \code{NULL} for identity).
#' @param theta.mu Numeric vector of length D (latent mean).
#' @param D Number of dimensions.
#' @param L Number of quadrature points per dimension, or \code{NULL} to
#'   auto-select from a built-in table.
#' @param theta.low,theta.up Quadrature range.
#' @return A list with \code{theta.norm} (grid matrix) and \code{pi}
#'   (normalised prior-weight column vector).
#' @keywords internal
make_quadrature_grid_mvn <- function(Corr, theta.mu, D, L = NULL,
                                     theta.low = -6, theta.up = 6) {
  if (is.null(L)) {
    L <- switch(as.character(D),
                "1" = 61, "2" = 31, "3" = 15, "4" = 9, "5" = 7, 3)
  }

  theta.norm.list <- vector("list", D)
  theta.axis <- seq(theta.low, theta.up, length.out = L)
  for (d in seq_len(D)) {
    theta.norm.list[[d]] <- theta.axis
  }
  theta.norm <- as.matrix(do.call(expand.grid, theta.norm.list))

  if (is.null(Corr) || anyNA(Corr)) Corr <- diag(D)
  Corr <- as.matrix(Corr)
  diag(Corr) <- 1

  logspace_sum <- function(x) {
    m <- max(x)
    if (!is.finite(m)) return(m)
    m + log(sum(exp(x - m)))
  }

  if (D == 1L) {
    log.pi.raw <- stats::dnorm(theta.norm[, 1L], mean = theta.mu[1L],
                               sd = 1, log = TRUE)
  } else {
    chol.ok <- FALSE
    jitter <- 0
    while (!chol.ok && jitter <= 1e-4) {
      chol.try <- try(chol(Corr + diag(jitter, D)), silent = TRUE)
      if (!inherits(chol.try, "try-error")) {
        chol.ok <- TRUE
      } else {
        jitter <- if (jitter == 0) 1e-10 else jitter * 10
      }
    }
    if (!chol.ok) {
      stop("Estimated 'Corr' is not positive definite enough for quadrature.", call. = FALSE)
    }
    centered.theta <- sweep(theta.norm, 2, theta.mu, "-")
    z.theta <- forwardsolve(t(chol.try), t(centered.theta))
    log.det <- 2 * sum(log(diag(chol.try)))
    quad <- colSums(z.theta^2)
    log.pi.raw <- -0.5 * (D * log(2 * pi) + log.det + quad)
  }
  log.pi.raw <- log.pi.raw - logspace_sum(log.pi.raw)
  pi <- matrix(exp(log.pi.raw), ncol = 1)

  list(theta.norm = theta.norm, pi = pi, L = L)
}

#' Apply response-pattern frequencies to grouped log-likelihoods
#'
#' @param lik A grouped likelihood list containing \code{log_marginal}.
#' @param count Frequency of each unique response pattern.
#' @return \code{lik} with aggregate \code{logLik} and
#'   \code{response.count} components.
#' @noRd
loglik_apply_group_count <- function(lik, count) {
  count <- as.numeric(count)
  if (is.null(lik$log_marginal) || length(lik$log_marginal) != length(count)) {
    stop("'lik' must contain one marginal log-likelihood per response group.",
         call. = FALSE)
  }
  lik$logLik <- sum(lik$log_marginal * count)
  lik$response.count <- count
  lik
}

#' Attach standard attributes to a logLik result vector
#'
#' @param results A named numeric vector containing the log-likelihood.
#' @param N Number of observations.
#' @param df Number of free parameters.
#' @param lik A list returned by a \code{cpp_loglik_*()} function.
#' @param theta.norm Quadrature grid matrix.
#' @param prob Model-implied probability matrix.
#' @param pi Quadrature or latent-class weights.
#' @param L Number of quadrature points per dimension.
#' @param theta.low,theta.up Quadrature range.
#' @param ... Additional named attributes.
#' @return \code{results} with class \code{"logLik"} and standard
#'   likelihood attributes.
#' @noRd
set_loglik_attributes <- function(results, N, df, lik, theta.norm,
                                   prob, pi, L, theta.low, theta.up, ...) {
  class(results) <- "logLik"
  attr(results, "nobs")        <- N
  attr(results, "df")          <- df
  attr(results, "L.theta.Xi")  <- lik$L.theta.Xi
  attr(results, "P.theta.Xi")  <- lik$P.theta.Xi
  attr(results, "theta.norm")  <- theta.norm
  attr(results, "prob")        <- prob
  attr(results, "pi")          <- pi
  attr(results, "L")           <- L
  attr(results, "theta.low")   <- theta.low
  attr(results, "theta.up")    <- theta.up

  dots <- list(...)
  if (length(dots) > 0L) {
    for (nm in names(dots)) {
      attr(results, nm) <- dots[[nm]]
    }
  }

  results
}

# ---- Extract pointwise log-likelihood -----------------------------------------

#' Extract pointwise log-likelihood matrix
#'
#' For Stan-fitted models, returns the posterior draws of pointwise
#' log-likelihood (an \eqn{S \times N_{\text{obs}}} matrix for most models,
#' or \eqn{S \times N} for FCDCM and FCGDINA).  For EM and iStEM fits,
#' returns \code{NULL}.
#'
#' The returned matrix can be passed directly to \code{\link[loo]{loo}} or
#' \code{\link[loo]{waic}} for model comparison.
#'
#' @param object A fitted ForceChoice model object.
#' @param ...    Additional arguments (currently ignored).
#' @return An \eqn{S \times N} matrix of pointwise log-likelihood draws
#'   (Stan), or \code{NULL} (EM / iStEM).
#'
#' @examples
#' \donttest{
#' # stan code, long time
#' sim <- sim.data.MIRT(N = 20, I = 6, D = 2, model = "2PL")
#' fit <- fit.MIRT(sim$response, model = "2PL", D = 2, method = "stan")
#' log_lik <- get.log_lik(fit)
#' loo::loo(log_lik)
#' loo::waic(log_lik)
#' }
#' @export
get.log_lik <- function(object, ...) {
  UseMethod("get.log_lik")
}

#' @describeIn get.log_lik Default method; returns \code{object$log_lik}
#'   when present and \code{NULL} otherwise.
#' @export
get.log_lik.default <- function(object, ...) {
  if (!is.null(object$log_lik)) {
    return(object$log_lik)
  }
  NULL
}

#' @describeIn get.fit.index TIRT model: pairwise binary responses with
#'   within-block pairs excluded via \code{bivariate.groups}.
#' @export
get.fit.index.TIRT <- function(object, ...) {
  block.items <- object$arguments$block.items
  Q.matrix <- object$Q.matrix
  response <- object$response
  par <- object$par$est
  D <- ncol(par) - 1L

  gamma.matrix <- object$gamma.matrix$est
  pairs.matrix <- object$pairs.matrix
  pairs.value  <- object$pairs.value
  fc.type      <- object$fc.type

  logLik <- object$logLik
  pi <- as.vector(attr(logLik, "pi"))
  theta.norm <- attr(logLik, "theta.norm")
  N.person <- nrow(response)
  I.pairs <- ncol(response)
  I.states <- nrow(par)
  npar <- object$npar
  N.block <- length(block.items)

  I.block <- length(block.items[[1]])
  is_case_a <- (I.block == 2 && D > 2)
  is_case_b <- (D == 2 && I.block == 2)

  if (is_case_b) {
    idx_lambda_fix <- block.items[[1]]
  } else {
    idx_lambda_fix <- integer(0)
  }

  if (is_case_a || is_case_b) {
    idx_psi_fix <- 1:I.states
  } else {
    idx_psi_fix <- sapply(block.items, function(x) x[length(x)])
  }

  idx_psi_est <- setdiff(1:I.states, idx_psi_fix)

  pairs.char <- apply(pairs.matrix, 1, function(x){
    return(paste0(x[1], x[2]))
  })

  I.block.vec <- sapply(block.items, length)
  pairs.per.block.all <- choose(I.block.vec, 2L)
  block.pair.counts <- integer(N.block)
  for (b in seq_len(N.block)) {
    k <- I.block.vec[b]
    if (fc.type[b] == "RANK") {
      block.pair.counts[b] <- choose(k, 2L)
    } else if (fc.type[b] == "MOLE") {
      block.pair.counts[b] <- 2L * k - 3L
    } else if (fc.type[b] == "PICK") {
      block.pair.counts[b] <- k - 1L
    }
  }

  has.pairs.value <- !is.null(pairs.value) && !all(fc.type == "RANK")

  if (has.pairs.value) {
    expanded <- cpp_tirt_expand_response_full(
      response = matrix(as.integer(response), nrow = N.person, ncol = ncol(response)),
      pairs_matrix = pairs.matrix,
      pairs_value = pairs.value,
      block_pair_counts = block.pair.counts
    )
    response.full <- expanded$response.full
    response.use <- response.full
    idx_gamma_est <- expanded$idx.gamma.est
  } else {
    response.use <- response
    idx_gamma_est <- seq_len(nrow(pairs.matrix))
  }
  idx_gamma_fix <- setdiff(seq_len(nrow(pairs.matrix)), idx_gamma_est)

  if (has.pairs.value) {
    cb.resp <- rep(seq_len(N.block), times = pairs.per.block.all)
  } else {
    cb.resp <- rep(seq_len(N.block), times = block.pair.counts)
  }

  extract.free.par <- function(par, Q.matrix, idx_lambda_fix, idx_psi_est,
                               gamma.matrix, pairs.matrix, D, pairs.char) {
    par.vec <- numeric(0)
    for(i in 1:I.states){
      if(! i %in% idx_lambda_fix){
        d <- which(Q.matrix[i, ] != 0)[1]
        par.vec <- c(par.vec, par[i, d])
      }
    }
    for (i in idx_psi_est) {
      par.vec <- c(par.vec, par[i, D + 1L])
    }

    if (length(idx_gamma_est) > 0) {
      pairs.char.est <- pairs.char[idx_gamma_est]
      for (i in seq_along(pairs.char.est)) {
        pos <- match(pairs.char.est[i], pairs.char)
        par.vec <- c(par.vec, gamma.matrix[pairs.matrix[pos, 1], pairs.matrix[pos, 2]])
      }
    }

    return(par.vec)
  }

  expand.free.par <- function(par.vec, ...) {
    lambda <- rep(NA_real_, I.states)
    lambda[idx_lambda_fix] <- 0.80
    if(is_case_a || is_case_b){
      psi2 <- rep(0.50, I.states)
    }else{
      psi2 <- rep(1.00, I.states)
    }
    gamma.matrix <- matrix(NA_real_, nrow = I.states, ncol = I.states)

    idx <- 1L
    for (i in 1:I.states) {
      if (! i %in% idx_lambda_fix) {
        lambda[i] <- par.vec[idx]
        idx <- idx + 1L
      }
    }

    for (i in idx_psi_est) {
      psi2[i] <- par.vec[idx]
      idx <- idx + 1L
    }

    for (fix_idx in idx_gamma_fix) {
      r <- pairs.matrix[fix_idx, 1]
      c <- pairs.matrix[fix_idx, 2]
      gamma.matrix[r, c] <- 0
      gamma.matrix[c, r] <- 0
    }
    if (length(idx_gamma_est) > 0) {
      for (i in seq_along(idx_gamma_est)) {
        r <- pairs.matrix[idx_gamma_est[i], 1]
        c <- pairs.matrix[idx_gamma_est[i], 2]
        gamma.matrix[r, c] <- par.vec[idx + i - 1]
        gamma.matrix[c, r] <- -par.vec[idx + i - 1]
      }
    }
    psi2 <- pmax(psi2, .Machine$double.eps)
    return(list(lambda = lambda, psi2 = psi2, gamma.matrix = gamma.matrix))
  }

  par.vec <- extract.free.par(par, Q.matrix, idx_lambda_fix, idx_psi_est,
                              gamma.matrix, pairs.matrix, D, pairs.char)

  loglik.fun <- function(par.vec, ...) {
    mats <- expand.free.par(par.vec, ...)
    prob_new <- model.TIRT(theta.norm, mats$lambda, mats$psi2,
                           mats$gamma.matrix, Q.matrix, pairs.matrix)

    if (has.pairs.value) {
      lik <- cpp_loglik_binary_person_pairs(
        prob = prob_new,
        response = matrix(as.integer(response), nrow = N.person, ncol = ncol(response)),
        pi = pi,
        pairs_matrix = pairs.matrix,
        pairs_value = pairs.value,
        block_pair_counts = block.pair.counts
      )
    } else {
      lik <- cpp_loglik_binary(
        prob = prob_new,
        response = matrix(as.integer(response), nrow = N.person, ncol = I.pairs),
        pi = pi
      )
    }

    lik$logLik
  }

  prob.fun <- function(par.vec, ...) {
    mats <- expand.free.par(par.vec, ...)
    model.TIRT(theta.norm, mats$lambda, mats$psi2,
               mats$gamma.matrix, Q.matrix, pairs.matrix)
  }

  good.of.fit(
    par.vec     = par.vec,
    loglik.fun  = loglik.fun,
    prob.fun    = prob.fun,
    response    = response.use,
    npar        = npar,
    pi          = pi,
    response.type = "binary",
    bivariate.groups = cb.resp,
    ...
  )
}

#' Simulate Data from the Forced-Choice DCM Model
#'
#' @description
#' Generates forced-choice paired-comparison data from a higher-order
#' cognitive diagnostic model (DINA/DINO). The continuous higher-order
#' trait governs attribute mastery probabilities via a logistic link;
#' two-statement blocks produce binary choices based on attribute
#' mastery patterns. In each block, selecting the first statement has
#' probability \eqn{\eta_0} when its condensed mastery is lower than the
#' second statement's, \eqn{0.5} when they are equal, and
#' \eqn{0.5+\eta_{AB}} when it is higher.
#'
#' @param N.person Integer; number of persons (default: 1000).
#' @param N.block Integer; number of forced-choice block pairs (default: 10).
#' @param D Integer; number of attributes (default: 3, must be >= 2).
#' @param dcm.type Character; \code{"DINA"} (default) or \code{"DINO"}.
#' @param control Optional list with entries \code{Q.matrix}, \code{block.items},
#'   \code{single}, \code{delta1}, \code{delta0},
#'   \code{theta}, \code{alpha}, \code{par}.  If \code{block.items} is
#'   omitted, statements are assembled into cross-Q-vector forced-choice
#'   blocks with balanced first/second positions and diversified attribute
#'   pairs.  \code{par} is a \eqn{B \times 2} matrix with columns
#'   \code{eta0} and \code{etaAB}.
#'
#' @return An object of class \code{"data.FCDCM"} containing \code{data},
#'   \code{response}, \code{theta}, \code{alpha}, \code{zeta}, \code{par},
#'   \code{Q.matrix}, \code{block.items},
#'   \code{prob}, and data-generating arguments.
#'
#' @seealso \code{\link{fit.FCDCM}}, \code{\link{model.FCDCM}}
#'
#' @examples
#' set.seed(123)
#' sim <- sim.data.FCDCM(N.person = 20, N.block = 3, D = 2,
#'                       dcm.type = "DINA")
#' str(sim$data)
#' head(sim$response)
#' head(sim$alpha)
#'
#' @export
sim.data.FCDCM <- function(N.person = 1000, N.block = 10, D = 3,
                           dcm.type = "DINA", control = NULL) {

  call <- match.call()

  if (is.null(control)) {
    control <- list()
  }

  N.person <- as.integer(N.person[1L])
  N.block <- as.integer(N.block[1L])
  D <- as.integer(D[1L])
  if (is.na(N.person) || N.person < 1L ||
      is.na(N.block) || N.block < 1L ||
      is.na(D) || D < 2L) {
    stop("'N.person', 'N.block', and 'D' must be valid positive integers.", call. = FALSE)
  }
  I.states <- 2 * N.block

  Q.matrix    <- control$Q.matrix
  single      <- control$single
  if (is.null(Q.matrix)) {
    if (is.null(single)) {
      single <- TRUE
    }
    Q.matrix <- sim.data.Q.CDM(D, I.states, single = single)
  }

  block.items <- control$block.items
  if(is.null(block.items)){
    # Group items by their Q-vector pattern and interleave to ensure
    # each block pairs items with DIFFERENT Q-vectors.  Blocks whose
    # two items share the same Q-vector are completely uninformative
    # under DINA/DINO (za = zb for every attribute class).
    q.hash <- apply(Q.matrix, 1L, paste, collapse = ":")
    q.groups <- split(seq_len(I.states), q.hash)
    q.groups <- lapply(q.groups, function(x) sample(x, length(x)))
    n.groups <- length(q.groups)
    if (n.groups >= 2L) {
      block.items <- fcdcm_balanced_q_blocks(q.groups, N.block)
    } else {
      interleaved <- sample(seq_len(I.states), I.states)
      block.items <- split(interleaved, rep(seq_len(N.block), each = 2L))
    }
  }
  patterns <- matrix(unlist(block.items), ncol=2, byrow=TRUE)

  dcm.type <- toupper(dcm.type)
  if (length(dcm.type) != I.states) {
    dcm.type <- rep(dcm.type[1L], I.states)
  }
  if (!all(dcm.type %in% c("DINA", "DINO"))) {
    stop("'dcm.type' must contain only 'DINA' or 'DINO'.", call. = FALSE)
  }

  delta1 <- control$delta1
  delta0 <- control$delta0
  theta  <- control$theta
  alpha  <- control$alpha

  if (is.null(alpha)) {
    if (is.null(delta1)) {
      delta1 <- rlnorm(D, meanlog = 0.25, sdlog = 0.25)
    } else {
      delta1 <- as.numeric(delta1)
      if (length(delta1) != D) {
        stop("'control$delta1' must have length ", D, ".", call. = FALSE)
      }
    }

    if (is.null(delta0)) {
      delta0 <- rnorm(D, mean = 0, sd = 1)
    } else {
      delta0 <- as.numeric(delta0)
      if (length(delta0) != D) {
        stop("'control$delta0' must have length ", D, ".", call. = FALSE)
      }
    }

    if (is.null(theta)) {
      theta <- rnorm(N.person, mean = 0, sd = 1)
    } else {
      theta <- as.numeric(theta)
      if (length(theta) != N.person) {
        stop("'control$theta' must have length ", N.person, ".", call. = FALSE)
      }
    }

    prob.alpha <- distribution_higher_order(theta = theta, delta1 = delta1, delta0 = delta0)
    alpha <- (prob.alpha > matrix(runif(N.person * D), N.person, D)) * 1L
  } else {
    alpha <- as.matrix(alpha)
    if (nrow(alpha) != N.person || ncol(alpha) != D) {
      stop("'control$alpha' must be a ", N.person, "x", D, " binary matrix.", call. = FALSE)
    }
    storage.mode(alpha) <- "integer"
    if (is.null(theta)) theta <- rep(NA_real_, N.person)
    if (is.null(delta1)) delta1 <- rep(NA_real_, D)
    if (is.null(delta0)) delta0 <- rep(NA_real_, D)
    prob.alpha <- if (all(is.finite(theta)) &&
                      all(is.finite(delta1)) &&
                      all(is.finite(delta0))) {
      distribution_higher_order(theta = theta, delta1 = delta1, delta0 = delta0)
    } else {
      matrix(NA_real_, N.person, D)
    }
  }

  zeta <- get.zeta(alpha = alpha, Q.matrix = Q.matrix, dcm.type = dcm.type)

  par <- control$par
  if(is.null(par)){
    par <- cbind(
      eta0 = runif(N.block, 0.05, 0.15),
      etaAB = runif(N.block, 0.30, 0.50)
    )
  }
  par <- fcdcm_normalize_par(par, B = N.block, arg = "control$par")

  prob <- model.FCDCM(zeta = zeta, par = par, patterns = patterns)
  response <- (prob > matrix(runif(N.person * N.block), N.person, N.block)) * 1L
  data <- as.matrix(get.data.from.response.FCDCM(response, block.items))

  block.names <- paste0("B", seq_len(N.block))
  item.names <- rownames(Q.matrix)
  if (is.null(item.names)) {
    item.names <- paste0("S", seq_len(I.states))
  }
  dim.names <- colnames(Q.matrix)
  if (is.null(dim.names)) {
    dim.names <- paste0("Dim.", seq_len(D))
  }

  names(delta1) <- names(delta0) <- dim.names


  colnames(alpha) <- colnames(prob.alpha) <- colnames(Q.matrix) <- dim.names
  colnames(zeta) <- rownames(Q.matrix) <- item.names
  colnames(response) <- colnames(data) <- colnames(prob) <- rownames(patterns) <-
    rownames(par) <- names(block.items) <- block.names
  colnames(patterns) <- c("SA", "SB")

  data.obj <- list(
    data        = data,
    response    = response,
    theta       = theta,
    alpha       = alpha,
    prob.alpha  = prob.alpha,
    zeta        = zeta,
    par         = par,
    delta1      = delta1,
    delta0      = delta0,
    Q.matrix    = Q.matrix,
    block.items = block.items,
    patterns    = patterns,
    dcm.type    = dcm.type,
    prob        = prob,
    N.person    = N.person,
    N.block     = N.block,
    I.block     = 2L,
    D           = D,
    I.states    = I.states,
    call        = call,
    arguments   = list(
      N.person = N.person,
      N.block  = N.block,
      D        = D,
      dcm.type = dcm.type,
      control  = control
    )
  )

  class(data.obj) <- "data.FCDCM"

  return(data.obj)
}

fcdcm_balanced_q_blocks <- function(q.groups, N.block) {
  q.keys <- names(q.groups)
  n.item <- vapply(q.groups, length, integer(1L))
  if (length(q.keys) == 2L && n.item[1L] == n.item[2L] &&
      n.item[1L] %% 2L == 0L) {
    n.first <- n.item[1L] %/% 2L
    blocks <- vector("list", N.block)
    pos <- 1L
    for (i in seq_len(n.first)) {
      blocks[[pos]] <- c(q.groups[[q.keys[1L]]][i],
                         q.groups[[q.keys[2L]]][n.first + i])
      pos <- pos + 1L
      blocks[[pos]] <- c(q.groups[[q.keys[2L]]][i],
                         q.groups[[q.keys[1L]]][n.first + i])
      pos <- pos + 1L
    }
    return(blocks[sample(seq_along(blocks))])
  }

  first.n <- n.item %/% 2L
  extra <- N.block - sum(first.n)
  if (extra > 0L) {
    add <- order(n.item %% 2L, n.item, decreasing = TRUE)[seq_len(extra)]
    first.n[add] <- first.n[add] + 1L
  }
  second.n <- n.item - first.n

  first.pool <- second.pool <- vector("list", length(q.keys))
  names(first.pool) <- names(second.pool) <- q.keys
  for (key in q.keys) {
    first.pool[[key]] <- q.groups[[key]][seq_len(first.n[key])]
    second.pool[[key]] <- q.groups[[key]][first.n[key] + seq_len(second.n[key])]
  }

  pair.keys <- combn(q.keys, 2L, function(x) paste(sort(x), collapse = "\r"))
  ordered.grid <- expand.grid(first = q.keys, second = q.keys,
                              stringsAsFactors = FALSE)
  ordered.grid <- ordered.grid[ordered.grid$first != ordered.grid$second, ,
                               drop = FALSE]
  ordered.keys <- paste(ordered.grid$first, ordered.grid$second, sep = "\r")

  build_once <- function() {
    fp <- first.pool
    sp <- second.pool
    pair.count <- setNames(integer(length(pair.keys)), pair.keys)
    ordered.count <- setNames(integer(length(ordered.keys)), ordered.keys)
    out <- vector("list", N.block)
    key.out <- matrix(NA_character_, N.block, 2L)

    for (b in seq_len(N.block)) {
      first.left <- vapply(fp, length, integer(1L))
      second.left <- vapply(sp, length, integer(1L))
      first.available <- q.keys[first.left > 0L]
      second.available <- q.keys[second.left > 0L]
      candidates <- expand.grid(first = first.available,
                                second = second.available,
                                stringsAsFactors = FALSE)
      score <- vapply(seq_len(nrow(candidates)), function(i) {
        first.key <- candidates$first[i]
        second.key <- candidates$second[i]
        f <- first.left
        s <- second.left
        f[first.key] <- f[first.key] - 1L
        s[second.key] <- s[second.key] - 1L
        feasible <- all(f <= sum(s) - s) && all(s <= sum(f) - f)
        same.penalty <- if (identical(first.key, second.key)) 1e7 else 0
        feasible.penalty <- if (!feasible) 1e5 else 0
        pair.key <- paste(sort(c(first.key, second.key)), collapse = "\r")
        ordered.key <- paste(first.key, second.key, sep = "\r")
        pair.penalty <- if (identical(first.key, second.key)) {
          0
        } else {
          10 * pair.count[pair.key] + ordered.count[ordered.key]
        }
        same.penalty + feasible.penalty + pair.penalty
      }, numeric(1L))
      chosen <- sample(which(score == min(score)), 1L)
      first.key <- candidates$first[chosen]
      second.key <- candidates$second[chosen]
      out[[b]] <- c(fp[[first.key]][1L], sp[[second.key]][1L])
      key.out[b, ] <- c(first.key, second.key)
      fp[[first.key]] <- fp[[first.key]][-1L]
      sp[[second.key]] <- sp[[second.key]][-1L]
      if (!identical(first.key, second.key)) {
        pair.key <- paste(sort(c(first.key, second.key)), collapse = "\r")
        ordered.key <- paste(first.key, second.key, sep = "\r")
        pair.count[pair.key] <- pair.count[pair.key] + 1L
        ordered.count[ordered.key] <- ordered.count[ordered.key] + 1L
      }
    }

    same <- sum(key.out[, 1L] == key.out[, 2L])
    list(
      block.items = out,
      same = same,
      score = same * 1e6 + sum(pair.count^2) + sum(ordered.count^2)
    )
  }

  best <- NULL
  for (attempt in seq_len(100L)) {
    candidate <- build_once()
    if (is.null(best) || candidate$score < best$score) best <- candidate
    if (candidate$same == 0L) return(candidate$block.items)
  }

  best$block.items
}
#' Compute FCDCM Block Response Probabilities
#' @param zeta Binary statement-level mastery status matrix.
#' @param par Block parameter matrix. A \eqn{B \times 2} matrix with columns
#'   \code{eta0} and \code{etaAB}. Equal-condensation probabilities
#'   (\eqn{\zeta_{A_b}=\zeta_{B_b}}) are fixed at 0.5.
#' @param patterns Block statement-pair matrix. Column 1 is the first
#'   statement \eqn{A_b}; column 2 is the second statement \eqn{B_b}.
#' @return Matrix of probabilities for choosing the first statement in each
#'   block:
#'   \eqn{\eta_{0b}} if \eqn{\zeta_{A_b}<\zeta_{B_b}},
#'   \eqn{0.5} if equal, and
#'   \eqn{0.5+\eta_{ABb}} if \eqn{\zeta_{A_b}>\zeta_{B_b}}.
#' @keywords internal
model.FCDCM <- function(zeta, par, patterns) {
  zeta <- as.matrix(zeta)
  patterns <- as.matrix(patterns)
  storage.mode(zeta) <- "integer"
  storage.mode(patterns) <- "integer"
  par <- fcdcm_normalize_par(par, B = nrow(patterns), arg = "par")
  cpp_model_FCDCM(zeta, par, patterns)
}

get.zeta <- function(alpha, Q.matrix, dcm.type = "DINA") {
  alpha <- as.matrix(alpha)
  Q.matrix <- as.matrix(Q.matrix)
  storage.mode(alpha) <- "numeric"
  storage.mode(Q.matrix) <- "numeric"

  N.person <- nrow(alpha)
  I.states <- nrow(Q.matrix)

  dcm.type <- toupper(dcm.type)
  if (length(dcm.type) != I.states) {
    dcm.type <- rep(dcm.type[1L], I.states)
  }

  eta <- alpha %*% t(Q.matrix)
  required <- rowSums(Q.matrix)
  zeta <- matrix(0L, N.person, I.states)
  is.dina <- dcm.type == "DINA"
  if (any(is.dina)) {
    zeta[, is.dina] <- sweep(eta[, is.dina, drop = FALSE], 2L,
                             required[is.dina], "==")
  }
  if (any(!is.dina)) {
    zeta[, !is.dina] <- eta[, !is.dina, drop = FALSE] > 0
  }

  storage.mode(zeta) <- "integer"
  return(zeta)
}

fcdcm_alpha_profile_logprob <- function(theta, delta1, delta0, alpha.patterns,
                                        eps = 1e-12) {
  theta <- as.numeric(theta)
  alpha.patterns <- as.matrix(alpha.patterns)
  prob.alpha <- distribution_higher_order(theta, delta1, delta0)
  prob.alpha <- pmin(pmax(prob.alpha, eps), 1 - eps)

  log.alpha <- log(prob.alpha) %*% t(alpha.patterns) +
    log1p(-prob.alpha) %*% t(1 - alpha.patterns)
  max.log <- apply(log.alpha, 1L, max)
  log.denom <- max.log +
    log(rowSums(exp(sweep(log.alpha, 1L, max.log, "-"))))
  sweep(log.alpha, 1L, log.denom, "-")
}

fcdcm_alpha_profile_prob <- function(theta, delta1, delta0, alpha.patterns,
                                     eps = 1e-12) {
  exp(fcdcm_alpha_profile_logprob(theta, delta1, delta0, alpha.patterns, eps))
}

fcdcm_latent_support <- function(theta, pi, delta1, delta0, par,
                                 alpha.patterns, zeta.patterns, patterns,
                                 eps = 1e-12) {
  theta <- as.matrix(theta)
  if (ncol(theta) != 1L) {
    theta <- matrix(as.numeric(theta[, 1L]), ncol = 1L)
  }
  pi <- as.vector(pi)
  if (length(pi) != nrow(theta)) {
    stop("'pi' must have one weight per theta support point.", call. = FALSE)
  }
  pi <- pmax(pi, 0)
  pi <- pi / sum(pi)

  support <- cpp_fcdcm_latent_support(
    theta = theta,
    pi = pi,
    delta1 = as.numeric(delta1),
    delta0 = as.numeric(delta0),
    par = par,
    alpha_patterns = alpha.patterns,
    zeta_patterns = zeta.patterns,
    patterns = patterns
  )
  colnames(support$theta) <- colnames(theta)
  colnames(support$alpha) <- colnames(alpha.patterns)
  colnames(support$alpha.weight) <- rownames(alpha.patterns)
  colnames(support$prob) <- rownames(patterns)
  colnames(support$class.prob) <- rownames(patterns)
  support
}

fcdcm_free_par_vec <- function(delta1, delta0, par) {
  par <- fcdcm_normalize_par(par)
  par.free <- par
  par.free[, "eta0"] <- pmin(pmax(par.free[, "eta0"] / 0.5, 1e-8), 1 - 1e-8)
  par.free[, "etaAB"] <- pmin(pmax(par.free[, "etaAB"] / 0.5, 1e-8), 1 - 1e-8)
  c(
    log(pmax(as.numeric(delta1), 1e-8)),
    as.numeric(delta0),
    as.numeric(qlogis(par.free))
  )
}

fcdcm_expand_free_par <- function(par.vec, D, B, dim.names = NULL,
                                  block.names = NULL,
                                  par.names = c("eta0", "etaAB")) {
  idx <- 1L
  delta1 <- exp(par.vec[idx:(idx + D - 1L)])
  idx <- idx + D
  delta0 <- par.vec[idx:(idx + D - 1L)]
  idx <- idx + D
  K <- length(par.names)
  par <- matrix(plogis(par.vec[idx:(idx + B * K - 1L)]),
                nrow = B, ncol = K)
  colnames(par) <- par.names
  if ("eta0" %in% par.names) par[, "eta0"] <- 0.5 * par[, "eta0"]
  if ("etaAB" %in% par.names) par[, "etaAB"] <- 0.5 * par[, "etaAB"]
  par <- fcdcm_normalize_par(par, B = B)
  pars <- list(delta1 = delta1, delta0 = delta0, par = par)
  names(pars$delta1) <- names(pars$delta0) <- dim.names
  rownames(pars$par) <- block.names
  pars
}

fcdcm_normalize_par <- function(par, B = NULL, arg = "par") {
  par <- as.matrix(par)
  storage.mode(par) <- "numeric"
  if (!is.null(B) && nrow(par) != B) {
    stop("'", arg, "' must have ", B, " rows.", call. = FALSE)
  }
  if (ncol(par) != 2L) {
    stop("'", arg, "' must be a B x 2 matrix (eta0, etaAB).",
         call. = FALSE)
  }
  out <- cbind(eta0 = par[, 1L], etaAB = par[, 2L])
  rownames(out) <- rownames(par)
  if (anyNA(out) || any(!is.finite(out))) {
    stop("'", arg, "' contains non-finite values.", call. = FALSE)
  }
  if (any(out[, "eta0"] <= 0 | out[, "eta0"] >= 0.5) ||
      any(out[, "etaAB"] <= 0 | out[, "etaAB"] >= 0.5)) {
    stop("FCDCM parameters require 0 < eta0 < .5 and 0 < etaAB < .5.",
         call. = FALSE)
  }
  out
}

# library(ForceChoice)
#
# ############################### data ###############################
# seed <- 13549
# N <- 500
# I <- 10
# D <- 2
# length.poly <- 3
# Q.matrix <- NULL
# Corr <- NULL
# rotate <- NULL
#
# set.seed(seed)
# data.obj <- sim.data.MGPCM(N = N, I = I, D = D, length.poly = length.poly,
#                            Q.matrix = Q.matrix, Corr = Corr, rotate = rotate)
#
# response <- data.obj$response
# head(response)
#
# # ############################### Stan MCMC ###############################
# # object.MGPCM.stan <- fit.MGPCM(response, D = D, Q.matrix = Q.matrix,
# #                                method = "stan",
# #                                control.method = c(list(
# #                                  init = "random",
# #                                  algorithm = "HMC"), list(seed = seed, vis = TRUE)))
# # cbind(data.obj$par, object.MGPCM.stan$par$est)
# # get.fit.index(object.MGPCM.stan)
#
# ############################### iStEM ###############################
# iStEM.control.method <- list(
#   burnin.maxitr = 40,
#   maxitr = 500,
#   theta.proposal.sd = 0.35,
#   optim.maxit = 50
# )
#
# object.MGPCM.iStEM <- fit.MGPCM(response, D = D, Q.matrix = Q.matrix,
#                                 method = "iStEM",
#                                 control.method = c(iStEM.control.method, list(seed = seed, vis = TRUE)))
# cbind(data.obj$par, object.MGPCM.iStEM$par$est)
#
# # ---- Validate logLik with new quadrature helpers ----
# ll <- logLik(object.MGPCM.iStEM)
# stopifnot(inherits(ll, "logLik"))
# stopifnot(is.finite(as.numeric(ll)))
# cat(sprintf("  MGPCM logLik: %.2f (df=%d)\n", as.numeric(ll), attr(ll, "df")))
#
# get.fit.index(object.MGPCM.iStEM)

#' Triplets of Pairwise Comparisons
#'
#' @description
#' A synthetic forced-choice data set containing pairwise comparison responses
#' for the first 200 of 2000 participants on four triplets. The data were
#' originally generated as part of Brown and Maydeu-Olivares (2012) and are
#' distributed in the published R package \pkg{thurstonianIRT}.
#'
#' In each triplet, participants ranked three alternative statements according
#' to their preference. The rankings were then converted into dichotomous
#' pairwise responses for all three within-triplet comparisons.
#'
#' @format A data frame with 200 rows and 12 integer columns. Values are 0/1
#'   pairwise preference indicators. Overall, the 12 statements measure three
#'   traits. Items 1, 4, 7, and 10 load on trait 1; items 2, 5, 8, and 11 load
#'   on trait 2; and items 3, 6, 9, and 12 load on trait 3. Items 4, 9, and 11
#'   are reverse-keyed.
#' \describe{
#'   \item{i1i2}{Response preference between item 1 and item 2.}
#'   \item{i1i3}{Response preference between item 1 and item 3.}
#'   \item{i2i3}{Response preference between item 2 and item 3.}
#'   \item{i4i5}{Response preference between item 4 and item 5.}
#'   \item{i4i6}{Response preference between item 4 and item 6.}
#'   \item{i5i6}{Response preference between item 5 and item 6.}
#'   \item{i7i8}{Response preference between item 7 and item 8.}
#'   \item{i7i9}{Response preference between item 7 and item 9.}
#'   \item{i8i9}{Response preference between item 8 and item 9.}
#'   \item{i10i11}{Response preference between item 10 and item 11.}
#'   \item{i10i12}{Response preference between item 10 and item 12.}
#'   \item{i11i12}{Response preference between item 11 and item 12.}
#' }
#'
#' @source
#' Brown, A. & Maydeu-Olivares, A. (2012). Fitting a Thurstonian IRT model to
#' forced-choice data using Mplus. Behavior Research Methods, 44, 1135-1147.
#' doi:10.3758/s13428-012-0217-x
#'
#' The data file was taken from the published R package \pkg{thurstonianIRT}.
#'
#' @examples
#' data("triplets")
#' dim(triplets)
#' head(triplets)
#'
#' @docType data
#' @keywords datasets
#' @name triplets
NULL

#' @title S3 Methods: plot
#'
#' @description
#' Draws log-likelihood trace plots for fitted \pkg{ForceChoice} model
#' objects. Stan, iStEM, and EM fits are visualised through the same
#' convergence target: the total log-likelihood recorded across iterations
#' of the corresponding estimation algorithm.
#'
#' @param x A fitted model object of class \code{"MIRT"},
#'   \code{"MGPCM"}, \code{"MGGUM"}, \code{"FCMIRT"}, \code{"FCDCM"},
#'   \code{"FCGGUM"}, \code{"FCGDINA"}, or \code{"TIRT"}.
#' @param y Ignored.
#' @param ... Ignored.
#'
#' @return Invisibly returns \code{NULL}. Plots are produced as a side
#'   effect on the current graphics device.
#'
#' @details
#' The horizontal axis is the saved Stan iteration, iStEM iteration, or EM
#' iteration. The vertical axis is the summed log-likelihood. Stan plots use
#' \code{log_lik} or \code{log_lik_group} from generated quantities and draw
#' all chains in one panel. iStEM plots use the iteration-level
#' \code{iStEM$logLik.trace}, and the stable chain is the final
#' \code{M * B} iStEM iterations. EM plots use \code{EM$logLik.trace}. For
#' Stan and iStEM fits, a vertical dashed line marks the stable-chain
#' boundary. EM fits do not have a stable-chain boundary.
#'
#' @name plot
NULL

`%||%` <- function(a, b) if (is.null(a)) b else a

# ---- Internal helpers ----

.fc_trace_palette <- function(n) {
  rep_len(c("#4F7FA7", "#B07A7A", "#6F9E7A", "#9A8F68",
            "#7B6FA6", "#6B9CA0"), n)
}

.fc_finite_ylim <- function(x) {
  x <- as.numeric(x)
  x <- x[is.finite(x)]
  if (length(x) == 0L) {
    stop("No finite log-likelihood values are available for plotting.",
         call. = FALSE)
  }
  r <- range(x)
  if (diff(r) == 0) r <- r + c(-0.5, 0.5)
  pad <- diff(r) * 0.05
  r + c(-pad, pad)
}

.fc_finite_xlim <- function(x) {
  x <- as.numeric(x)
  x <- x[is.finite(x)]
  if (length(x) == 0L) {
    stop("No finite iteration values are available for plotting.",
         call. = FALSE)
  }
  r <- range(x)
  if (diff(r) == 0) r <- r + c(-0.5, 0.5)
  r
}

.fc_axis_ticks <- function(lim, n = 8L, special = NULL) {
  ticks <- pretty(lim, n = n)
  ticks <- ticks[ticks >= lim[1L] & ticks <= lim[2L]]
  special <- special[is.finite(special)]
  special <- special[special >= lim[1L] & special <= lim[2L]]
  ticks <- c(ticks, special)
  if (length(ticks) == 0L) ticks <- lim
  sort(unique(ticks))
}

.fc_axis_labels <- function(ticks) {
  ticks <- as.numeric(ticks)
  finite <- ticks[is.finite(ticks)]
  nonzero <- abs(finite[finite != 0])
  scientific <- length(finite) > 0L && (
    max(abs(finite)) >= 1e6 ||
      (length(nonzero) > 0L && min(nonzero) < 1e-3)
  )
  if (scientific) {
    return(format(ticks, trim = TRUE, scientific = TRUE, digits = 3))
  }
  vapply(ticks, function(x) {
    if (!is.finite(x)) return("")
    if (abs(x - round(x)) < 1e-8) {
      return(format(round(x), trim = TRUE, scientific = FALSE))
    }
    sub("\\.?0+$", "", format(round(x, 2L), trim = TRUE,
                              scientific = FALSE))
  }, character(1L))
}

.fc_text_width <- function(labels, cex = 1) {
  labels <- labels[nzchar(labels)]
  if (length(labels) == 0L) return(0)
  out <- tryCatch(
    graphics::strwidth(labels, units = "inches", cex = cex),
    error = function(e) rep(0.11 * cex, length(labels))
  )
  max(out, na.rm = TRUE)
}

.fc_text_height <- function(labels, cex = 1) {
  labels <- labels[nzchar(labels)]
  if (length(labels) == 0L) return(0)
  out <- tryCatch(
    graphics::strheight(labels, units = "inches", cex = cex),
    error = function(e) rep(0.16 * cex, length(labels))
  )
  max(out, na.rm = TRUE)
}

.fc_line_height <- function() {
  line <- graphics::par("cin")[2L] * graphics::par("lheight")
  if (!is.finite(line) || line <= 0) line <- 0.2
  line
}

.fc_trace_layout <- function(x_labels, y_labels, xlab, ylab, main) {
  cex.axis <- 0.95
  cex.lab <- 1.00
  cex.main <- 1.05
  line <- .fc_line_height()

  y_tick_width <- .fc_text_width(y_labels, cex.axis)
  y_title_width <- .fc_text_height(ylab, cex.lab)
  x_tick_height <- .fc_text_height(x_labels, cex.axis)
  x_title_height <- .fc_text_height(xlab, cex.lab)
  main_height <- .fc_text_height(main, cex.main)

  axis_label_line <- 0.75
  xlab_line <- axis_label_line + x_tick_height / line + 1.25
  ylab_line <- axis_label_line + y_tick_width / line + 1.25
  bottom <- xlab_line + x_title_height / line + 0.75
  left <- ylab_line + y_title_width / line + 0.85
  top <- main_height / line + 1.8
  right <- 1.2

  graphics::par(
    mar = c(bottom, left, top, right),
    mgp = c(2.0, axis_label_line, 0),
    tcl = -0.25
  )
  list(cex.axis = cex.axis, cex.lab = cex.lab, cex.main = cex.main,
       xlab.line = xlab_line, ylab.line = ylab_line)
}

.fc_protected_ticks <- function(ticks, protected) {
  if (length(protected) == 0L) return(rep(FALSE, length(ticks)))
  tol <- max(1e-8, diff(range(ticks, finite = TRUE)) * 1e-10)
  vapply(ticks, function(x) any(abs(x - protected) <= tol), logical(1L))
}

.fc_select_x_ticks <- function(ticks, labels, xlim, protected = NULL,
                               cex.axis = 1) {
  ord <- order(ticks)
  ticks <- ticks[ord]
  labels <- labels[ord]
  protected <- protected[is.finite(protected)]
  is.protected <- .fc_protected_ticks(ticks, protected)

  plot_width <- graphics::par("pin")[1L]
  if (!is.finite(plot_width) || plot_width <= 0) plot_width <- 5
  label_width <- tryCatch(
    graphics::strwidth(labels, units = "inches", cex = cex.axis),
    error = function(e) rep(0.10 * cex.axis, length(labels))
  )
  label_width <- label_width / plot_width * diff(xlim)
  pad <- 0.015 * diff(xlim)

  keep <- rep(FALSE, length(ticks))
  for (i in seq_along(ticks)) {
    overlap <- which(keep & abs(ticks[i] - ticks) <
                       (label_width[i] + label_width) / 2 + pad)
    if (length(overlap) == 0L) {
      keep[i] <- TRUE
    } else if (is.protected[i]) {
      drop <- overlap[!is.protected[overlap]]
      keep[drop] <- FALSE
      keep[i] <- TRUE
    }
  }

  list(ticks = ticks[keep], labels = labels[keep])
}

.fc_stan_loglik_name <- function(stan.obj) {
  mcmc_names <- tryCatch(names(stan.obj@sim$samples[[1L]]),
                         error = function(e) NULL)
  if (!is.null(mcmc_names)) {
    base_mcmc <- sub("\\[.*$", "", mcmc_names)
    if ("log_lik" %in% base_mcmc) return("log_lik")
    if ("log_lik_group" %in% base_mcmc) return("log_lik_group")
  }

  pars <- tryCatch(
    rstan::summary(stan.obj)$summary,
    error = function(e) NULL
  )
  if (is.null(pars)) {
    stop("Cannot read Stan summary from 'stan.obj'.", call. = FALSE)
  }
  base <- sub("\\[.*$", "", rownames(pars))
  if ("log_lik" %in% base) return("log_lik")
  if ("log_lik_group" %in% base) return("log_lik_group")
  stop("Stan object does not contain 'log_lik' or 'log_lik_group'.",
       call. = FALSE)
}

.fc_stan_loglik_weights <- function(x, par_name, n_loglik) {
  if (par_name != "log_lik_group" || !inherits(x, "FCGDINA")) {
    return(rep(1, n_loglik))
  }
  prep <- fcgdina_prepare(
    data = x$response,
    Q.matrix = x$Q.matrix,
    model = x$model,
    block.items = x$block.items,
    fc.type = x$fc.type
  )
  if (length(prep$response.count) != n_loglik) {
    return(rep(1, n_loglik))
  }
  prep$response.count
}

.fc_stan_loglik_trace <- function(x) {
  par_name <- .fc_stan_loglik_name(x$stan.obj)
  arr <- tryCatch(
    rstan::extract(
      x$stan.obj, pars = par_name, permuted = FALSE, inc_warmup = TRUE
    ),
    error = function(e) {
      stop("Cannot extract Stan log-likelihood draws: ", conditionMessage(e),
           call. = FALSE)
    }
  )
  d <- dim(arr)
  if (is.null(d) || length(d) < 2L) {
    stop("Stan log-likelihood draws have an unsupported shape.",
         call. = FALSE)
  }
  if (length(d) == 2L) {
    out <- arr
  } else {
    n_loglik <- d[3L]
    weights <- .fc_stan_loglik_weights(x, par_name, n_loglik)
    out <- matrix(0, nrow = d[1L], ncol = d[2L])
    for (j in seq_len(n_loglik)) {
      slice <- arr[, , j, drop = FALSE]
      dim(slice) <- d[1:2]
      out <- out + slice * weights[j]
    }
  }
  storage.mode(out) <- "double"
  colnames(out) <- paste0("chain ", seq_len(ncol(out)))
  out
}

.fc_stan_stable_boundary <- function(stan.obj, n_iter) {
  sim <- stan.obj@sim
  iter <- sim$iter %||% n_iter
  warmup <- sim$warmup %||% 0L
  thin <- max(sim$thin %||% 1L)
  if (!is.numeric(warmup) || length(warmup) == 0L ||
      !is.finite(max(warmup)) || max(warmup) <= 0L ||
      !is.numeric(iter) || length(iter) == 0L || !is.finite(max(iter)) ||
      !is.finite(thin) || thin < 1L) {
    return(NA_real_)
  }
  post_warmup_saved <- if (!is.null(sim$n_save) && !is.null(sim$warmup2)) {
    max(as.numeric(sim$n_save) - as.numeric(sim$warmup2), na.rm = TRUE)
  } else {
    ceiling((max(iter) - max(warmup)) / thin)
  }
  boundary <- n_iter - post_warmup_saved + 1L
  if (!is.finite(boundary) || boundary <= 1 || boundary > n_iter) {
    return(NA_real_)
  }
  boundary
}

.fc_istem_loglik_trace <- function(x) {
  trace <- x$iStEM$logLik.trace
  if (is.null(trace)) {
    stop("iStEM object does not contain 'iStEM$logLik.trace'. Refit the ",
         "model to record iteration-level log-likelihood values.",
         call. = FALSE)
  }
  if (is.data.frame(trace)) {
    iter <- trace$iter %||% trace$batch %||% seq_len(nrow(trace))
    value <- trace$logLik %||% trace$log_lik %||% trace$loglik
  } else {
    value <- as.numeric(trace)
    iter <- seq_along(value)
  }
  value <- as.numeric(value)
  iter <- as.numeric(iter)
  keep <- is.finite(iter) & is.finite(value)
  if (!any(keep)) {
    stop("iStEM log-likelihood trace has no finite values.", call. = FALSE)
  }
  data.frame(iter = iter[keep], logLik = value[keep])
}

.fc_istem_boundary <- function(x) {
  boundary <- x$iStEM$stable.start
  if (is.null(boundary)) {
    boundary <- x$iStEM$stable.start.batch
  }
  if (!is.numeric(boundary) || length(boundary) != 1L ||
      !is.finite(boundary) || boundary <= 1) {
    return(NA_real_)
  }
  boundary
}

.fc_em_loglik_trace <- function(x) {
  trace <- x$EM$logLik.trace
  if (is.null(trace)) {
    stop("EM object does not contain 'EM$logLik.trace'.", call. = FALSE)
  }
  iter <- trace$iter %||% seq_len(nrow(trace))
  value <- trace$logLik %||% trace$log_lik %||% trace$loglik
  value <- as.numeric(value)
  iter <- as.numeric(iter)
  keep <- is.finite(iter) & is.finite(value)
  if (!any(keep)) {
    stop("EM log-likelihood trace has no finite values.", call. = FALSE)
  }
  data.frame(iter = iter[keep], logLik = value[keep])
}

.fc_plot_trace_frame <- function(xlim, ylim, xlab, ylab, main,
                                 boundary = NA_real_,
                                 legend_labels = NULL) {
  protected <- c(xlim[2L], boundary)
  x_ticks <- .fc_axis_ticks(xlim, n = 9L, special = protected)
  y_ticks <- .fc_axis_ticks(ylim)
  x_labels <- .fc_axis_labels(x_ticks)
  y_labels <- .fc_axis_labels(y_ticks)
  layout <- .fc_trace_layout(
    x_labels = x_labels, y_labels = y_labels, xlab = xlab, ylab = ylab,
    main = main
  )
  x_axis <- .fc_select_x_ticks(
    x_ticks, x_labels, xlim, protected = protected,
    cex.axis = layout$cex.axis
  )
  graphics::plot(
    NA_real_, NA_real_, type = "n", xlim = xlim, ylim = ylim,
    xlab = "", ylab = "", main = "",
    bty = "l", col.axis = "#374151", col.lab = "#374151",
    col.main = "#1F2937", axes = FALSE
  )
  graphics::grid(col = "#ECEFF1", lty = 1)
  graphics::axis(1, at = x_axis$ticks, labels = x_axis$labels,
                 col.axis = "#374151", cex.axis = layout$cex.axis)
  graphics::axis(2, at = y_ticks, labels = y_labels,
                 las = 1, col.axis = "#374151", cex.axis = layout$cex.axis)
  graphics::mtext(xlab, side = 1L, line = layout$xlab.line,
                  col = "#374151", cex = layout$cex.lab)
  graphics::mtext(ylab, side = 2L, line = layout$ylab.line,
                  col = "#374151", cex = layout$cex.lab)
  graphics::mtext(main, side = 3L, line = 0.6, font = 2L,
                  col = "#1F2937", cex = layout$cex.main)
  graphics::box(bty = "l")
  invisible(layout)
}

.fc_draw_boundary <- function(boundary, y_limits) {
  if (!is.finite(boundary)) return(invisible(NULL))
  graphics::abline(v = boundary, col = "#6B7280", lty = 2, lwd = 1.3)
  invisible(NULL)
}

.fc_trace_data <- function(x) {
  cls <- class(x)[1L]
  if (!is.null(x$stan.obj)) {
    logLik <- .fc_stan_loglik_trace(x)
    n_iter <- nrow(logLik)
    return(list(
      iter = seq_len(n_iter),
      logLik = logLik,
      boundary = .fc_stan_stable_boundary(x$stan.obj, n_iter),
      xlab = "Stan iteration",
      main = paste(cls, "Stan logLik trace"),
      legend_labels = if (ncol(logLik) > 1L) colnames(logLik) else NULL
    ))
  }

  if (!is.null(x$iStEM) && length(x$iStEM) > 0L) {
    trace <- .fc_istem_loglik_trace(x)
    return(list(
      iter = trace$iter,
      logLik = matrix(trace$logLik, ncol = 1L),
      boundary = .fc_istem_boundary(x),
      xlab = "iStEM iteration",
      main = paste(cls, "iStEM logLik trace"),
      legend_labels = NULL
    ))
  }

  if (!is.null(x$EM) && length(x$EM) > 0L) {
    trace <- .fc_em_loglik_trace(x)
    return(list(
      iter = trace$iter,
      logLik = matrix(trace$logLik, ncol = 1L),
      boundary = NA_real_,
      xlab = "EM iteration",
      main = paste(cls, "EM logLik trace"),
      legend_labels = NULL
    ))
  }

  stop("plot() requires Stan, iStEM, or EM log-likelihood trace data.",
       call. = FALSE)
}

.fc_draw_trace_legend <- function(labels, cols, boundary = NA_real_) {
  show_boundary <- is.finite(boundary)
  if (length(labels) == 0L && !show_boundary) return(invisible(NULL))
  legend_labels <- c(labels, if (show_boundary) "stable chain starts")
  legend_cols <- c(cols[seq_along(labels)], if (show_boundary) "#6B7280")
  legend_lty <- c(rep(1, length(labels)), if (show_boundary) 2)
  legend_lwd <- c(rep(2.0, length(labels)), if (show_boundary) 1.3)
  graphics::legend(
    "bottomright", legend = legend_labels, col = legend_cols,
    lty = legend_lty, lwd = legend_lwd, bty = "n",
    cex = max(0.7, min(0.85, 1.4 / sqrt(length(legend_labels)))),
    inset = 0.02, xpd = FALSE
  )
  invisible(NULL)
}

.fc_draw_loglik_trace <- function(trace) {
  logLik <- as.matrix(trace$logLik)
  if (nrow(logLik) != length(trace$iter)) {
    stop("Log-likelihood trace length is inconsistent with iterations.",
         call. = FALSE)
  }
  n_line <- ncol(logLik)
  ylim <- .fc_finite_ylim(logLik)
  xlim <- .fc_finite_xlim(trace$iter)
  legend_labels <- trace$legend_labels

  .fc_plot_trace_frame(
    xlim, ylim, xlab = trace$xlab,
    ylab = "Log-likelihood", main = trace$main,
    boundary = trace$boundary,
    legend_labels = legend_labels
  )
  .fc_draw_boundary(trace$boundary, ylim)

  cols <- .fc_trace_palette(n_line)
  lwd <- if (n_line > 1L) 2.0 else 1.8
  for (i in seq_len(n_line)) {
    graphics::lines(trace$iter, logLik[, i], col = cols[i], lwd = lwd)
  }
  .fc_draw_trace_legend(legend_labels, cols, boundary = trace$boundary)
  invisible(NULL)
}

.fc_plot_trace <- function(x, ...) {
  old_par <- graphics::par(no.readonly = TRUE)
  on.exit(graphics::par(old_par))
  .fc_draw_loglik_trace(.fc_trace_data(x))
}

.fc_plot_method <- function(x, y = NULL, ...) .fc_plot_trace(x, ...)

# ===========================================================================
# Plot methods
# ===========================================================================

#' @describeIn plot Log-likelihood trace plot for \code{MIRT} objects.
#' @method plot MIRT
#' @export
plot.MIRT <- .fc_plot_method

#' @describeIn plot Log-likelihood trace plot for \code{MGPCM} objects.
#' @method plot MGPCM
#' @export
plot.MGPCM <- .fc_plot_method

#' @describeIn plot Log-likelihood trace plot for \code{MGGUM} objects.
#' @method plot MGGUM
#' @export
plot.MGGUM <- .fc_plot_method

#' @describeIn plot Log-likelihood trace plot for \code{FCMIRT} objects.
#' @method plot FCMIRT
#' @export
plot.FCMIRT <- .fc_plot_method

#' @describeIn plot Log-likelihood trace plot for \code{FCDCM} objects.
#' @method plot FCDCM
#' @export
plot.FCDCM <- .fc_plot_method

#' @describeIn plot Log-likelihood trace plot for \code{FCGGUM} objects.
#' @method plot FCGGUM
#' @export
plot.FCGGUM <- .fc_plot_method

#' @describeIn plot Log-likelihood trace plot for \code{FCGDINA} objects.
#' @method plot FCGDINA
#' @export
plot.FCGDINA <- .fc_plot_method

#' @describeIn plot Log-likelihood trace plot for \code{TIRT} objects.
#' @method plot TIRT
#' @export
plot.TIRT <- .fc_plot_method

#' @title S3 Methods: fitted
#'
#' @description
#' Extract model-implied response probabilities from fitted
#' \pkg{ForceChoice} model objects. Returns the conditional probability of
#' each observed response given each person's estimated latent trait
#' \eqn{\hat{\boldsymbol{\theta}}_j}, evaluated via the model's response
#' function.
#'
#' @param object A fitted model object of class \code{"MIRT"},
#'   \code{"MGPCM"}, \code{"MGGUM"}, \code{"FCMIRT"}, \code{"FCDCM"},
#'   \code{"FCGDINA"}, \code{"FCGGUM"}, or \code{"TIRT"}.
#' @param ... Additional arguments (currently ignored).
#'
#' @return A numeric matrix of model-implied probabilities. Dimensions:
#'   \itemize{
#'     \item Binary models (MIRT, TIRT): \eqn{N \times I}
#'     \item Polytomous models (MGPCM, MGGUM): \eqn{N \times \sum K_i}
#'           (stacked category probabilities)
#'     \item Forced-choice models (FCMIRT, FCGDINA, FCGGUM): \eqn{N \times \sum K_b}
#'           (block-pattern probabilities)
#'     \item FCDCM: \eqn{N \times B} (marginal block-choice probabilities
#'           after integrating over \eqn{2^D} attribute patterns)
#'   }
#'
#' @details
#' For binary-response models, each entry is the endorsement probability
#' \eqn{P(Y_{ij}=1 \mid \hat{\boldsymbol{\theta}}_j)}. For polytomous
#' models, probabilities are returned in stacked format (one column per
#' category). For forced-choice models, each entry is the probability of
#' the observed ranking pattern in the given block.
#'
#' @name fitted
NULL


#' @title S3 Methods: residuals
#'
#' @description
#' Compute residuals from fitted \pkg{ForceChoice} model objects.
#' Supported types are raw (response), Pearson, and deviance residuals.
#'
#' @param object A fitted model object.
#' @param type Character; \code{"raw"} (default), \code{"pearson"}, or
#'   \code{"deviance"}.
#' @param ... Additional arguments (currently ignored).
#'
#' @return A numeric matrix with the same dimensions as the corresponding
#'   \code{fitted} output.
#'
#' @details
#' Residual types follow standard GLM conventions for binary responses:
#' \itemize{
#'   \item \strong{raw}: \eqn{e_{ij} = y_{ij} - \hat{p}_{ij}}
#'   \item \strong{Pearson}:
#'         \eqn{r_{ij} = e_{ij} / \sqrt{\hat{p}_{ij}(1-\hat{p}_{ij})}}
#'   \item \strong{deviance}:
#'         \eqn{d_{ij} = \mathrm{sign}(e_{ij}) \sqrt{2\left[y_{ij}\log\frac{y_{ij}}{\hat{p}_{ij}} + (1-y_{ij})\log\frac{1-y_{ij}}{1-\hat{p}_{ij}}\right]}}
#' }
#' For polytomous and forced-choice models, residuals are computed on a
#' category-indicator matrix. This treats ordered categories and nominal
#' forced-choice response patterns as multinomial outcomes rather than as
#' numeric scores.
#'
#' @name residuals
NULL


# ---- Internal helpers ----

.fc_fitted_binary <- function(object) {
  theta <- as.matrix(object$theta$est)
  par   <- as.matrix(object$par$est)
  # Dispatch to the appropriate model.* function based on class
  cls <- class(object)[1L]
  switch(cls,
    MIRT   = model.MIRT(theta, par),
    TIRT   = {
      D <- ncol(par) - 1L
      lambda <- unlist(lapply(seq_len(nrow(par)), function(x) {
        d <- which(object$Q.matrix[x, ] != 0)[1]
        return(par[x, d])
      }))
      psi2 <- par[, D + 1L]
      model.TIRT(theta, lambda, psi2, object$gamma.matrix$est,
                 object$Q.matrix, object$pairs.matrix)
    },
    stop("fitted() not implemented for class ", cls, call. = FALSE)
  )
}

.fc_fitted_polytomous <- function(object) {
  theta <- as.matrix(object$theta$est)
  par   <- as.matrix(object$par$est)
  cls <- class(object)[1L]
  switch(cls,
    MGPCM = model.MGPCM(theta, par),
    MGGUM = model.MGGUM(theta, par),
    stop("fitted() not implemented for class ", cls, call. = FALSE)
  )
}

.fc_fitted_fc <- function(object) {
  theta <- as.matrix(object$theta$est)
  par   <- as.matrix(object$par$est)
  cls <- class(object)[1L]
  switch(cls,
    FCMIRT = model.FCMIRT(theta, par, object$patterns.total, object$patterns),
    FCGGUM = model.FCGGUM(theta, par, object$patterns.total, object$patterns),
    stop("fitted() not implemented for class ", cls, call. = FALSE)
  )
}

.fc_fitted_fcdcm <- function(object) {
  theta <- as.matrix(object$theta$est)
  # P(alpha_k | theta_j) for all j = 1..N, k = 1..2^D
  alpha_prob <- fcdcm_alpha_profile_prob(
    theta = theta,
    delta1 = object$delta$est[, "delta1"],
    delta0 = object$delta$est[, "delta0"],
    alpha.patterns = object$alpha.patterns
  )
  # Block-choice probabilities for each zeta (condensation) pattern
  prob_per_pattern <- model.FCDCM(
    zeta = object$zeta.patterns,
    par = object$par$est,
    patterns = object$patterns
  )
  # Marginalize over alpha patterns:
  #   P(Y_jb = 1 | theta_j) = sum_k P(Y_b = 1 | zeta_k) * P(alpha_k | theta_j)
  alpha_prob %*% prob_per_pattern
}

.fc_fitted_fcgdina <- function(object) {
  prep <- fcgdina_prepare(
    data = object$response,
    Q.matrix = object$Q.matrix,
    model = object$model,
    block.items = object$block.items,
    fc.type = object$fc.type
  )
  prob.class <- fcgdina_prob_class_link(
    prep, object$delta$est, link = fcgdina_delta_link(object))
  class.prob <- object$class.post
  class.prob %*% prob.class
}

.fc_residuals_raw <- function(fitted, response) {
  response - fitted
}

.fc_residuals_pearson <- function(fitted, response) {
  raw <- response - fitted
  sd  <- sqrt(pmax(fitted * (1 - fitted), .Machine$double.eps))
  raw / sd
}

.fc_residuals_deviance <- function(fitted, response) {
  eps <- .Machine$double.eps
  p <- pmax(pmin(fitted, 1 - eps), eps)
  y <- response
  dev_contrib <- 2 * (y * log(pmax(y, eps) / p) +
                      (1 - y) * log(pmax(1 - y, eps) / (1 - p)))
  sign(y - p) * sqrt(pmax(dev_contrib, 0))
}

.fc_check_residual_dims <- function(fitted, response) {
  if (!identical(dim(fitted), dim(response))) {
    stop(
      "'fitted' and expanded response matrices must have identical dimensions.",
      call. = FALSE
    )
  }
}

.fc_binary_response <- function(response, fitted) {
  response <- as.matrix(response)
  if (anyNA(response) || any(response != floor(response)) ||
      !all(response %in% c(0, 1))) {
    stop("Binary residuals require a response matrix coded 0/1.",
         call. = FALSE)
  }
  storage.mode(response) <- "numeric"
  .fc_check_residual_dims(fitted, response)
  dimnames(response) <- dimnames(fitted)
  response
}

.fc_response_indicators <- function(response, n_cat, first = 0L,
                                    prefix = "Y") {
  response <- as.matrix(response)
  if (anyNA(response) || any(response != floor(response))) {
    stop("Expanded residuals require finite integer response categories.",
         call. = FALSE)
  }
  storage.mode(response) <- "integer"

  N <- nrow(response)
  I <- ncol(response)
  n_cat <- as.integer(n_cat)
  if (length(n_cat) != I || anyNA(n_cat) || any(n_cat < 2L)) {
    stop("'n_cat' must contain one category count >= 2 per response column.",
         call. = FALSE)
  }

  first <- as.integer(first)
  if (length(first) != 1L || is.na(first)) {
    stop("'first' must be a scalar integer category origin.", call. = FALSE)
  }

  out <- matrix(0, N, sum(n_cat))
  rownames(out) <- rownames(response)
  col.names <- character(sum(n_cat))
  base.names <- colnames(response)
  if (is.null(base.names)) base.names <- paste0(prefix, seq_len(I))

  idx <- 0L
  for (i in seq_len(I)) {
    yi <- response[, i]
    last <- first + n_cat[i] - 1L
    if (any(yi < first | yi > last)) {
      stop(
        "Response column ", i, " contains categories outside ",
        first, ":", last, ".",
        call. = FALSE
      )
    }
    out[cbind(seq_len(N), idx + yi - first + 1L)] <- 1
    col.names[idx + seq_len(n_cat[i])] <-
      paste0(base.names[i], ".", first:last)
    idx <- idx + n_cat[i]
  }

  colnames(out) <- col.names
  out
}

.fc_residuals <- function(fitted, response, type) {
  .fc_check_residual_dims(fitted, response)
  switch(type,
    raw      = .fc_residuals_raw(fitted, response),
    pearson  = .fc_residuals_pearson(fitted, response),
    deviance = .fc_residuals_deviance(fitted, response)
  )
}


# ===========================================================================
# MIRT
# ===========================================================================

#' @describeIn fitted Fitted probabilities for \code{MIRT} objects.
#'   Returns \eqn{N \times I} matrix of endorsement probabilities.
#' @method fitted MIRT
#' @export
fitted.MIRT <- function(object, ...) {
  .fc_fitted_binary(object)
}

#' @describeIn residuals Residuals for \code{MIRT} objects.
#' @method residuals MIRT
#' @export
residuals.MIRT <- function(object, type = c("raw", "pearson", "deviance"), ...) {
  type <- match.arg(type)
  fit <- fitted(object)
  resp <- .fc_binary_response(object$arguments$response, fit)
  .fc_residuals(fit, resp, type)
}

# ===========================================================================
# MGPCM
# ===========================================================================

#' @describeIn fitted Fitted probabilities for \code{MGPCM} objects.
#'   Returns stacked category probability matrix.
#' @method fitted MGPCM
#' @export
fitted.MGPCM <- function(object, ...) {
  .fc_fitted_polytomous(object)
}

#' @describeIn residuals Residuals for \code{MGPCM} objects.
#' @method residuals MGPCM
#' @export
residuals.MGPCM <- function(object, type = c("raw", "pearson", "deviance"), ...) {
  type <- match.arg(type)
  fit <- fitted(object)
  resp <- .fc_response_indicators(
    object$arguments$response, object$length.poly, first = 0L, prefix = "item"
  )
  dimnames(resp) <- dimnames(fit)
  .fc_residuals(fit, resp, type)
}

# ===========================================================================
# MGGUM
# ===========================================================================

#' @describeIn fitted Fitted probabilities for \code{MGGUM} objects.
#' @method fitted MGGUM
#' @export
fitted.MGGUM <- function(object, ...) {
  .fc_fitted_polytomous(object)
}

#' @describeIn residuals Residuals for \code{MGGUM} objects.
#' @method residuals MGGUM
#' @export
residuals.MGGUM <- function(object, type = c("raw", "pearson", "deviance"), ...) {
  type <- match.arg(type)
  fit <- fitted(object)
  resp <- .fc_response_indicators(
    object$arguments$response, object$length.poly, first = 0L, prefix = "item"
  )
  dimnames(resp) <- dimnames(fit)
  .fc_residuals(fit, resp, type)
}

# ===========================================================================
# FCMIRT
# ===========================================================================

#' @describeIn fitted Fitted probabilities for \code{FCMIRT} objects.
#'   Returns block-pattern probability matrix.
#' @method fitted FCMIRT
#' @export
fitted.FCMIRT <- function(object, ...) {
  .fc_fitted_fc(object)
}

#' @describeIn residuals Residuals for \code{FCMIRT} objects.
#' @method residuals FCMIRT
#' @export
residuals.FCMIRT <- function(object, type = c("raw", "pearson", "deviance"), ...) {
  type <- match.arg(type)
  fit <- fitted(object)
  resp <- .fc_response_indicators(
    object$response,
    vapply(object$patterns, nrow, integer(1L)),
    first = 1L,
    prefix = "block"
  )
  dimnames(resp) <- dimnames(fit)
  .fc_residuals(fit, resp, type)
}

# ===========================================================================
# FCDCM
# ===========================================================================

#' @describeIn fitted Fitted probabilities for \code{FCDCM} objects.
#'   Returns \eqn{N \times B} matrix of marginal block-choice probabilities
#'   after integrating over \eqn{2^D} attribute patterns.
#' @method fitted FCDCM
#' @export
fitted.FCDCM <- function(object, ...) {
  .fc_fitted_fcdcm(object)
}

#' @describeIn residuals Residuals for \code{FCDCM} objects.
#' @method residuals FCDCM
#' @export
residuals.FCDCM <- function(object, type = c("raw", "pearson", "deviance"), ...) {
  type <- match.arg(type)
  fit <- fitted(object)
  resp <- .fc_binary_response(object$response, fit)
  .fc_residuals(fit, resp, type)
}

# ===========================================================================
# FCGDINA
# ===========================================================================

#' @describeIn fitted Fitted probabilities for \code{FCGDINA} objects.
#'   Returns block-pattern probability matrix after integrating over posterior
#'   attribute classes.
#' @method fitted FCGDINA
#' @export
fitted.FCGDINA <- function(object, ...) {
  .fc_fitted_fcgdina(object)
}

#' @describeIn residuals Residuals for \code{FCGDINA} objects.
#' @method residuals FCGDINA
#' @export
residuals.FCGDINA <- function(object, type = c("raw", "pearson", "deviance"), ...) {
  type <- match.arg(type)
  fit <- fitted(object)
  resp <- .fc_response_indicators(
    object$response,
    vapply(object$patterns, nrow, integer(1L)),
    first = 1L,
    prefix = "block"
  )
  dimnames(resp) <- dimnames(fit)
  .fc_residuals(fit, resp, type)
}

# ===========================================================================
# FCGGUM
# ===========================================================================

#' @describeIn fitted Fitted probabilities for \code{FCGGUM} objects.
#' @method fitted FCGGUM
#' @export
fitted.FCGGUM <- function(object, ...) {
  .fc_fitted_fc(object)
}

#' @describeIn residuals Residuals for \code{FCGGUM} objects.
#' @method residuals FCGGUM
#' @export
residuals.FCGGUM <- function(object, type = c("raw", "pearson", "deviance"), ...) {
  type <- match.arg(type)
  fit <- fitted(object)
  resp <- .fc_response_indicators(
    object$response,
    vapply(object$patterns, nrow, integer(1L)),
    first = 1L,
    prefix = "block"
  )
  dimnames(resp) <- dimnames(fit)
  .fc_residuals(fit, resp, type)
}

# ===========================================================================
# TIRT
# ===========================================================================

#' @describeIn fitted Fitted probabilities for \code{TIRT} objects.
#'   Returns \eqn{N \times I_{pairs}} matrix of pairwise comparison
#'   probabilities.
#' @method fitted TIRT
#' @export
fitted.TIRT <- function(object, ...) {
  .fc_fitted_binary(object)
}

#' @describeIn residuals Residuals for \code{TIRT} objects.
#' @method residuals TIRT
#' @export
residuals.TIRT <- function(object, type = c("raw", "pearson", "deviance"), ...) {
  type <- match.arg(type)
  fit <- fitted(object)
  resp <- .fc_binary_response(object$response, fit)
  .fc_residuals(fit, resp, type)
}

#' Fit FCMIRT Model via Stan
#'
#' Fits the forced-choice MIRT model using Bayesian MCMC via Stan. Item-level
#' endorsement probabilities are defined by an MIRT model
#' (\code{"m1pl"}, \code{"m2pl"}, \code{"m3pl"}, or \code{"m4pl"}) and block
#' responses are modeled through the same RANK, MOLE, and PICK mechanisms used
#' by \code{\link[ForceChoice]{sim.data.FCMIRT}}.
#' The block difficulty/intercept parameters \code{b} are identified by a
#' zero-sum constraint within each forced-choice block; Stan samples the
#' \code{K - 1} free difficulties and reconstructs the final one.
#'
#' @param data An N x B data frame where each column is a forced-choice block
#'   and each cell is a ranking string such as \code{"3>1>2"}, or an N x B
#'   integer matrix of 1-based pattern indices.
#' @param model MIRT model type: \code{"m1pl"}, \code{"m2pl"}, \code{"m3pl"},
#'   or \code{"m4pl"}.
#' @param Q.matrix An I x D 0/1 matrix indicating active item loadings. For
#'   \code{"m1pl"}, active slopes are fixed to 1 and inactive slopes to 0.
#' @param block.items A list of length B giving global item indices in each
#'   block. Required for integer pattern-index data and for MOLE/PICK data.
#' @param D Number of latent dimensions. If \code{Q.matrix} is supplied,
#'   \code{D} is reset to \code{ncol(Q.matrix)}.
#' @param fc.type Forced-choice response type: \code{"RANK"}, \code{"MOLE"}, or
#'   \code{"PICK"}. A scalar value is recycled to all blocks.
#' @param control.model Optional named list of model-level controls and
#'   hyperparameters. Supported prior entries include \code{a.mu},
#'   \code{a.sigma}, \code{b.mu}, \code{b.sigma}, \code{c.mu},
#'   \code{c.sigma}, \code{d.mu}, \code{d.sigma}, and \code{theta.mu}. The
#'   Stan default for \code{a.sigma} is deliberately weak because
#'   forced-choice blocks identify relative utilities and tight common slope
#'   priors can over-shrink item slopes within the same block.
#' @param control.method Optional named list of method-specific controls. For
#'   Stan this includes \code{cores}, \code{vis}, \code{seed}, \code{chains},
#'   \code{iter}, \code{warmup}, \code{thin}, \code{init}, \code{algorithm},
#'   and Stan sampler controls.
#'
#' @noRd
fit.FCMIRT.stan <- function(data, model = "m2pl", Q.matrix = NULL, block.items = NULL,
                       D = NULL, fc.type = "RANK",
                       control.model = NULL,
                       control.method = NULL,
                       .call = NULL) {
  call <- if (is.null(.call)) match.call() else .call
  control <- fc_as_control_list(control.model, "control.model")
  control.method <- fc_as_control_list(control.method, "control.method")
  common.method <- fit_common_method_control(control.method)
  stan.method <- fc_stan_method_control(control.method)

  model <- tolower(model)
  model_type <- switch(model,
    "m1pl" = 1L,
    "m2pl" = 2L,
    "m3pl" = 3L,
    "m4pl" = 4L,
    stop("Unknown FCMIRT model type: ", model, call. = FALSE)
  )
  chains <- stan.method$chains
  iter <- stan.method$iter
  warmup <- stan.method$warmup
  thin <- stan.method$thin
  init <- stan.method$init
  algorithm <- stan.method$algorithm

  sm <- stanmodels[["FCMIRT"]]
  if (is.null(sm)) {
    stop("Stan model FCMIRT not found !", call. = FALSE)
  }

  mcmc_env <- setup_mcmc_env(common.method$cores)
  on.exit(restore_mcmc_env(mcmc_env), add = TRUE)

  stan_control <- build_stan_control(algorithm, control.method)

  fc.type.valid <- c("RANK", "MOLE", "PICK")
  fc.type <- toupper(fc.type)
  if (!all(fc.type %in% fc.type.valid)) {
    stop("'fc.type' must be one of: ", paste(fc.type.valid, collapse = ", "), call. = FALSE)
  }

  data.matrix <- as.matrix(data)
  N.block.data <- ncol(data.matrix)
  if (is.null(N.block.data) || N.block.data < 1L) {
    stop("'data' must have at least one column/block.", call. = FALSE)
  }
  if (length(fc.type) != N.block.data) {
    fc.type <- rep(fc.type[1L], N.block.data)
  }

  is.pattern.index.data <- is.numeric(data.matrix) || is.integer(data.matrix)

  if (is.null(block.items)) {
    if (is.pattern.index.data) {
      stop("'block.items' is required when 'data' is an integer matrix of pattern indices.", call. = FALSE)
    }
    if (any(fc.type != "RANK")) {
      stop("'block.items' is required for MOLE/PICK data because partial rankings do not identify all block items.", call. = FALSE)
    }
    block.items <- get.block.items.from.data(data)
  }

  if (is.pattern.index.data) {
    response <- matrix(as.integer(data.matrix),
                       nrow = nrow(data.matrix), ncol = ncol(data.matrix),
                       dimnames = dimnames(data.matrix))
  } else {
    response <- get.response.from.data(data, block.items, fc.type = fc.type)
  }

  if (!is.list(block.items)) {
    stop("'block.items' must be a list.", call. = FALSE)
  }
  N.block <- length(block.items)

  all_items <- unlist(block.items, use.names = FALSE)
  if (anyDuplicated(all_items)) {
    stop("'block.items' contains duplicate items.", call. = FALSE)
  }
  I <- length(all_items)
  if (!setequal(all_items, seq_len(I))) {
    stop("'block.items' must contain exactly the item indices 1 through ", I, ".", call. = FALSE)
  }
  if (any(vapply(block.items, length, integer(1L)) < 2L)) {
    stop("Each block in 'block.items' must contain at least two items.", call. = FALSE)
  }

  response <- as.matrix(response)
  N <- nrow(response)
  if (ncol(response) != N.block) {
    stop("'response' must have ", N.block, " columns (one per block).", call. = FALSE)
  }

  if (length(fc.type) != N.block) {
    fc.type <- rep(fc.type[1L], N.block)
  }
  fc.type.int <- match(fc.type, fc.type.valid)

  if (!is.null(Q.matrix)) {
    Q.matrix <- as.matrix(Q.matrix)
    if (anyNA(Q.matrix) || !all(Q.matrix %in% c(0, 1))) {
      stop("'Q.matrix' must be a matrix containing only 0 or 1.", call. = FALSE)
    }
    if (nrow(Q.matrix) != I) {
      stop("'Q.matrix' must have ", I, " rows, one for each item.", call. = FALSE)
    }
    if (any(rowSums(Q.matrix) < 1L)) {
      stop("Each item must measure at least one trait (each row of 'Q.matrix' needs a 1).", call. = FALSE)
    }
    D.q <- ncol(Q.matrix)
    if (!is.null(D) && D != D.q) {
      warning("'D' (", D, ") does not match ncol(Q.matrix) (", D.q, "); using ncol(Q.matrix).")
    }
    D <- D.q
    storage.mode(Q.matrix) <- "numeric"
  }

  if (is.null(D) || is.na(D)) {
    stop("Cannot determine 'D'. Please provide either 'Q.matrix' or 'D'.", call. = FALSE)
  }
  D <- as.integer(D)
  if (D < 1L) {
    stop("'D' must be a positive integer.", call. = FALSE)
  }

  if (is.null(Q.matrix)) {
    Q.matrix <- matrix(1, I, D)
  }

  corr_df <- free_corr_npar(D)

  patterns.total <- vector("list", N.block)
  patterns.obs <- vector("list", N.block)
  for (b in seq_len(N.block)) {
    patterns.total[[b]] <- get_permutations(block.items[[b]],
                                            length(block.items[[b]]))
    patterns.obs[[b]] <- get_permutations(
      block.items[[b]],
      switch(fc.type[b],
             RANK = length(block.items[[b]]),
             MOLE = 2L,
             PICK = 1L)
    )
  }

  for (b in seq_len(N.block)) {
    if (anyNA(response[, b]) ||
        any(response[, b] < 1L | response[, b] > nrow(patterns.obs[[b]]))) {
      stop("'data' contains invalid pattern indices for block ", b,
           "; valid values are 1 through ", nrow(patterns.obs[[b]]), ".", call. = FALSE)
    }
  }

  n_items <- integer(N.block)
  n_total <- integer(N.block)
  n_obs <- integer(N.block)
  fc_len <- integer(N.block)

  block_items_flat <- integer(0)
  patterns_total_flat <- integer(0)
  patterns_obs_flat <- integer(0)

  item_start <- integer(N.block + 1L)
  total_start <- integer(N.block + 1L)
  obs_start <- integer(N.block + 1L)
  item_start[1L] <- 1L
  total_start[1L] <- 1L
  obs_start[1L] <- 1L

  for (b in seq_len(N.block)) {
    items.b <- block.items[[b]]
    K <- length(items.b)
    n_items[b] <- K
    fc_len[b] <- switch(fc.type[b], RANK = K, MOLE = 2L, PICK = 1L)

    pat.total <- patterns.total[[b]]
    n_total[b] <- nrow(pat.total)
    pat.total.local <- matrix(match(pat.total, items.b), nrow = n_total[b])
    patterns_total_flat <- c(patterns_total_flat, as.integer(t(pat.total.local)))

    pat.obs <- patterns.obs[[b]]
    n_obs[b] <- nrow(pat.obs)
    pat.obs.local <- matrix(match(pat.obs, items.b), nrow = n_obs[b])
    patterns_obs_flat <- c(patterns_obs_flat, as.integer(t(pat.obs.local)))

    block_items_flat <- c(block_items_flat, items.b)

    item_start[b + 1L] <- item_start[b] + K
    total_start[b + 1L] <- total_start[b] + n_total[b] * K
    obs_start[b + 1L] <- obs_start[b] + n_obs[b] * fc_len[b]
  }

  N_obs <- N * N.block
  pid_long <- rep(seq_len(N), times = N.block)
  bid_long <- rep(seq_len(N.block), each = N)
  y_long <- as.integer(response)

  theta_mu_raw <- get_ctrl("theta.mu", rep(0, D), control)
  theta_mu <- array(theta_mu_raw, dim = D)

  stan.data <- list(
    N_obs = N_obs,
    N = N,
    B = N.block,
    I = I,
    D = D,
    model_type = model_type,
    pid = pid_long,
    bid = bid_long,
    y = y_long,
    Q_matrix = Q.matrix,
    n_items = n_items,
    fc_type = fc.type.int,
    total_block_items = length(block_items_flat),
    block_items = block_items_flat,
    item_start = item_start,
    n_total = n_total,
    total_cells_total = length(patterns_total_flat),
    patterns_total = patterns_total_flat,
    total_start = total_start,
    n_obs = n_obs,
    total_cells_obs = length(patterns_obs_flat),
    patterns_obs = patterns_obs_flat,
    obs_start = obs_start,
    fc_len = fc_len,
    a_mu = get_ctrl("a.mu", 0.0, control),
    a_sigma = get_ctrl("a.sigma", 1.0, control),
    b_mu = get_ctrl("b.mu", 0.0, control),
    b_sigma = get_ctrl("b.sigma", 1.0, control),
    c_mu = get_ctrl("c.mu", 0.0, control),
    c_sigma = get_ctrl("c.sigma", 0.35, control),
    d_mu = get_ctrl("d.mu", 0.65, control),
    d_sigma = get_ctrl("d.sigma", 1.0, control),
    theta_mu = theta_mu
  )

  set.seed(common.method$seed)

  stan.obj <- suppressWarnings(
    rstan::sampling(
      object = sm,
      data = stan.data,
      chains = chains,
      iter = iter,
      warmup = warmup,
      thin = thin,
      init = init,
      algorithm = algorithm,
      cores = mcmc_env$cores,
      control = stan_control,
      verbose = common.method$vis,
      show_messages = common.method$vis
    )
  )

  MCMC.obj <- rstan::extract(stan.obj, permuted = TRUE)
  stan.sum <- rstan::summary(stan.obj)$summary

  theta_est <- extract_theta_stan(MCMC.obj, stan.sum, N, D, rownames(response))

  par <- matrix(NA_real_, nrow = I, ncol = D + 3L)
  par.se <- matrix(NA_real_, nrow = I, ncol = D + 3L)
  par.Rhat <- matrix(NA_real_, nrow = I, ncol = D + 3L)
  par.free <- fcmirt_par_free_mask(model_type, Q.matrix, block.items)

  if (model == "m1pl") {
    par[, seq_len(D)] <- Q.matrix
    par.se[, seq_len(D)] <- NA_real_
    par.Rhat[, seq_len(D)] <- NA_real_
  } else {
    a_arr <- ensure_3d(MCMC.obj$a)
    a <- apply(a_arr, c(2, 3), mean)
    a.se <- apply(a_arr, c(2, 3), sd)
    par[, seq_len(D)] <- a
    par.se[, seq_len(D)] <- a.se
    par.Rhat[, seq_len(D)] <- extract_rhat_matrix(stan.sum, "a", I, D)
  }

  b_est <- apply(MCMC.obj$b, 2, mean)
  par[, D + 1L] <- b_est
  b_free_items <- unlist(lapply(block.items, function(items) {
    items[-length(items)]
  }), use.names = FALSE)
  b_free_draws <- as.matrix(MCMC.obj$b_free)
  par.se[b_free_items, D + 1L] <- apply(b_free_draws, 2L, sd)
  par.Rhat[b_free_items, D + 1L] <-
    extract_rhat_vector(stan.sum, "b_free", length(b_free_items))

  if (model %in% c("m3pl", "m4pl")) {
    c_est <- apply(MCMC.obj$c, 2, mean)
    c.se <- apply(MCMC.obj$c, 2, sd)
    c.idx <- grep("^c\\[", rownames(stan.sum))
    par[, D + 2L] <- c_est
    par.se[, D + 2L] <- c.se
    par.Rhat[, D + 2L] <- stan.sum[c.idx, "Rhat"]
  } else {
    par[, D + 2L] <- 0
    par.se[, D + 2L] <- NA_real_
    par.Rhat[, D + 2L] <- NA_real_
  }

  if (model == "m4pl") {
    d_est <- apply(MCMC.obj$d, 2, mean)
    d.se <- apply(MCMC.obj$d, 2, sd)
    d.idx <- grep("^d\\[", rownames(stan.sum))
    par[, D + 3L] <- d_est
    par.se[, D + 3L] <- d.se
    par.Rhat[, D + 3L] <- stan.sum[d.idx, "Rhat"]
  } else {
    par[, D + 3L] <- 1
    par.se[, D + 3L] <- NA_real_
    par.Rhat[, D + 3L] <- NA_real_
  }

  item_names <- paste0("item", all_items)
  colnames(par) <- colnames(par.se) <- colnames(par.Rhat) <- colnames(par.free) <-
    c(paste0("a", seq_len(D)), "b", "c", "d")
  rownames(par) <- rownames(par.se) <- rownames(par.Rhat) <- rownames(par.free) <-
    item_names
  par.se[!par.free] <- NA_real_
  par.Rhat[!par.free] <- NA_real_
  npar <- sum(par.free) + corr_df

  Corr_est <- extract_corr_stan(MCMC.obj, stan.sum, D)

  results <- list(
    npar = npar,
    method = "stan",
    theta = theta_est,
    par = list(est = par, se = par.se, Rhat = par.Rhat, free = par.free),
    Corr = Corr_est,
    stan.obj = stan.obj,
    MCMC.obj = MCMC.obj,
    log_lik = MCMC.obj$log_lik,
    model = model,
    Q.matrix = Q.matrix,
    block.items = block.items,
    fc.type = fc.type,
    response = response,
    patterns = patterns.obs,
    patterns.total = patterns.total,
    call = call,
    arguments = list(
      data = data,
      model = model,
      Q.matrix = Q.matrix,
      method = "stan",
      block.items = block.items,
      D = D,
      fc.type = fc.type,
      chains = chains,
      iter = iter,
      warmup = warmup,
      thin = thin,
      init = init,
      algorithm = algorithm,
      cores = common.method$cores,
      vis = common.method$vis,
      seed = common.method$seed,
      control.model = control,
      control.method = fit_effective_method_control(
        control.method, method = stan.method, common = common.method
      )
    )
  )

  class(results) <- "FCMIRT"

  L <- get_ctrl("L", NULL, control)
  results$logLik <- logLik.FCMIRT(results, L = L)

  return(results)
}

#' @describeIn get.fit.index FCMIRT model: forced-choice ranking data expanded
#'   to nominal binary indicators; within-block pairs excluded from bivariate
#'   moments.
#' @export
get.fit.index.FCMIRT <- function(object, ...) {
  model <- object$arguments$model
  response <- object$response
  par <- object$par$est
  D <- object$arguments$D
  Q.matrix <- object$Q.matrix
  block.items <- object$block.items
  patterns <- object$patterns
  patterns.total <- object$patterns.total

  logLik.obj <- object$logLik
  pi <- as.vector(attr(logLik.obj, "pi"))
  theta.norm <- attr(logLik.obj, "theta.norm")

  N <- nrow(response)
  B <- length(block.items)
  I <- length(unlist(block.items, use.names = FALSE))
  npar <- object$npar

  n_cat <- vapply(patterns, nrow, integer(1L))
  n_bin <- sum(n_cat - 1L)
  response.bin <- matrix(0L, N, n_bin)
  indicator.block <- integer(n_bin)

  idx.resp <- 0L
  for (b in seq_len(B)) {
    Kb <- n_cat[b]
    if (Kb > 1L) {
      for (j in seq_len(Kb - 1L)) {
        response.bin[, idx.resp + j] <- as.integer(response[, b] == j)
        indicator.block[idx.resp + j] <- b
      }
      idx.resp <- idx.resp + (Kb - 1L)
    }
  }

  extract.free.par <- function(par, ...) {
    par.vec <- numeric(0)
    if (model != "m1pl") {
      for (i in seq_len(I)) {
        for (j in seq_len(D)) {
          if (Q.matrix[i, j] == 1) par.vec <- c(par.vec, par[i, j])
        }
      }
    }
    for (items in block.items) {
      par.vec <- c(par.vec, par[items[-length(items)], D + 1L])
    }
    if (model %in% c("m3pl", "m4pl")) par.vec <- c(par.vec, par[, D + 2L])
    if (model == "m4pl") par.vec <- c(par.vec, par[, D + 3L])
    return(par.vec)
  }

  expand.free.par <- function(par.vec, ...) {
    par <- matrix(0, nrow = I, ncol = D + 3L)
    idx <- 1L
    if (model == "m1pl") {
      par[, seq_len(D)] <- Q.matrix
    } else {
      for (i in seq_len(I)) {
        for (j in seq_len(D)) {
          if (Q.matrix[i, j] == 1) {
            par[i, j] <- par.vec[idx]
            idx <- idx + 1L
          } else {
            par[i, j] <- 0.0
          }
        }
      }
    }

    for (items in block.items) {
      K <- length(items)
      b_free <- par.vec[idx:(idx + K - 2L)]
      par[items[-K], D + 1L] <- b_free
      par[items[K], D + 1L] <- -sum(b_free)
      idx <- idx + K - 1L
    }

    if (model %in% c("m1pl", "m2pl")) {
      par[, D + 2L] <- 0
    } else {
      for (i in seq_len(I)) {
        par[i, D + 2L] <- par.vec[idx]
        idx <- idx + 1L
      }
    }

    if (model %in% c("m1pl", "m2pl", "m3pl")) {
      par[, D + 3L] <- 1
    } else {
      for (i in seq_len(I)) {
        par[i, D + 3L] <- par.vec[idx]
        idx <- idx + 1L
      }
    }
    return(par)
  }

  par.vec <- extract.free.par(par)

  response.full <- matrix(0, N, sum(n_cat))
  idx.full <- 0L
  for (b in seq_len(B)) {
    Kb <- n_cat[b]
    for (p in seq_len(N)) {
      response.full[p, idx.full + response[p, b]] <- 1
    }
    idx.full <- idx.full + Kb
  }

  loglik.fun <- function(par.vec, ...) {
    par <- expand.free.par(par.vec, ...)
    prob_new <- model.FCMIRT(theta.norm, par, patterns.total, patterns)

    log_prob <- log(pmax(prob_new, .Machine$double.xmin))
    log_pi <- log(pi)

    pi.length <- nrow(theta.norm)
    log_L_theta_Xi <- matrix(0, N, pi.length)
    for (p in seq_len(N)) {
      response.p <- matrix(response.full[p, ], pi.length, ncol(prob_new), byrow = TRUE)
      log_L_theta_Xi[p, ] <- rowSums(response.p * log_prob) + log_pi
    }

    sum(apply(log_L_theta_Xi, 1, function(x) {
      m <- max(x)
      m + log(sum(exp(x - m)))
    }))
  }

  prob.fun <- function(par.vec, ...) {
    par <- expand.free.par(par.vec, ...)
    prob_full <- model.FCMIRT(theta.norm, par, patterns.total, patterns)
    prob_out <- matrix(0, nrow(prob_full), n_bin)

    idx.full <- 0L
    idx.bin <- 0L
    for (b in seq_len(B)) {
      Kb <- n_cat[b]
      if (Kb > 1L) {
        prob_out[, (idx.bin + 1L):(idx.bin + Kb - 1L)] <-
          prob_full[, (idx.full + 1L):(idx.full + Kb - 1L)]
        idx.bin <- idx.bin + (Kb - 1L)
      }
      idx.full <- idx.full + Kb
    }
    return(prob_out)
  }

  good.of.fit(
    par.vec = par.vec,
    loglik.fun = loglik.fun,
    prob.fun = prob.fun,
    response = response.bin,
    npar = npar,
    pi = pi,
    response.type = "nominal",
    nominal.groups = indicator.block,
    ...
  )
}

# library(ForceChoice)
#
# ############################### data ###############################
# N.person <- 300
# N.block <- 10
# I.block <- 3
# D <- 3
# fc.type <- "MOLE"
# control <- NULL
#
# seed <- 459821
#
# set.seed(seed)
# data.obj <- sim.data.FCGGUM(N.person = N.person, N.block = N.block,
#                             I.block = I.block, D = D, fc.type = fc.type,
#                             control = control)
# data.obj$par
# response <- data.obj$response
# block.items <- data.obj$block.items
# Q.matrix <- data.obj$Q.matrix
# fc.type <- data.obj$fc.type
# data <- get.data.from.response(response, block.items, fc.type = fc.type)
# head(data)
# head(response)
# all(data == data.obj$data)
# response.obj <- get.response.from.data(data, block.items, fc.type = fc.type)
# stopifnot(all(response == response.obj))
#
# # ---- Verify unified block-item extraction ----
# block.items.from.data <- get.block.items.from.data(data)
# stopifnot(length(block.items.from.data) == N.block)
# stopifnot(all(mapply(setequal, block.items, block.items.from.data)))
#
# # ---- Verify normalize_fc_type ----
# stopifnot(identical(
#   normalize_fc_type(NULL, 3L),
#   c("RANK", "RANK", "RANK")))
# stopifnot(identical(
#   normalize_fc_type(c("mole", "rank", "pick"), 3L),
#   c("MOLE", "RANK", "PICK")))
#
# # ############################### Stan MCMC ###############################
# # object.FCGGUM.stan <- fit.FCGGUM(data, Q.matrix = Q.matrix,
# #                                  block.items = block.items, D = D,
# #                                  fc.type = fc.type,
# #                                  method = "stan",
# #                                  control.method = c(list(
# #                                    init = "random",
# #                                    algorithm = "HMC"), list(seed = seed, vis = TRUE)))
# # cbind(data.obj$par, object.FCGGUM.stan$par$est)
# # object.FCGGUM.stan$par$Rhat
# # get.fit.index(object.FCGGUM.stan)
#
# ############################### iStEM ###############################
# iStEM.control.method <- list(
#   burnin.maxitr = 40,
#   maxitr = 500,
#   theta.proposal.sd = 0.35,
#   optim.maxit = 50
# )
#
# object.FCGGUM.iStEM <- fit.FCGGUM(data, Q.matrix = Q.matrix,
#                                   block.items = block.items, D = D,
#                                   fc.type = fc.type,
#                                   method = "iStEM",
#                                   control.method = c(iStEM.control.method, list(seed = seed, vis = TRUE)))
# cbind(data.obj$par, object.FCGGUM.iStEM$par$est)
#
# # ---- Validate logLik with new quadrature helpers ----
# ll <- logLik(object.FCGGUM.iStEM)
# stopifnot(inherits(ll, "logLik"))
# stopifnot(is.finite(as.numeric(ll)))
# stopifnot(!is.null(attr(ll, "theta.norm")))
# stopifnot(!is.null(attr(ll, "prob")))
# stopifnot(!is.null(attr(ll, "pi")))
# cat(sprintf("  FCGGUM logLik: %.2f (df=%d)\n", as.numeric(ll), attr(ll, "df")))
#
# get.fit.index(object.FCGGUM.iStEM)

#' Simulate Data from the Multidimensional Generalized Partial Credit Model
#'
#' @description
#' Generates polytomous (multi-category) response data from the MGPCM.
#' Category probabilities use the same softmax formulation as
#' \code{\link{model.MGPCM}}: \eqn{\log P(Y=k)} is proportional to
#' \eqn{k\,\mathbf{a}_i'\boldsymbol{\theta}_j + d_{ik}}.
#' The requested category counts are retained even when a finite sample does
#' not contain every possible category.
#'
#' @section Data Generation Process:
#' \enumerate{
#'   \item \strong{Latent traits:}
#'         \eqn{\boldsymbol{\theta}_j \sim N_D(\mathbf{0}, \boldsymbol{\Sigma})}.
#'   \item \strong{Item parameters:}
#'         \eqn{a_{id} \sim \text{Lognormal}(0.25, 0.25)} (active dimensions),
#'         \eqn{d_{i0}=0}, and
#'         \eqn{d_{i1},\dots,d_{i,K_i-1}} are sorted draws from
#'         \eqn{N(0,1)} used as category intercepts.
#'   \item \strong{Responses:}
#'         \eqn{Y_{ij} \sim \text{Categorical}(P(Y_{ij} = k \mid \boldsymbol{\theta}_j))}
#'         for \eqn{k = 0, \dots, K_i - 1}.
#' }
#'
#' @param N Integer; number of examinees (default: 500).
#' @param I Integer; number of items (default: 20).
#' @param D Integer; number of latent dimensions (default: 2).
#' @param length.poly Integer vector or scalar; number of categories per
#'   item. Recycled to length \eqn{I} if scalar (default: 5).
#' @param Q.matrix Optional \eqn{I \times D} 0/1 matrix. \code{NULL} uses
#'   a triangular identification pattern.
#' @param Corr Optional \eqn{D \times D} correlation matrix.
#' @param rotate Optional rotation method (\pkg{GPArotation} or
#'   \code{"promax"}).
#' @param promax_m Power parameter for Promax (default: 4).
#'
#' @return An object of class \code{"data.MGPCM"}, a list containing:
#' \describe{
#'   \item{\code{data}, \code{response}}{\eqn{N \times I} integer response
#'         matrices with categories coded from 0 to \eqn{K_i - 1}.}
#'   \item{\code{theta}}{\eqn{N \times D} matrix of true latent traits.}
#'   \item{\code{par}}{\eqn{I \times (D + \max_i K_i)} matrix of item
#'         parameters; discrimination columns are followed by category
#'         intercept columns.}
#'   \item{\code{probability}}{\eqn{N \times \sum_i K_i} matrix of stacked
#'         category probabilities.}
#'   \item{\code{Q.matrix}, \code{length.poly}, \code{Corr}}{Design matrix,
#'         category counts, and latent correlation matrix used to generate
#'         the data.}
#'   \item{\code{model}, \code{N}, \code{I}, \code{D}, \code{call}}{
#'         Data-generating metadata.}
#' }
#'
#' @seealso \code{\link{fit.MGPCM}}, \code{\link{model.MGPCM}}
#'
#' @examples
#' set.seed(123)
#' sim <- sim.data.MGPCM(N = 20, I = 5, D = 2, length.poly = 4)
#' str(sim$response)
#' dim(sim$probability)
#' sim$length.poly
#'
#' @export
sim.data.MGPCM <- function(N = 500, I = 20, D = 2, length.poly = 5,
                           Q.matrix = NULL, Corr = NULL, rotate = NULL,
                           promax_m = 4) {
  call <- match.call()

  N <- check_integer_scalar(N, "N")
  I <- check_integer_scalar(I, "I")
  D <- check_integer_scalar(D, "D")
  length.poly <- normalize_length_poly(length.poly, I)
  max_poly <- max(length.poly)

  Corr <- validate_corr_matrix(Corr, D)

  Q.matrix <- istem_prepare_01_q(I, D, Q.matrix, triangular = TRUE)

    theta <- generate_theta_mvn(N, D, Corr)

    a <- matrix(rlnorm(I * D, meanlog = 0.25, sdlog = 0.25), I, D) * Q.matrix

    if (!is.null(rotate) && D > 1) {
      rot <- apply_rotation_promax(a, theta, Corr, rotate, D, promax_m)
      a     <- rot$a
      theta <- rot$theta
      Corr  <- rot$Corr
    }

    par <- matrix(NA, nrow = I, ncol = D + max_poly)
    par[, 1:D] <- a
    mask <- matrix(NA, nrow = I, ncol = D + max_poly)
    mask[, 1:D] <- 1
    for (i in seq_len(I)) {

      par[i, D + 1] <- 0.0
      mask[i, D + 1] <- 1
      if (length.poly[i] > 1) {
        par[i, (D + 2):(D + length.poly[i])] <- sort(rnorm(length.poly[i] - 1, 0, 1))
        mask[i, (D + 2):(D + length.poly[i])] <- 1
      }
    }

    prob <- model.MGPCM(theta = theta, par = par)
    response <- matrix(0, N, I)
    idx <- 0
    for (i in seq_len(I)) {
      P.i <- prob[, (idx + 1):(idx + length.poly[i]), drop = FALSE]
      idx <- idx + length.poly[i]
      cdf <- t(apply(P.i, 1L, cumsum))
      response[, i] <- pmin(
        rowSums(cdf < stats::runif(N)), length.poly[i] - 1L
      )
    }

  if (D > 1) {
    colnames(a) <- paste0("a", seq_len(D))
  }
  colnames(par) <- c(paste0("a", seq_len(D)), paste0("d", 0:(max_poly - 1)))
  rownames(par) <- paste0("item", seq_len(I))
  colnames(response) <- paste0("item", seq_len(I))

  ak <- matrix(0:(max_poly - 1), I, max_poly, byrow = TRUE)
  colnames(ak) <- paste0("ak", 0:(max_poly - 1))
  par.mirt <- cbind(par[, 1:D, drop = FALSE], ak,
                    par[, (D + 1):ncol(par), drop = FALSE])

  data.obj <- list(
    data        = response,
    response    = response,
    theta       = theta,
    par         = par,
    par.mirt    = par.mirt,
    Q.matrix    = Q.matrix,
    block.items = NULL,
    Corr        = Corr,
    length.poly = length.poly,
    mask        = mask,
    probability = prob,
    N           = N,
    I           = I,
    D           = D,
    call        = call,
    arguments   = list(
      N           = N,
      I           = I,
      D           = D,
      length.poly = length.poly,
      Q.matrix    = Q.matrix,
      Corr        = Corr,
      rotate      = rotate,
      promax_m    = promax_m
    )
  )

  class(data.obj) <- "data.MGPCM"

  return(data.obj)
}

#' Compute MGPCM Category Response Probabilities
#'
#' @param theta \eqn{N \times D} matrix of latent trait values.
#' @param par \eqn{I \times (D + \max K_i)} parameter matrix with
#'   columns \code{a1..aD, d0, d1, ...}. Missing category columns for
#'   shorter items should be \code{NA}; \code{d0} is typically fixed at 0.
#' @return An \eqn{N \times \sum_i K_i} matrix of category probabilities
#'   (stacked by item), where
#'   \eqn{P(Y_{ij}=k) \propto \exp\{k\,\mathbf{a}_i'
#'   \boldsymbol{\theta}_j + d_{ik}\}}.
#' @keywords internal
model.MGPCM <- function(theta, par){
  cpp_model_MGPCM(theta, par)
}

# library(ForceChoice)
#
# ############################### data ###############################
# seed <- 4592166
# N <- 500
# I <- 15
# D <- 2
# length.poly <- 3
# Corr <- NULL
# Q.matrix <- NULL
#
# set.seed(seed)
# data.obj <- sim.data.MGGUM(N = N, I = I, D = D, length.poly = length.poly,
#                            Q.matrix = Q.matrix, Corr = Corr)
#
# data.obj$par
# response <- data.obj$response
# head(response)
# apply(response, 2, table)
# Q.matrix <- data.obj$arguments$Q.matrix
#
# # ############################### Stan MCMC ###############################
# # object.MGGUM.stan <- fit.MGGUM(response, D = D, Q.matrix = Q.matrix,
# #                                length.poly = length.poly,
# #                                method = "stan",
# #                                control.method = c(list(
# #                                  init = "random",
# #                                  algorithm = "HMC"), list(seed = seed, vis = TRUE)))
# # cbind(data.obj$par, object.MGGUM.stan$par$est)
# # object.MGGUM.stan$par$Rhat
# # get.fit.index(object.MGGUM.stan)
#
# ############################### iStEM ###############################
# iStEM.control.method <- list(
#   burnin.maxitr = 40,
#   maxitr = 500,
#   theta.proposal.sd = 0.35,
#   optim.maxit = 50
# )
#
# object.MGGUM.iStEM <- fit.MGGUM(response, D = D, Q.matrix = Q.matrix,
#                                 length.poly = length.poly,
#                                 method = "iStEM",
#                                 control.method = c(iStEM.control.method, list(seed = seed, vis = TRUE)))
# cbind(data.obj$par, object.MGGUM.iStEM$par$est)
#
# # ---- Validate logLik with new quadrature helpers ----
# ll <- logLik(object.MGGUM.iStEM)
# stopifnot(inherits(ll, "logLik"))
# stopifnot(is.finite(as.numeric(ll)))
# cat(sprintf("  MGGUM logLik: %.2f (df=%d)\n", as.numeric(ll), attr(ll, "df")))
#
# get.fit.index(object.MGGUM.iStEM)

#' Fit MIRT Model via Stan
#'
#' @param response An N x I matrix of binary responses (0/1)
#' @param model Model type: \code{"m1pl"}, \code{"m2pl"}, \code{"m3pl"}, or \code{"m4pl"}
#' @param D Number of latent dimensions
#' @param Q.matrix An I x D 0/1 matrix indicating which discrimination parameters
#'   are freely estimated (1) and which are fixed to 0 (default: NULL, builds
#'   a triangular identification structure).
#' @param chains Number of MCMC chains
#' @param iter Number of iterations per chain
#' @param warmup Number of warmup iterations per chain
#' @param thin Thinning interval
#' @param init Initial values (0 = random)
#' @param algorithm Sampling algorithm: \code{"HMC"} (default), \code{"NUTS"},
#'   or \code{"Fixed_param"}.
#' @param control.model A named list of model-level controls and
#'   hyperparameters:
#'   \itemize{
#'     \item \code{a.mu}, \code{a.sigma}: lognormal prior for a parameters
#'     \item \code{b.mu}, \code{b.sigma}: normal prior for item difficulties
#'     \item \code{c.mu}, \code{c.sigma}: uniform support for c parameters
#'     \item \code{d.mu}, \code{d.sigma}: uniform support for d parameters
#'     \item \code{theta.mu}: prior mean vector for theta
#'     \item \code{L}: quadrature grid size used by \code{logLik.MIRT()}
#'   }
#' @param control.method A named list of Stan-specific controls:
#'   \itemize{
#'     \item \code{cores}, \code{vis}, \code{seed}: execution controls
#'     \item \code{chains}, \code{iter}, \code{warmup}, \code{thin},
#'           \code{init}, \code{algorithm}: Stan execution settings
#'     \item \code{adapt_delta}, \code{max_treedepth}, \code{stepsize},
#'           \code{int_time}, \code{metric}: Stan algorithm control parameters
#'   }
#'
#' @noRd
fit.MIRT.stan <- function(response, model = "m2pl", D = 2, Q.matrix = NULL,
                          control.model = NULL, control.method = NULL,
                          .call = NULL) {

  call <- if (is.null(.call)) match.call() else .call

  control <- fc_as_control_list(control.model, "control.model")
  control.method <- fc_as_control_list(control.method, "control.method")
  common.method <- fit_common_method_control(control.method)
  stan.method <- fc_stan_method_control(control.method)
  algorithm <- stan.method$algorithm

  model <- tolower(model)

  model_type <- switch(model,
    "m1pl" = 1L,
    "m2pl" = 2L,
    "m3pl" = 3L,
    "m4pl" = 4L,
    stop("Unknown MIRT model type: ", model, call. = FALSE)
  )

  sm <- stanmodels[["MIRT"]]

  if (is.null(sm)) {
    stop("Stan model ", model, " not found !", call. = FALSE)
  }

  mcmc_env <- setup_mcmc_env(common.method$cores)
  on.exit(restore_mcmc_env(mcmc_env), add = TRUE)

  stan_control <- build_stan_control(algorithm, control.method)

  N <- nrow(response)
  I <- ncol(response)

  response <- istem_prepare_response(response, binary = TRUE)
  Q.matrix <- istem_prepare_01_q(I, D, Q.matrix, triangular = TRUE)

  corr_df <- free_corr_npar(D)

  data.long <- data.frame(
    y   = as.vector(response),
    pid = rep(1:N, times = I),
    iid = rep(1:I, each = N)
  )

  theta_mu_raw <- get_ctrl("theta.mu", rep(0, D), control)
  theta_mu     <- array(theta_mu_raw, dim = D)

  stan.data <- list(
    N_obs      = nrow(data.long),
    N          = N,
    I          = I,
    D          = D,
    model_type = model_type,
    Q_matrix   = Q.matrix,
    y          = data.long$y,
    pid        = data.long$pid,
    iid        = data.long$iid,
    a_mu       = get_ctrl("a.mu", 0.25, control),
    a_sigma    = get_ctrl("a.sigma", 0.25, control),
    b_mu       = get_ctrl("b.mu", 0.0, control),
    b_sigma    = get_ctrl("b.sigma", 1.0, control),
    c_mu       = get_ctrl("c.mu", 0.0, control),
    c_sigma    = get_ctrl("c.sigma", 0.35, control),
    d_mu       = get_ctrl("d.mu", 0.65, control),
    d_sigma    = get_ctrl("d.sigma", 1.0, control),
    theta_mu   = theta_mu
  )

  set.seed(common.method$seed)

  stan.obj <- suppressWarnings(
    rstan::sampling(
      object        = sm,
      data          = stan.data,
      chains        = stan.method$chains,
      iter          = stan.method$iter,
      warmup        = stan.method$warmup,
      thin          = stan.method$thin,
      init          = stan.method$init,
      algorithm     = algorithm,
      cores         = mcmc_env$cores,
      control       = stan_control,
      verbose       = common.method$vis,
      show_messages = common.method$vis
    )
  )

  MCMC.obj <- rstan::extract(stan.obj, permuted = TRUE)
  stan.sum <- rstan::summary(stan.obj)$summary

  theta_est <- extract_theta_stan(MCMC.obj, stan.sum, N, D, rownames(response))

  b    <- apply(MCMC.obj$b, 2, mean)
  b.se <- apply(MCMC.obj$b, 2, sd)
  b.idx <- grep("^b\\[", rownames(stan.sum))

  par    <- matrix(NA_real_, nrow = I, ncol = D + 3)
  par.se <- matrix(NA_real_, nrow = I, ncol = D + 3)
  par.Rhat <- matrix(NA_real_, nrow = I, ncol = D + 3)

  if (model %in% c("m2pl", "m3pl", "m4pl")) {
    a_arr  <- ensure_3d(MCMC.obj$a)
    a      <- apply(a_arr, c(2, 3), mean)
    a.se   <- apply(a_arr, c(2, 3), sd)

    par[, 1:D]        <- a
    par.se[, 1:D]     <- a.se
    par.Rhat[, 1:D]   <- extract_rhat_matrix(stan.sum, "a", I, D)

  } else if (model == "m1pl") {
    par[, 1:D]      <- 1
    par.se[, 1:D]   <- NA_real_
    par.Rhat[, 1:D] <- NA_real_
  }

  par[, D + 1]        <- b
  par.se[, D + 1]     <- b.se
  par.Rhat[, D + 1]   <- stan.sum[b.idx, "Rhat"]

  if (model %in% c("m3pl", "m4pl")) {
    c_est    <- apply(MCMC.obj$c, 2, mean)
    c_est.se <- apply(MCMC.obj$c, 2, sd)
    c.idx <- grep("^c\\[", rownames(stan.sum))
    par[, D + 2]      <- c_est
    par.se[, D + 2]   <- c_est.se
    par.Rhat[, D + 2] <- stan.sum[c.idx, "Rhat"]
  } else {
    par[, D + 2]      <- 0
    par.se[, D + 2]   <- NA_real_
    par.Rhat[, D + 2] <- NA_real_
  }

  if (model == "m4pl") {
    d_est    <- apply(MCMC.obj$d, 2, mean)
    d_est.se <- apply(MCMC.obj$d, 2, sd)
    d.idx <- grep("^d\\[", rownames(stan.sum))
    par[, D + 3]      <- d_est
    par.se[, D + 3]   <- d_est.se
    par.Rhat[, D + 3] <- stan.sum[d.idx, "Rhat"]
  } else {
    par[, D + 3]      <- 1
    par.se[, D + 3]   <- NA_real_
    par.Rhat[, D + 3] <- NA_real_
  }

  par.free <- mirt_par_free_mask(model_type, Q.matrix)
  colnames(par) <- colnames(par.se) <- colnames(par.Rhat) <- colnames(par.free) <-
    c(paste0("a", 1:D), "b", "c", "d")
  rownames(par) <- rownames(par.se) <- rownames(par.Rhat) <- rownames(par.free) <-
    colnames(response)
  par.se[!par.free] <- NA_real_
  par.Rhat[!par.free] <- NA_real_
  npar <- sum(par.free) + corr_df

  Corr_est <- extract_corr_stan(MCMC.obj, stan.sum, D)
  par_est  <- list(est = par, se = par.se, Rhat = par.Rhat, free = par.free)

  results <- list(
    npar     = npar,
    method   = "stan",
    theta    = theta_est,
    par      = par_est,
    Corr     = Corr_est,
    stan.obj = stan.obj,
    MCMC.obj = MCMC.obj,
    log_lik = MCMC.obj$log_lik,
    Q.matrix = Q.matrix,
    call = call,
    arguments = list(
      response  = response,
      model     = model,
      D         = D,
      Q.matrix  = Q.matrix,
      method    = "stan",
      cores     = common.method$cores,
      vis       = common.method$vis,
      seed      = common.method$seed,
      control.model = control,
      control.method = fit_effective_method_control(
        control.method, method = stan.method, common = common.method
      )
    )
  )

  class(results) <- "MIRT"

  L <- get_ctrl("L", NULL, control)
  results$logLik <- logLik.MIRT(results, L=L)

  return(results)
}

#' @describeIn logLik-class FCGDINA model: marginal forced-choice probabilities over
#'   all \eqn{2^D} attribute profiles.
#' @export
logLik.FCGDINA <- function(object, ...) {
  prep <- fcgdina_prepare(
    data = object$response,
    Q.matrix = object$Q.matrix,
    model = object$model,
    block.items = object$block.items,
    fc.type = object$fc.type
  )

  prob.class <- fcgdina_prob_class_link(
    prep, object$delta$est, link = fcgdina_delta_link(object))
  pi <- object$pi
  lik <- fcgdina_marginal_loglik(prob.class, prep, pi, grouped = TRUE)

  results <- c(`log Likelihood` = lik)
  attr(results, "nobs") <- prep$N
  attr(results, "df") <- object$npar
  attr(results, "pi") <- pi
  attr(results, "prob.class") <- prob.class
  class(results) <- "logLik"
  results
}

#' @describeIn logLik-class FCGGUM model: forced-choice unfolding block
#'   probabilities with multivariate normal quadrature.
#' @export
logLik.FCGGUM <- function(object, theta.low = NULL, theta.up = NULL, L = NULL,
                          ...) {

  par <- object$par$est
  response <- object$response
  block.items <- object$block.items
  patterns <- object$patterns
  patterns.total <- object$patterns.total

  N <- nrow(response)
  B <- length(block.items)

  df <- object$npar
  D <- object$arguments$D
  grid.control <- istem_latent_grid_control(
    object$arguments$control.model, D, L = L,
    theta.low = theta.low, theta.up = theta.up
  )
  L <- grid.control$L
  theta.low <- grid.control$theta.lower
  theta.up <- grid.control$theta.upper

  theta.mu <- array(
    if (is.null(object$arguments$control.model$theta.mu)) rep(0, D)
    else object$arguments$control.model$theta.mu,
    dim = D
  )

  grid <- make_quadrature_grid_mvn(
    object$Corr$est, theta.mu, D, L, theta.low, theta.up
  )

  prob <- model.FCGGUM(grid$theta.norm, par, patterns.total, patterns)
  response.group <- istem_response_groups(response)
  lik <- cpp_loglik_indexed(
    prob = prob,
    response = matrix(as.integer(response.group$response),
                      nrow = response.group$G, ncol = B),
    pi = as.vector(grid$pi),
    block_sizes = as.integer(vapply(patterns, nrow, integer(1L))),
    response_base = 1L
  )
  lik <- loglik_apply_group_count(lik, response.group$count)
  results <- c(`log Likelihood` = lik$logLik)
  results <- set_loglik_attributes(results, N, df, lik, grid$theta.norm,
                                    prob, grid$pi, grid$L, theta.low, theta.up,
                                    response.unique = response.group$response,
                                    response.count = response.group$count,
                                    response.group = response.group$group)
  return(results)
}

################################# MGPCM iStEM #################################

fit.MGPCM.iStEM <- function(response, D = 2, Q.matrix = NULL,
                            length.poly = NULL,
                            control.model = NULL, control.method = NULL,
                            .call = NULL) {

  call <- if (is.null(.call)) match.call() else .call
  control.model <- fc_as_control_list(control.model, "control.model")
  control.method <- fc_as_control_list(control.method, "control.method")
  common.method <- fit_common_method_control(control.method)
  vis <- common.method$vis
  method <- istem_method_control(control.method)
  set.seed(common.method$seed)

  response <- as.matrix(response)
  if (anyNA(response) || any(response < 0) ||
      any(response != floor(response))) {
    stop("'response' must be an N x I matrix of integer responses starting at 0.",
         call. = FALSE)
  }
  storage.mode(response) <- "integer"

  N <- nrow(response)
  I <- ncol(response)
  response.group <- istem_response_groups(response)
  response.fit <- response.group$response
  count.fit <- response.group$count
  D <- as.integer(D)
  if (D < 1L) {
    stop("'D' must be a positive integer.", call. = FALSE)
  }
  method <- istem_apply_grid_control(method, control.model, D)
  istem_check_method_control(method)
  control.model <- istem_effective_grid_model_control(control.model, method)

  length.poly <- istem_prepare_length_poly(response, length.poly)
  max_poly <- max(length.poly)
  Q.matrix <- istem_prepare_01_q(I, D, Q.matrix, triangular = TRUE)

  prior <- mgpcm_istem_prior_control(control.model)
  bounds <- list(
    a.lower = get_ctrl("a.lower", 1e-4, control.method),
    a.upper = get_ctrl("a.upper", 6, control.method),
    d.lower = get_ctrl("d.lower", -8, control.method),
    d.upper = get_ctrl("d.upper", 8, control.method)
  )
  mgpcm_istem_check_bounds(bounds)

  theta_mu <- array(get_ctrl("theta.mu", rep(0, D), control.model), dim = D)
  include_corr <- D > 1L && !method$fix.corr
  corr_df <- free_corr_npar(D, include_corr)
  par <- mgpcm_istem_initial_par(response.fit, Q.matrix, length.poly, bounds,
                                 prior)

  state <- list(
    response = response.fit,
    response.count = count.fit,
    par = par,
    theta = istem_random_theta(response.group$G, D, theta_mu, method$theta.lower,
                               method$theta.upper),
    Corr = istem_random_corr(D, include_corr),
    theta_mu = theta_mu,
    Q.matrix = Q.matrix,
    length.poly = length.poly,
    max.poly = max_poly,
    optim.maxit = method$optim.maxit,
    bounds = bounds,
    prior = prior
  )

  run <- istem_run(
    state = state,
    method = method,
    N = N,
    vis = vis,
    label = "MGPCM",
    update_parameters = mgpcm_istem_update_parameters,
    param_vec = function(state) {
      mgpcm_istem_param_vec(state$par, state$Corr, state$Q.matrix,
                            state$length.poly, include_corr)
    },
    sample_theta = mgpcm_istem_sample_theta,
    logLik_fun = function(state) {
      mgpcm_istem_loglik_trace(
        state, method$L, method$theta.lower, method$theta.upper
      )
    }
  )

  final <- mgpcm_istem_param_unpack(
    pv = run$chain.mean,
    pv_se = run$chain.sd,
    pv_rhat = NULL,
    I = I,
    D = D,
    Q.matrix = Q.matrix,
    length.poly = length.poly,
    max_poly = max_poly,
    Corr = run$state$Corr,
    include_corr = include_corr
  )

  theta.est <- istem_expand_group_matrix(
    run$theta.est, response.group$group, rownames(response))
  theta.se <- istem_expand_group_matrix(
    run$theta.se, response.group$group, rownames(response))
  theta_est <- istem_named_theta(theta.est, theta.se, response, D)
  par <- final$par
  par.se <- final$par.se
  par.Rhat <- final$par.rhat
  par.free <- mgpcm_par_free_mask(Q.matrix, length.poly, max_poly)
  colnames(par) <- colnames(par.se) <- colnames(par.Rhat) <- colnames(par.free) <-
    c(paste0("a", seq_len(D)), paste0("d", 0:(max_poly - 1L)))
  rownames(par) <- rownames(par.se) <- rownames(par.Rhat) <- rownames(par.free) <-
    colnames(response)
  par.se[!par.free] <- NA_real_
  par.Rhat[!par.free] <- NA_real_
  npar <- sum(par.free) + corr_df

  Corr.Rhat <- final$Corr.rhat
  dimnames(Corr.Rhat) <- list(paste0("Dim.", seq_len(D)), paste0("Dim.", seq_len(D)))

  results <- list(
    npar = npar,
    method = "iStEM",
    theta = theta_est,
    par = list(est = par, se = par.se, Rhat = par.Rhat, free = par.free),
    Corr = list(est = final$Corr, se = final$Corr.se, Rhat = Corr.Rhat),
    stan.obj = NULL,
    MCMC.obj = NULL,
    Q.matrix = Q.matrix,
    length.poly = length.poly,
    call = call,
    arguments = list(
      response = response,
      D = D,
      Q.matrix = Q.matrix,
      length.poly = length.poly,
      method = "iStEM",
      cores = common.method$cores,
      vis = vis,
      seed = common.method$seed,
      control.model = control.model,
      control.method = fit_effective_method_control(
        control.method, method = c(method, bounds), common = common.method
      )
    ),
    iStEM = utils::modifyList(run$iStEM, list(prior = prior))
  )

  class(results) <- "MGPCM"
  results$logLik <- logLik.MGPCM(results, L = method$L,
                                  theta.low = method$theta.lower,
                                  theta.up = method$theta.upper)
  results
}

mgpcm_istem_prior_control <- function(control.model) {
  use_prior <- get_ctrl("use.prior", TRUE, control.model)
  if (!is.logical(use_prior) || length(use_prior) != 1L || is.na(use_prior)) {
    stop("'use.prior' in 'control.model' must be TRUE or FALSE.",
         call. = FALSE)
  }
  list(
    use.prior = use_prior,
    a.mu = istem_scalar(get_ctrl("a.mu", 0.25, control.model), "a.mu"),
    a.sigma = istem_scalar(get_ctrl("a.sigma", 0.25, control.model),
                           "a.sigma", positive = TRUE),
    d.mu = istem_scalar(get_ctrl("d.mu", 0, control.model), "d.mu"),
    d.sigma = istem_scalar(get_ctrl("d.sigma", 1, control.model),
                           "d.sigma", positive = TRUE)
  )
}

mgpcm_istem_check_bounds <- function(bounds) {
  for (nm in names(bounds)) {
    bounds[[nm]] <- istem_scalar(bounds[[nm]], nm, arg = "control.method")
  }
  if (bounds$a.lower <= 0 || bounds$a.lower >= bounds$a.upper) {
    stop("'a.lower' must be positive and smaller than 'a.upper'.",
         call. = FALSE)
  }
  if (bounds$d.lower >= bounds$d.upper) {
    stop("'d.lower' must be smaller than 'd.upper'.", call. = FALSE)
  }
  invisible(NULL)
}

mgpcm_istem_initial_par <- function(response, Q.matrix, length.poly, bounds,
                                    prior) {
  I <- ncol(response)
  D <- ncol(Q.matrix)
  max_poly <- max(length.poly)
  par <- matrix(NA_real_, I, D + max_poly)
  a <- matrix(0, I, D)
  a[Q.matrix == 1] <- istem_rlnorm_bounded(
    sum(Q.matrix == 1), prior$a.mu, prior$a.sigma,
    bounds$a.lower, bounds$a.upper
  )
  par[, seq_len(D)] <- a
  for (i in seq_len(I)) {
    Ki <- length.poly[i]
    d_i <- rep(NA_real_, max_poly)
    d_i[1L] <- 0
    if (Ki > 1L) {
      d_i[2:Ki] <- istem_rnorm_bounded(
        Ki - 1L, prior$d.mu, prior$d.sigma,
        bounds$d.lower, bounds$d.upper
      )
    }
    par[i, D + seq_len(max_poly)] <- d_i
  }
  par
}

mgpcm_istem_sample_theta <- function(state, theta_grid_length,
                                     theta_lower, theta_upper) {
  chol <- istem_corr_chol(state$Corr)
  cpp_gibbs_mgpcm_theta(
    theta_ = state$theta,
    par_ = state$par,
    response_ = state$response,
    length_poly_ = as.integer(state$length.poly),
    theta_mu_ = state$theta_mu,
    chol_corr_ = chol$chol,
    log_diag_sum = chol$log_diag_sum,
    step = theta_grid_length,
    lower = theta_lower,
    upper = theta_upper
  )
}

mgpcm_istem_update_parameters <- function(state) {
  for (i in seq_len(ncol(state$response))) {
    state$par[i, ] <- mgpcm_istem_fit_item(
      y = state$response[, i],
      theta = state$theta,
      par_i = state$par[i, ],
      q_i = state$Q.matrix[i, ],
      Ki = state$length.poly[i],
      weight = istem_state_weight(state),
      optim_maxit = state$optim.maxit,
      bounds = state$bounds,
      prior = state$prior
    )
  }
  state
}

mgpcm_istem_fit_item <- function(y, theta, par_i, q_i, Ki, weight, optim_maxit,
                                 bounds, prior) {
  D <- ncol(theta)
  active <- q_i == 1
  x0 <- c(par_i[seq_len(D)][active], par_i[D + 2:Ki])
  lower <- c(rep(bounds$a.lower, sum(active)), rep(bounds$d.lower, Ki - 1L))
  upper <- c(rep(bounds$a.upper, sum(active)), rep(bounds$d.upper, Ki - 1L))

  obj <- function(x) {
    item <- mgpcm_istem_x_to_item(x, par_i, active, D, Ki)
    ll <- cpp_mgpcm_item_loglik_weighted(
      theta = theta,
      a = item$a,
      d = item$d,
      response = as.integer(y),
      weight = weight
    )
    lp <- mgpcm_istem_item_log_prior(item, active, prior)
    if (!is.finite(lp)) return(Inf)
    -ll - lp
  }

  opt <- tryCatch(
    stats::optim(
      par = x0,
      fn = obj,
      method = "L-BFGS-B",
      lower = lower,
      upper = upper,
      control = list(maxit = optim_maxit)
    ),
    error = function(e) NULL
  )
  if (is.null(opt) || !is.finite(opt$value)) {
    return(par_i)
  }
  item <- mgpcm_istem_x_to_item(opt$par, par_i, active, D, Ki)
  out <- par_i
  out[seq_len(D)] <- item$a
  out[D + seq_len(Ki)] <- item$d
  out
}

mgpcm_istem_x_to_item <- function(x, par_i, active, D, Ki) {
  a <- rep(0, D)
  idx <- 1L
  n_active <- sum(active)
  if (n_active > 0L) {
    a[active] <- x[idx:(idx + n_active - 1L)]
    idx <- idx + n_active
  }
  d <- rep(NA_real_, Ki)
  d[1L] <- 0
  if (Ki > 1L) {
    d[2:Ki] <- x[idx:(idx + Ki - 2L)]
  }
  out <- par_i
  out[seq_len(D)] <- a
  out[D + seq_len(Ki)] <- d
  list(par = out, a = a, d = d)
}

mgpcm_istem_item_log_prior <- function(item, active, prior) {
  if (!isTRUE(prior$use.prior)) return(0)
  lp <- 0
  a_active <- item$a[active]
  if (length(a_active) > 0L) {
    if (any(a_active <= 0)) return(-Inf)
    lp <- lp + sum(stats::dlnorm(
      a_active, meanlog = prior$a.mu, sdlog = prior$a.sigma, log = TRUE
    ))
  }
  if (length(item$d) > 1L) {
    lp <- lp + sum(stats::dnorm(
      item$d[-1L], mean = prior$d.mu, sd = prior$d.sigma, log = TRUE
    ))
  }
  lp
}

mgpcm_istem_param_vec <- function(par, Corr, Q.matrix, length.poly,
                                  include_corr) {
  D <- ncol(Q.matrix)
  pv <- par[, seq_len(D), drop = FALSE][Q.matrix == 1]
  for (i in seq_along(length.poly)) {
    Ki <- length.poly[i]
    if (Ki > 1L) {
      pv <- c(pv, par[i, D + 2:Ki])
    }
  }
  c(pv, istem_param_corr_vec(Corr, include_corr))
}

mgpcm_istem_param_unpack <- function(pv, pv_se, pv_rhat = NULL, I, D, Q.matrix,
                                     length.poly, max_poly, Corr, include_corr) {
  idx <- 1L
  par <- matrix(NA_real_, I, D + max_poly)
  par.se <- matrix(NA_real_, I, D + max_poly)
  par.rhat <- matrix(NA_real_, I, D + max_poly)
  has_rhat <- !is.null(pv_rhat)
  a <- matrix(0, I, D)
  a.se <- matrix(NA_real_, I, D)
  a.rhat <- matrix(NA_real_, I, D)
  n_active <- sum(Q.matrix == 1)
  if (n_active > 0L) {
    a[Q.matrix == 1] <- pv[idx:(idx + n_active - 1L)]
    a.se[Q.matrix == 1] <- pv_se[idx:(idx + n_active - 1L)]
    if (has_rhat) a.rhat[Q.matrix == 1] <- pv_rhat[idx:(idx + n_active - 1L)]
    idx <- idx + n_active
  }
  par[, seq_len(D)] <- a
  par.se[, seq_len(D)] <- a.se
  par.rhat[, seq_len(D)] <- a.rhat
  for (i in seq_len(I)) {
    Ki <- length.poly[i]
    par[i, D + 1L] <- 0
    par.se[i, D + 1L] <- NA_real_
    if (Ki > 1L) {
      n_d <- Ki - 1L
      par[i, D + 2:Ki] <- pv[idx:(idx + n_d - 1L)]
      par.se[i, D + 2:Ki] <- pv_se[idx:(idx + n_d - 1L)]
      if (has_rhat) par.rhat[i, D + 2:Ki] <- pv_rhat[idx:(idx + n_d - 1L)]
      idx <- idx + n_d
    }
  }
  corr <- istem_unpack_corr(pv, pv_se, idx, D, Corr, include_corr)
  corr.rhat <- istem_unpack_corr_rhat(pv_rhat, idx, D, include_corr)
  list(par = par, par.se = par.se, par.rhat = par.rhat,
       Corr = corr$Corr, Corr.se = corr$Corr.se, Corr.rhat = corr.rhat)
}

#' @describeIn get.fit.index MGGUM model: polytomous unfolding responses with
#'   M2* category-collapsed construction.
#' @export
get.fit.index.MGGUM <- function(object, ...) {
  response <- object$arguments$response
  par <- object$par$est
  D <- object$arguments$D
  Q.matrix <- object$Q.matrix
  length.poly <- object$length.poly

  logLik.obj <- object$logLik
  pi <- as.vector(attr(logLik.obj, "pi"))
  theta.norm <- attr(logLik.obj, "theta.norm")
  N <- nrow(response)
  I <- ncol(response)
  max_poly <- max(length.poly)
  npar <- object$npar

  extract.free.par <- function(par, ...) {
    par.vec <- numeric(0)
    for (i in 1:I) {
      for (j in 1:D) {
        if (abs(Q.matrix[i, j]) == 1) par.vec <- c(par.vec, par[i, j])
      }
    }
    for (i in 1:I) {
      for (j in 1:D) {
        if (abs(Q.matrix[i, j]) == 1) par.vec <- c(par.vec, par[i, D + j])
      }
    }
    for (i in 1:I) {
      Ki <- length.poly[i]
      if (Ki > 1) {
        cols <- (D + D + 2):(D + D + Ki)
        par.vec <- c(par.vec, par[i, cols])
      }
    }
    return(par.vec)
  }

  expand.free.par <- function(par.vec, ...) {
    par <- matrix(NA_real_, nrow = I, ncol = D + D + max_poly)
    idx <- 1L
    for (i in 1:I) {
      for (j in 1:D) {
        if (abs(Q.matrix[i, j]) == 1) { par[i, j] <- par.vec[idx]; idx <- idx + 1L }
        else { par[i, j] <- 0.0 }
      }
    }
    for (i in 1:I) {
      for (j in 1:D) {
        if (abs(Q.matrix[i, j]) == 1) {
          par[i, D + j] <- par.vec[idx]
          idx <- idx + 1L
        } else {
          par[i, D + j] <- 0.0
        }
      }
    }
    par[, D + D + 1] <- 0
    for (i in 1:I) {
      Ki <- length.poly[i]
      if (Ki > 1) {
        for (k in 1:(Ki - 1)) {
          par[i, D + D + 1 + k] <- par.vec[idx]
          idx <- idx + 1L
        }
      }
    }
    return(par)
  }

  par.vec <- extract.free.par(par)

  response.unfold <- matrix(0, N, sum(length.poly))
  idx <- 0
  for (i in 1:I) {
    response.temp <- matrix(0, N, length.poly[i])
    for (p in 1:N) {
      response.temp[p, response[p, i] + 1] <- 1
    }
    response.unfold[, (idx + 1):(idx + length.poly[i])] <- response.temp
    idx <- idx + length.poly[i]
  }

  loglik.fun <- function(par.vec, ...) {
    par <- expand.free.par(par.vec, ...)
    prob_new <- model.MGGUM(theta.norm, par)

    eps        <- .Machine$double.xmin
    log_prob   <- log(pmax(prob_new, eps))
    log_pi     <- log(pi)

    pi.length <- nrow(theta.norm)
    log_L_theta_Xi <- matrix(0, N, pi.length)
    for (p in 1:N) {
      response.p <- matrix(response.unfold[p, ], pi.length, ncol(prob_new), byrow = TRUE)
      log_L_theta_Xi[p, ] <- rowSums(response.p * log_prob) + log_pi
    }

    logLik.value <- sum(apply(log_L_theta_Xi, 1, function(x) {
      m <- max(x)
      m + log(sum(exp(x - m)))
    }))
    return(logLik.value)
  }

  prob.fun <- function(par.vec, ...) {
    par <- expand.free.par(par.vec, ...)
    model.MGGUM(theta.norm, par)
  }

  good.of.fit(
    par.vec    = par.vec,
    loglik.fun = loglik.fun,
    prob.fun   = prob.fun,
    response   = response,
    npar       = npar,
    pi         = pi,
    response.type = "polytomous",
    response.K = length.poly,
    ...
  )
}

#' Simulate FCMIRT Data
#'
#' @description
#' Simulates forced-choice ranking data using a MIRT item response model
#' (\code{"m1pl"} through \code{"m4pl"}) as the item-level endorsement model.
#' Forced-choice responses are generated by applying the sequential
#' Luce/Plackett choice rule to the logits of the item-level endorsement
#' probabilities, with RANK, MOLE, and PICK patterns handled by
#' \code{\link{model.FCMIRT}}.
#' For identification, item difficulty/intercept parameters \code{b} are
#' centered within each forced-choice block so that their block sum is zero.
#'
#' @param N.person Number of persons.
#' @param N.block Number of forced-choice blocks.
#' @param I.block Number of items per block when \code{control$block.items} is
#'   not supplied.
#' @param D Number of latent dimensions. If \code{control$Q.matrix} is supplied,
#'   \code{D} is reset to \code{ncol(control$Q.matrix)}.
#' @param model MIRT model type: \code{"m1pl"}, \code{"m2pl"}, \code{"m3pl"}, or
#'   \code{"m4pl"}.
#' @param fc.type Forced-choice response type: \code{"RANK"}, \code{"MOLE"}, or
#'   \code{"PICK"}. A scalar value is recycled to all blocks.
#' @param control Optional list. Supported entries are \code{Q.matrix},
#'   \code{block.items}, and \code{Corr}.
#'
#' @return An object of class \code{"data.FCMIRT"}, a list containing
#'   \code{data}, \code{response}, \code{theta}, \code{par}, \code{Q.matrix},
#'   \code{block.items}, \code{Corr}, \code{model}, \code{patterns},
#'   \code{patterns.total}, \code{probability}, \code{prob}, and
#'   data-generating arguments.
#'
#' @seealso \code{\link{fit.FCMIRT}}, \code{\link{model.FCMIRT}}
#'
#' @examples
#' set.seed(123)
#' sim <- sim.data.FCMIRT(N.person = 20, N.block = 3, I.block = 2,
#'                        D = 2, model = "m2pl", fc.type = "RANK")
#' str(sim$data)
#' head(sim$response)
#' dim(sim$prob)
#'
#' @export
sim.data.FCMIRT <- function(N.person = 1000, N.block = 10, I.block = 2,
                            D = 3, model = "m2pl", fc.type = "RANK",
                            control = NULL) {

  call <- match.call()

  if (is.null(control)) {
    control <- list()
  }

  # ---- Model validation ----
  model <- tolower(model)
  model.valid <- c("m1pl", "m2pl", "m3pl", "m4pl")
  if (!model %in% model.valid) {
    stop("'model' must be one of: ", paste(model.valid, collapse = ", "), call. = FALSE)
  }

  # ---- Scalar integer validation ----
  N.person <- check_integer_scalar(N.person, "N.person", 1L)
  D        <- check_integer_scalar(D,        "D",        1L)
  I.block  <- check_integer_scalar(I.block,  "I.block",  2L)
  N.block  <- check_integer_scalar(N.block,  "N.block",  1L)

  # ---- Q-matrix and block configuration ----
  Q.matrix    <- control$Q.matrix
  block.items <- control$block.items

  res <- resolve_fc_blocks(
    Q.matrix       = Q.matrix,
    block.items    = block.items,
    I.block        = I.block,
    N.block        = N.block,
    D              = D,
    q.valid.values = c(0, 1),
    q.row.check    = function(r) any(r > 0)
  )
  I.states   <- res$I.states
  N.block    <- res$N.block
  I.block    <- res$I.block
  block.items <- res$block.items
  Q.matrix   <- res$Q.matrix
  D          <- res$D

  # ---- fc.type validation ----
  fc.type <- normalize_fc_type(fc.type, N.block)

  # ---- Corr and theta ----
  Corr  <- validate_corr_matrix(control$Corr, D)
  theta <- generate_theta_mvn(N.person, D, Corr)

  # ---- Auto-generate Q-matrix if not supplied ----
  if (is.null(Q.matrix)) {
    Q.matrix <- sim.Q.matrix.FC(I.states = I.states, D = D,
                                block.items = block.items,
                                allow.negative = FALSE)
  }

  # ---- MIRT item parameters ----
  if (model == "m1pl") {
    a <- Q.matrix
  } else {
    a <- matrix(rlnorm(I.states * D, meanlog = 0.25, sdlog = 0.25),
                nrow = I.states, ncol = D) * Q.matrix
  }

  b <- rnorm(I.states, 0, 1)
  for (items in block.items) {
    b[items] <- b[items] - mean(b[items])
  }
  c <- runif(I.states, 0.00, 0.35)
  d <- runif(I.states, 0.65, 1.00)

  if (model %in% c("m1pl", "m2pl")) {
    c <- rep(0.0, I.states)
    d <- rep(1.0, I.states)
  } else if (model == "m3pl") {
    d <- rep(1.0, I.states)
  }

  par <- cbind(a, b, c, d)

  # ---- Item-level endorsement probabilities ----
  probability <- model.MIRT(theta = theta, par = par)

  # ---- FC permutation patterns ----
  pat <- generate_fc_permutation_patterns(block.items, fc.type)
  patterns.total <- pat$patterns.total
  patterns       <- pat$patterns

  # ---- FC pattern probabilities ----
  prob <- model.FCMIRT(theta = theta, par = par,
                       patterns.total = patterns.total,
                       patterns = patterns)

  # ---- Response sampling ----
  samples <- sample_fc_responses(prob, patterns, N.person, N.block)
  response <- samples$response
  data     <- samples$data

  # ---- Row/column names ----
  rownames(theta) <- paste0("person", seq_len(N.person))
  colnames(theta) <- paste0("theta", seq_len(D))
  rownames(response) <- rownames(data) <- rownames(theta)
  colnames(response) <- colnames(data) <- paste0("block.", seq_len(N.block))

  rownames(par) <- paste0("item", seq_len(I.states))
  colnames(par) <- c(paste0("a", seq_len(D)), "b", "c", "d")

  rownames(Q.matrix) <- paste0("item", seq_len(I.states))
  colnames(Q.matrix) <- paste0("dim", seq_len(D))

  rownames(Corr) <- colnames(Corr) <- paste0("dim", seq_len(D))

  # ---- Output assembly ----
  data.obj <- list(
    data           = data,
    response       = response,
    theta          = theta,
    par            = par,
    Q.matrix       = Q.matrix,
    block.items    = block.items,
    Corr           = Corr,
    model          = model,
    patterns       = patterns,
    patterns.total = patterns.total,
    probability    = probability,
    prob           = prob,
    N.person       = N.person,
    N.block        = N.block,
    I.block        = I.block,
    D              = D,
    I.states       = I.states,
    fc.type        = fc.type,
    call           = call,
    arguments      = list(
      N.person = N.person,
      N.block  = N.block,
      I.block  = I.block,
      D        = D,
      model    = model,
      fc.type  = fc.type,
      control  = control
    )
  )

  class(data.obj) <- "data.FCMIRT"

  return(data.obj)
}


#' Compute FCMIRT Forced-Choice Pattern Probabilities
#'
#' @param theta N x D matrix of latent trait values.
#' @param par I x (D + 3) MIRT item parameter matrix, where b is the
#'   item difficulty and the item-level linear predictor is a' theta - b.
#' @param patterns.total List of full-ranking pattern matrices.
#' @param patterns List of observed pattern matrices.
#'
#' @return N x sum(pattern counts) probability matrix. For each block,
#'   item endorsement probabilities are first computed by \code{model.MIRT},
#'   transformed to logits, and then combined with a sequential
#'   Luce/Plackett rule. MOLE and PICK probabilities are obtained by summing
#'   compatible full-ranking probabilities and normalizing over the observed
#'   block patterns.
#' @keywords internal
model.FCMIRT <- function(theta, par, patterns.total, patterns) {
  cpp_model_FCMIRT(theta, par, patterns.total, patterns)
}

# ---- Shared utilities for sim.data.* functions ---------------------------------
#
# This file contains helper functions that are used across multiple
# sim.data.* data-simulation functions.  They are tagged @keywords internal
# and are not exported.


# ---- Integer scalar validation ------------------------------------------------

#' Validate a scalar integer parameter
#'
#' Checks that \code{x} is a single finite value coercible to integer and
#' at least \code{min.val}.  Used by forced-choice \code{sim.data.*}
#' functions to validate \code{N.person}, \code{N.block}, \code{I.block},
#' and \code{D}.
#'
#' @param x Value to check.
#' @param name Parameter name (used in the error message).
#' @param min.val Minimum allowed value (default 1L).
#' @return \code{as.integer(x)} on success; raises an error otherwise.
#' @keywords internal
check_integer_scalar <- function(x, name, min.val = 1L) {
  if (length(x) != 1L || !is.numeric(x) || is.na(x) || !is.finite(x) ||
      x < min.val || x != floor(x)) {
    stop("'", name, "' must be an integer >= ", min.val, ".",
         call. = FALSE)
  }
  as.integer(x)
}

normalize_length_poly <- function(length.poly, I) {
  if (length(length.poly) == 1L) {
    length.poly <- rep(length.poly, I)
  } else if (length(length.poly) != I) {
    stop("'length.poly' must have length 1 or 'I'.", call. = FALSE)
  }
  if (!is.numeric(length.poly) || anyNA(length.poly) ||
      any(!is.finite(length.poly)) || any(length.poly != floor(length.poly)) ||
      any(length.poly < 2L)) {
    stop("'length.poly' must contain integers >= 2.", call. = FALSE)
  }
  as.integer(length.poly)
}


# ---- Theta generation ---------------------------------------------------------

#' Generate latent trait vectors from a multivariate normal distribution
#'
#' When \code{D > 1}, uses \code{MASS::mvrnorm} with the supplied correlation
#' matrix.  When \code{D == 1}, uses standard normal draws.
#'
#' @param N Number of persons.
#' @param D Number of latent dimensions.
#' @param Corr A \eqn{D \times D} correlation matrix.
#' @return An \eqn{N \times D} numeric matrix of latent trait values.
#' @keywords internal
generate_theta_mvn <- function(N, D, Corr) {
  if (D > 1L) {
    if (!requireNamespace("MASS", quietly = TRUE)) {
      stop("Package 'MASS' is required for multivariate normal theta generation.", call. = FALSE)
    }
    MASS::mvrnorm(n = N, mu = rep(0, D), Sigma = Corr)
  } else {
    matrix(rnorm(N, 0, 1), ncol = 1L)
  }
}


# ---- Factor rotation ----------------------------------------------------------

# Apply a loading-space transformation consistently to loadings, row-wise
# latent scores, and their column covariance matrix.  If A* = A T, preserving
# theta %*% t(A) requires theta* = theta T^{-T}; consequently the latent
# covariance becomes T^{-1} Corr T^{-T}.
#
# @param a I x D loading matrix.
# @param theta N x D matrix whose rows are latent scores.
# @param Corr D x D latent covariance/correlation matrix.
# @param Tmat D x D loading transformation matrix.
# @param standardize Whether to rescale the transformed latent dimensions to
#   unit variance while applying the inverse rescaling to theta and a.
# @return A list containing transformed a, theta, Corr, and the effective Tmat.
# @noRd
.fc_transform_rotation <- function(a, theta, Corr, Tmat,
                                   standardize = FALSE) {
  a <- as.matrix(a)
  theta <- as.matrix(theta)
  Corr <- as.matrix(Corr)
  Tmat <- as.matrix(Tmat)

  D <- ncol(a)
  if (ncol(theta) != D ||
      !identical(dim(Corr), c(D, D)) ||
      !identical(dim(Tmat), c(D, D))) {
    stop("'a', 'theta', 'Corr', and 'Tmat' have incompatible dimensions.",
         call. = FALSE)
  }
  if (anyNA(a) || anyNA(theta) || anyNA(Corr) || anyNA(Tmat) ||
      any(!is.finite(a)) || any(!is.finite(theta)) ||
      any(!is.finite(Corr)) || any(!is.finite(Tmat))) {
    stop("Rotation inputs must contain only finite numeric values.",
         call. = FALSE)
  }

  T_inv <- tryCatch(
    solve(Tmat),
    error = function(e) {
      stop("Rotation transformation matrix is singular: ",
           conditionMessage(e), call. = FALSE)
    }
  )

  a_rot <- a %*% Tmat
  theta_rot <- theta %*% t(T_inv)
  Corr_rot <- T_inv %*% Corr %*% t(T_inv)
  Corr_rot <- (Corr_rot + t(Corr_rot)) / 2
  T_effective <- Tmat

  if (isTRUE(standardize)) {
    variances <- diag(Corr_rot)
    if (any(!is.finite(variances)) ||
        any(variances <= .Machine$double.eps)) {
      stop("Rotated latent variances must be finite and positive.",
           call. = FALSE)
    }

    scale_mat <- diag(sqrt(variances), nrow = D)
    scale_inv <- diag(1 / sqrt(variances), nrow = D)
    a_rot <- a_rot %*% scale_mat
    theta_rot <- theta_rot %*% scale_inv
    Corr_rot <- scale_inv %*% Corr_rot %*% scale_inv
    Corr_rot <- (Corr_rot + t(Corr_rot)) / 2
    diag(Corr_rot) <- 1
    T_effective <- Tmat %*% scale_mat
  }

  dimnames(a_rot) <- dimnames(a)
  dimnames(theta_rot) <- dimnames(theta)
  dimnames(Corr_rot) <- dimnames(Corr)
  dimnames(T_effective) <- dimnames(Tmat)

  list(
    a = a_rot,
    theta = theta_rot,
    Corr = Corr_rot,
    Tmat = T_effective
  )
}

#' Apply factor rotation to discrimination matrix and latent traits
#'
#' Supports \code{"promax"} (via \code{stats::promax}) and GPArotation
#' methods (\code{"varimax"}, \code{"oblimin"}, etc.).  Transforms the
#' discrimination matrix \code{a}, the latent trait matrix \code{theta},
#' and the correlation matrix \code{Corr} accordingly.
#'
#' @param a \eqn{I \times D} discrimination (loading) matrix.
#' @param theta \eqn{N \times D} latent trait matrix.
#' @param Corr \eqn{D \times D} correlation matrix.
#' @param rotate Rotation method name (character).
#' @param D Number of dimensions.
#' @param promax_m Power parameter for \code{"promax"} (default 4).
#' @return A named list with components \code{a}, \code{theta}, \code{Corr}.
#' @keywords internal
apply_rotation_promax <- function(a, theta, Corr, rotate, D, promax_m = 4) {
  if (is.null(rotate) || D <= 1L) {
    return(list(a = a, theta = theta, Corr = Corr))
  }

  if (tolower(rotate) == "promax") {
    if (!requireNamespace("stats", quietly = TRUE)) {
      stop("Package 'stats' is required for promax rotation.", call. = FALSE)
    }
    rot.res <- stats::promax(a, m = promax_m)
    T.mat <- rot.res$rotmat
  } else {
    if (rotate == "Varimax") {
      rotate <- "varimax"
    }
    orth.methods <- c("varimax", "quartimax")
    obl.methods  <- c("oblimin", "quartimin", "oblimax",
                      "entropy", "simplimax", "geominQ")
    if (rotate %in% orth.methods) {
      if (!requireNamespace("GPArotation", quietly = TRUE)) {
        stop("Package 'GPArotation' is required for rotation method '", rotate, "'.", call. = FALSE)
      }
      rot.res <- GPArotation::GPForth(a, method = rotate)
      T.mat <- rot.res$Th
    } else if (rotate %in% obl.methods) {
      if (!requireNamespace("GPArotation", quietly = TRUE)) {
        stop("Package 'GPArotation' is required for rotation method '", rotate, "'.", call. = FALSE)
      }
      rot.res <- GPArotation::GPFoblq(a, method = rotate)
      T.mat <- solve(t(rot.res$Th))
    } else {
      warning("Unknown rotation method '", rotate, "'. Skipping rotation.")
      return(list(a = a, theta = theta, Corr = Corr))
    }
  }

  rotated <- .fc_transform_rotation(
    a = a,
    theta = theta,
    Corr = Corr,
    Tmat = T.mat,
    standardize = TRUE
  )
  rotated[c("a", "theta", "Corr")]
}


# ---- Forced-choice block / Q-matrix resolution --------------------------------

#' Resolve forced-choice block and Q-matrix configuration
#'
#' Shared validation and auto-generation logic used by
#' \code{sim.data.FCMIRT}, \code{sim.data.FCGGUM}, \code{sim.data.FCGDINA},
#' and \code{sim.data.TIRT}.
#'
#' @param Q.matrix User-supplied Q-matrix or \code{NULL}.
#' @param block.items User-supplied block-item list or \code{NULL}.
#' @param I.block Default items-per-block (scalar).
#' @param N.block Default number of blocks (scalar).
#' @param D Default number of dimensions (scalar).
#' @param q.valid.values Allowed Q-matrix values (e.g. \code{c(0, 1)} for
#'   MIRT/DCM, \code{c(-1, 0, 1)} for GGUM/TIRT).
#' @param q.row.check Function applied to each row of Q.matrix; must return
#'   \code{TRUE} for every row.  Example: \code{function(r) any(r != 0)} or
#'   \code{function(r) sum(abs(r)) == 1L}.
#'
#' @return A named list with components \code{I.states}, \code{N.block},
#'   \code{I.block}, \code{block.items}, \code{Q.matrix}, \code{D}.
#'   \code{Q.matrix} is \code{NULL} if it was not supplied and should be
#'   generated later (e.g. by \code{sim.Q.matrix.FC} or \code{sim.data.Q.CDM}).
#' @keywords internal
resolve_fc_blocks <- function(Q.matrix, block.items, I.block, N.block, D,
                              q.valid.values, q.row.check) {
  if (!is.null(Q.matrix)) {
    Q.matrix <- as.matrix(Q.matrix)
    if (anyNA(Q.matrix) || !all(Q.matrix %in% q.valid.values)) {
      stop("'Q.matrix' must contain only ",
           paste(q.valid.values, collapse = ", "), ".", call. = FALSE)
    }
    if (!all(apply(Q.matrix, 1L, q.row.check))) {
      stop("Each row of 'Q.matrix' violates the required row constraint.", call. = FALSE)
    }
    storage.mode(Q.matrix) <- "numeric"
    I.states <- nrow(Q.matrix)
    D        <- ncol(Q.matrix)
  } else {
    I.states <- N.block * I.block
  }

  if (!is.null(block.items)) {
    if (!is.list(block.items)) {
      stop("'block.items' must be a list.", call. = FALSE)
    }
    N.block <- length(block.items)
    if (N.block < 1L) {
      stop("'block.items' must contain at least one block.", call. = FALSE)
    }

    block.lengths <- vapply(block.items, length, integer(1L))
    if (any(block.lengths < 2L)) {
      stop("Each block in 'block.items' must contain at least two items.", call. = FALSE)
    }
    I.block <- if (length(unique(block.lengths)) == 1L) {
      block.lengths[1L]
    } else {
      block.lengths
    }

    all.items <- unlist(block.items, use.names = FALSE)
    if (anyDuplicated(all.items)) {
      stop("'block.items' contains duplicate items.", call. = FALSE)
    }

    if (is.null(Q.matrix)) {
      I.states <- length(all.items)
    }
    if (!setequal(all.items, seq_len(I.states))) {
      stop("'block.items' must contain exactly all item indices from 1 to the number of items.", call. = FALSE)
    }
  } else {
    if (is.null(Q.matrix)) {
      I.states <- N.block * I.block
    } else {
      if (I.states %% I.block != 0L) {
        stop("Number of items in 'Q.matrix' is not divisible by I.block.", call. = FALSE)
      }
      N.block <- as.integer(I.states / I.block)
    }
    block.items <- split(seq_len(I.states),
                         rep(seq_len(N.block), each = I.block))
  }

  list(I.states   = I.states,
       N.block    = N.block,
       I.block    = I.block,
       block.items = block.items,
       Q.matrix   = Q.matrix,
       D          = D)
}


# ---- Forced-choice permutation patterns ---------------------------------------

#' Generate forced-choice permutation patterns
#'
#' Produces the full-ranking patterns (\code{patterns.total}) and the
#' observed patterns (\code{patterns}) for each block, based on the
#' forced-choice response type.
#'
#' @param block.items List of integer vectors (item indices per block).
#' @param fc.type Character vector of length \code{length(block.items)}
#'   with values \code{"RANK"}, \code{"MOLE"}, or \code{"PICK"}.
#' @return A named list with components \code{patterns.total} (list of
#'   full-ranking matrices) and \code{patterns} (list of observed-pattern
#'   matrices).
#' @keywords internal
generate_fc_permutation_patterns <- function(block.items, fc.type) {
  N.block <- length(block.items)

  patterns.total <- lapply(block.items, function(x) {
    get_permutations(x, length(x))
  })

  patterns <- vector("list", N.block)
  for (bb in seq_len(N.block)) {
    patterns[[bb]] <- get_permutations(
      block.items[[bb]],
      switch(fc.type[bb],
             RANK = length(block.items[[bb]]),
             MOLE = 2L,
             PICK = 1L)
    )
  }

  list(patterns.total = patterns.total, patterns = patterns)
}


# ---- Forced-choice response sampling ------------------------------------------

#' Sample forced-choice responses from block-level pattern probabilities
#'
#' Uses inverse-CDF multinomial sampling (identical to the C++ implementation
#' in \code{forced_choice_from_agree}) to draw a discrete pattern index for
#' each person-block combination.
#'
#' @param prob \eqn{N \times \sum_B P_b} matrix of block-level pattern
#'   probabilities (stacked by block).
#' @param patterns List of observed-pattern matrices (one per block).
#' @param N.person Number of persons.
#' @param N.block Number of blocks.
#' @return A named list with \code{response} (\eqn{N \times B} integer
#'   matrix of pattern indices) and \code{data} (\eqn{N \times B} character
#'   matrix of ranking strings like \code{"1>3>2"}).
#' @keywords internal
sample_fc_responses <- function(prob, patterns, N.person, N.block) {
  response <- matrix(NA_integer_, N.person, N.block)
  data     <- matrix(NA_character_, N.person, N.block)

  idx <- 0L
  for (bb in seq_len(N.block)) {
    patterns.cur <- patterns[[bb]]
    bR <- nrow(patterns.cur)
    cols <- idx + seq_len(bR)
    for (pp in seq_len(N.person)) {
      prob.bp <- cumsum(prob[pp, cols])
      y <- which(runif(1, 0, 1) <= prob.bp)[1L]
      if (is.na(y)) {
        y <- bR
      }
      response[pp, bb] <- y
      data[pp, bb] <- paste(patterns.cur[y, ], collapse = ">")
    }
    idx <- idx + bR
  }

  list(response = response, data = data)
}


# ---- Pattern key conversion ---------------------------------------------------

#' Convert binary matrix rows to string keys
#'
#' Collapses each row of a binary matrix to a single string by pasting
#' the entries without separator (e.g. \code{c(0,1,0)} becomes \code{"010"}).
#'
#' @param x A numeric or integer matrix.
#' @return A character vector of length \code{nrow(x)}.
#' @keywords internal
pattern_key <- function(x) {
  x <- as.matrix(x)
  apply(x, 1L, paste0, collapse = "")
}


# ---- Probability clamping -----------------------------------------------------

#' Clamp probabilities away from 0 and 1
#'
#' Ensures all values lie in \eqn{[eps, 1-eps]} to avoid \code{log(0)}
#' or division-by-zero issues in downstream computations.
#'
#' @param p Numeric vector or matrix of probabilities.
#' @param eps Small positive value (default \code{1e-12}).
#' @return Numeric vector or matrix with the same dimensions as \code{p}.
#' @keywords internal
clip_prob <- function(p, eps = 1e-12) {
  pmin(pmax(as.numeric(p), eps), 1 - eps)
}


# ---- CDM item response probabilities ------------------------------------------

#' Build CDM item response probabilities for attribute mastery patterns
#'
#' Computes the probability of endorsing an item for each attribute mastery
#' pattern under the specified CDM condensation rule (DINA, DINO, ACDM, or
#' GDINA).  \code{p0} is the lower-bound (guessing) probability and
#' \code{p1} is the upper-bound (1 - slipping) probability.
#'
#' @param patterns A binary matrix of attribute mastery patterns.
#' @param model One of \code{"DINA"}, \code{"DINO"}, \code{"ACDM"}, \code{"GDINA"}.
#' @param p0 Lower-bound probability (guessing).
#' @param p1 Upper-bound probability (1 - slipping).
#' @param mono.constraint Logical; if \code{TRUE} (default), enforce the
#'   monotonicity constraint that mastering more attributes never decreases
#'   the success probability.  Only applies to \code{model = "GDINA"}.
#' @return A named numeric vector of probabilities, one per pattern row.
#' @keywords internal
make_item_prob_cdm <- function(patterns, model, p0, p1,
                                mono.constraint = TRUE) {
  model <- toupper(model)
  patterns <- as.matrix(patterns)

  K <- ncol(patterns)
  score <- rowSums(patterns)

  if (p0 > p1) {
    stop("'P0' should be less than or equal to 'P1'.", call. = FALSE)
  }

  p <- switch(
    model,

    DINA = ifelse(score == K, p1, p0),

    DINO = ifelse(score > 0, p1, p0),

    ACDM = {
      if (K > 0) {
        w <- runif(K)
        w <- w / sum(w)
        as.vector(p0 + patterns %*% ((p1 - p0) * w))
      } else {
        rep(p0, length(score))
      }
    },

    GDINA = {
      if (K <= 1L) {
        out <- c(p0, p1)
      } else if (mono.constraint) {
        # Enforce monotonicity: mastering more attributes never
        # decreases the success probability (matching GDINA package).
        out <- rep(p0, length(score))
        out[score == K] <- p1
        mid <- which(score > 0 & score < K)
        pre <- get_precedent_patterns(patterns)
        for (l in mid) {
          out[l] <- runif(1, max(out[pre[[l]]]), p1)
        }
      } else {
        # No monotonicity: random probabilities in [p0, p1]
        out <- rep(p0, length(score))
        out[score == K] <- p1
        mid <- score > 0 & score < K
        out[mid] <- runif(sum(mid), p0, p1)
      }
      out
    }
  )

  p <- clip_prob(p)
  names(p) <- pattern_key(patterns)
  p
}


# ---- Forced-choice transformation (R implementation) --------------------------

#' Compute forced-choice pattern probabilities from item endorsement probabilities
#'
#' Transforms item-level endorsement probabilities into block-level ranking
#' pattern probabilities via the sequential Luce/Plackett choice rule.
#' This is an R implementation of the C++ \code{forced_choice_from_agree}
#' function used by \code{cpp_model_FCMIRT} and \code{cpp_model_FCGGUM}.
#'
#' @param agree An \eqn{N \times I} matrix of item endorsement probabilities.
#' @param patterns.total List of full-ranking pattern matrices (one per block).
#' @param patterns List of observed-pattern matrices (one per block).
#' @return An \eqn{N \times \sum_B P_b} matrix of forced-choice pattern
#'   probabilities, where \eqn{P_b} is the number of observed patterns for
#'   block \eqn{b}.
#' @keywords internal
forced_choice_from_agree <- function(agree, patterns.total, patterns) {
  agree <- as.matrix(agree)
  N <- nrow(agree)
  N.block <- length(patterns.total)

  if (length(patterns) != N.block) {
    stop("'patterns.total' and 'patterns' must have the same length.", call. = FALSE)
  }

  # Logit transform with numerical clamping (preserve matrix dimensions)
  eps <- 1e-12
  agree.clamped <- agree
  agree.clamped[agree.clamped < eps] <- eps
  agree.clamped[agree.clamped > 1 - eps] <- 1 - eps
  logit <- log(agree.clamped) - log1p(-agree.clamped)

  # Total number of observed pattern columns
  prob.cols <- sum(vapply(patterns, nrow, integer(1L)))
  prob <- matrix(NA_real_, N, prob.cols)

  idx <- 0L

  for (b in seq_len(N.block)) {
    pat.total <- patterns.total[[b]]
    pat.obs <- patterns[[b]]

    bR.total <- nrow(pat.total)
    bC.total <- ncol(pat.total)
    bR <- nrow(pat.obs)
    bC <- ncol(pat.obs)

    if (bR.total <= 0L || bC.total <= 0L || bR <= 0L || bC <= 0L) {
      stop("Pattern matrices must have positive dimensions.", call. = FALSE)
    }

    # Compute log-probability of each full ranking for each person
    log.prob.total <- matrix(0, N, bR.total)

    for (br in seq_len(bR.total)) {
      for (p in seq_len(N)) {
        log.prob.br <- 0.0

        for (bc in seq_len(bC.total - 1L)) {
          selected <- as.integer(pat.total[br, bc])

          # Log-sum-exp over remaining candidates
          candidates <- as.integer(pat.total[br, bc:bC.total])
          terms <- logit[p, candidates]
          max.term <- max(terms)
          denom.sum <- sum(exp(terms - max.term))
          log.denom <- max.term + log(denom.sum)

          log.prob.br <- log.prob.br + logit[p, selected] - log.denom
        }

        log.prob.total[p, br] <- log.prob.br
      }
    }

    # Softmax normalization of full ranking probabilities
    prob.total <- matrix(0, N, bR.total)
    for (p in seq_len(N)) {
      max.log <- max(log.prob.total[p, ])
      denom.sum <- sum(exp(log.prob.total[p, ] - max.log))
      log.denom <- max.log + log(denom.sum)
      prob.total[p, ] <- exp(log.prob.total[p, ] - log.denom)
    }

    # Aggregate full rankings to observed patterns
    cols <- idx + seq_len(bR)

    if (bC == bC.total) {
      # RANK: direct match; find each observed pattern in full rankings
      matches <- integer(bR)
      for (br in seq_len(bR)) {
        for (fr in seq_len(bR.total)) {
          if (all(pat.obs[br, ] == pat.total[fr, ])) {
            matches[br] <- fr
            break
          }
        }
        if (matches[br] <= 0L) {
          stop("'patterns' contains a full ranking absent from 'patterns.total'.", call. = FALSE)
        }
      }
      for (br in seq_len(bR)) {
        prob[, cols[br]] <- prob.total[, matches[br]]
      }

    } else if (bC == 2L) {
      # MOLE: sum full rankings where first matches AND last matches
      matches <- vector("list", bR)
      for (br in seq_len(bR)) {
        first <- as.integer(pat.obs[br, 1L])
        last  <- as.integer(pat.obs[br, 2L])
        matched.fr <- integer(0L)
        for (fr in seq_len(bR.total)) {
          full.first <- as.integer(pat.total[fr, 1L])
          full.last  <- as.integer(pat.total[fr, bC.total])
          if (full.first == first && full.last == last) {
            matched.fr <- c(matched.fr, fr)
          }
        }
        matches[[br]] <- matched.fr
      }
      for (br in seq_len(bR)) {
        if (length(matches[[br]]) > 0L) {
          prob[, cols[br]] <- rowSums(
            prob.total[, matches[[br]], drop = FALSE]
          )
        } else {
          prob[, cols[br]] <- 0.0
        }
      }

    } else if (bC == 1L) {
      # PICK: sum full rankings where first item matches
      matches <- vector("list", bR)
      for (br in seq_len(bR)) {
        first <- as.integer(pat.obs[br, 1L])
        matched.fr <- integer(0L)
        for (fr in seq_len(bR.total)) {
          full.first <- as.integer(pat.total[fr, 1L])
          if (full.first == first) {
            matched.fr <- c(matched.fr, fr)
          }
        }
        matches[[br]] <- matched.fr
      }
      for (br in seq_len(bR)) {
        if (length(matches[[br]]) > 0L) {
          prob[, cols[br]] <- rowSums(
            prob.total[, matches[[br]], drop = FALSE]
          )
        } else {
          prob[, cols[br]] <- 0.0
        }
      }

    } else {
      stop("Unsupported forced-choice pattern length.", call. = FALSE)
    }

    # Normalize observed pattern probabilities within each block
    for (p in seq_len(N)) {
      row.sum <- sum(prob[p, cols])
      if (row.sum <= 0.0 || !is.finite(row.sum)) {
        stop("Computed forced-choice probabilities cannot be normalized.", call. = FALSE)
      }
      prob[p, cols] <- prob[p, cols] / row.sum
    }

    idx <- idx + bR
  }

  return(prob)
}


# ---- CDM monotonicity constraint ----------------------------------------------

#' Find precedent (subset) patterns for monotonicity enforcement
#'
#' For each attribute mastery pattern, identifies which other patterns are
#' proper subsets (i.e., mastering a subset of the attributes).  Used to
#' enforce the GDINA monotonicity constraint: mastering more attributes
#' should never decrease the success probability.
#'
#' Equivalent to \code{preloclist} in the GDINA R package.
#'
#' @param patterns A binary matrix of attribute mastery patterns
#'   (\eqn{L \times K}).
#' @return A list of length \eqn{L}; the \eqn{l}-th element is an integer
#'   vector of row indices corresponding to patterns that are proper
#'   subsets of pattern \eqn{l}.
#' @keywords internal
get_precedent_patterns <- function(patterns) {
  patterns <- as.matrix(patterns)
  L <- nrow(patterns)
  pre <- vector("list", L)
  for (l in seq_len(L)) {
    x <- patterns[l, ]
    # A pattern j is a subset of pattern l if:
    #   x OR patterns[j,] == x   AND   patterns[j,] != x
    is_subset <- rep(FALSE, L)
    for (j in seq_len(L)) {
      if (j == l) next
      if (all((x | patterns[j, ]) == x) && any(patterns[j, ] != x)) {
        is_subset[j] <- TRUE
      }
    }
    pre[[l]] <- which(is_subset)
  }
  pre
}

# ---- CDM design matrix --------------------------------------------------------

#' Build the CDM design matrix for a set of attribute patterns
#'
#' Constructs the design matrix \eqn{X} used in the cognitive diagnostic
#' model probability formulation \eqn{p = X\delta}.  The structure depends
#' on the model type:
#' \describe{
#'   \item{DINA}{Intercept + single mastery indicator (all attributes mastered).}
#'   \item{DINO}{Intercept + single "at-least-one" indicator.}
#'   \item{ACDM}{Intercept + main effects of each attribute.}
#'   \item{GDINA}{Full saturated interaction matrix (all \eqn{2^K} columns).}
#' }
#'
#' @param patterns A binary matrix of attribute mastery patterns
#'   (\eqn{L \times K}).
#' @param model One of \code{"DINA"}, \code{"DINO"}, \code{"ACDM"}, \code{"GDINA"}.
#' @return A numeric design matrix \eqn{X} with \eqn{L} rows.  Column count
#'   depends on the model.
#' @keywords internal
get_design_matrix_cdm <- function(patterns, model = "GDINA") {
  model <- toupper(model)
  patterns <- as.matrix(patterns)

  L <- nrow(patterns)
  K <- ncol(patterns)

  if (!model %in% c("DINA", "DINO", "ACDM", "GDINA")) {
    stop("'model' must be one of: DINA, DINO, ACDM, GDINA.", call. = FALSE)
  }

  if (model == "DINA") {
    X <- cbind(
      intercept = 1,
      master    = as.integer(rowSums(patterns) == K)
    )
  } else if (model == "DINO") {
    X <- cbind(
      intercept    = 1,
      at_least_one = as.integer(rowSums(patterns) > 0)
    )
  } else if (model == "ACDM") {
    X <- cbind(intercept = 1, patterns)
    colnames(X) <- c("intercept", paste0("main", seq_len(K)))
  } else {
    # GDINA: full interaction matrix
    X <- outer(
      seq_len(L),
      seq_len(L),
      Vectorize(function(l1, l2) prod(patterns[l1, ] ^ patterns[l2, ]))
    )
    colnames(X) <- paste0("d", pattern_key(patterns))
  }

  storage.mode(X) <- "numeric"
  X
}

#' Goodness-of-Fit Indices for Latent Probability Models
#'
#' @description
#' Computes a comprehensive suite of model-fit diagnostics for probability
#' models used in IRT, CDM, latent class, and forced-choice applications.
#' Indices include likelihood-based information criteria, limited-information
#' goodness-of-fit statistics (M2, RMSEA, SRMSR), incremental/comparative fit
#' indices (CFI, TLI), pseudo-\eqn{R^2} measures, residual diagnostics,
#' local-dependence statistics (Yen's Q3), and posterior classification
#' diagnostics.
#'
#' @section Likelihood and Information Criteria:
#'
#' Let \eqn{\ell = \log L(\hat{\boldsymbol{\psi}})} be the maximized
#' log-likelihood, \eqn{N} the sample size, and \eqn{p} the number of free
#' parameters (\code{npar}).
#'
#' \strong{Deviance:}
#' \deqn{D = -2\ell}
#'
#' \strong{AIC} (Akaike, 1974):
#' \deqn{\text{AIC} = D + 2p}
#'
#' \strong{AICc} (Hurvich & Tsai, 1989), small-sample correction:
#' \deqn{\text{AICc} = \text{AIC} + \frac{2p(p+1)}{N - p - 1},
#'       \quad N > p + 1}
#'
#' \strong{BIC} (Schwarz, 1978):
#' \deqn{\text{BIC} = D + p \log N}
#'
#' \strong{CAIC} (Bozdogan, 1987), consistent AIC:
#' \deqn{\text{CAIC} = D + p(\log N + 1)}
#'
#' \strong{SABIC} (Sclove, 1987), sample-size adjusted BIC:
#' \deqn{\text{SABIC} = D + p \log\bigl(\frac{N + 2}{24}\bigr)}
#'
#' \strong{HQIC} (Hannan & Quinn, 1979):
#' \deqn{\text{HQIC} = D + 2p \log(\log N), \quad N > e}
#'
#' @section Limited-Information M2 Family:
#'
#' The limited-information framework (Maydeu-Olivares & Joe, 2005, 2006) uses
#' first- and second-order marginal moments rather than the full
#' \eqn{I}-way contingency table. This is essential for sparse high-dimensional
#' categorical responses where full-information tests break down.
#'
#' Let \eqn{\mathbf{m}(\boldsymbol{\psi})} be the vector of model-implied
#' univariate and bivariate moments (dimension \eqn{M}), and let
#' \eqn{\hat{\boldsymbol{\Xi}}_2} be their asymptotic covariance matrix
#' under the model.
#'
#' \strong{M2 statistic:}
#' \deqn{
#'   M_2 = N\,
#'   \mathbf{e}'\hat{\mathbf{C}}_2\mathbf{e},
#' }
#' where \eqn{\mathbf{e} = \mathbf{p} - \boldsymbol{\pi}(\hat{\boldsymbol{\psi}})}
#' is the vector of marginal residuals, and
#' \eqn{\hat{\mathbf{C}}_2 = \boldsymbol{\Delta}_c(\boldsymbol{\Delta}_c'
#' \hat{\boldsymbol{\Xi}}_2 \boldsymbol{\Delta}_c)^{-1}\boldsymbol{\Delta}_c'}
#' with \eqn{\boldsymbol{\Delta}_c} the orthogonal complement of the Jacobian
#' matrix \eqn{\boldsymbol{\Delta} = \partial\boldsymbol{\pi}/\partial\boldsymbol{\psi}'}.
#'
#' Under correct model specification, \eqn{M_2 \sim \chi^2_{M - p}}
#' asymptotically.
#'
#' \strong{RMSEA (Root Mean Square Error of Approximation):}
#' \deqn{
#'   \text{RMSEA} = \sqrt{\max\!\left(0,\; \frac{M_2 - df}{N \cdot df}\right)},
#'   \qquad df = M - p
#' }
#' Confidence intervals are obtained via non-central \eqn{\chi^2} inversion.
#' The close-fit test evaluates \eqn{H_0: \text{RMSEA} \le 0.05}.
#'
#' \strong{SRMSR (Standardized Root Mean Square Residual):}
#' \deqn{
#'   \text{SRMSR} = \sqrt{
#'     \frac{1}{J} \sum_{i < j} (r_{ij}^{\text{obs}} - r_{ij}^{\text{model}})^2
#'   },
#' }
#' where \eqn{r_{ij}} are the observed and model-implied correlation residuals,
#' summed over \eqn{J} unique off-diagonal pairs.
#'
#' \strong{McDonald's NCI (Non-Centrality Index):}
#' \deqn{
#'   \text{NCI} = \exp\!\left(-\frac{\max(M_2 - df, 0)}{2N}\right)
#' }
#'
#' @section Incremental / Comparative Fit Indices:
#'
#' \strong{CFI (Comparative Fit Index):}
#' \deqn{
#'   \text{CFI} = 1 - \frac{\max(M_2 - df, 0)}
#'                           {\max(M_2^{\text{null}} - df^{\text{null}}, M_2 - df, 0)},
#' }
#' where the null (independence) model has \eqn{M_2^{\text{null}}} with
#' \eqn{df^{\text{null}}} degrees of freedom.
#'
#' \strong{TLI (Tucker--Lewis Index, a.k.a. NNFI):}
#' \deqn{
#'   \text{TLI} = \frac{M_2^{\text{null}} / df^{\text{null}} - M_2 / df}
#'                     {M_2^{\text{null}} / df^{\text{null}} - 1}
#' }
#'
#' \strong{IFI (Incremental Fit Index, a.k.a. BFI):}
#' \deqn{
#'   \text{IFI} = \frac{M_2^{\text{null}} - M_2}{M_2^{\text{null}} - df}
#' }
#'
#' @section Pseudo-\eqn{R^2} Measures:
#'
#' \strong{McFadden's \eqn{R^2}:}
#' \deqn{R^2_{\text{McF}} = 1 - \frac{\ell}{\ell_0}}
#'
#' \strong{McFadden's adjusted \eqn{R^2}:}
#' \deqn{R^2_{\text{McF,adj}} = 1 - \frac{\ell - p}{\ell_0}}
#'
#' \strong{Cox--Snell \eqn{R^2}:}
#' \deqn{R^2_{\text{CS}} = 1 - \exp\!\left(\frac{2}{N}(\ell_0 - \ell)\right)}
#'
#' \strong{Nagelkerke \eqn{R^2}:}
#' \deqn{R^2_{\text{N}} = \frac{R^2_{\text{CS}}}
#'                                 {1 - \exp(2\ell_0 / N)}}
#'
#' \strong{Aldrich--Nelson \eqn{R^2}:}
#' \deqn{R^2_{\text{AN}} = \frac{G^2}{G^2 + N},
#'       \quad G^2 = D_0 - D}
#'
#' \strong{Veall--Zimmermann \eqn{R^2}:}
#' \deqn{R^2_{\text{VZ}} = R^2_{\text{AN}} \times
#'       \frac{-2\ell_0 + N}{-2\ell_0}}
#'
#' where \eqn{\ell_0} is the null-model (independence) log-likelihood and
#' \eqn{D_0} the corresponding deviance.
#'
#' @section Yen's Q3 Local Dependence:
#'
#' For each pair of items \eqn{(i, k)}, the residual correlation is
#' \deqn{
#'   Q_{3,ik} = \text{Corr}(r_{ji}, r_{jk}),
#'   \qquad r_{ji} = Y_{ji} - E[Y_{ji} \mid \hat{\boldsymbol{\theta}}_j],
#' }
#' where residuals are computed from posterior expected scores. Adjusted
#' Q3 values subtract the mean Q3 across all pairs to center the
#' distribution (Christensen, Makransky, & Horton, 2017).
#'
#' @section Heuristic Guidelines:
#'
#' The \code{print.summary.good.of.fit()} method reports descriptive
#' recommended ranges based on common conventions. These are heuristics,
#' not decision rules; fit cutoffs depend on model family, dimensionality,
#' category sparsity, local dependence, and sample size.
#' \itemize{
#'   \item \strong{RMSEA}: \eqn{\le 0.05} close fit; \eqn{\le 0.08} reasonable;
#'         \eqn{\le 0.10} mediocre (Browne & Cudeck, 1993)
#'   \item \strong{SRMSR}: \eqn{\le 0.08} acceptable (Hu & Bentler, 1999)
#'   \item \strong{CFI / TLI}: \eqn{\ge 0.95} good; \eqn{\ge 0.90} acceptable
#' }
#'
#' @param par.vec Numeric vector of free model parameters.
#' @param loglik.fun Function returning the fitted model log-likelihood.
#' @param prob.fun Function returning conditional response probabilities at
#'   the latent support points.
#' @param response An \eqn{N \times I} response matrix (binary, polytomous,
#'   or nominal coding depending on \code{response.type}).
#' @param npar Integer; number of free parameters in the model.
#' @param pi Numeric vector of latent support weights (summing to 1).
#' @param pi.fun Optional function returning latent support weights given
#'   \code{par.vec} and \code{...}. If \code{NULL}, \code{pi} is used as-is.
#' @param alpha Tail probability for the RMSEA confidence interval.
#'   Default \code{0.05} yields a 90% CI (two-sided \eqn{2\alpha}).
#'   Use \code{0.025} for 95% or \code{0.005} for 99%.
#' @param response.type Character string specifying the response coding:
#'   \code{"auto"} (default; detects from probability dimensions),
#'   \code{"binary"} (each column is a binary indicator),
#'   \code{"polytomous"} (stacked category probabilities, requires
#'   \code{response.K}), or \code{"nominal"} (expanded binary indicators
#'   with mutually exclusive categories within groups, requires
#'   \code{nominal.groups}).
#' @param response.K Integer vector of length \eqn{I} giving the number of
#'   categories for each response column. Required when
#'   \code{response.type = "polytomous"}.
#' @param bivariate.groups Optional integer vector of length \eqn{I};
#'   response column pairs within the same group are excluded from bivariate
#'   limited-information moments. Useful for forced-choice or testlet
#'   designs where within-block comparisons are structural zeros.
#' @param nominal.groups Integer vector of length matching the number of
#'   expanded binary indicators for \code{response.type = "nominal"}.
#'   Indicators sharing the same group value belong to the same original
#'   nominal item/block.
#' @param exclusive.groups Deprecated alias for \code{nominal.groups}.
#' @param n.boot Integer; number of bootstrap replicates for bootstrapped
#'   diagnostics. Default \code{1000}; set to \code{0} to skip.
#' @param ... Additional arguments passed to \code{loglik.fun},
#'   \code{prob.fun}, and \code{pi.fun}.
#'
#' @return An object of class \code{"good.of.fit"} (a list). Key components:
#' \describe{
#'   \item{\code{LogLik}, \code{deviance}}{Log-likelihood and deviance.}
#'   \item{\code{npar}, \code{N}, \code{nitems}}{Parameter count, sample
#'         size, number of items/indicators.}
#'   \item{\code{AIC}, \code{AICc}, \code{BIC}, \code{CAIC}, \code{SABIC},
#'         \code{HQIC}}{Information criteria.}
#'   \item{\code{M2.value}, \code{df}, \code{p.value}}{M2 statistic, degrees
#'         of freedom, asymptotic p-value.}
#'   \item{\code{RMSEA2}, \code{RMSEA.lower}, \code{RMSEA.upper},
#'         \code{RMSEA.CI.level}, \code{RMSEA.p.close}}{RMSEA estimate,
#'         confidence interval, interval level, close-fit test p-value.}
#'   \item{\code{SRMSR}, \code{RMSR}}{Standardized and raw root mean
#'         square residuals.}
#'   \item{\code{CFI}, \code{TLI}, \code{IFI}}{Incremental fit indices.}
#'   \item{\code{McFadden.R2}, \code{Nagelkerke.R2}, \code{CoxSnell.R2},
#'         \code{AldrichNelson.R2}, \code{VeallZimmermann.R2}}{Pseudo-\eqn{R^2}
#'         measures.}
#'   \item{\code{residuals}, \code{residuals.standardized},
#'         \code{correlation.residuals}}{Residual vectors.}
#'   \item{\code{moments.observed}, \code{moments.predicted}}{First- and
#'         second-order marginal moments.}
#'   \item{\code{q3.mean}, \code{q3.abs.max}, \code{q3.adjusted.abs.max},
#'         \code{q3.p95}}{Yen's Q3 local dependence statistics.}
#'   \item{\code{mean.max.posterior}, \code{median.max.posterior},
#'         \code{min.max.posterior}}{Posterior classification diagnostics.}
#'   \item{\code{null.LogLik}, \code{null.M2.value}, \code{null.df}}{Null
#'         (independence) model diagnostics.}
#'   \item{\code{LRT}, \code{LRT.df}, \code{LRT.p}}{Likelihood ratio test
#'         against the null model.}
#' }
#'
#' @references
#' Akaike, H. (1974). A new look at the statistical model identification.
#' \emph{IEEE Transactions on Automatic Control}, 19, 716--723.
#'
#' Bozdogan, H. (1987). Model selection and Akaike's Information Criterion
#' (AIC): The general theory and its analytical extensions.
#' \emph{Psychometrika}, 52, 345--370. \doi{10.1007/BF02294361}
#'
#' Browne, M. W., & Cudeck, R. (1993). Alternative ways of assessing model
#' fit. In K. A. Bollen & J. S. Long (Eds.), \emph{Testing structural
#' equation models} (pp. 136--162). Sage.
#'
#'
#' Christensen, K. B., Makransky, G., & Horton, M. C. (2017). Critical
#' values for Yen's Q3. \emph{Applied Psychological Measurement}, 41,
#' 178--194. \doi{10.1177/0146621616677520}
#'
#' Hannan, E. J., & Quinn, B. G. (1979). The determination of the order of
#' an autoregression. \emph{Journal of the Royal Statistical Society:
#' Series B}, 41, 190--195.
#'
#' Hu, L. T., & Bentler, P. M. (1999). Cutoff criteria for fit indexes in
#' covariance structure analysis. \emph{Structural Equation Modeling}, 6,
#' 1--55. \doi{10.1080/10705519909540118}
#'
#' Hurvich, C. M., & Tsai, C. L. (1989). Regression and time series model
#' selection in small samples. \emph{Biometrika}, 76, 297--307.
#' \doi{10.1093/biomet/76.2.297}
#'
#' MacCallum, R. C., Browne, M. W., & Sugawara, H. M. (1996). Power
#' analysis and determination of sample size for covariance structure
#' modeling. \emph{Psychological Methods}, 1, 130--149.
#'
#' Maydeu-Olivares, A., & Joe, H. (2005). Limited- and full-information
#' estimation and goodness-of-fit testing in 2^n contingency tables: A
#' unified framework. \emph{Journal of the American Statistical
#' Association}, 100, 1009--1020.
#'
#' Maydeu-Olivares, A., & Joe, H. (2006). Limited information
#' goodness-of-fit testing in multidimensional contingency tables.
#' \emph{Psychometrika}, 71, 713--732. \doi{10.1007/s11336-005-1295-9}
#'
#' McFadden, D. (1974). Conditional logit analysis of qualitative choice
#' behavior. In P. Zarembka (Ed.), \emph{Frontiers in econometrics}
#' (pp. 105--142). Academic Press.
#'
#' Schwarz, G. (1978). Estimating the dimension of a model. \emph{The Annals
#' of Statistics}, 6, 461--464. \doi{10.1214/aos/1176344136}
#'
#' Sclove, S. L. (1987). Application of model-selection criteria to some
#' problems in multivariate analysis. \emph{Psychometrika}, 52, 333--343.
#'
#' Yen, W. M. (1984). Effects of local item dependence on the fit and
#' equating performance of the three-parameter logistic model. \emph{Applied
#' Psychological Measurement}, 8, 125--145. \doi{10.1177/014662168400800201}
#'
#' @seealso
#' \code{\link{get.fit.index}} for model-specific wrappers,
#' \code{\link{summary.good.of.fit}} for formatted fit tables,
#' \code{\link{print.good.of.fit}} for the print method.
#'
#' @export
good.of.fit <- function(par.vec, loglik.fun, prob.fun, response, npar, pi,
                        pi.fun = NULL,
                        alpha = 0.05,
                        response.type = c("auto", "binary", "polytomous", "nominal", "ordinary"),
                        response.K = NULL,
                        bivariate.groups = NULL,
                        nominal.groups = NULL,
                        exclusive.groups = NULL,
                        n.boot = 1000L, ...) {

  N <- nrow(response)
  I <- ncol(response)
  n.boot <- as.integer(n.boot[1L])
  if (is.na(n.boot) || n.boot < 0L) n.boot <- 0L

  response <- matrix(as.numeric(response), nrow = N, ncol = I)

  observed.K <- apply(response, 2L, function(x) {
    x <- x[!is.na(x)]
    if (!length(x)) return(NA_integer_)
    as.integer(max(x) + 1L)
  })
  if (is.null(response.K)) {
    K <- observed.K
  } else {
    K <- as.integer(response.K)
    if (length(K) != I) {
      stop("'response.K' must have one entry per response column.", call. = FALSE)
    }
    if (any(!is.na(observed.K) & observed.K > K)) {
      stop("'response.K' is smaller than an observed response category.", call. = FALSE)
    }
  }
  K[is.na(K)] <- 2L
  response.type <- match.arg(response.type)
  if (response.type == "ordinary") {
    response.type <- "auto"
  }
  if (!is.null(exclusive.groups)) {
    nominal.groups <- exclusive.groups
    response.type <- "nominal"
  }

  # Nominal-data branch:
  # response is already expanded into K-1 binary indicators per nominal item.
  # Indicators from the same original item/block are mutually exclusive; they
  # contribute first-order moments and Xi11 covariances, but no bivariate
  # limited-information margins. TIRT and ordinary binary/polytomous models
  # keep using the default binary/polytomous branches.
  if (!is.null(exclusive.groups)) {
    warning("'exclusive.groups' is deprecated; use response.type = 'nominal' with 'nominal.groups'.",
            call. = FALSE)
  }

  prob_result <- prob.fun(par.vec, ...)
  prob_result <- pmin(pmax(prob_result, 1e-50), 1 - 1e-50)

  get_pi <- function(par.vec, n_support, ...) {
    out <- if (is.null(pi.fun)) pi else pi.fun(par.vec, ...)
    if (is.null(out)) {
      out <- rep(1 / n_support, n_support)
    }
    out <- as.vector(out)
    if (length(out) != n_support) {
      stop("'pi' must have length equal to nrow(prob.fun(par.vec)).", call. = FALSE)
    }
    if (anyNA(out) || any(!is.finite(out)) || any(out < 0) || sum(out) <= 0) {
      stop("'pi' must contain finite non-negative weights with a positive sum.", call. = FALSE)
    }
    out / sum(out)
  }
  pi <- get_pi(par.vec, nrow(prob_result), ...)

  if (response.type == "auto") {
    response.type <- if (ncol(prob_result) == ncol(response)) "binary" else "polytomous"
  }
  if (response.type == "binary" && ncol(prob_result) != ncol(response)) {
    stop("response.type = 'binary' requires prob.fun() to return one probability column per response column.", call. = FALSE)
  }
  if (response.type == "polytomous" && ncol(prob_result) != sum(K)) {
    stop("response.type = 'polytomous' requires prob.fun() to return stacked category probabilities.", call. = FALSE)
  }
  if (response.type == "nominal" && ncol(prob_result) != ncol(response)) {
    stop("response.type = 'nominal' requires prob.fun() to return one probability column per expanded indicator.", call. = FALSE)
  }

  if (response.type == "nominal") {
    nominal.groups <- as.integer(nominal.groups)
    if (length(nominal.groups) != I) {
      stop("'nominal.groups' must have one entry per response column.", call. = FALSE)
    }
    if (any(K > 2L)) {
      stop("response.type = 'nominal' requires expanded binary indicator responses.", call. = FALSE)
    }
    if (is.null(bivariate.groups)) {
      bivariate.groups <- nominal.groups
    }
  }
  if (!is.null(bivariate.groups)) {
    bivariate.groups <- as.integer(bivariate.groups)
    if (length(bivariate.groups) != I) {
      stop("'bivariate.groups' must have one entry per response column.", call. = FALSE)
    }
  }
  is_nominal <- response.type == "nominal"
  is_polytomous <- response.type == "polytomous"

  if (is_polytomous) {
    EI_list <- cpp_compute_EIs(prob_result, K)
    EIs     <- EI_list$EIs
    EIs2    <- EI_list$EIs2
  } else {
    EIs  <- prob_result
    EIs2 <- prob_result
  }

  # Filter out columns with zero observations (e.g. unobserved pairs in MOLE/PICK)
  col.has.obs <- colSums(!is.na(response)) > 0
  prob.col.has.obs <- NULL
  if (!all(col.has.obs)) {
    if (is_polytomous) {
      prob.col.has.obs <- rep(col.has.obs, times = K)
    } else {
      prob.col.has.obs <- col.has.obs
    }
    response <- response[, col.has.obs, drop = FALSE]
    EIs      <- EIs[, col.has.obs, drop = FALSE]
    EIs2     <- EIs2[, col.has.obs, drop = FALSE]
    K        <- K[col.has.obs]
    if (is_nominal) {
      nominal.groups <- nominal.groups[col.has.obs]
    }
    if (!is.null(bivariate.groups)) {
      bivariate.groups <- bivariate.groups[col.has.obs]
    }
    I        <- ncol(response)
  }

  observed.responses.n <- sum(!is.na(response))
  logLik <- loglik.fun(par.vec, ...)

  deviance <- -2 * logLik
  AIC <- deviance + 2 * npar
  AICc <- if (N > (npar + 1L)) {
    AIC + (2 * npar * (npar + 1L)) / (N - npar - 1L)
  } else {
    NA_real_
  }
  BIC <- deviance + log(N) * npar
  CAIC <- deviance + npar * (log(N) + 1)
  SABIC <- deviance + npar * log((N + 2) / 24)
  HQIC <- if (N > exp(1)) deviance + 2 * npar * log(log(N)) else NA_real_
  logLik.per.case <- logLik / N
  logLik.per.response <- if (observed.responses.n > 0L) logLik / observed.responses.n else NA_real_
  neg.logLik.per.case <- -logLik.per.case
  neg.logLik.per.response <- -logLik.per.response
  deviance.per.case <- deviance / N
  deviance.per.response <- if (observed.responses.n > 0L) deviance / observed.responses.n else NA_real_
  perplexity.per.case <- .gof_safe_exp(neg.logLik.per.case)
  perplexity.per.response <- .gof_safe_exp(neg.logLik.per.response)
  RMSEA.CI.level <- 1 - 2 * alpha

  # Compute all bivariate pairs
  pairs.total <- if (I > 1L) cpp_make_pairs(as.integer(I)) else matrix(integer(0), nrow = 2L, ncol = 0L)
  if (is_nominal && I > 1L && !is.null(pairs.total)) {
    pairs.total <- cpp_filter_pairs_between_groups(pairs.total, nominal.groups)
  }
  if (!is_nominal && !is.null(bivariate.groups) && I > 1L && !is.null(pairs.total)) {
    pairs.total <- cpp_filter_pairs_between_groups(pairs.total, bivariate.groups)
  }
  pairs.n <- ncol(pairs.total)

  moments.n <- I + pairs.n
  npar.per.case <- npar / N
  npar.per.moment <- npar / moments.n

  moments.predicted.func <- function(par.vec, ...) {
    prob <- prob.fun(par.vec, ...)
    prob <- pmin(pmax(prob, 1e-50), 1 - 1e-50)
    pi.current <- get_pi(par.vec, nrow(prob), ...)
    if (!is.null(prob.col.has.obs)) {
      prob <- prob[, prob.col.has.obs, drop = FALSE]
    }
    if (is_nominal) {
      return(cpp_moments_predicted_grouped(prob, pi.current, pairs.total, FALSE, K, nominal.groups))
    }
    cpp_moments_predicted(prob, pi.current, pairs.total, is_polytomous, K)
  }

  moments.predicted <- moments.predicted.func(par.vec, ...)

  nq <- nrow(EIs)
  pa_vec <- colSums(EIs * matrix(pi, nq, ncol(EIs), byrow=FALSE))

  # Xi via C++
  Xi_list <- if (is_nominal) {
    cpp_compute_Xi_grouped(EIs, EIs2, pi, pairs.total, nominal.groups)
  } else {
    cpp_compute_Xi(EIs, EIs2, pi, pairs.total)
  }
  Xi2 <- Xi_list$Xi2

  # Jacobian of predicted moments with respect to free parameters
  delta <- numDeriv::jacobian(moments.predicted.func, par.vec, ...)

  observed.summary <- cpp_gof_observed_summary(response, pairs.total)
  moments.observed <- observed.summary$moments
  cross <- observed.summary$cross
  cN    <- observed.summary$count
  mu    <- observed.summary$means
  R.obs <- stats::cov2cor(cross / cN - outer(mu, mu))

  E1 <- pa_vec
  E2 <- if (is_nominal) {
    cpp_compute_E2_grouped(EIs, EIs2, pi, nominal.groups)
  } else {
    cpp_compute_E2(EIs, EIs2, pi)
  }
  Kr <- stats::cov2cor(E2 - outer(E1, E1))
  lt <- lower.tri(R.obs)
  if (!is.null(bivariate.groups)) {
    lt <- lt & outer(bivariate.groups, bivariate.groups, "!=")
  }
  lt.n <- sum(lt, na.rm = TRUE)
  SRMSR <- if (lt.n > 0L) {
    sqrt(sum((R.obs[lt] - Kr[lt])^2, na.rm = TRUE) / lt.n)
  } else {
    NA_real_
  }
  correlation.residuals <- if (lt.n > 0L) R.obs[lt] - Kr[lt] else numeric(0)

  residuals <- moments.observed - moments.predicted
  residual.variance <- diag(Xi2)
  residuals.standardized <- sqrt(N) * residuals / sqrt(residual.variance)
  residuals.standardized[!is.finite(residuals.standardized)] <- NA_real_
  residuals.univariate <- residuals[seq_len(I)]
  residuals.bivariate <- if (pairs.n > 0L) residuals[I + seq_len(pairs.n)] else numeric(0)
  residuals.standardized.univariate <- residuals.standardized[seq_len(I)]
  residuals.standardized.bivariate <- if (pairs.n > 0L) {
    residuals.standardized[I + seq_len(pairs.n)]
  } else {
    numeric(0)
  }

  covariance.residuals <- numeric(0)
  if (pairs.n > 0L) {
    pair.idx1 <- pairs.total[1L, ]
    pair.idx2 <- pairs.total[2L, ]
    obs.pair.moments <- moments.observed[I + seq_len(pairs.n)]
    pred.pair.moments <- moments.predicted[I + seq_len(pairs.n)]
    obs.cov.pairs <- obs.pair.moments -
      moments.observed[pair.idx1] * moments.observed[pair.idx2]
    pred.cov.pairs <- pred.pair.moments - pa_vec[pair.idx1] * pa_vec[pair.idx2]
    covariance.residuals <- obs.cov.pairs - pred.cov.pairs
  }

  finite_summary <- function(x, fun) {
    x <- x[is.finite(x)]
    if (!length(x)) return(NA_real_)
    fun(x)
  }
  safe_ratio <- function(numerator, denominator) {
    if (!is.finite(numerator) || !is.finite(denominator) ||
        abs(denominator) < .Machine$double.eps) {
      return(NA_real_)
    }
    numerator / denominator
  }

  RMSR <- finite_summary(residuals, function(x) sqrt(mean(x^2)))
  RMSR.univariate <- finite_summary(residuals.univariate, function(x) sqrt(mean(x^2)))
  RMSR.bivariate <- finite_summary(residuals.bivariate, function(x) sqrt(mean(x^2)))
  mean.abs.residual <- finite_summary(abs(residuals), mean)
  mean.abs.residual.univariate <- finite_summary(abs(residuals.univariate), mean)
  mean.abs.residual.bivariate <- finite_summary(abs(residuals.bivariate), mean)
  max.abs.residual <- finite_summary(abs(residuals), max)
  max.abs.residual.univariate <- finite_summary(abs(residuals.univariate), max)
  max.abs.residual.bivariate <- finite_summary(abs(residuals.bivariate), max)
  RMS.standardized.residual <- finite_summary(residuals.standardized, function(x) sqrt(mean(x^2)))
  RMS.standardized.residual.univariate <- finite_summary(
    residuals.standardized.univariate, function(x) sqrt(mean(x^2))
  )
  RMS.standardized.residual.bivariate <- finite_summary(
    residuals.standardized.bivariate, function(x) sqrt(mean(x^2))
  )
  mean.abs.standardized.residual <- finite_summary(abs(residuals.standardized), mean)
  mean.abs.standardized.residual.univariate <- finite_summary(
    abs(residuals.standardized.univariate), mean
  )
  mean.abs.standardized.residual.bivariate <- finite_summary(
    abs(residuals.standardized.bivariate), mean
  )
  max.abs.standardized.residual <- finite_summary(abs(residuals.standardized), max)
  max.abs.standardized.residual.univariate <- finite_summary(
    abs(residuals.standardized.univariate), max
  )
  max.abs.standardized.residual.bivariate <- finite_summary(
    abs(residuals.standardized.bivariate), max
  )
  prop.abs.standardized.residual.gt.1.96 <- finite_summary(abs(residuals.standardized), function(x) mean(x > 1.96))
  prop.abs.standardized.residual.gt.2.58 <- finite_summary(abs(residuals.standardized), function(x) mean(x > 2.58))
  prop.abs.standardized.residual.gt.3.29 <- finite_summary(abs(residuals.standardized), function(x) mean(x > 3.29))
  mean.abs.correlation.residual <- finite_summary(abs(correlation.residuals), mean)
  max.abs.correlation.residual <- finite_summary(abs(correlation.residuals), max)
  q95.abs.correlation.residual <- finite_summary(abs(correlation.residuals), function(x) {
    as.numeric(stats::quantile(x, probs = 0.95, names = FALSE, type = 8))
  })
  SRMSCR <- finite_summary(covariance.residuals, function(x) sqrt(mean(x^2)))
  mean.abs.covariance.residual <- finite_summary(abs(covariance.residuals), mean)
  max.abs.covariance.residual <- finite_summary(abs(covariance.residuals), max)

  prob.posterior <- prob_result
  if (!is.null(prob.col.has.obs)) {
    prob.posterior <- prob.posterior[, prob.col.has.obs, drop = FALSE]
  }
  posterior.diagnostics <- .gof_posterior_diagnostics(
    prob = prob.posterior,
    response = response,
    pi = pi,
    response.type = response.type,
    K = K,
    nominal.groups = if (is_nominal) nominal.groups else NULL,
    EIs = EIs,
    pair.mask = lt
  )
  null.logLik <- function() {
    cpp_gof_null_loglik(
      response = response,
      K = K,
      response_type = response.type,
      nominal_groups = if (is_nominal) nominal.groups else integer(0),
      first_moments = moments.observed[1:I]
    )
  }

  logLik.nul <- null.logLik()
  deviance.null <- -2 * logLik.nul
  pseudo.R2 <- 1 - safe_ratio(logLik, logLik.nul)
  McFadden.R2 <- pseudo.R2
  McFadden.adj.R2 <- 1 - safe_ratio(logLik - npar, logLik.nul)
  CoxSnell.R2 <- if (is.finite(logLik.nul) && is.finite(logLik)) {
    1 - .gof_safe_exp((2 / N) * (logLik.nul - logLik))
  } else {
    NA_real_
  }
  Nagelkerke.R2 <- if (is.finite(CoxSnell.R2) && is.finite(logLik.nul)) {
    safe_ratio(CoxSnell.R2, 1 - .gof_safe_exp((2 / N) * logLik.nul))
  } else {
    NA_real_
  }

  LRT <- deviance.null - deviance
  LRT.df <- npar
  LRT.p <- if (LRT > 0 && LRT.df > 0L) 1 - stats::pchisq(LRT, LRT.df) else NA_real_
  G2.null <- LRT

  AldrichNelson.R2 <- if (is.finite(LRT) && is.finite(N) && LRT + N > 0)
    LRT / (LRT + N) else NA_real_
  VeallZimmermann.R2 <- if (is.finite(AldrichNelson.R2) && is.finite(logLik.nul))
    AldrichNelson.R2 * ((-2 * logLik.nul + N) / (-2 * logLik.nul)) else NA_real_

  first.mean.null <- moments.observed[1:I]
  first.second.null <- observed.summary$seconds
  first.var.null <- pmax(.Machine$double.xmin, first.second.null - first.mean.null^2)
  diag.Xi2.null <- first.var.null
  if (pairs.n > 0L) {
    pair.mean.null <- first.mean.null[pairs.total[1L, ]] * first.mean.null[pairs.total[2L, ]]
    pair.second.null <- first.second.null[pairs.total[1L, ]] * first.second.null[pairs.total[2L, ]]
    pair.var.null <- pmax(.Machine$double.xmin, pair.second.null - pair.mean.null^2)
    diag.Xi2.null <- c(diag.Xi2.null, pair.var.null)
  }

  moments.observed.null <- numeric(moments.n)
  moments.observed.null[1:I] <- moments.observed[1:I]
  if (I > 1L) {
    for (k in seq_len(pairs.n)) {
      ii <- pairs.total[1L, k]
      jj <- pairs.total[2L, k]
      moments.observed.null[I + k] <- moments.observed.null[ii] * moments.observed.null[jj]
    }
  }

  residuals.null <- moments.observed - moments.observed.null
  M2.null <- as.numeric(N * sum(residuals.null^2 / diag.Xi2.null))
  df.null <- moments.n

  extended.results <- list(
    logLik.per.response = logLik.per.response,
    neg.logLik.per.case = neg.logLik.per.case,
    neg.logLik.per.response = neg.logLik.per.response,
    deviance.per.case = deviance.per.case,
    deviance.per.response = deviance.per.response,
    perplexity.per.case = perplexity.per.case,
    perplexity.per.response = perplexity.per.response,
    observed.responses.n = observed.responses.n,
    npar.per.case = npar.per.case,
    npar.per.moment = npar.per.moment,
    RMSR.univariate = RMSR.univariate,
    RMSR.bivariate = RMSR.bivariate,
    mean.abs.residual.univariate = mean.abs.residual.univariate,
    mean.abs.residual.bivariate = mean.abs.residual.bivariate,
    max.abs.residual.univariate = max.abs.residual.univariate,
    max.abs.residual.bivariate = max.abs.residual.bivariate,
    RMS.standardized.residual.univariate = RMS.standardized.residual.univariate,
    RMS.standardized.residual.bivariate = RMS.standardized.residual.bivariate,
    mean.abs.standardized.residual.univariate = mean.abs.standardized.residual.univariate,
    mean.abs.standardized.residual.bivariate = mean.abs.standardized.residual.bivariate,
    max.abs.standardized.residual.univariate = max.abs.standardized.residual.univariate,
    max.abs.standardized.residual.bivariate = max.abs.standardized.residual.bivariate,
    q95.abs.correlation.residual = q95.abs.correlation.residual,
    SRMSCR = SRMSCR,
    mean.abs.covariance.residual = mean.abs.covariance.residual,
    max.abs.covariance.residual = max.abs.covariance.residual,
    pseudo.R2 = pseudo.R2,
    McFadden.R2 = McFadden.R2,
    McFadden.adj.R2 = McFadden.adj.R2,
    CoxSnell.R2 = CoxSnell.R2,
    Nagelkerke.R2 = Nagelkerke.R2,
    AldrichNelson.R2 = AldrichNelson.R2,
    VeallZimmermann.R2 = VeallZimmermann.R2,
    LRT = LRT,
    LRT.df = LRT.df,
    LRT.p = LRT.p,
    G2.null = G2.null,
    case.logLik = posterior.diagnostics$case.logLik,
    case.logLik.summary = posterior.diagnostics$case.logLik.summary,
    mean.max.posterior = posterior.diagnostics$mean.max.posterior,
    median.max.posterior = posterior.diagnostics$median.max.posterior,
    min.max.posterior = posterior.diagnostics$min.max.posterior,
    q3.mean = posterior.diagnostics$q3.mean,
    q3.max = posterior.diagnostics$q3.max,
    q3.abs.max = posterior.diagnostics$q3.abs.max,
    q3.p95 = posterior.diagnostics$q3.p95,
    q3.adjusted.max = posterior.diagnostics$q3.adjusted.max,
    q3.adjusted.abs.max = posterior.diagnostics$q3.adjusted.abs.max,
    q3.adjusted.p95 = posterior.diagnostics$q3.adjusted.p95,
    covariance.residuals = covariance.residuals
  )

  delta.qr <- qr(delta)
  if ((ncol(delta) + 1L) > ncol(qr.Q(delta.qr, complete = TRUE))) {
    warning('M2 cannot be calculated: too few degrees of freedom. M2 and related indices set to NA.', call. = FALSE)

    results <- list(
      LogLik            = logLik,
      logLik.per.case   = logLik.per.case,
      deviance          = deviance,
      npar              = npar,
      N                 = N,
      nitems            = I,
      response.type     = response.type,
      moments.n         = moments.n,
      pairs.n           = pairs.n,
      AIC               = AIC,
      AICc              = AICc,
      BIC               = BIC,
      CAIC              = CAIC,
      SABIC             = SABIC,
      HQIC              = HQIC,
      M2.value          = NA_real_,
      df                = NA_real_,
      p.value           = NA_real_,
      RMSEA2            = NA_real_,
      RMSEA.lower       = NA_real_,
      RMSEA.upper       = NA_real_,
      RMSEA.CI.level    = RMSEA.CI.level,
      RMSEA.p.close     = NA_real_,
      SRMSR             = SRMSR,
      RMSR              = RMSR,
      mean.abs.residual = mean.abs.residual,
      max.abs.residual  = max.abs.residual,
      RMS.standardized.residual = RMS.standardized.residual,
      mean.abs.standardized.residual = mean.abs.standardized.residual,
      max.abs.standardized.residual = max.abs.standardized.residual,
      prop.abs.standardized.residual.gt.1.96 = prop.abs.standardized.residual.gt.1.96,
      prop.abs.standardized.residual.gt.2.58 = prop.abs.standardized.residual.gt.2.58,
      prop.abs.standardized.residual.gt.3.29 = prop.abs.standardized.residual.gt.3.29,
      mean.abs.correlation.residual = mean.abs.correlation.residual,
      max.abs.correlation.residual = max.abs.correlation.residual,
      pseudo.R2               = pseudo.R2,
      AldrichNelson.R2        = AldrichNelson.R2,
      VeallZimmermann.R2      = VeallZimmermann.R2,
      CFI               = NA_real_,
      TLI               = NA_real_,
      IFI               = NA_real_,
      McDonald.NCI      = NA_real_,
      null.LogLik       = logLik.nul,
      null.deviance     = deviance.null,
      null.M2.value     = M2.null,
      null.df           = df.null,
      LRT               = LRT,
      LRT.df            = LRT.df,
      LRT.p             = LRT.p,
      G2.null           = G2.null,
      residuals         = residuals,
      residuals.standardized = residuals.standardized,
      correlation.residuals = correlation.residuals,
      moments.observed  = moments.observed,
      moments.predicted = moments.predicted
    )

    for (nm in names(extended.results)) {
      results[[nm]] <- extended.results[[nm]]
    }
    class(results) <- "good.of.fit"
    return(results)
  }
  deltac <- qr.Q(delta.qr, complete = TRUE)[, (ncol(delta) + 1L):ncol(qr.Q(delta.qr, complete = TRUE)), drop = FALSE]

  inner <- crossprod(deltac, Xi2 %*% deltac)
  C2 <- tryCatch(
    deltac %*% solve(inner) %*% t(deltac),
    error = function(e) {
      deltac %*% MASS::ginv(inner) %*% t(deltac)
    }
  )

  M2.value <- as.numeric(abs(N * crossprod(residuals, C2 %*% residuals)))

  df <- moments.n - npar
  p.value <- 1 - stats::pchisq(M2.value, df)
  RMSEA2 <- sqrt(max(0, M2.value - df) / (N * df))
  RMSEA2.CI.value <- RMSEA.CI(M2.value, df, N, ci.lower = alpha, ci.upper = 1-alpha)
  RMSEA.lower <- RMSEA2.CI.value[1]
  RMSEA.upper <- RMSEA2.CI.value[2]
  RMSEA.p.close <- 1 - stats::pchisq(
    M2.value, df = df, ncp = N * df * 0.05^2
  )
  McDonald.NCI <- exp(-max(M2.value - df, 0) / (2 * N))

  CFI <- NA_real_; TLI <- NA_real_
  if (M2.null > M2.value) {
    CFI <- min(1, max(0, 1 - (M2.value - df) / (M2.null - df.null)))
    TLI <- (M2.null / df.null - M2.value / df) / (M2.null / df.null - 1)
  } else {
    warning('Null model M2 <= fitted model M2; CFI/TLI not computed.', call. = FALSE)
  }
  IFI <- safe_ratio(M2.null - M2.value, M2.null - df)

  results <- list(
    LogLik            = logLik,
    logLik.per.case   = logLik.per.case,
    deviance          = deviance,
    npar              = npar,
    N                 = N,
    nitems            = I,
    response.type     = response.type,
    moments.n         = moments.n,
    pairs.n           = pairs.n,
    AIC               = AIC,
    AICc              = AICc,
    BIC               = BIC,
    CAIC              = CAIC,
    SABIC             = SABIC,
    HQIC              = HQIC,
    M2.value          = M2.value,
    df                = df,
    p.value           = p.value,
    RMSEA2            = RMSEA2,
    RMSEA.lower       = RMSEA.lower,
    RMSEA.upper       = RMSEA.upper,
    RMSEA.CI.level    = RMSEA.CI.level,
    RMSEA.p.close     = RMSEA.p.close,
    SRMSR             = SRMSR,
    RMSR              = RMSR,
    mean.abs.residual = mean.abs.residual,
    max.abs.residual  = max.abs.residual,
    RMS.standardized.residual = RMS.standardized.residual,
    mean.abs.standardized.residual = mean.abs.standardized.residual,
    max.abs.standardized.residual = max.abs.standardized.residual,
    prop.abs.standardized.residual.gt.1.96 = prop.abs.standardized.residual.gt.1.96,
    prop.abs.standardized.residual.gt.2.58 = prop.abs.standardized.residual.gt.2.58,
    prop.abs.standardized.residual.gt.3.29 = prop.abs.standardized.residual.gt.3.29,
    mean.abs.correlation.residual = mean.abs.correlation.residual,
    max.abs.correlation.residual = max.abs.correlation.residual,
    pseudo.R2               = pseudo.R2,
    AldrichNelson.R2        = AldrichNelson.R2,
    VeallZimmermann.R2      = VeallZimmermann.R2,
    CFI               = CFI,
    TLI               = TLI,
    IFI               = IFI,
    McDonald.NCI      = McDonald.NCI,
    null.LogLik       = logLik.nul,
    null.deviance     = deviance.null,
    null.M2.value     = M2.null,
    null.df           = df.null,
    LRT               = LRT,
    LRT.df            = LRT.df,
    LRT.p             = LRT.p,
    G2.null           = G2.null,
    residuals         = residuals,
    residuals.standardized = residuals.standardized,
    correlation.residuals = correlation.residuals,
    moments.observed  = moments.observed,
    moments.predicted = moments.predicted
  )

  for (nm in names(extended.results)) {
    results[[nm]] <- extended.results[[nm]]
  }
  class(results) <- "good.of.fit"

  return(results)
}

RMSEA.CI <- function(M2, df, N, ci.lower = 0.05, ci.upper = 0.95){

  lower.lambda <- function(lambda) {
    stats::pchisq(M2, df = df, ncp = lambda) - ci.upper
  }
  upper.lambda <- function(lambda) {
    stats::pchisq(M2, df = df, ncp = lambda) - ci.lower
  }

  lambda.l <- try(
    stats::uniroot(f = lower.lambda, lower = 0, upper = M2)$root,
    silent = TRUE
  )
  lambda.u <- try(
    stats::uniroot(
      f = upper.lambda, lower = 0, upper = max(N, M2 * 5)
    )$root,
    silent = TRUE
  )

  if (!is(lambda.l, "try-error")) {
    RMSEA.lower <- sqrt(lambda.l/(N * df))
  }else {
    RMSEA.lower <- 0
  }

  if (!is(lambda.u, "try-error")) {
    RMSEA.upper <- sqrt(lambda.u/(N * df))
  }else {
    RMSEA.upper <- 0
  }

  return(c(RMSEA.lower, RMSEA.upper))
}

.gof_safe_exp <- function(x) {
  out <- rep(NA_real_, length(x))
  finite <- is.finite(x)
  out[finite] <- exp(pmin(x[finite], log(.Machine$double.xmax)))
  out[is.infinite(x) & x < 0] <- 0
  if (length(out) == 1L) out[[1L]] else out
}

.gof_log_sum_exp <- function(x) {
  m <- max(x, na.rm = TRUE)
  if (!is.finite(m)) {
    return(m)
  }
  m + log(sum(exp(x - m)))
}

.gof_summary6 <- function(x) {
  x <- x[is.finite(x)]
  if (!length(x)) {
    return(c(Min = NA_real_, Q1 = NA_real_, Median = NA_real_,
             Mean = NA_real_, Q3 = NA_real_, Max = NA_real_))
  }
  c(
    Min = min(x),
    Q1 = as.numeric(stats::quantile(x, 0.25, names = FALSE, type = 8)),
    Median = stats::median(x),
    Mean = mean(x),
    Q3 = as.numeric(stats::quantile(x, 0.75, names = FALSE, type = 8)),
    Max = max(x)
  )
}

.gof_empty_posterior_diagnostics <- function() {
  list(
    case.logLik = numeric(0),
    case.logLik.summary = .gof_summary6(numeric(0)),
    mean.max.posterior = NA_real_,
    median.max.posterior = NA_real_,
    min.max.posterior = NA_real_,
    q3.mean = NA_real_,
    q3.max = NA_real_,
    q3.abs.max = NA_real_,
    q3.p95 = NA_real_,
    q3.adjusted.max = NA_real_,
    q3.adjusted.abs.max = NA_real_,
    q3.adjusted.p95 = NA_real_
  )
}

.gof_case_loglik <- function(prob, response, pi, response.type, K,
                             nominal.groups = NULL) {
  eps <- 1e-50
  prob <- pmin(pmax(prob, eps), 1 - eps)
  pi <- as.vector(pi)
  pi <- pmax(pi, eps)
  pi <- pi / sum(pi)
  N <- nrow(response)
  nq <- nrow(prob)
  I <- ncol(response)
  log.joint <- matrix(log(pi), nrow = N, ncol = nq, byrow = TRUE)

  if (response.type == "binary") {
    log.p <- log(prob)
    log.q <- log1p(-prob)
    for (j in seq_len(I)) {
      y <- response[, j]
      idx1 <- which(!is.na(y) & y >= 0.5)
      idx0 <- which(!is.na(y) & y < 0.5)
      if (length(idx1)) {
        log.joint[idx1, ] <- log.joint[idx1, , drop = FALSE] +
          matrix(log.p[, j], nrow = length(idx1), ncol = nq, byrow = TRUE)
      }
      if (length(idx0)) {
        log.joint[idx0, ] <- log.joint[idx0, , drop = FALSE] +
          matrix(log.q[, j], nrow = length(idx0), ncol = nq, byrow = TRUE)
      }
    }
  } else if (response.type == "polytomous") {
    offsets <- cumsum(c(0L, K[-length(K)]))
    for (j in seq_len(I)) {
      y <- response[, j]
      bad <- which(!is.na(y) & (y < 0 | y >= K[j] | y != floor(y)))
      if (length(bad)) {
        log.joint[bad, ] <- -Inf
      }
      for (cat in seq_len(K[j]) - 1L) {
        idx <- which(!is.na(y) & y == cat)
        if (length(idx)) {
          col <- offsets[j] + cat + 1L
          log.joint[idx, ] <- log.joint[idx, , drop = FALSE] +
            matrix(log(prob[, col]), nrow = length(idx), ncol = nq, byrow = TRUE)
        }
      }
    }
  } else if (response.type == "nominal") {
    groups <- unique(nominal.groups)
    for (g in groups) {
      cols <- which(nominal.groups == g)
      yy <- response[, cols, drop = FALSE]
      complete <- rowSums(is.na(yy)) == 0L
      active <- rowSums(yy == 1, na.rm = TRUE)
      invalid <- which(complete & active > 1L)
      if (length(invalid)) {
        log.joint[invalid, ] <- -Inf
      }
      idx0 <- which(complete & active == 0L)
      if (length(idx0)) {
        p0 <- pmax(eps, 1 - rowSums(prob[, cols, drop = FALSE]))
        log.joint[idx0, ] <- log.joint[idx0, , drop = FALSE] +
          matrix(log(p0), nrow = length(idx0), ncol = nq, byrow = TRUE)
      }
      for (h in seq_along(cols)) {
        idx <- which(complete & active == 1L & yy[, h] == 1)
        if (length(idx)) {
          log.joint[idx, ] <- log.joint[idx, , drop = FALSE] +
            matrix(log(prob[, cols[h]]), nrow = length(idx), ncol = nq, byrow = TRUE)
        }
      }
    }
  } else {
    stop("Unsupported response.type for posterior diagnostics.", call. = FALSE)
  }

  case.logLik <- apply(log.joint, 1L, .gof_log_sum_exp)
  posterior <- matrix(NA_real_, nrow = N, ncol = nq)
  valid <- is.finite(case.logLik)
  if (any(valid)) {
    posterior[valid, ] <- exp(sweep(log.joint[valid, , drop = FALSE],
                                    1L, case.logLik[valid], "-"))
  }
  list(case.logLik = case.logLik, posterior = posterior)
}

.gof_posterior_diagnostics <- function(prob, response, pi, response.type, K,
                                       nominal.groups, EIs, pair.mask) {
  out <- .gof_empty_posterior_diagnostics()
  case <- tryCatch(
    .gof_case_loglik(prob, response, pi, response.type, K, nominal.groups),
    error = function(e) NULL
  )
  if (is.null(case)) {
    return(out)
  }

  posterior <- case$posterior
  valid <- rowSums(is.na(posterior)) == 0L
  out$case.logLik <- case$case.logLik
  out$case.logLik.summary <- .gof_summary6(case$case.logLik)
  if (!any(valid)) {
    return(out)
  }

  post.valid <- posterior[valid, , drop = FALSE]
  max.post <- apply(post.valid, 1L, max)
  out$mean.max.posterior <- mean(max.post)
  out$median.max.posterior <- stats::median(max.post)
  out$min.max.posterior <- min(max.post)

  if (ncol(response) > 1L && ncol(EIs) == ncol(response)) {
    expected <- posterior %*% EIs
    residual.matrix <- response - expected
    residual.matrix[is.na(response)] <- NA_real_
    q3 <- suppressWarnings(stats::cor(residual.matrix, use = "pairwise.complete.obs"))
    q3.values <- q3[pair.mask]
    q3.values <- q3.values[is.finite(q3.values)]
    if (length(q3.values)) {
      q3.adjusted <- q3.values - mean(q3.values)
      out$q3.mean <- mean(q3.values)
      out$q3.max <- max(q3.values)
      out$q3.abs.max <- max(abs(q3.values))
      out$q3.p95 <- as.numeric(stats::quantile(q3.values, 0.95, names = FALSE, type = 8))
      out$q3.adjusted.max <- max(q3.adjusted)
      out$q3.adjusted.abs.max <- max(abs(q3.adjusted))
      out$q3.adjusted.p95 <- as.numeric(stats::quantile(q3.adjusted, 0.95, names = FALSE, type = 8))
    }
  }

  out
}

#' Simulate Data from the Multidimensional Generalized Graded Unfolding Model
#'
#' @description
#' Generates polytomous responses from the same distance-based MGGUM
#' probability function used by \code{\link{model.MGGUM}} and
#' \code{\link{fit.MGGUM}}. For each item, active discriminations are drawn
#' from \eqn{U(0.5, 2)}, active locations follow the sign of the Q-matrix,
#' \eqn{\tau_{i0}=0}, and the remaining \eqn{\tau} values are ordered
#' negative thresholds.
#'
#' @section Data Generation Process:
#' \enumerate{
#'   \item Draw latent traits from
#'         \eqn{N_D(\mathbf{0}, \boldsymbol{\Sigma})}.
#'   \item Generate discrimination, location, and threshold parameters under
#'         the sign constraints encoded by \code{Q.matrix}.
#'   \item Compute stacked category probabilities with
#'         \code{\link{model.MGGUM}}.
#'   \item Sample one ordinal response per person and item. A category can be
#'         absent in a finite sample; \code{length.poly} retains the intended
#'         support.
#' }
#'
#' @param N Integer; number of examinees (default: 500).
#' @param I Integer; number of items (default: 20).
#' @param D Integer; number of latent dimensions (default: 2).
#' @param length.poly Integer vector or scalar indicating the number of
#'   categories for each item. A scalar is recycled to length \eqn{I}.
#' @param Q.matrix Optional \eqn{I \times D} matrix with values -1, 0, or 1.
#'   \itemize{
#'     \item \code{1}: active dimension with positive-side \code{delta}
#'     \item \code{-1}: active dimension with negative-side \code{delta}
#'     \item \code{0}: \code{a} and \code{delta} fixed to 0 (inactive dimension)
#'   }
#'   The sign controls the item-location side, not the sign of discrimination.
#'   If \code{NULL}, a default matrix of randomly signed active entries is used.
#' @param Corr Optional \eqn{D \times D} correlation matrix. If \code{NULL},
#'   the identity matrix is used.
#'
#' @return An object of class \code{"data.MGGUM"}, a list containing:
#' \describe{
#'   \item{\code{data}, \code{response}}{\eqn{N \times I} integer response
#'         matrices with categories coded from 0 to \eqn{K_i - 1}.}
#'   \item{\code{theta}}{\eqn{N \times D} matrix of true latent traits.}
#'   \item{\code{par}}{\eqn{I \times (2D + \max_i K_i)} matrix of GGUM item
#'         parameters: discrimination columns, location columns, and stacked
#'         threshold columns.}
#'   \item{\code{probability}}{\eqn{N \times \sum_i K_i} matrix of stacked
#'         category probabilities.}
#'   \item{\code{Q.matrix}, \code{length.poly}, \code{Corr}}{Design matrix,
#'         category counts, and latent correlation matrix used to generate
#'         the data.}
#'   \item{\code{N}, \code{I}, \code{D}, \code{call}}{Data-generating
#'         metadata.}
#' }
#'
#' @seealso \code{\link{fit.MGGUM}}, \code{\link{model.MGGUM}}
#'
#' @examples
#' set.seed(123)
#' sim <- sim.data.MGGUM(N = 20, I = 5, D = 2, length.poly = 4)
#' str(sim$response)
#' dim(sim$par)
#' table(sim$response[, 1])
#'
#' @export
sim.data.MGGUM <- function(N = 500, I = 20, D = 2, length.poly = 5,
                           Q.matrix = NULL, Corr = NULL) {
  call <- match.call()

  N <- check_integer_scalar(N, "N")
  I <- check_integer_scalar(I, "I")
  D <- check_integer_scalar(D, "D")
  length.poly <- normalize_length_poly(length.poly, I)
  max_poly <- max(length.poly)

  Corr <- validate_corr_matrix(Corr, D)

  if (is.null(Q.matrix)) {
    Q.matrix <- matrix(sample(c(-1, 1), I * D, replace = TRUE), I, D)
  }
  Q.matrix <- istem_prepare_signed_q(I, D, Q.matrix)

    theta <- generate_theta_mvn(N, D, Corr)

    # Generate a parameters respecting Q.matrix
    # a ~ U(0.50, 2.00), free when |Q| = 1, fixed to 0 when Q = 0
    a <- matrix(runif(I * D, 0.5, 2.0), I, D) * abs(Q.matrix)

    # par matrix: I x (D + D + max_poly)
    # Columns: a1..aD | delta1..deltaD | tau1(=0) tau2..tau_{max_poly}
    par <- matrix(NA, nrow = I, ncol = D + D + max_poly)
    par[, 1:D] <- a
    mask <- matrix(NA, nrow = I, ncol = D + D + max_poly)
    mask[, 1:D] <- 1
    for (i in seq_len(I)) {

      # delta parameters: sign determined by Q.matrix, magnitude ~ U(0, 2.00)
      for (d in seq_len(D)) {
        if (Q.matrix[i, d] == 1) {
          par[i, D + d] <- runif(1, 0.0, 2.0)
        } else if (Q.matrix[i, d] == -1) {
          par[i, D + d] <- runif(1, -2.0, 0.0)
        } else {
          par[i, D + d] <- 0.0
        }
      }
      mask[i, (D + 1):(D + D)] <- 1

      # tau parameters: tau0 = 0 (fixed), tau1 < tau2 < ... < tau_{K-1} < 0
      par[i, D + D + 1] <- 0.0
      mask[i, D + D + 1] <- 1
      Ki <- length.poly[i]
      if (Ki > 1) {
        Kf <- Ki - 1  # number of free tau parameters
        tau_free <- numeric(Kf)
        for (t in seq_len(Kf)) {
          lo <- -2.00 + (t - 1) * 2.0 / Kf
          hi <- -2.00 +  t      * 2.0 / Kf
          tau_free[t] <- runif(1, lo, hi)
        }
        # Sort ascending to ensure tau1 < tau2 < ... < tau_{K-1} < 0
        tau_sorted <- sort(tau_free, decreasing = FALSE)
        par[i, (D + D + 2):(D + D + Ki)] <- tau_sorted
        mask[i, (D + D + 2):(D + D + Ki)] <- 1
      }
    }

    prob <- model.MGGUM(theta = theta, par = par)
    response <- matrix(0, N, I)
    idx <- 0
    for (i in seq_len(I)) {
      P.i <- prob[, (idx + 1):(idx + length.poly[i]), drop = FALSE]
      idx <- idx + length.poly[i]
      cdf <- t(apply(P.i, 1L, cumsum))
      response[, i] <- pmin(
        rowSums(cdf < stats::runif(N)), length.poly[i] - 1L
      )
    }

  if (D > 1) {
    colnames(a) <- paste0("a", seq_len(D))
  }
  colnames(par) <- c(paste0("a", seq_len(D)),
                     paste0("delta", seq_len(D)),
                     paste0("tau", 0:(max_poly - 1)))
  rownames(par) <- paste0("item", seq_len(I))
  colnames(response) <- paste0("item", seq_len(I))

  ak <- matrix(0:(max_poly - 1), I, max_poly, byrow = TRUE)
  colnames(ak) <- paste0("ak", 0:(max_poly - 1))
  par.mirt <- cbind(par[, 1:D, drop = FALSE],
                    par[, (D + 1):(D + D), drop = FALSE],
                    ak,
                    par[, (D + D + 1):ncol(par), drop = FALSE])

  data.obj <- list(
    data        = response,
    response    = response,
    theta       = theta,
    par         = par,
    par.mirt    = par.mirt,
    Q.matrix    = Q.matrix,
    block.items = NULL,
    Corr        = Corr,
    length.poly = length.poly,
    mask        = mask,
    probability = prob,
    N           = N,
    I           = I,
    D           = D,
    call        = call,
    arguments   = list(
      N           = N,
      I           = I,
      D           = D,
      length.poly = length.poly,
      Q.matrix    = Q.matrix,
      Corr        = Corr
    )
  )

  class(data.obj) <- "data.MGGUM"

  return(data.obj)
}

#' Compute MGGUM Category Response Probabilities
#'
#' @param theta \eqn{N \times D} matrix of latent trait values.
#' @param par \eqn{I \times (2D + \max K_i)} parameter matrix with columns
#'   \code{a1..aD, delta1..deltaD, tau0, tau1, ...}. Missing category
#'   columns for shorter items should be \code{NA}; \code{tau0} is fixed at 0.
#' @return An \eqn{N \times \sum_i K_i} matrix of stacked category
#'   probabilities. The implementation uses
#'   \eqn{r_{ij} = [\sum_d a_{id}^2(\theta_{jd}-\delta_{id})^2]^{1/2}}
#'   and
#'   \eqn{P(Y_{ij}=k) \propto
#'   \exp\{k r_{ij}-\psi_{ik}\} +
#'   \exp\{(2K_i-1-k)r_{ij}-\psi_{ik}\}}.
#' @keywords internal
model.MGGUM <- function(theta, par){
  cpp_model_MGGUM(theta, par)
}

#' @title S3 Methods: predict
#'
#' @description
#' Predict response probabilities from fitted \pkg{ForceChoice} model
#' objects. Given a fitted model and optionally new person parameter
#' (latent trait) values, returns the model-implied response probabilities.
#' When \code{newdata} is not provided, predictions are made at the
#' estimated latent trait values for each person.
#'
#' @param object A fitted model object of class \code{"MIRT"},
#'   \code{"MGPCM"}, \code{"MGGUM"}, \code{"FCMIRT"}, \code{"FCDCM"},
#'   \code{"FCGDINA"}, \code{"FCGGUM"}, or \code{"TIRT"}.
#' @param newdata An optional \eqn{M \times D} matrix of latent trait
#'   values at which to compute predictions. If \code{NULL} (default),
#'   predictions use the estimated \code{object$theta$est}.
#' @param type Character; \code{"probability"} (default) returns
#'   response probabilities, \code{"response"} returns expected category
#'   or simulated binary responses (random draws).
#' @param ... Additional arguments (currently ignored).
#'
#' @return
#' \describe{
#'   \item{\code{type = "probability"}}{For binary models (MIRT, TIRT):
#'     an \eqn{M \times I} matrix of endorsement probabilities. For
#'     polytomous models (MGPCM, MGGUM): an \eqn{M \times \sum K_i}
#'     matrix of stacked category probabilities. For forced-choice
#'     models (FCMIRT, FCGGUM): an \eqn{M \times \sum K_b} matrix of
#'     block-pattern probabilities. For FCDCM: an \eqn{M \times B}
#'     matrix of marginal block-choice probabilities.}
#'   \item{\code{type = "response"}}{For binary models: an
#'     \eqn{M \times I} matrix of simulated 0/1 responses. For
#'     polytomous models: an \eqn{M \times I} matrix of category
#'     indices. For forced-choice models: an \eqn{M \times B} matrix
#'     of chosen pattern indices.}
#' }
#'
#' @name predict
NULL


# ---- Internal helpers ----

.fc_predict_prob <- function(object, newdata) {
  cls <- class(object)[1L]
  if (cls == "FCGDINA") {
    prep <- fcgdina_prepare(
      data = object$response,
      Q.matrix = object$Q.matrix,
      model = object$model,
      block.items = object$block.items,
      fc.type = object$fc.type
    )
    prob.class <- fcgdina_prob_class_link(
      prep, object$delta$est, link = fcgdina_delta_link(object))
    if (is.null(newdata)) {
      class.prob <- object$class.post
    } else {
      nd <- as.matrix(newdata)
      if (ncol(nd) == prep$D && all(nd %in% c(0, 1))) {
        key <- pattern_key(nd)
        cls.idx <- match(key, pattern_key(prep$alpha.patterns))
        if (anyNA(cls.idx)) stop("'newdata' contains invalid alpha patterns.",
                                 call. = FALSE)
        class.prob <- matrix(0, nrow(nd), prep$C)
        class.prob[cbind(seq_len(nrow(nd)), cls.idx)] <- 1
      } else if (ncol(nd) == prep$C) {
        rs <- rowSums(nd)
        if (any(!is.finite(rs)) || any(rs <= 0)) {
          stop("'newdata' class-probability rows must have positive sums.",
               call. = FALSE)
        }
        class.prob <- nd / rs
      } else {
        stop("'newdata' for FCGDINA must be an M x D binary alpha matrix ",
             "or an M x 2^D class-probability matrix.", call. = FALSE)
      }
    }
    return(class.prob %*% prob.class)
  }

  theta <- if (is.null(newdata)) as.matrix(object$theta$est) else as.matrix(newdata)
  switch(cls,
    MIRT   = model.MIRT(theta, as.matrix(object$par$est)),
    MGPCM  = model.MGPCM(theta, as.matrix(object$par$est)),
    MGGUM  = model.MGGUM(theta, as.matrix(object$par$est)),
    FCMIRT = model.FCMIRT(theta, as.matrix(object$par$est),
                          object$patterns.total, object$patterns),
    FCDCM  = {
      alpha_prob <- fcdcm_alpha_profile_prob(
        theta = theta,
        delta1 = object$delta$est[, "delta1"],
        delta0 = object$delta$est[, "delta0"],
        alpha.patterns = object$alpha.patterns
      )
      prob_per_pattern <- model.FCDCM(
        zeta = object$zeta.patterns,
        par = object$par$est,
        patterns = object$patterns
      )
      alpha_prob %*% prob_per_pattern
    },
    FCGGUM = model.FCGGUM(theta, as.matrix(object$par$est),
                          object$patterns.total, object$patterns),
    TIRT   = {
      par <- as.matrix(object$par$est)
      D <- ncol(par) - 1L
      lambda <- unlist(lapply(seq_len(nrow(par)), function(x) {
        d <- which(object$Q.matrix[x, ] != 0)[1]
        return(par[x, d])
      }))
      psi2 <- par[, D + 1L]
      model.TIRT(theta, lambda, psi2, object$gamma.matrix$est,
                 object$Q.matrix, object$pairs.matrix)
    }
  )
}

.fc_predict_response_binary <- function(prob) {
  matrix(as.integer(runif(length(prob)) < prob), nrow = nrow(prob),
         ncol = ncol(prob), dimnames = dimnames(prob))
}

.fc_predict_response_polytomous <- function(prob, length.poly) {
  N <- nrow(prob)
  I <- length(length.poly)
  resp <- matrix(0L, N, I)
  idx <- 1L
  for (i in seq_len(I)) {
    Ki <- length.poly[i]
    prob_i <- prob[, idx:(idx + Ki - 1L), drop = FALSE]
    cum_prob <- t(apply(prob_i, 1L, cumsum))
    u <- runif(N)
    resp[, i] <- rowSums(u > cum_prob)
    idx <- idx + Ki
  }
  resp
}

.fc_predict_response_fc <- function(prob, patterns) {
  N <- nrow(prob)
  B <- length(patterns)
  n_cat <- vapply(patterns, nrow, integer(1L))
  resp <- matrix(0L, N, B)
  idx <- 1L
  for (b in seq_len(B)) {
    Kb <- n_cat[b]
    prob_b <- prob[, idx:(idx + Kb - 1L), drop = FALSE]
    cum_prob <- t(apply(prob_b, 1L, cumsum))
    u <- runif(N)
    resp[, b] <- rowSums(u > cum_prob) + 1L
    idx <- idx + Kb
  }
  resp
}


# ===========================================================================
# MIRT
# ===========================================================================

#' @describeIn predict Predict response probabilities or simulated
#'   responses from \code{MIRT} objects.
#' @method predict MIRT
#' @export
predict.MIRT <- function(object, newdata = NULL,
                         type = c("probability", "response"), ...) {
  type <- match.arg(type)
  prob <- .fc_predict_prob(object, newdata)
  switch(type,
    probability = prob,
    response    = .fc_predict_response_binary(prob)
  )
}

# ===========================================================================
# MGPCM
# ===========================================================================

#' @describeIn predict Predict response probabilities from \code{MGPCM} objects.
#' @method predict MGPCM
#' @export
predict.MGPCM <- function(object, newdata = NULL,
                          type = c("probability", "response"), ...) {
  type <- match.arg(type)
  prob <- .fc_predict_prob(object, newdata)
  switch(type,
    probability = prob,
    response    = .fc_predict_response_polytomous(prob, object$length.poly)
  )
}

# ===========================================================================
# MGGUM
# ===========================================================================

#' @describeIn predict Predict response probabilities from \code{MGGUM} objects.
#' @method predict MGGUM
#' @export
predict.MGGUM <- function(object, newdata = NULL,
                          type = c("probability", "response"), ...) {
  type <- match.arg(type)
  prob <- .fc_predict_prob(object, newdata)
  switch(type,
    probability = prob,
    response    = .fc_predict_response_polytomous(prob, object$length.poly)
  )
}

# ===========================================================================
# FCMIRT
# ===========================================================================

#' @describeIn predict Predict response probabilities from \code{FCMIRT} objects.
#' @method predict FCMIRT
#' @export
predict.FCMIRT <- function(object, newdata = NULL,
                           type = c("probability", "response"), ...) {
  type <- match.arg(type)
  prob <- .fc_predict_prob(object, newdata)
  switch(type,
    probability = prob,
    response    = .fc_predict_response_fc(prob, object$patterns)
  )
}

# ===========================================================================
# FCDCM
# ===========================================================================

#' @describeIn predict Predict response probabilities from \code{FCDCM} objects.
#' @method predict FCDCM
#' @export
predict.FCDCM <- function(object, newdata = NULL,
                          type = c("probability", "response"), ...) {
  type <- match.arg(type)
  prob <- .fc_predict_prob(object, newdata)
  switch(type,
    probability = prob,
    response    = .fc_predict_response_binary(prob)
  )
}

# ===========================================================================
# FCGDINA
# ===========================================================================

#' @describeIn predict Predict response probabilities from \code{FCGDINA} objects.
#'   For \code{newdata}, provide either binary alpha profiles or class
#'   probability rows.
#' @method predict FCGDINA
#' @export
predict.FCGDINA <- function(object, newdata = NULL,
                            type = c("probability", "response"), ...) {
  type <- match.arg(type)
  prob <- .fc_predict_prob(object, newdata)
  switch(type,
    probability = prob,
    response    = .fc_predict_response_fc(prob, object$patterns)
  )
}

# ===========================================================================
# FCGGUM
# ===========================================================================

#' @describeIn predict Predict response probabilities from \code{FCGGUM} objects.
#' @method predict FCGGUM
#' @export
predict.FCGGUM <- function(object, newdata = NULL,
                           type = c("probability", "response"), ...) {
  type <- match.arg(type)
  prob <- .fc_predict_prob(object, newdata)
  switch(type,
    probability = prob,
    response    = .fc_predict_response_fc(prob, object$patterns)
  )
}

# ===========================================================================
# TIRT
# ===========================================================================

#' @describeIn predict Predict response probabilities from \code{TIRT} objects.
#' @method predict TIRT
#' @export
predict.TIRT <- function(object, newdata = NULL,
                         type = c("probability", "response"), ...) {
  type <- match.arg(type)
  prob <- .fc_predict_prob(object, newdata)
  switch(type,
    probability = prob,
    response    = .fc_predict_response_binary(prob)
  )
}

################################# fit shared helpers #################################

fc_as_control_list <- function(control, arg) {
  if (is.null(control)) {
    return(list())
  }
  if (!is.list(control)) {
    stop("'", arg, "' must be NULL or a named list.", call. = FALSE)
  }
  if (length(control) > 0L) {
    control_names <- names(control)
    if (is.null(control_names) || any(!nzchar(control_names))) {
      stop("'", arg, "' must be a named list.", call. = FALSE)
    }
  }
  control
}


################################# model name aliases #################################

model_aliases <- list(
  "rasch" = "m1pl", "1pl" = "m1pl", "m1pl" = "m1pl",
  "2pl"  = "m2pl", "m2pl" = "m2pl",
  "3pl"  = "m3pl", "m3pl" = "m3pl",
  "4pl"  = "m4pl", "m4pl" = "m4pl"
)

#' Resolve IRT model type with alias support
#'
#' Accepts both internal codes (\code{"m1pl"}, \code{"m2pl"},
#' \code{"m3pl"}, \code{"m4pl"}) and user-friendly aliases
#' (\code{"Rasch"}, \code{"1PL"}, \code{"2PL"}, \code{"3PL"},
#' \code{"4PL"}).
#'
#' @param model Character scalar; the model identifier.
#' @return Lower-case internal model code.
#' @keywords internal
resolve_model_type <- function(model) {
  if (length(model) != 1L || is.na(model) || !nzchar(as.character(model))) {
    stop("'model' must be a non-missing character scalar.", call. = FALSE)
  }
  model <- tolower(as.character(model))
  resolved <- model_aliases[[model]]
  if (is.null(resolved)) {
    stop(
      "'model' must be one of: ",
      paste(sort(unique(unlist(model_aliases, use.names = FALSE))), collapse = ", "),
      " (or their aliases: Rasch, 1PL, 2PL, 3PL, 4PL).",
      call. = FALSE
    )
  }
  resolved
}

#' Map resolved model name to integer code
#'
#' Converts the canonical model string returned by \code{resolve_model_type()}
#' to the integer code expected by the Stan models (1 = 1PL, 2 = 2PL,
#' 3 = 3PL, 4 = 4PL).
#'
#' @param model Character string (canonical model name).
#' @return Integer model type code.
#' @keywords internal
model_type_to_int <- function(model) {
  switch(model,
    m1pl = 1L, m2pl = 2L, m3pl = 3L, m4pl = 4L,
    stop("Unknown model: ", model, call. = FALSE)
  )
}


################################# auto-detection helpers #################################

#' Resolve the number of latent dimensions
#'
#' Uses an explicit \eqn{D}, or infers it from the number of columns in a
#' supplied Q-matrix. Latent dimensionality is never guessed from the number
#' of items or blocks.
#'
#' @param D User-supplied D or NULL.
#' @param Q.matrix Optional Q-matrix.
#' @param block.items Deprecated internal argument; ignored.
#' @param I Deprecated internal argument; ignored.
#' @return Integer D.
#' @keywords internal
resolve_D <- function(D, Q.matrix = NULL, block.items = NULL, I = NULL) {
  if (!is.null(D)) {
    return(check_integer_scalar(D, "D"))
  }

  if (!is.null(Q.matrix)) {
    Q.matrix <- as.matrix(Q.matrix)
    if (ncol(Q.matrix) >= 1L) {
      return(ncol(Q.matrix))
    }
  }

  stop("'D' must be supplied when 'Q.matrix' is NULL.", call. = FALSE)
}


#' Auto-detect forced-choice response type from data
#'
#' When \code{fc.type} is \code{NULL}, inspects the ranking strings in the
#' data columns to infer whether they represent RANK, MOLE, or PICK.
#'
#' @param fc.type User-supplied fc.type or NULL.
#' @param data The forced-choice data matrix.
#' @param block.items Optional block-item list.
#' @return Character vector of fc.type values (one per block).
#' @keywords internal
resolve_fc_type <- function(fc.type, data, block.items = NULL) {
  data.matrix <- as.matrix(data)
  N.block <- ncol(data.matrix)
  if (!is.null(fc.type)) {
    return(normalize_fc_type(fc.type, N.block))
  }

  if (!is.character(data.matrix)) {
    return(rep("RANK", N.block))
  }

  if (!is.null(block.items) && length(block.items) != N.block) {
    stop("'block.items' must have one element per data column.",
         call. = FALSE)
  }

  inferred <- character(N.block)
  for (b in seq_len(N.block)) {
    x <- trimws(as.character(data.matrix[, b]))
    x <- x[!is.na(x) & nzchar(x)]
    if (length(x) == 0L) {
      stop("Block ", b, " contains no non-missing responses.",
           call. = FALSE)
    }
    widths <- lengths(strsplit(x, ">", fixed = TRUE))
    if (length(unique(widths)) != 1L) {
      stop("Block ", b, " mixes response formats; supply a consistent ",
           "'fc.type' and 'block.items'.", call. = FALSE)
    }
    width <- widths[1L]
    if (is.null(block.items)) {
      parsed <- lapply(strsplit(x, ">", fixed = TRUE), function(parts) {
        suppressWarnings(as.integer(trimws(parts)))
      })
      if (any(vapply(parsed, anyNA, logical(1L)))) {
        stop("Cannot parse item indices in block ", b, ".", call. = FALSE)
      }
      item_sets <- vapply(
        parsed, function(items) paste(sort(unique(items)), collapse = ","),
        character(1L)
      )
      if (length(unique(item_sets)) != 1L ||
          length(unique(parsed[[1L]])) != width) {
        stop("Partial rankings require explicit 'block.items' and ",
             "'fc.type'.", call. = FALSE)
      }
      inferred[b] <- "RANK"
    } else {
      K <- length(block.items[[b]])
      inferred[b] <- if (width == K) {
        "RANK"
      } else if (width == 2L && K > 2L) {
        "MOLE"
      } else if (width == 1L) {
        "PICK"
      } else {
        stop("Block ", b, " response width does not match 'block.items'.",
             call. = FALSE)
      }
    }
  }
  inferred
}


#' Auto-detect block.items from forced-choice ranking data
#'
#' Parses ranking strings like \code{"2>1>3"} to extract the set of item
#' indices in each block.  Only works when data columns contain character
#' ranking strings.
#'
#' @param block.items User-supplied block.items or NULL.
#' @param data The forced-choice data matrix.
#' @return A list of integer vectors (one per block), or NULL if
#'   auto-detection is not possible.
#' @keywords internal
resolve_block_items <- function(block.items, data) {
  data.matrix <- as.matrix(data)
  N.block <- ncol(data.matrix)
  if (!is.null(block.items)) {
    if (!is.list(block.items) || length(block.items) != N.block) {
      stop("'block.items' must be a list with one element per data column.",
           call. = FALSE)
    }
    block.items <- lapply(seq_along(block.items), function(b) {
      items <- block.items[[b]]
      if (!is.numeric(items) || anyNA(items) || any(!is.finite(items)) ||
          any(items != floor(items)) || any(items < 1) ||
          anyDuplicated(items)) {
        stop("'block.items[[", b, "]]' must contain unique positive ",
             "integer indices.", call. = FALSE)
      }
      as.integer(items)
    })
    if (anyDuplicated(unlist(block.items, use.names = FALSE))) {
      stop("Item indices must be unique across 'block.items'.",
           call. = FALSE)
    }
    return(block.items)
  }

  if (!is.character(data.matrix)) return(NULL)

  block.items <- vector("list", N.block)

  for (b in seq_len(N.block)) {
    x <- as.character(data.matrix[, b])
    x <- x[!is.na(x) & nzchar(x)]
    if (length(x) == 0L) {
      stop("Block ", b, " contains no valid data for auto-detection. ",
           "Please supply 'block.items' explicitly.", call. = FALSE)
    }
    parsed <- lapply(strsplit(x, ">", fixed = TRUE), function(parts) {
      suppressWarnings(as.integer(trimws(parts)))
    })
    if (any(vapply(parsed, anyNA, logical(1L)))) {
      stop("Cannot parse item indices from block ", b, " data. ",
           "Please supply 'block.items' explicitly.", call. = FALSE)
    }
    item.sets <- lapply(parsed, function(items) sort(unique(items)))
    keys <- vapply(
      item.sets, function(items) paste(items, collapse = ","), character(1L)
    )
    if (length(unique(keys)) != 1L ||
        any(lengths(item.sets) != lengths(parsed))) {
      stop("Block ", b, " contains partial or inconsistent rankings. ",
           "Please supply 'block.items' explicitly.", call. = FALSE)
    }
    block.items[[b]] <- item.sets[[1L]]
  }

  # Validate: items should form a contiguous set 1..I
  all_items <- sort(unique(unlist(block.items, use.names = FALSE)))
  I <- length(all_items)
  if (!all(all_items == seq_len(I))) {
    stop("Auto-detected block items are not a contiguous set 1..", I, ". ",
         "Please supply 'block.items' explicitly.", call. = FALSE)
  }

  message("block.items auto-detected: ", N.block, " blocks, ",
          I, " unique items")
  block.items
}

fit_common_method_control <- function(control.method) {
  control.method <- fc_as_control_list(control.method, "control.method")

  chains <- as.integer(get_ctrl("chains", 2L, control.method))
  cores <- get_ctrl("cores", chains, control.method)
  cores <- as.integer(cores)
  if (length(cores) != 1L || is.na(cores) || cores < 1L) {
    stop("'cores' in 'control.method' must be a positive integer.",
         call. = FALSE)
  }

  vis <- get_ctrl("vis", TRUE, control.method)
  if (!is.logical(vis) || length(vis) != 1L || is.na(vis)) {
    stop("'vis' in 'control.method' must be TRUE or FALSE.", call. = FALSE)
  }

  seed <- get_ctrl(
    "seed", sample.int(.Machine$integer.max, 1L), control.method
  )
  if ((!is.numeric(seed) && !is.integer(seed)) ||
      length(seed) != 1L || !is.finite(seed)) {
    stop("'seed' in 'control.method' must be a finite numeric scalar.",
         call. = FALSE)
  }
  seed <- as.integer(seed)
  if (is.na(seed)) {
    stop("'seed' in 'control.method' must be a valid integer.", call. = FALSE)
  }

  list(cores = cores, vis = vis, seed = seed)
}

fit_effective_method_control <- function(control.method, method = list(),
                                         common = list()) {
  control.method <- fc_as_control_list(control.method, "control.method")
  defaults <- utils::modifyList(common, method)
  utils::modifyList(defaults, control.method)
}

fc_stan_method_control <- function(control.method) {
  control.method <- fc_as_control_list(control.method, "control.method")

  iter <- as.integer(get_ctrl("iter", 5000L, control.method))
  stan.method <- list(
    chains = as.integer(get_ctrl("chains", 2L, control.method)),
    iter = iter,
    warmup = as.integer(get_ctrl("warmup", floor(iter / 2), control.method)),
    thin = as.integer(get_ctrl("thin", 1L, control.method)),
    init = get_ctrl("init", "random", control.method),
    algorithm = get_ctrl("algorithm", "HMC", control.method)
  )

  stan.method$algorithm <- match.arg(
    stan.method$algorithm,
    c("NUTS", "HMC", "Fixed_param")
  )

  integer_names <- c("chains", "iter", "warmup", "thin")
  for (nm in integer_names) {
    if (is.na(stan.method[[nm]]) || stan.method[[nm]] < 1L) {
      stop("'", nm, "' in 'control.method' must be a positive integer.",
           call. = FALSE)
    }
  }
  if (stan.method$warmup >= stan.method$iter) {
    stop("'warmup' in 'control.method' must be smaller than 'iter'.",
         call. = FALSE)
  }

  stan.method
}

istem_scalar <- function(value, name, positive = FALSE, arg = "control.model") {
  if (!is.numeric(value) || length(value) != 1L || !is.finite(value)) {
    stop("'", name, "' in '", arg, "' must be a finite numeric scalar.",
         call. = FALSE)
  }
  value <- as.numeric(value)
  if (positive && value <= 0) {
    stop("'", name, "' in '", arg, "' must be positive.", call. = FALSE)
  }
  value
}

istem_clip <- function(x, lower, upper) {
  pmin(pmax(x, lower), upper)
}

istem_random_theta <- function(N, D, theta.mu, lower, upper) {
  theta.mu <- rep(as.numeric(theta.mu), length.out = D)
  theta <- matrix(stats::rnorm(N * D, mean = rep(theta.mu, each = N), sd = 1),
                  nrow = N, ncol = D)
  istem_clip(theta, lower, upper)
}

istem_rnorm_bounded <- function(n, mean, sd, lower, upper) {
  istem_clip(stats::rnorm(n, mean = mean, sd = sd), lower, upper)
}

istem_rlnorm_bounded <- function(n, meanlog, sdlog, lower, upper) {
  istem_clip(stats::rlnorm(n, meanlog = meanlog, sdlog = sdlog), lower, upper)
}

istem_random_corr <- function(D, include.corr = TRUE) {
  if (D <= 1L || !isTRUE(include.corr)) {
    return(diag(D))
  }
  z <- matrix(stats::rnorm(D * (D + 1L)), nrow = D)
  corr <- stats::cov2cor(tcrossprod(z))
  diag(corr) <- 1
  corr
}

istem_response_groups <- function(response) {
  response <- as.matrix(response)
  key <- apply(response, 1L, paste, collapse = "\r")
  level <- unique(key)
  group <- match(key, level)
  first <- match(level, key)
  unique.response <- response[first, , drop = FALSE]
  storage.mode(unique.response) <- storage.mode(response)
  list(
    response = unique.response,
    count = as.integer(tabulate(group, nbins = length(level))),
    group = as.integer(group),
    first = as.integer(first),
    G = length(level)
  )
}

istem_response_pair_groups <- function(response, pairs.value) {
  response <- as.matrix(response)
  response.key <- apply(response, 1L, paste, collapse = "\r")
  pair.key <- vapply(pairs.value, function(person) {
    paste(vapply(person, function(block) {
      block <- as.matrix(block)
      paste(nrow(block), ncol(block), paste(as.integer(block), collapse = ","),
            sep = ":")
    }, character(1L)), collapse = "\r")
  }, character(1L))
  key <- paste(response.key, pair.key, sep = "\f")
  level <- unique(key)
  group <- match(key, level)
  first <- match(level, key)
  unique.response <- response[first, , drop = FALSE]
  storage.mode(unique.response) <- storage.mode(response)
  list(
    response = unique.response,
    pairs.value = pairs.value[first],
    count = as.integer(tabulate(group, nbins = length(level))),
    group = as.integer(group),
    first = as.integer(first),
    G = length(level)
  )
}

istem_expand_group_matrix <- function(x, group, row_names = NULL) {
  out <- x[group, , drop = FALSE]
  if (!is.null(row_names)) rownames(out) <- row_names
  out
}

istem_state_weight <- function(state) {
  if (!is.null(state$response.count)) state$response.count else rep(1, nrow(state$theta))
}

istem_default_L <- function(D) {
  switch(as.character(D),
         "1" = 61L, "2" = 31L, "3" = 15L, "4" = 9L, "5" = 7L, 3L)
}

istem_latent_grid_control <- function(control.model, D, L = NULL,
                                      theta.low = NULL, theta.up = NULL) {
  control.model <- fc_as_control_list(control.model, "control.model")
  D <- as.integer(D)
  if (length(D) != 1L || is.na(D) || D < 1L) {
    stop("'D' must be a positive integer.", call. = FALSE)
  }

  if (is.null(L)) {
    L <- get_ctrl("L", NULL, control.model)
  }
  if (is.null(L)) {
    L <- istem_default_L(D)
  }
  L <- as.integer(L)
  if (length(L) != 1L || is.na(L) || !is.finite(L) || L < 2L) {
    stop("'L' in 'control.model' must be an integer of at least 2.",
         call. = FALSE)
  }

  if (is.null(theta.low)) {
    theta.low <- get_ctrl("theta.lower", -6, control.model)
  }
  if (is.null(theta.up)) {
    theta.up <- get_ctrl("theta.upper", 6, control.model)
  }
  theta.low <- as.numeric(theta.low)
  theta.up <- as.numeric(theta.up)
  if (length(theta.low) != 1L || length(theta.up) != 1L ||
      !is.finite(theta.low) || !is.finite(theta.up) ||
      theta.low >= theta.up) {
    stop("'theta.lower' must be smaller than 'theta.upper' in 'control.model'.",
         call. = FALSE)
  }

  list(L = L, theta.lower = theta.low, theta.upper = theta.up)
}

istem_apply_grid_control <- function(method, control.model, D) {
  grid <- istem_latent_grid_control(control.model, D)
  method$L <- grid$L
  method$theta.lower <- grid$theta.lower
  method$theta.upper <- grid$theta.upper
  method
}

istem_effective_grid_model_control <- function(control.model, method) {
  utils::modifyList(
    control.model,
    list(L = method$L,
         theta.lower = method$theta.lower,
         theta.upper = method$theta.upper)
  )
}

istem_method_control <- function(control.method) {
  control.method <- fc_as_control_list(control.method, "control.method")
  list(
    M = as.integer(get_ctrl("M", 10L, control.method)),
    B = as.integer(get_ctrl("B", 20L, control.method)),
    burnin.maxitr = as.integer(get_ctrl("burnin.maxitr", 100L, control.method)),
    maxitr = as.integer(get_ctrl("maxitr", 2000L, control.method)),
    eps1 = get_ctrl("eps1", 1.5, control.method),
    eps2 = get_ctrl("eps2", 0.4, control.method),
    frac1 = get_ctrl("frac1", 0.1, control.method),
    frac2 = get_ctrl("frac2", 0.5, control.method),
    optim.maxit = as.integer(get_ctrl("optim.maxit", 200L, control.method)),
    corr.optim.maxit = as.integer(get_ctrl("corr.optim.maxit", 50L,
                                           control.method)),
    fix.corr = isTRUE(get_ctrl("fix.corr", FALSE, control.method)),
    estimate.se = isTRUE(get_ctrl("estimate.se", TRUE, control.method))
  )
}

istem_check_method_control <- function(method) {
  if (method$M < 2L || method$B < 1L) {
    stop("'M' must be at least 2 and 'B' must be at least 1 for iStEM.",
         call. = FALSE)
  }
  if (method$burnin.maxitr < method$M) {
    stop("'burnin.maxitr' must be at least as large as 'M'.", call. = FALSE)
  }
  if (method$maxitr < method$M) {
    stop("'maxitr' must be at least as large as 'M'.", call. = FALSE)
  }
  if (!is.null(method$L) &&
      (!is.finite(method$L) || as.integer(method$L) < 2L)) {
    stop("'L' in 'control.model' must be an integer of at least 2.",
         call. = FALSE)
  }
  if (!is.null(method$theta.lower) || !is.null(method$theta.upper)) {
    if (!is.finite(method$theta.lower) || !is.finite(method$theta.upper) ||
        method$theta.lower >= method$theta.upper) {
      stop("'theta.lower' must be smaller than 'theta.upper' in 'control.model'.",
           call. = FALSE)
    }
  }
  if (!is.finite(method$corr.optim.maxit) || method$corr.optim.maxit < 1L) {
    stop("'corr.optim.maxit' must be a positive integer.", call. = FALSE)
  }
  invisible(NULL)
}

istem_progress <- function(fmt, ..., width = 120L, newline = FALSE) {
  msg <- sprintf(fmt, ...)
  pad <- strrep(" ", max(0L, as.integer(width) - nchar(msg, type = "width")))
  cat("\r", msg, pad, if (newline) "\n" else "", sep = "")
  flush.console()
  invisible(NULL)
}

istem_attach_corr_cache <- function(C) {
  C <- as.matrix(C)
  chol_sigma <- tryCatch(chol(C), error = function(e) NULL)
  if (is.null(chol_sigma)) {
    return(C)
  }
  attr(C, "chol") <- chol_sigma
  attr(C, "log_diag_sum") <- sum(log(diag(chol_sigma)))
  C
}

istem_safe_corr <- function(x) {
  if (is.matrix(x) && nrow(x) == ncol(x)) {
    C <- x
  } else {
    C <- stats::cor(x)
  }
  C[!is.finite(C)] <- 0
  C <- (C + t(C)) / 2
  diag(C) <- 1
  eig <- tryCatch(eigen(C, symmetric = TRUE), error = function(e) NULL)
  if (is.null(eig)) {
    C <- diag(ncol(C))
    return(istem_attach_corr_cache(C))
  }
  eig$values <- pmax(eig$values, 1e-6)
  C <- eig$vectors %*% diag(eig$values, nrow = length(eig$values)) %*%
    t(eig$vectors)
  C <- stats::cov2cor(C)
  C[!is.finite(C)] <- 0
  C <- (C + t(C)) / 2
  diag(C) <- 1
  istem_attach_corr_cache(C)
}

istem_corr_chol <- function(Corr) {
  chol_sigma <- attr(Corr, "chol", exact = TRUE)
  log_diag_sum <- attr(Corr, "log_diag_sum", exact = TRUE)
  if (is.null(chol_sigma) || is.null(log_diag_sum)) {
    Corr <- istem_safe_corr(Corr)
    chol_sigma <- attr(Corr, "chol", exact = TRUE)
    log_diag_sum <- attr(Corr, "log_diag_sum", exact = TRUE)
  }
  list(chol = chol_sigma, log_diag_sum = log_diag_sum)
}

istem_corr_objective <- function(v, theta, theta.mu) {
  D <- ncol(theta)
  C <- diag(D)
  C[lower.tri(C)] <- v
  C <- C + t(C)
  diag(C) <- 1

  chol_sigma <- tryCatch(chol(C), error = function(e) NULL)
  if (is.null(chol_sigma)) {
    return(1e100)
  }

  z <- tryCatch(
    forwardsolve(t(chol_sigma), t(sweep(theta, 2L, theta.mu, "-"))),
    error = function(e) NULL
  )
  if (is.null(z) || any(!is.finite(z))) {
    return(1e100)
  }

  nrow(theta) * 2 * sum(log(diag(chol_sigma))) + sum(z^2)
}

istem_corr_objective_weighted <- function(v, theta, theta.mu, weight) {
  D <- ncol(theta)
  C <- diag(D)
  C[lower.tri(C)] <- v
  C <- C + t(C)
  diag(C) <- 1

  chol_sigma <- tryCatch(chol(C), error = function(e) NULL)
  if (is.null(chol_sigma)) {
    return(1e100)
  }

  z <- tryCatch(
    forwardsolve(t(chol_sigma), t(sweep(theta, 2L, theta.mu, "-"))),
    error = function(e) NULL
  )
  if (is.null(z) || any(!is.finite(z))) {
    return(1e100)
  }

  sum(weight) * 2 * sum(log(diag(chol_sigma))) +
    sum(sweep(z^2, 2L, weight, "*"))
}

istem_update_corr <- function(theta, theta.mu, Corr, maxit) {
  D <- ncol(theta)
  if (D <= 1L) {
    return(matrix(1, 1L, 1L))
  }

  Corr <- istem_safe_corr(Corr)
  x0 <- Corr[lower.tri(Corr)]
  opt <- tryCatch(
    stats::optim(
      par = x0,
      fn = istem_corr_objective,
      theta = theta,
      theta.mu = theta.mu,
      method = "L-BFGS-B",
      lower = rep(-0.995, length(x0)),
      upper = rep(0.995, length(x0)),
      control = list(maxit = maxit)
    ),
    error = function(e) NULL
  )

  if (!is.null(opt) && is.finite(opt$value)) {
    C <- diag(D)
    C[lower.tri(C)] <- opt$par
    C <- C + t(C)
    diag(C) <- 1
    return(istem_safe_corr(C))
  }

  Corr
}

istem_update_corr_weighted <- function(theta, theta.mu, weight, Corr, maxit) {
  D <- ncol(theta)
  if (D <= 1L) {
    return(matrix(1, 1L, 1L))
  }

  Corr <- istem_safe_corr(Corr)
  x0 <- Corr[lower.tri(Corr)]
  opt <- tryCatch(
    stats::optim(
      par = x0,
      fn = istem_corr_objective_weighted,
      theta = theta,
      theta.mu = theta.mu,
      weight = weight,
      method = "L-BFGS-B",
      lower = rep(-0.995, length(x0)),
      upper = rep(0.995, length(x0)),
      control = list(maxit = maxit)
    ),
    error = function(e) NULL
  )

  if (!is.null(opt) && is.finite(opt$value)) {
    C <- diag(D)
    C[lower.tri(C)] <- opt$par
    C <- C + t(C)
    diag(C) <- 1
    return(istem_safe_corr(C))
  }

  Corr
}

istem_batch_var <- function(plist, n) {
  out <- cpp_istem_batch_var(plist, as.integer(n))
  out[!is.finite(out)] <- Inf
  out
}

istem_geweke_z <- function(chain, frac1 = 0.2, frac2 = 0.5) {
  if (is.null(dim(chain))) {
    chain <- matrix(chain, nrow = 1L)
  }
  z <- coda::geweke.diag(
    coda::mcmc(t(chain)),
    frac1 = frac1,
    frac2 = frac2
  )$z
  z[!is.finite(z)] <- 0
  z
}

istem_kernel <- function(state, B, theta_grid_length, theta_lower, theta_upper,
                         fix.corr, corr_optim_maxit, update_parameters, param_vec,
                         sample_theta, logLik_fun = NULL) {
  state$Corr <- istem_attach_corr_cache(state$Corr)
  N <- nrow(state$theta)
  D <- ncol(state$theta)
  npar <- length(param_vec(state))
  pmatrix <- matrix(NA_real_, npar, B)
  logLik.trace <- rep(NA_real_, B)
  theta_sum <- matrix(0, N, D)
  theta_sq_sum <- matrix(0, N, D)

  for (b in seq_len(B)) {
    sampled <- sample_theta(
      state = state,
      theta_grid_length = theta_grid_length,
      theta_lower = theta_lower,
      theta_upper = theta_upper
    )
    state$theta <- sampled$theta

    if (!fix.corr && D > 1L) {
      weight <- istem_state_weight(state)
      state$Corr <- if (is.null(state$response.count)) {
        istem_update_corr(
          theta = state$theta,
          theta.mu = state$theta_mu,
          Corr = state$Corr,
          maxit = corr_optim_maxit
        )
      } else {
        istem_update_corr_weighted(
          theta = state$theta,
          theta.mu = state$theta_mu,
          weight = weight,
          Corr = state$Corr,
          maxit = corr_optim_maxit
        )
      }
    }

    state <- update_parameters(state)
    pmatrix[, b] <- param_vec(state)
    if (!is.null(logLik_fun)) {
      logLik.trace[b] <- logLik_fun(state)
    }
    theta_sum <- theta_sum + state$theta
    theta_sq_sum <- theta_sq_sum + state$theta^2
  }

  list(
    pmatrix = pmatrix,
    state = state,
    logLik.trace = logLik.trace,
    theta_sum = theta_sum,
    theta_sq_sum = theta_sq_sum
  )
}

istem_trace_logLik_values <- function(x) {
  value <- x$logLik.trace %||% x$logLik %||% x$log_lik %||% x$loglik
  if (is.null(value)) return(NA_real_)
  if (is.data.frame(value)) {
    value <- value$logLik %||% value$log_lik %||% value$loglik
  }
  value <- as.numeric(value)
  if (length(value) == 0L) NA_real_ else value
}

istem_run_batches <- function(state, method, N, vis, label, run_batch) {
  M <- method$M
  B <- method$B
  burnin.maxitr <- method$burnin.maxitr
  maxitr <- method$maxitr
  eps1 <- method$eps1
  eps2 <- method$eps2
  frac1 <- method$frac1
  frac2 <- method$frac2

  plist <- vector("list", M)
  trace.logLik <- numeric(0L)
  batch.extra.names <- NULL
  batch.extra <- list()

  if (vis) {
    cat("\n", label, " iStEM burn-in phase:\n", sep = "")
  }

  for (m in seq_len(M)) {
    if (vis) {
      istem_progress("  burn-in batch %04d/%04d", m, M)
    }
    x <- run_batch(state)
    state <- x$state
    plist[[m]] <- x$pmatrix
    trace.logLik <- c(trace.logLik, istem_trace_logLik_values(x))
    if (is.null(batch.extra.names)) {
      batch.extra.names <- setdiff(names(x), c("state", "pmatrix", "logLik",
                                               "log_lik", "loglik",
                                               "logLik.trace"))
      batch.extra <- stats::setNames(vector("list", length(batch.extra.names)),
                                     batch.extra.names)
      for (nm in batch.extra.names) batch.extra[[nm]] <- vector("list", M)
    }
    for (nm in batch.extra.names) batch.extra[[nm]][[m]] <- x[[nm]]
  }

  z <- istem_geweke_z(Reduce(cbind, plist), frac1 = frac1, frac2 = frac2)
  d.hat <- istem_batch_var(plist, n = M)

  burn.in.size <- 0L
  total.number.of.batch <- M
  m <- M
  while (sum(z^2) >= length(z) * eps1 &&
         m < burnin.maxitr) {

    x <- run_batch(state)
    state <- x$state
    trace.logLik <- c(trace.logLik, istem_trace_logLik_values(x))
    plist <- c(plist[-1L], list(x$pmatrix))
    for (nm in batch.extra.names) {
      batch.extra[[nm]] <- c(batch.extra[[nm]][-1L], list(x[[nm]]))
    }

    z <- istem_geweke_z(Reduce(cbind, plist), frac1 = frac1, frac2 = frac2)
    d.hat <- istem_batch_var(plist, n = M)
    burn.in.size <- burn.in.size + B
    total.number.of.batch <- total.number.of.batch + 1L
    m <- m + 1L

    if (vis) {
      istem_progress(
        "  burn-in batch %04d | Geweke mean z^2 %10.4f <= %-10.4f",
        m,
        sum(z^2) / length(z),
        eps1
      )
    }
  }

  burnin.z2.per.par <- sum(z^2) / length(z)
  burnin.max.Nu <- max(d.hat * N, na.rm = TRUE)
  burnin.converged <- burnin.z2.per.par < eps1
  if (!burnin.converged && m >= burnin.maxitr) {
    warning(
      label,
      " iStEM burn-in stopped at 'burnin.maxitr' before the Geweke criterion was met: ",
      "Geweke mean z^2 = ", signif(burnin.z2.per.par, 5),
      ", criterion = ", signif(eps1, 5), ".",
      call. = FALSE
    )
  }

  stable.plist <- plist
  stable.extra <- batch.extra
  n <- M
  d.hat <- istem_batch_var(stable.plist, n = n)

  if (vis) {
    cat("\nAfter burn-in phase:\n")
  }

  while (max(d.hat * N, na.rm = TRUE) >= eps2 && n < maxitr) {
    if (vis) {
      istem_progress(
        "  valid batch %04d | max N*u %10.4f <= %-10.4f",
        n,
        max(d.hat * N, na.rm = TRUE),
        eps2
      )
    }

    x <- run_batch(state)
    state <- x$state
    trace.logLik <- c(trace.logLik, istem_trace_logLik_values(x))
    stable.plist[[n + 1L]] <- x$pmatrix
    for (nm in batch.extra.names) stable.extra[[nm]][[n + 1L]] <- x[[nm]]
    n <- n + 1L
    total.number.of.batch <- total.number.of.batch + 1L
    d.hat <- istem_batch_var(stable.plist, n = n)
  }

  if (vis) {
    istem_progress(
      "  valid batch %04d | max N*u %10.4f <= %-10.4f",
      n,
      max(d.hat * N, na.rm = TRUE),
      eps2,
      newline = TRUE
    )
    cat("Length of final MC chain = ", n * B, "\n", sep = "")
  }

  final.max.Nu <- max(d.hat * N, na.rm = TRUE)
  final.converged <- final.max.Nu < eps2
  if (!final.converged && n >= maxitr) {
    warning(
      label,
      " iStEM stopped at 'maxitr' before the Monte Carlo error criterion was met: ",
      "max N*u = ", signif(final.max.Nu, 5),
      ", criterion = ", signif(eps2, 5), ".",
      call. = FALSE
    )
  }

  chain.length <- n * B
  stable.start <- max(1L, length(trace.logLik) - M * B + 1L)
  stable.start.batch <- ceiling(stable.start / B)
  logLik.trace <- data.frame(
    iter = seq_along(trace.logLik),
    logLik = trace.logLik
  )
  list(
    state = state,
    stable.plist = stable.plist,
    stable.extra = stable.extra,
    stable.start = stable.start,
    stable.start.batch = stable.start.batch,
    chain.length = chain.length,
    n.batch = n,
    iStEM = list(
      M = M,
      B = B,
      burn.in.size = burn.in.size,
      total.number.of.batch = total.number.of.batch,
      final.chain = chain.length,
      burnin.converged = burnin.converged,
      converged = final.converged,
      burnin.z2.per.par = burnin.z2.per.par,
      burnin.max.Nu = burnin.max.Nu,
      final.max.Nu = final.max.Nu,
      geweke.z = z,
      batch.var = d.hat,
      stable.start = stable.start,
      stable.start.batch = stable.start.batch,
      logLik.trace = logLik.trace,
      itemlist = stable.plist
    )
  )
}

istem_run <- function(state, method, N, vis, label,
                      update_parameters, param_vec,
                      sample_theta, logLik_fun = NULL) {
  L <- method$L
  theta_lower <- method$theta.lower
  theta_upper <- method$theta.upper
  corr_optim_maxit <- method$corr.optim.maxit
  fix.corr <- method$fix.corr
  estimate.se <- method$estimate.se

  run <- istem_run_batches(
    state = state,
    method = method,
    N = N,
    vis = vis,
    label = label,
    run_batch = function(state) {
      x <- istem_kernel(
        state = state,
        B = method$B,
        theta_grid_length = L,
        theta_lower = theta_lower,
        theta_upper = theta_upper,
        fix.corr = fix.corr,
        corr_optim_maxit = corr_optim_maxit,
        update_parameters = update_parameters,
        param_vec = param_vec,
        sample_theta = sample_theta,
        logLik_fun = logLik_fun
      )
      x
    }
  )

  chain_mat <- Reduce(cbind, run$stable.plist)
  chain_mean <- rowMeans(chain_mat)
  chain_sd <- if (estimate.se && ncol(chain_mat) > 1L) {
    apply(chain_mat, 1L, stats::sd)
  } else {
    rep(NA_real_, length(chain_mean))
  }
  chain_rhat <- rep(NA_real_, length(chain_mean))  # iStEM is not MCMC; Rhat not applicable

  theta_sum <- Reduce("+", run$stable.extra$theta_sum)
  theta_sq_sum <- Reduce("+", run$stable.extra$theta_sq_sum)
  chain_length <- run$chain.length
  theta_est <- theta_sum / chain_length
  theta_second <- theta_sq_sum / chain_length
  theta_se <- sqrt(pmax(theta_second - theta_est^2, 0))

  list(
    state = run$state,
    chain.mean = chain_mean,
    chain.sd = chain_sd,
    chain.rhat = chain_rhat,
    chain.mat = chain_mat,
    theta.est = theta_est,
    theta.se = theta_se,
    iStEM = utils::modifyList(
      run$iStEM,
      list(
      accept.rate = NA_real_,
        L = L
      )
    )
  )
}

istem_loglik_grid <- function(state, L, theta.lower, theta.upper) {
  D <- ncol(state$theta)
  make_quadrature_grid_mvn(
    state$Corr, state$theta_mu, D, L, theta.lower, theta.upper
  )
}

istem_loglik_binary_sum <- function(prob, response, count, pi) {
  response <- as.matrix(response)
  lik <- cpp_loglik_binary(
    prob = prob,
    response = matrix(as.integer(response), nrow = nrow(response),
                      ncol = ncol(response)),
    pi = as.vector(pi)
  )
  loglik_apply_group_count(lik, count)$logLik
}

istem_loglik_indexed_sum <- function(prob, response, count, pi, block_sizes,
                                     response_base) {
  response <- as.matrix(response)
  lik <- cpp_loglik_indexed(
    prob = prob,
    response = matrix(as.integer(response), nrow = nrow(response),
                      ncol = ncol(response)),
    pi = as.vector(pi),
    block_sizes = as.integer(block_sizes),
    response_base = as.integer(response_base)
  )
  loglik_apply_group_count(lik, count)$logLik
}

mirt_istem_loglik_trace <- function(state, L, theta.lower, theta.upper) {
  grid <- istem_loglik_grid(state, L, theta.lower, theta.upper)
  prob <- model.MIRT(grid$theta.norm, state$par)
  istem_loglik_binary_sum(prob, state$response, state$response.count, grid$pi)
}

mgpcm_istem_loglik_trace <- function(state, L, theta.lower, theta.upper) {
  grid <- istem_loglik_grid(state, L, theta.lower, theta.upper)
  prob <- model.MGPCM(grid$theta.norm, state$par)
  istem_loglik_indexed_sum(
    prob, state$response, state$response.count, grid$pi,
    block_sizes = state$length.poly, response_base = 0L
  )
}

mggum_istem_loglik_trace <- function(state, L, theta.lower, theta.upper) {
  grid <- istem_loglik_grid(state, L, theta.lower, theta.upper)
  prob <- model.MGGUM(grid$theta.norm, state$par)
  istem_loglik_indexed_sum(
    prob, state$response, state$response.count, grid$pi,
    block_sizes = state$length.poly, response_base = 0L
  )
}

fcmirt_istem_loglik_trace <- function(state, L, theta.lower, theta.upper) {
  grid <- istem_loglik_grid(state, L, theta.lower, theta.upper)
  prob <- model.FCMIRT(grid$theta.norm, state$par,
                       state$patterns.total, state$patterns)
  istem_loglik_indexed_sum(
    prob, state$response, state$response.count, grid$pi,
    block_sizes = vapply(state$patterns, nrow, integer(1L)),
    response_base = 1L
  )
}

fcggum_istem_loglik_trace <- function(state, L, theta.lower, theta.upper) {
  grid <- istem_loglik_grid(state, L, theta.lower, theta.upper)
  prob <- model.FCGGUM(grid$theta.norm, state$par,
                       state$patterns.total, state$patterns)
  istem_loglik_indexed_sum(
    prob, state$response, state$response.count, grid$pi,
    block_sizes = vapply(state$patterns, nrow, integer(1L)),
    response_base = 1L
  )
}

tirt_istem_loglik_trace <- function(state, L, theta.lower, theta.upper) {
  grid <- istem_loglik_grid(state, L, theta.lower, theta.upper)
  gamma.matrix <- tirt_istem_gamma_matrix(
    state$gamma, rep(NA_real_, length(state$gamma)),
    state$pairs.matrix, state$I.states
  )$est
  prob <- model.TIRT(grid$theta.norm, state$lambda, state$psi^2,
                     gamma.matrix, state$Q.matrix, state$pairs.matrix)

  response <- as.matrix(state$response.unique)
  block.pair.counts <- vapply(seq_along(state$block.items), function(b) {
    k <- length(state$block.items[[b]])
    as.integer(switch(state$fc.type[b],
      RANK = choose(k, 2),
      MOLE = 2L * k - 3L,
      PICK = k - 1L,
      stop("'fc.type' must be RANK, MOLE, or PICK.", call. = FALSE)
    ))
  }, integer(1L))

  if (!is.null(state$pairs.value.unique) && !all(state$fc.type == "RANK")) {
    lik <- cpp_loglik_binary_person_pairs(
      prob = prob,
      response = matrix(as.integer(response), nrow = nrow(response),
                        ncol = ncol(response)),
      pi = as.vector(grid$pi),
      pairs_matrix = state$pairs.matrix,
      pairs_value = state$pairs.value.unique,
      block_pair_counts = block.pair.counts
    )
  } else {
    lik <- cpp_loglik_binary(
      prob = prob,
      response = matrix(as.integer(response), nrow = nrow(response),
                        ncol = ncol(response)),
      pi = as.vector(grid$pi)
    )
  }
  loglik_apply_group_count(lik, state$response.count)$logLik
}

istem_prepare_01_q <- function(I, D, Q.matrix, triangular = TRUE,
                               require.row = FALSE) {
  if (!is.null(Q.matrix)) {
    Q.matrix <- as.matrix(Q.matrix)
    if (nrow(Q.matrix) != I || ncol(Q.matrix) != D ||
        anyNA(Q.matrix) || !all(Q.matrix %in% c(0, 1))) {
      stop("'Q.matrix' must be an I x D matrix containing only 0 and 1.",
           call. = FALSE)
    }
  }

  if (is.null(Q.matrix)) {
    if (triangular && D > I) {
      stop("Triangular identification requires 'D' <= number of items.",
           call. = FALSE)
    }
    Q.matrix <- matrix(1, I, D)
    if (triangular && D > 1L) {
      for (k in seq_len(D)) {
        r <- I - D + k
        for (j in seq_len(D)) {
          if (k > D - j + 1L) {
            Q.matrix[r, j] <- 0
          }
        }
      }
    }
  }

  if (require.row && any(rowSums(Q.matrix) < 1L)) {
    stop("Each item must measure at least one trait (each row of 'Q.matrix' needs a 1).",
         call. = FALSE)
  }

  storage.mode(Q.matrix) <- "numeric"
  Q.matrix
}

istem_prepare_signed_q <- function(I, D, Q.matrix, require.row = FALSE) {
  if (!is.null(Q.matrix)) {
    Q.matrix <- as.matrix(Q.matrix)
    if (nrow(Q.matrix) != I || ncol(Q.matrix) != D ||
        anyNA(Q.matrix) || !all(Q.matrix %in% c(-1, 0, 1))) {
      stop("'Q.matrix' must be an I x D matrix containing only -1, 0, and 1.",
           call. = FALSE)
    }
  }
  if (is.null(Q.matrix)) {
    Q.matrix <- matrix(1, I, D)
  }
  if (require.row && any(rowSums(abs(Q.matrix)) < 1L)) {
    stop("Each item must measure at least one trait (each row of 'Q.matrix' needs a non-zero entry).",
         call. = FALSE)
  }
  storage.mode(Q.matrix) <- "numeric"
  Q.matrix
}

istem_prepare_response <- function(response, binary = FALSE) {
  response <- as.matrix(response)
  if (!is.numeric(response) || length(dim(response)) != 2L ||
      nrow(response) < 1L || ncol(response) < 1L) {
    stop("'data' must be a non-empty numeric matrix.", call. = FALSE)
  }
  if (anyNA(response) || any(!is.finite(response))) {
    stop("'data' must not contain missing or non-finite responses.",
         call. = FALSE)
  }
  if (any(response != floor(response))) {
    stop("'data' responses must be integer-valued.", call. = FALSE)
  }
  if (binary) {
    if (any(!response %in% c(0, 1))) {
      stop("'data' must contain only binary responses coded 0 and 1.",
           call. = FALSE)
    }
  } else if (any(response < 0)) {
    stop("'data' responses must be non-negative integers.", call. = FALSE)
  }
  storage.mode(response) <- "integer"
  response
}

istem_prepare_length_poly <- function(response, length.poly = NULL) {
  response <- istem_prepare_response(response, binary = FALSE)
  observed.length.poly <- apply(response, 2, max, na.rm = TRUE) + 1L
  I <- ncol(response)
  if (is.null(length.poly)) {
    length.poly <- observed.length.poly
  } else {
    if (length(length.poly) != I) {
      if (length(length.poly) == 1L) {
        length.poly <- rep(length.poly, I)
      } else {
        stop("'length.poly' must be a scalar or have length equal to ncol(response).",
             call. = FALSE)
      }
    }
    if (!is.numeric(length.poly) || anyNA(length.poly) ||
        any(!is.finite(length.poly)) || any(length.poly != floor(length.poly))) {
      stop("'length.poly' must contain finite integers.", call. = FALSE)
    }
    length.poly <- as.integer(length.poly)
    if (any(length.poly < 2L)) {
      stop("'length.poly' must contain integers >= 2.", call. = FALSE)
    }
    if (any(observed.length.poly > length.poly)) {
      stop("'length.poly' is smaller than the largest observed response category for at least one item.",
           call. = FALSE)
    }
  }
  if (any(length.poly < 2L)) {
    stop("Every item must have at least two response categories.",
         call. = FALSE)
  }
  as.integer(length.poly)
}

istem_named_theta <- function(theta, theta.se, response, D) {
  colnames(theta) <- colnames(theta.se) <- paste0("Dim.", seq_len(D))
  rownames(theta) <- rownames(theta.se) <- rownames(response)
  theta.Rhat <- matrix(NA_real_, nrow(theta), D, dimnames = dimnames(theta))
  list(est = theta, se = theta.se, Rhat = theta.Rhat)
}

istem_param_corr_vec <- function(Corr, include_corr) {
  if (include_corr) Corr[lower.tri(Corr)] else numeric(0)
}

istem_unpack_corr <- function(pv, pv_se, idx, D, Corr, include_corr) {
  Corr.se <- matrix(NA_real_, D, D)
  if (D == 1L) {
    Corr <- matrix(1, 1, 1)
    Corr.se <- matrix(0, 1, 1)
  } else if (include_corr) {
    n_corr <- D * (D - 1L) / 2L
    corr_vals <- pv[idx:(idx + n_corr - 1L)]
    corr_se <- pv_se[idx:(idx + n_corr - 1L)]
    idx <- idx + n_corr
    Corr <- diag(D)
    Corr[lower.tri(Corr)] <- corr_vals
    Corr <- t(Corr)
    Corr[lower.tri(Corr)] <- corr_vals
    Corr.se[lower.tri(Corr.se)] <- corr_se
    Corr.se <- t(Corr.se)
    Corr.se[lower.tri(Corr.se)] <- corr_se
    diag(Corr.se) <- 0
    Corr <- istem_safe_corr(Corr)
  } else {
    Corr.se[,] <- NA_real_
    diag(Corr.se) <- 0
  }
  list(Corr = Corr, Corr.se = Corr.se, idx = idx)
}

istem_unpack_corr_rhat <- function(pv_rhat, idx, D, include_corr) {
  if (is.null(pv_rhat)) return(matrix(NA_real_, D, D))
  Corr.rhat <- matrix(NA_real_, D, D)
  diag(Corr.rhat) <- NA_real_
  if (D > 1L && include_corr) {
    n_corr <- D * (D - 1L) / 2L
    corr_rhat <- pv_rhat[idx:(idx + n_corr - 1L)]
    Corr.rhat[lower.tri(Corr.rhat)] <- corr_rhat
    Corr.rhat <- t(Corr.rhat)
    Corr.rhat[lower.tri(Corr.rhat)] <- corr_rhat
  }
  Corr.rhat
}


################################# Forced-choice iStEM helpers #################################

istem_prepare_fc_data <- function(data, block.items = NULL, fc.type = "RANK") {
  fc.type.valid <- c("RANK", "MOLE", "PICK")
  fc.type <- toupper(fc.type)
  if (!all(fc.type %in% fc.type.valid)) {
    stop("'fc.type' must be one of: ", paste(fc.type.valid, collapse = ", "),
         call. = FALSE)
  }

  data.matrix <- as.matrix(data)
  N.block.data <- ncol(data.matrix)
  if (is.null(N.block.data) || N.block.data < 1L) {
    stop("'data' must have at least one column/block.", call. = FALSE)
  }
  if (length(fc.type) != N.block.data) {
    fc.type <- rep(fc.type[1L], N.block.data)
  }

  is.pattern.index.data <- is.numeric(data.matrix) || is.integer(data.matrix)
  if (is.null(block.items)) {
    if (is.pattern.index.data) {
      stop("'block.items' is required when 'data' is an integer matrix of pattern indices.",
           call. = FALSE)
    }
    if (any(fc.type != "RANK")) {
      stop("'block.items' is required for MOLE/PICK data because partial rankings do not identify all block items.",
           call. = FALSE)
    }
    block.items <- get.block.items.from.data(data)
  }

  if (is.pattern.index.data) {
    response <- matrix(as.integer(data.matrix),
                       nrow = nrow(data.matrix),
                       ncol = ncol(data.matrix),
                       dimnames = dimnames(data.matrix))
  } else {
    response <- get.response.from.data(data, block.items, fc.type = fc.type)
  }

  if (!is.list(block.items)) {
    stop("'block.items' must be a list.", call. = FALSE)
  }
  N.block <- length(block.items)
  all_items <- unlist(block.items, use.names = FALSE)
  if (anyDuplicated(all_items)) {
    stop("'block.items' contains duplicate items.", call. = FALSE)
  }
  I <- length(all_items)
  if (!setequal(all_items, seq_len(I))) {
    stop("'block.items' must contain exactly the item indices 1 through ", I,
         ".", call. = FALSE)
  }
  if (any(vapply(block.items, length, integer(1L)) < 2L)) {
    stop("Each block in 'block.items' must contain at least two items.",
         call. = FALSE)
  }

  response <- as.matrix(response)
  if (ncol(response) != N.block) {
    stop("'response' must have ", N.block, " columns (one per block).",
         call. = FALSE)
  }
  if (length(fc.type) != N.block) {
    fc.type <- rep(fc.type[1L], N.block)
  }

  patterns.total <- vector("list", N.block)
  patterns <- vector("list", N.block)
  for (b in seq_len(N.block)) {
    patterns.total[[b]] <- get_permutations(block.items[[b]],
                                            length(block.items[[b]]))
    patterns[[b]] <- get_permutations(
      block.items[[b]],
      switch(fc.type[b],
             RANK = length(block.items[[b]]),
             MOLE = 2L,
             PICK = 1L)
    )
  }

  for (b in seq_len(N.block)) {
    if (anyNA(response[, b]) ||
        any(response[, b] < 1L | response[, b] > nrow(patterns[[b]]))) {
      stop("'data' contains invalid pattern indices for block ", b,
           "; valid values are 1 through ", nrow(patterns[[b]]), ".",
           call. = FALSE)
    }
  }

  storage.mode(response) <- "integer"

  list(
    response = response,
    block.items = block.items,
    fc.type = fc.type,
    patterns = patterns,
    patterns.total = patterns.total,
    N = nrow(response),
    B = N.block,
    I = I,
    all.items = all_items
  )
}

sim.data.Q.CDM <- function(D, I.states, single=TRUE){
  if(D < 2)
    stop("The number of attributes (D) must be more than 1.\n", call. = FALSE)
  if(I.states < 2)
    stop("The number of items (I.states) must be more than 1.\n", call. = FALSE)

  alpha <- attributepattern(D)
  D.sum <- rowSums(alpha)
  if(single){
    idx <- which(D.sum <= 1 & D.sum > 0)
    # Distribute items evenly across attributes to avoid unbalanced Q-matrices
    # that produce many uninformative within-attribute blocks.
    n.attr <- length(idx)
    base <- I.states %/% n.attr
    remainder <- I.states %% n.attr
    n.per.attr <- rep(base, n.attr)
    if (remainder > 0L) n.per.attr[seq_len(remainder)] <- n.per.attr[seq_len(remainder)] + 1L
    Q <- alpha[rep(idx, times = n.per.attr), , drop = FALSE]
    Q <- Q[sample(nrow(Q)), , drop = FALSE]  # shuffle rows
  }else{
    Q <- alpha[sample(which(D.sum <= 3 & D.sum > 0), I.states, replace = TRUE), ]
  }

  rownames(Q) <- paste0("S", 1:I.states)
  return(Q)
}

attributepattern <- function(D) {
  patterns <- lapply(0:D, function(m) {
    if (m == 0) {
      matrix(0L, 1, D)
    } else {
      t(apply(combn(D, m), 2, function(idx) {
        x <- integer(D)
        x[idx] <- 1L
        x
      }))
    }
  })

  out <- do.call(rbind, patterns)
  colnames(out) <- paste0("A", seq_len(D))
  out
}

sim.Q.matrix.FC <- function(I.states, D, block.items, allow.negative = FALSE) {
  if (length(I.states) != 1L || is.na(I.states) || I.states < 1L) {
    stop("'I.states' must be a positive integer.", call. = FALSE)
  }
  if (length(D) != 1L || is.na(D) || D < 1L) {
    stop("'D' must be a positive integer.", call. = FALSE)
  }
  if (length(allow.negative) != 1L || is.na(allow.negative)) {
    stop("'allow.negative' must be TRUE or FALSE.", call. = FALSE)
  }
  if (!is.list(block.items) || length(block.items) < 1L) {
    stop("'block.items' must be a non-empty list.", call. = FALSE)
  }

  I.states <- as.integer(I.states)
  D <- as.integer(D)

  block.items <- lapply(seq_along(block.items), function(b) {
    items.b <- block.items[[b]]
    if (!is.numeric(items.b) && !is.integer(items.b)) {
      stop("Each element of 'block.items' must contain item indices.", call. = FALSE)
    }
    items.b.integer <- as.integer(items.b)
    if (length(items.b.integer) < 1L || anyNA(items.b.integer) ||
        any(items.b != items.b.integer) ||
        any(items.b.integer < 1L | items.b.integer > I.states)) {
      stop("'block.items' must contain valid item indices from 1 to 'I.states'.", call. = FALSE)
    }
    items.b.integer
  })

  all.items <- unlist(block.items, use.names = FALSE)
  if (anyDuplicated(all.items)) {
    stop("'block.items' contains duplicate items.", call. = FALSE)
  }
  if (!setequal(all.items, seq_len(I.states))) {
    stop("'block.items' must contain exactly all item indices from 1 to 'I.states'.", call. = FALSE)
  }

  Q.matrix <- matrix(0, nrow = I.states, ncol = D)
  for (b in seq_along(block.items)) {
    items.b <- block.items[[b]]
    block.traits <- sample(seq_len(D), length(items.b),
                           replace = length(items.b) > D)
    q.values <- if (isTRUE(allow.negative)) {
      sample(c(-1, 1), length(items.b), replace = TRUE)
    } else {
      rep(1, length(items.b))
    }
    Q.matrix[cbind(items.b, block.traits)] <- q.values
  }

  storage.mode(Q.matrix) <- "numeric"
  Q.matrix
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----eval=FALSE---------------------------------------------------------------
# library(ForceChoice)
# 
# # Simulate binary response data
# sim <- sim.data.MIRT(N = 20, I = 6, D = 2, model = "2PL")
# 
# # Fit via iStEM
# fit <- fit.MIRT(sim$data, model = "2PL", D = 2,
#                 method = "iStEM",
#                 control.method = list(
#                   vis = FALSE, seed = 123,
#                   M = 2, B = 2, burnin.maxitr = 2,
#                   maxitr = 3, eps1 = 10, eps2 = 10,
#                   estimate.se = FALSE))
# 
# # Examine results
# print(fit)
# summary(fit)
# 
# # Item parameter estimates (first 6 items)
# head(coef(fit))
# 
# # Factor correlation matrix
# fit$Corr$est
# 
# # Trait recovery
# diag(cor(fit$theta$est, sim$theta))

## ----eval=FALSE---------------------------------------------------------------
# # Compute comprehensive fit indices
# gof <- get.fit.index(fit)
# 
# # Summary of fit indices
# summary(gof)
# 
# # Extract specific indices
# gof$M2       # Limited-information M2 statistic
# gof$RMSEA    # RMSEA with 90% CI
# gof$CFI      # Comparative Fit Index
# gof$TLI      # Tucker-Lewis Index
# gof$SRMSR    # Standardized Root Mean Square Residual
# gof$AIC      # Akaike Information Criterion
# gof$BIC      # Bayesian Information Criterion

## ----eval=FALSE---------------------------------------------------------------
# # Simulate forced-choice ranking data
# sim <- sim.data.FCMIRT(N.person = 20, N.block = 3, I.block = 2,
#                        D = 2, model = "2PL", fc.type = "RANK")
# 
# # The data contains ranking strings
# head(sim$data)
# 
# # Fit: block.items and fc.type are auto-detected
# fit <- fit.FCMIRT(sim$data, model = "2PL", D = 2,
#                   method = "iStEM",
#                   control.method = list(
#                     vis = FALSE, seed = 123,
#                     M = 2, B = 2, burnin.maxitr = 2,
#                     maxitr = 3, eps1 = 10, eps2 = 10,
#                     estimate.se = FALSE))
# 
# # Trait recovery
# cor(fit$theta$est, sim$theta)
# 
# # Goodness-of-fit (uses nominal binary expansion)
# gof <- get.fit.index(fit)
# summary(gof)

## ----eval=FALSE---------------------------------------------------------------
# sim <- sim.data.TIRT(N.person = 20, N.block = 3, I.block = 2,
#                      D = 2, fc.type = "RANK")
# 
# fit <- fit.TIRT(sim$data, Q.matrix = sim$Q.matrix,
#                 block.items = sim$block.items,
#                 method = "iStEM",
#                 control.method = list(
#                   vis = FALSE, seed = 123,
#                   M = 2, B = 2, burnin.maxitr = 2,
#                   maxitr = 3, eps1 = 10, eps2 = 10,
#                   estimate.se = FALSE))
# 
# # Structural parameters: loadings and uniquenesses
# head(coef(fit))
# 
# # Gamma matrix (pairwise intercepts)
# fit$gamma.matrix$est[1:5, 1:5]

## ----eval=FALSE---------------------------------------------------------------
# sim <- sim.data.FCDCM(N.person = 20, N.block = 3, D = 2,
#                       dcm.type = "DINA")
# 
# fit <- fit.FCDCM(sim$data, Q.matrix = sim$Q.matrix,
#                  block.items = sim$block.items,
#                  method = "iStEM",
#                  control.method = list(
#                    vis = FALSE, seed = 123,
#                    M = 2, B = 2, burnin.maxitr = 2,
#                    maxitr = 3, eps1 = 10, eps2 = 10,
#                    estimate.se = FALSE))
# 
# # Posterior attribute mastery probabilities
# head(fit$alpha$prob)
# 
# # Attribute mastery proportions
# colMeans(fit$alpha$prob > 0.5)
# 
# # Higher-order IRT parameters
# fit$delta$est

## ----eval=FALSE---------------------------------------------------------------
# sim <- sim.data.FCGDINA(N.person = 20, N.block = 2, I.block = 2,
#                         D = 2, model = "GDINA", fc.type = "RANK")
# 
# fit <- fit.FCGDINA(sim$data, Q.matrix = sim$Q.matrix,
#                    block.items = sim$block.items, model = "GDINA",
#                    fc.type = sim$fc.type, method = "EM",
#                    control.method = list(vis = FALSE, seed = 123,
#                                          maxitr = 2,
#                                          estimate.se = FALSE))
# 
# # Posterior attribute mastery probabilities
# head(fit$alpha$est)
# 
# # CDM item-parameter estimates
# coef(fit, type = "delta")

## ----eval=FALSE---------------------------------------------------------------
# sim <- sim.data.MGPCM(N = 20, I = 6, D = 2, length.poly = 4)
# 
# fit <- fit.MGPCM(sim$data, D = 2, method = "iStEM",
#                  control.method = list(
#                    vis = FALSE, seed = 123,
#                    M = 2, B = 2, burnin.maxitr = 2,
#                    maxitr = 3, eps1 = 10, eps2 = 10,
#                    estimate.se = FALSE))
# 
# # Category threshold parameters
# coef(fit)

## ----eval=FALSE---------------------------------------------------------------
# sim <- sim.data.MGGUM(N = 20, I = 6, D = 2, length.poly = 4)
# 
# fit <- fit.MGGUM(sim$data, D = 2, method = "iStEM",
#                  control.method = list(
#                    vis = FALSE, seed = 123,
#                    M = 2, B = 2, burnin.maxitr = 2,
#                    maxitr = 3, eps1 = 10, eps2 = 10,
#                    estimate.se = FALSE))
# 
# coef(fit)

## ----eval=FALSE---------------------------------------------------------------
# fit_rot <- rotate.MIRT(fit, rotate = "oblimin")
# 
# # Compare original and rotated loadings
# head(coef(fit))
# head(coef(fit_rot))

## ----eval=FALSE---------------------------------------------------------------
# # Item characteristic curves (ICC)
# plot(fit, type = "icc", items = 1:8)
# 
# # Person parameter distributions
# plot(fit, type = "theta")
# 
# # iStEM convergence trace
# plot(fit, type = "trace")

## ----eval=FALSE---------------------------------------------------------------
# fit <- fit.MIRT(data, model = "2PL", D = 2, method = "iStEM",
#                 control.method = list(
#                   vis = FALSE,
#                   seed = 123,      # Reproducibility
#                   M = 2, B = 2, burnin.maxitr = 2,
#                   maxitr = 3, eps1 = 10, eps2 = 10,
#                   estimate.se = FALSE))

## ----eval=FALSE---------------------------------------------------------------
# # stan code, long time
# fit <- fit.MIRT(data, model = "2PL", D = 2, method = "stan",
#                 control.method = list(chains = 1, iter = 200,
#                                       warmup = 100, cores = 1,
#                                       seed = 123))

## ----eval=FALSE---------------------------------------------------------------
# fit <- fit.MIRT(data, model = "2PL", D = 2, method = "iStEM",
#                 control.method = list(
#                   seed = 123,
#                   M = 2,            # Burn-in batches for convergence
#                   B = 2,            # Iterations per batch
#                   burnin.maxitr = 2,
#                   maxitr = 3,
#                   eps1 = 1.5,       # Geweke convergence threshold
#                   eps2 = 0.4        # MC error tolerance
#                 ))

################################# FCGDINA iStEM #################################

fit.FCGDINA.iStEM <- function(data, Q.matrix, model, block.items, fc.type,
                              control.model, control.method, .call) {
  call <- if (is.null(.call)) match.call() else .call
  control <- fc_as_control_list(control.model, "control.model")
  control.method <- fc_as_control_list(control.method, "control.method")
  common.method <- fit_common_method_control(control.method)
  method <- istem_method_control(control.method)
  method <- istem_apply_grid_control(method, control, 1L)
  istem_check_method_control(method)
  control <- istem_effective_grid_model_control(control, method)

  prep <- fcgdina_prepare(data, Q.matrix, model, block.items, fc.type)

  bounds <- list(
    delta.lower = get_ctrl("delta.lower", -4, control.method),
    delta.upper = get_ctrl("delta.upper", 4, control.method)
  )
  if (!is.finite(bounds$delta.lower) || !is.finite(bounds$delta.upper) ||
      bounds$delta.lower >= bounds$delta.upper) {
    stop("'delta.lower' must be smaller than 'delta.upper'.", call. = FALSE)
  }

  delta.current <- fcgdina_initial_delta(prep)
  prior <- rep(1.0 / prep$C, prep$C)

  set.seed(common.method$seed)

  fcgdina_batch <- function(delta.current) {
    pmatrix <- matrix(NA_real_, prep$total_delta_len, method$B)
    logLik.trace <- rep(NA_real_, method$B)
    class.post <- NULL
    for (s in seq_len(method$B)) {
      prob.class <- fcgdina_prob_class(prep, delta.current)
      class.post <- fcgdina_class_posterior(prob.class, prep, prior,
                                            grouped = TRUE)
      prior <<- fcgdina_weighted_colmean(class.post, prep$response.count)
      class.weight <- fcgdina_sample_class_weight(
        class.post, count = prep$response.count)
      delta.current <- fcgdina_fit_delta(
        prep = prep, delta.current = delta.current,
        class.weight = class.weight, method = method, bounds = bounds)
      pmatrix[, s] <- unlist(delta.current, use.names = FALSE)
      prob.class <- fcgdina_prob_class(prep, delta.current)
      logLik.trace[s] <- fcgdina_marginal_loglik(
        prob.class, prep, prior, grouped = TRUE
      )
    }
    list(delta = delta.current, pmatrix = pmatrix,
         logLik.trace = logLik.trace, class.post = class.post)
  }

  run <- istem_run_batches(
    state = list(delta = delta.current, class.post = NULL),
    method = method,
    N = prep$N,
    vis = common.method$vis,
    label = "FCGDINA",
    run_batch = function(state) {
      x <- fcgdina_batch(state$delta)
      list(
        state = list(delta = x$delta, class.post = x$class.post),
        pmatrix = x$pmatrix,
        logLik.trace = x$logLik.trace
      )
    }
  )

  chain.mat <- Reduce(cbind, run$stable.plist)
  chain.mean <- rowMeans(chain.mat)
  chain.sd <- if (method$estimate.se && ncol(chain.mat) > 1L) {
    apply(chain.mat, 1L, stats::sd)
  } else {
    rep(NA_real_, length(chain.mean))
  }
  delta.list.est <- fcgdina_unpack_delta(chain.mean, prep)
  delta.list.se <- fcgdina_unpack_delta(chain.sd, prep)
  delta.list.Rhat <- fcgdina_unpack_delta(rep(NA_real_, length(chain.mean)), prep)
  delta.store <- fcgdina_delta_store(chain.mat, prep)

  prob.class <- fcgdina_prob_class(prep, delta.list.est)
  # Final E-step with converged delta and prior
  class.post.group <- fcgdina_class_posterior(prob.class, prep, prior,
                                              grouped = TRUE)
  class.post <- fcgdina_expand_group_matrix(class.post.group, prep)
  alpha.prob <- class.post %*% prep$alpha.patterns
  colnames(alpha.prob) <- paste0("alpha", seq_len(prep$D))
  rownames(alpha.prob) <- rownames(prep$response)
  alpha.est <- (alpha.prob > 0.5) * 1L
  storage.mode(alpha.est) <- "integer"
  colnames(class.post) <- pattern_key(prep$alpha.patterns)
  rownames(class.post) <- rownames(prep$response)

  results <- list(
    npar = prep$total_delta_len + prep$C - 1L,
    method = "iStEM",
    alpha = list(est = alpha.est, se = matrix(NA_real_, prep$N, prep$D),
                 Rhat = matrix(NA_real_, prep$N, prep$D),
                 prob = alpha.prob),
    class.post = class.post,
    delta = list(est = delta.list.est, se = delta.list.se,
                 Rhat = delta.list.Rhat),
    pi = prior,
    delta.store = delta.store,
    Q.matrix = prep$Q.matrix,
    model = prep$model,
    block.items = prep$block.items,
    patterns = prep$patterns,
    patterns.total = prep$patterns.total,
    design.matrix.list = prep$design.matrix.list,
    alpha.patterns = prep$alpha.patterns,
    fc.type = prep$fc.type,
    response = prep$response,
    stan.obj = NULL,
    MCMC.obj = NULL,
    EM = NULL,
    iStEM = utils::modifyList(
      run$iStEM,
      list(
        accept.rate = 1,
        chain.mat = chain.mat,
        control = method
      )
    ),
    call = call,
    arguments = list(
      data = data,
      Q.matrix = prep$Q.matrix,
      block.items = prep$block.items,
      model = prep$model,
      fc.type = prep$fc.type,
      method = "iStEM",
      cores = common.method$cores,
      vis = common.method$vis,
      seed = common.method$seed,
      control.model = control,
      control.method = fit_effective_method_control(
        control.method, method = utils::modifyList(method, bounds),
        common = common.method)
    )
  )
  class(results) <- "FCGDINA"
  results$logLik <- logLik.FCGDINA(results)
  results
}


fcgdina_initial_delta <- function(prep) {
  delta.current <- vector("list", prep$I.states)
  for (i in seq_len(prep$I.states)) {
    nc <- prep$delta.len[i]
    if (nc == 1L) {
      delta.current[[i]] <- 0.5
    } else if (nc == 2L) {
      delta.current[[i]] <- c(0.2, 0.6)
    } else {
      delta.current[[i]] <- c(0.2, rep(0.6 / (nc - 1L), nc - 1L))
    }
  }
  delta.current
}

fcgdina_unpack_delta <- function(delta.vec, prep) {
  delta.list <- vector("list", prep$I.states)
  for (i in seq_len(prep$I.states)) {
    idx <- prep$delta_offset[i] + seq_len(prep$delta.len[i])
    delta.list[[i]] <- as.numeric(delta.vec[idx])
  }
  delta.list
}

fcgdina_delta_store <- function(chain.mat, prep) {
  out <- vector("list", prep$I.states)
  for (i in seq_len(prep$I.states)) {
    idx <- prep$delta_offset[i] + seq_len(prep$delta.len[i])
    x <- t(chain.mat[idx, , drop = FALSE])
    colnames(x) <- paste0("delta", seq_len(ncol(x)))
    out[[i]] <- x
  }
  out
}

fcgdina_prob_class <- function(prep, delta.list) {
  delta.vec <- unlist(delta.list, use.names = FALSE)
  cpp_model_FCGDINA(
    as.matrix(prep$alpha.patterns), prep$flat_design,
    prep$design.rows, prep$design.cols,
    prep$design_offset, prep$delta_offset,
    delta.vec, prep$class.map.list,
    prep$patterns.total, prep$patterns,
    prep$C, prep$I.states)
}

fcgdina_delta_link <- function(object) {
  link <- object$delta.link
  if (is.null(link)) link <- "identity"
  match.arg(link, c("identity", "logit"))
}

fcgdina_prob_class_link <- function(prep, delta.list,
                                    link = c("identity", "logit")) {
  link <- match.arg(link)
  if (link == "identity") {
    return(fcgdina_prob_class(prep, delta.list))
  }

  prob.item <- compute_fcgdina_item_prob(
    alpha = prep$alpha.patterns,
    delta.list = delta.list,
    design.matrix.list = prep$design.matrix.list,
    Q.matrix = prep$Q.matrix,
    link = "logit"
  )
  forced_choice_from_agree(prob.item, prep$patterns.total, prep$patterns)
}

fcgdina_class_posterior <- function(prob.class, prep, prior,
                                    grouped = FALSE) {
  response <- if (grouped) prep$response.unique else prep$response
  N <- if (grouped) prep$G else prep$N
  cpp_fcgdina_class_posterior(
    prob.class, response, prep$patterns, prior, N, prep$B, prep$C)
}

fcgdina_expand_group_matrix <- function(x, prep) {
  x[prep$response.group, , drop = FALSE]
}

fcgdina_weighted_colmean <- function(x, count) {
  colSums(x * count) / sum(count)
}

fcgdina_sample_class_weight <- function(class.post, count = NULL) {
  N <- nrow(class.post)
  C <- ncol(class.post)
  cum.post <- class.post
  if (C > 1L) {
    for (j in 2L:C) {
      cum.post[, j] <- cum.post[, j] + cum.post[, j - 1L]
    }
  }
  cum.post[, C] <- 1
  cls <- max.col(cum.post >= stats::runif(N), ties.method = "first")
  weight <- matrix(0, N, C)
  weight[cbind(seq_len(N), cls)] <- if (is.null(count)) 1 else count
  weight
}

fcgdina_marginal_loglik <- function(prob.class, prep, prior,
                                    grouped = TRUE) {
  response <- if (grouped) prep$response.unique else prep$response
  count <- if (grouped) prep$response.count else rep.int(1L, nrow(response))
  log.prob <- log(pmax(prob.class, 1e-16))
  log.prior <- log(pmax(prior, 1e-16))
  ll <- 0.0

  for (n in seq_len(nrow(response))) {
    lp <- log.prior
    col.idx <- 0L
    for (b in seq_len(prep$B)) {
      y <- response[n, b]
      lp <- lp + log.prob[, col.idx + y]
      col.idx <- col.idx + prep$n.obs[b]
    }
    max.lp <- max(lp)
    ll <- ll + count[n] * (max.lp + log(sum(exp(lp - max.lp))))
  }
  ll
}

fcgdina_fit_delta <- function(prep, delta.current, class.weight,
                              method, bounds) {
  for (b in seq_len(prep$B)) {
    delta.current <- fcgdina_fit_delta_block(
      prep = prep, delta.current = delta.current,
      class.weight = class.weight, method = method,
      bounds = bounds, b = b)
  }
  delta.current
}

fcgdina_fit_delta_block <- function(prep, delta.current, class.weight,
                                    method, bounds, b) {
  items <- prep$block.items[[b]]
  delta.vec <- unlist(delta.current[items], use.names = FALSE)
  opt <- stats::optim(
    par = delta.vec,
    fn = function(par) {
      delta.block <- fcgdina_unpack_delta_block(par, prep, items)
      prob.block <- fcgdina_prob_class_block(prep, delta.block, b)
      ell <- fcgdina_weighted_loglik_block(prob.block, prep, class.weight, b)
      if (!is.finite(ell)) return(1e10)
      -ell
    },
    method = "L-BFGS-B",
    lower = rep(bounds$delta.lower, length(delta.vec)),
    upper = rep(bounds$delta.upper, length(delta.vec)),
    control = list(maxit = method$optim.maxit, fnscale = 1)
  )
  delta.current[items] <- fcgdina_unpack_delta_block(opt$par, prep, items)
  delta.current
}

fcgdina_unpack_delta_block <- function(delta.vec, prep, items) {
  delta.block <- vector("list", length(items))
  idx <- 1L
  for (j in seq_along(items)) {
    nc <- prep$delta.len[items[j]]
    delta.block[[j]] <- as.numeric(delta.vec[idx:(idx + nc - 1L)])
    idx <- idx + nc
  }
  delta.block
}

fcgdina_prob_class_block <- function(prep, delta.block, b) {
  delta.vec <- unlist(delta.block, use.names = FALSE)
  cpp_model_FCGDINA(
    as.matrix(prep$alpha.patterns),
    prep$block.flat.design[[b]],
    prep$block.design.rows[[b]],
    prep$block.design.cols[[b]],
    prep$block.design.offset[[b]],
    prep$block.delta.offset[[b]],
    delta.vec,
    prep$block.class.map.list[[b]],
    list(prep$patterns.total.local[[b]]),
    list(prep$patterns.local[[b]]),
    prep$C,
    length(prep$block.items[[b]]))
}

fcgdina_weighted_loglik_block <- function(prob.block, prep, class.weight, b) {
  N <- nrow(class.weight)
  response <- if (N == prep$G) prep$response.unique[, b] else prep$response[, b]
  cpp_fcgdina_weighted_loglik_block(
    prob.block, response, class.weight, N, prep$C)
}

# ---- Shared FCGDINA preparation -----------------------------------------------

fcgdina_prepare <- function(data, Q.matrix, model, block.items, fc.type) {
  data.matrix <- as.matrix(data)
  N <- nrow(data.matrix)
  B <- ncol(data.matrix)
  is.pattern.index.data <- is.numeric(data.matrix) || is.integer(data.matrix)

  model <- toupper(model[1L])
  if (!model %in% c("DINA", "DINO", "ACDM", "GDINA")) {
    stop("'model' must be one of: DINA, DINO, ACDM, GDINA.", call. = FALSE)
  }

  fc.type <- normalize_fc_type(fc.type, B)

  if (is.null(block.items)) {
    if (is.pattern.index.data) {
      stop("'block.items' is required when 'data' is an integer matrix of pattern indices.", call. = FALSE)
    }
    if (any(fc.type != "RANK")) {
      stop("'block.items' is required for MOLE/PICK data because partial rankings do not identify all block items.", call. = FALSE)
    }
    block.items <- get.block.items.from.data(data)
  }
  if (!is.list(block.items) || length(block.items) != B) {
    stop("'block.items' must be a list of length ", B, ".", call. = FALSE)
  }
  block.items <- lapply(block.items, as.integer)

  all.items <- unlist(block.items, use.names = FALSE)
  I.states <- length(all.items)
  if (anyDuplicated(all.items)) {
    stop("'block.items' contains duplicate items.", call. = FALSE)
  }
  if (!setequal(all.items, seq_len(I.states))) {
    stop("'block.items' must contain exactly item indices 1:", I.states, ".", call. = FALSE)
  }

  Q.matrix <- as.matrix(Q.matrix)
  if (anyNA(Q.matrix) || !all(Q.matrix %in% c(0, 1))) {
    stop("'Q.matrix' must be a 0/1 matrix.", call. = FALSE)
  }
  if (nrow(Q.matrix) != I.states) {
    stop("'Q.matrix' must have ", I.states, " rows.", call. = FALSE)
  }
  if (any(rowSums(Q.matrix) < 1L)) {
    stop("Each statement must require at least one attribute.", call. = FALSE)
  }
  D <- ncol(Q.matrix)
  storage.mode(Q.matrix) <- "numeric"

  pat <- generate_fc_permutation_patterns(block.items, fc.type)
  patterns.total <- pat$patterns.total
  patterns <- pat$patterns

  if (is.pattern.index.data) {
    response <- matrix(as.integer(data.matrix), nrow = N, ncol = B,
                       dimnames = dimnames(data.matrix))
  } else {
    response <- get.response.from.data(data, block.items, fc.type)
  }
  for (b in seq_len(B)) {
    if (anyNA(response[, b]) ||
        any(response[, b] < 1L | response[, b] > nrow(patterns[[b]]))) {
      stop("'data' contains invalid pattern indices for block ", b,
           "; valid values are 1 through ", nrow(patterns[[b]]), ".", call. = FALSE)
    }
  }
  storage.mode(response) <- "integer"

  alpha.patterns <- as.matrix(do.call(expand.grid, rep(list(0:1), D)))
  storage.mode(alpha.patterns) <- "integer"
  colnames(alpha.patterns) <- paste0("A", seq_len(D))
  C <- nrow(alpha.patterns)

  design.matrix.list <- vector("list", I.states)
  delta.len <- integer(I.states)
  design.rows <- integer(I.states)
  design.cols <- integer(I.states)
  class.map.list <- vector("list", I.states)
  class.map.mat <- matrix(NA_integer_, I.states, C)

  for (i in seq_len(I.states)) {
    att <- which(Q.matrix[i, ] > 0)
    patterns.i <- if (length(att) > 0L) {
      attributepattern(length(att))
    } else {
      matrix(0L, 1L, 0L)
    }
    X.i <- get_design_matrix_cdm(patterns.i, model)

    design.matrix.list[[i]] <- X.i
    delta.len[i] <- ncol(X.i)
    design.rows[i] <- nrow(X.i)
    design.cols[i] <- ncol(X.i)

    if (length(att) > 0L) {
      reduced.key <- pattern_key(alpha.patterns[, att, drop = FALSE])
      full.map <- match(reduced.key, pattern_key(patterns.i))
    } else {
      full.map <- rep(1L, C)
    }
    class.map.list[[i]] <- as.integer(full.map)
    class.map.mat[i, ] <- as.integer(full.map)
  }

  total_design_entries <- sum(design.rows * design.cols)
  flat_design <- numeric(total_design_entries)
  design_offset <- integer(I.states)
  delta_offset <- integer(I.states)
  off_design <- 0L
  off_delta <- 0L
  for (i in seq_len(I.states)) {
    design_offset[i] <- off_design
    delta_offset[i] <- off_delta
    X.i <- design.matrix.list[[i]]
    nr <- nrow(X.i)
    nc <- ncol(X.i)
    flat_design[(off_design + 1L):(off_design + nr * nc)] <- as.numeric(t(X.i))
    off_design <- off_design + nr * nc
    off_delta <- off_delta + nc
  }
  total_delta_len <- off_delta

  fc.type.int <- match(fc.type, c("RANK", "MOLE", "PICK"))
  n.items <- vapply(block.items, length, integer(1L))
  n.total <- vapply(patterns.total, nrow, integer(1L))
  n.obs <- vapply(patterns, nrow, integer(1L))
  fc.len <- as.integer(ifelse(fc.type == "RANK", n.items,
                              ifelse(fc.type == "MOLE", 2L, 1L)))

  response.key <- apply(response, 1L, paste, collapse = "\r")
  response.levels <- unique(response.key)
  response.group <- match(response.key, response.levels)
  response.first <- match(response.levels, response.key)
  response.unique <- response[response.first, , drop = FALSE]
  response.count <- as.integer(
    tabulate(response.group, nbins = length(response.levels))
  )
  storage.mode(response.unique) <- "integer"

  block.items.flat <- integer(sum(n.items))
  patterns.total.flat <- integer(sum(n.total * n.items))
  patterns.obs.flat <- integer(sum(n.obs * fc.len))
  patterns.total.local <- vector("list", B)
  patterns.local <- vector("list", B)
  obs.pattern.start <- c(1L, cumsum(n.obs) + 1L)
  obs.full.start <- integer(sum(n.obs) + 1L)
  obs.full.index <- integer(sum(n.total))

  item.start <- c(1L, cumsum(n.items) + 1L)
  total.start <- c(1L, cumsum(n.total * n.items) + 1L)
  obs.start <- c(1L, cumsum(n.obs * fc.len) + 1L)

  obs.full.start[1L] <- 1L
  obs.link.pos <- 1L
  obs.global <- 1L
  for (b in seq_len(B)) {
    items <- block.items[[b]]

    pat.total <- patterns.total[[b]]
    pat.total.local <- matrix(match(pat.total, items), nrow = n.total[b])
    patterns.total.local[[b]] <- pat.total.local
    total.idx <- seq.int(total.start[b], total.start[b + 1L] - 1L)
    patterns.total.flat[total.idx] <- as.integer(t(pat.total.local))

    pat.obs <- patterns[[b]]
    pat.obs.local <- matrix(match(pat.obs, items), nrow = n.obs[b])
    patterns.local[[b]] <- pat.obs.local
    obs.idx <- seq.int(obs.start[b], obs.start[b + 1L] - 1L)
    patterns.obs.flat[obs.idx] <- as.integer(t(pat.obs.local))

    for (tt in seq_len(n.obs[b])) {
      full.idx <- switch(
        fc.type[b],
        RANK = tt,
        MOLE = which(pat.total.local[, 1L] == pat.obs.local[tt, 1L] &
                       pat.total.local[, n.items[b]] == pat.obs.local[tt, 2L]),
        PICK = which(pat.total.local[, 1L] == pat.obs.local[tt, 1L])
      )
      n.link <- length(full.idx)
      obs.full.index[obs.link.pos:(obs.link.pos + n.link - 1L)] <-
        as.integer(full.idx)
      obs.link.pos <- obs.link.pos + n.link
      obs.global <- obs.global + 1L
      obs.full.start[obs.global] <- obs.link.pos
    }

    item.idx <- seq.int(item.start[b], item.start[b + 1L] - 1L)
    block.items.flat[item.idx] <- as.integer(items)
  }
  obs.full.index <- obs.full.index[seq_len(obs.link.pos - 1L)]

  block.flat.design <- vector("list", B)
  block.design.rows <- vector("list", B)
  block.design.cols <- vector("list", B)
  block.design.offset <- vector("list", B)
  block.delta.offset <- vector("list", B)
  block.class.map.list <- vector("list", B)
  for (b in seq_len(B)) {
    items <- block.items[[b]]
    block.design.rows[[b]] <- design.rows[items]
    block.design.cols[[b]] <- design.cols[items]
    block.design.offset[[b]] <- integer(length(items))
    block.delta.offset[[b]] <- integer(length(items))
    block.class.map.list[[b]] <- class.map.list[items]

    off.design <- 0L
    off.delta <- 0L
    flat.design <- numeric(sum(design.rows[items] * design.cols[items]))
    for (j in seq_along(items)) {
      i <- items[j]
      X.i <- design.matrix.list[[i]]
      nr <- nrow(X.i)
      nc <- ncol(X.i)
      n.entries <- nr * nc
      block.design.offset[[b]][j] <- off.design
      block.delta.offset[[b]][j] <- off.delta
      flat.design[(off.design + 1L):(off.design + n.entries)] <- as.numeric(t(X.i))
      off.design <- off.design + n.entries
      off.delta <- off.delta + nc
    }
    block.flat.design[[b]] <- flat.design
  }

  list(
    data = data.matrix,
    response = response,
    response.unique = response.unique,
    response.count = response.count,
    response.group = as.integer(response.group),
    G = length(response.levels),
    N = N,
    B = B,
    I.states = I.states,
    D = D,
    C = C,
    model = model,
    Q.matrix = Q.matrix,
    block.items = block.items,
    fc.type = fc.type,
    patterns = patterns,
    patterns.total = patterns.total,
    patterns.local = patterns.local,
    patterns.total.local = patterns.total.local,
    alpha.patterns = alpha.patterns,
    design.matrix.list = design.matrix.list,
    class.map.list = class.map.list,
    delta.len = delta.len,
    design.rows = design.rows,
    design.cols = design.cols,
    flat_design = flat_design,
    design_offset = design_offset,
    delta_offset = delta_offset,
    block.flat.design = block.flat.design,
    block.design.rows = block.design.rows,
    block.design.cols = block.design.cols,
    block.design.offset = block.design.offset,
    block.delta.offset = block.delta.offset,
    block.class.map.list = block.class.map.list,
    total_design_entries = total_design_entries,
    total_delta_len = total_delta_len,
    class_map_mat = class.map.mat,
    fc.type.int = fc.type.int,
    n.items = n.items,
    n.total = n.total,
    n.obs = n.obs,
    fc.len = fc.len,
    obs.pattern.start = obs.pattern.start,
    obs.full.start = obs.full.start,
    obs.full.index = obs.full.index,
    block.items.flat = block.items.flat,
    patterns.total.flat = patterns.total.flat,
    patterns.obs.flat = patterns.obs.flat,
    item.start = item.start,
    total.start = total.start,
    obs.start = obs.start
  )
}

#' @describeIn get.fit.index MIRT model: binary responses with Gauss--Hermite
#'   quadrature over \eqn{D}-dimensional latent space.
#' @export
get.fit.index.MIRT <- function(object, ...) {
  model <- object$arguments$model
  response <- object$arguments$response
  par <- object$par$est
  D <- object$arguments$D
  Q.matrix <- object$Q.matrix

  logLik <- object$logLik
  pi <- as.vector(attr(logLik, "pi"))
  theta.norm <- attr(logLik, "theta.norm")

  N <- nrow(response)
  I <- ncol(response)
  npar <- object$npar

  extract.free.par <- function(par, ...) {
    par.vec <- numeric(0)
    if (model != "m1pl") {
      for (i in 1:I) {
        for (j in 1:D) {
          if (Q.matrix[i, j] == 1) par.vec <- c(par.vec, par[i, j])
        }
      }
    }
    par.vec <- c(par.vec, par[, D + 1L])
    if (model %in% c("m3pl", "m4pl")) par.vec <- c(par.vec, par[, D + 2L])
    if (model == "m4pl") par.vec <- c(par.vec, par[, D + 3L])
    return(par.vec)
  }

  expand.free.par <- function(par.vec, ...) {
    par <- matrix(0, nrow = I, ncol = D + 3L)
    idx <- 1L
    if (model == "m1pl") {
      par[, 1:D] <- 1
    } else {
      for (i in 1:I) {
        for (j in 1:D) {
          if (Q.matrix[i, j] == 1) { par[i, j] <- par.vec[idx]; idx <- idx + 1L }
          else { par[i, j] <- 0.0 }
        }
      }
    }
    for (i in 1:I) { par[i, D + 1L] <- par.vec[idx]; idx <- idx + 1L }

    if (model %in% c("m1pl", "m2pl")) {
      par[, D + 2L] <- 0
    } else {
      for (i in 1:I) { par[i, D + 2L] <- par.vec[idx]; idx <- idx + 1L }
    }

    if (model %in% c("m1pl", "m2pl", "m3pl")) {
      par[, D + 3L] <- 1
    } else {
      for (i in 1:I) { par[i, D + 3L] <- par.vec[idx]; idx <- idx + 1L }
    }
    return(par)
  }

  par.vec <- extract.free.par(par, model, Q.matrix, I, D)

  loglik.fun <- function(par.vec, ...){
    par <- expand.free.par(par.vec, ...)
    prob_new <- model.MIRT(theta.norm, par)

    eps               <- .Machine$double.xmin
    log_prob          <- log(pmax(prob_new,        eps))
    log_one_minus_prob <- log(pmax(1 - prob_new,   eps))
    log_pi            <- log(pi)

    pi.length <- nrow(theta.norm)
    log_L_theta_Xi <- matrix(0, N, pi.length)
    for (p in 1:N) {
      response.p <- matrix(response[p, ], pi.length, I, byrow = TRUE)
      log_L_theta_Xi[p, ] <- rowSums(
        response.p * log_prob + (1 - response.p) * log_one_minus_prob
      ) + log_pi
    }

    logLik.value <- sum(apply(log_L_theta_Xi, 1, function(x) {
      m <- max(x)
      m + log(sum(exp(x - m)))
    }))
    return(logLik.value)
  }

  prob.fun <- function(par.vec, ...){
    par <- expand.free.par(par.vec, ...)
    prob_out <- model.MIRT(theta.norm, par)
    return(prob_out)
  }

  good.of.fit(
    par.vec    = par.vec,
    loglik.fun = loglik.fun,
    prob.fun   = prob.fun,
    response   = response,
    npar       = npar,
    pi         = pi,
    response.type = "binary",
    ...
  )
}

################################# FCGGUM iStEM #################################

fit.FCGGUM.iStEM <- function(data, Q.matrix = NULL, block.items = NULL,
                             D = NULL, fc.type = "RANK",
                             control.model = NULL, control.method = NULL,
                             .call = NULL) {

  call <- if (is.null(.call)) match.call() else .call
  control.model <- fc_as_control_list(control.model, "control.model")
  control.method <- fc_as_control_list(control.method, "control.method")
  common.method <- fit_common_method_control(control.method)
  vis <- common.method$vis
  method <- istem_method_control(control.method)
  set.seed(common.method$seed)

  fc <- istem_prepare_fc_data(data, block.items, fc.type)
  I <- fc$I
  N <- fc$N
  response.group <- istem_response_groups(fc$response)
  response.fit <- response.group$response
  count.fit <- response.group$count
  if (!is.null(Q.matrix)) {
    Q.matrix <- as.matrix(Q.matrix)
    if (anyNA(Q.matrix) || !all(Q.matrix %in% c(-1, 0, 1))) {
      stop("'Q.matrix' must be a matrix containing only -1, 0, or 1.",
           call. = FALSE)
    }
    if (nrow(Q.matrix) != I) {
      stop("'Q.matrix' must have ", I, " rows, one for each item.",
           call. = FALSE)
    }
    if (any(rowSums(abs(Q.matrix)) < 1L)) {
      stop("Each item must measure at least one trait (each row of 'Q.matrix' needs a non-zero entry).",
           call. = FALSE)
    }
    D.q <- ncol(Q.matrix)
    if (!is.null(D) && D != D.q) {
      warning("'D' (", D, ") does not match ncol(Q.matrix) (", D.q,
              "); using ncol(Q.matrix).")
    }
    D <- D.q
  }
  if (is.null(D) || is.na(D)) {
    stop("Cannot determine 'D'. Please provide either 'Q.matrix' or 'D'.",
         call. = FALSE)
  }
  D <- as.integer(D)
  if (D < 2L) {
    stop("FCGGUM requires at least 2 latent dimensions (D >= 2).",
         call. = FALSE)
  }
  if (is.null(Q.matrix)) {
    Q.matrix <- matrix(1, I, D)
  }
  storage.mode(Q.matrix) <- "numeric"
  method <- istem_apply_grid_control(method, control.model, D)
  istem_check_method_control(method)
  control.model <- istem_effective_grid_model_control(control.model, method)

  length.poly <- rep(2L, I)
  max_poly <- 2L
  prior <- mggum_istem_prior_control(control.model)
  bounds <- list(
    a.lower = get_ctrl("a.lower", 1e-4, control.method),
    a.upper = get_ctrl("a.upper", 6, control.method),
    delta.lower = get_ctrl("delta.lower", -6, control.method),
    delta.upper = get_ctrl("delta.upper", 6, control.method),
    tau.gap.lower = get_ctrl("tau.gap.lower", 1e-4, control.method),
    tau.gap.upper = get_ctrl("tau.gap.upper", 6, control.method)
  )
  mggum_istem_check_bounds(bounds)

  theta_mu <- array(get_ctrl("theta.mu", rep(0, D), control.model), dim = D)
  include_corr <- D > 1L && !method$fix.corr
  corr_df <- free_corr_npar(D, include_corr)
  par <- mggum_istem_initial_par(Q.matrix, length.poly, max_poly, bounds,
                                 prior)

  state <- list(
    response = response.fit,
    response.count = count.fit,
    par = par,
    theta = istem_random_theta(response.group$G, D, theta_mu, method$theta.lower,
                               method$theta.upper),
    Corr = istem_random_corr(D, include_corr),
    theta_mu = theta_mu,
    Q.matrix = Q.matrix,
    length.poly = length.poly,
    max.poly = max_poly,
    block.items = fc$block.items,
    patterns = fc$patterns,
    patterns.total = fc$patterns.total,
    optim.maxit = method$optim.maxit,
    bounds = bounds,
    prior = prior
  )

  run <- istem_run(
    state = state,
    method = method,
    N = N,
    vis = vis,
    label = "FCGGUM",
    update_parameters = fcggum_istem_update_parameters,
    param_vec = function(state) {
      mggum_istem_param_vec(state$par, state$Corr, state$Q.matrix,
                            state$length.poly, include_corr)
    },
    sample_theta = fcggum_istem_sample_theta,
    logLik_fun = function(state) {
      fcggum_istem_loglik_trace(
        state, method$L, method$theta.lower, method$theta.upper
      )
    }
  )

  final <- mggum_istem_param_unpack(
    pv = run$chain.mean,
    pv_se = run$chain.sd,
    pv_rhat = NULL,
    I = I, D = D,
    Q.matrix = Q.matrix,
    length.poly = length.poly,
    max_poly = max_poly,
    Corr = run$state$Corr,
    include_corr = include_corr
  )

  theta.est <- istem_expand_group_matrix(
    run$theta.est, response.group$group, rownames(fc$response))
  theta.se <- istem_expand_group_matrix(
    run$theta.se, response.group$group, rownames(fc$response))
  theta_est <- istem_named_theta(theta.est, theta.se, fc$response, D)
  par <- final$par
  par.se <- final$par.se
  par.Rhat <- final$par.rhat
  par.free <- mggum_par_free_mask(Q.matrix, length.poly, max_poly)
  colnames(par) <- colnames(par.se) <- colnames(par.Rhat) <- colnames(par.free) <-
    c(paste0("a", seq_len(D)),
      paste0("delta", seq_len(D)),
      paste0("tau", 0:(max_poly - 1L)))
  rownames(par) <- rownames(par.se) <- rownames(par.Rhat) <- rownames(par.free) <-
    paste0("item", fc$all.items)
  par.se[!par.free] <- NA_real_
  par.Rhat[!par.free] <- NA_real_
  npar <- sum(par.free) + corr_df

  Corr.Rhat <- final$Corr.rhat
  dimnames(Corr.Rhat) <- list(paste0("Dim.", seq_len(D)), paste0("Dim.", seq_len(D)))

  results <- list(
    npar = npar,
    method = "iStEM",
    theta = theta_est,
    par = list(est = par, se = par.se, Rhat = par.Rhat, free = par.free),
    Corr = list(est = final$Corr, se = final$Corr.se, Rhat = Corr.Rhat),
    stan.obj = NULL,
    MCMC.obj = NULL,
    Q.matrix = Q.matrix,
    block.items = fc$block.items,
    fc.type = fc$fc.type,
    response = fc$response,
    length.poly = length.poly,
    patterns = fc$patterns,
    patterns.total = fc$patterns.total,
    call = call,
    arguments = list(
      data = data,
      Q.matrix = Q.matrix,
      block.items = fc$block.items,
      D = D,
      fc.type = fc$fc.type,
      method = "iStEM",
      cores = common.method$cores,
      vis = vis,
      seed = common.method$seed,
      control.model = control.model,
      control.method = fit_effective_method_control(
        control.method, method = c(method, bounds), common = common.method
      )
    ),
    iStEM = utils::modifyList(run$iStEM, list(prior = prior))
  )

  class(results) <- "FCGGUM"
  results$logLik <- logLik.FCGGUM(results, L = method$L,
                                    theta.low = method$theta.lower,
                                    theta.up = method$theta.upper)
  results
}

fcggum_istem_sample_theta <- function(state, theta_grid_length,
                                      theta_lower, theta_upper) {
  chol <- istem_corr_chol(state$Corr)
  cpp_gibbs_fcggum_theta(
    theta_ = state$theta,
    par_ = state$par,
    response_ = state$response,
    theta_mu_ = state$theta_mu,
    chol_corr_ = chol$chol,
    log_diag_sum = chol$log_diag_sum,
    patterns_total = state$patterns.total,
    patterns = state$patterns,
    step = theta_grid_length,
    lower = theta_lower,
    upper = theta_upper
  )
}

fcggum_istem_update_parameters <- function(state) {
  for (b in seq_along(state$block.items)) {
    items <- state$block.items[[b]]
    state$par[items, ] <- fcggum_istem_fit_block(
      y = state$response[, b],
      theta = state$theta,
      par_block = state$par[items, , drop = FALSE],
      q_block = state$Q.matrix[items, , drop = FALSE],
      patterns_total = cpp_istem_local_patterns(
        as.matrix(state$patterns.total[[b]]), as.integer(items)
      ),
      patterns = cpp_istem_local_patterns(
        as.matrix(state$patterns[[b]]), as.integer(items)
      ),
      weight = istem_state_weight(state),
      optim_maxit = state$optim.maxit,
      bounds = state$bounds,
      prior = state$prior
    )
  }
  state
}

fcggum_istem_fit_block <- function(y, theta, par_block, q_block,
                                   patterns_total, patterns, weight, optim_maxit,
                                   bounds, prior) {
  D <- ncol(q_block)
  active <- q_block != 0
  x0 <- c(par_block[, seq_len(D), drop = FALSE][active],
          par_block[, D + seq_len(D), drop = FALSE][active],
          -par_block[, D + D + 2L])
  delta_lower <- ifelse(q_block[active] > 0, max(0, bounds$delta.lower),
                        bounds$delta.lower)
  delta_upper <- ifelse(q_block[active] > 0, bounds$delta.upper,
                        min(0, bounds$delta.upper))
  lower <- c(rep(bounds$a.lower, sum(active)),
             delta_lower,
             rep(bounds$tau.gap.lower, nrow(par_block)))
  upper <- c(rep(bounds$a.upper, sum(active)),
             delta_upper,
             rep(bounds$tau.gap.upper, nrow(par_block)))

  obj <- function(x) {
    block <- fcggum_istem_x_to_block(x, par_block, q_block)
    ll <- cpp_fcggum_block_loglik_weighted(
      theta = theta,
      par_block = block,
      response = as.integer(y),
      patterns_total = patterns_total,
      patterns = patterns,
      weight = weight
    )
    lp <- 0
    for (i in seq_len(nrow(block))) {
      item <- list(
        a = block[i, seq_len(D)],
        delta = block[i, D + seq_len(D)],
        tau = block[i, D + D + 1:2]
      )
      lp <- lp + mggum_istem_item_log_prior(item, q_block[i, ], prior)
    }
    if (!is.finite(lp)) return(Inf)
    -ll - lp
  }

  opt <- tryCatch(
    stats::optim(
      par = x0,
      fn = obj,
      method = "L-BFGS-B",
      lower = lower,
      upper = upper,
      control = list(maxit = optim_maxit)
    ),
    error = function(e) NULL
  )
  if (is.null(opt) || !is.finite(opt$value)) {
    return(par_block)
  }
  fcggum_istem_x_to_block(opt$par, par_block, q_block)
}

fcggum_istem_x_to_block <- function(x, par_block, q_block) {
  D <- ncol(q_block)
  I <- nrow(q_block)
  active <- q_block != 0
  idx <- 1L
  out <- par_block
  a <- matrix(0, I, D)
  n_active <- sum(active)
  if (n_active > 0L) {
    a[active] <- x[idx:(idx + n_active - 1L)]
    idx <- idx + n_active
  }
  delta <- matrix(0, I, D)
  if (n_active > 0L) {
    delta[active] <- x[idx:(idx + n_active - 1L)]
    idx <- idx + n_active
  }
  tau_gap <- x[idx:(idx + I - 1L)]
  out[, seq_len(D)] <- a
  out[, D + seq_len(D)] <- delta
  out[, D + D + 1L] <- 0
  out[, D + D + 2L] <- -tau_gap
  out
}

#' @describeIn get.fit.index FCGGUM model: forced-choice unfolding with nominal
#'   binary expansion; within-block pairs excluded.
#' @export
get.fit.index.FCGGUM <- function(object, ...) {
  response <- object$response
  par <- object$par$est
  D <- object$arguments$D
  Q.matrix <- object$Q.matrix
  block.items <- object$block.items
  patterns <- object$patterns
  patterns.total <- object$patterns.total

  logLik.obj <- object$logLik
  pi <- as.vector(attr(logLik.obj, "pi"))
  theta.norm <- attr(logLik.obj, "theta.norm")
  N <- nrow(response)
  B <- length(block.items)
  I <- length(unlist(block.items))
  length.poly <- object$length.poly
  max_poly <- max(length.poly)
  npar <- object$npar

  n_cat <- sapply(patterns, nrow)
  n_bin <- sum(n_cat - 1L)
  response.bin <- matrix(0L, N, n_bin)
  indicator.block <- integer(n_bin)
  idx.resp <- 0L
  for (b in 1:B) {
    Kb <- n_cat[b]
    if (Kb > 1L) {
      for (j in 1:(Kb - 1L)) {
        response.bin[, idx.resp + j] <- as.integer(response[, b] == j)
        indicator.block[idx.resp + j] <- b
      }
      idx.resp <- idx.resp + (Kb - 1L)
    }
  }

  extract.free.par <- function(par, ...) {
    par.vec <- numeric(0)
    for (i in 1:I) {
      for (j in 1:D) {
        if (abs(Q.matrix[i, j]) == 1) par.vec <- c(par.vec, par[i, j])
      }
    }
    for (i in 1:I) {
      for (j in 1:D) {
        if (abs(Q.matrix[i, j]) == 1) par.vec <- c(par.vec, par[i, D + j])
      }
    }
    for (i in 1:I) {
      Ki <- length.poly[i]
      if (Ki > 1) {
        cols <- (D + D + 2):(D + D + Ki)
        par.vec <- c(par.vec, par[i, cols])
      }
    }
    return(par.vec)
  }

  expand.free.par <- function(par.vec, ...) {
    par <- matrix(NA_real_, nrow = I, ncol = D + D + max_poly)
    idx <- 1L
    for (i in 1:I) {
      for (j in 1:D) {
        if (abs(Q.matrix[i, j]) == 1) { par[i, j] <- par.vec[idx]; idx <- idx + 1L }
        else { par[i, j] <- 0.0 }
      }
    }
    for (i in 1:I) {
      for (j in 1:D) {
        if (abs(Q.matrix[i, j]) == 1) {
          par[i, D + j] <- par.vec[idx]
          idx <- idx + 1L
        } else {
          par[i, D + j] <- 0.0
        }
      }
    }
    par[, D + D + 1] <- 0
    for (i in 1:I) {
      Ki <- length.poly[i]
      if (Ki > 1) {
        for (k in 1:(Ki - 1)) {
          par[i, D + D + 1 + k] <- par.vec[idx]
          idx <- idx + 1L
        }
      }
    }
    return(par)
  }

  par.vec <- extract.free.par(par)

  response.full <- matrix(0, N, sum(n_cat))
  idx.full <- 0L
  for (b in 1:B) {
    Kb <- n_cat[b]
    for (p in 1:N) {
      response.full[p, idx.full + response[p, b]] <- 1
    }
    idx.full <- idx.full + Kb
  }

  loglik.fun <- function(par.vec, ...) {
    par <- expand.free.par(par.vec, ...)
    prob_new <- model.FCGGUM(theta.norm, par, patterns.total, patterns)

    eps        <- .Machine$double.xmin
    log_prob   <- log(pmax(prob_new, eps))
    log_pi     <- log(pi)

    pi.length <- nrow(theta.norm)
    log_L_theta_Xi <- matrix(0, N, pi.length)
    for (p in 1:N) {
      response.p <- matrix(response.full[p, ], pi.length, ncol(prob_new), byrow = TRUE)
      log_L_theta_Xi[p, ] <- rowSums(response.p * log_prob) + log_pi
    }

    logLik.value <- sum(apply(log_L_theta_Xi, 1, function(x) {
      m <- max(x)
      m + log(sum(exp(x - m)))
    }))
    return(logLik.value)
  }

  prob.fun <- function(par.vec, ...) {
    par <- expand.free.par(par.vec, ...)
    prob_full <- model.FCGGUM(theta.norm, par, patterns.total, patterns)
    prob_out <- matrix(0, nrow(prob_full), n_bin)
    idx.full <- 0L
    idx.bin  <- 0L
    for (b in 1:B) {
      Kb <- n_cat[b]
      if (Kb > 1L) {
        prob_out[, (idx.bin + 1L):(idx.bin + Kb - 1L)] <-
          prob_full[, (idx.full + 1L):(idx.full + Kb - 1L)]
        idx.bin  <- idx.bin + (Kb - 1L)
      }
      idx.full <- idx.full + Kb
    }
    return(prob_out)
  }

  good.of.fit(
    par.vec          = par.vec,
    loglik.fun       = loglik.fun,
    prob.fun         = prob.fun,
    response         = response.bin,
    npar             = npar,
    pi               = pi,
    response.type    = "nominal",
    nominal.groups   = indicator.block,
    ...
  )
}

################################# TIRT iStEM #################################

fit.TIRT.iStEM <- function(data, Q.matrix, block.items = NULL,
                           fc.type = "RANK",
                           control.model = NULL, control.method = NULL,
                           .call = NULL) {

  call <- if (is.null(.call)) match.call() else .call
  control.model <- fc_as_control_list(control.model, "control.model")
  control.method <- fc_as_control_list(control.method, "control.method")
  common.method <- fit_common_method_control(control.method)
  vis <- common.method$vis
  method <- istem_method_control(control.method)
  set.seed(common.method$seed)

  prep <- tirt_istem_prepare_data(data, Q.matrix, block.items, fc.type,
                                  control.model)
  D <- prep$D
  N <- prep$N.person
  G <- prep$G
  I.states <- prep$I.states
  method <- istem_apply_grid_control(method, control.model, D)
  istem_check_method_control(method)
  control.model <- istem_effective_grid_model_control(control.model, method)

  prior <- tirt_istem_prior_control(control.model)
  bounds <- list(
    lambda.lower = get_ctrl("lambda.lower", prior$lambda.alpha, control.method),
    lambda.upper = get_ctrl("lambda.upper", prior$lambda.beta, control.method),
    psi.lower = get_ctrl("psi.lower", 0.01, control.method),
    psi.upper = get_ctrl("psi.upper", 5, control.method),
    gamma.lower = get_ctrl("gamma.lower", -6, control.method),
    gamma.upper = get_ctrl("gamma.upper", 6, control.method)
  )
  tirt_istem_check_bounds(bounds)

  theta_mu <- array(get_ctrl("theta.mu", rep(0, D), control.model), dim = D)
  init <- tirt_istem_initial_par(prep, prior, bounds)
  include_corr <- D > 1L && !method$fix.corr

  state <- c(
    prep,
    init,
    list(
      theta = istem_random_theta(G, D, theta_mu, method$theta.lower,
                                 method$theta.upper),
      Corr = istem_random_corr(D, include_corr),
      theta_mu = theta_mu,
      optim.maxit = method$optim.maxit,
      bounds = bounds,
      prior = prior
    )
  )

  run <- istem_run(
    state = state,
    method = method,
    N = N,
    vis = vis,
    label = "TIRT",
    update_parameters = tirt_istem_update_parameters,
    sample_theta = tirt_istem_sample_theta,
    param_vec = function(state) {
      tirt_istem_param_vec(state$lambda, state$psi, state$gamma,
                           state$Idx.lambda.est, state$Idx.psi.est,
                           state$Idx.gamma.est, state$Corr, include_corr)
    },
    logLik_fun = function(state) {
      tirt_istem_loglik_trace(
        state, method$L, method$theta.lower, method$theta.upper
      )
    }
  )

  final <- tirt_istem_param_unpack(
    pv = run$chain.mean,
    pv_se = run$chain.sd,
    pv_rhat = NULL,
    state = run$state,
    include_corr = include_corr
  )

  theta.est <- istem_expand_group_matrix(
    run$theta.est, prep$response.group, rownames(prep$response))
  theta.se <- istem_expand_group_matrix(
    run$theta.se, prep$response.group, rownames(prep$response))
  theta_est <- istem_named_theta(theta.est, theta.se, prep$response, D)

  lambda_sign <- apply(prep$Q.matrix, 1, function(x) {
    nz <- which(x != 0)
    if (length(nz) == 0L) return(1)
    sign(x[nz[1L]])
  })
  lambda_signed <- final$lambda * lambda_sign
  lambda.free <- rep(FALSE, I.states)
  lambda.free[prep$Idx.lambda.est] <- TRUE
  psi.free <- rep(FALSE, I.states)
  psi.free[prep$Idx.psi.est] <- TRUE
  gamma.free <- rep(FALSE, length(final$gamma))
  gamma.free[prep$Idx.gamma.est] <- TRUE

  psi2 <- final$psi^2
  psi2.se <- 2 * final$psi * final$psi.se
  psi2.rhat <- final$psi.rhat
  final$lambda.se[!lambda.free] <- NA_real_
  final$lambda.rhat[!lambda.free] <- NA_real_
  psi2.se[!psi.free] <- NA_real_
  psi2.rhat[!psi.free] <- NA_real_
  final$gamma.se[!gamma.free] <- NA_real_
  final$gamma.rhat[!gamma.free] <- NA_real_

  par_est <- list(
    est = cbind(lambda_signed * abs(prep$Q.matrix), psi2),
    se = cbind(final$lambda.se * abs(prep$Q.matrix), psi2.se),
    Rhat = cbind(final$lambda.rhat * abs(prep$Q.matrix), psi2.rhat),
    free = tirt_par_free_mask(prep$Q.matrix, lambda.free, psi.free)
  )
  colnames(par_est$est) <- colnames(par_est$se) <- colnames(par_est$Rhat) <-
    colnames(par_est$free) <-
    c(paste0("Dim.", seq_len(D)), "psi2")
  rownames(par_est$est) <- rownames(par_est$se) <- rownames(par_est$Rhat) <-
    rownames(par_est$free) <- paste0("item", seq_len(I.states))
  par_est$se[!par_est$free] <- NA_real_
  par_est$Rhat[!par_est$free] <- NA_real_

  gamma.matrix <- tirt_istem_gamma_matrix(final$gamma, final$gamma.se,
                                          prep$pairs.matrix, I.states)

  Corr.Rhat <- final$Corr.rhat
  dimnames(Corr.Rhat) <- list(paste0("Dim.", seq_len(D)), paste0("Dim.", seq_len(D)))

  results <- list(
    npar = length(run$chain.mean),
    method = "iStEM",
    theta = theta_est,
    par = par_est,
    gamma = list(est = final$gamma, se = final$gamma.se,
                 Rhat = final$gamma.rhat,
                 free = gamma.free),
    Corr = list(est = final$Corr, se = final$Corr.se, Rhat = Corr.Rhat),
    gamma.matrix = list(est = gamma.matrix$est, se = gamma.matrix$se,
                        Rhat = gamma.matrix$Rhat),
    Q.matrix = prep$Q.matrix,
    block.items = prep$block.items,
    pairs.matrix = prep$pairs.matrix,
    pairs.value = prep$pairs.value,
    response = prep$response,
    fc.type = prep$fc.type,
    stan.obj = NULL,
    MCMC.obj = NULL,
    call = call,
    arguments = list(
      data = data,
      Q.matrix = prep$Q.matrix,
      block.items = prep$block.items,
      fc.type = prep$fc.type,
      method = "iStEM",
      cores = common.method$cores,
      vis = vis,
      seed = common.method$seed,
      control.model = control.model,
      control.method = fit_effective_method_control(
        control.method, method = c(method, bounds), common = common.method
      )
    ),
    iStEM = utils::modifyList(run$iStEM, list(prior = prior))
  )
  class(results) <- "TIRT"

  results$logLik <- logLik(results, L = method$L,
                           theta.low = method$theta.lower,
                           theta.up = method$theta.upper)
  results
}

tirt_istem_prior_control <- function(control.model) {
  lambda_alpha <- istem_scalar(get_ctrl("lambda.alpha", 0.40, control.model),
                               "lambda.alpha", positive = TRUE)
  lambda_beta <- istem_scalar(get_ctrl("lambda.beta", 0.90, control.model),
                              "lambda.beta", positive = TRUE)
  if (lambda_alpha >= lambda_beta) {
    stop("'lambda.alpha' must be smaller than 'lambda.beta'.",
         call. = FALSE)
  }
  list(
    use.prior = isTRUE(get_ctrl("use.prior", TRUE, control.model)),
    lambda.alpha = lambda_alpha,
    lambda.beta = lambda_beta,
    psi.mu = istem_scalar(get_ctrl("psi.mu", 0.80, control.model), "psi.mu",
                          positive = TRUE),
    psi.sigma = istem_scalar(get_ctrl("psi.sigma", 0.30, control.model),
                             "psi.sigma", positive = TRUE),
    gamma.mu = istem_scalar(get_ctrl("gamma.mu", 0, control.model),
                            "gamma.mu"),
    gamma.sigma = istem_scalar(get_ctrl("gamma.sigma", 0.8, control.model),
                               "gamma.sigma", positive = TRUE)
  )
}

tirt_istem_check_bounds <- function(bounds) {
  for (nm in names(bounds)) {
    bounds[[nm]] <- istem_scalar(bounds[[nm]], nm, arg = "control.method")
  }
  if (bounds$lambda.lower <= 0 ||
      bounds$lambda.lower >= bounds$lambda.upper) {
    stop("'lambda.lower' must be positive and smaller than 'lambda.upper'.",
         call. = FALSE)
  }
  if (bounds$psi.lower <= 0 || bounds$psi.lower >= bounds$psi.upper) {
    stop("'psi.lower' must be positive and smaller than 'psi.upper'.",
         call. = FALSE)
  }
  if (bounds$gamma.lower >= bounds$gamma.upper) {
    stop("'gamma.lower' must be smaller than 'gamma.upper'.", call. = FALSE)
  }
  invisible(NULL)
}

tirt_istem_prepare_data <- function(data, Q.matrix, block.items, fc.type,
                                    control.model) {
  fc.type <- toupper(fc.type)
  N.block.guess <- if (is.list(block.items)) length(block.items) else ncol(data)
  if (length(fc.type) != N.block.guess) {
    fc.type <- rep(fc.type[1L], N.block.guess)
  }
  if (is.null(block.items)) {
    block.items <- get.block.items.from.data(data)
  }
  result <- get.response.from.data.TIRT(data, block.items, fc.type = fc.type)
  response <- result$response
  pairs.value <- result$pairs.value

  if (any(is.na(response))) {
    first_missing <- which(is.na(response), arr.ind = TRUE)[1L, ]
    stop(
      sprintf(
        "TIRT responses must not contain missing values (row %d, column %d).",
        first_missing[1L], first_missing[2L]
      ),
      call. = FALSE
    )
  }
  if (!all(response %in% c(0, 1))) {
    stop("TIRT responses must contain only 0 and 1.",
         call. = FALSE)
  }

  if (!is.matrix(Q.matrix)) {
    stop("'Q.matrix' must be a matrix.", call. = FALSE)
  }
  if (!all(Q.matrix %in% c(-1, 0, 1))) {
    stop("'Q.matrix' must contain only -1, 0, or 1.", call. = FALSE)
  }

  I.states <- nrow(Q.matrix)
  D <- ncol(Q.matrix)
  if (I.states < 1L) {
    stop("'Q.matrix' must have at least one row (item).", call. = FALSE)
  }
  if (D < 2L) {
    stop("At least two trait dimensions must be specified in Q.matrix (TIRT requires D >= 2).",
         call. = FALSE)
  }
  if (any(rowSums(abs(Q.matrix)) < 1L)) {
    stop("Each item must measure at least one trait (row sums of abs('Q.matrix') must be >= 1).",
         call. = FALSE)
  }

  if (!is.list(block.items) || length(block.items) < 1L) {
    stop("'block.items' must be a non-empty list.", call. = FALSE)
  }
  if (!all(vapply(block.items, function(x) length(x) >= 2L, logical(1L)))) {
    stop("Each block in 'block.items' must contain at least 2 items.",
         call. = FALSE)
  }
  all_items_in_blocks <- unlist(block.items, use.names = FALSE)
  if (!all(all_items_in_blocks %in% seq_len(I.states))) {
    stop("Item indices in 'block.items' must be within 1:I.states.",
         call. = FALSE)
  }
  if (anyDuplicated(all_items_in_blocks)) {
    stop("Item indices in 'block.items' must be unique across all blocks.",
         call. = FALSE)
  }
  if (length(all_items_in_blocks) != I.states) {
    stop("The total number of items in 'block.items' must exactly match nrow(Q.matrix).",
         call. = FALSE)
  }

  I.block <- length(block.items[[1L]])
  if (!all(vapply(block.items, length, integer(1L)) == I.block)) {
    stop("All blocks in 'block.items' must have the same number of items.",
         call. = FALSE)
  }
  N.block <- length(block.items)
  if (length(fc.type) != N.block) {
    fc.type <- rep(fc.type[1L], N.block)
  }

  I.block.vec <- vapply(block.items, length, integer(1L))
  block.pair.counts <- integer(N.block)
  for (b in seq_len(N.block)) {
    k <- I.block.vec[b]
    block.pair.counts[b] <- switch(fc.type[b],
      RANK = choose(k, 2),
      MOLE = 2L * k - 3L,
      PICK = k - 1L,
      stop("'fc.type' must be RANK, MOLE, or PICK.", call. = FALSE)
    )
  }
  N.pairs.theoretical <- sum(block.pair.counts)
  if (ncol(response) != N.pairs.theoretical) {
    stop("Column dimension of response matrix does not match block.items/fc.type.",
         call. = FALSE)
  }

  response.raw <- response
  pairs.value.raw <- pairs.value
  if (!is.null(pairs.value) && !all(fc.type == "RANK")) {
    response.group <- istem_response_pair_groups(response, pairs.value)
    pairs.value <- response.group$pairs.value
  } else {
    response.group <- istem_response_groups(response)
    if (!is.null(pairs.value)) {
      pairs.value <- pairs.value[response.group$first]
    }
  }
  response <- response.group$response

  N.pairs.all <- sum(choose(I.block.vec, 2))
  pairs.matrix <- matrix(0L, N.pairs.all, 2L)
  idx <- 0L
  for (b in seq_len(N.block)) {
    pairs.cur <- t(combn(block.items[[b]], 2))
    pairs.matrix[idx + seq_len(nrow(pairs.cur)), ] <- pairs.cur
    idx <- idx + nrow(pairs.cur)
  }
  pair.keys <- paste0(pairs.matrix[, 1L], "_", pairs.matrix[, 2L])

  N.person <- nrow(response.raw)
  G <- response.group$G
  N.obs <- G * ncol(response)
  Y <- integer(N.obs)
  Idx.person <- integer(N.obs)
  Idx.item.i <- integer(N.obs)
  Idx.item.k <- integer(N.obs)
  Idx.pair <- integer(N.obs)

  idx <- 0L
  col_offset <- 0L
  for (b in seq_len(N.block)) {
    n_pairs.b <- block.pair.counts[b]
    if (fc.type[b] == "RANK") {
      pairs.b <- t(combn(block.items[[b]], 2))
      for (p in seq_len(G)) {
        for (pp in seq_len(n_pairs.b)) {
          idx <- idx + 1L
          Y[idx] <- response[p, col_offset + pp]
          Idx.person[idx] <- p
          Idx.item.i[idx] <- pairs.b[pp, 1L]
          Idx.item.k[idx] <- pairs.b[pp, 2L]
          Idx.pair[idx] <- match(paste0(pairs.b[pp, 1L], "_",
                                        pairs.b[pp, 2L]), pair.keys)
        }
      }
    } else {
      for (p in seq_len(G)) {
        pairs.cur <- pairs.value[[p]][[b]]
        if (!is.matrix(pairs.cur)) pairs.cur <- matrix(pairs.cur, ncol = 2)
        for (pp in seq_len(n_pairs.b)) {
          idx <- idx + 1L
          Y[idx] <- response[p, col_offset + pp]
          Idx.person[idx] <- p
          Idx.item.i[idx] <- pairs.cur[pp, 1L]
          Idx.item.k[idx] <- pairs.cur[pp, 2L]
          Idx.pair[idx] <- match(paste0(pairs.cur[pp, 1L], "_",
                                        pairs.cur[pp, 2L]), pair.keys)
        }
      }
    }
    col_offset <- col_offset + n_pairs.b
  }

  Idx.gamma.est <- sort(unique(Idx.pair))
  Idx.gamma.fix <- setdiff(seq_len(N.pairs.all), Idx.gamma.est)

  Idx.lambda.est.raw <- which(rowSums(abs(Q.matrix)) > 0)
  is.case.b <- D == 2L && I.block == 2L
  if (is.case.b) {
    first_pair_items <- block.items[[1L]]
    Idx.lambda.fix <- first_pair_items
    lambda.fix.val <- rep(0.80, length(first_pair_items))
    Idx.lambda.est <- setdiff(Idx.lambda.est.raw, Idx.lambda.fix)
  } else {
    Idx.lambda.est <- Idx.lambda.est.raw
    Idx.lambda.fix <- integer(0)
    lambda.fix.val <- numeric(0)
  }

  last_item_iblock <- unname(vapply(block.items, function(x) x[length(x)],
                                    integer(1L)))
  is.case.a <- I.block == 2L && D > 2L
  if (is.case.a || is.case.b) {
    Idx.psi.fix <- seq_len(I.states)
    psi.fix.val <- rep(sqrt(0.5), I.states)
    Idx.psi.est <- integer(0)
  } else {
    Idx.psi.fix <- last_item_iblock
    psi.fix.val <- rep(1, length(last_item_iblock))
    Idx.psi.est <- setdiff(seq_len(I.states), Idx.psi.fix)
  }

  gamma.mu <- get_ctrl("gamma.mu", 0, control.model)
  gamma.fix.val <- rep(gamma.mu, length(Idx.gamma.fix))
  if (length(Idx.gamma.fix) > 0L) {
    warning(sprintf(
      "TIRT: %d of %d possible pairwise comparisons were never observed. The corresponding gamma parameters are fixed to the prior mean (%.2f).",
      length(Idx.gamma.fix), N.pairs.all, gamma.mu
    ))
  }

  obs.by.person <- split(seq_along(Y), Idx.person)

  list(
    response = response.raw,
    response.unique = response,
    response.count = response.group$count,
    response.group = response.group$group,
    G = G,
    Q.matrix = Q.matrix,
    block.items = block.items,
    pairs.value = pairs.value.raw,
    pairs.value.unique = pairs.value,
    pairs.matrix = pairs.matrix,
    fc.type = fc.type,
    N.person = N.person,
    I.states = I.states,
    D = D,
    Y = Y,
    Idx.person = Idx.person,
    Idx.item.i = Idx.item.i,
    Idx.item.k = Idx.item.k,
    Idx.pair = Idx.pair,
    obs.by.person = obs.by.person,
    Idx.lambda.est = Idx.lambda.est,
    Idx.lambda.fix = Idx.lambda.fix,
    lambda.fix.val = lambda.fix.val,
    Idx.psi.est = Idx.psi.est,
    Idx.psi.fix = Idx.psi.fix,
    psi.fix.val = psi.fix.val,
    Idx.gamma.est = Idx.gamma.est,
    Idx.gamma.fix = Idx.gamma.fix,
    gamma.fix.val = gamma.fix.val
  )
}

tirt_istem_initial_par <- function(prep, prior, bounds) {
  lambda <- stats::runif(prep$I.states, bounds$lambda.lower,
                         bounds$lambda.upper)
  if (length(prep$Idx.lambda.fix) > 0L) {
    lambda[prep$Idx.lambda.fix] <- prep$lambda.fix.val
  }
  psi <- istem_rnorm_bounded(prep$I.states, prior$psi.mu, prior$psi.sigma,
                             bounds$psi.lower, bounds$psi.upper)
  if (length(prep$Idx.psi.fix) > 0L) {
    psi[prep$Idx.psi.fix] <- prep$psi.fix.val
  }
  gamma <- istem_rnorm_bounded(nrow(prep$pairs.matrix), prior$gamma.mu,
                               prior$gamma.sigma, bounds$gamma.lower,
                               bounds$gamma.upper)
  if (length(prep$Idx.gamma.fix) > 0L) {
    gamma[prep$Idx.gamma.fix] <- prep$gamma.fix.val
  }
  list(lambda = lambda, psi = psi, gamma = gamma)
}

tirt_istem_sample_theta <- function(state, theta_grid_length,
                                    theta_lower, theta_upper) {
  chol <- istem_corr_chol(state$Corr)
  cpp_gibbs_tirt_theta(
    theta_ = state$theta,
    Q_matrix_ = state$Q.matrix,
    lambda_ = state$lambda,
    psi_ = state$psi,
    gamma_ = state$gamma,
    Y_ = state$Y,
    obs_by_person_ = state$obs.by.person,
    Idx_item_i_ = state$Idx.item.i,
    Idx_item_k_ = state$Idx.item.k,
    Idx_pair_ = state$Idx.pair,
    theta_mu_ = state$theta_mu,
    chol_corr_ = chol$chol,
    log_diag_sum = chol$log_diag_sum,
    step = theta_grid_length,
    lower = theta_lower,
    upper = theta_upper
  )
}

tirt_istem_update_parameters <- function(state) {
  # Pre-compute qtheta once (changes only when theta changes, i.e. between batches)
  qtheta <- state$theta %*% t(state$Q.matrix)

  items_to_opt <- union(state$Idx.lambda.est, state$Idx.psi.est)

  # ---- Build static observation caches (invariant across coord-ascent passes) ----
  # Per-item cache: for each item, the observation indices and structural info
  item_cache <- vector("list", length(items_to_opt))
  names(item_cache) <- as.character(items_to_opt)

  for (idx in seq_along(items_to_opt)) {
    i <- items_to_opt[idx]
    obs_idx <- which(state$Idx.item.i == i | state$Idx.item.k == i)
    if (length(obs_idx) == 0L) {
      item_cache[[idx]] <- NULL
      next
    }

    persons  <- state$Idx.person[obs_idx]
    ii       <- state$Idx.item.i[obs_idx]
    ik       <- state$Idx.item.k[obs_idx]
    ip       <- state$Idx.pair[obs_idx]
    is_first <- ii == i

    # qt_target and qt_other are invariant (theta is fixed during param update)
    qt_target <- numeric(length(obs_idx))
    qt_other  <- numeric(length(obs_idx))

    idx_f <- which(is_first)
    if (length(idx_f) > 0L) {
      qt_target[idx_f] <- qtheta[cbind(persons[idx_f], ii[idx_f])]
      qt_other[idx_f]  <- qtheta[cbind(persons[idx_f], ik[idx_f])]
    }
    idx_s <- which(!is_first)
    if (length(idx_s) > 0L) {
      qt_target[idx_s] <- qtheta[cbind(persons[idx_s], ik[idx_s])]
      qt_other[idx_s]  <- qtheta[cbind(persons[idx_s], ii[idx_s])]
    }

    item_cache[[idx]] <- list(
      i         = i,
      qt_target = qt_target,
      qt_other  = qt_other,
      other_item = ifelse(is_first, ik, ii),   # the OTHER item in each obs
      pair_idx   = ip,
      is_first   = as.integer(is_first),
      Y          = state$Y[obs_idx],
      weight     = state$response.count[persons],
      opt_lambda = i %in% state$Idx.lambda.est,
      opt_psi    = i %in% state$Idx.psi.est
    )
  }

  # Per-pair cache
  pair_cache <- vector("list", length(state$Idx.gamma.est))
  names(pair_cache) <- as.character(state$Idx.gamma.est)

  for (idx in seq_along(state$Idx.gamma.est)) {
    ip <- state$Idx.gamma.est[idx]
    obs_idx <- which(state$Idx.pair == ip)
    if (length(obs_idx) == 0L) {
      pair_cache[[idx]] <- NULL
      next
    }

    persons <- state$Idx.person[obs_idx]
    ii      <- state$Idx.item.i[obs_idx]
    ik      <- state$Idx.item.k[obs_idx]

    # Pre-compute the part of mu that is independent of gamma
    qt_i <- qtheta[cbind(persons, ii)]
    qt_k <- qtheta[cbind(persons, ik)]

    pair_cache[[idx]] <- list(
      ip      = ip,
      persons = persons,
      ii      = ii,
      ik      = ik,
      qt_i    = qt_i,
      qt_k    = qt_k,
      Y       = state$Y[obs_idx],
      weight  = state$response.count[persons]
    )
  }

  # ---- Coordinate ascent: iterate items -> pairs until convergence ----
  max_ca_passes <- 10L
  for (ca_pass in seq_len(max_ca_passes)) {
    max_change <- 0.0

    # -- Pass 1: update all item (lambda, psi) --
    for (cache in item_cache) {
      if (is.null(cache)) next
      i <- cache$i

      # Recompute parameter-dependent arrays (may have changed since last pass)
      lambda_other <- state$lambda[cache$other_item]
      psi_other    <- state$psi[cache$other_item]
      gamma_pair   <- state$gamma[cache$pair_idx]

      old_lambda <- state$lambda[i]
      old_psi    <- state$psi[i]

      if (cache$opt_lambda && cache$opt_psi) {
        x0    <- c(old_lambda, old_psi)
        lower <- c(state$bounds$lambda.lower, state$bounds$psi.lower)
        upper <- c(state$bounds$lambda.upper, state$bounds$psi.upper)

        obj <- function(x) {
          ll <- cpp_tirt_item_loglik_weighted(
            cache$qt_target, cache$qt_other,
            lambda_other, psi_other, gamma_pair,
            cache$is_first, cache$Y, cache$weight, x[1L], x[2L])
          lp <- if (isTRUE(state$prior$use.prior)) {
            stats::dnorm(x[2L], mean = state$prior$psi.mu,
                         sd = state$prior$psi.sigma, log = TRUE)
          } else 0
          if (!is.finite(lp)) return(Inf)
          -ll - lp
        }

        opt <- tryCatch(
          stats::optim(par = x0, fn = obj, method = "L-BFGS-B",
                       lower = lower, upper = upper,
                       control = list(maxit = state$optim.maxit)),
          error = function(e) NULL
        )
        if (!is.null(opt) && is.finite(opt$value)) {
          state$lambda[i] <- opt$par[1L]
          state$psi[i]    <- opt$par[2L]
        }

      } else if (cache$opt_lambda) {
        psi_fixed <- old_psi
        obj <- function(x) {
          ll <- cpp_tirt_item_loglik_weighted(
            cache$qt_target, cache$qt_other,
            lambda_other, psi_other, gamma_pair,
            cache$is_first, cache$Y, cache$weight, x[1L], psi_fixed)
          -ll
        }
        opt <- tryCatch(
          stats::optim(par = old_lambda, fn = obj, method = "L-BFGS-B",
                       lower = state$bounds$lambda.lower,
                       upper = state$bounds$lambda.upper,
                       control = list(maxit = state$optim.maxit)),
          error = function(e) NULL
        )
        if (!is.null(opt) && is.finite(opt$value)) {
          state$lambda[i] <- opt$par[1L]
        }

      } else if (cache$opt_psi) {
        lambda_fixed <- old_lambda
        obj <- function(x) {
          ll <- cpp_tirt_item_loglik_weighted(
            cache$qt_target, cache$qt_other,
            lambda_other, psi_other, gamma_pair,
            cache$is_first, cache$Y, cache$weight, lambda_fixed, x[1L])
          lp <- if (isTRUE(state$prior$use.prior)) {
            stats::dnorm(x[1L], mean = state$prior$psi.mu,
                         sd = state$prior$psi.sigma, log = TRUE)
          } else 0
          if (!is.finite(lp)) return(Inf)
          -ll - lp
        }
        opt <- tryCatch(
          stats::optim(par = old_psi, fn = obj, method = "L-BFGS-B",
                       lower = state$bounds$psi.lower,
                       upper = state$bounds$psi.upper,
                       control = list(maxit = state$optim.maxit)),
          error = function(e) NULL
        )
        if (!is.null(opt) && is.finite(opt$value)) {
          state$psi[i] <- opt$par[1L]
        }
      }

      max_change <- max(max_change,
                        abs(state$lambda[i] - old_lambda),
                        abs(state$psi[i]    - old_psi))
    }

    # -- Pass 2: update all gamma --
    for (cache in pair_cache) {
      if (is.null(cache)) next
      ip <- cache$ip

      # Recompute mu_no_gamma and denom (depend on lambda, psi which may have changed)
      mu_no_gamma <- state$lambda[cache$ii] * cache$qt_i -
                     state$lambda[cache$ik] * cache$qt_k
      denom <- sqrt(state$psi[cache$ii]^2 + state$psi[cache$ik]^2)

      old_gamma <- state$gamma[ip]

      obj <- function(x) {
        ll <- cpp_tirt_pair_loglik_weighted(
          mu_no_gamma, denom, cache$Y, cache$weight, x[1L])
        lp <- if (isTRUE(state$prior$use.prior)) {
          stats::dnorm(x[1L], mean = state$prior$gamma.mu,
                       sd = state$prior$gamma.sigma, log = TRUE)
        } else 0
        if (!is.finite(lp)) return(Inf)
        -ll - lp
      }

      opt <- tryCatch(
        stats::optim(par = old_gamma, fn = obj, method = "L-BFGS-B",
                     lower = state$bounds$gamma.lower,
                     upper = state$bounds$gamma.upper,
                     control = list(maxit = state$optim.maxit)),
        error = function(e) NULL
      )
      if (!is.null(opt) && is.finite(opt$value)) {
        state$gamma[ip] <- opt$par[1L]
      }

      max_change <- max(max_change, abs(state$gamma[ip] - old_gamma))
    }

    if (max_change < 1e-6) break
  }

  state
}

tirt_istem_param_vec <- function(lambda, psi, gamma, Idx.lambda.est,
                                 Idx.psi.est, Idx.gamma.est, Corr,
                                 include_corr) {
  c(lambda[Idx.lambda.est],
    psi[Idx.psi.est],
    gamma[Idx.gamma.est],
    istem_param_corr_vec(Corr, include_corr))
}

tirt_istem_param_unpack <- function(pv, pv_se, pv_rhat = NULL, state, include_corr) {
  idx <- 1L
  has_rhat <- !is.null(pv_rhat)
  lambda <- state$lambda
  lambda.se <- rep(NA_real_, length(lambda))
  lambda.rhat <- rep(NA_real_, length(lambda))
  if (length(state$Idx.lambda.est) > 0L) {
    n <- length(state$Idx.lambda.est)
    lambda[state$Idx.lambda.est] <- pv[idx:(idx + n - 1L)]
    lambda.se[state$Idx.lambda.est] <- pv_se[idx:(idx + n - 1L)]
    if (has_rhat) lambda.rhat[state$Idx.lambda.est] <- pv_rhat[idx:(idx + n - 1L)]
    idx <- idx + n
  }
  psi <- state$psi
  psi.se <- rep(NA_real_, length(psi))
  psi.rhat <- rep(NA_real_, length(psi))
  if (length(state$Idx.psi.est) > 0L) {
    n <- length(state$Idx.psi.est)
    psi[state$Idx.psi.est] <- pv[idx:(idx + n - 1L)]
    psi.se[state$Idx.psi.est] <- pv_se[idx:(idx + n - 1L)]
    if (has_rhat) psi.rhat[state$Idx.psi.est] <- pv_rhat[idx:(idx + n - 1L)]
    idx <- idx + n
  }
  gamma <- state$gamma
  gamma.se <- rep(NA_real_, length(gamma))
  gamma.rhat <- rep(NA_real_, length(gamma))
  if (length(state$Idx.gamma.est) > 0L) {
    n <- length(state$Idx.gamma.est)
    gamma[state$Idx.gamma.est] <- pv[idx:(idx + n - 1L)]
    gamma.se[state$Idx.gamma.est] <- pv_se[idx:(idx + n - 1L)]
    if (has_rhat) gamma.rhat[state$Idx.gamma.est] <- pv_rhat[idx:(idx + n - 1L)]
    idx <- idx + n
  }
  corr <- istem_unpack_corr(pv, pv_se, idx, state$D, state$Corr, include_corr)
  corr.rhat <- istem_unpack_corr_rhat(pv_rhat, idx, state$D, include_corr)
  list(
    lambda = lambda,    lambda.se = lambda.se,    lambda.rhat = lambda.rhat,
    psi = psi,          psi.se = psi.se,          psi.rhat = psi.rhat,
    gamma = gamma,      gamma.se = gamma.se,      gamma.rhat = gamma.rhat,
    Corr = corr$Corr,   Corr.se = corr$Corr.se,   Corr.rhat = corr.rhat
  )
}

tirt_istem_gamma_matrix <- function(gamma, gamma.se, pairs.matrix, I.states) {
  gamma.matrix <- matrix(NA_real_, I.states, I.states)
  gamma.matrix.se <- matrix(NA_real_, I.states, I.states)
  gamma.matrix.Rhat <- matrix(NA_real_, I.states, I.states)
  for (r in seq_len(nrow(pairs.matrix))) {
    i <- pairs.matrix[r, 1L]
    k <- pairs.matrix[r, 2L]
    gamma.matrix[i, k] <- gamma[r]
    gamma.matrix.se[i, k] <- gamma.se[r]
    gamma.matrix.Rhat[i, k] <- NA_real_
    gamma.matrix[k, i] <- -gamma[r]
    gamma.matrix.se[k, i] <- gamma.se[r]
    gamma.matrix.Rhat[k, i] <- NA_real_
  }
  list(est = gamma.matrix, se = gamma.matrix.se, Rhat = gamma.matrix.Rhat)
}

#' Simulate Forced-Choice GDINA Data
#'
#' @description
#' Generates forced-choice response data from a cognitive diagnostic item
#' model. The item-level model may be DINA, DINO, ACDM, or GDINA. The resulting
#' statement endorsement probabilities are transformed into forced-choice
#' response probabilities for \code{"RANK"}, \code{"MOLE"}, or \code{"PICK"}
#' blocks using the same sequential Luce--Plackett mechanism used by
#' \code{\link{sim.data.FCMIRT}} and \code{\link{sim.data.FCGGUM}}.
#'
#' @section Data-Generating Process:
#'
#' The simulation proceeds in four steps:
#' \enumerate{
#'   \item Generate or validate the Q-matrix and forced-choice block
#'         structure.
#'   \item Generate latent attribute profiles \eqn{\boldsymbol{\alpha}_n}.
#'   \item Generate item-level CDM endorsement probabilities using guessing
#'         and slipping-style bounds \code{P0} and \code{P1}; then recover the
#'         corresponding CDM delta parameters from the design matrix.
#'   \item Convert item endorsement probabilities into block-level
#'         forced-choice pattern probabilities and sample observed responses.
#' }
#'
#' Attribute profiles can be generated from a higher-order latent trait model,
#' a thresholded multivariate normal model, a uniform distribution over all
#' attribute patterns, or supplied directly through \code{control$alpha}.
#'
#' @param N.person Number of persons.
#' @param N.block Number of forced-choice blocks.
#' @param I.block Number of items per block when \code{control$block.items} is
#'   not supplied. Must be at least 2.
#' @param D Number of latent attributes. If \code{control$Q.matrix} is supplied,
#'   \code{D} is reset to \code{ncol(control$Q.matrix)}.
#' @param model CDM item model: \code{"DINA"}, \code{"DINO"}, \code{"ACDM"}, or
#'   \code{"GDINA"}.
#' @param fc.type Forced-choice response type: \code{"RANK"}, \code{"MOLE"}, or
#'   \code{"PICK"}. A scalar value is recycled to all blocks; a vector of
#'   length \eqn{B} may be supplied for mixed block formats.
#' @param control Optional named list controlling the data-generating process.
#'   Supported entries are described below.
#'
#' @section Control List:
#'
#' \describe{
#'   \item{\code{Q.matrix}}{Optional \eqn{I \times D} binary Q-matrix. If
#'         omitted, a Q-matrix is generated by \code{sim.data.Q.CDM()}.}
#'   \item{\code{block.items}}{Optional list of item indices per block. If
#'         omitted, blocks are formed sequentially using \code{N.block} and
#'         \code{I.block}.}
#'   \item{\code{single}}{Logical passed to \code{sim.data.Q.CDM()} when
#'         \code{Q.matrix} is generated internally. Default \code{TRUE}.}
#'   \item{\code{alpha}}{Optional \eqn{N \times D} binary matrix of true
#'         attribute profiles. If supplied, latent attributes are not
#'         generated.}
#'   \item{\code{distribution}}{Latent attribute distribution when
#'         \code{alpha} is not supplied. Options are \code{"horder"}
#'         (default; alias \code{"higher.order"}), \code{"mvnorm"}, and
#'         \code{"uniform"}.}
#'   \item{\code{delta1}, \code{delta0}, \code{theta}}{Higher-order
#'         discrimination, threshold, and person trait values used when
#'         \code{distribution = "horder"}. Defaults are generated from
#'         log-normal, normal, and standard normal distributions.}
#'   \item{\code{Corr}, \code{threshold}}{Correlation matrix and thresholds
#'         used when \code{distribution = "mvnorm"}.}
#'   \item{\code{gs}}{Optional \eqn{I \times 2} matrix with columns
#'         \code{P0} and \code{P1}; \code{P0} is the lower endorsement
#'         probability and \code{P1} is the upper endorsement probability for
#'         each statement. Defaults are sampled from \code{U(0, .2)} and
#'         \code{U(.8, 1)}.}
#'   \item{\code{mono.constraint}}{Logical; whether to enforce monotonic item
#'         probabilities for GDINA item generation. Default \code{TRUE}.}
#' }
#'
#' @return An object of class \code{"data.FCGDINA"}, a list containing:
#' \describe{
#'   \item{\code{data}}{\eqn{N \times B} character matrix of forced-choice
#'         responses, such as \code{"1>3>2"} for ranking blocks.}
#'   \item{\code{response}}{\eqn{N \times B} integer matrix of 1-based pattern
#'         indices matching \code{patterns}.}
#'   \item{\code{alpha}}{\eqn{N \times D} binary matrix of true latent
#'         attribute profiles.}
#'   \item{\code{gs}}{\eqn{I \times 2} matrix of item probability bounds
#'         \code{P0} and \code{P1}.}
#'   \item{\code{delta.list}}{List of true CDM delta vectors, one per
#'         statement.}
#'   \item{\code{design.matrix.list}}{List of CDM design matrices used to map
#'         delta parameters to item endorsement probabilities.}
#'   \item{\code{Q.matrix}}{\eqn{I \times D} Q-matrix.}
#'   \item{\code{block.items}}{List of statement indices per forced-choice
#'         block.}
#'   \item{\code{model}}{CDM item model used for simulation.}
#'   \item{\code{patterns}}{List of observed response-pattern matrices per
#'         block.}
#'   \item{\code{patterns.total}}{List of full-ranking pattern matrices per
#'         block.}
#'   \item{\code{prob.item}}{\eqn{N \times I} matrix of item-level endorsement
#'         probabilities for sampled persons.}
#'   \item{\code{prob.states}}{\eqn{I \times 2^D} matrix of item endorsement
#'         probabilities for every attribute pattern.}
#'   \item{\code{prob}}{\eqn{N \times \sum_b P_b} matrix of forced-choice
#'         response-pattern probabilities.}
#'   \item{\code{N.person}, \code{N.block}, \code{I.block}, \code{D},
#'         \code{I.states}, \code{fc.type}}{Final data-generating dimensions
#'         and response type.}
#'   \item{\code{call}, \code{arguments}}{Matched call and effective
#'         simulation arguments.}
#' }
#'
#' @references
#' de la Torre, J. (2011). The generalized DINA model framework.
#' \emph{Psychometrika}, 76(2), 179--199.
#' \doi{10.1007/s11336-011-9207-7}
#'
#' @seealso \code{\link{fit.FCGDINA}}, \code{\link{model.FCGDINA}},
#'   \code{\link{sim.data.FCMIRT}}, \code{\link{sim.data.FCGGUM}},
#'   \code{\link{sim.data.FCDCM}}
#'
#' @examples
#' set.seed(123)
#' sim <- sim.data.FCGDINA(N.person = 20, N.block = 3, I.block = 2,
#'                         D = 2, model = "DINA", fc.type = "RANK")
#'
#' str(sim$data)
#' head(sim$response)
#' head(sim$alpha)
#' sim$Q.matrix
#'
#' # Mixed response formats across blocks.
#' sim.mix <- sim.data.FCGDINA(
#'   N.person = 12, N.block = 3, I.block = 3, D = 2,
#'   model = "GDINA", fc.type = c("RANK", "MOLE", "PICK")
#' )
#' vapply(sim.mix$patterns, nrow, integer(1))
#'
#' @export
sim.data.FCGDINA <- function(N.person = 1000, N.block = 10, I.block = 2,
                             D = 3, model = "GDINA", fc.type = "RANK",
                             control = NULL) {

  call <- match.call()

  if (is.null(control)) {
    control <- list()
  }

  # ---- Model validation ----
  model <- toupper(model[1L])
  model.valid <- c("DINA", "DINO", "ACDM", "GDINA")
  if (!model %in% model.valid) {
    stop("'model' must be one of: ", paste(model.valid, collapse = ", "), call. = FALSE)
  }

  # ---- Scalar integer validation ----
  N.person <- check_integer_scalar(N.person, "N.person", 1L)
  D        <- check_integer_scalar(D,        "D",        1L)
  I.block  <- check_integer_scalar(I.block,  "I.block",  2L)
  N.block  <- check_integer_scalar(N.block,  "N.block",  1L)

  # ---- Q-matrix and block configuration ----
  Q.matrix    <- control$Q.matrix
  block.items <- control$block.items

  res <- resolve_fc_blocks(
    Q.matrix       = Q.matrix,
    block.items    = block.items,
    I.block        = I.block,
    N.block        = N.block,
    D              = D,
    q.valid.values = c(0, 1),
    q.row.check    = function(r) any(r > 0)
  )
  I.states   <- res$I.states
  N.block    <- res$N.block
  I.block    <- res$I.block
  block.items <- res$block.items
  Q.matrix   <- res$Q.matrix
  D          <- res$D

  # ---- fc.type validation ----
  fc.type <- normalize_fc_type(fc.type, N.block)

  # ---- Generate Q-matrix if not supplied ----
  if (is.null(Q.matrix)) {
    single <- get_ctrl("single", TRUE, control)
    Q.matrix <- sim.data.Q.CDM(D, I.states, single = single)
  }

  # ---- Alpha generation ----
  alpha <- control$alpha
  if (!is.null(alpha)) {
    alpha <- as.matrix(alpha)
    if (nrow(alpha) != N.person || ncol(alpha) != D ||
        anyNA(alpha) || !all(alpha %in% c(0, 1))) {
      stop("'control$alpha' must be a ", N.person, " x ", D, " binary matrix.", call. = FALSE)
    }
    storage.mode(alpha) <- "integer"
  } else {
    distribution <- tolower(get_ctrl("distribution", "horder", control))
    if (distribution == "higher.order") {
      distribution <- "horder"
    }
    if (!distribution %in% c("horder", "mvnorm", "uniform")) {
      stop("'control$distribution' must be one of: horder, mvnorm, uniform.", call. = FALSE)
    }

    if (distribution == "horder") {
      delta1 <- as.numeric(get_ctrl("delta1", rlnorm(D, meanlog = 0.25, sdlog = 0.25), control))
      delta0 <- as.numeric(get_ctrl("delta0", rnorm(D), control))
      theta  <- as.numeric(get_ctrl("theta",  rnorm(N.person), control))

      if (length(delta1) != D) {
        stop("'control$delta1' must have length ", D, ".", call. = FALSE)
      }
      if (length(delta0) != D) {
        stop("'control$delta0' must have length ", D, ".", call. = FALSE)
      }
      if (length(theta) != N.person) {
        stop("'control$theta' must have length ", N.person, ".", call. = FALSE)
      }

      prob.alpha <- distribution_higher_order(
        theta = theta, delta1 = delta1, delta0 = delta0
      )
      alpha <- (prob.alpha > matrix(runif(N.person * D), N.person, D)) * 1L
    } else if (distribution == "mvnorm") {
      Corr <- validate_corr_matrix(control$Corr, D)
      threshold <- as.numeric(get_ctrl("threshold", seq_len(D) / (D + 1), control))
      if (length(threshold) != D) {
        stop("'control$threshold' must have length ", D, ".", call. = FALSE)
      }
      alpha <- distribution_multi_normal(N.person, Corr, threshold)
    } else {
      patterns.all <- attributepattern(D)
      alpha <- distribution_uniform(N.person, patterns.all)
    }
  }

  # ---- Guessing/slipping parameters ----
  gs <- control$gs
  if (is.null(gs)) {
    gs <- cbind(
      P0 = runif(I.states, 0.00, 0.20),
      P1 = runif(I.states, 0.80, 1.00)
    )
  } else {
    gs <- as.matrix(gs)
    if (nrow(gs) != I.states || ncol(gs) != 2L ||
        anyNA(gs) || !is.numeric(gs)) {
      stop("'control$gs' must be an I.states x 2 numeric matrix.", call. = FALSE)
    }
    colnames(gs) <- c("P0", "P1")
  }

  if (any(gs[, 1L] > gs[, 2L])) {
    stop("Each row of 'gs' must satisfy P0 <= P1.", call. = FALSE)
  }

  # ---- Item-level endorsement probabilities and delta parameters ----
  patterns.full <- attributepattern(D)
  pattern.names <- pattern_key(patterns.full)

  prob.states <- matrix(
    NA_real_,
    nrow = I.states,
    ncol = nrow(patterns.full),
    dimnames = list(paste0("item", seq_len(I.states)), pattern.names)
  )

  delta.list        <- vector("list", I.states)
  design.matrix.list <- vector("list", I.states)

  for (i in seq_len(I.states)) {
    att <- which(Q.matrix[i, ] > 0)
    patterns.i <- if (length(att) > 0) attributepattern(length(att)) else matrix(0L, 1, 0)

    # Generate valid CDM item response probabilities
    mono <- get_ctrl("mono.constraint", TRUE, control)
    p.i <- make_item_prob_cdm(
      patterns        = patterns.i,
      model           = model,
      p0              = gs[i, 1L],
      p1              = gs[i, 2L],
      mono.constraint = mono
    )

    # Build design matrix and back-solve for delta (true model parameters)
    X.i <- get_design_matrix_cdm(patterns.i, model)
    delta.i <- qr.solve(X.i, p.i)

    # Map reduced-pattern probabilities back to full pattern space
    if (length(att) > 0) {
      local.key <- pattern_key(patterns.full[, att, drop = FALSE])
    } else {
      local.key <- rep("", nrow(patterns.full))
    }
    prob.states[i, ] <- p.i[local.key]

    delta.list[[i]]        <- delta.i
    design.matrix.list[[i]] <- X.i
  }

  alpha.names <- pattern_key(alpha)
  alpha.index <- match(alpha.names, pattern.names)

  prob.item <- t(prob.states[, alpha.index, drop = FALSE])
  colnames(prob.item) <- paste0("item", seq_len(I.states))

  # ---- FC permutation patterns ----
  pat <- generate_fc_permutation_patterns(block.items, fc.type)
  patterns.total <- pat$patterns.total
  patterns       <- pat$patterns

  # ---- FC pattern probabilities ----
  prob <- model.FCGDINA(
    alpha = alpha,
    delta.list = delta.list,
    design.matrix.list = design.matrix.list,
    Q.matrix = Q.matrix,
    patterns.total = patterns.total,
    patterns = patterns,
    prob.item = prob.item
  )

  # ---- Response sampling ----
  samples <- sample_fc_responses(prob, patterns, N.person, N.block)
  response <- samples$response
  data     <- samples$data

  # ---- Row/column names ----
  rownames(alpha) <- paste0("person", seq_len(N.person))
  colnames(alpha) <- paste0("alpha", seq_len(D))
  rownames(response) <- rownames(data) <- rownames(alpha)
  colnames(response) <- colnames(data) <- paste0("block.", seq_len(N.block))

  rownames(Q.matrix) <- paste0("item", seq_len(I.states))
  colnames(Q.matrix) <- paste0("dim", seq_len(D))

  rownames(gs) <- paste0("item", seq_len(I.states))

  rownames(prob.item) <- rownames(alpha)
  rownames(prob.states) <- paste0("item", seq_len(I.states))

  # ---- Output assembly ----
  data.obj <- list(
    data              = data,
    response          = response,
    alpha             = alpha,
    gs                = gs,
    delta.list        = delta.list,
    design.matrix.list = design.matrix.list,
    Q.matrix          = Q.matrix,
    block.items       = block.items,
    model             = model,
    patterns          = patterns,
    patterns.total    = patterns.total,
    prob.item         = prob.item,
    prob.states       = prob.states,
    prob              = prob,
    N.person          = N.person,
    N.block           = N.block,
    I.block           = I.block,
    D                 = D,
    I.states          = I.states,
    fc.type           = fc.type,
    call              = call,
    arguments         = list(
      N.person = N.person,
      N.block  = N.block,
      I.block  = I.block,
      D        = D,
      model    = model,
      fc.type  = fc.type,
      control  = control
    )
  )

  class(data.obj) <- "data.FCGDINA"

  return(data.obj)
}


#' Compute FCGDINA forced-choice pattern probabilities
#'
#' @description
#' Computes block-level forced-choice response probabilities from known
#' attribute profiles and item-level CDM delta parameters. This is the
#' deterministic probability engine used by \code{\link{sim.data.FCGDINA}} for
#' response generation and is useful for checking a fitted or simulated
#' FCGDINA parameterization.
#'
#' @details
#' Item endorsement probabilities are first computed by
#' \code{\link{compute_fcgdina_item_prob}}. For person \eqn{n} and statement
#' \eqn{i}, the probability is
#' \deqn{
#'   p_{ni} = \mathbf{x}_i(\boldsymbol{\alpha}_n)' \boldsymbol{\delta}_i,
#' }
#' where \eqn{\mathbf{x}_i(\boldsymbol{\alpha}_n)} is the relevant row of the
#' item CDM design matrix. The item probabilities are then converted to
#' block-level forced-choice pattern probabilities by
#' \code{\link{forced_choice_from_agree}}.
#'
#' @param alpha \eqn{N \times D} binary matrix of attribute profiles.
#' @param delta.list List of per-item delta parameter vectors. The length must
#'   equal \code{nrow(Q.matrix)}.
#' @param design.matrix.list List of per-item CDM design matrices generated by
#'   \code{\link{get_design_matrix_cdm}}.
#' @param Q.matrix \eqn{I \times D} binary Q-matrix.
#' @param patterns.total List of full-ranking pattern matrices.
#' @param patterns List of observed pattern matrices.
#' @param prob.item Optional precomputed item endorsement-probability matrix.
#'   Supplying it avoids recomputation in simulation workflows.
#'
#' @return An \eqn{N \times \sum_b P_b} matrix of forced-choice response
#'   probabilities, where \eqn{P_b} is the number of observable patterns in
#'   block \eqn{b}.
#'
#' @seealso \code{\link{sim.data.FCGDINA}},
#'   \code{\link{compute_fcgdina_item_prob}},
#'   \code{\link{forced_choice_from_agree}}
#'
#' @examples
#' sim <- sim.data.FCGDINA(N.person = 10, N.block = 2, I.block = 2,
#'                         D = 2, model = "DINA")
#' prob <- model.FCGDINA(
#'   alpha = sim$alpha,
#'   delta.list = sim$delta.list,
#'   design.matrix.list = sim$design.matrix.list,
#'   Q.matrix = sim$Q.matrix,
#'   patterns.total = sim$patterns.total,
#'   patterns = sim$patterns
#' )
#' stopifnot(all(abs(rowSums(prob) - length(sim$block.items)) < 1e-8))
#'
#' @export
#' @keywords internal
model.FCGDINA <- function(alpha, delta.list, design.matrix.list,
                          Q.matrix, patterns.total, patterns,
                          prob.item = NULL) {
  if (is.null(prob.item)) {
    prob.item <- compute_fcgdina_item_prob(
      alpha = alpha, delta.list = delta.list,
      design.matrix.list = design.matrix.list,
      Q.matrix = Q.matrix
    )
  }
  forced_choice_from_agree(prob.item, patterns.total, patterns)
}

#' Compute FCGDINA item-level endorsement probabilities
#'
#' @description
#' Computes statement endorsement probabilities for supplied attribute
#' profiles under an FCGDINA item-level CDM parameterization.
#'
#' @param alpha \eqn{N \times D} binary matrix of attribute profiles.
#' @param delta.list List of per-item delta parameter vectors.
#' @param design.matrix.list List of per-item design matrices.
#' @param Q.matrix \eqn{I \times D} binary Q-matrix.
#' @param gs Optional \eqn{I \times 2} matrix of lower and upper probability
#'   bounds. If supplied, computed probabilities are clamped to
#'   \code{gs[i, 1]} and \code{gs[i, 2]} for item \eqn{i}.
#' @param eps Numeric probability clamp used when \code{gs = NULL}. Default
#'   \code{1e-12}.
#' @param link Item probability link. \code{"identity"} uses
#'   \eqn{p = X\delta}; \code{"logit"} uses
#'   \eqn{p = \mathrm{logit}^{-1}(X\delta)}. The latter is used by the Stan
#'   FCGDINA backend.
#'
#' @return An \eqn{N \times I} matrix of item endorsement probabilities.
#'
#' @details
#' The function maps each full attribute pattern in \code{alpha} to the
#' reduced pattern required by each item-specific Q-vector, evaluates the
#' corresponding design row, and multiplies by that item's delta vector.
#' This function does not sample responses.
#'
#' @seealso \code{\link{model.FCGDINA}}, \code{\link{sim.data.FCGDINA}}
#'
#' @keywords internal
compute_fcgdina_item_prob <- function(alpha, delta.list, design.matrix.list,
                                       Q.matrix, gs = NULL, eps = 1e-12,
                                       link = c("identity", "logit")) {
  link <- match.arg(link)
  alpha <- as.matrix(alpha)
  N <- nrow(alpha)
  D <- ncol(alpha)
  I <- nrow(Q.matrix)

  patterns.full <- attributepattern(D)
  pattern.names <- pattern_key(patterns.full)
  alpha.names <- pattern_key(alpha)
  alpha.idx <- match(alpha.names, pattern.names)

  prob.item <- matrix(NA_real_, N, I)

  for (i in seq_len(I)) {
    att <- which(Q.matrix[i, ] > 0)
    X <- design.matrix.list[[i]]
    delta <- delta.list[[i]]

    if (length(att) > 0) {
      reduced.key <- pattern_key(patterns.full[, att, drop = FALSE])
      reduced.idx <- match(reduced.key[alpha.idx],
                           pattern_key(attributepattern(length(att))))
    } else {
      reduced.idx <- rep(1L, N)
    }

    eta.i <- as.vector(X %*% delta)
    p.i <- if (link == "logit") stats::plogis(eta.i) else eta.i

    if (!is.null(gs)) {
      p.i <- pmin(pmax(p.i, gs[i, 1L]), gs[i, 2L])
    } else {
      p.i <- pmin(pmax(p.i, eps), 1 - eps)
    }

    prob.item[, i] <- p.i[reduced.idx]
  }

  prob.item
}

#' Fit MGGUM Model via Stan
#'
#' @param response An N x I matrix of integer responses (0, 1, ..., K_i - 1)
#' @param D Number of latent dimensions
#' @param Q.matrix An I x D matrix with values -1, 0, or 1.
#'   \itemize{
#'     \item \code{1}: active dimension with \code{delta} constrained/regularized on the positive side
#'     \item \code{-1}: active dimension with \code{delta} constrained/regularized on the negative side
#'     \item \code{0}: \code{a} and \code{delta} fixed to 0 (inactive dimension)
#'   }
#'   The sign controls the side of the item-location prior/support, not the
#'   sign of discrimination; \code{a} is nonnegative whenever \code{|Q| = 1}.
#'   If NULL, builds a matrix of all 1's (default: positive-side locations).
#' @param length.poly Optional scalar or length-I integer vector giving the
#'   intended number of categories per item. If \code{NULL}, inferred as
#'   \code{max(response[, i]) + 1} for each item.
#' @param control.model Optional named list of model-level controls and
#'   hyperparameters.
#' @param control.method Optional named list of method-specific controls. For
#'   Stan this includes \code{cores}, \code{vis}, \code{seed}, \code{chains},
#'   \code{iter}, \code{warmup}, \code{thin}, \code{init}, \code{algorithm},
#'   and Stan sampler controls.
#'
#' @noRd
fit.MGGUM.stan <- function(response, D = 2, Q.matrix = NULL, length.poly = NULL,
                       control.model = NULL,
                       control.method = NULL,
                       .call = NULL) {
  call <- if (is.null(.call)) match.call() else .call
  control <- fc_as_control_list(control.model, "control.model")
  control.method <- fc_as_control_list(control.method, "control.method")
  common.method <- fit_common_method_control(control.method)
  stan.method <- fc_stan_method_control(control.method)
  chains <- stan.method$chains
  iter <- stan.method$iter
  warmup <- stan.method$warmup
  thin <- stan.method$thin
  init <- stan.method$init
  algorithm <- stan.method$algorithm

  sm <- stanmodels[["MGGUM"]]

  if (is.null(sm)) {
    stop("Stan model MGGUM not found !", call. = FALSE)
  }

  mcmc_env <- setup_mcmc_env(common.method$cores)
  on.exit(restore_mcmc_env(mcmc_env), add = TRUE)

  stan_control <- build_stan_control(algorithm, control.method)

  N <- nrow(response)
  I <- ncol(response)

  response <- istem_prepare_response(response, binary = FALSE)
  length.poly <- istem_prepare_length_poly(response, length.poly)
  max_poly <- max(length.poly)

  Q.matrix <- istem_prepare_signed_q(I, D, Q.matrix)

  corr_df <- free_corr_npar(D)

  # Convert response to long format
  data.long <- data.frame(
    y   = as.vector(response),
    pid = rep(1:N, times = I),
    iid = rep(1:I, each = N)
  )

  theta_mu_raw <- get_ctrl("theta.mu", rep(0, D), control)
  theta_mu     <- array(theta_mu_raw, dim = D)

  stan.data <- list(
    N_obs           = nrow(data.long),
    N               = N,
    I               = I,
    D               = D,
    max_poly        = max_poly,
    y               = data.long$y,
    pid             = data.long$pid,
    iid             = data.long$iid,
    length_poly     = as.integer(length.poly),
    Q_matrix        = Q.matrix,
    a_mu            = get_ctrl("a.mu", 0, control),
    a_sigma         = get_ctrl("a.sigma", 0.5, control),
    delta_pos_mu    = get_ctrl("delta.pos.mu", 1, control),
    delta_pos_sigma = get_ctrl("delta.pos.sigma", 0.5, control),
    delta_neg_mu    = get_ctrl("delta.neg.mu", -1, control),
    delta_neg_sigma = get_ctrl("delta.neg.sigma", 0.5, control),
    tau_mu          = get_ctrl("tau.mu", 0, control),
    tau_sigma       = get_ctrl("tau.sigma", 0.5, control),
    theta_mu        = theta_mu
  )

  set.seed(common.method$seed)

  stan.obj <- suppressWarnings(
    rstan::sampling(
      object        = sm,
      data          = stan.data,
      chains        = chains,
      iter          = iter,
      warmup        = warmup,
      thin          = thin,
      init          = init,
      algorithm     = algorithm,
      cores         = mcmc_env$cores,
      control       = stan_control,
      verbose       = common.method$vis,
      show_messages = common.method$vis
    )
  )

  MCMC.obj <- rstan::extract(stan.obj, permuted = TRUE)
  stan.sum <- rstan::summary(stan.obj)$summary

  # ---- Extract theta ----
  theta_est <- extract_theta_stan(MCMC.obj, stan.sum, N, D, rownames(response))

  # ---- Build par matrix: I x (D + D + max_poly) ----
  # Columns: a1..aD, delta1..deltaD, tau0(=0), tau1, ..., tau_{max_poly-1}
  par      <- matrix(NA_real_, nrow = I, ncol = D + D + max_poly)
  par.se   <- matrix(NA_real_, nrow = I, ncol = D + D + max_poly)
  par.Rhat <- matrix(NA_real_, nrow = I, ncol = D + D + max_poly)

  # Extract a parameters
  a_arr <- ensure_3d(MCMC.obj$a)
  a     <- apply(a_arr, c(2, 3), mean)
  a.se  <- apply(a_arr, c(2, 3), sd)

  par[, 1:D]      <- a
  par.se[, 1:D]   <- a.se
  par.Rhat[, 1:D] <- extract_rhat_matrix(stan.sum, "a", I, D)

  # Extract delta parameters
  delta_arr <- ensure_3d(MCMC.obj$delta)
  delta     <- apply(delta_arr, c(2, 3), mean)
  delta.se  <- apply(delta_arr, c(2, 3), sd)

  par[, (D + 1):(D + D)]      <- delta
  par.se[, (D + 1):(D + D)]   <- delta.se
  par.Rhat[, (D + 1):(D + D)] <- extract_rhat_matrix(stan.sum, "delta", I, D)

  # tau0 = 0 (fixed for all items)
  par[, D + D + 1]      <- 0
  par.se[, D + D + 1]   <- NA_real_
  par.Rhat[, D + D + 1] <- NA_real_

  # Extract tau parameters from MCMC output
  tau_arr <- ensure_3d(MCMC.obj$tau)
  tau     <- apply(tau_arr, c(2, 3), mean)
  tau.se  <- apply(tau_arr, c(2, 3), sd)
  for (i in 1:I) {
    Ki <- length.poly[i]
    if (Ki > 1) {
      tau_cols <- (D + D + 2):(D + D + Ki)
      par[i, tau_cols]      <- tau[i, 2:Ki]
      par.se[i, tau_cols]   <- tau.se[i, 2:Ki]
      # Map back to stan.sum indices: tau[i,2] .. tau[i,Ki]
      tau_idx_i <- grep(paste0("^tau\\[", i, ","), rownames(stan.sum))
      tau_rhat_i <- stan.sum[tau_idx_i, "Rhat"]
      par.Rhat[i, tau_cols] <- tau_rhat_i[2:Ki]
    }
  }

  par.free <- mggum_par_free_mask(Q.matrix, length.poly, max_poly)
  colnames(par) <- colnames(par.se) <- colnames(par.Rhat) <- colnames(par.free) <-
    c(paste0("a", 1:D),
      paste0("delta", 1:D),
      paste0("tau", 0:(max_poly - 1)))
  rownames(par) <- rownames(par.se) <- rownames(par.Rhat) <- rownames(par.free) <-
    colnames(response)
  par.se[!par.free] <- NA_real_
  par.Rhat[!par.free] <- NA_real_
  npar <- sum(par.free) + corr_df

  # ---- Extract Corr ----
  Corr_est <- extract_corr_stan(MCMC.obj, stan.sum, D)

  # ---- Assemble results ----
  par_est   <- list(est = par, se = par.se, Rhat = par.Rhat, free = par.free)

  results <- list(
    npar     = npar,
    method   = "stan",
    theta    = theta_est,
    par      = par_est,
    Corr     = Corr_est,
    stan.obj = stan.obj,
    MCMC.obj = MCMC.obj,
    log_lik = MCMC.obj$log_lik,
    Q.matrix    = Q.matrix,
    length.poly = length.poly,
    call        = call,
    arguments = list(
      response  = response,
      D         = D,
      Q.matrix  = Q.matrix,
      method    = "stan",
      length.poly = length.poly,
      chains    = chains,
      iter      = iter,
      warmup    = warmup,
      thin      = thin,
      init      = init,
      algorithm = algorithm,
      cores     = common.method$cores,
      vis       = common.method$vis,
      seed      = common.method$seed,
      control.model = control,
      control.method = fit_effective_method_control(
        control.method, method = stan.method, common = common.method
      )
    )
  )

  class(results) <- "MGGUM"

  L <- get_ctrl("L", NULL, control)
  results$logLik <- logLik.MGGUM(results, L=L)

  return(results)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

cpp_fcggum_block_loglik_weighted <- function(theta, par_block, response, patterns_total, patterns, weight) {
    .Call(`_ForceChoice_cpp_fcggum_block_loglik_weighted`, theta, par_block, response, patterns_total, patterns, weight)
}

cpp_fcmirt_block_loglik_weighted <- function(theta, par_block, response, patterns_total, patterns, weight) {
    .Call(`_ForceChoice_cpp_fcmirt_block_loglik_weighted`, theta, par_block, response, patterns_total, patterns, weight)
}

cpp_gibbs_mirt_theta <- function(theta_, par_, response_, theta_mu_, chol_corr_, log_diag_sum, step = 15.0, lower = -6.0, upper = 6.0) {
    .Call(`_ForceChoice_cpp_gibbs_mirt_theta`, theta_, par_, response_, theta_mu_, chol_corr_, log_diag_sum, step, lower, upper)
}

cpp_gibbs_mgpcm_theta <- function(theta_, par_, response_, length_poly_, theta_mu_, chol_corr_, log_diag_sum, step = 15.0, lower = -6.0, upper = 6.0) {
    .Call(`_ForceChoice_cpp_gibbs_mgpcm_theta`, theta_, par_, response_, length_poly_, theta_mu_, chol_corr_, log_diag_sum, step, lower, upper)
}

cpp_gibbs_mggum_theta <- function(theta_, par_, response_, length_poly_, theta_mu_, chol_corr_, log_diag_sum, step = 15.0, lower = -6.0, upper = 6.0) {
    .Call(`_ForceChoice_cpp_gibbs_mggum_theta`, theta_, par_, response_, length_poly_, theta_mu_, chol_corr_, log_diag_sum, step, lower, upper)
}

cpp_gibbs_fcmirt_theta <- function(theta_, par_, response_, theta_mu_, chol_corr_, log_diag_sum, patterns_total, patterns, step = 15.0, lower = -6.0, upper = 6.0) {
    .Call(`_ForceChoice_cpp_gibbs_fcmirt_theta`, theta_, par_, response_, theta_mu_, chol_corr_, log_diag_sum, patterns_total, patterns, step, lower, upper)
}

cpp_gibbs_fcggum_theta <- function(theta_, par_, response_, theta_mu_, chol_corr_, log_diag_sum, patterns_total, patterns, step = 15.0, lower = -6.0, upper = 6.0) {
    .Call(`_ForceChoice_cpp_gibbs_fcggum_theta`, theta_, par_, response_, theta_mu_, chol_corr_, log_diag_sum, patterns_total, patterns, step, lower, upper)
}

cpp_gibbs_tirt_theta <- function(theta_, Q_matrix_, lambda_, psi_, gamma_, Y_, obs_by_person_, Idx_item_i_, Idx_item_k_, Idx_pair_, theta_mu_, chol_corr_, log_diag_sum, step = 15.0, lower = -6.0, upper = 6.0) {
    .Call(`_ForceChoice_cpp_gibbs_tirt_theta`, theta_, Q_matrix_, lambda_, psi_, gamma_, Y_, obs_by_person_, Idx_item_i_, Idx_item_k_, Idx_pair_, theta_mu_, chol_corr_, log_diag_sum, step, lower, upper)
}

cpp_compute_EIs <- function(prob, K) {
    .Call(`_ForceChoice_cpp_compute_EIs`, prob, K)
}

cpp_moments_predicted <- function(prob, pi, pairs_total, is_poly, K) {
    .Call(`_ForceChoice_cpp_moments_predicted`, prob, pi, pairs_total, is_poly, K)
}

cpp_moments_predicted_grouped <- function(prob, pi, pairs_total, is_poly, K, group) {
    .Call(`_ForceChoice_cpp_moments_predicted_grouped`, prob, pi, pairs_total, is_poly, K, group)
}

cpp_compute_Xi <- function(EIs, EIs2, pi, pairs_total) {
    .Call(`_ForceChoice_cpp_compute_Xi`, EIs, EIs2, pi, pairs_total)
}

cpp_compute_Xi_grouped <- function(EIs, EIs2, pi, pairs_total, group) {
    .Call(`_ForceChoice_cpp_compute_Xi_grouped`, EIs, EIs2, pi, pairs_total, group)
}

cpp_compute_E2 <- function(EIs, EIs2, pi) {
    .Call(`_ForceChoice_cpp_compute_E2`, EIs, EIs2, pi)
}

cpp_compute_E2_grouped <- function(EIs, EIs2, pi, group) {
    .Call(`_ForceChoice_cpp_compute_E2_grouped`, EIs, EIs2, pi, group)
}

cpp_make_pairs <- function(n) {
    .Call(`_ForceChoice_cpp_make_pairs`, n)
}

cpp_filter_pairs_between_groups <- function(pairs_total, group) {
    .Call(`_ForceChoice_cpp_filter_pairs_between_groups`, pairs_total, group)
}

cpp_gof_observed_summary <- function(response, pairs_total) {
    .Call(`_ForceChoice_cpp_gof_observed_summary`, response, pairs_total)
}

cpp_gof_null_loglik <- function(response, K, response_type, nominal_groups, first_moments) {
    .Call(`_ForceChoice_cpp_gof_null_loglik`, response, K, response_type, nominal_groups, first_moments)
}

cpp_mirt_item_loglik_weighted <- function(theta, a, b, c, upper, response, weight) {
    .Call(`_ForceChoice_cpp_mirt_item_loglik_weighted`, theta, a, b, c, upper, response, weight)
}

cpp_mgpcm_item_loglik_weighted <- function(theta, a, d, response, weight) {
    .Call(`_ForceChoice_cpp_mgpcm_item_loglik_weighted`, theta, a, d, response, weight)
}

cpp_mggum_item_loglik_weighted <- function(theta, par_i, Ki, response, weight) {
    .Call(`_ForceChoice_cpp_mggum_item_loglik_weighted`, theta, par_i, Ki, response, weight)
}

cpp_tirt_item_loglik_weighted <- function(qt_target, qt_other, lambda_other, psi_other, gamma_pair, target_is_first, Y, weight, lambda_val, psi_val) {
    .Call(`_ForceChoice_cpp_tirt_item_loglik_weighted`, qt_target, qt_other, lambda_other, psi_other, gamma_pair, target_is_first, Y, weight, lambda_val, psi_val)
}

cpp_tirt_pair_loglik_weighted <- function(mu_no_gamma, denom, Y, weight, gamma_val) {
    .Call(`_ForceChoice_cpp_tirt_pair_loglik_weighted`, mu_no_gamma, denom, Y, weight, gamma_val)
}

cpp_model_MIRT <- function(theta, par) {
    .Call(`_ForceChoice_cpp_model_MIRT`, theta, par)
}

cpp_model_MGGUM <- function(theta, par) {
    .Call(`_ForceChoice_cpp_model_MGGUM`, theta, par)
}

cpp_model_MGPCM <- function(theta, par) {
    .Call(`_ForceChoice_cpp_model_MGPCM`, theta, par)
}

cpp_model_FCGGUM <- function(theta, par, patterns_total, patterns) {
    .Call(`_ForceChoice_cpp_model_FCGGUM`, theta, par, patterns_total, patterns)
}

cpp_model_FCMIRT <- function(theta, par, patterns_total, patterns) {
    .Call(`_ForceChoice_cpp_model_FCMIRT`, theta, par, patterns_total, patterns)
}

cpp_model_FCDCM <- function(zeta, par, patterns) {
    .Call(`_ForceChoice_cpp_model_FCDCM`, zeta, par, patterns)
}

cpp_model_FCDCM_marginal <- function(theta, delta1, delta0, par, alpha_patterns, zeta_patterns, patterns) {
    .Call(`_ForceChoice_cpp_model_FCDCM_marginal`, theta, delta1, delta0, par, alpha_patterns, zeta_patterns, patterns)
}

cpp_fcdcm_case_loglik_class_prob <- function(theta, response, delta1, delta0, alpha_patterns, class_prob) {
    .Call(`_ForceChoice_cpp_fcdcm_case_loglik_class_prob`, theta, response, delta1, delta0, alpha_patterns, class_prob)
}

cpp_gibbs_fcdcm_theta <- function(theta, response, delta1, delta0, par, alpha_patterns, zeta_patterns, patterns, step, lower, upper) {
    .Call(`_ForceChoice_cpp_gibbs_fcdcm_theta`, theta, response, delta1, delta0, par, alpha_patterns, zeta_patterns, patterns, step, lower, upper)
}

cpp_fcdcm_sample_alpha <- function(theta, response, delta1, delta0, alpha_patterns, zeta_patterns, class_prob) {
    .Call(`_ForceChoice_cpp_fcdcm_sample_alpha`, theta, response, delta1, delta0, alpha_patterns, zeta_patterns, class_prob)
}

cpp_fcdcm_posterior_alpha_class_prob <- function(theta, response, delta1, delta0, alpha_patterns, class_prob) {
    .Call(`_ForceChoice_cpp_fcdcm_posterior_alpha_class_prob`, theta, response, delta1, delta0, alpha_patterns, class_prob)
}

cpp_fcdcm_latent_support <- function(theta, pi, delta1, delta0, par, alpha_patterns, zeta_patterns, patterns) {
    .Call(`_ForceChoice_cpp_fcdcm_latent_support`, theta, pi, delta1, delta0, par, alpha_patterns, zeta_patterns, patterns)
}

cpp_model_FCGDINA <- function(alpha_pat, flat_design, design_rows, design_cols, design_offset, delta_offset, delta, class_map_list, patterns_total, patterns, C, I_states) {
    .Call(`_ForceChoice_cpp_model_FCGDINA`, alpha_pat, flat_design, design_rows, design_cols, design_offset, delta_offset, delta, class_map_list, patterns_total, patterns, C, I_states)
}

cpp_fcgdina_class_posterior <- function(prob_class, response, patterns, prior, N, B, C) {
    .Call(`_ForceChoice_cpp_fcgdina_class_posterior`, prob_class, response, patterns, prior, N, B, C)
}

cpp_fcgdina_weighted_loglik <- function(prob_class, response, patterns, class_weight, N, B, C) {
    .Call(`_ForceChoice_cpp_fcgdina_weighted_loglik`, prob_class, response, patterns, class_weight, N, B, C)
}

cpp_fcgdina_weighted_loglik_block <- function(prob_block, response_block, class_weight, N, C) {
    .Call(`_ForceChoice_cpp_fcgdina_weighted_loglik_block`, prob_block, response_block, class_weight, N, C)
}

cpp_fcgdina_marginal_loglik <- function(prob_class, response, patterns, prior, N, B, C) {
    .Call(`_ForceChoice_cpp_fcgdina_marginal_loglik`, prob_class, response, patterns, prior, N, B, C)
}

cpp_fcgdina_delta_logit_to_prob <- function(delta_logit, flat_design, design_rows, design_cols, design_offset, delta_offset, I_states) {
    .Call(`_ForceChoice_cpp_fcgdina_delta_logit_to_prob`, delta_logit, flat_design, design_rows, design_cols, design_offset, delta_offset, I_states)
}

cpp_get_permutations <- function(values, r) {
    .Call(`_ForceChoice_cpp_get_permutations`, values, r)
}

cpp_istem_local_patterns <- function(patterns, items) {
    .Call(`_ForceChoice_cpp_istem_local_patterns`, patterns, items)
}

cpp_istem_batch_var <- function(plist, n) {
    .Call(`_ForceChoice_cpp_istem_batch_var`, plist, n)
}

cpp_extract_rhat_matrix <- function(rhat, row_names, param_name, nrow_out, ncol_out) {
    .Call(`_ForceChoice_cpp_extract_rhat_matrix`, rhat, row_names, param_name, nrow_out, ncol_out)
}

cpp_istem_log_mvn_kernel <- function(x, mu, chol_sigma, log_diag_sum) {
    .Call(`_ForceChoice_cpp_istem_log_mvn_kernel`, x, mu, chol_sigma, log_diag_sum)
}

cpp_data_from_response_block <- function(response, patterns) {
    .Call(`_ForceChoice_cpp_data_from_response_block`, response, patterns)
}

cpp_response_from_data_block <- function(data, patterns) {
    .Call(`_ForceChoice_cpp_response_from_data_block`, data, patterns)
}

cpp_get_block_items_from_data <- function(data) {
    .Call(`_ForceChoice_cpp_get_block_items_from_data`, data)
}

cpp_get_block_items_fcdcm <- function(data) {
    .Call(`_ForceChoice_cpp_get_block_items_fcdcm`, data)
}

cpp_loglik_binary <- function(prob, response, pi) {
    .Call(`_ForceChoice_cpp_loglik_binary`, prob, response, pi)
}

cpp_loglik_indexed <- function(prob, response, pi, block_sizes, response_base = 0L) {
    .Call(`_ForceChoice_cpp_loglik_indexed`, prob, response, pi, block_sizes, response_base)
}

cpp_tirt_expand_response_full <- function(response, pairs_matrix, pairs_value, block_pair_counts) {
    .Call(`_ForceChoice_cpp_tirt_expand_response_full`, response, pairs_matrix, pairs_value, block_pair_counts)
}

cpp_loglik_binary_person_pairs <- function(prob, response, pi, pairs_matrix, pairs_value, block_pair_counts) {
    .Call(`_ForceChoice_cpp_loglik_binary_person_pairs`, prob, response, pi, pairs_matrix, pairs_value, block_pair_counts)
}

cpp_get_response_from_data_tirt <- function(data, block_items, fc_type) {
    .Call(`_ForceChoice_cpp_get_response_from_data_tirt`, data, block_items, fc_type)
}

cpp_get_data_from_response_tirt <- function(response, block_items, fc_type, pairs_value) {
    .Call(`_ForceChoice_cpp_get_data_from_response_tirt`, response, block_items, fc_type, pairs_value)
}

cpp_extract_rhat_vector <- function(rhat, row_names, param_name, n_out) {
    .Call(`_ForceChoice_cpp_extract_rhat_vector`, rhat, row_names, param_name, n_out)
}

cpp_check_response <- function(response, length_poly) {
    .Call(`_ForceChoice_cpp_check_response`, response, length_poly)
}

#' Fit the Forced-Choice Diagnostic Classification Model (FCDCM)
#'
#' @description
#' Fits a higher-order cognitive diagnostic model for forced-choice (paired-
#' comparison) data. Each block presents two statements, and the respondent
#' selects the one they agree with more. The discrete attribute mastery
#' profile \eqn{\boldsymbol{\alpha}_j} for each person is marginalized over
#' all \eqn{2^D} possible patterns, linking a continuous higher-order latent
#' trait \eqn{\theta_j} to the DCM condensation rule at the statement level.
#'
#' @section Model Specification:
#'
#' \strong{Higher-order latent structure.}
#' A unidimensional continuous trait \eqn{\theta_j \sim N(0, 1)} governs the
#' probability of mastering each attribute \eqn{d = 1, \dots, D}:
#' \deqn{
#'   P(\alpha_{jd} = 1 \mid \theta_j) =
#'   \frac{1}{1 + \exp\bigl[-\bigl(\delta_{1d}\,\theta_j - \delta_{1d}\,\delta_{0d}\bigr)\bigr]},
#' }
#' where \eqn{\delta_{1d} > 0} is the discrimination and
#' \eqn{\delta_{0d}} is the difficulty (threshold) for attribute \eqn{d}.
#' Attributes are conditionally independent given \eqn{\theta_j}.
#'
#' \strong{Statement-level condensation.}
#' For each statement \eqn{i = 1, \dots, I} with Q-vector
#' \eqn{\mathbf{q}_i = (q_{i1}, \dots, q_{iD})'}, the latent mastery status
#' \eqn{\zeta_{ij} \in \{0, 1\}} is determined by one of two condensation
#' rules:
#'
#' \emph{DINA (conjunctive):}
#' \deqn{
#'   \zeta_{ij} = \prod_{d: q_{id} = 1} \alpha_{jd}.
#' }
#' All required attributes must be mastered.
#'
#' \emph{DINO (disjunctive):}
#' \deqn{
#'   \zeta_{ij} = 1 - \prod_{d: q_{id} = 1} (1 - \alpha_{jd}).
#' }
#' At least one required attribute must be mastered.
#'
#' \strong{Forced-choice block response.}
#' Block \eqn{b} consists of two statements \eqn{(A_b, B_b)}. The probability
#' that the respondent selects statement \eqn{A_b} over \eqn{B_b} follows
#' the original FC-DCM specification and fixes equal-condensation
#' probabilities to chance:
#' \deqn{
#'   P(\text{choose } A_b \mid \zeta_{A_b}, \zeta_{B_b}) =
#'   \begin{cases}
#'     \eta_{0b},        & \text{if } \zeta_{A_b} < \zeta_{B_b}, \\
#'     0.5,              & \text{if } \zeta_{A_b} = \zeta_{B_b}, \\
#'     0.5+\eta_{ABb},   & \text{if } \zeta_{A_b} > \zeta_{B_b}.
#'   \end{cases}
#' }
#' with \eqn{0 < \eta_{0b} < 0.5} and \eqn{0 < \eta_{ABb} < 0.5}. Thus
#' equal statement-level mastery produces chance choice, choosing the lower
#' mastery statement has probability \eqn{\eta_{0b}}, and choosing the higher
#' mastery statement has probability \eqn{0.5+\eta_{ABb}}. The probability
#' of choosing \eqn{B_b} is the complement.
#'
#' The marginal likelihood integrates over all \eqn{2^D} attribute patterns:
#' \deqn{
#'   P(\mathbf{Y}_j \mid \theta_j) =
#'   \sum_{\boldsymbol{\alpha} \in \{0,1\}^D}
#'   P(\boldsymbol{\alpha} \mid \theta_j) \times
#'   \prod_{b=1}^{B} P(Y_{jb} \mid \zeta_{A_b}, \zeta_{B_b}).
#' }
#'
#' @section Design and Identifiability:
#'
#' A block with \eqn{\zeta_{A_b}=\zeta_{B_b}} contributes no direct
#' information for distinguishing the two equal condensation states
#' (both are fixed at 0.5).  For single-attribute statements and \eqn{D=2},
#' if all blocks compare the same ordered attribute pair, the response
#' distribution distinguishes \code{10} from \code{01} but leaves \code{00}
#' and \code{11} to be separated only by the higher-order structural model.
#' Good global fit can therefore coexist with weak profile classification.
#'
#' A forced-choice Q/block design should provide repeated cross-attribute
#' comparisons, avoid pairing statements with identical Q-vectors, and keep
#' each attribute represented in both statement positions.  A practical
#' numerical target, consistent with the original FC-DCM simulation logic, is
#' that each attribute appears multiple times as the first/predominant
#' statement and multiple times as the second/inferior statement, with
#' comparisons distributed across different attribute pairs.  The automatic
#' simulator uses this balanced assembly rule when \code{block.items} is not
#' supplied.
#'
#' @section Estimation Methods:
#'
#' \describe{
#'   \item{\strong{Stan} (\code{method = "stan"}):}{
#'     Full Bayesian inference via HMC. The discrete attribute profile is
#'     exactly marginalized in the Stan likelihood.}
#'   \item{\strong{iStEM} (\code{method = "iStEM"}):}{
#'     Improved Stochastic EM. Person sampling considers the joint space of
#'     \eqn{\theta_j} and \eqn{\boldsymbol{\alpha}_j}. Parameter updates
#'     use closed-form or numerical optimization steps.}
#' }
#'
#' @param data An \eqn{N \times B} forced-choice data matrix. Each cell must
#'   be a two-statement ranking string such as \code{"1>3"} (first is
#'   selected over second), or \code{0}/\code{1} coded responses where
#'   \code{1} means selecting the first statement in the block.
#' @param Q.matrix An \eqn{I \times D} binary Q-matrix for the statement-
#'   level DCM. \eqn{q_{id} = 1} if attribute \eqn{d} is required by
#'   statement \eqn{i}. Must have exactly \eqn{2B} rows.
#' @param block.items A list of length \eqn{B}; each element is an integer
#'   vector of the two global statement indices in that block. If \code{NULL},
#'   parsed from \code{data} assuming \code{"A>B"} format where \code{A} and
#'   \code{B} are statement indices.
#' @param dcm.type Character vector (length 1 or \eqn{I}): \code{"DINA"}
#'   (default, conjunctive) or \code{"DINO"} (disjunctive). Recycled if
#'   scalar.
#' @param method Estimation method: \code{"iStEM"} (default) or
#'   \code{"stan"}.
#' @param control.model A named list of model-level hyperparameters for the
#'   higher-order DCM. Supported entries:
#'   \describe{
#'     \item{\code{delta1.mu}, \code{delta1.sigma}}{Prior mean and SD
#'           for \eqn{\delta_{1d}} (higher-order discrimination parameters;
#'           log-normal). Defaults: 0, 0.5. The log-normal prior constrains
#'           \eqn{\delta_{1d} > 0}, reflecting the assumption that higher
#'           \eqn{\theta_j} increases the probability of mastering each
#'           attribute. Larger values of \eqn{\delta_{1d}} indicate stronger
#'           relationships between the continuous trait \eqn{\theta_j} and
#'           attribute mastery.}
#'     \item{\code{delta0.mu}, \code{delta0.sigma}}{Prior mean and SD
#'           for \eqn{\delta_{0d}} (higher-order difficulty/threshold
#'           parameters; normal). Defaults: 0, 1. The probability of
#'           mastering attribute \eqn{d} at \eqn{\theta_j = 0} is
#'           \eqn{\{1+\exp(\delta_{1d}\delta_{0d})\}^{-1}}.
#'           Higher values of \eqn{\delta_{0d}} indicate more difficult
#'           attributes.}
#'     \item{\code{eta0.mu}, \code{eta0.sigma}}{Prior mean and SD for
#'           \eqn{\eta_{0b}} (truncated normal on (0, 0.5)). Defaults: 0.10,
#'           0.10. The prior encodes the expectation that \eqn{\eta_{0b}}
#'           is small (rarely choose the lower-mastery statement).}
#'     \item{\code{etaAB.mu}, \code{etaAB.sigma}}{Prior mean and SD for
#'           \eqn{\eta_{ABb}} (truncated normal on (0, 0.5)). Defaults: 0.30,
#'           0.10. The prior encodes the expectation that \eqn{\eta_{ABb}}
#'           is larger (usually choose the higher-mastery statement).}
#'     \item{\code{use.prior}}{Logical; if \code{TRUE} (default), the
#'           normal priors on the block response parameters are applied
#'           during iStEM estimation. Set to \code{FALSE} to use flat
#'           likelihood updates.}
#'     \item{\code{par}}{Optional initial \eqn{B \times 2} matrix for iStEM
#'           with columns \code{eta0} and \code{etaAB}.}
#'     \item{\code{L}}{Theta grid size for numerical integration and
#'           the unidimensional iStEM theta Gibbs sampler. Default: 61.}
#'     \item{\code{theta.lower}, \code{theta.upper}}{Bounds for the
#'           unidimensional theta grid used by marginal log-likelihood
#'           computation and iStEM block Gibbs sampling. Defaults: -6, 6.}
#'   }
#' @param control.method A named list of method-specific tuning parameters.
#'   Common entries (used by both Stan and iStEM):
#'   \describe{
#'     \item{\code{cores}}{Number of CPU cores for parallel chains
#'           (Stan) or ignored (iStEM). Default: the number of
#'           \code{chains}.}
#'     \item{\code{vis}}{Logical; if \code{TRUE} (default), prints progress
#'           information to the console.}
#'     \item{\code{seed}}{Random seed for reproducibility.
#'           Default: a random integer.}
#'   }
#'   Stan-specific entries:
#'   \describe{
#'     \item{\code{chains}}{Number of MCMC chains (default: 2).}
#'     \item{\code{iter}}{Total iterations per chain (default: 5000).}
#'     \item{\code{warmup}}{Warmup/burn-in iterations per chain
#'           (default: \code{iter / 2}).}
#'     \item{\code{thin}}{Thinning interval (default: 1).}
#'     \item{\code{init}}{Initial values: \code{"random"} (default)
#'           for uniform(-2, 2) initialization, or a list of initial
#'           values per chain.}
#'     \item{\code{algorithm}}{MCMC algorithm: \code{"HMC"} (default),
#'           \code{"HMC"}, or \code{"Fixed_param"}.}
#'     \item{\code{adapt_delta}}{Target average acceptance probability
#'           (NUTS; default: 0.95). The discrete attribute-profile
#'           marginalization can create a challenging posterior;
#'           consider increasing to 0.9--0.95 if divergences occur.}
#'     \item{\code{max_treedepth}}{Maximum tree depth (NUTS; default: 10).
#'           Increase if "max treedepth exceeded" warnings appear.}
#'     \item{\code{stepsize}}{Initial step size for the leapfrog
#'           integrator (auto-tuned by Stan if not set).}
#'     \item{\code{int_time}}{Total integration time for HMC trajectories
#'           (only when \code{algorithm = "HMC"}).}
#'     \item{\code{metric}}{Mass matrix type: \code{"unit_e"},
#'           \code{"diag_e"} (default), or \code{"dense_e"}.}
#'     \item{\code{adapt_engaged}}{Logical; if \code{TRUE} (default),
#'           warmup adaptation is enabled.}
#'     \item{\code{adapt_init_buffer}, \code{adapt_term_buffer},
#'           \code{adapt_window}}{Warmup adaptation scheduling parameters
#'           (defaults: 25, 50, 25).}
#'   }
#'   iStEM-specific entries:
#'   \describe{
#'     \item{\code{M}}{Number of burn-in batches retained for Geweke
#'           convergence diagnosis (default: 10; must be \eqn{\ge 2}).}
#'     \item{\code{B}}{Batch size: MCMC iterations per batch (default: 20).}
#'     \item{\code{burnin.maxitr}}{Maximum burn-in batches (default: 100).}
#'     \item{\code{maxitr}}{Maximum total batches (default: 2000).}
#'     \item{\code{eps1}}{Geweke z-score convergence threshold for
#'           burn-in (default: 1.5).}
#'     \item{\code{eps2}}{Monte Carlo error tolerance for the final
#'           chain (default: 0.4).}
#'     \item{\code{frac1}, \code{frac2}}{Fractions for the Geweke
#'           diagnostic (defaults: 0.1, 0.5).}
#'     \item{\code{optim.maxit}}{Maximum iterations for parameter
#'           optimization steps (default: 50). In FCDCM iStEM, the
#'           higher-order \eqn{\delta} parameters and block \eqn{\eta}
#'           parameters are updated via closed-form or numerical
#'           optimization.}
#'     \item{\code{fix.corr}}{Logical; in the FCDCM, the latent trait
#'           is unidimensional, so this parameter has no effect
#'           (default: \code{FALSE}).}
#'     \item{\code{estimate.se}}{Logical; if \code{TRUE} (default),
#'           standard errors are computed from the final Monte Carlo
#'           chain.}
#'   }
#'
#' @return An object of class \code{"FCDCM"} containing:
#' \describe{
#'   \item{\code{npar}}{Number of free parameters (\eqn{2D + 2B}).}
#'   \item{\code{method}}{\code{"stan"} or \code{"iStEM"}.}
#'   \item{\code{theta}}{List with \code{est}, \code{se}, \code{Rhat}
#'         (\eqn{N \times 1}); higher-order trait estimates.}
#'   \item{\code{delta}}{List with \code{est}, \code{se}, \code{Rhat}
#'         (\eqn{D \times 2}); higher-order \eqn{\delta_1}, \eqn{\delta_0}
#'         parameters.}
#'   \item{\code{par}}{List with \code{est}, \code{se}, \code{Rhat}
#'         (\eqn{B \times 2}); block response parameters
#'         \code{eta0} and \code{etaAB}.}
#'   \item{\code{alpha}}{List with \code{est}, \code{se}, \code{Rhat}, and
#'         \code{prob}; \code{est} is the binary classified attribute profile
#'         (\eqn{N \times D}) and \code{prob} contains posterior mastery
#'         probabilities.}
#'   \item{\code{class.post}}{Posterior probabilities for all \eqn{2^D}
#'         attribute profiles.}
#'   \item{\code{alpha.patterns}, \code{zeta.patterns}}{Full enumeration
#'         of attribute mastery patterns and their condensation outputs.}
#'   \item{\code{logLik}}{Marginal log-likelihood (class \code{"logLik"}).}
#' }
#'
#' @references
#' de la Torre, J., & Douglas, J. A. (2004). Higher-order latent trait
#'   models for cognitive diagnosis. \emph{Psychometrika}, 69(3), 333--353.
#'   \doi{10.1007/BF02295640}
#'
#' Junker, B. W., & Sijtsma, K. (2001). Cognitive assessment models with
#'   few assumptions, and connections with nonparametric item response
#'   theory. \emph{Applied Psychological Measurement}, 25(3), 258--272.
#'
#' @seealso
#' \code{\link{sim.data.FCDCM}}, \code{\link{get.fit.index.FCDCM}},
#' \code{\link{logLik.FCDCM}}
#'
#' @examples
#' sim <- sim.data.FCDCM(N.person = 20, N.block = 3, D = 2,
#'                       dcm.type = "DINA")
#' fit <- fit.FCDCM(sim$data, Q.matrix = sim$Q.matrix,
#'                  block.items = sim$block.items,
#'                  dcm.type = "DINA", method = "iStEM",
#'                  control.method = list(
#'                    seed = 123, vis = FALSE,
#'                    M = 2, B = 2, burnin.maxitr = 2,
#'                    maxitr = 3, eps1 = 10, eps2 = 10,
#'                    estimate.se = FALSE))
#' head(fit$alpha$est)                # binary attribute profiles (0/1)
#' head(fit$alpha$prob)               # marginal mastery probabilities
#' gof <- get.fit.index(fit)
#' summary(gof)
#'
#' @export
fit.FCDCM <- function(data, Q.matrix, block.items = NULL,
                      dcm.type = "DINA",
                      method = c("iStEM", "stan"),
                      control.model = NULL,
                      control.method = NULL) {
  call <- match.call()
  method <- match.arg(method)

  block.items <- resolve_block_items(block.items, data)

  control.model  <- fc_as_control_list(control.model, "control.model")
  control.method <- fc_as_control_list(control.method, "control.method")

  if (method == "stan") {
    return(fit.FCDCM.stan(
      data = data,
      Q.matrix = Q.matrix,
      block.items = block.items,
      dcm.type = dcm.type,
      control.model = control.model,
      control.method = control.method,
      .call = call
    ))
  }

  fit.FCDCM.iStEM(
    data = data,
    Q.matrix = Q.matrix,
    block.items = block.items,
    dcm.type = dcm.type,
    control.model = control.model,
    control.method = control.method,
    .call = call
  )
}

#' @describeIn logLik-class MGPCM model: polytomous responses integrated over the
#'   fitted multivariate normal latent distribution.
#' @export
logLik.MGPCM <- function(object, theta.low = NULL, theta.up = NULL, L = NULL,
                         ...) {

  par <- object$par$est
  response <- object$arguments$response
  length.poly <- object$length.poly

  N <- nrow(response)
  I <- ncol(response)

  df <- object$npar

  D <- object$arguments$D
  grid.control <- istem_latent_grid_control(
    object$arguments$control.model, D, L = L,
    theta.low = theta.low, theta.up = theta.up
  )
  L <- grid.control$L
  theta.low <- grid.control$theta.lower
  theta.up <- grid.control$theta.upper

  theta.mu <- array(
    if (is.null(object$arguments$control.model$theta.mu)) rep(0, D)
    else object$arguments$control.model$theta.mu,
    dim = D
  )

  grid <- make_quadrature_grid_mvn(
    object$Corr$est, theta.mu, D, L, theta.low, theta.up
  )

  prob <- model.MGPCM(grid$theta.norm, par)
  response.group <- istem_response_groups(response)
  lik <- cpp_loglik_indexed(
    prob = prob,
    response = matrix(as.integer(response.group$response),
                      nrow = response.group$G, ncol = I),
    pi = as.vector(grid$pi),
    block_sizes = as.integer(length.poly),
    response_base = 0L
  )
  lik <- loglik_apply_group_count(lik, response.group$count)
  results <- c(`log Likelihood` = lik$logLik)
  results <- set_loglik_attributes(results, N, df, lik, grid$theta.norm,
                                    prob, grid$pi, grid$L, theta.low, theta.up,
                                    response.unique = response.group$response,
                                    response.count = response.group$count,
                                    response.group = response.group$group)
  return(results)
}

# ===========================================================================
# rotate.MGPCM
# ===========================================================================

#' @describeIn rotate Applies a factor rotation to the posterior mean
#'   discrimination parameters of \code{MGPCM} objects. The latent trait
#'   estimates and factor correlation matrix are transformed consistently.
#'   Category intercepts and posterior uncertainty summaries are retained
#'   from the original fit without recomputation.
#' @method rotate MGPCM
#' @export
rotate.MGPCM <- function(object,
                         method,
                         vis          = TRUE,
                         target.a     = NULL,
                         target.theta = NULL,
                         pst.W        = NULL,
                         lp.p         = 1,
                         lp.gpaiter   = 5,
                         promax_m     = 4,
                         ...) {

  call <- match.call()

  D <- ncol(object$Q.matrix)
  if (is.null(D) || D < 1L) {
    stop("Cannot determine the latent dimension from object$Q.matrix.",
         call. = FALSE)
  }

  par_dimnames   <- dimnames(object$par$est)
  theta_dimnames <- dimnames(object$theta$est)
  Corr_dimnames  <- dimnames(object$Corr$est)

  mirt_object <- object
  mirt_object$par$est <- cbind(
    object$par$est[, seq_len(D), drop = FALSE],
    matrix(0, nrow(object$par$est), 3L)
  )
  mirt_object$arguments$model <- "m2pl"
  class(mirt_object) <- "MIRT"

  rotated <- rotate.MIRT(
    object       = mirt_object,
    method       = method,
    vis          = vis,
    target.a     = target.a,
    target.theta = target.theta,
    pst.W        = pst.W,
    lp.p         = lp.p,
    lp.gpaiter   = lp.gpaiter,
    promax_m     = promax_m,
    ...
  )

  out <- object
  out$par$est[, seq_len(D)] <- rotated$object$par$est[, seq_len(D), drop = FALSE]
  out$theta$est <- rotated$object$theta$est
  out$Corr$est  <- rotated$object$Corr$est

  dimnames(out$par$est)   <- par_dimnames
  dimnames(out$theta$est) <- theta_dimnames
  dimnames(out$Corr$est)  <- Corr_dimnames
  out$rotated             <- rotated$object$rotated
  out$rotation_method     <- method

  results <- list(
    object = out,
    call = call,
    arguments = list(
      object=object,
      method=method,
      vis=vis,
      target.a=target.a,
      target.theta=target.theta,
      pst.W=pst.W,
      lp.p=lp.p,
      lp.gpaiter=lp.gpaiter,
      promax_m=promax_m,
      ...
    )
  )

  class(results) <- "RotateMGPCM"
  return(results)
}

################################# FCMIRT iStEM #################################

fit.FCMIRT.iStEM <- function(data, model = "m2pl", Q.matrix = NULL,
                             block.items = NULL, D = NULL, fc.type = "RANK",
                             control.model = NULL, control.method = NULL,
                             .call = NULL) {

  call <- if (is.null(.call)) match.call() else .call
  control.model <- fc_as_control_list(control.model, "control.model")
  control.method <- fc_as_control_list(control.method, "control.method")
  common.method <- fit_common_method_control(control.method)
  vis <- common.method$vis
  method <- istem_method_control(control.method)
  set.seed(common.method$seed)

  model <- tolower(model)
  model_type <- model_type_to_int(model)

  fc <- istem_prepare_fc_data(data, block.items, fc.type)
  I <- fc$I
  N <- fc$N
  response.group <- istem_response_groups(fc$response)
  response.fit <- response.group$response
  count.fit <- response.group$count
  if (!is.null(Q.matrix)) {
    Q.matrix <- as.matrix(Q.matrix)
    if (anyNA(Q.matrix) || !all(Q.matrix %in% c(0, 1))) {
      stop("'Q.matrix' must be a matrix containing only 0 or 1.",
           call. = FALSE)
    }
    if (nrow(Q.matrix) != I) {
      stop("'Q.matrix' must have ", I, " rows, one for each item.",
           call. = FALSE)
    }
    if (any(rowSums(Q.matrix) < 1L)) {
      stop("Each item must measure at least one trait (each row of 'Q.matrix' needs a 1).",
           call. = FALSE)
    }
    D.q <- ncol(Q.matrix)
    if (!is.null(D) && D != D.q) {
      warning("'D' (", D, ") does not match ncol(Q.matrix) (", D.q,
              "); using ncol(Q.matrix).")
    }
    D <- D.q
  }
  if (is.null(D) || is.na(D)) {
    stop("Cannot determine 'D'. Please provide either 'Q.matrix' or 'D'.",
         call. = FALSE)
  }
  D <- as.integer(D)
  if (D < 1L) {
    stop("'D' must be a positive integer.", call. = FALSE)
  }
  if (is.null(Q.matrix)) {
    Q.matrix <- matrix(1, I, D)
  }
  storage.mode(Q.matrix) <- "numeric"
  method <- istem_apply_grid_control(method, control.model, D)
  istem_check_method_control(method)
  control.model <- istem_effective_grid_model_control(control.model, method)

  prior <- mirt_istem_prior_control(control.model)
  bounds <- list(
    a.lower = get_ctrl("a.lower", 1e-4, control.method),
    a.upper = get_ctrl("a.upper", 6, control.method),
    b.lower = get_ctrl("b.lower", -6, control.method),
    b.upper = get_ctrl("b.upper", 6, control.method),
    c.lower = get_ctrl("c.lower", prior$c.lower, control.method),
    c.upper = get_ctrl("c.upper", prior$c.upper, control.method),
    d.lower = get_ctrl("d.lower", prior$d.lower, control.method),
    d.upper = get_ctrl("d.upper", prior$d.upper, control.method)
  )
  mirt_istem_check_bounds(bounds, model_type)

  theta_mu <- array(get_ctrl("theta.mu", rep(0, D), control.model), dim = D)
  include_corr <- D > 1L && !method$fix.corr
  par <- fcmirt_istem_initial_par(model_type, Q.matrix, bounds, prior,
                                  fc$block.items)

  state <- list(
    response = response.fit,
    response.count = count.fit,
    par = par,
    theta = istem_random_theta(response.group$G, D, theta_mu, method$theta.lower,
                               method$theta.upper),
    Corr = istem_random_corr(D, include_corr),
    theta_mu = theta_mu,
    model.type = model_type,
    Q.matrix = Q.matrix,
    block.items = fc$block.items,
    patterns = fc$patterns,
    patterns.total = fc$patterns.total,
    optim.maxit = method$optim.maxit,
    bounds = bounds,
    prior = prior
  )

  run <- istem_run(
    state = state,
    method = method,
    N = N,
    vis = vis,
    label = "FCMIRT",
    update_parameters = fcmirt_istem_update_parameters,
    param_vec = function(state) {
      fcmirt_istem_param_vec(state$par, state$Corr, state$model.type,
                             state$Q.matrix, state$block.items, include_corr)
    },
    sample_theta = fcmirt_istem_sample_theta,
    logLik_fun = function(state) {
      fcmirt_istem_loglik_trace(
        state, method$L, method$theta.lower, method$theta.upper
      )
    }
  )

  final <- fcmirt_istem_param_unpack(
    pv = run$chain.mean,
    pv_se = run$chain.sd,
    pv_rhat = NULL,
    I = I, D = D,
    model_type = model_type,
    Q.matrix = Q.matrix,
    block.items = fc$block.items,
    Corr = run$state$Corr,
    include_corr = include_corr
  )

  theta.est <- istem_expand_group_matrix(
    run$theta.est, response.group$group, rownames(fc$response))
  theta.se <- istem_expand_group_matrix(
    run$theta.se, response.group$group, rownames(fc$response))
  theta_est <- istem_named_theta(theta.est, theta.se, fc$response, D)
  par <- final$par
  par.se <- final$par.se
  par.Rhat <- final$par.rhat
  par.free <- fcmirt_par_free_mask(model_type, Q.matrix, fc$block.items)
  colnames(par) <- colnames(par.se) <- colnames(par.Rhat) <- colnames(par.free) <-
    c(paste0("a", seq_len(D)), "b", "c", "d")
  rownames(par) <- rownames(par.se) <- rownames(par.Rhat) <- rownames(par.free) <-
    paste0("item", fc$all.items)
  par.se[!par.free] <- NA_real_
  par.Rhat[!par.free] <- NA_real_

  corr_df <- free_corr_npar(D, include_corr)
  npar <- sum(par.free) + corr_df

  Corr.Rhat <- final$Corr.rhat
  dimnames(Corr.Rhat) <- list(paste0("Dim.", seq_len(D)), paste0("Dim.", seq_len(D)))

  results <- list(
    npar = npar,
    method = "iStEM",
    theta = theta_est,
    par = list(est = par, se = par.se, Rhat = par.Rhat, free = par.free),
    Corr = list(est = final$Corr, se = final$Corr.se, Rhat = Corr.Rhat),
    stan.obj = NULL,
    MCMC.obj = NULL,
    model = model,
    Q.matrix = Q.matrix,
    block.items = fc$block.items,
    fc.type = fc$fc.type,
    response = fc$response,
    patterns = fc$patterns,
    patterns.total = fc$patterns.total,
    call = call,
    arguments = list(
      data = data,
      model = model,
      Q.matrix = Q.matrix,
      block.items = fc$block.items,
      D = D,
      fc.type = fc$fc.type,
      method = "iStEM",
      cores = common.method$cores,
      vis = vis,
      seed = common.method$seed,
      control.model = control.model,
      control.method = fit_effective_method_control(
        control.method, method = c(method, bounds), common = common.method
      )
    ),
    iStEM = utils::modifyList(run$iStEM, list(prior = prior))
  )

  class(results) <- "FCMIRT"
  results$logLik <- logLik.FCMIRT(results, L = method$L,
                                    theta.low = method$theta.lower,
                                    theta.up = method$theta.upper)
  results
}

fcmirt_par_free_mask <- function(model_type, Q.matrix, block.items) {
  I <- nrow(Q.matrix)
  D <- ncol(Q.matrix)
  free <- matrix(FALSE, nrow = I, ncol = D + 3L)
  if (model_type >= 2L) {
    free[, seq_len(D)] <- Q.matrix == 1
  }
  for (items in block.items) {
    K <- length(items)
    free[items[-K], D + 1L] <- TRUE
  }
  if (model_type >= 3L) {
    free[, D + 2L] <- TRUE
  }
  if (model_type == 4L) {
    free[, D + 3L] <- TRUE
  }
  colnames(free) <- c(paste0("a", seq_len(D)), "b", "c", "d")
  free
}

fcmirt_istem_initial_par <- function(model_type, Q.matrix, bounds, prior,
                                     block.items) {
  I <- nrow(Q.matrix)
  D <- ncol(Q.matrix)
  par <- matrix(NA_real_, I, D + 3L)
  if (model_type == 1L) {
    par[, seq_len(D)] <- Q.matrix
  } else {
    a <- matrix(0, I, D)
    a[Q.matrix == 1] <- istem_rlnorm_bounded(
      sum(Q.matrix == 1), prior$a.mu, prior$a.sigma,
      bounds$a.lower, bounds$a.upper
    )
    par[, seq_len(D)] <- a
  }
  par[, D + 1L] <- NA_real_
  for (items in block.items) {
    b <- istem_rnorm_bounded(length(items), prior$b.mu, prior$b.sigma,
                             bounds$b.lower, bounds$b.upper)
    b <- istem_clip(b - mean(b), bounds$b.lower, bounds$b.upper)
    b <- b - mean(b)
    par[items, D + 1L] <- b
  }
  par[, D + 2L] <- if (model_type >= 3L) {
    stats::runif(I, max(bounds$c.lower, prior$c.lower),
                 min(bounds$c.upper, prior$c.upper))
  } else {
    0
  }
  par[, D + 3L] <- if (model_type == 4L) {
    stats::runif(I, max(bounds$d.lower, prior$d.lower),
                 min(bounds$d.upper, prior$d.upper))
  } else {
    1
  }
  par
}

fcmirt_istem_sample_theta <- function(state, theta_grid_length,
                                      theta_lower, theta_upper) {
  chol <- istem_corr_chol(state$Corr)
  cpp_gibbs_fcmirt_theta(
    theta_ = state$theta,
    par_ = state$par,
    response_ = state$response,
    theta_mu_ = state$theta_mu,
    chol_corr_ = chol$chol,
    log_diag_sum = chol$log_diag_sum,
    patterns_total = state$patterns.total,
    patterns = state$patterns,
    step = theta_grid_length,
    lower = theta_lower,
    upper = theta_upper
  )
}

fcmirt_istem_update_parameters <- function(state) {
  for (b in seq_along(state$block.items)) {
    items <- state$block.items[[b]]
    state$par[items, ] <- fcmirt_istem_fit_block(
      y = state$response[, b],
      theta = state$theta,
      par_block = state$par[items, , drop = FALSE],
      q_block = state$Q.matrix[items, , drop = FALSE],
      patterns_total = cpp_istem_local_patterns(
        as.matrix(state$patterns.total[[b]]), as.integer(items)
      ),
      patterns = cpp_istem_local_patterns(
        as.matrix(state$patterns[[b]]), as.integer(items)
      ),
      model_type = state$model.type,
      weight = istem_state_weight(state),
      optim_maxit = state$optim.maxit,
      bounds = state$bounds,
      prior = state$prior
    )
  }
  state
}

fcmirt_istem_fit_block <- function(y, theta, par_block, q_block,
                                   patterns_total, patterns, model_type,
                                   weight, optim_maxit, bounds, prior) {
  D <- ncol(q_block)
  active <- q_block == 1
  x0 <- numeric(0)
  lower <- numeric(0)
  upper <- numeric(0)
  if (model_type >= 2L) {
    x0 <- c(x0, par_block[, seq_len(D), drop = FALSE][active])
    lower <- c(lower, rep(bounds$a.lower, sum(active)))
    upper <- c(upper, rep(bounds$a.upper, sum(active)))
  }
  x0 <- c(x0, par_block[-nrow(par_block), D + 1L])
  lower <- c(lower, rep(bounds$b.lower, nrow(par_block) - 1L))
  upper <- c(upper, rep(bounds$b.upper, nrow(par_block) - 1L))
  if (model_type >= 3L) {
    x0 <- c(x0, par_block[, D + 2L])
    lower <- c(lower, rep(bounds$c.lower, nrow(par_block)))
    upper <- c(upper, rep(bounds$c.upper, nrow(par_block)))
  }
  if (model_type == 4L) {
    x0 <- c(x0, par_block[, D + 3L])
    lower <- c(lower, rep(bounds$d.lower, nrow(par_block)))
    upper <- c(upper, rep(bounds$d.upper, nrow(par_block)))
  }

  obj <- function(x) {
    block <- fcmirt_istem_x_to_block(x, par_block, q_block, model_type)
    K <- nrow(block)
    if (K > 1L && any(block[-K, D + 1L] < bounds$b.lower |
                       block[-K, D + 1L] > bounds$b.upper)) {
      return(Inf)
    }
    b_last <- block[K, D + 1L]
    penalty <- 0
    if (b_last < bounds$b.lower || b_last > bounds$b.upper) {
      excess <- max(bounds$b.lower - b_last, b_last - bounds$b.upper, 0)
      penalty <- 10 * excess^2
    }
    ll <- cpp_fcmirt_block_loglik_weighted(
      theta = theta,
      par_block = block,
      response = as.integer(y),
      patterns_total = patterns_total,
      patterns = patterns,
      weight = weight
    )
    lp <- fcmirt_istem_block_log_prior(block, q_block, model_type, prior)
    if (!is.finite(lp)) return(Inf)
    -ll - lp + penalty
  }

  opt <- tryCatch(
    stats::optim(
      par = x0,
      fn = obj,
      method = "L-BFGS-B",
      lower = lower,
      upper = upper,
      control = list(maxit = optim_maxit)
    ),
    error = function(e) NULL
  )
  if (is.null(opt) || !is.finite(opt$value)) {
    return(par_block)
  }
  fcmirt_istem_x_to_block(opt$par, par_block, q_block, model_type)
}

fcmirt_istem_x_to_block <- function(x, par_block, q_block, model_type) {
  D <- ncol(q_block)
  I <- nrow(q_block)
  idx <- 1L
  out <- par_block
  if (model_type == 1L) {
    out[, seq_len(D)] <- q_block
  } else {
    a <- matrix(0, I, D)
    n_active <- sum(q_block == 1)
    if (n_active > 0L) {
      a[q_block == 1] <- x[idx:(idx + n_active - 1L)]
      idx <- idx + n_active
    }
    out[, seq_len(D)] <- a
  }
  b_free <- x[idx:(idx + I - 2L)]
  out[-I, D + 1L] <- b_free
  out[I, D + 1L] <- -sum(b_free)
  idx <- idx + I - 1L
  out[, D + 2L] <- if (model_type >= 3L) {
    val <- x[idx:(idx + I - 1L)]
    idx <- idx + I
    val
  } else {
    0
  }
  out[, D + 3L] <- if (model_type == 4L) {
    x[idx:(idx + I - 1L)]
  } else {
    1
  }
  out
}

fcmirt_istem_block_log_prior <- function(par_block, q_block, model_type, prior) {
  if (!isTRUE(prior$use.prior)) return(0)
  D <- ncol(q_block)
  lp <- sum(stats::dnorm(
    par_block[, D + 1L], mean = prior$b.mu, sd = prior$b.sigma, log = TRUE
  ))
  if (model_type >= 2L) {
    a_active <- par_block[, seq_len(D), drop = FALSE][q_block == 1]
    if (any(a_active <= 0)) return(-Inf)
    lp <- lp + sum(stats::dlnorm(
      a_active, meanlog = prior$a.mu, sdlog = prior$a.sigma, log = TRUE
    ))
  }
  if (model_type >= 3L &&
      any(par_block[, D + 2L] < prior$c.lower |
          par_block[, D + 2L] > prior$c.upper)) {
    return(-Inf)
  }
  if (model_type == 4L &&
      any(par_block[, D + 3L] < prior$d.lower |
          par_block[, D + 3L] > prior$d.upper)) {
    return(-Inf)
  }
  lp
}

fcmirt_istem_param_vec <- function(par, Corr, model_type, Q.matrix,
                                   block.items, include_corr) {
  D <- ncol(Q.matrix)
  b_free <- unlist(lapply(block.items, function(items) {
    par[items[-length(items)], D + 1L]
  }), use.names = FALSE)
  pv <- if (model_type == 1L) {
    b_free
  } else {
    c(par[, seq_len(D), drop = FALSE][Q.matrix == 1], b_free)
  }
  if (model_type >= 3L) {
    pv <- c(pv, par[, D + 2L])
  }
  if (model_type == 4L) {
    pv <- c(pv, par[, D + 3L])
  }
  c(pv, istem_param_corr_vec(Corr, include_corr))
}

fcmirt_istem_param_unpack <- function(pv, pv_se, pv_rhat = NULL, I, D, model_type,
                                      Q.matrix, block.items, Corr, include_corr) {
  idx <- 1L
  par <- matrix(NA_real_, I, D + 3L)
  par.se <- matrix(NA_real_, I, D + 3L)
  par.rhat <- matrix(NA_real_, I, D + 3L)
  has_rhat <- !is.null(pv_rhat)
  if (model_type == 1L) {
    par[, seq_len(D)] <- Q.matrix
    par.se[, seq_len(D)] <- NA_real_
  } else {
    a <- matrix(0, I, D)
    a.se <- matrix(NA_real_, I, D)
    a.rhat <- matrix(NA_real_, I, D)
    n_active <- sum(Q.matrix == 1)
    if (n_active > 0L) {
      a[Q.matrix == 1] <- pv[idx:(idx + n_active - 1L)]
      a.se[Q.matrix == 1] <- pv_se[idx:(idx + n_active - 1L)]
      if (has_rhat) a.rhat[Q.matrix == 1] <- pv_rhat[idx:(idx + n_active - 1L)]
      idx <- idx + n_active
    }
    par[, seq_len(D)] <- a
    par.se[, seq_len(D)] <- a.se
    par.rhat[, seq_len(D)] <- a.rhat
  }
  par[, D + 1L] <- 0
  par.se[, D + 1L] <- NA_real_
  for (items in block.items) {
    K <- length(items)
    b_free <- pv[idx:(idx + K - 2L)]
    b_free_se <- pv_se[idx:(idx + K - 2L)]
    par[items[-K], D + 1L] <- b_free
    par[items[K], D + 1L] <- -sum(b_free)
    par.se[items[-K], D + 1L] <- b_free_se
    if (has_rhat) {
      b_free_rhat <- pv_rhat[idx:(idx + K - 2L)]
      par.rhat[items[-K], D + 1L] <- b_free_rhat
    }
    idx <- idx + K - 1L
  }
  if (model_type >= 3L) {
    par[, D + 2L] <- pv[idx:(idx + I - 1L)]
    par.se[, D + 2L] <- pv_se[idx:(idx + I - 1L)]
    if (has_rhat) par.rhat[, D + 2L] <- pv_rhat[idx:(idx + I - 1L)]
    idx <- idx + I
  } else {
    par[, D + 2L] <- 0
    par.se[, D + 2L] <- NA_real_
  }
  if (model_type == 4L) {
    par[, D + 3L] <- pv[idx:(idx + I - 1L)]
    par.se[, D + 3L] <- pv_se[idx:(idx + I - 1L)]
    if (has_rhat) par.rhat[, D + 3L] <- pv_rhat[idx:(idx + I - 1L)]
    idx <- idx + I
  } else {
    par[, D + 3L] <- 1
    par.se[, D + 3L] <- NA_real_
  }
  corr <- istem_unpack_corr(pv, pv_se, idx, D, Corr, include_corr)
  corr.rhat <- if (has_rhat) {
    istem_unpack_corr_rhat(pv_rhat, idx, D, include_corr)
  } else {
    matrix(NA_real_, D, D)
  }
  list(par = par, par.se = par.se, par.rhat = par.rhat,
       Corr = corr$Corr, Corr.se = corr$Corr.se, Corr.rhat = corr.rhat)
}

# library(ForceChoice)
#
# ############################### data ###############################
# N.person <- 500L
# N.block <- 20
# I.block <- 2L
# D <- 2L
# model <- "GDINA"
# fc.type <- "RANK"
# control <- list(single=FALSE)
#
# seed <- 3546
# set.seed(seed)
#
# data.obj <- sim.data.FCGDINA(
#   N.person = N.person, N.block = N.block, I.block = I.block,
#   D = D, model = model, fc.type = fc.type, control = control
# )
#
# response <- data.obj$response
# data <- data.obj$data
# block.items <- data.obj$block.items
# Q.matrix <- data.obj$Q.matrix
# fc.type <- data.obj$fc.type
#
# # ---- Validate data conversion round-trip ----
# data.from.response <- get.data.from.response(
#   response, block.items, fc.type = fc.type
# )
# response.from.data <- get.response.from.data(
#   data, block.items, fc.type = fc.type
# )
# stopifnot(all(as.matrix(data) == as.matrix(data.from.response)))
# stopifnot(all(response == response.from.data))
#
# # ---- Verify unified block-item extraction ----
# block.items.from.data <- get.block.items.from.data(data)
# stopifnot(length(block.items.from.data) == N.block)
# stopifnot(all(mapply(setequal, block.items, block.items.from.data)))
#
# # ---- Verify pi estimation ----
# cat(sprintf("  True alpha patterns: %d\n", nrow(data.obj$alpha.patterns)))
# cat(sprintf("  Structural parameters (pi): %d free\n", 2^D - 1L))
#
# ############################## Stan MCMC ###############################
# stan.control.method <- list(
#   init   = "random",
#   algorithm = "HMC"
# )
#
# object.FCGDINA.stan <- fit.FCGDINA(data, Q.matrix = Q.matrix,
#                                     model = model,
#                                     block.items = block.items,
#                                     fc.type = fc.type,
#                                     method = "stan",
#                                     control.method = c(stan.control.method, list(seed = seed, vis = TRUE)))
# cbind(do.call(c, data.obj$delta), do.call(c, object.FCGDINA.stan$delta$est))
# object.FCGDINA.stan$delta$Rhat
# object.FCGDINA.stan$pi
# get.fit.index(object.FCGDINA.stan)
# mean(abs((object.FCGDINA.stan$alpha$est > 0.5) * 1L - data.obj$alpha))
#
# # ############################## S3 ###############################
# class(object.FCGDINA.stan)
# methods(class = class(object.FCGDINA.stan))
#
# coef(object.FCGDINA.stan, "par")
# confint(object.FCGDINA.stan)
# deviance(object.FCGDINA.stan)
# extract(object.FCGDINA.stan, "delta")
# fitted(object.FCGDINA.stan)
# get.fit.index(object.FCGDINA.stan)
# logLik(object.FCGDINA.stan)
# nobs(object.FCGDINA.stan)
# plot(object.FCGDINA.stan)
# predict(object.FCGDINA.stan)
# residuals(object.FCGDINA.stan)
# temp <- summary(object.FCGDINA.stan)
# update(object.FCGDINA.stan, control.model=list(L=31))
# vcov(object.FCGDINA.stan)
#
# ############################### EM ###############################
# EM.control.method <- list(
#   maxitr = 2000L,
#   minitr = 2L,
#   tol = 1e-4,
#   par.tol = 1e-4,
#   optim.maxit = 200L,
#   estimate.se = FALSE
# )
#
# object.FCGDINA.EM <- fit.FCGDINA(
#   data, Q.matrix = Q.matrix,
#   model = model,
#   block.items = block.items,
#   fc.type = fc.type,
#   control.method = c(EM.control.method, list(seed = seed, vis = TRUE))
# )
#
# cbind(do.call(c, data.obj$delta), do.call(c, object.FCGDINA.EM$delta$est))
# object.FCGDINA.EM$pi
# get.fit.index(object.FCGDINA.EM)
# mean(abs((object.FCGDINA.EM$alpha$est > 0.5) * 1L - data.obj$alpha))
#
# # ############################## S3 ###############################
# class(object.FCGDINA.EM)
# methods(class = class(object.FCGDINA.EM))
#
# coef(object.FCGDINA.EM, "par")
# confint(object.FCGDINA.EM)
# deviance(object.FCGDINA.EM)
# extract(object.FCGDINA.EM, "delta")
# fitted(object.FCGDINA.EM)
# get.fit.index(object.FCGDINA.EM)
# logLik(object.FCGDINA.EM)
# nobs(object.FCGDINA.EM)
# plot(object.FCGDINA.EM)
# predict(object.FCGDINA.EM)
# residuals(object.FCGDINA.EM)
# temp <- summary(object.FCGDINA.EM)
# # update(object.FCGDINA.EM, control.model=list(L=31))
# vcov(object.FCGDINA.EM)
#
# ############################### iStEM ###############################
# iStEM.control.method <- list(
#   burnin.maxitr = 40,
#   maxitr = 500
# )
#
# object.FCGDINA.iStEM <- fit.FCGDINA(
#   data, Q.matrix = Q.matrix,
#   model = model,
#   block.items = block.items,
#   fc.type = fc.type,
#   method = "iStEM",
#   control.method = c(iStEM.control.method, list(seed = seed, vis = TRUE))
# )
#
# cbind(do.call(c, data.obj$delta), do.call(c, object.FCGDINA.iStEM$delta$est))
# object.FCGDINA.iStEM$pi
# get.fit.index(object.FCGDINA.iStEM)
# mean(abs((object.FCGDINA.iStEM$alpha$est > 0.5) * 1L - data.obj$alpha))
#
# # ############################## S3 ###############################
# class(object.FCGDINA.iStEM)
# methods(class = class(object.FCGDINA.iStEM))
#
# coef(object.FCGDINA.iStEM, "par")
# confint(object.FCGDINA.iStEM)
# deviance(object.FCGDINA.iStEM)
# extract(object.FCGDINA.iStEM, "delta")
# fitted(object.FCGDINA.iStEM)
# get.fit.index(object.FCGDINA.iStEM)
# logLik(object.FCGDINA.iStEM)
# nobs(object.FCGDINA.iStEM)
# plot(object.FCGDINA.iStEM)
# predict(object.FCGDINA.iStEM)
# residuals(object.FCGDINA.iStEM)
# temp <- summary(object.FCGDINA.iStEM)
# # update(object.FCGDINA.iStEM, control.model=list(L=31))
# vcov(object.FCGDINA.iStEM)

#' script_path <- rstudioapi::getActiveDocumentContext()$path
#' print(script_path)
#' working_directory <- dirname(dirname(script_path))
#' setwd(working_directory)
#'
#' pkgdown::build_site()
#'
#' pkgload::load_all()
#' rstantools::use_rstan()
#' devtools::document()
#'
#' devtools::build_manual()
#'
#' devtools::build()
#'
#' detach("package:ForceChoice", unload = TRUE, character.only = TRUE)
#' .rs.restartR()
#' devtools::install()
#' devtools::check(manual = TRUE)
#'
#' a <- devtools::spell_check(vignettes = TRUE, use_wordlist = TRUE)
#' a
#' a[1]
#'
#' devtools::check_rhub()
#'
#' devtools::check_win_devel()
#'
#' devtools::release()
#' devtools::submit_cran()
#'
#'
#' NEWS.md
#' DESCRIPTION
#' cran-comments.md
#'
#'
#' pack <- available.packages()
#'
#' which(rownames(pack) == "Qval")
#'
#'
# sum <- 0
# library(cranlogs)
# downloads <- cran_downloads(packages = "LCAP", from = "2026-01-22", to = "last-day")
# sum <- sum + sum(downloads$count)
# downloads <- cran_downloads(packages = "Qval", from = "2014-06-30", to = "last-day")
# sum <- sum + sum(downloads$count)
# downloads <- cran_downloads(packages = "EFAfactors", from = "2024-02-04", to = "last-day")
# sum <- sum + sum(downloads$count)
# downloads <- cran_downloads(packages = "LSTMfactors", from = "2024-09-25", to = "last-day")
# sum <- sum + sum(downloads$count)
# sum
#'
#'

#' @title S3 Methods: rotate
#'
#' @description
#' Applies factor rotation to the posterior mean parameter estimates of
#' fitted exploratory multidimensional models from the \pkg{ForceChoice}
#' package. Rotation reparameterises the latent space to improve
#' interpretability of factor loadings while preserving the model-implied
#' covariance structure. Standard errors and other posterior uncertainty
#' summaries are carried over from the original fit object without
#' recomputation. No per-iteration rotation or cross-iteration alignment is
#' performed.
#'
#' For GPArotation methods, theta is re-standardized to a standard normal
#' scale after rotation. For Promax rotation, no post-rotation
#' standardization is applied. Loadings and the factor correlation matrix
#' are updated accordingly to preserve model-implied covariances.
#'
#' Rotation is not permitted when the fitted \code{Q.matrix} contains zeros
#' because that confirmatory structure would not generally be preserved
#' under rotation.
#'
#' @param object A fitted model object of class \code{"MIRT"} (returned by
#'   \code{\link[ForceChoice]{fit.MIRT}}) or \code{"MGPCM"} (returned by
#'   \code{\link[ForceChoice]{fit.MGPCM}}).
#' @param method Character string specifying the rotation method. Supported
#'   methods include \code{"promax"}, \code{"Varimax"}, \code{"quartimax"},
#'   \code{"oblimin"}, \code{"quartimin"}, \code{"geominT"}, \code{"geominQ"},
#'   \code{"targetT"}, \code{"targetQ"}, \code{"pstT"}, \code{"pstQ"}, and
#'   others provided by the \pkg{GPArotation} package.
#' @param vis Logical; if \code{TRUE} (default), displays informational
#'   messages during rotation.
#' @param target.a Numeric matrix of target loadings for target rotations
#'   (\code{"targetT"}, \code{"targetQ"}, \code{"pstT"}, \code{"pstQ"}).
#' @param target.theta Numeric matrix of target latent traits for target
#'   rotations (\code{"targetT"}, \code{"targetQ"} only).
#' @param pst.W Numeric weight matrix for PST rotations
#'   (\code{"pstT"}, \code{"pstQ"}).
#' @param lp.p Numeric; p parameter for Lp rotation (default = 1).
#' @param lp.gpaiter Integer; max GPA iterations for Lp rotation
#'   (default = 5).
#' @param promax_m Numeric; power parameter for Promax rotation
#'   (default = 4).
#' @param ... Additional arguments passed to the \pkg{GPArotation} rotation
#'   function.
#'
#' @return
#' \describe{
#'   \item{\code{rotate.MIRT}}{An object of class \code{"RotateMIRT"}
#'     containing the rotated \code{"MIRT"} fit object (\code{object}),
#'     the matched call (\code{call}), and the rotation arguments
#'     (\code{arguments}). Standard errors in the fitted object are
#'     preserved without recomputation.}
#'   \item{\code{rotate.MGPCM}}{An object of class \code{"RotateMGPCM"}
#'     containing the rotated \code{"MGPCM"} fit object (\code{object}),
#'     the matched call (\code{call}), and the rotation arguments
#'     (\code{arguments}). Category intercepts are retained from the
#'     original fit.}
#' }
#'
#' @details
#' Only posterior means are rotated. For \code{MGPCM} objects, the first
#' \eqn{D} columns of \code{object$par$est} are treated as discrimination
#' parameters; all category intercepts are unchanged.
#'
#' @seealso \code{\link[ForceChoice]{fit.MIRT}},
#'   \code{\link[ForceChoice]{fit.MGPCM}}
#' @name rotate
NULL

#' @export
rotate <- function(object, ...) {
  UseMethod("rotate")
}

# library(ForceChoice)
#
# ############################### data ###############################
# N.person <- 500
# N.block <- 10
# I.block <- 2
# D <- 2
# model <- "m1pl"
# fc.type <- "MOLE"
# control <- NULL
#
# seed <- 362154
#
# set.seed(seed)
# data.obj <- sim.data.FCMIRT(N.person = N.person, N.block = N.block,
#                             I.block = I.block, D = D, model = model,
#                             fc.type = fc.type, control = control)
# data.obj$par
# response <- data.obj$response
# block.items <- data.obj$block.items
# Q.matrix <- data.obj$Q.matrix
# fc.type <- data.obj$fc.type
# block.b.sum <- vapply(block.items, function(items) {
#   sum(data.obj$par[items, D + 1L])
# }, numeric(1L))
# stopifnot(max(abs(block.b.sum)) < 1e-10)
# data <- get.data.from.response(response, block.items, fc.type = fc.type)
# head(data)
# head(response)
# all(data == data.obj$data)
# response.obj <- get.response.from.data(data, block.items, fc.type = fc.type)
# stopifnot(all(response == response.obj))
#
# # ---- Verify unified block-item extraction ----
# block.items.from.data <- get.block.items.from.data(data)
# stopifnot(length(block.items.from.data) == N.block)
# stopifnot(all(mapply(setequal, block.items, block.items.from.data)))
#
# # ############################### Stan MCMC ###############################
# # object.FCMIRT.stan <- fit.FCMIRT(data, model = model, Q.matrix = Q.matrix,
# #                                  block.items = block.items, D = D,
# #                                  fc.type = fc.type,
# #                                  method = "stan",
# #                                  control.method = c(list(
# #                                    init = "random",
# #                                    algorithm = "HMC"), list(seed = seed, vis = TRUE)))
# # cbind(data.obj$par, object.FCMIRT.stan$par$est)
# # object.FCMIRT.stan$par$Rhat
# # get.fit.index(object.FCMIRT.stan)
#
# ############################### iStEM ###############################
# iStEM.control.method <- list(
#   burnin.maxitr = 40,
#   maxitr = 500,
#   theta.proposal.sd = 0.10,
#   optim.maxit = 50
# )
#
# object.FCMIRT.iStEM <- fit.FCMIRT(data, model = model, Q.matrix = Q.matrix,
#                                   block.items = block.items, D = D,
#                                   fc.type = fc.type,
#                                   method = "iStEM",
#                                   control.method = c(iStEM.control.method, list(seed = seed, vis = TRUE)))
# cbind(data.obj$par, object.FCMIRT.iStEM$par$est)
#
# # ---- Validate logLik with new quadrature helpers ----
# ll <- logLik(object.FCMIRT.iStEM)
# stopifnot(inherits(ll, "logLik"))
# stopifnot(is.finite(as.numeric(ll)))
# stopifnot(!is.null(attr(ll, "theta.norm")))
# cat(sprintf("  FCMIRT logLik: %.2f (df=%d)\n", as.numeric(ll), attr(ll, "df")))
#
# get.fit.index(object.FCMIRT.iStEM)

#' @title S3 Methods: summary
#'
#' @description
#' Generates structured, comprehensive summaries of objects produced by the
#' \pkg{ForceChoice} package. This generic function dispatches to
#' class-specific methods that extract and organize key information including
#' model configurations, fit statistics, parameter estimates, and convergence
#' diagnostics. Designed for programmatic access and downstream reporting.
#'
#' @param object An object of one of the following classes:
#'   \itemize{
#'     \item Fitted model objects: \code{"MIRT"}, \code{"MGPCM"},
#'       \code{"MGGUM"}, \code{"FCMIRT"}, \code{"FCDCM"}, \code{"FCGGUM"},
#'       \code{"FCGDINA"}, \code{"TIRT"}
#'     \item Goodness-of-fit objects: \code{"good.of.fit"}
#'   }
#' @param digits Number of decimal places for numeric output (default: 4).
#'   Applied uniformly across all methods.
#' @param ... Additional arguments passed to or from other methods (currently
#'   ignored).
#'
#' @return Invisibly returns a structured list containing summary components.
#'   The exact structure depends on the class of \code{object}. All returned
#'   objects carry an appropriate S3 class (e.g., \code{"summary.MIRT"},
#'   \code{"summary.good.of.fit"}) for use with corresponding \code{print}
#'   methods.
#'
#' @details
#' Each method returns a named list with class-specific components optimized
#' for structured access and formatted printing:
#'
#' \describe{
#'   \item{\strong{Fitted Model Objects}}{
#'     Returns a \code{summary.<Class>} object with components:
#'     \describe{
#'       \item{\code{call}}{Original function call.}
#'       \item{\code{model.info}}{List: \code{family}, \code{description},
#'         \code{D}, \code{method}, and model-specific fields (\code{model},
#'         \code{fc.type}, \code{dcm.type}, \code{I.states}, \code{N.block}).}
#'       \item{\code{data.info}}{List: \code{N}, \code{I} or \code{B},
#'         \code{response.type}.}
#'       \item{\code{fit.stats}}{List: \code{LogLik}, \code{npar},
#'         \code{AIC}, \code{BIC}.}
#'       \item{\code{par.summary}}{Matrix: parameter summary (Min, Q1, Median,
#'         Mean, Q3, Max) across items/statements/blocks for each parameter type.}
#'       \item{\code{theta.summary}}{Matrix: person parameter summary per
#'         dimension.}
#'       \item{\code{Corr}}{Matrix: factor inter-trait correlation matrix
#'         (where applicable).}
#'       \item{\code{convergence}}{List: algorithm, batch counts, convergence
#'         flags, latent grid length \code{L} (iStEM) or diagnostic note (Stan).}
#'       \item{\code{digits}}{Numeric: precision used for formatting.}
#'     }
#'     Additional model-specific components:
#'     \describe{
#'       \item{\code{delta.summary}}{FCDCM only: higher-order delta parameters.}
#'       \item{\code{classification}}{FCDCM only: observed attribute patterns,
#'         mean maximum posterior.}
#'       \item{\code{lambda.summary}}{TIRT only: factor loading summary.}
#'     }
#'   }
#'
#'   \item{\strong{Goodness-of-Fit Objects (\code{good.of.fit})}}{
#'     Returns a \code{summary.good.of.fit} object with components:
#'     \describe{
#'       \item{\code{Sample_Size}, \code{Items}, \code{Response_Type},
#'         \code{Parameters}, \code{Moments}, \code{LogLik},
#'         \code{Deviance}}{Overview statistics.}
#'       \item{\code{IC}}{Named vector: AIC, AICc, BIC, CAIC, SABIC, HQIC.}
#'       \item{\code{M2}}{List: M2 statistic, df, p-value, RMSEA with CI.}
#'       \item{\code{Residual_Fit}}{Named vector: SRMSR, RMSR, max
#'         standardized residual.}
#'       \item{\code{Comparative_Fit}}{Named vector: CFI, TLI, IFI.}
#'       \item{\code{Pseudo_R2}}{Named vector: McFadden, Cox--Snell,
#'         Nagelkerke, Aldrich--Nelson, Veall--Zimmermann.}
#'       \item{\code{Local_Dependence}}{Named vector: Q3 mean, max, adjusted
#'         max, P95.}
#'       \item{\code{Classification}}{Named vector: posterior entropy,
#'         max posterior.}
#'       \item{\code{Null_Model}}{Named vector: null model LogLik, deviance,
#'         M2, df.}
#'       \item{\code{tables}}{List of data frames for formatted printing.}
#'     }
#'   }
#' }
#'
#' @name summary
NULL


# ===========================================================================
# summary.good.of.fit
# ===========================================================================

#' @describeIn summary Summary method for \code{good.of.fit} objects.
#'   Extracts and structures all fit indices: information criteria,
#'   limited-information M2, RMSEA, SRMSR, CFI/TLI/IFI, pseudo-\eqn{R^2},
#'   local dependence (Yen's Q3), posterior classification, and null-model
#'   diagnostics.
#' @method summary good.of.fit
#' @export
#' @rawNamespace export(summary.good.of.fit)
summary.good.of.fit <- function(object, digits = 4, ...) {

  safe_extract <- function(obj, name) {
    if (!is.null(obj[[name]])) obj[[name]] else NA_real_
  }

  res <- list(
    Sample_Size = object$N,
    Items       = safe_extract(object, "nitems"),
    Response_Type = if (!is.null(object$response.type)) object$response.type else NA_character_,
    Parameters  = object$npar,
    Moments     = safe_extract(object, "moments.n"),
    Bivariate_Moments = safe_extract(object, "pairs.n"),
    Observed_Responses = safe_extract(object, "observed.responses.n"),
    LogLik      = object$LogLik,
    LogLik_per_case = safe_extract(object, "logLik.per.case"),
    LogLik_per_response = safe_extract(object, "logLik.per.response"),
    Deviance    = safe_extract(object, "deviance"),

    IC = c(
      AIC   = safe_extract(object, "AIC"),
      AICc  = safe_extract(object, "AICc"),
      BIC   = safe_extract(object, "BIC"),
      CAIC  = safe_extract(object, "CAIC"),
      SABIC = safe_extract(object, "SABIC"),
      HQIC  = safe_extract(object, "HQIC")
    ),

    M2 = list(
      Statistic   = safe_extract(object, "M2.value"),
      df          = safe_extract(object, "df"),
      p_value     = safe_extract(object, "p.value"),
      RMSEA       = safe_extract(object, "RMSEA2"),
      RMSEA_lower = safe_extract(object, "RMSEA.lower"),
      RMSEA_upper = safe_extract(object, "RMSEA.upper"),
      RMSEA_CI_level = safe_extract(object, "RMSEA.CI.level"),
      RMSEA_p_close = safe_extract(object, "RMSEA.p.close")
    ),

    Residual_Fit = c(
      SRMSR = safe_extract(object, "SRMSR"),
      RMSR = safe_extract(object, "RMSR"),
      MaxAbsStdResidual = safe_extract(object, "max.abs.standardized.residual"),
      Q95AbsCorrResidual = safe_extract(object, "q95.abs.correlation.residual")
    ),

    Comparative_Fit = c(
      CFI       = safe_extract(object, "CFI"),
      TLI       = safe_extract(object, "TLI"),
      IFI       = safe_extract(object, "IFI")
    ),

    Pseudo_R2 = c(
      McFadden_R2        = safe_extract(object, "McFadden.R2"),
      McFadden_adj_R2    = safe_extract(object, "McFadden.adj.R2"),
      CoxSnell_R2        = safe_extract(object, "CoxSnell.R2"),
      Nagelkerke_R2      = safe_extract(object, "Nagelkerke.R2"),
      AldrichNelson_R2   = safe_extract(object, "AldrichNelson.R2"),
      VeallZimmermann_R2 = safe_extract(object, "VeallZimmermann.R2")
    ),

    Local_Dependence = c(
      Q3_mean = safe_extract(object, "q3.mean"),
      Q3_max = safe_extract(object, "q3.max"),
      Q3_abs_max = safe_extract(object, "q3.abs.max"),
      Q3_p95 = safe_extract(object, "q3.p95"),
      Adjusted_Q3_max = safe_extract(object, "q3.adjusted.max"),
      Adjusted_Q3_abs_max = safe_extract(object, "q3.adjusted.abs.max"),
      Adjusted_Q3_p95 = safe_extract(object, "q3.adjusted.p95")
    ),

    Classification = c(
      Posterior_entropy = safe_extract(object, "posterior.entropy"),
      Normalized_entropy = safe_extract(object, "posterior.entropy.normalized"),
      Mean_max_posterior = safe_extract(object, "mean.max.posterior"),
      Median_max_posterior = safe_extract(object, "median.max.posterior"),
      Min_max_posterior = safe_extract(object, "min.max.posterior")
    ),

    Null_Model = c(
      LogLik = safe_extract(object, "null.LogLik"),
      Deviance = safe_extract(object, "null.deviance"),
      M2 = safe_extract(object, "null.M2.value"),
      df = safe_extract(object, "null.df")
    )
  )

  res$tables <- .gof_summary_tables(object, digits = digits)

  class(res) <- "summary.good.of.fit"
  attr(res, "digits") <- digits
  return(res)
}


# ===========================================================================
# Internal helpers for model summaries
# ===========================================================================

.fc_summary_theta <- function(theta_est, digits) {
  if (is.null(theta_est)) return(NULL)
  est <- as.matrix(theta_est)
  D <- ncol(est)
  stats <- matrix(NA_real_, nrow = D, ncol = 6)
  rownames(stats) <- paste0("Dim.", seq_len(D))
  colnames(stats) <- c("Min", "Q1", "Median", "Mean", "Q3", "Max")
  for (d in seq_len(D)) {
    x <- est[, d]
    x <- x[is.finite(x)]
    if (length(x)) {
      stats[d, ] <- c(min(x), stats::quantile(x, 0.25), stats::median(x),
                      mean(x), stats::quantile(x, 0.75), max(x))
    }
  }
  round(stats, digits)
}

.fc_summary_par_range <- function(par_est, par_free, digits) {
  if (is.null(par_est)) return(NULL)
  est <- as.matrix(par_est)
  free <- if (is.null(par_free)) matrix(TRUE, nrow(est), ncol(est)) else as.matrix(par_free)
  nc <- ncol(est)
  res <- matrix(NA_real_, nrow = nc, ncol = 6)
  rownames(res) <- colnames(est)
  colnames(res) <- c("Min", "Q1", "Median", "Mean", "Q3", "Max")
  for (j in seq_len(nc)) {
    vals <- est[, j]
    if (!is.null(free) && ncol(free) == nc) {
      vals <- vals[free[, j]]
    }
    vals <- vals[is.finite(vals)]
    if (length(vals) && !all(vals == 0)) {
      res[j, ] <- c(min(vals), stats::quantile(vals, 0.25),
                    stats::median(vals), mean(vals),
                    stats::quantile(vals, 0.75), max(vals))
    }
  }
  keep <- rowSums(is.finite(res)) > 0L
  if (!any(keep)) return(NULL)
  round(res[keep, , drop = FALSE], digits)
}

.fc_summary_corr <- function(Corr_est, digits) {
  if (is.null(Corr_est)) return(NULL)
  corr <- as.matrix(Corr_est)
  round(corr, digits)
}

.fc_summary_convergence <- function(object) {
  if (!is.null(object$iStEM) && is.list(object$iStEM) && length(object$iStEM)) {
    istem <- object$iStEM
    conv <- list(
      algorithm = "iStEM (Improved Stochastic EM)",
      burn_in_batches = istem$burn.in.size,
      total_batches   = istem$total.number.of.batch,
      burnin_converged = istem$burnin.converged,
      final_converged  = if (!is.null(istem$converged)) istem$converged else NA,
      L = if (!is.null(istem$L)) {
        istem$L
      } else {
        NA
      }
    )
    return(conv)
  }
  if (!is.null(object$EM) && is.list(object$EM) && length(object$EM)) {
    em <- object$EM
    boot <- em$bootstrap
    conv <- list(
      algorithm = "EM (posterior-weight deterministic EM)",
      iterations = em$iter,
      final_converged = if (!is.null(em$converged)) em$converged else NA,
      final_logLik = em$final.logLik,
      final_ll_change = em$final.ll.change,
      final_max_delta_change = em$final.max.delta.change,
      bootstrap_R = if (!is.null(boot)) boot$R else 0L,
      bootstrap_successful = if (!is.null(boot)) boot$successful else 0L
    )
    return(conv)
  }
  if (!is.null(object$stan.obj)) {
    conv <- list(
      algorithm = "Stan (Hamiltonian Monte Carlo)",
      note = "See stan.obj for full convergence diagnostics."
    )
    return(conv)
  }
  list(algorithm = object$method, note = "No detailed convergence info available.")
}

.fc_null_default <- function(x, default) {
  if (is.null(x)) default else x
}

.fc_model_name <- function(class_name, object) {
  switch(class_name,
    MIRT   = paste0("Multidimensional IRT (",
                    toupper(.fc_null_default(.fc_null_default(object$arguments$model, object$model), "m2pl")),
                    ")"),
    MGPCM  = "Multidimensional Generalized Partial Credit Model",
    MGGUM  = "Multidimensional Generalized Graded Unfolding Model",
    FCMIRT = "Forced-Choice Multidimensional IRT",
    FCDCM  = "Forced-Choice Diagnostic Classification Model",
    FCGDINA = "Forced-Choice GDINA",
    FCGGUM = "Forced-Choice Generalized Graded Unfolding Model",
    TIRT   = "Thurstonian IRT for Forced-Choice",
    class_name
  )
}


# ===========================================================================
# Model summary methods
# ===========================================================================

# ---- summary.MIRT ----

#' @describeIn summary Summary method for \code{MIRT} objects.
#'   Multidimensional IRT (1PL--4PL): extracts binary-response model
#'   configuration, fit statistics (LogLik, AIC, BIC), factor correlation
#'   matrix, person and item parameter summaries, and convergence
#'   diagnostics.
#' @method summary MIRT
#' @export
summary.MIRT <- function(object, digits = 4, ...) {
  response <- object$arguments$response
  N <- nrow(response)
  I <- ncol(response)
  D <- object$arguments$D
  model <- if (!is.null(object$arguments$model)) object$arguments$model
           else if (!is.null(object$model)) object$model else "m2pl"
  logLik.val <- as.numeric(object$logLik)

  AIC <- -2 * logLik.val + 2 * object$npar
  BIC <- -2 * logLik.val + object$npar * log(N)

  res <- list(
    call       = object$call,
    model.info = list(
      family = "MIRT",
      description = .fc_model_name("MIRT", object),
      model  = model,
      D      = D,
      method = object$method
    ),
    data.info = list(
      N = N,
      I = I,
      response.type = "binary (0/1)"
    ),
    fit.stats = list(
      LogLik = logLik.val,
      npar   = object$npar,
      AIC    = AIC,
      BIC    = BIC
    ),
    par.summary   = .fc_summary_par_range(object$par$est, object$par$free, digits),
    theta.summary = .fc_summary_theta(object$theta$est, digits),
    Corr          = .fc_summary_corr(object$Corr$est, digits),
    convergence   = .fc_summary_convergence(object),
    digits        = digits
  )
  class(res) <- "summary.MIRT"
  invisible(res)
}

# ---- summary.MGPCM ----

#' @describeIn summary Summary method for \code{MGPCM} objects.
#'   Multidimensional Generalized Partial Credit Model: polytomous
#'   responses with category-specific step parameters.
#' @method summary MGPCM
#' @export
summary.MGPCM <- function(object, digits = 4, ...) {
  response <- object$arguments$response
  N <- nrow(response)
  I <- ncol(response)
  D <- object$arguments$D
  logLik.val <- as.numeric(object$logLik)

  AIC <- -2 * logLik.val + 2 * object$npar
  BIC <- -2 * logLik.val + object$npar * log(N)

  cat_range <- if (!is.null(object$length.poly)) {
    paste0(min(object$length.poly), "--", max(object$length.poly))
  } else "unknown"

  res <- list(
    call       = object$call,
    model.info = list(
      family = "MGPCM",
      description = .fc_model_name("MGPCM", object),
      D      = D,
      method = object$method
    ),
    data.info = list(
      N = N,
      I = I,
      response.type = paste0("polytomous (", cat_range, " categories)")
    ),
    fit.stats = list(
      LogLik = logLik.val,
      npar   = object$npar,
      AIC    = AIC,
      BIC    = BIC
    ),
    par.summary   = .fc_summary_par_range(object$par$est, object$par$free, digits),
    theta.summary = .fc_summary_theta(object$theta$est, digits),
    Corr          = .fc_summary_corr(object$Corr$est, digits),
    convergence   = .fc_summary_convergence(object),
    digits        = digits
  )
  class(res) <- "summary.MGPCM"
  invisible(res)
}

# ---- summary.MGGUM ----

#' @describeIn summary Summary method for \code{MGGUM} objects.
#'   Multidimensional Generalized Graded Unfolding Model: ideal-point
#'   polytomous responses with discrimination (a), location (delta), and
#'   threshold (tau) parameters.
#' @method summary MGGUM
#' @export
summary.MGGUM <- function(object, digits = 4, ...) {
  response <- object$arguments$response
  N <- nrow(response)
  I <- ncol(response)
  D <- object$arguments$D
  logLik.val <- as.numeric(object$logLik)

  AIC <- -2 * logLik.val + 2 * object$npar
  BIC <- -2 * logLik.val + object$npar * log(N)

  cat_range <- if (!is.null(object$length.poly)) {
    paste0(min(object$length.poly), "--", max(object$length.poly))
  } else "unknown"

  res <- list(
    call       = object$call,
    model.info = list(
      family = "MGGUM",
      description = .fc_model_name("MGGUM", object),
      D      = D,
      method = object$method
    ),
    data.info = list(
      N = N,
      I = I,
      response.type = paste0("polytomous (", cat_range, " categories)")
    ),
    fit.stats = list(
      LogLik = logLik.val,
      npar   = object$npar,
      AIC    = AIC,
      BIC    = BIC
    ),
    par.summary   = .fc_summary_par_range(object$par$est, object$par$free, digits),
    theta.summary = .fc_summary_theta(object$theta$est, digits),
    Corr          = .fc_summary_corr(object$Corr$est, digits),
    convergence   = .fc_summary_convergence(object),
    digits        = digits
  )
  class(res) <- "summary.MGGUM"
  invisible(res)
}

# ---- summary.FCMIRT ----

#' @describeIn summary Summary method for \code{FCMIRT} objects.
#'   Forced-Choice Multidimensional IRT: dominance model with sequential
#'   ranking over item endorsement logits at the block level. Extracts
#'   block configuration, FC type
#'   (RANK/MOLE/PICK), and statement-level parameter summaries.
#' @method summary FCMIRT
#' @export
summary.FCMIRT <- function(object, digits = 4, ...) {
  N <- nrow(object$response)
  B <- length(object$block.items)
  I <- length(unlist(object$block.items))
  D <- object$arguments$D
  model <- if (!is.null(object$arguments$model)) object$arguments$model
           else if (!is.null(object$model)) object$model else "m2pl"
  logLik.val <- as.numeric(object$logLik)

  AIC <- -2 * logLik.val + 2 * object$npar
  BIC <- -2 * logLik.val + object$npar * log(N)

  fc.types <- if (!is.null(object$fc.type)) {
    paste(unique(object$fc.type), collapse = ", ")
  } else "RANK"

  res <- list(
    call       = object$call,
    model.info = list(
      family = "FCMIRT",
      description = .fc_model_name("FCMIRT", object),
      model  = model,
      D      = D,
      method = object$method,
      fc.type = fc.types
    ),
    data.info = list(
      N = N,
      B = B,
      I = I,
      response.type = paste0("forced-choice (", fc.types, ")")
    ),
    fit.stats = list(
      LogLik = logLik.val,
      npar   = object$npar,
      AIC    = AIC,
      BIC    = BIC
    ),
    par.summary   = .fc_summary_par_range(object$par$est, object$par$free, digits),
    theta.summary = .fc_summary_theta(object$theta$est, digits),
    Corr          = .fc_summary_corr(object$Corr$est, digits),
    convergence   = .fc_summary_convergence(object),
    digits        = digits
  )
  class(res) <- "summary.FCMIRT"
  invisible(res)
}

# ---- summary.FCDCM ----

#' @describeIn summary Summary method for \code{FCDCM} objects.
#'   Forced-Choice Diagnostic Classification Model: higher-order trait with
#'   exact marginalization over \eqn{2^D} attribute profiles. Extracts
#'   DCM type (DINA/DINO), higher-order delta parameters, block eta
#'   parameters, and posterior classification diagnostics.
#' @method summary FCDCM
#' @export
summary.FCDCM <- function(object, digits = 4, ...) {
  response <- object$response
  N <- nrow(response)
  B <- ncol(response)
  D <- object$arguments$D
  logLik.val <- as.numeric(object$logLik)

  AIC <- -2 * logLik.val + 2 * object$npar
  BIC <- -2 * logLik.val + object$npar * log(N)

  dcm.type <- if (!is.null(object$dcm.type)) {
    paste(unique(object$dcm.type), collapse = ", ")
  } else "DINA"

  # Posterior classification summary
  alpha.est <- object$alpha
  if (!is.null(alpha.est)) {
    if (is.list(alpha.est) && !is.null(alpha.est$est)) {
      alpha.mat <- as.matrix(alpha.est$est)
    } else if (is.list(alpha.est) && !is.null(alpha.est$prob)) {
      alpha.mat <- as.matrix(alpha.est$prob)
    } else {
      alpha.mat <- as.matrix(alpha.est)
    }
    n.patterns <- nrow(unique(alpha.mat > 0.5))
    max.post <- if (!is.null(object$class.post)) {
      mean(apply(as.matrix(object$class.post), 1, max), na.rm = TRUE)
    } else NA_real_

    class.info <- list(
      n_patterns_observed = n.patterns,
      n_patterns_possible = 2^D,
      mean_max_posterior = max.post
    )
  } else {
    class.info <- list(n_patterns_observed = NA_integer_,
                       n_patterns_possible = 2^D,
                       mean_max_posterior = NA_real_)
  }

  res <- list(
    call       = object$call,
    model.info = list(
      family = "FCDCM",
      description = .fc_model_name("FCDCM", object),
      D      = D,
      model  = dcm.type,
      method = object$method
    ),
    data.info = list(
      N = N,
      B = B,
      I = nrow(object$Q.matrix),
      response.type = "forced-choice binary blocks"
    ),
    fit.stats = list(
      LogLik = logLik.val,
      npar   = object$npar,
      AIC    = AIC,
      BIC    = BIC,
      Corr   = 1
    ),
    delta.summary = if (!is.null(object$delta$est)) {
      round(as.matrix(object$delta$est), digits)
    } else NULL,
    par.summary   = .fc_summary_par_range(object$par$est, NULL, digits),
    theta.summary = .fc_summary_theta(object$theta$est, digits),
    classification = class.info,
    convergence   = .fc_summary_convergence(object),
    digits        = digits
  )
  class(res) <- "summary.FCDCM"
  invisible(res)
}

# ---- summary.FCGDINA ----

#' @describeIn summary Summary method for \code{FCGDINA} objects.
#'   Forced-Choice GDINA model: CDM item model (DINA/DINO/ACDM/GDINA) with
#'   sequential forced-choice ranking over item endorsement logits.
#' @method summary FCGDINA
#' @export
summary.FCGDINA <- function(object, digits = 4, ...) {
  response <- object$response
  N <- nrow(response)
  B <- length(object$block.items)
  I <- length(unlist(object$block.items, use.names = FALSE))
  D <- ncol(object$Q.matrix)
  logLik.val <- as.numeric(object$logLik)

  AIC <- -2 * logLik.val + 2 * object$npar
  BIC <- -2 * logLik.val + object$npar * log(N)

  fc.types <- if (!is.null(object$fc.type)) {
    paste(unique(object$fc.type), collapse = ", ")
  } else "RANK"

  max.delta.len <- max(vapply(object$delta$est, length, integer(1L)))
  delta.mat <- do.call(rbind, lapply(object$delta$est, function(x) {
    c(x, rep(NA_real_, max.delta.len - length(x)))
  }))
  rownames(delta.mat) <- paste0("item", seq_len(nrow(delta.mat)))
  colnames(delta.mat) <- paste0("delta", seq_len(ncol(delta.mat)))

  alpha.mat <- as.matrix(object$alpha$est)
  class.info <- list(
    n_patterns_observed = nrow(unique(alpha.mat > 0.5)),
    n_patterns_possible = 2^D,
    mean_max_posterior = if (!is.null(object$class.post)) {
      mean(apply(object$class.post, 1L, max))
    } else NA_real_
  )

  pi.stats <- if (!is.null(object$pi)) {
    nz <- sum(object$pi > 1e-8)
    list(n_nonzero = nz, max = max(object$pi), min = min(object$pi),
         entropy = -sum(object$pi[object$pi > 0] * log(object$pi[object$pi > 0])))
  } else NULL

  res <- list(
    call = object$call,
    model.info = list(
      family = "FCGDINA",
      description = .fc_model_name("FCGDINA", object),
      D = D,
      model = object$model,
      method = object$method,
      fc.type = fc.types
    ),
    data.info = list(
      N = N,
      B = B,
      I = I,
      response.type = paste0("forced-choice CDM (", fc.types, ")")
    ),
    fit.stats = list(
      LogLik = logLik.val,
      npar = object$npar,
      AIC = AIC,
      BIC = BIC
    ),
    delta.summary = round(delta.mat, digits),
    alpha.summary = .fc_summary_theta(alpha.mat, digits),
    classification = class.info,
    pi.stats = pi.stats,
    convergence = .fc_summary_convergence(object),
    digits = digits
  )
  class(res) <- "summary.FCGDINA"
  invisible(res)
}

# ---- summary.FCGGUM ----

#' @describeIn summary Summary method for \code{FCGGUM} objects.
#'   Forced-Choice Generalized Graded Unfolding Model: ideal-point model
#'   with sequential ranking over binary GGUM endorsement logits. Extracts
#'   forced-choice block structure, unfolding parameters (a, delta, tau),
#'   and convergence info.
#' @method summary FCGGUM
#' @export
summary.FCGGUM <- function(object, digits = 4, ...) {
  N <- nrow(object$response)
  B <- length(object$block.items)
  I <- length(unlist(object$block.items))
  D <- object$arguments$D
  logLik.val <- as.numeric(object$logLik)

  AIC <- -2 * logLik.val + 2 * object$npar
  BIC <- -2 * logLik.val + object$npar * log(N)

  fc.types <- if (!is.null(object$fc.type)) {
    paste(unique(object$fc.type), collapse = ", ")
  } else "RANK"

  cat_range <- if (!is.null(object$length.poly)) {
    paste0(min(object$length.poly), "--", max(object$length.poly))
  } else "unknown"

  res <- list(
    call       = object$call,
    model.info = list(
      family = "FCGGUM",
      description = .fc_model_name("FCGGUM", object),
      D      = D,
      method = object$method,
      fc.type = fc.types
    ),
    data.info = list(
      N = N,
      B = B,
      I = I,
      response.type = paste0("forced-choice unfolding (", fc.types, ", ",
                             cat_range, " categories)")
    ),
    fit.stats = list(
      LogLik = logLik.val,
      npar   = object$npar,
      AIC    = AIC,
      BIC    = BIC
    ),
    par.summary   = .fc_summary_par_range(object$par$est, object$par$free, digits),
    theta.summary = .fc_summary_theta(object$theta$est, digits),
    Corr          = .fc_summary_corr(object$Corr$est, digits),
    convergence   = .fc_summary_convergence(object),
    digits        = digits
  )
  class(res) <- "summary.FCGGUM"
  invisible(res)
}

# ---- summary.TIRT ----

#' @describeIn summary Summary method for \code{TIRT} objects.
#'   Thurstonian IRT for Forced-Choice: pairwise probit comparison of latent
#'   utility differences. Extracts statement loadings (lambda), uniquenesses
#'   (psi2), pairwise gamma matrix, and factor correlations.
#' @method summary TIRT
#' @export
summary.TIRT <- function(object, digits = 4, ...) {
  N <- nrow(object$response)
  I.pairs <- ncol(object$response)
  N.block <- length(object$block.items)
  I.states <- nrow(object$par$est)
  D <- ncol(object$par$est) - 1L
  logLik.val <- as.numeric(object$logLik)

  AIC <- -2 * logLik.val + 2 * object$npar
  BIC <- -2 * logLik.val + object$npar * log(N)

  fc.types <- if (!is.null(object$fc.type)) {
    paste(unique(object$fc.type), collapse = ", ")
  } else "RANK"

  res <- list(
    call       = object$call,
    model.info = list(
      family = "TIRT",
      description = .fc_model_name("TIRT", object),
      D      = D,
      I.states = I.states,
      N.block = N.block,
      method = object$method,
      fc.type = fc.types
    ),
    data.info = list(
      N = N,
      I.pairs = I.pairs,
      I.states = I.states,
      response.type = paste0("pairwise binary (", fc.types, ")")
    ),
    fit.stats = list(
      LogLik = logLik.val,
      npar   = object$npar,
      AIC    = AIC,
      BIC    = BIC
    ),
    lambda.summary = .fc_summary_par_range(
      object$par$est[, seq_len(D), drop = FALSE], NULL, digits),
    psi2.summary = if (D + 1L <= ncol(object$par$est)) {
      matrix(summary(as.vector(object$par$est[, D + 1L])), nrow = 1L)
    } else NULL,
    theta.summary = .fc_summary_theta(object$theta$est, digits),
    Corr          = .fc_summary_corr(object$Corr$est, digits),
    convergence   = .fc_summary_convergence(object),
    digits        = digits
  )
  class(res) <- "summary.TIRT"
  invisible(res)
}


# ===========================================================================
# Internal gof table builders (used by summary.good.of.fit)
# ===========================================================================

.gof_summary_tables <- function(object, digits = 4) {
  rmsea.ci <- .gof_rmsea_ci_text(object, digits)

  tables <- list(
    Overview                  = .gof_overview_table(object, digits),
    Absolute_Fit_M2           = .gof_absolute_m2_table(object, rmsea.ci, digits),
    Comparative_Fit           = .gof_comparative_fit_table(object, digits),
    Likelihood_and_IC         = .gof_likelihood_ic_table(object, digits),
    Pseudo_R2                 = .gof_pseudo_r2_table(object, digits),
    Local_Dependence          = .gof_ld_table(object, digits),
    Classification            = .gof_class_table(object, digits)
  )

  tables[vapply(tables, function(x) nrow(x) > 0L, logical(1L))]
}

.gof_overview_table <- function(object, digits) {
  null.desc <- "independence model (item margins only)"
  if (!is.null(object$response.type) && object$response.type == "nominal") {
    null.desc <- "independence model (block margins only)"
  }
  data.frame(
    Field = c("Sample size", "Observed responses", "Items / indicators",
              "Response type", "Free parameters", "Moments",
              "Bivariate moments", "Null model"),
    Value = c(
      .gof_format(object$N, digits, int = TRUE),
      .gof_format(object$observed.responses.n, digits, int = TRUE),
      .gof_format(object$nitems, digits, int = TRUE),
      if (!is.null(object$response.type)) object$response.type else "NA",
      .gof_format(object$npar, digits, int = TRUE),
      .gof_format(object$moments.n, digits, int = TRUE),
      .gof_format(object$pairs.n, digits, int = TRUE),
      null.desc
    ),
    stringsAsFactors = FALSE
  )
}

.gof_absolute_m2_table <- function(object, rmsea.ci, digits) {
  .gof_metric_table(
    c("M2", "df", "p-value",
      "RMSEA", "RMSEA CI", "RMSEA p-close", "SRMSR",
      "McDonald NCI"),
    list(object$M2.value, object$df,
         object$p.value, object$RMSEA2, rmsea.ci,
         object$RMSEA.p.close, object$SRMSR, object$McDonald.NCI),
    c("lower", "positive", "> .05",
      "<= .05", "narrow", "> .05", "<= .08", ">= .90"),
    c("limited-information test",
      "degrees of freedom", .gof_interpret("p.value", object$p.value, object),
      .gof_interpret("RMSEA2", object$RMSEA2, object),
      "RMSEA confidence interval",
      .gof_interpret("RMSEA.p.close", object$RMSEA.p.close, object),
      .gof_interpret("SRMSR", object$SRMSR, object),
      .gof_interpret("McDonald.NCI", object$McDonald.NCI, object)),
    digits
  )
}

.gof_comparative_fit_table <- function(object, digits) {
  .gof_metric_table(
    c("CFI", "TLI", "IFI"),
    list(object$CFI, object$TLI, object$IFI),
    c(">= .90", ">= .90", ">= .90"),
    c(.gof_interpret("CFI", object$CFI, object),
      .gof_interpret("TLI", object$TLI, object),
      .gof_interpret("IFI", object$IFI, object)),
    digits
  )
}

.gof_likelihood_ic_table <- function(object, digits) {
  .gof_metric_table(
    c("SUB:Likelihood",
      "Log-likelihood", "Deviance (-2LL)",
      "SUB:Null-model Improvement",
      "G2 (vs null)", "df", "p",
      "SUB:Information Criteria",
      "AIC", "BIC", "SABIC", "CAIC"),
    list(NA, object$LogLik, object$deviance,
         NA, object$LRT, object$LRT.df, object$LRT.p,
         NA, object$AIC, object$BIC, object$SABIC, object$CAIC),
    c("", "higher", "lower",
      "", "", "", "< .05",
      "", "lower", "lower", "lower", "lower"),
    c("",
      "fitted log-likelihood", "fitted model deviance",
      "",
      "G2 = 2(LL_fitted - LL_null)", "npar (free parameters)",
      "significant => model improves over null",
      "",
      "Akaike IC", "Bayesian IC", "sample-adjusted BIC",
      "consistent AIC"),
    digits
  )
}

.gof_pseudo_r2_table <- function(object, digits) {
  .gof_metric_table(
    c("McFadden R2", "McFadden adj. R2",
      "Cox-Snell R2", "Nagelkerke R2",
      "Aldrich-Nelson R2", "Veall-Zimmermann R2"),
    list(object$McFadden.R2, object$McFadden.adj.R2,
         object$CoxSnell.R2, object$Nagelkerke.R2,
         object$AldrichNelson.R2, object$VeallZimmermann.R2),
    c("higher", "higher", "higher", "higher", "higher", "higher"),
    c("1 - (LL_fitted / LL_null)",
      "McFadden R2 penalised by npar",
      "1 - exp(-G2_null / N)",
      "(Cox-Snell) / (1 - exp(2*LL_null/N))",
      "G2_null / (G2_null + N)",
      "Aldrich-Nelson with upper-bound correction"),
    digits
  )
}

.gof_ld_table <- function(object, digits) {
  .gof_metric_table(
    c("Q3 mean", "Q3 |max|", "Adjusted Q3 |max|", "Q3 P95"),
    list(object$q3.mean, object$q3.abs.max, object$q3.adjusted.abs.max,
         object$q3.p95),
    c("near 0", "lower", "lower", "lower"),
    c("mean residual correlation", "largest absolute Q3",
      "mean-adjusted largest absolute Q3", "95th percentile Q3"),
    digits
  )
}

.gof_class_table <- function(object, digits) {
  .gof_metric_table(
    c("Normalized entropy", "Mean max posterior", "Min max posterior"),
    list(object$posterior.entropy.normalized, object$mean.max.posterior,
         object$min.max.posterior),
    c(">= .70", "higher", "higher"),
    c("classification separation", "average maximum posterior",
      "weakest case maximum posterior"),
    digits
  )
}

# ---- gof helpers ----

.gof_metric_table <- function(metric, estimate, recommended, interpretation, digits) {
  if (!is.list(estimate)) {
    estimate <- as.list(estimate)
  }
  estimate_chr <- vapply(estimate, .gof_format, character(1L), digits = digits)

  out <- data.frame(
    Metric         = metric,
    Estimate       = estimate_chr,
    Recommended    = recommended,
    Interpretation = interpretation,
    stringsAsFactors = FALSE
  )
  out
}

.gof_rmsea_ci_text <- function(object, digits) {
  lo <- object$RMSEA.lower
  hi <- object$RMSEA.upper
  if (is.null(lo) || is.null(hi) || is.na(lo) || is.na(hi)) {
    return(NA_character_)
  }
  level <- object$RMSEA.CI.level
  if (is.null(level) || is.na(level)) {
    level <- 0.90
  }
  paste0(.gof_format(100 * level, 0), "% [",
         .gof_format(lo, digits), ", ", .gof_format(hi, digits), "]")
}

.gof_format <- function(x, digits = 4, int = FALSE) {
  if (is.null(x) || length(x) == 0L) {
    return("NA")
  }
  x <- x[[1L]]
  if (is.character(x)) {
    if (is.na(x) || !nzchar(x)) {
      return("NA")
    }
    numeric.pattern <- "^[-+]?((\\d+\\.?\\d*)|(\\.\\d+))([eE][-+]?\\d+)?$"
    if (grepl(numeric.pattern, x)) {
      x <- as.numeric(x)
    } else {
      return(x)
    }
  }
  if (is.logical(x)) {
    return(as.character(x))
  }
  if (is.na(x)) {
    return("NA")
  }
  if (!is.finite(x)) {
    return(as.character(x))
  }
  if (int || abs(x - round(x)) < .Machine$double.eps^0.5) {
    return(formatC(round(x), format = "d", big.mark = ","))
  }
  formatC(x, format = "f", digits = digits, big.mark = ",")
}

.gof_interpret <- function(metric, value, object) {
  if (length(value) == 0L || is.null(value) || is.na(value)) {
    return("not available")
  }
  switch(metric,
    p.value = if (value >= 0.05) "limited-information misfit not significant"
              else "significant limited-information misfit",
    M2.value = "limited-information marginal discrepancy",
    df = "degrees of freedom for the fit statistic",
    RMSEA2 = if (value <= 0.05) "close approximate fit"
             else if (value <= 0.08) "reasonable approximate fit"
             else if (value <= 0.10) "mediocre approximate fit"
             else "poor approximate fit",
    RMSEA.CI = "interval estimate for approximate-fit error",
    RMSEA.p.close = if (value >= 0.05) "close-fit hypothesis not rejected"
                    else "close-fit hypothesis rejected",
    SRMSR = if (value <= 0.08) "small standardized marginal residuals"
            else "large standardized marginal residuals",
    McDonald.NCI = if (value >= 0.95) "very high noncentrality fit"
                   else if (value >= 0.90) "acceptable noncentrality fit"
                   else "low noncentrality fit",
    CFI = .gof_interpret_incremental(value),
    TLI = .gof_interpret_incremental(value),
    NFI = .gof_interpret_incremental(value),
    IFI = .gof_interpret_incremental(value),
    RFI = .gof_interpret_incremental(value),
    pattern.p = if (value >= 0.05) "full-pattern misfit not significant"
                else "significant full-pattern misfit",
    "not available"
  )
}

.gof_interpret_incremental <- function(value) {
  if (is.na(value)) {
    return("not available")
  }
  if (value >= 0.95) {
    "good relative fit by common heuristic"
  } else if (value >= 0.90) {
    "acceptable relative fit by common heuristic"
  } else {
    "weak relative fit by common heuristic"
  }
}

#' Simulate Data from the Forced-Choice GGUM Model
#'
#' @description
#' Generates forced-choice ranking data using the GGUM ideal-point model as the
#' item-level endorsement process. Within-block rankings follow the same
#' sequential Luce/Plackett mechanism as \code{\link{model.FCGGUM}}: the
#' utility for each statement is the logit of its binary GGUM endorsement
#' probability.
#'
#' @param N.person Integer; number of persons (default: 1000).
#' @param N.block Integer; number of forced-choice blocks (default: 10).
#' @param I.block Integer; items per block (default: 2).
#' @param D Integer; number of latent dimensions (default: 3, must be >= 2).
#' @param fc.type Character; \code{"RANK"} (default), \code{"MOLE"}, or
#'   \code{"PICK"}.
#' @param control Optional list with entries \code{Q.matrix},
#'   \code{block.items}, and \code{Corr}.
#'
#' @return An object of class \code{"data.FCGGUM"} containing \code{data},
#'   \code{response}, \code{theta}, \code{par}, \code{Q.matrix},
#'   \code{block.items}, \code{Corr}, \code{patterns}, \code{patterns.total},
#'   \code{prob}, \code{mask}, and data-generating arguments.
#'
#' @seealso \code{\link{fit.FCGGUM}}, \code{\link{model.FCGGUM}}
#'
#' @examples
#' set.seed(123)
#' sim <- sim.data.FCGGUM(N.person = 20, N.block = 3, I.block = 2,
#'                        D = 2, fc.type = "RANK")
#' str(sim$data)
#' head(sim$response)
#' dim(sim$prob)
#'
#' @export
sim.data.FCGGUM <- function(N.person = 1000, N.block = 10, I.block = 2,
                            D = 3, fc.type = "RANK", control = NULL) {

  call <- match.call()

  if (D < 2) {
    stop("FCGGUM requires at least 2 latent dimensions (D >= 2).", call. = FALSE)
  }

  if (is.null(control)) control <- list()

  # ---- Scalar integer validation ----
  N.person <- check_integer_scalar(N.person, "N.person", 1L)
  D        <- check_integer_scalar(D,        "D",        2L)
  I.block  <- check_integer_scalar(I.block,  "I.block",  2L)
  N.block  <- check_integer_scalar(N.block,  "N.block",  1L)

  # ---- Q-matrix and block configuration ----
  Q.matrix    <- control$Q.matrix
  block.items <- control$block.items

  res <- resolve_fc_blocks(
    Q.matrix       = Q.matrix,
    block.items    = block.items,
    I.block        = I.block,
    N.block        = N.block,
    D              = D,
    q.valid.values = c(-1, 0, 1),
    q.row.check    = function(r) any(r != 0)
  )
  I.states   <- res$I.states
  N.block    <- res$N.block
  I.block    <- res$I.block
  block.items <- res$block.items
  Q.matrix   <- res$Q.matrix
  D          <- res$D

  if (D < 2) {
    stop("FCGGUM requires at least 2 latent dimensions (D >= 2).", call. = FALSE)
  }

  # ---- fc.type validation ----
  fc.type <- normalize_fc_type(fc.type, N.block)

  # ---- Corr and theta ----
  Corr  <- validate_corr_matrix(control$Corr, D)
  theta <- generate_theta_mvn(N.person, D, Corr)

  # ---- Auto-generate Q-matrix if not supplied ----
  if (is.null(Q.matrix)) {
    Q.matrix <- sim.Q.matrix.FC(I.states = I.states, D = D,
                                block.items = block.items,
                                allow.negative = TRUE)
  }

  # ---- GGUM item parameters ----
  a <- matrix(runif(I.states * D, 0.5, 2.0), I.states, D) * abs(Q.matrix)

  par <- matrix(NA, nrow = I.states, ncol = D + D + 2)
  par[, 1:D] <- a
  mask <- matrix(NA, nrow = I.states, ncol = D + D + 2)
  mask[, 1:D] <- 1
  for (i in seq_len(I.states)) {

    for (d in seq_len(D)) {
      if (Q.matrix[i, d] == 1) {
        par[i, D + d] <- runif(1, 0.0, 2.0)
      } else if (Q.matrix[i, d] == -1) {
        par[i, D + d] <- runif(1, -2.0, 0.0)
      } else {
        par[i, D + d] <- 0.0
      }
    }
    mask[i, (D + 1):(D + D)] <- 1

    par[i, D + D + 1] <- 0.0
    mask[i, D + D + 1] <- 1
    Ki <- 2
    Kf <- Ki - 1
    tau_free <- numeric(Kf)
    for (t in seq_len(Kf)) {
      lo <- -2.00 + (t - 1) * 2.0 / Kf
      hi <- -2.00 +  t      * 2.0 / Kf
      tau_free[t] <- runif(1, lo, hi)
    }

    tau_sorted <- sort(tau_free, decreasing = FALSE)
    par[i, (D + D + 2):(D + D + Ki)] <- tau_sorted
    mask[i, (D + D + 2):(D + D + Ki)] <- 1
  }

  # ---- FC permutation patterns ----
  pat <- generate_fc_permutation_patterns(block.items, fc.type)
  patterns.total <- pat$patterns.total
  patterns       <- pat$patterns

  # ---- FC pattern probabilities ----
  prob <- model.FCGGUM(theta, par, patterns.total, patterns)

  # ---- Response sampling ----
  samples <- sample_fc_responses(prob, patterns, N.person, N.block)
  response <- samples$response
  data     <- samples$data

  # ---- Row/column names ----
  rownames(theta) <- paste0("person", seq_len(N.person))
  if (D > 1) {
    colnames(theta) <- paste0("theta", seq_len(D))
  }

  rownames(par) <- paste0("item", seq_len(I.states))
  colnames(par) <- c(paste0("a", seq_len(D)),
                     paste0("delta", seq_len(D)),
                     paste0("tau", 0:1))
  rownames(Q.matrix) <- paste0("item", seq_len(I.states))
  colnames(Q.matrix) <- paste0("dim", seq_len(D))

  # ---- Output assembly ----
  data.obj <- list(
    data           = data,
    response       = response,
    theta          = theta,
    par            = par,
    Q.matrix       = Q.matrix,
    block.items    = block.items,
    Corr           = Corr,
    patterns       = patterns,
    patterns.total = patterns.total,
    prob           = prob,
    mask           = mask,
    N.person       = N.person,
    N.block        = N.block,
    I.block        = I.block,
    D              = D,
    I.states       = I.states,
    fc.type        = fc.type,
    call           = call,
    arguments      = list(
      N.person = N.person,
      N.block  = N.block,
      I.block  = I.block,
      D        = D,
      fc.type  = fc.type,
      control  = control
    )
  )

  class(data.obj) <- "data.FCGGUM"

  return(data.obj)
}

#' Compute FCGGUM Forced-Choice Pattern Probabilities
#' @param theta Latent trait matrix.
#' @param par Binary GGUM item parameter matrix with columns
#'   \code{a1..aD, delta1..deltaD, tau0, tau1}; \code{tau0} is fixed at 0
#'   and \code{tau1 < 0}.
#' @param patterns.total Full ranking patterns.
#' @param patterns Observed ranking patterns.
#' @return Block ranking probability matrix. The code first computes binary
#'   GGUM endorsement probabilities from the distance-based MGGUM formula,
#'   transforms them to logits, and then applies the sequential
#'   Luce/Plackett rule. MOLE and PICK probabilities are sums over
#'   compatible full rankings normalized over observed patterns.
#' @keywords internal
model.FCGGUM <- function(theta, par, patterns.total, patterns){
  cpp_model_FCGGUM(theta, par, patterns.total, patterns)
}

#' @title Extract Log-Likelihood from Fitted Models
#'
#' @description
#' Extracts the marginal log-likelihood from a fitted ForceChoice model
#' object. For continuous-trait IRT models, the log-likelihood is computed
#' by numerically integrating the joint likelihood of the observed responses
#' over the latent trait distribution using Cartesian-product normal
#' quadrature. For FCGDINA, the marginal likelihood is obtained by summing
#' over the \eqn{2^D} discrete attribute profiles.
#'
#' For a sample of \eqn{N} independent persons with responses
#' \eqn{\mathbf{Y}_j}, the marginal log-likelihood is:
#' \deqn{
#'   \ell = \sum_{j=1}^{N} \log \int
#'   P(\mathbf{Y}_j \mid \boldsymbol{\theta})\,
#'   \phi_D(\boldsymbol{\theta}; \mathbf{0}, \boldsymbol{\Sigma})\,
#'   d\boldsymbol{\theta},
#' }
#' where \eqn{\phi_D(\cdot)} is the \eqn{D}-variate normal density with
#' correlation matrix \eqn{\boldsymbol{\Sigma}}. The integral is approximated
#' on a Cartesian-product quadrature grid over \eqn{[-6, 6]^D}.
#'
#' The continuous-trait quadrature weights are normalized normal densities
#' computed on the grid points, with the fitted correlation matrix used when
#' available. FCGDINA instead uses the estimated class-proportion vector
#' \eqn{\boldsymbol{\pi}} over attribute profiles.
#'
#' @param object A fitted model object of class \code{"MIRT"}, \code{"MGPCM"},
#'   \code{"MGGUM"}, \code{"FCMIRT"}, \code{"FCDCM"}, \code{"FCGDINA"},
#'   \code{"FCGGUM"}, or \code{"TIRT"}.
#' @param theta.low Numeric; lower bound for the quadrature grid on the
#'   latent scale (default: \code{-6}).
#' @param theta.up Numeric; upper bound for the quadrature grid (default:
#'   \code{6}).
#' @param L Integer; number of quadrature points per dimension. If
#'   \code{NULL}, automatically chosen based on \eqn{D} (e.g., 61 for
#'   \eqn{D = 1}, 31 for \eqn{D = 2}, 15 for \eqn{D = 3}).
#' @param ... Additional arguments (currently unused).
#'
#' @return An object of class \code{"logLik"} with attributes:
#' \itemize{
#'   \item \code{nobs}: number of observations (\eqn{N})
#'   \item \code{df}: number of free parameters
#'   \item \code{theta.norm}: continuous-trait quadrature grid points, when
#'         applicable
#'   \item \code{prob}: model-implied response probabilities at grid points,
#'         when applicable
#'   \item \code{pi}: quadrature weights or FCGDINA class probabilities
#'   \item \code{L}: quadrature grid size used, when applicable
#'   \item \code{prob.class}: FCGDINA class-conditional forced-choice
#'         probabilities, when applicable
#' }
#'
#' @seealso
#' \code{\link{fit.MIRT}}, \code{\link{get.fit.index}},
#' \code{\link{good.of.fit}}
#'
#' @examples
#' sim <- sim.data.MIRT(N = 20, I = 6, D = 2, model = "m2pl")
#' fit <- fit.MIRT(
#'   sim$response, model = "m2pl", D = 2, method = "iStEM",
#'   control.method = list(
#'     vis = FALSE, seed = 123,
#'     M = 2, B = 2, burnin.maxitr = 2,
#'     maxitr = 3, eps1 = 10, eps2 = 10,
#'     estimate.se = FALSE)
#' )
#' ll <- logLik(fit, L = 9)
#' as.numeric(ll)
#' attr(ll, "df")
#'
#' @name logLik-class
#' @rdname logLik-class
#' @importFrom stats logLik
NULL

# The following suppresses roxygen2 auto-detection of class<-"logLik"
# as an S4 class definition.
#' @title S3 Methods for logLik
#'
#' @description
#' S3 methods for extracting marginal log-likelihoods from fitted
#' \pkg{ForceChoice} model objects.
#'
#' @param object A fitted model object of class \code{"MIRT"},
#'   \code{"MGPCM"}, \code{"MGGUM"}, \code{"FCMIRT"}, \code{"FCDCM"},
#'   \code{"FCGDINA"}, \code{"FCGGUM"}, or \code{"TIRT"}.
#' @param theta.low Numeric; lower bound for the quadrature grid on the
#'   latent scale. Used by continuous-trait models; if \code{NULL}, the value
#'   stored in the fitted object's controls is used, falling back to
#'   \code{-6}.
#' @param theta.up Numeric; upper bound for the quadrature grid on the latent
#'   scale. Used by continuous-trait models; if \code{NULL}, the value stored
#'   in the fitted object's controls is used, falling back to \code{6}.
#' @param L Integer; number of quadrature points per dimension. If
#'   \code{NULL}, the package chooses a dimension-dependent default.
#' @param ... Additional arguments passed to the method. Currently ignored by
#'   the implemented methods.
#'
#' @return An object of class \code{"logLik"}: a length-one numeric vector
#'   containing the marginal log-likelihood, with at least attributes
#'   \code{nobs} (sample size) and \code{df} (number of free parameters).
#'   Continuous-trait methods also attach quadrature diagnostics such as
#'   \code{theta.norm}, \code{prob}, \code{pi}, and \code{L}; FCGDINA attaches
#'   latent-class probabilities and class-conditional response probabilities.
#'
#' @details
#' For MIRT, MGPCM, MGGUM, FCMIRT, FCGGUM, TIRT, and FCDCM objects, the
#' likelihood is evaluated by marginalizing over a normal quadrature grid
#' (and, for FCDCM, over attribute profiles conditional on the higher-order
#' trait). For FCGDINA objects, the likelihood is evaluated exactly over the
#' \eqn{2^D} latent attribute profiles.
#'
#' @name logLik-class
#' @rdname logLik-class
#' @keywords internal
NULL

#' Forced-Choice Data Conversion Formats
#'
#' @description
#' Documents the three data objects used by the forced-choice conversion
#' helpers: human-readable \code{data}, block definitions
#' \code{block.items}, and model-ready \code{response} matrices.  This
#' overview is intended as the starting point before using
#' \code{\link{get.response.from.data}},
#' \code{\link{get.data.from.response}}, the TIRT-specific converters, or
#' the FCDCM-specific converters.
#'
#' @details
#' The package uses two layers of forced-choice representation:
#' \describe{
#'   \item{\code{data}}{An \eqn{N \times B} matrix or data frame that is easy
#'     to read and edit.  Rows are persons and columns are forced-choice
#'     blocks.  A cell records the observed ordering with item labels separated
#'     by \code{">"}.  For example, \code{"3>4>5"} means item 3 was ranked
#'     above item 4, which was ranked above item 5.}
#'   \item{\code{block.items}}{A list of length \eqn{B}.  Element \eqn{b}
#'     gives the item labels appearing in column \eqn{b} of \code{data}.  This
#'     object is the bridge between column order in the data file and item rows
#'     in model parameter matrices such as \code{Q.matrix}.}
#'   \item{\code{response}}{The compact numeric representation consumed by
#'     model fitting code.  Its meaning depends on the model family.  For
#'     FCGGUM, FCMIRT, and FCGDINA, \code{response} is an \eqn{N \times B}
#'     matrix of 1-based pattern indices.  For TIRT, \code{response} is an
#'     \eqn{N \times P} matrix of pairwise binary outcomes.  For FCDCM,
#'     \code{response} is an \eqn{N \times B} binary matrix for two-item
#'     blocks.}
#' }
#'
#' @section Choosing a Converter:
#' \describe{
#'   \item{FCGGUM, FCMIRT, and FCGDINA}{Use
#'     \code{\link{get.response.from.data}} to convert character strings to
#'     pattern indices, and \code{\link{get.data.from.response}} for the
#'     inverse conversion.}
#'   \item{TIRT}{Use \code{\link{get.response.from.data.TIRT}} and
#'     \code{\link{get.data.from.response.TIRT}}.  TIRT expands each block to
#'     pairwise comparisons, so its \code{response} columns are pairs rather
#'     than block-level pattern numbers.}
#'   \item{FCDCM}{Use \code{\link{get.response.from.data.FCDCM}} and
#'     \code{\link{get.data.from.response.FCDCM}}.  Each block has exactly two
#'     statements, so \code{response = 1} means the first statement in
#'     \code{block.items[[b]]} was preferred and \code{response = 0} means the
#'     second statement was preferred.}
#'   \item{Recovering \code{block.items}}{Use
#'     \code{\link{get.block.items.from.data}} for generic multi-item
#'     forced-choice data and \code{\link{get.block.items.from.data.FCDCM}}
#'     for FCDCM pair data.  Recovery is only possible for item labels that
#'     actually appear in the observed strings.}
#' }
#'
#' @section Core Conventions:
#' \itemize{
#'   \item The \eqn{b}th element of \code{block.items} always corresponds to
#'     the \eqn{b}th column of \code{data} or \code{response}.
#'   \item \code{fc.type = "RANK"} expects a complete ranking such as
#'     \code{"3>4>5"} for a three-item block.
#'   \item \code{fc.type = "MOLE"} expects a most-least string such as
#'     \code{"3>5"}: item 3 is most preferred and item 5 is least preferred.
#'   \item \code{fc.type = "PICK"} expects a single most-preferred item such
#'     as \code{"3"}.
#'   \item Generic FC pattern-index responses are 1-based.  They are row
#'     numbers in the deterministic pattern matrix generated internally; see
#'     \code{\link{generate_fc_permutation_patterns}} for the public pattern
#'     generator.
#'   \item TIRT data should use consecutive item labels across blocks, for
#'     example \code{list(1:2, 3:5, 6:8)}.  This is the format returned by
#'     \code{\link{sim.data.TIRT}} and expected by the TIRT conversion
#'     routines.
#' }
#'
#' @seealso
#' \code{\link{get.block.items.from.data}},
#' \code{\link{get.response.from.data}},
#' \code{\link{get.data.from.response}},
#' \code{\link{get.response.from.data.TIRT}},
#' \code{\link{get.data.from.response.TIRT}},
#' \code{\link{get.response.from.data.FCDCM}},
#' \code{\link{get.data.from.response.FCDCM}}
#'
#' @examples
#' # Generic FC conversion: response is a block-level pattern index.
#' block.items <- list(1:2, 3:5)
#' dat <- data.frame(
#'   block.1 = c("1>2", "2>1"),
#'   block.2 = c("3>4>5", "5>4>3"),
#'   stringsAsFactors = FALSE
#' )
#' resp <- get.response.from.data(dat, block.items, fc.type = "RANK")
#' resp
#' get.data.from.response(resp, block.items, fc.type = "RANK")
#'
#' # TIRT conversion: response is pairwise 0/1 data.
#' tirt <- get.response.from.data.TIRT(dat, block.items, fc.type = "RANK")
#' tirt$response
#' get.data.from.response.TIRT(
#'   tirt$response, block.items, fc.type = "RANK",
#'   pairs.value = tirt$pairs.value
#' )
#'
#' # FCDCM conversion: response is binary within each two-item block.
#' fcdcm.items <- list(c(1L, 3L), c(2L, 4L))
#' fcdcm.dat <- data.frame(
#'   block.1 = c("1>3", "3>1"),
#'   block.2 = c("2>4", "4>2"),
#'   stringsAsFactors = FALSE
#' )
#' fcdcm.resp <- get.response.from.data.FCDCM(fcdcm.dat, fcdcm.items)
#' fcdcm.resp$response
#' get.data.from.response.FCDCM(fcdcm.resp$response, fcdcm.items)
#'
#' @name forced_choice_data_conversion
#' @keywords utilities
NULL

# ---- Block-item extraction ----------------------------------------------------

#' Extract Block Items from Forced-Choice Data
#'
#' @description
#' Infers the item composition of each forced-choice block from a character
#' data matrix.  Each cell should encode the within-block item ordering
#' (e.g. \code{"2>1>3"} for a rank-3 block).  The function parses all
#' non-missing cells per block and returns sorted, unique item indices.
#'
#' This function is the recommended way to recover \code{block.items} when
#' only character-rank data are available.  It works for all forced-choice
#' types (\code{"RANK"}, \code{"MOLE"}, \code{"PICK"}) as long as every
#' item appears at least once across persons.
#'
#' @details
#' The returned \code{block.items} list defines the block layout used by the
#' other converters.  If \code{data} has \eqn{B} columns, the result has
#' \eqn{B} elements, and result element \eqn{b} belongs to data column
#' \eqn{b}.  Item labels are treated as global item indices; they are not
#' recoded to local positions.
#'
#' The function scans observed strings only.  Therefore, for partial-response
#' formats, it can only recover items that appear in the observed partial
#' strings:
#' \itemize{
#'   \item For \code{"RANK"}, a complete ranking normally contains every item
#'     in the block, so one valid row can be sufficient.
#'   \item For \code{"MOLE"}, only most- and least-preferred items appear in
#'     each cell.  An item that is never chosen as most or least cannot be
#'     recovered from \code{data} alone.
#'   \item For \code{"PICK"}, only picked items appear.  Items that are never
#'     picked cannot be recovered from \code{data} alone.
#' }
#' In those partial-response cases, pass a complete user-defined
#' \code{block.items} list to the fitting or conversion function when the raw
#' data do not mention every item.
#'
#' @param data An \eqn{N \times B} character matrix (or data frame) of
#'   forced-choice responses.  Each entry encodes an item ordering using
#'   integer item indices separated by \code{">"} (e.g. \code{"3>1>2"}).
#'   Missing values (\code{NA}) are permitted and skipped.
#'
#' @return A list of length \eqn{B}.  Each element is an integer vector of
#'   global item indices appearing in that block, sorted in ascending order.
#'
#' @seealso
#' \code{\link{get.block.items.from.data.FCDCM}} for the FCDCM (binary-pair)
#' variant.
#'
#' \code{\link{get.response.from.data}} to convert character data to an
#' integer pattern-index matrix.
#'
#' @export
#'
#' @examples
#' # 3 persons, 2 blocks.
#' # Block 1 contains items 1 and 2; block 2 contains items 3, 4, and 5.
#' dat <- data.frame(
#'   block.1 = c("1>2", "2>1", "1>2"),
#'   block.2 = c("3>4>5", "5>4>3", "4>3>5"),
#'   stringsAsFactors = FALSE
#' )
#' get.block.items.from.data(dat)
get.block.items.from.data <- function(data) {
  cpp_get_block_items_from_data(as.matrix(data))
}

#' Extract Block Items from FCDCM Data
#'
#' @description
#' Recovers the two-item composition of each FCDCM block from a character
#' data matrix of \code{"a>b"} / \code{"b>a"} preference strings.  Because
#' every FCDCM block always compares exactly two statements, this function
#' only needs to inspect the first non-missing row of each column.
#'
#' @details
#' The returned item pair is sorted in ascending order.  For example, both
#' \code{"1>3"} and \code{"3>1"} imply \code{c(1L, 3L)} for that block.  This
#' sorted pair is then the default coding direction used by
#' \code{\link{get.response.from.data.FCDCM}}: response value 1 means the
#' first element of the pair was preferred, and response value 0 means the
#' second element was preferred.
#'
#' This helper assumes the two compared statements are fixed within each
#' column.  If a column accidentally mixes different item pairs, only the first
#' parseable non-missing cell determines the returned pair, so such data should
#' be cleaned before conversion.
#'
#' @param data An \eqn{N \times B} character matrix (or data frame) of
#'   binary-preference strings.  Each cell must be of the form
#'   \code{"a>b"} or \code{"b>a"} where \code{a} and \code{b} are global
#'   statement indices.  Missing values are skipped.
#'
#' @return A list of length \eqn{B}.  Each element is a two-element integer
#'   vector \code{c(a, b)} with \code{a < b}.
#'
#' @seealso
#' \code{\link{get.block.items.from.data}} for the generic multi-item
#' variant used by FCGGUM, FCMIRT, and TIRT.
#'
#' \code{\link{get.response.from.data.FCDCM}} to convert FCDCM data strings
#' to a 0/1 response matrix.
#'
#' @export
#'
#' @examples
#' dat <- data.frame(
#'   b1 = c("1>3", "3>1", "1>3"),
#'   b2 = c("2>4", "2>4", "4>2"),
#'   stringsAsFactors = FALSE
#' )
#' get.block.items.from.data.FCDCM(dat)
get.block.items.from.data.FCDCM <- function(data) {
  cpp_get_block_items_fcdcm(as.matrix(data))
}


# ---- TIRT response <-> data conversion ----------------------------------------

#' Convert TIRT Pairwise-Response Matrix to Forced-Choice Data Strings
#'
#' @description
#' Converts a Thurstonian IRT pairwise-response matrix back to the
#' human-readable forced-choice strings used in \code{data}.  This is the
#' inverse of \code{\link{get.response.from.data.TIRT}}.
#'
#' @details
#' TIRT differs from the generic FC converters because the model is expressed
#' through pairwise comparisons.  The input \code{response} is therefore not a
#' block-level pattern-index matrix.  It is an \eqn{N \times P} binary matrix,
#' where each column is one observed item pair and each entry is coded:
#' \itemize{
#'   \item 1: the first item in that pair was preferred;
#'   \item 0: the second item in that pair was preferred.
#' }
#'
#' The number of response columns contributed by a block with \eqn{K} items is
#' determined by \code{fc.type}:
#' \describe{
#'   \item{RANK}{All \eqn{K(K-1)/2} unordered pairs are observed.  Pair columns
#'     are generated in the order \eqn{(1,2), (1,3), \dots, (K-1,K)} using the
#'     item labels in \code{block.items[[b]]}.}
#'   \item{MOLE}{Only the pairs needed to identify the most- and least-preferred
#'     items are retained.  The block contributes \eqn{2K - 3} response columns
#'     per person.}
#'   \item{PICK}{Only the pairs involving the picked item are retained.  The
#'     block contributes \eqn{K - 1} response columns per person.}
#' }
#'
#' For \code{"MOLE"} and \code{"PICK"}, the identities of the retained pairs
#' can vary by person.  The \code{pairs.value} object records those
#' person-specific pair definitions and is required to reconstruct data
#' strings without ambiguity.  It is normally copied directly from the
#' \code{pairs.value} component returned by
#' \code{\link{get.response.from.data.TIRT}}.
#'
#' TIRT conversion assumes consecutive item labels across blocks, for example
#' \code{list(1:2, 3:5)}.  This is the layout returned by
#' \code{\link{sim.data.TIRT}}.
#'
#' @param response An \eqn{N \times P} integer matrix of pairwise binary
#'   responses.  Entries must be 1 (first item in the pair preferred) or 0
#'   (second item in the pair preferred).
#' @param block.items A list of length \eqn{B} giving the consecutive item
#'   labels in each forced-choice block.
#' @param fc.type Character vector of length \eqn{B} (or a scalar recycled
#'   to all blocks): \code{"RANK"}, \code{"MOLE"}, or \code{"PICK"}.
#' @param pairs.value A list of length \eqn{N}, where each element is itself
#'   a list of length \eqn{B} giving the pair-definition matrix for each
#'   block.  Required when any block uses \code{"MOLE"} or \code{"PICK"}.
#'   For all-\code{"RANK"} data this argument may be omitted because the pair
#'   definitions are fixed by \code{block.items}.
#'
#' @return A data frame with \eqn{N} rows and \eqn{B} columns.  Column names
#'   are \code{"block.1"}, \code{"block.2"}, etc.  Cell values are character
#'   strings encoding the within-block ordering: full rankings for
#'   \code{"RANK"}, \code{"best>worst"} for \code{"MOLE"}, and single
#'   integers for \code{"PICK"}.
#'
#' @seealso
#' \code{\link{get.response.from.data.TIRT}} for the reverse conversion.
#'
#' \code{\link{get.data.from.response}} for the generic (non-TIRT) converter
#' used by FCGGUM and FCMIRT.
#'
#' @export
#'
#' @examples
#' # 2 RANK blocks: block 1 has items 1,2; block 2 has items 3,4,5.
#' items <- list(1:2, 3:5)
#' resp <- cbind(
#'   c(1, 0),            # block 1 pair: 1 vs 2
#'   c(1, 0),            # block 2 pair: 3 vs 4
#'   c(1, 0),            # block 2 pair: 3 vs 5
#'   c(1, 0)             # block 2 pair: 4 vs 5
#' )
#' get.data.from.response.TIRT(resp, items, fc.type = "RANK")
get.data.from.response.TIRT <- function(response, block.items, fc.type = NULL,
                                        pairs.value = NULL) {
  response.matrix <- as.matrix(response)
  if (!is.numeric(response.matrix) ||
      any(!is.na(response.matrix) & !is.finite(response.matrix)) ||
      any(!is.na(response.matrix) & response.matrix != floor(response.matrix)) ||
      any(!is.na(response.matrix) & !response.matrix %in% c(0, 1))) {
    stop("'response' must contain only integer 0, 1, or NA values.",
         call. = FALSE)
  }
  N.block <- length(block.items)
  fc.type <- normalize_fc_type(fc.type, N.block)

  if (is.null(pairs.value)) {
    if (any(fc.type %in% c("MOLE", "PICK"))) {
      stop("'pairs.value' is required when 'fc.type' includes 'MOLE' or 'PICK'.", call. = FALSE)
    }
    pairs.value <- vector("list", nrow(response.matrix))
  }

  data <- cpp_get_data_from_response_tirt(
    response = matrix(as.integer(response.matrix), nrow = nrow(response.matrix),
                      ncol = ncol(response.matrix)),
    block_items = block.items,
    fc_type     = fc.type,
    pairs_value = pairs.value
  )

  data <- as.data.frame(data, stringsAsFactors = FALSE)
  for (b in seq_len(N.block)) {
    if (fc.type[b] == "PICK") {
      data[[b]] <- as.numeric(data[[b]])
    }
  }
  colnames(data) <- paste0("block.", seq_len(N.block))
  rownames(data) <- rownames(response.matrix)
  data
}

#' Convert TIRT Forced-Choice Data to Pairwise-Response Matrix
#'
#' @description
#' Parses human-readable forced-choice data strings (full rankings,
#' most-least choices, or best-only picks) into the Thurstonian IRT
#' pairwise binary response format.  Each unordered item pair within a
#' block receives a 1/0 outcome indicating which item was preferred.
#'
#' @details
#' This function expands each block column in \code{data} into one or more
#' pairwise response columns.  Pair columns are grouped by block in the same
#' order as \code{block.items}.  Within a pair column, the coding is always
#' 1 if the first item in the pair is preferred, and 0 if the second item is
#' preferred.
#'
#' Item labels must match the values supplied in \code{block.items}; they do
#' not need to be consecutive. For example, if
#' \code{block.items = list(c(2, 7), c(10, 20, 30))}, the second block can
#' contain \code{"10>20>30"}, \code{"30>10"}, or \code{"20"}, depending on
#' \code{fc.type}.
#'
#' @section Pattern Naming and Pair Encoding:
#' For a block with \eqn{K} items, the number of pair columns depends on
#' \code{fc.type}: \eqn{K(K-1)/2} for \code{"RANK"}, \eqn{2K - 3} for
#' \code{"MOLE"}, and \eqn{K - 1} for \code{"PICK"}.  For each retained pair:
#' \itemize{
#'   \item \strong{RANK}: the item ranked earlier in the ordering is
#'         preferred.  For \code{block.items[[b]] = 3:5}, pair order is
#'         \code{(3,4)}, \code{(3,5)}, \code{(4,5)}.  The string
#'         \code{"3>4>5"} therefore produces pairwise responses
#'         \code{1, 1, 1}.
#'   \item \strong{MOLE}: only the most- and least-preferred items are
#'         identified (e.g. \code{"3>2"} in a 3-item block).  The
#'         remaining items are treated as tied in the middle, producing
#'         preference relations consistent with the partial ordering.
#'   \item \strong{PICK}: only the most-preferred item is identified
#'         (e.g. \code{"3"} in a 3-item block).  All other items are
#'         treated as equally less preferred.
#' }
#' The \code{pairs.value} component records the actual pair matrices used for
#' each person and block.  It is essential for converting MOLE/PICK responses
#' back to \code{data}, because the retained pair set can differ across
#' persons.
#'
#' @param data An \eqn{N \times B} character matrix or data frame of
#'   forced-choice responses.  Each cell encodes the within-block item
#'   ordering using item labels from \code{block.items} separated by \code{">"}
#'   (e.g. \code{"3>1>2"} for RANK, \code{"3>2"} for MOLE, \code{"3"}
#'   for PICK).
#' @param block.items A list of length \eqn{B} giving the unique item labels
#'   in each forced-choice block.
#' @param fc.type Character vector of length \eqn{B} (or a scalar
#'   recycled to all blocks): \code{"RANK"}, \code{"MOLE"}, or
#'   \code{"PICK"}.
#'
#' @return A list with two components:
#' \describe{
#'   \item{response}{An \eqn{N \times P} integer matrix of pairwise
#'     binary outcomes (1/0).  For block \eqn{b}, the number of contributed
#'     columns is \eqn{K_b(K_b-1)/2} for RANK, \eqn{2K_b - 3} for MOLE, and
#'     \eqn{K_b - 1} for PICK.}
#'   \item{pairs.value}{A list of length \eqn{N}, each element being
#'     a list of length \eqn{B} of integer pair-definition matrices.
#'     Required as input for
#'     \code{\link{get.data.from.response.TIRT}}.}
#' }
#'
#' @seealso
#' \code{\link{get.data.from.response.TIRT}} for the reverse conversion.
#'
#' \code{\link{get.response.from.data}} for the generic (non-TIRT)
#' converter used by FCGGUM and FCMIRT.
#'
#' @export
#'
#' @examples
#' # 2 persons, 2 RANK blocks.
#' dat <- data.frame(
#'   block.1 = c("1>2", "2>1"),
#'   block.2 = c("3>4>5", "5>4>3"),
#'   stringsAsFactors = FALSE
#' )
#' items <- list(1:2, 3:5)
#' result <- get.response.from.data.TIRT(dat, items, fc.type = "RANK")
#' result$response
#' result$pairs.value[[1]]
#' get.data.from.response.TIRT(
#'   result$response, items, fc.type = "RANK",
#'   pairs.value = result$pairs.value
#' )
#'
#' # Mixed partial-response blocks.
#' dat.partial <- data.frame(
#'   mole = c("1>3", "2>1"),
#'   pick = c("4", "5"),
#'   stringsAsFactors = FALSE
#' )
#' items.partial <- list(1:3, 4:6)
#' partial <- get.response.from.data.TIRT(
#'   dat.partial, items.partial, fc.type = c("MOLE", "PICK")
#' )
#' partial$response
#' get.data.from.response.TIRT(
#'   partial$response, items.partial, fc.type = c("MOLE", "PICK"),
#'   pairs.value = partial$pairs.value
#' )
get.response.from.data.TIRT <- function(data, block.items, fc.type = NULL) {
  N.block <- ncol(data)
  fc.type <- normalize_fc_type(fc.type, N.block)

  result <- cpp_get_response_from_data_tirt(
    data        = as.matrix(data),
    block_items = block.items,
    fc_type     = fc.type
  )

  response <- result$response
  colnames(response) <- paste0("pairs", seq_len(ncol(response)))
  rownames(response) <- rownames(data)
  result$response <- response
  result
}


# ---- Generic FC response <-> data conversion (FCGGUM / FCMIRT) ----------------

#' Convert an Integer Response (Pattern-Index) Matrix to FC Data Strings
#'
#' @description
#' Converts an integer matrix of pattern indices back to human-readable
#' forced-choice data strings.  This is the generic converter used by
#' \strong{FCGGUM}, \strong{FCMIRT}, and \strong{FCGDINA} where each
#' column of the response matrix is an index into the block's observed
#' permutation-pattern list.
#'
#' @section Pattern Generation and Numbering:
#' For a block with \eqn{K} items, the set of possible response patterns
#' depends on the forced-choice type:
#' \describe{
#'   \item{RANK}{All \eqn{K!} full permutations of the \eqn{K} items
#'     are generated.  The patterns are listed in lexicographic order
#'     of the within-block item positions.  For example, a 3-item
#'     block with items \code{c(2,5,8)} produces \eqn{3! = 6} patterns:
#'     \code{(2,5,8)}, \code{(2,8,5)}, \code{(5,2,8)}, \code{(5,8,2)},
#'     \code{(8,2,5)}, \code{(8,5,2)}.  Pattern index 1 means the
#'     first pattern was observed.}
#'   \item{MOLE}{For a \eqn{K}-item block, \eqn{K(K-1)} most-least
#'     observation patterns are possible: \eqn{K} choices for the most
#'     preferred item times \eqn{K-1} choices for the least preferred
#'     item.  Pattern \eqn{(a,b)} means item \eqn{a} was most
#'     preferred and item \eqn{b} was least preferred.}
#'   \item{PICK}{For a \eqn{K}-item block, exactly \eqn{K} patterns
#'     exist: one for each possible most-preferred item.}
#' }
#' In all cases the pattern index (an integer from 1 to the number of
#' observed patterns) is the row number in the matrix returned by
#' the internal permutation helper for that block. The same deterministic
#' pattern construction is exposed by
#' \code{\link{generate_fc_permutation_patterns}}.
#'
#' This format is a compact block-level encoding.  One response column equals
#' one forced-choice block, even when the block contains more than two items.
#' This is the representation expected by the generic forced-choice model
#' families FCGGUM, FCMIRT, and FCGDINA.  It should not be confused with the
#' TIRT pairwise format, where one block usually expands to several pair
#' columns.
#'
#' @param response An \eqn{N \times B} integer matrix of pattern
#'   indices.  Each entry is an integer between 1 and the number of
#'   observed patterns for that block.
#' @param block.items A list of length \eqn{B} giving the global item
#'   indices in each forced-choice block.
#' @param fc.type Character vector of length \eqn{B} (or a scalar
#'   recycled to all blocks): \code{"RANK"}, \code{"MOLE"}, or
#'   \code{"PICK"}.
#'
#' @return A data frame with \eqn{N} rows and \eqn{B} columns.  Column
#'   names are \code{"block.1"}, \dots, \code{"block.B"}.  Cell values
#'   are character strings: full rankings (\code{"2>5>8"}) for
#'   \code{"RANK"}, \code{"best>worst"} pairs for \code{"MOLE"}, and
#'   single integers for \code{"PICK"}.
#'
#' @seealso
#' \code{\link{get.response.from.data}} for the reverse conversion.
#'
#' \code{\link{generate_fc_permutation_patterns}} for pattern generation.
#'
#' \code{\link{get.data.from.response.TIRT}} for the TIRT-specific
#' converter.
#'
#' @export
#'
#' @examples
#' # 2 persons, 2 RANK blocks.
#' items <- list(1:2, 3:5)
#' resp <- cbind(c(1, 2), c(1, 6))
#' get.data.from.response(resp, items, fc.type = "RANK")
#'
#' # Mixed block formats: first block is most-least, second block is pick.
#' mixed.items <- list(1:3, 4:6)
#' mixed.resp <- cbind(c(1, 6), c(1, 3))
#' get.data.from.response(
#'   mixed.resp, mixed.items, fc.type = c("MOLE", "PICK")
#' )
get.data.from.response <- function(response, block.items, fc.type = "RANK") {
  N.person <- nrow(response)
  N.block  <- length(block.items)
  fc.type  <- normalize_fc_type(fc.type, N.block)

  patterns <- vector("list", N.block)
  for (b in seq_len(N.block)) {
    patterns[[b]] <- get_permutations(
      block.items[[b]],
      switch(fc.type[b],
             RANK = length(block.items[[b]]),
             MOLE = 2,
             PICK = 1))
  }

  data <- matrix(NA_character_, N.person, N.block)
  for (b in seq_len(N.block)) {
    data[, b] <- cpp_data_from_response_block(as.integer(response[, b]),
                                              patterns[[b]])
  }

  data <- as.data.frame(data, stringsAsFactors = FALSE)
  colnames(data) <- paste0("block.", seq_len(N.block))
  rownames(data) <- rownames(response)
  data
}

#' Convert FC Data Strings to an Integer Response (Pattern-Index) Matrix
#'
#' @description
#' Parses human-readable forced-choice data strings into integer pattern
#' indices.  This is the generic converter used by \strong{FCGGUM},
#' \strong{FCMIRT}, and \strong{FCGDINA}.  Each data cell is matched
#' against the full set of permutation/observation patterns for that
#' block, and the resulting row index is stored in the response matrix.
#'
#' @section Pattern Matching:
#' The conversion first generates all possible observation patterns for
#' each block via the internal permutation helper:
#' \itemize{
#'   \item \strong{RANK} (\eqn{K} items): all \eqn{K!} full rankings,
#'     ordered lexicographically by within-block position.
#'   \item \strong{MOLE}: \eqn{K(K-1)} most-least observation patterns.
#'   \item \strong{PICK}: \eqn{K} best-only patterns.
#' }
#' Each data cell string is parsed into an integer vector of global item
#' indices, which is then matched (as an ordered sequence) against the
#' pattern matrix.  The 1-based row index of the match becomes the
#' response value.  Pattern ordering is deterministic and reproducible.
#'
#' A non-missing cell that cannot be matched to a valid pattern is returned as
#' \code{NA_integer_}.  This is useful for data-screening workflows: after
#' conversion, check \code{anyNA(response)} before passing the response matrix
#' to a fitting function.
#'
#' @param data An \eqn{N \times B} character matrix or data frame.  Cells
#'   encode within-block item orderings as global indices separated by
#'   \code{">"} (e.g. \code{"3>1>2"} for RANK, \code{"3>2"} for MOLE,
#'   \code{"3"} for PICK).
#' @param block.items A list of length \eqn{B} giving the global item
#'   indices in each forced-choice block.
#' @param fc.type Character vector of length \eqn{B} (or a scalar
#'   recycled to all blocks): \code{"RANK"}, \code{"MOLE"}, or
#'   \code{"PICK"}.
#'
#' @return An \eqn{N \times B} integer matrix.  Each entry is a 1-based
#'   pattern index (row number in the block's pattern matrix).  Column
#'   names are \code{"block.1"}, \dots, \code{"block.B"}.
#'
#' @seealso
#' \code{\link{get.data.from.response}} for the reverse conversion.
#'
#' \code{\link{generate_fc_permutation_patterns}} for pattern generation
#' details.
#'
#' \code{\link{get.response.from.data.TIRT}} for the TIRT-specific
#' converter.
#'
#' @export
#'
#' @examples
#' # 2 persons, 2 RANK blocks.
#' items <- list(1:2, 3:5)
#' dat <- data.frame(
#'   block.1 = c("1>2", "2>1"),
#'   block.2 = c("3>4>5", "5>4>3"),
#'   stringsAsFactors = FALSE
#' )
#' resp <- get.response.from.data(dat, items, fc.type = "RANK")
#' resp
#' get.data.from.response(resp, items, fc.type = "RANK")
#'
#' # Mixed block formats: most-least plus pick.
#' dat.mixed <- data.frame(
#'   mole = c("1>3", "3>2"),
#'   pick = c("4", "6"),
#'   stringsAsFactors = FALSE
#' )
#' mixed.items <- list(1:3, 4:6)
#' get.response.from.data(
#'   dat.mixed, mixed.items, fc.type = c("MOLE", "PICK")
#' )
get.response.from.data <- function(data, block.items, fc.type = "RANK") {
  N.block <- length(block.items)
  fc.type <- normalize_fc_type(fc.type, N.block)

  patterns <- vector("list", N.block)
  for (b in seq_len(N.block)) {
    patterns[[b]] <- get_permutations(
      block.items[[b]],
      switch(fc.type[b],
             RANK = length(block.items[[b]]),
             MOLE = 2,
             PICK = 1))
  }

  response <- matrix(NA_integer_, nrow(data), N.block)
  for (b in seq_len(N.block)) {
    response[, b] <- cpp_response_from_data_block(as.character(data[, b]),
                                                  patterns[[b]])
  }

  colnames(response) <- paste0("block.", seq_len(N.block))
  rownames(response) <- rownames(data)
  response
}


# ---- FCDCM response <-> data conversion ---------------------------------------

#' Convert an FCDCM Response Matrix to FC Data Strings
#'
#' @description
#' Converts a binary FCDCM response matrix (0/1 or 1/2 coding) back to
#' human-readable \code{"a>b"} / \code{"b>a"} preference strings.  Each
#' block compares exactly two items; the response indicates which item
#' was selected.
#'
#' @section Response Coding:
#' Two coding conventions are accepted:
#' \itemize{
#'   \item \strong{0/1 coding}: 1 means the first-named item in
#'         \code{block.items} was preferred; 0 means the second-named
#'         item was preferred.
#'   \item \strong{1/2 coding}: 1 means the first-named item; 2 means
#'         the second-named item.  This is auto-detected and converted.
#' }
#' The output always uses \code{"a>b"} format where \code{a} and
#' \code{b} are the two item indices in the comparison order.
#'
#' The direction is controlled by \code{block.items}.  If
#' \code{block.items[[1]] = c(1L, 3L)}, then response value 1 becomes
#' \code{"1>3"} and response value 0 becomes \code{"3>1"}.  With 1/2 coding,
#' value 1 has the same meaning as above and value 2 means the second item.
#' A response matrix must use one coding convention consistently; mixed
#' \code{0/1/2} values are rejected.
#'
#' @param response An \eqn{N \times B} integer matrix coded as 0/1 or
#'   1/2, where \eqn{B} is the number of FCDCM blocks.
#' @param block.items A list of length \eqn{B}.  Each element is an
#'   integer vector of length 2 giving the two global statement indices
#'   compared in that block.
#'
#' @return A data frame with \eqn{N} rows and \eqn{B} columns.  Each
#'   cell is a character string \code{"a>b"} or \code{"b>a"}.
#'
#' @seealso
#' \code{\link{get.response.from.data.FCDCM}} for the reverse
#' conversion.
#'
#' \code{\link{get.data.from.response}} for the generic multi-item
#' converter.
#'
#' @export
#'
#' @examples
#' items <- list(c(1L, 3L), c(2L, 4L))
#' resp <- cbind(c(1, 0, 1), c(0, 1, 1))
#' dat <- get.data.from.response.FCDCM(resp, items)
#' dat
#' get.response.from.data.FCDCM(dat, items)$response
#'
#' # The same preferences can also be supplied as 1/2 responses.
#' resp12 <- cbind(c(1, 2, 1), c(2, 1, 1))
#' get.data.from.response.FCDCM(resp12, items)
get.data.from.response.FCDCM <- function(response, block.items) {
  response.matrix <- as.matrix(response)
  N.person <- nrow(response.matrix)
  N.block  <- ncol(response.matrix)

  if (is.null(N.person) || is.null(N.block) ||
      N.person < 1L || N.block < 1L) {
    stop("'response' must be an N x B response matrix.", call. = FALSE)
  }
  if (!is.list(block.items) || length(block.items) != N.block) {
    stop("'block.items' must be a list with one two-statement entry per block.",
         call. = FALSE)
  }
  block.items <- lapply(block.items, as.integer)
  if (any(vapply(block.items, length, integer(1L)) != 2L)) {
    stop("Each FCDCM block must contain exactly two statement indices.",
         call. = FALSE)
  }
  block.item.values <- unlist(block.items, use.names = FALSE)
  if (anyNA(block.item.values) || any(block.item.values < 1L)) {
    stop("FCDCM statement indices in 'block.items' must be positive integers.",
         call. = FALSE)
  }

  response01 <- matrix(as.integer(response.matrix),
                       nrow = N.person, ncol = N.block,
                       dimnames = dimnames(response.matrix))
  vals <- sort(unique(as.vector(response01)))
  vals <- vals[!is.na(vals)]

  if (all(vals %in% c(0L, 1L))) {
    # FCDCM-native coding: 1 = first, 0 = second.
  } else if (all(vals %in% c(1L, 2L))) {
    response01 <- matrix(as.integer(response01 == 1L),
                         nrow = N.person, ncol = N.block,
                         dimnames = dimnames(response.matrix))
  } else {
    stop("FCDCM responses must be coded as 0/1 or 1/2.", call. = FALSE)
  }
  if (anyNA(response01)) {
    stop("FCDCM responses must not contain missing values.", call. = FALSE)
  }

  data <- matrix(NA_character_, N.person, N.block,
                 dimnames = dimnames(response.matrix))
  for (b in seq_len(N.block)) {
    first  <- paste0(block.items[[b]][1L], ">", block.items[[b]][2L])
    second <- paste0(block.items[[b]][2L], ">", block.items[[b]][1L])
    data[, b] <- ifelse(response01[, b] == 1L, first, second)
  }

  data <- as.data.frame(data, stringsAsFactors = FALSE)
  if (is.null(colnames(response.matrix))) {
    colnames(data) <- paste0("block.", seq_len(N.block))
  }
  rownames(data) <- rownames(response.matrix)
  data
}

#' Convert FCDCM Data Strings to a Binary Response Matrix
#'
#' @description
#' Parses FCDCM preference strings (\code{"a>b"} / \code{"b>a"}) into a
#' 0/1 binary response matrix.  Each block compares exactly two items.
#'
#' @section Response Coding:
#' The output uses 0/1 coding: 1 indicates preference for the
#' first-named item in \code{block.items[[b]]}; 0 indicates preference
#' for the second-named item.  This coding is the native format expected
#' by the FCDCM model estimation functions.
#'
#' @details
#' For each block, only two strings are valid: \code{"a>b"} and \code{"b>a"},
#' where \code{a = block.items[[b]][1]} and \code{b = block.items[[b]][2]}.
#' Leading and trailing whitespace is ignored.  Missing values or strings that
#' do not match the block-specific item pair produce an error, because FCDCM
#' fitting requires a complete binary response matrix.
#'
#' If \code{block.items} was obtained from
#' \code{\link{get.block.items.from.data.FCDCM}}, the pair is sorted in
#' ascending order.  In that common workflow, response value 1 means the
#' smaller item index was preferred and response value 0 means the larger item
#' index was preferred.
#'
#' @param data An \eqn{N \times B} character matrix or data frame.  Each
#'   cell must be of the form \code{"a>b"} or \code{"b>a"} where
#'   \code{a} and \code{b} are the two item indices for that block.
#' @param block.items A list of length \eqn{B}.  Each element is an
#'   integer vector of length 2 giving the two global statement indices
#'   compared in that block, in the order used for the 0/1 coding.
#'
#' @return A named list with component \code{response}, an \eqn{N \times B}
#'   integer matrix coded 0/1.
#'
#' @seealso
#' \code{\link{get.data.from.response.FCDCM}} for the reverse conversion.
#'
#' \code{\link{get.response.from.data}} for the generic multi-item
#' converter.
#'
#' @export
#'
#' @examples
#' items <- list(c(1L, 3L), c(2L, 4L))
#' dat <- data.frame(
#'   b1 = c("1>3", "3>1", "1>3"),
#'   b2 = c("2>4", "2>4", "4>2"),
#'   stringsAsFactors = FALSE
#' )
#' get.response.from.data.FCDCM(dat, items)
#' get.data.from.response.FCDCM(
#'   get.response.from.data.FCDCM(dat, items)$response, items
#' )
get.response.from.data.FCDCM <- function(data, block.items) {
  data.matrix <- as.matrix(data)
  N.person <- nrow(data.matrix)
  N.block  <- ncol(data.matrix)

  if (is.null(N.person) || is.null(N.block) ||
      N.person < 1L || N.block < 1L) {
    stop("'data' must be an N x B forced-choice data matrix.", call. = FALSE)
  }
  if (!is.list(block.items) || length(block.items) != N.block) {
    stop("'block.items' must be a list with one two-statement entry per block.",
         call. = FALSE)
  }
  block.items <- lapply(block.items, as.integer)
  if (any(vapply(block.items, length, integer(1L)) != 2L)) {
    stop("Each FCDCM block must contain exactly two statement indices.",
         call. = FALSE)
  }
  block.item.values <- unlist(block.items, use.names = FALSE)
  if (anyNA(block.item.values) || any(block.item.values < 1L)) {
    stop("FCDCM statement indices in 'block.items' must be positive integers.",
         call. = FALSE)
  }

  response <- matrix(NA_integer_, N.person, N.block,
                     dimnames = dimnames(data.matrix))
  for (b in seq_len(N.block)) {
    first  <- paste0(block.items[[b]][1L], ">", block.items[[b]][2L])
    second <- paste0(block.items[[b]][2L], ">", block.items[[b]][1L])
    x <- trimws(as.character(data.matrix[, b]))
    response[x == first,  b] <- 1L
    response[x == second, b] <- 0L
  }
  if (anyNA(response)) {
    stop("FCDCM data must contain only block-specific strings like '1>3' ",
         "or '3>1'.", call. = FALSE)
  }

  if (is.null(colnames(data.matrix))) {
    colnames(response) <- paste0("block.", seq_len(N.block))
  }
  rownames(response) <- rownames(data.matrix)
  list(response = response)
}


# ---- DCM alpha distribution ---------------------------------------
distribution_higher_order <- function(theta, delta1, delta0) {
  theta <- as.numeric(theta)
  delta1 <- as.numeric(delta1)
  delta0 <- as.numeric(delta0)

  N <- length(theta)
  D <- length(delta1)
  eta <- outer(theta, delta1) -
    matrix(delta1 * delta0, N, D, byrow = TRUE)

  prob.alpha <- plogis(eta)
  return(prob.alpha)
}

distribution_multi_normal <- function(N, Corr, threshold){
  D <- length(threshold)
  theta <- MASS::mvrnorm(n = N, mu = rep(0, D), Sigma = Corr)

  alpha <- (theta > matrix(qnorm(threshold), N, D, byrow = TRUE)) * 1L
  return(alpha)
}

distribution_uniform <- function(N, patterns){
  L <- nrow(patterns)
  alpha <- patterns[sample(1:L, N, replace = TRUE), ]
  return(alpha)
}

#' @describeIn logLik-class FCDCM model: higher-order trait quadrature with exact
#'   marginalization over discrete attribute profiles.
#' @export
logLik.FCDCM <- function(object, theta.low = NULL, theta.up = NULL, L = NULL,
                         ...) {
  response <- object$response
  N <- nrow(response)
  B <- ncol(response)
  df <- object$npar

  grid.control <- istem_latent_grid_control(
    object$arguments$control.model, 1L, L = L,
    theta.low = theta.low, theta.up = theta.up
  )
  L <- grid.control$L
  theta.low <- grid.control$theta.lower
  theta.up <- grid.control$theta.upper

  theta.norm <- matrix(seq(theta.low, theta.up, length.out = L), ncol = 1L)
  colnames(theta.norm) <- "theta"
  pi <- stats::dnorm(theta.norm[, 1L])
  pi <- pi / sum(pi)

  delta <- object$delta$est
  support <- fcdcm_latent_support(
    theta = theta.norm,
    pi = pi,
    delta1 = delta[, "delta1"],
    delta0 = delta[, "delta0"],
    par = object$par$est,
    alpha.patterns = object$alpha.patterns,
    zeta.patterns = object$zeta.patterns,
    patterns = object$patterns
  )

  response.group <- istem_response_groups(response)
  lik <- cpp_loglik_binary(
    prob = support$prob,
    response = matrix(as.integer(response.group$response),
                      nrow = response.group$G, ncol = B),
    pi = support$pi
  )
  lik <- loglik_apply_group_count(lik, response.group$count)
  results <- c(`log Likelihood` = lik$logLik)
  results <- set_loglik_attributes(results, N, df, lik, theta.norm,
                                    support$prob, matrix(support$pi, ncol = 1L),
                                    L, theta.low, theta.up,
                                    pi.theta = matrix(pi, ncol = 1L),
                                    support.theta = support$theta,
                                    support.alpha = support$alpha,
                                    class.prob = support$class.prob,
                                    response.unique = response.group$response,
                                    response.count = response.group$count,
                                    response.group = response.group$group)
  results
}

#' @describeIn get.fit.index FCGDINA model: forced-choice CDM with nominal
#'   binary expansion; within-block pairs excluded.
#' @export
get.fit.index.FCGDINA <- function(object, ...) {
  response <- object$response
  block.items <- object$block.items
  patterns <- object$patterns

  prep <- fcgdina_prepare(
    data = response,
    Q.matrix = object$Q.matrix,
    model = object$model,
    block.items = block.items,
    fc.type = object$fc.type
  )

  N <- nrow(response)
  B <- length(block.items)
  npar <- object$npar
  logLik.obj <- object$logLik
  pi <- as.vector(attr(logLik.obj, "pi"))

  n_cat <- vapply(patterns, nrow, integer(1L))
  n_bin <- sum(n_cat - 1L)
  response.bin <- matrix(0L, N, n_bin)
  indicator.block <- integer(n_bin)

  idx.resp <- 0L
  for (b in seq_len(B)) {
    Kb <- n_cat[b]
    if (Kb > 1L) {
      for (j in seq_len(Kb - 1L)) {
        response.bin[, idx.resp + j] <- as.integer(response[, b] == j)
        indicator.block[idx.resp + j] <- b
      }
      idx.resp <- idx.resp + (Kb - 1L)
    }
  }

  par.vec <- unlist(object$delta$est, use.names = FALSE)

  expand.free.par <- function(par.vec, ...) {
    delta.lst <- vector("list", prep$I.states)
    off <- 0L
    for (i in seq_len(prep$I.states)) {
      nc <- prep$delta.len[i]
      delta.lst[[i]] <- par.vec[(off + 1L):(off + nc)]
      off <- off + nc
    }
    delta.lst
  }

  prob.full.fun <- function(par.vec, ...) {
    delta.lst <- expand.free.par(par.vec)
    fcgdina_prob_class_link(prep, delta.lst, link = fcgdina_delta_link(object))
  }

  loglik.fun <- function(par.vec, ...) {
    prob.full <- prob.full.fun(par.vec)
    cpp_fcgdina_marginal_loglik(
      prob.full, prep$response, prep$patterns, pi,
      prep$N, prep$B, prep$C)
  }

  prob.fun <- function(par.vec, ...) {
    prob.full <- prob.full.fun(par.vec)
    prob.out <- matrix(0, nrow(prob.full), n_bin)

    idx.full <- 0L
    idx.bin <- 0L
    for (b in seq_len(B)) {
      Kb <- n_cat[b]
      if (Kb > 1L) {
        prob.out[, (idx.bin + 1L):(idx.bin + Kb - 1L)] <-
          prob.full[, (idx.full + 1L):(idx.full + Kb - 1L)]
        idx.bin <- idx.bin + (Kb - 1L)
      }
      idx.full <- idx.full + Kb
    }
    prob.out
  }

  good.of.fit(
    par.vec = par.vec,
    loglik.fun = loglik.fun,
    prob.fun = prob.fun,
    response = response.bin,
    npar = npar,
    pi = pi,
    response.type = "nominal",
    nominal.groups = indicator.block,
    ...
  )
}

#' Fit MIRT Model via iStEM
#'
#' iStEM uses the same default item-prior family as the Stan backend in the
#' item MAP update: \code{a ~ lognormal(a.mu, a.sigma)},
#' \code{b ~ normal(b.mu, b.sigma)}, \code{c ~ uniform(c.mu, c.sigma)}, and
#' \code{d ~ uniform(d.mu, d.sigma)}. Set \code{use.prior = FALSE} in
#' \code{control.model} to use the previous unpenalized item update.
#'
#' @noRd
fit.MIRT.iStEM <- function(response, model = "m2pl", D = 2, Q.matrix = NULL,
                           control.model = NULL, control.method = NULL,
                           .call = NULL) {

  call <- if (is.null(.call)) match.call() else .call
  control.model <- fc_as_control_list(control.model, "control.model")
  control.method <- fc_as_control_list(control.method, "control.method")
  common.method <- fit_common_method_control(control.method)
  vis <- common.method$vis
  set.seed(common.method$seed)

  response <- as.matrix(response)
  if (anyNA(response) || !all(response %in% c(0, 1))) {
    stop("'response' must be an N x I matrix of binary responses (0/1).", call. = FALSE)
  }
  storage.mode(response) <- "integer"

  model <- tolower(model)
  model_type <- model_type_to_int(model)

  N <- nrow(response)
  I <- ncol(response)
  response.group <- istem_response_groups(response)
  response.fit <- response.group$response
  count.fit <- response.group$count
  D <- as.integer(D)
  if (D < 1L) {
    stop("'D' must be a positive integer.", call. = FALSE)
  }

  q_info <- mirt_istem_prepare_q(I, D, Q.matrix, model)
  Q.matrix <- q_info$Q.matrix

  method <- istem_method_control(control.method)
  method <- istem_apply_grid_control(method, control.model, D)
  istem_check_method_control(method)
  control.model <- istem_effective_grid_model_control(control.model, method)
  M            <- method$M
  B            <- method$B
  burnin.maxitr <- method$burnin.maxitr
  maxitr       <- method$maxitr
  eps1         <- method$eps1
  eps2         <- method$eps2
  frac1        <- method$frac1
  frac2        <- method$frac2
  L            <- method$L
  theta_lower  <- method$theta.lower
  theta_upper  <- method$theta.upper
  optim_maxit  <- method$optim.maxit
  corr_optim_maxit <- method$corr.optim.maxit
  fix.corr     <- method$fix.corr
  estimate.se  <- method$estimate.se
  include_corr <- D > 1L && !fix.corr
  corr_df      <- free_corr_npar(D, include_corr)

  theta_mu_raw <- get_ctrl("theta.mu", rep(0, D), control.model)
  theta_mu <- array(theta_mu_raw, dim = D)
  prior <- mirt_istem_prior_control(control.model)

  bounds <- list(
    a.lower = get_ctrl("a.lower", 1e-4, control.method),
    a.upper = get_ctrl("a.upper", 6, control.method),
    b.lower = get_ctrl("b.lower", -6, control.method),
    b.upper = get_ctrl("b.upper", 6, control.method),
    c.lower = get_ctrl("c.lower", prior$c.lower, control.method),
    c.upper = get_ctrl("c.upper", prior$c.upper, control.method),
    d.lower = get_ctrl("d.lower", prior$d.lower, control.method),
    d.upper = get_ctrl("d.upper", prior$d.upper, control.method)
  )
  mirt_istem_check_bounds(bounds, model_type)

  istem.method <- c(
    list(
      M = M,
      B = B,
      burnin.maxitr = burnin.maxitr,
      maxitr = maxitr,
      eps1 = eps1,
      eps2 = eps2,
      frac1 = frac1,
      frac2 = frac2,
      L = L,
      theta.lower = theta_lower,
      theta.upper = theta_upper,
      optim.maxit = optim_maxit,
      corr.optim.maxit = corr_optim_maxit,
      fix.corr = fix.corr,
      estimate.se = estimate.se
    ),
    bounds
  )

  par <- mirt_istem_initial_par(response.fit, model_type, Q.matrix, bounds, prior)
  theta <- istem_random_theta(response.group$G, D, theta_mu,
                              theta_lower, theta_upper)
  Corr <- istem_random_corr(D, include_corr)

  state <- list(
    response = response.fit,
    response.count = count.fit,
    par = par,
    theta = theta,
    Corr = Corr,
    theta_mu = theta_mu,
    Q.matrix = Q.matrix,
    model_type = model_type,
    optim_maxit = optim_maxit,
    bounds = bounds,
    prior = prior
  )

  run <- istem_run(
    state = state,
    method = istem.method,
    N = N,
    vis = vis,
    label = "MIRT",
    update_parameters = function(state) {
      for (j in seq_len(ncol(state$response))) {
        state$par[j, ] <- mirt_istem_fit_item(
          y = state$response[, j],
          theta = state$theta,
          par_i = state$par[j, ],
          model_type = state$model_type,
          q_i = state$Q.matrix[j, ],
          weight = istem_state_weight(state),
          optim_maxit = state$optim_maxit,
          bounds = state$bounds,
          prior = state$prior
        )
      }
      state
    },
    param_vec = function(state) {
      mirt_istem_param_vec(state$par, state$Corr, state$model_type,
                           state$Q.matrix, include_corr)
    },
    sample_theta = function(state, theta_grid_length, theta_lower, theta_upper) {
      mirt_istem_sample_theta_batch(
        theta = state$theta,
        response = state$response,
        par = state$par,
        Corr = state$Corr,
        theta_mu = state$theta_mu,
        grid_length = theta_grid_length,
        lower = theta_lower,
        upper = theta_upper
      )
    },
    logLik_fun = function(state) {
      mirt_istem_loglik_trace(state, L, theta_lower, theta_upper)
    }
  )

  final <- mirt_istem_param_unpack(
    pv = run$chain.mean,
    pv_se = run$chain.sd,
    pv_rhat = NULL,
    I = I,
    D = D,
    model_type = model_type,
    Q.matrix = Q.matrix,
    Corr = run$state$Corr,
    include_corr = include_corr
  )

  theta.est <- istem_expand_group_matrix(
    run$theta.est, response.group$group, rownames(response))
  theta.se <- istem_expand_group_matrix(
    run$theta.se, response.group$group, rownames(response))
  theta <- istem_named_theta(theta.est, theta.se, response, D)

  par <- final$par
  par.se <- final$par.se
  par.Rhat <- final$par.rhat
  par.free <- mirt_par_free_mask(model_type, Q.matrix)
  colnames(par) <- colnames(par.se) <- colnames(par.Rhat) <- colnames(par.free) <-
    c(paste0("a", seq_len(D)), "b", "c", "d")
  rownames(par) <- rownames(par.se) <- rownames(par.Rhat) <- rownames(par.free) <-
    colnames(response)
  par.se[!par.free] <- NA_real_
  par.Rhat[!par.free] <- NA_real_
  npar <- sum(par.free) + corr_df

  Corr.Rhat <- final$Corr.rhat
  dimnames(Corr.Rhat) <- list(paste0("Dim.", seq_len(D)), paste0("Dim.", seq_len(D)))
  Corr <- list(est = final$Corr, se = final$Corr.se, Rhat = Corr.Rhat)

  results <- list(
    npar = npar,
    method = "iStEM",
    theta = theta,
    par = list(est = par, se = par.se, Rhat = par.Rhat, free = par.free),
    Corr = Corr,
    stan.obj = NULL,
    MCMC.obj = NULL,
    Q.matrix = Q.matrix,
    call = call,
    arguments = list(
      response = response,
      model = model,
      D = D,
      Q.matrix = Q.matrix,
      method = "iStEM",
      cores = common.method$cores,
      vis = vis,
      seed = common.method$seed,
      control.model = control.model,
      control.method = fit_effective_method_control(
        control.method, method = istem.method, common = common.method
      )
    ),
    iStEM = utils::modifyList(run$iStEM, list(prior = prior))
  )

  class(results) <- "MIRT"
  results$logLik <- logLik.MIRT(results, L = method$L,
                                theta.low = method$theta.lower,
                                theta.up = method$theta.upper)

  results
}

mirt_istem_prepare_q <- function(I, D, Q.matrix, model) {
  if (!is.null(Q.matrix)) {
    Q.matrix <- as.matrix(Q.matrix)
    if (ncol(Q.matrix) != D || nrow(Q.matrix) != I) {
      Q.matrix <- NULL
    }
  }

  if (is.null(Q.matrix)) {
    Q.matrix <- matrix(1, I, D)
    if (D > 1L) {
      for (k in seq_len(D)) {
        r <- I - D + k
        for (j in seq_len(D)) {
          if (k > D - j + 1L) {
            Q.matrix[r, j] <- 0
          }
        }
      }
    }
  }

  list(Q.matrix = Q.matrix)
}

mirt_istem_prior_control <- function(control.model) {
  use_prior <- get_ctrl("use.prior", TRUE, control.model)
  if (!is.logical(use_prior) || length(use_prior) != 1L || is.na(use_prior)) {
    stop("'use.prior' in 'control.model' must be TRUE or FALSE.",
         call. = FALSE)
  }

  a_mu <- istem_scalar(get_ctrl("a.mu", 0.25, control.model), "a.mu")
  a_sigma <- istem_scalar(
    get_ctrl("a.sigma", 0.25, control.model),
    "a.sigma",
    positive = TRUE
  )
  b_mu <- istem_scalar(get_ctrl("b.mu", 0.0, control.model), "b.mu")
  b_sigma <- istem_scalar(
    get_ctrl("b.sigma", 1.0, control.model),
    "b.sigma",
    positive = TRUE
  )
  c_lower <- istem_scalar(get_ctrl("c.mu", 0.0, control.model), "c.mu")
  c_upper <- istem_scalar(get_ctrl("c.sigma", 0.35, control.model), "c.sigma")
  d_lower <- istem_scalar(get_ctrl("d.mu", 0.65, control.model), "d.mu")
  d_upper <- istem_scalar(get_ctrl("d.sigma", 1.0, control.model), "d.sigma")

  if (c_lower >= c_upper) {
    stop("'c.mu' must be smaller than 'c.sigma' for the c uniform support.",
         call. = FALSE)
  }
  if (d_lower >= d_upper) {
    stop("'d.mu' must be smaller than 'd.sigma' for the d uniform support.",
         call. = FALSE)
  }

  list(
    use.prior = use_prior,
    a.mu = a_mu,
    a.sigma = a_sigma,
    b.mu = b_mu,
    b.sigma = b_sigma,
    c.mu = c_lower,
    c.sigma = c_upper,
    d.mu = d_lower,
    d.sigma = d_upper,
    c.lower = c_lower,
    c.upper = c_upper,
    d.lower = d_lower,
    d.upper = d_upper
  )
}

mirt_istem_check_bounds <- function(bounds, model_type) {
  for (nm in names(bounds)) {
    if (!is.numeric(bounds[[nm]]) || length(bounds[[nm]]) != 1L ||
        !is.finite(bounds[[nm]])) {
      stop("'", nm, "' in 'control.method' must be a finite numeric scalar.",
           call. = FALSE)
    }
    bounds[[nm]] <- as.numeric(bounds[[nm]])
  }

  if (model_type >= 2L && bounds$a.lower <= 0) {
    stop("'a.lower' in 'control.method' must be positive for iStEM.",
         call. = FALSE)
  }
  if (bounds$a.lower >= bounds$a.upper) {
    stop("'a.lower' must be smaller than 'a.upper'.", call. = FALSE)
  }
  if (bounds$b.lower >= bounds$b.upper) {
    stop("'b.lower' must be smaller than 'b.upper'.", call. = FALSE)
  }
  if (model_type >= 3L && bounds$c.lower >= bounds$c.upper) {
    stop("'c.lower' must be smaller than 'c.upper'.", call. = FALSE)
  }
  if (model_type == 4L && bounds$d.lower >= bounds$d.upper) {
    stop("'d.lower' must be smaller than 'd.upper'.", call. = FALSE)
  }
  invisible(NULL)
}

mirt_istem_initial_par <- function(response, model_type, Q.matrix, bounds,
                                   prior) {
  I <- ncol(response)
  D <- ncol(Q.matrix)
  b <- istem_rnorm_bounded(I, prior$b.mu, prior$b.sigma,
                           bounds$b.lower, bounds$b.upper)

  if (model_type == 1L) {
    a <- matrix(1, I, D)
  } else {
    a <- matrix(0, I, D)
    a[Q.matrix == 1] <- istem_rlnorm_bounded(
      sum(Q.matrix == 1), prior$a.mu, prior$a.sigma,
      bounds$a.lower, bounds$a.upper
    )
  }

  c_par <- if (model_type >= 3L) {
    stats::runif(I, max(bounds$c.lower, prior$c.lower),
                 min(bounds$c.upper, prior$c.upper))
  } else {
    rep(0, I)
  }
  d_par <- if (model_type == 4L) {
    stats::runif(I, max(bounds$d.lower, prior$d.lower),
                 min(bounds$d.upper, prior$d.upper))
  } else {
    rep(1, I)
  }

  cbind(a, b, c_par, d_par)
}

mirt_istem_sample_theta_batch <- function(theta, response, par, Corr, theta_mu,
                                          grid_length, lower, upper) {
  chol <- istem_corr_chol(Corr)
  cpp_gibbs_mirt_theta(
    theta_ = theta,
    par_ = par,
    response_ = response,
    theta_mu_ = theta_mu,
    chol_corr_ = chol$chol,
    log_diag_sum = chol$log_diag_sum,
    step = grid_length,
    lower = lower,
    upper = upper
  )
}

mirt_istem_fit_item <- function(y, theta, par_i, model_type, q_i, weight,
                                optim_maxit, bounds, prior) {
  D <- ncol(theta)
  active <- q_i == 1

  if (model_type == 1L) {
    x0 <- par_i[D + 1L]
    lower <- bounds$b.lower
    upper <- bounds$b.upper
  } else {
    x0 <- c(par_i[seq_len(D)][active], par_i[D + 1L])
    lower <- c(rep(bounds$a.lower, sum(active)), bounds$b.lower)
    upper <- c(rep(bounds$a.upper, sum(active)), bounds$b.upper)

    if (model_type >= 3L) {
      x0 <- c(x0, par_i[D + 2L])
      lower <- c(lower, bounds$c.lower)
      upper <- c(upper, bounds$c.upper)
    }
    if (model_type == 4L) {
      x0 <- c(x0, par_i[D + 3L])
      lower <- c(lower, bounds$d.lower)
      upper <- c(upper, bounds$d.upper)
    }
  }

  obj <- function(x) {
    item <- mirt_istem_x_to_item(x, par_i, model_type, active, D)
    ll <- cpp_mirt_item_loglik_weighted(
      theta = theta,
      a = item$a,
      b = item$b,
      c = item$c,
      upper = item$d,
      response = as.integer(y),
      weight = weight
    )
    log_prior <- mirt_istem_item_log_prior(
      item = item,
      model_type = model_type,
      active = active,
      prior = prior
    )
    if (!is.finite(log_prior)) {
      return(Inf)
    }
    -ll - log_prior
  }

  opt <- tryCatch(
    stats::optim(
      par = x0,
      fn = obj,
      method = "L-BFGS-B",
      lower = lower,
      upper = upper,
      control = list(maxit = optim_maxit)
    ),
    error = function(e) NULL
  )

  if (is.null(opt) || !is.finite(opt$value)) {
    return(par_i)
  }

  item <- mirt_istem_x_to_item(opt$par, par_i, model_type, active, D)
  c(item$a, item$b, item$c, item$d)
}

mirt_istem_item_log_prior <- function(item, model_type, active, prior) {
  if (!isTRUE(prior$use.prior)) {
    return(0)
  }

  lp <- stats::dnorm(item$b, mean = prior$b.mu, sd = prior$b.sigma, log = TRUE)
  if (model_type >= 2L) {
    a_active <- item$a[active]
    if (any(a_active <= 0)) {
      return(-Inf)
    }
    lp <- lp + sum(stats::dlnorm(
      a_active,
      meanlog = prior$a.mu,
      sdlog = prior$a.sigma,
      log = TRUE
    ))
  }
  if (model_type >= 3L &&
      (item$c < prior$c.lower || item$c > prior$c.upper)) {
    return(-Inf)
  }
  if (model_type == 4L &&
      (item$d < prior$d.lower || item$d > prior$d.upper)) {
    return(-Inf)
  }

  lp
}

mirt_istem_x_to_item <- function(x, par_i, model_type, active, D) {
  idx <- 1L
  if (model_type == 1L) {
    a <- rep(1, D)
    b <- x[1L]
    c_par <- 0
    d_par <- 1
  } else {
    a <- rep(0, D)
    n_active <- sum(active)
    if (n_active > 0L) {
      a[active] <- x[idx:(idx + n_active - 1L)]
      idx <- idx + n_active
    }
    b <- x[idx]
    idx <- idx + 1L
    c_par <- if (model_type >= 3L) x[idx] else 0
    if (model_type >= 3L) {
      idx <- idx + 1L
    }
    d_par <- if (model_type == 4L) x[idx] else 1
  }
  list(a = a, b = b, c = c_par, d = d_par)
}

mirt_istem_param_vec <- function(par, Corr, model_type, Q.matrix, include_corr) {
  D <- ncol(Q.matrix)
  pv <- if (model_type == 1L) {
    par[, D + 1L]
  } else {
    c(par[, seq_len(D), drop = FALSE][Q.matrix == 1], par[, D + 1L])
  }

  if (model_type >= 3L) {
    pv <- c(pv, par[, D + 2L])
  }
  if (model_type == 4L) {
    pv <- c(pv, par[, D + 3L])
  }
  if (include_corr) {
    pv <- c(pv, Corr[lower.tri(Corr)])
  }
  pv
}

mirt_istem_param_unpack <- function(pv, pv_se, pv_rhat = NULL, I, D, model_type,
                                    Q.matrix, Corr, include_corr) {
  idx <- 1L
  par <- matrix(NA_real_, I, D + 3L)
  par.se <- matrix(NA_real_, I, D + 3L)
  par.rhat <- matrix(NA_real_, I, D + 3L)
  has_rhat <- !is.null(pv_rhat)

  if (model_type == 1L) {
    par[, seq_len(D)] <- 1
    par.se[, seq_len(D)] <- NA_real_
    par[, D + 1L] <- pv[idx:(idx + I - 1L)]
    par.se[, D + 1L] <- pv_se[idx:(idx + I - 1L)]
    if (has_rhat) par.rhat[, D + 1L] <- pv_rhat[idx:(idx + I - 1L)]
    idx <- idx + I
  } else {
    a <- matrix(0, I, D)
    a.se <- matrix(NA_real_, I, D)
    a.rhat <- matrix(NA_real_, I, D)
    n_active <- sum(Q.matrix == 1)
    a[Q.matrix == 1] <- pv[idx:(idx + n_active - 1L)]
    a.se[Q.matrix == 1] <- pv_se[idx:(idx + n_active - 1L)]
    if (has_rhat) a.rhat[Q.matrix == 1] <- pv_rhat[idx:(idx + n_active - 1L)]
    idx <- idx + n_active
    par[, seq_len(D)] <- a
    par.se[, seq_len(D)] <- a.se
    par.rhat[, seq_len(D)] <- a.rhat
    par[, D + 1L] <- pv[idx:(idx + I - 1L)]
    par.se[, D + 1L] <- pv_se[idx:(idx + I - 1L)]
    if (has_rhat) par.rhat[, D + 1L] <- pv_rhat[idx:(idx + I - 1L)]
    idx <- idx + I
  }

  if (model_type >= 3L) {
    par[, D + 2L] <- pv[idx:(idx + I - 1L)]
    par.se[, D + 2L] <- pv_se[idx:(idx + I - 1L)]
    if (has_rhat) par.rhat[, D + 2L] <- pv_rhat[idx:(idx + I - 1L)]
    idx <- idx + I
  } else {
    par[, D + 2L] <- 0
    par.se[, D + 2L] <- NA_real_
  }

  if (model_type == 4L) {
    par[, D + 3L] <- pv[idx:(idx + I - 1L)]
    par.se[, D + 3L] <- pv_se[idx:(idx + I - 1L)]
    if (has_rhat) par.rhat[, D + 3L] <- pv_rhat[idx:(idx + I - 1L)]
    idx <- idx + I
  } else {
    par[, D + 3L] <- 1
    par.se[, D + 3L] <- NA_real_
  }

  Corr.se <- matrix(NA_real_, D, D)
  if (D == 1L) {
    Corr <- matrix(1, 1, 1)
    Corr.se <- matrix(0, 1, 1)
  } else if (include_corr) {
    n_corr <- D * (D - 1L) / 2L
    corr_vals <- pv[idx:(idx + n_corr - 1L)]
    corr_se <- pv_se[idx:(idx + n_corr - 1L)]
    Corr <- diag(D)
    Corr[lower.tri(Corr)] <- corr_vals
    Corr <- t(Corr)
    Corr[lower.tri(Corr)] <- corr_vals
    Corr.se[lower.tri(Corr.se)] <- corr_se
    Corr.se <- t(Corr.se)
    Corr.se[lower.tri(Corr.se)] <- corr_se
    diag(Corr.se) <- 0
    Corr <- istem_safe_corr(Corr)
  } else {
    Corr.se[,] <- NA_real_
    diag(Corr.se) <- 0
  }
  Corr.rhat <- istem_unpack_corr_rhat(pv_rhat, idx, D, include_corr)

  list(par = par, par.se = par.se, par.rhat = par.rhat,
       Corr = Corr, Corr.se = Corr.se, Corr.rhat = Corr.rhat)
}

#' @describeIn logLik-class TIRT model: pairwise binary probit probabilities
#'   with multivariate normal quadrature over traits.
#' @export
logLik.TIRT <- function(object, theta.low = NULL, theta.up = NULL, L = NULL,
                        ...) {

  response <- object$response
  block.items <- object$arguments$block.items
  Q.matrix <- object$Q.matrix
  par <- object$par$est
  gamma.matrix <- object$gamma.matrix$est
  pairs.matrix <- object$pairs.matrix
  pairs.value  <- object$pairs.value
  fc.type      <- object$fc.type

  N.block <- length(block.items)
  N.person <- nrow(response)

  df <- object$npar

  D <- ncol(par) - 1
  grid.control <- istem_latent_grid_control(
    object$arguments$control.model, D, L = L,
    theta.low = theta.low, theta.up = theta.up
  )
  L <- grid.control$L
  theta.low <- grid.control$theta.lower
  theta.up <- grid.control$theta.upper
  I.states <- nrow(par)
  I.pairs <- ncol(response)
  lambda <- unlist(lapply(1:I.states, function(x) {
    d <- which(Q.matrix[x, ] != 0)[1]
    return(par[x, d])
  }))
  psi2 <- pmax(par[, D+1], .Machine$double.eps)

  theta.mu <- array(
    if (is.null(object$arguments$control.model$theta.mu)) rep(0, D)
    else object$arguments$control.model$theta.mu,
    dim = D
  )

  grid <- make_quadrature_grid_mvn(
    object$Corr$est, theta.mu, D, L, theta.low, theta.up
  )

  # Compute probabilities for all possible pairs
  prob <- model.TIRT(grid$theta.norm, lambda, psi2, gamma.matrix, Q.matrix, pairs.matrix)

  # Determine if we have person-specific pair data (MOLE / PICK)
  has.pairs.value <- !is.null(pairs.value) && !all(fc.type == "RANK")

  if (has.pairs.value) {
    # MOLE / PICK: use person-specific observed pairs
    I.block.vec <- sapply(block.items, length)
    block.pair.counts <- integer(N.block)
    for (b in 1:N.block) {
      k <- I.block.vec[b]
      if (fc.type[b] == "RANK") {
        block.pair.counts[b] <- choose(k, 2)
      } else if (fc.type[b] == "MOLE") {
        block.pair.counts[b] <- 2L * k - 3L
      } else if (fc.type[b] == "PICK") {
        block.pair.counts[b] <- k - 1L
      }
    }

    response.group <- istem_response_pair_groups(response, pairs.value)
    lik <- cpp_loglik_binary_person_pairs(
      prob = prob,
      response = matrix(as.integer(response.group$response),
                        nrow = response.group$G, ncol = I.pairs),
      pi = as.vector(grid$pi),
      pairs_matrix = pairs.matrix,
      pairs_value = response.group$pairs.value,
      block_pair_counts = block.pair.counts
    )
    lik <- loglik_apply_group_count(lik, response.group$count)
    results <- c(`log Likelihood` = lik$logLik)
    results <- set_loglik_attributes(results, N.person, df, lik, grid$theta.norm,
                                      prob, grid$pi, grid$L, theta.low, theta.up,
                                      response.unique = response.group$response,
                                      response.count = response.group$count,
                                      response.group = response.group$group)
    return(results)
  } else {
    response.group <- istem_response_groups(response)
    lik <- cpp_loglik_binary(
      prob = prob,
      response = matrix(as.integer(response.group$response),
                        nrow = response.group$G, ncol = I.pairs),
      pi = as.vector(grid$pi)
    )
    lik <- loglik_apply_group_count(lik, response.group$count)
    results <- c(`log Likelihood` = lik$logLik)
    results <- set_loglik_attributes(results, N.person, df, lik, grid$theta.norm,
                                      prob, grid$pi, grid$L, theta.low, theta.up,
                                      response.unique = response.group$response,
                                      response.count = response.group$count,
                                      response.group = response.group$group)
    return(results)
  }
}

#' Simulate Data from the Multidimensional IRT (MIRT) Model
#'
#' @description
#' Generates binary response data, latent trait vectors, and item parameters
#' from the multidimensional extension of the 1PL--4PL item response models.
#' The latent traits are drawn from a multivariate normal distribution with
#' a user-specified correlation matrix. Discrimination parameters respect a
#' Q-matrix structure and may be optionally rotated.
#'
#' @section Data Generation Process:
#'
#' \enumerate{
#'   \item \strong{Latent traits:}
#'         \eqn{\boldsymbol{\theta}_j \sim N_D(\mathbf{0}, \boldsymbol{\Sigma})},
#'         where \eqn{\boldsymbol{\Sigma}} is the \eqn{D \times D} correlation
#'         matrix \code{Corr}.
#'   \item \strong{Item parameters:}
#'         \itemize{
#'           \item \eqn{a_{id} \sim \text{Lognormal}(0.25, 0.25)} for
#'                 \eqn{q_{id} = 1} (M2PL/M3PL/M4PL);
#'                 \eqn{a_{id} = 1} for all items and dimensions in M1PL.
#'           \item \eqn{b_i \sim N(0, 1)} (difficulty).
#'           \item \eqn{c_i \sim U(0, 0.35)} (lower asymptote; M3PL/M4PL only).
#'           \item \eqn{d_i \sim U(0.65, 1)} (upper asymptote; M4PL only).
#'         }
#'   \item \strong{Response probabilities:}
#'         \eqn{P_{ij} = P(Y_{ij} = 1 \mid \boldsymbol{\theta}_j)} per the
#'         specified MIRT model (see \code{\link{fit.MIRT}} for the IRF
#'         equations).
#'   \item \strong{Binary responses:}
#'         \eqn{Y_{ij} \sim \text{Bernoulli}(P_{ij})}.
#' }
#'
#' Optional rotation of the loading matrix \code{a} and corresponding
#' transformation of \code{theta} and \code{Corr} is applied before
#' response generation.
#'
#' @param N Integer; number of examinees (default: 500).
#' @param I Integer; number of items (default: 20).
#' @param D Integer; number of latent dimensions (default: 2).
#' @param model Character; one of \code{"m1pl"}, \code{"m2pl"} (default),
#'   \code{"m3pl"}, \code{"m4pl"}.
#' @param Q.matrix An optional \eqn{I \times D} 0/1 matrix. If \code{NULL},
#'   a default structure with triangular identification pattern is used.
#' @param Corr An optional \eqn{D \times D} correlation matrix. If
#'   \code{NULL}, the identity matrix is used (orthogonal factors).
#' @param rotate Optional rotation method name (passed to
#'   \pkg{GPArotation} or \code{stats::promax}). Applicable only when
#'   \eqn{D > 1} and all Q-matrix entries are non-zero.
#' @param promax_m Integer; power parameter for \code{"promax"} rotation
#'   (default: 4).
#'
#' @return An object of class \code{"data.MIRT"}, a list containing:
#' \describe{
#'   \item{\code{data}, \code{response}}{\eqn{N \times I} binary response
#'         matrix (identical).}
#'   \item{\code{theta}}{\eqn{N \times D} matrix of true latent trait
#'         values.}
#'   \item{\code{par}}{\eqn{I \times (D+3)} matrix of true item parameters
#'         (columns \code{a1..aD, b, c, d}).}
#'   \item{\code{probability}}{\eqn{N \times I} matrix of response
#'         probabilities.}
#'   \item{\code{Q.matrix}}{\eqn{I \times D} Q-matrix used.}
#'   \item{\code{Corr}}{\eqn{D \times D} correlation matrix.}
#'   \item{\code{model}, \code{N}, \code{I}, \code{D}}{Data-generating
#'         parameters.}
#' }
#'
#' @seealso
#' \code{\link{fit.MIRT}} for fitting the MIRT model,
#' \code{\link{model.MIRT}} for computing response probabilities.
#'
#' @examples
#' # Basic 2D 2PL simulation
#' sim <- sim.data.MIRT(N = 20, I = 6, D = 2, model = "m2pl")
#'
#' # With correlated factors and rotation
#' Corr <- matrix(c(1, 0.5, 0.5, 1), 2, 2)
#' sim_rot <- sim.data.MIRT(N = 20, I = 6, D = 2, model = "m2pl",
#'                          Q.matrix = matrix(1, 6, 2),
#'                          Corr = Corr, rotate = "oblimin")
#'
#' # Fit the simulated data
#' fit <- fit.MIRT(
#'   sim$response, model = "m2pl", D = 2, method = "iStEM",
#'   control.method = list(
#'     vis = FALSE, seed = 123,
#'     M = 2, B = 2, burnin.maxitr = 2,
#'     maxitr = 3, eps1 = 10, eps2 = 10,
#'     estimate.se = FALSE)
#' )
#' cor(fit$theta$est, sim$theta)  # trait recovery
#'
#' @export
sim.data.MIRT <- function(N = 500, I = 20, D = 2, model = "m2pl",
                          Q.matrix = NULL, Corr = NULL, rotate = NULL,
                          promax_m = 4) {

  call <- match.call()

  N <- check_integer_scalar(N, "N")
  I <- check_integer_scalar(I, "I")
  D <- check_integer_scalar(D, "D")
  model <- resolve_model_type(model)
  if (model == "m1pl" && D > 1L) {
    stop("M1PL simulation is supported only for D = 1.", call. = FALSE)
  }

  Corr <- validate_corr_matrix(Corr, D)

  theta <- generate_theta_mvn(N, D, Corr)

  Q.matrix <- istem_prepare_01_q(
    I, D, Q.matrix, triangular = TRUE, require.row = TRUE
  )

  if (model %in% c("m2pl", "m3pl", "m4pl")) {
    a <- matrix(rlnorm(I * D, meanlog = 0.25, sdlog = 0.25), I, D) * Q.matrix
  } else {
    a <- matrix(1, I, D)
  }

  b <- rnorm(I, 0, 1)
  c <- runif(I, 0.00, 0.35)
  d <- runif(I, 0.65, 1.00)

  if (model == "m1pl") {
    a <- matrix(1, I, D)
    c <- rep(0.0, I)
    d <- rep(1.0, I)
  } else if (model == "m2pl") {
    c <- rep(0.0, I)
    d <- rep(1.0, I)
  } else if (model == "m3pl") {
    d <- rep(1.0, I)
  }

  if (!is.null(rotate) && D > 1 && model != "m1pl") {
    if (any(Q.matrix == 0)) {
      stop("Rotation is not allowed when Q.matrix contains zeros (confirmatory MIRT). ",
           "The Q.matrix specifies a confirmatory structure that rotation would destroy.", call. = FALSE)
    }

    rot <- apply_rotation_promax(a, theta, Corr, rotate, D, promax_m)
    a     <- rot$a
    theta <- rot$theta
    Corr  <- rot$Corr
  }

  if (D > 1) {
    colnames(a) <- paste0("a", seq_len(D))
  }

  par <- cbind(a, b, c, d)

  probability <- model.MIRT(theta = theta, par = par)
  response <- +(probability > matrix(runif(N * I), nrow = N, ncol = I))
  colnames(response) <- seq_len(I)

  data.obj <- list(
    data        = response,
    response    = response,
    theta       = theta,
    par         = par,
    Q.matrix    = Q.matrix,
    block.items = NULL,
    Corr        = Corr,
    model       = model,
    N           = N,
    I           = I,
    D           = D,
    probability = probability,
    call        = call,
    arguments   = list(
      N         = N,
      I         = I,
      D         = D,
      model     = model,
      Q.matrix  = Q.matrix,
      Corr      = Corr,
      rotate    = rotate,
      promax_m  = promax_m
    )
  )

  class(data.obj) <- "data.MIRT"

  return(data.obj)
}


#' Compute MIRT Item Response Probabilities
#'
#' @description
#' Computes the probability of a correct response for each person--item
#' combination under the multidimensional 1PL--4PL models. This is the
#' core IRF (item response function) used throughout the package.
#'
#' @param theta An \eqn{N \times D} matrix of latent trait values.
#' @param par An \eqn{I \times (D+3)} parameter matrix with columns
#'   \code{a1..aD, b, c, d}, where \code{b} is the item difficulty.
#'   The linear predictor is \eqn{\mathbf{a}_i'\boldsymbol{\theta}_j - b_i}.
#'
#' @return An \eqn{N \times I} matrix of response probabilities
#'   \eqn{P(Y_{ij} = 1 \mid \boldsymbol{\theta}_j)}.
#'
#' @seealso \code{\link{fit.MIRT}} for the full model specification.
#' @keywords internal
model.MIRT <- function(theta, par) {
  cpp_model_MIRT(theta, par)
}

---
title: "Introduction to ForceChoice"
author: "Haijiang Qin & Lei Guo"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to ForceChoice}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## Overview

The **ForceChoice** package provides a unified framework for fitting,
simulating, and evaluating forced-choice and traditional item response
theory (IRT) models. It supports eight model families with full Bayesian
estimation via Stan (Hamiltonian Monte Carlo), a fast iterative stochastic EM
(iStEM) algorithm, and a deterministic EM backend for FCGDINA.

This vignette is intended as a reproducible user guide rather than a
complete methodological review. Model formulas and object components are
described using the parameterization implemented in **ForceChoice**. The
reference list gives DOI links where available so readers can verify the
underlying psychometric sources.

### Model Families

| Family | Description | Data Type |
|---|---|---|
| **MIRT** | Multidimensional IRT (1PL--4PL) | Binary |
| **MGPCM** | Multidimensional Generalized Partial Credit | Polytomous |
| **MGGUM** | Multidimensional Generalized Graded Unfolding | Polytomous |
| **FCMIRT** | Forced-Choice MIRT with Luce--Plackett ranking | Ranking/MOLE/PICK |
| **FCGGUM** | Forced-Choice GGUM with ranking | Ranking/MOLE/PICK |
| **TIRT** | Thurstonian IRT with pairwise probit | Ranking/MOLE/PICK |
| **FCDCM** | Forced-Choice Diagnostic Classification | Paired comparison |
| **FCGDINA** | Forced-Choice GDINA diagnostic model | Ranking/MOLE/PICK |

The forced-choice families accept full ranking (`"RANK"`), most-least
(`"MOLE"`), or best-only (`"PICK"`) data when supported by the corresponding
fitting function. FCDCM is the exception: it is a paired-comparison model
where every block contains exactly two statements.

### Model and Backend Coverage

| Family | Main response process | Backends | Primary outputs |
|---|---|---|---|
| MIRT | Dominance IRT for binary items | Stan, iStEM | item parameters, $\theta$, factor correlations |
| MGPCM | Dominance IRT for ordered categories | Stan, iStEM | category intercepts, $\theta$, factor correlations |
| MGGUM | Ideal-point/unfolding IRT | Stan, iStEM | slopes, locations, thresholds, $\theta$ |
| FCMIRT | MIRT endorsement plus Luce--Plackett ranking | Stan, iStEM | item parameters, $\theta$, block fit |
| FCGGUM | GGUM endorsement plus Luce--Plackett ranking | Stan, iStEM | unfolding item parameters, $\theta$, block fit |
| TIRT | Pairwise Thurstonian probit comparisons | Stan, iStEM | loadings, uniquenesses, $\theta$ |
| FCDCM | Higher-order DCM for two-statement FC blocks | Stan, iStEM | attribute profiles, higher-order parameters |
| FCGDINA | GDINA/DINA/DINO/ACDM plus FC ranking | Stan, iStEM, EM | attribute profiles, CDM item parameters |

### Estimation Backends

- **Stan** (`method = "stan"`): Full Bayesian inference via HMC/NUTS.
  Provides posterior means, standard deviations, and R-hat convergence
  diagnostics. Suitable for final inference with small-to-moderate datasets.

- **iStEM** (`method = "iStEM"`): Iterative Stochastic EM combining
  Metropolis-within-Gibbs person sampling with L-BFGS-B item optimization.
  Scales to large datasets. Convergence monitored via Geweke diagnostics.

- **EM** (`method = "EM"`): Deterministic posterior-weight EM for FCGDINA.

## Installation

```r
# Development version from GitHub
remotes::install_github("Naidantu/ForceChoice")
```

## Quick Start: Traditional IRT

### Simulating and Fitting a MIRT Model

```{r, eval=FALSE}
library(ForceChoice)

# Simulate binary response data
sim <- sim.data.MIRT(N = 20, I = 6, D = 2, model = "2PL")

# Fit via iStEM
fit <- fit.MIRT(sim$data, model = "2PL", D = 2,
                method = "iStEM",
                control.method = list(
                  vis = FALSE, seed = 123,
                  M = 2, B = 2, burnin.maxitr = 2,
                  maxitr = 3, eps1 = 10, eps2 = 10,
                  estimate.se = FALSE))

# Examine results
print(fit)
summary(fit)

# Item parameter estimates (first 6 items)
head(coef(fit))

# Factor correlation matrix
fit$Corr$est

# Trait recovery
diag(cor(fit$theta$est, sim$theta))
```

### Goodness-of-Fit Evaluation

```{r, eval=FALSE}
# Compute comprehensive fit indices
gof <- get.fit.index(fit)

# Summary of fit indices
summary(gof)

# Extract specific indices
gof$M2       # Limited-information M2 statistic
gof$RMSEA    # RMSEA with 90% CI
gof$CFI      # Comparative Fit Index
gof$TLI      # Tucker-Lewis Index
gof$SRMSR    # Standardized Root Mean Square Residual
gof$AIC      # Akaike Information Criterion
gof$BIC      # Bayesian Information Criterion
```

## Forced-Choice Modeling

### Simulating and Fitting FCMIRT

Forced-choice data uses ranking strings (e.g., `"2>1>3"` means item 2
is preferred over item 1 over item 3).

```{r, eval=FALSE}
# Simulate forced-choice ranking data
sim <- sim.data.FCMIRT(N.person = 20, N.block = 3, I.block = 2,
                       D = 2, model = "2PL", fc.type = "RANK")

# The data contains ranking strings
head(sim$data)

# Fit: block.items and fc.type are auto-detected
fit <- fit.FCMIRT(sim$data, model = "2PL", D = 2,
                  method = "iStEM",
                  control.method = list(
                    vis = FALSE, seed = 123,
                    M = 2, B = 2, burnin.maxitr = 2,
                    maxitr = 3, eps1 = 10, eps2 = 10,
                    estimate.se = FALSE))

# Trait recovery
cor(fit$theta$est, sim$theta)

# Goodness-of-fit (uses nominal binary expansion)
gof <- get.fit.index(fit)
summary(gof)
```

### Thurstonian IRT (TIRT)

```{r, eval=FALSE}
sim <- sim.data.TIRT(N.person = 20, N.block = 3, I.block = 2,
                     D = 2, fc.type = "RANK")

fit <- fit.TIRT(sim$data, Q.matrix = sim$Q.matrix,
                block.items = sim$block.items,
                method = "iStEM",
                control.method = list(
                  vis = FALSE, seed = 123,
                  M = 2, B = 2, burnin.maxitr = 2,
                  maxitr = 3, eps1 = 10, eps2 = 10,
                  estimate.se = FALSE))

# Structural parameters: loadings and uniquenesses
head(coef(fit))

# Gamma matrix (pairwise intercepts)
fit$gamma.matrix$est[1:5, 1:5]
```

### Forced-Choice DCM (FCDCM)

```{r, eval=FALSE}
sim <- sim.data.FCDCM(N.person = 20, N.block = 3, D = 2,
                      dcm.type = "DINA")

fit <- fit.FCDCM(sim$data, Q.matrix = sim$Q.matrix,
                 block.items = sim$block.items,
                 method = "iStEM",
                 control.method = list(
                   vis = FALSE, seed = 123,
                   M = 2, B = 2, burnin.maxitr = 2,
                   maxitr = 3, eps1 = 10, eps2 = 10,
                   estimate.se = FALSE))

# Posterior attribute mastery probabilities
head(fit$alpha$prob)

# Attribute mastery proportions
colMeans(fit$alpha$prob > 0.5)

# Higher-order IRT parameters
fit$delta$est
```

### Forced-Choice GDINA (FCGDINA)

FCGDINA is the diagnostic-classification counterpart for multi-statement
forced-choice blocks. Unlike FCDCM, which is restricted to paired
comparisons under a higher-order DCM structure, FCGDINA supports
`"GDINA"`, `"DINA"`, `"DINO"`, and `"ACDM"` statement-level models and can
be fitted to full ranking, most-least, or best-only forced-choice data.

```{r, eval=FALSE}
sim <- sim.data.FCGDINA(N.person = 20, N.block = 2, I.block = 2,
                        D = 2, model = "GDINA", fc.type = "RANK")

fit <- fit.FCGDINA(sim$data, Q.matrix = sim$Q.matrix,
                   block.items = sim$block.items, model = "GDINA",
                   fc.type = sim$fc.type, method = "EM",
                   control.method = list(vis = FALSE, seed = 123,
                                         maxitr = 2,
                                         estimate.se = FALSE))

# Posterior attribute mastery probabilities
head(fit$alpha$est)

# CDM item-parameter estimates
coef(fit, type = "delta")
```

## Working with Polytomous Data

### MGPCM (Dominance)

```{r, eval=FALSE}
sim <- sim.data.MGPCM(N = 20, I = 6, D = 2, length.poly = 4)

fit <- fit.MGPCM(sim$data, D = 2, method = "iStEM",
                 control.method = list(
                   vis = FALSE, seed = 123,
                   M = 2, B = 2, burnin.maxitr = 2,
                   maxitr = 3, eps1 = 10, eps2 = 10,
                   estimate.se = FALSE))

# Category threshold parameters
coef(fit)
```

### MGGUM (Ideal-Point / Unfolding)

```{r, eval=FALSE}
sim <- sim.data.MGGUM(N = 20, I = 6, D = 2, length.poly = 4)

fit <- fit.MGGUM(sim$data, D = 2, method = "iStEM",
                 control.method = list(
                   vis = FALSE, seed = 123,
                   M = 2, B = 2, burnin.maxitr = 2,
                   maxitr = 3, eps1 = 10, eps2 = 10,
                   estimate.se = FALSE))

coef(fit)
```

## Solution Rotation

For exploratory MIRT analyses, post-hoc rotation helps achieve simple
structure:

```{r, eval=FALSE}
fit_rot <- rotate.MIRT(fit, rotate = "oblimin")

# Compare original and rotated loadings
head(coef(fit))
head(coef(fit_rot))
```

## Diagnostic Plots

```{r, eval=FALSE}
# Item characteristic curves (ICC)
plot(fit, type = "icc", items = 1:8)

# Person parameter distributions
plot(fit, type = "theta")

# iStEM convergence trace
plot(fit, type = "trace")
```

## Estimation Control

### Common Parameters (top-level)

```{r, eval=FALSE}
fit <- fit.MIRT(data, model = "2PL", D = 2, method = "iStEM",
                control.method = list(
                  vis = FALSE,
                  seed = 123,      # Reproducibility
                  M = 2, B = 2, burnin.maxitr = 2,
                  maxitr = 3, eps1 = 10, eps2 = 10,
                  estimate.se = FALSE))
```

### Stan-Specific Control

```{r, eval=FALSE}
# stan code, long time
fit <- fit.MIRT(data, model = "2PL", D = 2, method = "stan",
                control.method = list(chains = 1, iter = 200,
                                      warmup = 100, cores = 1,
                                      seed = 123))
```

### iStEM-Specific Control (advanced)

```{r, eval=FALSE}
fit <- fit.MIRT(data, model = "2PL", D = 2, method = "iStEM",
                control.method = list(
                  seed = 123,
                  M = 2,            # Burn-in batches for convergence
                  B = 2,            # Iterations per batch
                  burnin.maxitr = 2,
                  maxitr = 3,
                  eps1 = 1.5,       # Geweke convergence threshold
                  eps2 = 0.4        # MC error tolerance
                ))
```

## References

- Brown, A., & Maydeu-Olivares, A. (2011). Item response modeling of
  forced-choice questionnaires. *Educational and Psychological Measurement*,
  *71*(3), 460--502. <https://doi.org/10.1177/0013164410375112>

- de la Torre, J. (2011). The generalized DINA model framework.
  *Psychometrika*, *76*(2), 179--199.
  <https://doi.org/10.1007/s11336-011-9207-7>

- Huang, H.-Y. (2022). Diagnostic classification model for forced-choice
  items and noncognitive tests. *Educational and Psychological Measurement*,
  *83*(1), 146--180. <https://doi.org/10.1177/00131644211069906>

- Lee, P., Joo, S.-H., Stark, S., & Chernyshenko, O. S. (2018). GGUM-RANK
  statement and person parameter estimation with multidimensional forced
  choice triplets. *Applied Psychological Measurement*, *43*(3), 226--240.
  <https://doi.org/10.1177/0146621618768294>

- Luce, R. D. (1959). *Individual choice behavior: A theoretical analysis*.
  Wiley.

- Maydeu-Olivares, A., & Joe, H. (2005). Limited- and full-information
  estimation and goodness-of-fit testing in 2^n contingency tables: A
  unified framework. *Journal of the American Statistical Association*,
  *100*(471), 1009--1020.
  <https://doi.org/10.1198/016214504000002069>

- Maydeu-Olivares, A., & Joe, H. (2006). Limited information goodness-of-fit
  testing in multidimensional contingency tables. *Psychometrika*, *71*(4),
  713--732. <https://doi.org/10.1007/s11336-005-1295-9>

- Muraki, E. (1992). A generalized partial credit model: Application of an
  EM algorithm. *Applied Psychological Measurement*, *16*(2), 159--176.
  <https://doi.org/10.1177/014662169201600206>

- Plackett, R. L. (1975). The analysis of permutations. *Journal of the
  Royal Statistical Society: Series C (Applied Statistics)*, *24*(2),
  193--202. <https://doi.org/10.2307/2346567>

- Reckase, M. D. (2009). *Multidimensional Item Response Theory*. Springer.
  <https://doi.org/10.1007/978-0-387-89976-3>

- Roberts, J. S., Donoghue, J. R., & Laughlin, J. E. (2000). A general item
  response theory model for unfolding unidimensional polytomous responses.
  *Applied Psychological Measurement*, *24*(1), 3--32.
  <https://doi.org/10.1177/01466216000241001>

- Zheng, C., Liu, J., Li, Y., Xu, P., Zhang, B., Wei, R., Zhang, W.,
  Liu, B., & Huang, J. (2024). A 2PLM-RANK multidimensional forced-choice
  model and its fast estimation algorithm. *Behavior Research Methods*,
  *56*(6), 6363--6388. <https://doi.org/10.3758/s13428-023-02315-x>

---
title: "ForceChoice Model Theory and Selection Guide"
author: "Haijiang Qin & Lei Guo"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Model Theory and Selection Guide}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Overview

This vignette describes the theoretical foundations of the eight model
families in **ForceChoice** and provides guidance for model selection.
Core models support Stan (full Bayesian HMC) and iStEM (iterative Stochastic
EM); FCGDINA additionally supports deterministic EM.

The equations below use the parameterizations implemented in the package.
For several families, especially MGPCM and MGGUM, this means the notation is
not a verbatim copy of the original unidimensional source article. The cited
papers identify the model family and response-process assumptions; the
implementation details are those exposed by the corresponding `fit.*()`
documentation.

## Traditional (Single-Stimulus) Models

### MIRT: Multidimensional Item Response Theory

The MIRT model extends unidimensional IRT to multiple latent dimensions.
For binary response $Y_{ij} \in \{0, 1\}$, the item response function is:

$$
P(Y_{ij}=1 \mid \boldsymbol{\theta}_j) = c_i + (d_i - c_i) \times
\frac{1}{1 + \exp[-( \mathbf{a}_i' \boldsymbol{\theta}_j - b_i)]}
$$

where $\mathbf{a}_i = (a_{i1}, \ldots, a_{iD})'$ is the discrimination
vector, $b_i$ is the difficulty (intercept), $c_i$ is the lower asymptote
(pseudo-guessing), and $d_i$ is the upper asymptote.

**Model variants:**

| Model | Free Parameters | Fixed |
|---|---|---|
| M1PL (Rasch) | $b_i$ | $a_{id} = q_{id}$, $c_i=0$, $d_i=1$ |
| M2PL | $a_{id}, b_i$ | $c_i=0$, $d_i=1$ |
| M3PL | $a_{id}, b_i, c_i$ | $d_i=1$ |
| M4PL | $a_{id}, b_i, c_i, d_i$ | - |

**When to use:** Binary response data where items vary in discrimination
and difficulty. Dominance model: higher trait always increases endorsement
probability.

**Key reference:** Reckase, M. D. (2009). *Multidimensional Item Response
Theory*. Springer.

---

### MGPCM: Multidimensional Generalized Partial Credit Model

For polytomous responses $Y_{ij} \in \{0, 1, \ldots, K_i-1\}$,
**ForceChoice** implements a multidimensional softmax parameterization:

$$
P(Y_{ij} = k \mid \boldsymbol{\theta}_j) =
\frac{\exp\{k\eta_{ij} + d_{ik}\}}
{\sum_{r=0}^{K_i-1}\exp\{r\eta_{ij} + d_{ir}\}},
\qquad
\eta_{ij}=\sum_{d=1}^{D}a_{id}\theta_{jd}.
$$

The $d_{ik}$ values are category intercepts rather than cumulative step
difficulties; $d_{i0} \equiv 0$ is fixed for identification. This
implementation nests the generalized partial credit model of Muraki (1992)
in the unidimensional case.

**When to use:** Polytomous (ordered category) data such as Likert-type
responses. A dominance model: higher trait implies higher category.

**Key reference:** Muraki, E. (1992). A generalized partial credit model:
Application of an EM algorithm. *Applied Psychological Measurement*,
*16*(2), 159--176.

---

### MGGUM: Multidimensional Generalized Graded Unfolding Model

The MGGUM is an ideal-point (unfolding) model where endorsement is governed
by distance between the person and item locations. In **ForceChoice**, define

$$
r_{ij} =
\left[
  \sum_{d=1}^{D} a_{id}^2(\theta_{jd}-\delta_{id})^2
\right]^{1/2},
\qquad
S_i = \sum_{d=1}^{D} a_{id},
$$

and let $\tau_{i0}=0$, $\psi_{ik}=S_i\sum_{v=0}^{k}\tau_{iv}$, and
$M_i=2K_i-1$. Then

$$
P(Y_{ij}=k \mid \boldsymbol{\theta}_j)=
\frac{
  \exp\{kr_{ij}-\psi_{ik}\}+
  \exp\{(M_i-k)r_{ij}-\psi_{ik}\}
}{
  \sum_{r=0}^{K_i-1}
  [\exp\{rr_{ij}-\psi_{ir}\}+
   \exp\{(M_i-r)r_{ij}-\psi_{ir}\}]
}.
$$

The signed Q-matrix ($q_{id} \in \{-1, 0, 1\}$) indicates active
dimensions and the side of the latent continuum on which the item location
lies. The sign controls $\delta_{id}$, not the sign of the discrimination
parameter.

**When to use:** Attitude/ preference measurement where both ends of the
trait continuum can lead to disagreement (e.g., "I like moderate
temperatures": both extreme cold-lovers and heat-lovers disagree).

**Key references:**
- Roberts, J. S., Donoghue, J. R., & Laughlin, J. E. (2000). A general
  item response theory model for unfolding unidimensional polytomous
  responses. *Applied Psychological Measurement*, *24*(1), 3--32.
- Tu, N., Zhang, B., Angrave, L., Sun, T., & Neuman, M. (2023). Estimating
  the Multidimensional Generalized Graded Unfolding Model with covariates
  using a Bayesian approach. *Journal of Intelligence*, *11*, 163.

---

## Forced-Choice (Comparative) Models

Forced-choice (FC) formats ask respondents to compare items within blocks
rather than rating items in isolation. This can reduce response-style
biases such as acquiescence, social desirability, and extreme responding,
but it introduces comparative/ipsative information with model-specific
identification requirements.

### FCMIRT: Forced-Choice MIRT

Item-level endorsement follows the MIRT model (1PL--4PL). At the block
level, **ForceChoice** uses a sequential Luce--Plackett ranking kernel with
utilities equal to the logit of the item-level endorsement probability:

$$
u_i(\boldsymbol{\theta}_j) =
\log\left[\frac{P_i(\boldsymbol{\theta}_j)}
{1-P_i(\boldsymbol{\theta}_j)}\right].
$$

$$
P(i_{(1)} \succ \cdots \succ i_{(K_b)} \mid \boldsymbol{\theta}_j) =
\prod_{m=1}^{K_b-1}
\frac{\exp\{u_{i_{(m)}}(\boldsymbol{\theta}_j)\}}
{\sum_{r=m}^{K_b}\exp\{u_{i_{(r)}}(\boldsymbol{\theta}_j)\}}.
$$

**Identification:** Block difficulties sum to zero: $\sum_{i \in b} b_i = 0$.

**FC types:**
- **RANK**: Full ranking of all items within each block
- **MOLE**: Only most and least preferred items identified
- **PICK**: Only most preferred item identified

**When to use:** Forced-choice questionnaires with dominance items
(personality, vocational preferences).

**Key reference:** Zheng, C., Liu, J., Li, Y., et al. (2024). A 2PLM-RANK
multidimensional forced-choice model and its fast estimation algorithm.
*Behavior Research Methods*, *56*, 6363--6388.

**Ranking-model references:** Luce (1959) and Plackett (1975).

---

### FCGGUM: Forced-Choice GGUM

Combines the GGUM ideal-point model at the item level with the same
sequential Luce--Plackett ranking kernel used by FCMIRT. The block-level
ranking probability uses the logit of the GGUM endorsement probability as
the statement utility.

**Identification:** Same block-level sum-to-zero constraint as FCMIRT.
Signed Q-matrix indicates both dimension membership and statement direction.

**When to use:** Forced-choice questionnaires with ideal-point/unfolding
items (e.g., personality items where both extremes disagree).

**Key reference:** Lee, P., Joo, S.-H., Stark, S., & Chernyshenko, O. S.
(2018). GGUM-RANK statement and person parameter estimation with
multidimensional forced choice triplets. *Applied Psychological
Measurement*, *43*(3), 226--240.

---

### TIRT: Thurstonian IRT for Forced-Choice

The TIRT model works directly with pairwise comparisons via a probit link.
For each pair $(i, k)$ in block $b$, the latent comparative judgment is:

$$
y_{j,ik}^* = -\gamma_{ik} + \lambda_i \mathbf{q}_i'\boldsymbol{\theta}_j -
\lambda_k \mathbf{q}_k'\boldsymbol{\theta}_j + \varepsilon_{j,i} - \varepsilon_{j,k}
$$

with $\varepsilon_{j,i} \sim N(0, \psi_i^2)$. The probability of preferring
$i$ over $k$ is:

$$
P(Y_{j,ik}=1 \mid \boldsymbol{\theta}_j) =
\Phi\left(\frac{-\gamma_{ik} + \lambda_i\mathbf{q}_i'\boldsymbol{\theta}_j -
\lambda_k\mathbf{q}_k'\boldsymbol{\theta}_j}
{\sqrt{\psi_i^2 + \psi_k^2}}\right)
$$

**Identification constraints:**
- **Case A** ($I_b=2, D>2$): All $\psi_i^2 = 0.5$
- **Case B** ($D=2, I_b=2$): First block $\lambda_i = 0.80$, all $\psi_i^2 = 0.5$
- **General case**: Last item per block has $\psi_i^2 = 1.0$

**When to use:** When you want a factor-analytic decomposition of
forced-choice data with factor loadings and uniquenesses, rather than the
item-level endorsement approach of FCMIRT.

**Key reference:** Brown, A., & Maydeu-Olivares, A. (2011). Item response
modeling of forced-choice questionnaires. *Educational and Psychological
Measurement*, *71*(3), 460--502.

---

### FCDCM: Forced-Choice Diagnostic Classification Model

A higher-order cognitive diagnostic model for paired-comparison FC data.
Each block contains exactly two statements. The discrete attribute mastery
profile $\boldsymbol{\alpha}_j$ is governed by a continuous higher-order
trait $\theta_j$:

$$
P(\alpha_{jd} = 1 \mid \theta_j) =
\frac{1}{1 + \exp[-(\delta_{1d}\theta_j - \delta_{1d}\delta_{0d})]}
$$

**Condensation rules:**
- **DINA (conjunctive):** $\zeta_{ij} = \prod_{d: q_{id}=1} \alpha_{jd}$
  (all required attributes must be mastered)
- **DINO (disjunctive):** $\zeta_{ij} = 1 - \prod_{d: q_{id}=1} (1-\alpha_{jd})$
  (at least one required attribute mastered)

Block response probabilities are parameterized by $\eta_{0b}$ (base rate)
and $\eta_{ABb}$ (joint-mastery bonus), with exact marginalization over
all $2^D$ attribute profiles.

**When to use:** Diagnostic classification with forced-choice format -
classifying respondents into discrete attribute mastery profiles while
avoiding response biases via the FC format.

**Key references:**
- Huang, H.-Y. (2022). Diagnostic classification model for forced-choice
  items and noncognitive tests. *Educational and Psychological Measurement*,
  *83*(1), 146--180.
- Zhu, Y.-A., Xu, J., Wang, D., Li, X., Cai, Y., & Tu, D. (2024). A
  ranking forced choice diagnostic classification model for psychological
  assessment. *British Journal of Mathematical and Statistical Psychology*,
  *78*, 617--646.

---

### FCGDINA: Forced-Choice GDINA

FCGDINA combines a statement-level cognitive diagnostic model with a
forced-choice block response model. The statement model may be GDINA, DINA,
DINO, or ACDM. For a person with attribute profile
$\boldsymbol{\alpha}_j \in \{0,1\}^D$, a statement-level endorsement
probability is computed from the selected CDM design matrix. The block
probabilities are then obtained by the same Luce--Plackett transformation
used by FCMIRT and FCGGUM.

The likelihood marginalizes over all $2^D$ attribute profiles:

$$
P(\mathbf{Y}_j) =
\sum_{c=1}^{2^D}\pi_c
\prod_{b=1}^{B}
P(Y_{jb}\mid \boldsymbol{\alpha}_c, \boldsymbol{\delta}, \text{block}_b).
$$

**Important identification note:** Forced-choice data identify within-block
utility contrasts, not absolute single-statement endorsement levels. In the
Stan implementation, the nonidentified null space of the delta effects is
removed and the reported coefficients use the package's identifying
convention. EM and iStEM provide probability-scale CDM estimates under the
same block response likelihood.

**When to use:** Diagnostic classification with ranking, most-least, or
best-only forced-choice blocks, especially when the statement-level CDM
needs GDINA/ACDM flexibility rather than the paired-comparison FCDCM
structure.

**Key references:** de la Torre (2011) for GDINA; Luce (1959) and Plackett
(1975) for the ranking kernel.

---

## Model Selection Guide

| Criterion | Recommended Model |
|---|---|
| Binary responses, dominance | `fit.MIRT()` |
| Polytomous (Likert), dominance | `fit.MGPCM()` |
| Polytomous, ideal-point | `fit.MGGUM()` |
| FC ranking, dominance items | `fit.FCMIRT()` |
| FC ranking, ideal-point items | `fit.FCGGUM()` |
| FC ranking, factor-analytic approach | `fit.TIRT()` |
| FC paired comparison, diagnostic classification | `fit.FCDCM()` |
| FC ranking/MOLE/PICK, diagnostic classification | `fit.FCGDINA()` |

## Estimation Method Selection

| Criterion | Recommended |
|---|---|
| Final inference, moderate N | Stan |
| Large datasets, exploratory analysis | iStEM |
| Need posterior SDs and R-hat | Stan |
| Need fast point estimates + SEs | iStEM |
| Complex model, multi-chain diagnostics | Stan |
| Very large N (> 5000) or many items | iStEM |

---

## Goodness-of-Fit Framework

All models use the limited-information M2 framework
(Maydeu-Olivares & Joe, 2005, 2006):

- **M2 statistic**: Tests the null of exact model fit using
  univariate and bivariate margins.
- **RMSEA**: Approximate fit index with 90% confidence interval.
- **CFI / TLI**: Comparative fit against an independence baseline.
- **SRMSR**: Standardized root mean square residual.
- **Yen's Q3**: Local dependence diagnostics from residual correlations.

For FC models, within-block pairwise residuals are excluded (by design,
items in the same block are locally dependent through the ranking
mechanism).

## References

Brown, A., & Maydeu-Olivares, A. (2011). Item response modeling of
forced-choice questionnaires. *Educational and Psychological Measurement*,
*71*(3), 460--502. <https://doi.org/10.1177/0013164410375112>

de la Torre, J. (2011). The generalized DINA model framework.
*Psychometrika*, *76*(2), 179--199.
<https://doi.org/10.1007/s11336-011-9207-7>

Huang, H.-Y. (2022). Diagnostic classification model for forced-choice
items and noncognitive tests. *Educational and Psychological Measurement*,
*83*(1), 146--180. <https://doi.org/10.1177/00131644211069906>

Lee, P., Joo, S.-H., Stark, S., & Chernyshenko, O. S. (2018). GGUM-RANK
statement and person parameter estimation with multidimensional forced
choice triplets. *Applied Psychological Measurement*, *43*(3), 226--240.
<https://doi.org/10.1177/0146621618768294>

Luce, R. D. (1959). *Individual choice behavior: A theoretical analysis*.
Wiley.

Maydeu-Olivares, A., & Joe, H. (2005). Limited- and full-information
estimation and goodness-of-fit testing in 2^n contingency tables: A unified
framework. *Journal of the American Statistical Association*, *100*(471),
1009--1020. <https://doi.org/10.1198/016214504000002069>

Maydeu-Olivares, A., & Joe, H. (2006). Limited information goodness-of-fit
testing in multidimensional contingency tables. *Psychometrika*, *71*(4),
713--732. <https://doi.org/10.1007/s11336-005-1295-9>

Muraki, E. (1992). A generalized partial credit model: Application of an EM
algorithm. *Applied Psychological Measurement*, *16*(2), 159--176.
<https://doi.org/10.1177/014662169201600206>

Plackett, R. L. (1975). The analysis of permutations. *Journal of the Royal
Statistical Society: Series C (Applied Statistics)*, *24*(2), 193--202.
<https://doi.org/10.2307/2346567>

Reckase, M. D. (2009). *Multidimensional Item Response Theory*. Springer.
<https://doi.org/10.1007/978-0-387-89976-3>

Roberts, J. S., Donoghue, J. R., & Laughlin, J. E. (2000). A general item
response theory model for unfolding unidimensional polytomous responses.
*Applied Psychological Measurement*, *24*(1), 3--32.
<https://doi.org/10.1177/01466216000241001>

Tu, N., Zhang, B., Angrave, L., Sun, T., & Neuman, M. (2023). Estimating the
multidimensional generalized graded unfolding model with covariates using a
Bayesian approach. *Journal of Intelligence*, *11*(8), 163.
<https://doi.org/10.3390/jintelligence11080163>

Zheng, C., Liu, J., Li, Y., Xu, P., Zhang, B., Wei, R., Zhang, W.,
Liu, B., & Huang, J. (2024). A 2PLM-RANK multidimensional forced-choice
model and its fast estimation algorithm. *Behavior Research Methods*,
*56*(6), 6363--6388. <https://doi.org/10.3758/s13428-023-02315-x>

Zhu, Y.-A., Xu, J., Wang, D., Li, X., Cai, Y., & Tu, D. (2024). A ranking
forced choice diagnostic classification model for psychological assessment
using forced choice questionnaires. *British Journal of Mathematical and
Statistical Psychology*, *78*(2), 617--646.
<https://doi.org/10.1111/bmsp.12376>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{normalize_fc_type}
\alias{normalize_fc_type}
\title{Normalise and recycle forced-choice type vector}
\usage{
normalize_fc_type(fc.type, N.block, default = "RANK")
}
\arguments{
\item{fc.type}{Character vector (or \code{NULL}) describing the FC format of
each block.  Recycled to length \code{N.block} and uppercased.}

\item{N.block}{Number of blocks.}

\item{default}{Default FC type used when \code{fc.type} is \code{NULL}.}
}
\value{
A character vector of length \code{N.block}.
}
\description{
Normalise and recycle forced-choice type vector
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-utils.R
\name{clip_prob}
\alias{clip_prob}
\title{Clamp probabilities away from 0 and 1}
\usage{
clip_prob(p, eps = 1e-12)
}
\arguments{
\item{p}{Numeric vector or matrix of probabilities.}

\item{eps}{Small positive value (default \code{1e-12}).}
}
\value{
Numeric vector or matrix with the same dimensions as \code{p}.
}
\description{
Ensures all values lie in \eqn{[eps, 1-eps]} to avoid \code{log(0)}
or division-by-zero issues in downstream computations.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.data.MIRT.R
\name{sim.data.MIRT}
\alias{sim.data.MIRT}
\title{Simulate Data from the Multidimensional IRT (MIRT) Model}
\usage{
sim.data.MIRT(
  N = 500,
  I = 20,
  D = 2,
  model = "m2pl",
  Q.matrix = NULL,
  Corr = NULL,
  rotate = NULL,
  promax_m = 4
)
}
\arguments{
\item{N}{Integer; number of examinees (default: 500).}

\item{I}{Integer; number of items (default: 20).}

\item{D}{Integer; number of latent dimensions (default: 2).}

\item{model}{Character; one of \code{"m1pl"}, \code{"m2pl"} (default),
\code{"m3pl"}, \code{"m4pl"}.}

\item{Q.matrix}{An optional \eqn{I \times D} 0/1 matrix. If \code{NULL},
a default structure with triangular identification pattern is used.}

\item{Corr}{An optional \eqn{D \times D} correlation matrix. If
\code{NULL}, the identity matrix is used (orthogonal factors).}

\item{rotate}{Optional rotation method name (passed to
\pkg{GPArotation} or \code{stats::promax}). Applicable only when
\eqn{D > 1} and all Q-matrix entries are non-zero.}

\item{promax_m}{Integer; power parameter for \code{"promax"} rotation
(default: 4).}
}
\value{
An object of class \code{"data.MIRT"}, a list containing:
\describe{
\item{\code{data}, \code{response}}{\eqn{N \times I} binary response
matrix (identical).}
\item{\code{theta}}{\eqn{N \times D} matrix of true latent trait
values.}
\item{\code{par}}{\eqn{I \times (D+3)} matrix of true item parameters
(columns \code{a1..aD, b, c, d}).}
\item{\code{probability}}{\eqn{N \times I} matrix of response
probabilities.}
\item{\code{Q.matrix}}{\eqn{I \times D} Q-matrix used.}
\item{\code{Corr}}{\eqn{D \times D} correlation matrix.}
\item{\code{model}, \code{N}, \code{I}, \code{D}}{Data-generating
parameters.}
}
}
\description{
Generates binary response data, latent trait vectors, and item parameters
from the multidimensional extension of the 1PL--4PL item response models.
The latent traits are drawn from a multivariate normal distribution with
a user-specified correlation matrix. Discrimination parameters respect a
Q-matrix structure and may be optionally rotated.
}
\section{Data Generation Process}{


\enumerate{
\item \strong{Latent traits:}
\eqn{\boldsymbol{\theta}_j \sim N_D(\mathbf{0}, \boldsymbol{\Sigma})},
where \eqn{\boldsymbol{\Sigma}} is the \eqn{D \times D} correlation
matrix \code{Corr}.
\item \strong{Item parameters:}
\itemize{
\item \eqn{a_{id} \sim \text{Lognormal}(0.25, 0.25)} for
\eqn{q_{id} = 1} (M2PL/M3PL/M4PL);
\eqn{a_{id} = 1} for all items and dimensions in M1PL.
\item \eqn{b_i \sim N(0, 1)} (difficulty).
\item \eqn{c_i \sim U(0, 0.35)} (lower asymptote; M3PL/M4PL only).
\item \eqn{d_i \sim U(0.65, 1)} (upper asymptote; M4PL only).
}
\item \strong{Response probabilities:}
\eqn{P_{ij} = P(Y_{ij} = 1 \mid \boldsymbol{\theta}_j)} per the
specified MIRT model (see \code{\link{fit.MIRT}} for the IRF
equations).
\item \strong{Binary responses:}
\eqn{Y_{ij} \sim \text{Bernoulli}(P_{ij})}.
}

Optional rotation of the loading matrix \code{a} and corresponding
transformation of \code{theta} and \code{Corr} is applied before
response generation.
}

\examples{
# Basic 2D 2PL simulation
sim <- sim.data.MIRT(N = 20, I = 6, D = 2, model = "m2pl")

# With correlated factors and rotation
Corr <- matrix(c(1, 0.5, 0.5, 1), 2, 2)
sim_rot <- sim.data.MIRT(N = 20, I = 6, D = 2, model = "m2pl",
                         Q.matrix = matrix(1, 6, 2),
                         Corr = Corr, rotate = "oblimin")

# Fit the simulated data
fit <- fit.MIRT(
  sim$response, model = "m2pl", D = 2, method = "iStEM",
  control.method = list(
    vis = FALSE, seed = 123,
    M = 2, B = 2, burnin.maxitr = 2,
    maxitr = 3, eps1 = 10, eps2 = 10,
    estimate.se = FALSE)
)
cor(fit$theta$est, sim$theta)  # trait recovery

}
\seealso{
\code{\link{fit.MIRT}} for fitting the MIRT model,
\code{\link{model.MIRT}} for computing response probabilities.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3vcov.R
\name{nobs}
\alias{nobs}
\alias{nobs.MIRT}
\alias{nobs.MGPCM}
\alias{nobs.MGGUM}
\alias{nobs.FCMIRT}
\alias{nobs.FCDCM}
\alias{nobs.FCGDINA}
\alias{nobs.FCGGUM}
\alias{nobs.TIRT}
\title{S3 Methods: nobs}
\usage{
\method{nobs}{MIRT}(object, ...)

\method{nobs}{MGPCM}(object, ...)

\method{nobs}{MGGUM}(object, ...)

\method{nobs}{FCMIRT}(object, ...)

\method{nobs}{FCDCM}(object, ...)

\method{nobs}{FCGDINA}(object, ...)

\method{nobs}{FCGGUM}(object, ...)

\method{nobs}{TIRT}(object, ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
An integer: the sample size \eqn{N}.
}
\description{
Extracts the number of observations (persons) from a fitted
\pkg{ForceChoice} model object. Returns the number of rows in the
response matrix used for model fitting.
}
\section{Methods (by class)}{
\itemize{
\item \code{nobs(MIRT)}: Number of observations (persons) for \code{MIRT} objects.

\item \code{nobs(MGPCM)}: Number of observations for \code{MGPCM} objects.

\item \code{nobs(MGGUM)}: Number of observations for \code{MGGUM} objects.

\item \code{nobs(FCMIRT)}: Number of observations for \code{FCMIRT} objects.

\item \code{nobs(FCDCM)}: Number of observations for \code{FCDCM} objects.

\item \code{nobs(FCGDINA)}: Number of observations for \code{FCGDINA} objects.

\item \code{nobs(FCGGUM)}: Number of observations for \code{FCGGUM} objects.

\item \code{nobs(TIRT)}: Number of observations for \code{TIRT} objects.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools-data.R
\name{get.response.from.data.TIRT}
\alias{get.response.from.data.TIRT}
\title{Convert TIRT Forced-Choice Data to Pairwise-Response Matrix}
\usage{
get.response.from.data.TIRT(data, block.items, fc.type = NULL)
}
\arguments{
\item{data}{An \eqn{N \times B} character matrix or data frame of
forced-choice responses.  Each cell encodes the within-block item
ordering using item labels from \code{block.items} separated by \code{">"}
(e.g. \code{"3>1>2"} for RANK, \code{"3>2"} for MOLE, \code{"3"}
for PICK).}

\item{block.items}{A list of length \eqn{B} giving the unique item labels
in each forced-choice block.}

\item{fc.type}{Character vector of length \eqn{B} (or a scalar
recycled to all blocks): \code{"RANK"}, \code{"MOLE"}, or
\code{"PICK"}.}
}
\value{
A list with two components:
\describe{
\item{response}{An \eqn{N \times P} integer matrix of pairwise
binary outcomes (1/0).  For block \eqn{b}, the number of contributed
columns is \eqn{K_b(K_b-1)/2} for RANK, \eqn{2K_b - 3} for MOLE, and
\eqn{K_b - 1} for PICK.}
\item{pairs.value}{A list of length \eqn{N}, each element being
a list of length \eqn{B} of integer pair-definition matrices.
Required as input for
\code{\link{get.data.from.response.TIRT}}.}
}
}
\description{
Parses human-readable forced-choice data strings (full rankings,
most-least choices, or best-only picks) into the Thurstonian IRT
pairwise binary response format.  Each unordered item pair within a
block receives a 1/0 outcome indicating which item was preferred.
}
\details{
This function expands each block column in \code{data} into one or more
pairwise response columns.  Pair columns are grouped by block in the same
order as \code{block.items}.  Within a pair column, the coding is always
1 if the first item in the pair is preferred, and 0 if the second item is
preferred.

Item labels must match the values supplied in \code{block.items}; they do
not need to be consecutive. For example, if
\code{block.items = list(c(2, 7), c(10, 20, 30))}, the second block can
contain \code{"10>20>30"}, \code{"30>10"}, or \code{"20"}, depending on
\code{fc.type}.
}
\section{Pattern Naming and Pair Encoding}{

For a block with \eqn{K} items, the number of pair columns depends on
\code{fc.type}: \eqn{K(K-1)/2} for \code{"RANK"}, \eqn{2K - 3} for
\code{"MOLE"}, and \eqn{K - 1} for \code{"PICK"}.  For each retained pair:
\itemize{
\item \strong{RANK}: the item ranked earlier in the ordering is
preferred.  For \code{block.items[[b]] = 3:5}, pair order is
\code{(3,4)}, \code{(3,5)}, \code{(4,5)}.  The string
\code{"3>4>5"} therefore produces pairwise responses
\code{1, 1, 1}.
\item \strong{MOLE}: only the most- and least-preferred items are
identified (e.g. \code{"3>2"} in a 3-item block).  The
remaining items are treated as tied in the middle, producing
preference relations consistent with the partial ordering.
\item \strong{PICK}: only the most-preferred item is identified
(e.g. \code{"3"} in a 3-item block).  All other items are
treated as equally less preferred.
}
The \code{pairs.value} component records the actual pair matrices used for
each person and block.  It is essential for converting MOLE/PICK responses
back to \code{data}, because the retained pair set can differ across
persons.
}

\examples{
# 2 persons, 2 RANK blocks.
dat <- data.frame(
  block.1 = c("1>2", "2>1"),
  block.2 = c("3>4>5", "5>4>3"),
  stringsAsFactors = FALSE
)
items <- list(1:2, 3:5)
result <- get.response.from.data.TIRT(dat, items, fc.type = "RANK")
result$response
result$pairs.value[[1]]
get.data.from.response.TIRT(
  result$response, items, fc.type = "RANK",
  pairs.value = result$pairs.value
)

# Mixed partial-response blocks.
dat.partial <- data.frame(
  mole = c("1>3", "2>1"),
  pick = c("4", "5"),
  stringsAsFactors = FALSE
)
items.partial <- list(1:3, 4:6)
partial <- get.response.from.data.TIRT(
  dat.partial, items.partial, fc.type = c("MOLE", "PICK")
)
partial$response
get.data.from.response.TIRT(
  partial$response, items.partial, fc.type = c("MOLE", "PICK"),
  pairs.value = partial$pairs.value
)
}
\seealso{
\code{\link{get.data.from.response.TIRT}} for the reverse conversion.

\code{\link{get.response.from.data}} for the generic (non-TIRT)
converter used by FCGGUM and FCMIRT.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.data.TIRT.R
\name{model.TIRT}
\alias{model.TIRT}
\title{Compute TIRT Pairwise Comparison Probabilities}
\usage{
model.TIRT(theta, lambda, psi2, gamma.matrix, Q.matrix, pairs.matrix)
}
\arguments{
\item{theta}{Latent trait matrix.}

\item{lambda}{Factor loading magnitudes per statement. If signed values
are supplied from a fitted object, the function uses \code{abs(lambda)}
and the sign in \code{Q.matrix} carries the direction.}

\item{psi2}{Uniqueness variances per statement.}

\item{gamma.matrix}{Pairwise intercept matrix.}

\item{Q.matrix}{Design Q-matrix.}

\item{pairs.matrix}{Matrix of statement-index pairs.}
}
\value{
Matrix of pairwise choice probabilities:
\eqn{\Phi\{[-\gamma_{ik} + (|\lambda_i|q_i -
  |\lambda_k|q_k)'\theta_j] / \sqrt{\psi_i^2+\psi_k^2}\}}.
}
\description{
Compute TIRT Pairwise Comparison Probabilities
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.data.MGGUM.R
\name{model.MGGUM}
\alias{model.MGGUM}
\title{Compute MGGUM Category Response Probabilities}
\usage{
model.MGGUM(theta, par)
}
\arguments{
\item{theta}{\eqn{N \times D} matrix of latent trait values.}

\item{par}{\eqn{I \times (2D + \max K_i)} parameter matrix with columns
\code{a1..aD, delta1..deltaD, tau0, tau1, ...}. Missing category
columns for shorter items should be \code{NA}; \code{tau0} is fixed at 0.}
}
\value{
An \eqn{N \times \sum_i K_i} matrix of stacked category
probabilities. The implementation uses
\eqn{r_{ij} = [\sum_d a_{id}^2(\theta_{jd}-\delta_{id})^2]^{1/2}}
and
\eqn{P(Y_{ij}=k) \propto
  \exp\{k r_{ij}-\psi_{ik}\} +
  \exp\{(2K_i-1-k)r_{ij}-\psi_{ik}\}}.
}
\description{
Compute MGGUM Category Response Probabilities
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-utils.R
\name{resolve_fc_blocks}
\alias{resolve_fc_blocks}
\title{Resolve forced-choice block and Q-matrix configuration}
\usage{
resolve_fc_blocks(
  Q.matrix,
  block.items,
  I.block,
  N.block,
  D,
  q.valid.values,
  q.row.check
)
}
\arguments{
\item{Q.matrix}{User-supplied Q-matrix or \code{NULL}.}

\item{block.items}{User-supplied block-item list or \code{NULL}.}

\item{I.block}{Default items-per-block (scalar).}

\item{N.block}{Default number of blocks (scalar).}

\item{D}{Default number of dimensions (scalar).}

\item{q.valid.values}{Allowed Q-matrix values (e.g. \code{c(0, 1)} for
MIRT/DCM, \code{c(-1, 0, 1)} for GGUM/TIRT).}

\item{q.row.check}{Function applied to each row of Q.matrix; must return
\code{TRUE} for every row.  Example: \code{function(r) any(r != 0)} or
\code{function(r) sum(abs(r)) == 1L}.}
}
\value{
A named list with components \code{I.states}, \code{N.block},
\code{I.block}, \code{block.items}, \code{Q.matrix}, \code{D}.
\code{Q.matrix} is \code{NULL} if it was not supplied and should be
generated later (e.g. by \code{sim.Q.matrix.FC} or \code{sim.data.Q.CDM}).
}
\description{
Shared validation and auto-generation logic used by
\code{sim.data.FCMIRT}, \code{sim.data.FCGGUM}, \code{sim.data.FCGDINA},
and \code{sim.data.TIRT}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.data.MGPCM.R
\name{sim.data.MGPCM}
\alias{sim.data.MGPCM}
\title{Simulate Data from the Multidimensional Generalized Partial Credit Model}
\usage{
sim.data.MGPCM(
  N = 500,
  I = 20,
  D = 2,
  length.poly = 5,
  Q.matrix = NULL,
  Corr = NULL,
  rotate = NULL,
  promax_m = 4
)
}
\arguments{
\item{N}{Integer; number of examinees (default: 500).}

\item{I}{Integer; number of items (default: 20).}

\item{D}{Integer; number of latent dimensions (default: 2).}

\item{length.poly}{Integer vector or scalar; number of categories per
item. Recycled to length \eqn{I} if scalar (default: 5).}

\item{Q.matrix}{Optional \eqn{I \times D} 0/1 matrix. \code{NULL} uses
a triangular identification pattern.}

\item{Corr}{Optional \eqn{D \times D} correlation matrix.}

\item{rotate}{Optional rotation method (\pkg{GPArotation} or
\code{"promax"}).}

\item{promax_m}{Power parameter for Promax (default: 4).}
}
\value{
An object of class \code{"data.MGPCM"}, a list containing:
\describe{
\item{\code{data}, \code{response}}{\eqn{N \times I} integer response
matrices with categories coded from 0 to \eqn{K_i - 1}.}
\item{\code{theta}}{\eqn{N \times D} matrix of true latent traits.}
\item{\code{par}}{\eqn{I \times (D + \max_i K_i)} matrix of item
parameters; discrimination columns are followed by category
intercept columns.}
\item{\code{probability}}{\eqn{N \times \sum_i K_i} matrix of stacked
category probabilities.}
\item{\code{Q.matrix}, \code{length.poly}, \code{Corr}}{Design matrix,
category counts, and latent correlation matrix used to generate
the data.}
\item{\code{model}, \code{N}, \code{I}, \code{D}, \code{call}}{
Data-generating metadata.}
}
}
\description{
Generates polytomous (multi-category) response data from the MGPCM.
Category probabilities use the same softmax formulation as
\code{\link{model.MGPCM}}: \eqn{\log P(Y=k)} is proportional to
\eqn{k\,\mathbf{a}_i'\boldsymbol{\theta}_j + d_{ik}}.
The requested category counts are retained even when a finite sample does
not contain every possible category.
}
\section{Data Generation Process}{

\enumerate{
\item \strong{Latent traits:}
\eqn{\boldsymbol{\theta}_j \sim N_D(\mathbf{0}, \boldsymbol{\Sigma})}.
\item \strong{Item parameters:}
\eqn{a_{id} \sim \text{Lognormal}(0.25, 0.25)} (active dimensions),
\eqn{d_{i0}=0}, and
\eqn{d_{i1},\dots,d_{i,K_i-1}} are sorted draws from
\eqn{N(0,1)} used as category intercepts.
\item \strong{Responses:}
\eqn{Y_{ij} \sim \text{Categorical}(P(Y_{ij} = k \mid \boldsymbol{\theta}_j))}
for \eqn{k = 0, \dots, K_i - 1}.
}
}

\examples{
set.seed(123)
sim <- sim.data.MGPCM(N = 20, I = 5, D = 2, length.poly = 4)
str(sim$response)
dim(sim$probability)
sim$length.poly

}
\seealso{
\code{\link{fit.MGPCM}}, \code{\link{model.MGPCM}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3vcov.R
\name{confint}
\alias{confint}
\alias{confint.MIRT}
\alias{confint.MGPCM}
\alias{confint.MGGUM}
\alias{confint.FCMIRT}
\alias{confint.FCDCM}
\alias{confint.FCGDINA}
\alias{confint.FCGGUM}
\alias{confint.TIRT}
\title{S3 Methods: confint}
\usage{
\method{confint}{MIRT}(object, parm = NULL, level = 0.95, ...)

\method{confint}{MGPCM}(object, parm = NULL, level = 0.95, ...)

\method{confint}{MGGUM}(object, parm = NULL, level = 0.95, ...)

\method{confint}{FCMIRT}(object, parm = NULL, level = 0.95, ...)

\method{confint}{FCDCM}(object, parm = NULL, level = 0.95, ...)

\method{confint}{FCGDINA}(object, parm = NULL, level = 0.95, ...)

\method{confint}{FCGGUM}(object, parm = NULL, level = 0.95, ...)

\method{confint}{TIRT}(object, parm = NULL, level = 0.95, ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{parm}{Optional numeric indices or character names selecting free
parameters. By default, intervals are returned for all free parameters.}

\item{level}{Numeric; confidence level (default: \code{0.95} for 95\%
intervals).}

\item{...}{Additional arguments (currently ignored).}
}
\value{
A numeric matrix with two columns, \code{2.5\%} and
\code{97.5\%} (column names adjust automatically for non-default
\code{level}). Each row corresponds to one free parameter.
}
\description{
Computes Wald-type confidence intervals for the model parameters of a
fitted \pkg{ForceChoice} model object. Intervals are constructed as
\eqn{\hat{\psi}_k \pm z_{1-\alpha/2}\,\mathrm{SE}_k}, where
\eqn{\hat{\psi}_k} is the point estimate and \eqn{\mathrm{SE}_k} is
the standard error of the \eqn{k}-th free parameter.
}
\section{Methods (by class)}{
\itemize{
\item \code{confint(MIRT)}: Confidence intervals for \code{MIRT} parameters.

\item \code{confint(MGPCM)}: Confidence intervals for \code{MGPCM} parameters.

\item \code{confint(MGGUM)}: Confidence intervals for \code{MGGUM} parameters.

\item \code{confint(FCMIRT)}: Confidence intervals for \code{FCMIRT} parameters.

\item \code{confint(FCDCM)}: Confidence intervals for \code{FCDCM} parameters.

\item \code{confint(FCGDINA)}: Confidence intervals for \code{FCGDINA} parameters.

\item \code{confint(FCGGUM)}: Confidence intervals for \code{FCGGUM} parameters.

\item \code{confint(TIRT)}: Confidence intervals for \code{TIRT} parameters.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools-data.R
\name{get.data.from.response.FCDCM}
\alias{get.data.from.response.FCDCM}
\title{Convert an FCDCM Response Matrix to FC Data Strings}
\usage{
get.data.from.response.FCDCM(response, block.items)
}
\arguments{
\item{response}{An \eqn{N \times B} integer matrix coded as 0/1 or
1/2, where \eqn{B} is the number of FCDCM blocks.}

\item{block.items}{A list of length \eqn{B}.  Each element is an
integer vector of length 2 giving the two global statement indices
compared in that block.}
}
\value{
A data frame with \eqn{N} rows and \eqn{B} columns.  Each
cell is a character string \code{"a>b"} or \code{"b>a"}.
}
\description{
Converts a binary FCDCM response matrix (0/1 or 1/2 coding) back to
human-readable \code{"a>b"} / \code{"b>a"} preference strings.  Each
block compares exactly two items; the response indicates which item
was selected.
}
\section{Response Coding}{

Two coding conventions are accepted:
\itemize{
\item \strong{0/1 coding}: 1 means the first-named item in
\code{block.items} was preferred; 0 means the second-named
item was preferred.
\item \strong{1/2 coding}: 1 means the first-named item; 2 means
the second-named item.  This is auto-detected and converted.
}
The output always uses \code{"a>b"} format where \code{a} and
\code{b} are the two item indices in the comparison order.

The direction is controlled by \code{block.items}.  If
\code{block.items[[1]] = c(1L, 3L)}, then response value 1 becomes
\code{"1>3"} and response value 0 becomes \code{"3>1"}.  With 1/2 coding,
value 1 has the same meaning as above and value 2 means the second item.
A response matrix must use one coding convention consistently; mixed
\code{0/1/2} values are rejected.
}

\examples{
items <- list(c(1L, 3L), c(2L, 4L))
resp <- cbind(c(1, 0, 1), c(0, 1, 1))
dat <- get.data.from.response.FCDCM(resp, items)
dat
get.response.from.data.FCDCM(dat, items)$response

# The same preferences can also be supplied as 1/2 responses.
resp12 <- cbind(c(1, 2, 1), c(2, 1, 1))
get.data.from.response.FCDCM(resp12, items)
}
\seealso{
\code{\link{get.response.from.data.FCDCM}} for the reverse
conversion.

\code{\link{get.data.from.response}} for the generic multi-item
converter.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-utils.R
\name{forced_choice_from_agree}
\alias{forced_choice_from_agree}
\title{Compute forced-choice pattern probabilities from item endorsement probabilities}
\usage{
forced_choice_from_agree(agree, patterns.total, patterns)
}
\arguments{
\item{agree}{An \eqn{N \times I} matrix of item endorsement probabilities.}

\item{patterns.total}{List of full-ranking pattern matrices (one per block).}

\item{patterns}{List of observed-pattern matrices (one per block).}
}
\value{
An \eqn{N \times \sum_B P_b} matrix of forced-choice pattern
probabilities, where \eqn{P_b} is the number of observed patterns for
block \eqn{b}.
}
\description{
Transforms item-level endorsement probabilities into block-level ranking
pattern probabilities via the sequential Luce/Plackett choice rule.
This is an R implementation of the C++ \code{forced_choice_from_agree}
function used by \code{cpp_model_FCMIRT} and \code{cpp_model_FCGGUM}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools-data.R
\name{get.block.items.from.data}
\alias{get.block.items.from.data}
\title{Extract Block Items from Forced-Choice Data}
\usage{
get.block.items.from.data(data)
}
\arguments{
\item{data}{An \eqn{N \times B} character matrix (or data frame) of
forced-choice responses.  Each entry encodes an item ordering using
integer item indices separated by \code{">"} (e.g. \code{"3>1>2"}).
Missing values (\code{NA}) are permitted and skipped.}
}
\value{
A list of length \eqn{B}.  Each element is an integer vector of
global item indices appearing in that block, sorted in ascending order.
}
\description{
Infers the item composition of each forced-choice block from a character
data matrix.  Each cell should encode the within-block item ordering
(e.g. \code{"2>1>3"} for a rank-3 block).  The function parses all
non-missing cells per block and returns sorted, unique item indices.

This function is the recommended way to recover \code{block.items} when
only character-rank data are available.  It works for all forced-choice
types (\code{"RANK"}, \code{"MOLE"}, \code{"PICK"}) as long as every
item appears at least once across persons.
}
\details{
The returned \code{block.items} list defines the block layout used by the
other converters.  If \code{data} has \eqn{B} columns, the result has
\eqn{B} elements, and result element \eqn{b} belongs to data column
\eqn{b}.  Item labels are treated as global item indices; they are not
recoded to local positions.

The function scans observed strings only.  Therefore, for partial-response
formats, it can only recover items that appear in the observed partial
strings:
\itemize{
\item For \code{"RANK"}, a complete ranking normally contains every item
in the block, so one valid row can be sufficient.
\item For \code{"MOLE"}, only most- and least-preferred items appear in
each cell.  An item that is never chosen as most or least cannot be
recovered from \code{data} alone.
\item For \code{"PICK"}, only picked items appear.  Items that are never
picked cannot be recovered from \code{data} alone.
}
In those partial-response cases, pass a complete user-defined
\code{block.items} list to the fitting or conversion function when the raw
data do not mention every item.
}
\examples{
# 3 persons, 2 blocks.
# Block 1 contains items 1 and 2; block 2 contains items 3, 4, and 5.
dat <- data.frame(
  block.1 = c("1>2", "2>1", "1>2"),
  block.2 = c("3>4>5", "5>4>3", "4>3>5"),
  stringsAsFactors = FALSE
)
get.block.items.from.data(dat)
}
\seealso{
\code{\link{get.block.items.from.data.FCDCM}} for the FCDCM (binary-pair)
variant.

\code{\link{get.response.from.data}} to convert character data to an
integer pattern-index matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.data.MIRT.R
\name{model.MIRT}
\alias{model.MIRT}
\title{Compute MIRT Item Response Probabilities}
\usage{
model.MIRT(theta, par)
}
\arguments{
\item{theta}{An \eqn{N \times D} matrix of latent trait values.}

\item{par}{An \eqn{I \times (D+3)} parameter matrix with columns
\code{a1..aD, b, c, d}, where \code{b} is the item difficulty.
The linear predictor is \eqn{\mathbf{a}_i'\boldsymbol{\theta}_j - b_i}.}
}
\value{
An \eqn{N \times I} matrix of response probabilities
\eqn{P(Y_{ij} = 1 \mid \boldsymbol{\theta}_j)}.
}
\description{
Computes the probability of a correct response for each person--item
combination under the multidimensional 1PL--4PL models. This is the
core IRF (item response function) used throughout the package.
}
\seealso{
\code{\link{fit.MIRT}} for the full model specification.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools-mcmc.R
\name{make_quadrature_grid_mvn}
\alias{make_quadrature_grid_mvn}
\title{Multivariate normal quadrature grid with Cholesky-stabilised weights}
\usage{
make_quadrature_grid_mvn(
  Corr,
  theta.mu,
  D,
  L = NULL,
  theta.low = -6,
  theta.up = 6
)
}
\arguments{
\item{Corr}{D x D correlation matrix (can be \code{NULL} for identity).}

\item{theta.mu}{Numeric vector of length D (latent mean).}

\item{D}{Number of dimensions.}

\item{L}{Number of quadrature points per dimension, or \code{NULL} to
auto-select from a built-in table.}

\item{theta.low, theta.up}{Quadrature range.}
}
\value{
A list with \code{theta.norm} (grid matrix) and \code{pi}
(normalised prior-weight column vector).
}
\description{
Used by all continuous-trait \code{logLik()} methods.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-utils.R
\name{get_design_matrix_cdm}
\alias{get_design_matrix_cdm}
\title{Build the CDM design matrix for a set of attribute patterns}
\usage{
get_design_matrix_cdm(patterns, model = "GDINA")
}
\arguments{
\item{patterns}{A binary matrix of attribute mastery patterns
(\eqn{L \times K}).}

\item{model}{One of \code{"DINA"}, \code{"DINO"}, \code{"ACDM"}, \code{"GDINA"}.}
}
\value{
A numeric design matrix \eqn{X} with \eqn{L} rows.  Column count
depends on the model.
}
\description{
Constructs the design matrix \eqn{X} used in the cognitive diagnostic
model probability formulation \eqn{p = X\delta}.  The structure depends
on the model type:
\describe{
\item{DINA}{Intercept + single mastery indicator (all attributes mastered).}
\item{DINO}{Intercept + single "at-least-one" indicator.}
\item{ACDM}{Intercept + main effects of each attribute.}
\item{GDINA}{Full saturated interaction matrix (all \eqn{2^K} columns).}
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.data.FCMIRT.R
\name{sim.data.FCMIRT}
\alias{sim.data.FCMIRT}
\title{Simulate FCMIRT Data}
\usage{
sim.data.FCMIRT(
  N.person = 1000,
  N.block = 10,
  I.block = 2,
  D = 3,
  model = "m2pl",
  fc.type = "RANK",
  control = NULL
)
}
\arguments{
\item{N.person}{Number of persons.}

\item{N.block}{Number of forced-choice blocks.}

\item{I.block}{Number of items per block when \code{control$block.items} is
not supplied.}

\item{D}{Number of latent dimensions. If \code{control$Q.matrix} is supplied,
\code{D} is reset to \code{ncol(control$Q.matrix)}.}

\item{model}{MIRT model type: \code{"m1pl"}, \code{"m2pl"}, \code{"m3pl"}, or
\code{"m4pl"}.}

\item{fc.type}{Forced-choice response type: \code{"RANK"}, \code{"MOLE"}, or
\code{"PICK"}. A scalar value is recycled to all blocks.}

\item{control}{Optional list. Supported entries are \code{Q.matrix},
\code{block.items}, and \code{Corr}.}
}
\value{
An object of class \code{"data.FCMIRT"}, a list containing
\code{data}, \code{response}, \code{theta}, \code{par}, \code{Q.matrix},
\code{block.items}, \code{Corr}, \code{model}, \code{patterns},
\code{patterns.total}, \code{probability}, \code{prob}, and
data-generating arguments.
}
\description{
Simulates forced-choice ranking data using a MIRT item response model
(\code{"m1pl"} through \code{"m4pl"}) as the item-level endorsement model.
Forced-choice responses are generated by applying the sequential
Luce/Plackett choice rule to the logits of the item-level endorsement
probabilities, with RANK, MOLE, and PICK patterns handled by
\code{\link{model.FCMIRT}}.
For identification, item difficulty/intercept parameters \code{b} are
centered within each forced-choice block so that their block sum is zero.
}
\examples{
set.seed(123)
sim <- sim.data.FCMIRT(N.person = 20, N.block = 3, I.block = 2,
                       D = 2, model = "m2pl", fc.type = "RANK")
str(sim$data)
head(sim$response)
dim(sim$prob)

}
\seealso{
\code{\link{fit.FCMIRT}}, \code{\link{model.FCMIRT}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3plot.R
\name{plot}
\alias{plot}
\alias{plot.MIRT}
\alias{plot.MGPCM}
\alias{plot.MGGUM}
\alias{plot.FCMIRT}
\alias{plot.FCDCM}
\alias{plot.FCGGUM}
\alias{plot.FCGDINA}
\alias{plot.TIRT}
\title{S3 Methods: plot}
\usage{
\method{plot}{MIRT}(x, y = NULL, ...)

\method{plot}{MGPCM}(x, y = NULL, ...)

\method{plot}{MGGUM}(x, y = NULL, ...)

\method{plot}{FCMIRT}(x, y = NULL, ...)

\method{plot}{FCDCM}(x, y = NULL, ...)

\method{plot}{FCGGUM}(x, y = NULL, ...)

\method{plot}{FCGDINA}(x, y = NULL, ...)

\method{plot}{TIRT}(x, y = NULL, ...)
}
\arguments{
\item{x}{A fitted model object of class \code{"MIRT"},
\code{"MGPCM"}, \code{"MGGUM"}, \code{"FCMIRT"}, \code{"FCDCM"},
\code{"FCGGUM"}, \code{"FCGDINA"}, or \code{"TIRT"}.}

\item{y}{Ignored.}

\item{...}{Ignored.}
}
\value{
Invisibly returns \code{NULL}. Plots are produced as a side
effect on the current graphics device.
}
\description{
Draws log-likelihood trace plots for fitted \pkg{ForceChoice} model
objects. Stan, iStEM, and EM fits are visualised through the same
convergence target: the total log-likelihood recorded across iterations
of the corresponding estimation algorithm.
}
\details{
The horizontal axis is the saved Stan iteration, iStEM iteration, or EM
iteration. The vertical axis is the summed log-likelihood. Stan plots use
\code{log_lik} or \code{log_lik_group} from generated quantities and draw
all chains in one panel. iStEM plots use the iteration-level
\code{iStEM$logLik.trace}, and the stable chain is the final
\code{M * B} iStEM iterations. EM plots use \code{EM$logLik.trace}. For
Stan and iStEM fits, a vertical dashed line marks the stable-chain
boundary. EM fits do not have a stable-chain boundary.
}
\section{Methods (by class)}{
\itemize{
\item \code{plot(MIRT)}: Log-likelihood trace plot for \code{MIRT} objects.

\item \code{plot(MGPCM)}: Log-likelihood trace plot for \code{MGPCM} objects.

\item \code{plot(MGGUM)}: Log-likelihood trace plot for \code{MGGUM} objects.

\item \code{plot(FCMIRT)}: Log-likelihood trace plot for \code{FCMIRT} objects.

\item \code{plot(FCDCM)}: Log-likelihood trace plot for \code{FCDCM} objects.

\item \code{plot(FCGGUM)}: Log-likelihood trace plot for \code{FCGGUM} objects.

\item \code{plot(FCGDINA)}: Log-likelihood trace plot for \code{FCGDINA} objects.

\item \code{plot(TIRT)}: Log-likelihood trace plot for \code{TIRT} objects.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3loo.R
\name{get.log_lik}
\alias{get.log_lik}
\alias{get.log_lik.default}
\title{Extract pointwise log-likelihood matrix}
\usage{
get.log_lik(object, ...)

\method{get.log_lik}{default}(object, ...)
}
\arguments{
\item{object}{A fitted ForceChoice model object.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
An \eqn{S \times N} matrix of pointwise log-likelihood draws
(Stan), or \code{NULL} (EM / iStEM).
}
\description{
For Stan-fitted models, returns the posterior draws of pointwise
log-likelihood (an \eqn{S \times N_{\text{obs}}} matrix for most models,
or \eqn{S \times N} for FCDCM and FCGDINA).  For EM and iStEM fits,
returns \code{NULL}.
}
\details{
The returned matrix can be passed directly to \code{\link[loo]{loo}} or
\code{\link[loo]{waic}} for model comparison.
}
\section{Methods (by class)}{
\itemize{
\item \code{get.log_lik(default)}: Default method; returns \code{object$log_lik}
when present and \code{NULL} otherwise.

}}
\examples{
\donttest{
# stan code, long time
sim <- sim.data.MIRT(N = 20, I = 6, D = 2, model = "2PL")
fit <- fit.MIRT(sim$response, model = "2PL", D = 2, method = "stan")
log_lik <- get.log_lik(fit)
loo::loo(log_lik)
loo::waic(log_lik)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.data.FCGGUM.R
\name{sim.data.FCGGUM}
\alias{sim.data.FCGGUM}
\title{Simulate Data from the Forced-Choice GGUM Model}
\usage{
sim.data.FCGGUM(
  N.person = 1000,
  N.block = 10,
  I.block = 2,
  D = 3,
  fc.type = "RANK",
  control = NULL
)
}
\arguments{
\item{N.person}{Integer; number of persons (default: 1000).}

\item{N.block}{Integer; number of forced-choice blocks (default: 10).}

\item{I.block}{Integer; items per block (default: 2).}

\item{D}{Integer; number of latent dimensions (default: 3, must be >= 2).}

\item{fc.type}{Character; \code{"RANK"} (default), \code{"MOLE"}, or
\code{"PICK"}.}

\item{control}{Optional list with entries \code{Q.matrix},
\code{block.items}, and \code{Corr}.}
}
\value{
An object of class \code{"data.FCGGUM"} containing \code{data},
\code{response}, \code{theta}, \code{par}, \code{Q.matrix},
\code{block.items}, \code{Corr}, \code{patterns}, \code{patterns.total},
\code{prob}, \code{mask}, and data-generating arguments.
}
\description{
Generates forced-choice ranking data using the GGUM ideal-point model as the
item-level endorsement process. Within-block rankings follow the same
sequential Luce/Plackett mechanism as \code{\link{model.FCGGUM}}: the
utility for each statement is the logit of its binary GGUM endorsement
probability.
}
\examples{
set.seed(123)
sim <- sim.data.FCGGUM(N.person = 20, N.block = 3, I.block = 2,
                       D = 2, fc.type = "RANK")
str(sim$data)
head(sim$response)
dim(sim$prob)

}
\seealso{
\code{\link{fit.FCGGUM}}, \code{\link{model.FCGGUM}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ForceChoice-package.R
\docType{package}
\name{ForceChoice-package}
\alias{ForceChoice}
\alias{ForceChoice-package}
\title{ForceChoice: Forced-Choice Modeling Based on IRT and CDM}
\description{
The \pkg{ForceChoice} package provides a unified framework for fitting,
simulating, and evaluating forced-choice and traditional item response
models. It supports eight model families spanning dominance (MIRT),
ideal-point/unfolding (MGGUM), polytomous (MGPCM), and forced-choice
(FCMIRT, FCGGUM, TIRT, FCDCM, FCGDINA) paradigms, with full Bayesian
estimation via Stan, fast improved stochastic EM (iStEM), and a
deterministic EM backend for FCGDINA.
}
\section{Model Families}{


\strong{Traditional (single-stimulus) models:}
\describe{
\item{\code{\link{fit.MIRT}}}{Multidimensional IRT (1PL--4PL).
Binary responses with optional Q-matrix structure.}
\item{\code{\link{fit.MGPCM}}}{Multidimensional Generalized Partial
Credit Model. Polytomous ordered-category responses.}
\item{\code{\link{fit.MGGUM}}}{Multidimensional Generalized Graded
Unfolding Model. Ideal-point polytomous responses.}
}

\strong{Forced-choice (comparative) models:}
\describe{
\item{\code{\link{fit.FCMIRT}}}{Forced-choice MIRT. Item-level
dominance model with sequential ranking over item endorsement
logits at the block level.}
\item{\code{\link{fit.FCGGUM}}}{Forced-choice GGUM. Item-level
ideal-point model with sequential ranking over binary GGUM
endorsement logits.}
\item{\code{\link{fit.TIRT}}}{Thurstonian IRT for forced-choice.
Pairwise probit comparison of latent utility differences.}
\item{\code{\link{fit.FCDCM}}}{Forced-choice DCM. Higher-order
cognitive diagnostic model with exact attribute-profile
marginalization.}
\item{\code{\link{fit.FCGDINA}}}{Forced-choice GDINA. General CDM
item-response structure with forced-choice block likelihood.}
}
}

\section{Estimation Methods}{


Model families support a common estimation interface:
\describe{
\item{\strong{Stan} (\code{method = "stan"}):}{
Full Bayesian inference via Hamiltonian Monte Carlo
(NUTS/HMC). Provides posterior means, standard deviations,
and convergence diagnostics (\eqn{\hat{R}}).}
\item{\strong{iStEM} (\code{method = "iStEM"}):}{
Improved Stochastic EM (iStEM) algorithm
combining finite-grid block Gibbs person sampling with
L-BFGS-B item-parameter optimization. Scales well to moderate
data sets.}
\item{\strong{EM} (\code{method = "EM"}):}{
Deterministic posterior-weight EM for FCGDINA. Useful as a
reproducible baseline or fast diagnostic estimator.}
}
}

\section{Core Workflow}{


A typical analysis follows four steps:
\enumerate{
\item \strong{Simulate} (or load) data with \code{sim.data.*()}.
\item \strong{Fit} a model with \code{fit.*()}.
\item \strong{Rotate} the solution if needed with
the \code{\link{rotate}} S3 generic.
\item \strong{Evaluate} fit with \code{\link{get.fit.index}()}
and \code{\link{summary.good.of.fit}()}.
}
}

\section{Goodness-of-Fit}{


The package uses the limited-information M2 framework
(Maydeu-Olivares & Joe, 2005, 2006) implemented in
\code{\link{good.of.fit}}. Fit indices include:
\itemize{
\item Information criteria: AIC, AICc, BIC, CAIC, SABIC, HQIC
\item Absolute fit: M2, RMSEA (with CI), SRMSR, McDonald NCI
\item Comparative fit: CFI, TLI, IFI
\item Pseudo-\eqn{R^2}: McFadden, Cox--Snell, Nagelkerke, etc.
\item Local dependence: Yen's Q3 (raw and adjusted)
\item Classification: posterior entropy, mean max posterior
}
}

\section{Reproducibility}{


The package is designed so that simulation, estimation, extraction, and
model checking can be scripted end to end. Simulation functions return the
true parameters and the effective call arguments; fitted objects retain the
original input arguments, estimates, convergence diagnostics, and
likelihood-related quantities used by \code{\link{logLik}} and
\code{\link{get.fit.index}}. For computationally expensive Stan or iStEM
workflows, manuscript replication code should set seeds explicitly and use
a small demonstration configuration in addition to any full-scale analysis.
}

\examples{
set.seed(123)
sim <- sim.data.FCGDINA(N.person = 12, N.block = 2, I.block = 2,
                        D = 2, model = "DINA")
str(sim$response)
head(sim$Q.matrix)

}
\author{
\strong{Maintainer}: Haijiang Qin \email{haijiang133@outlook.com} (\href{https://orcid.org/0009-0000-6721-5653}{ORCID}) [copyright holder]

Authors:
\itemize{
  \item Lei Guo \email{happygl1229@swu.edu.cn} (\href{https://orcid.org/0000-0002-8273-3587}{ORCID}) [copyright holder]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{validate_corr_matrix}
\alias{validate_corr_matrix}
\title{Validate a user-supplied correlation matrix}
\usage{
validate_corr_matrix(Corr, D, label = "'Corr'")
}
\arguments{
\item{Corr}{A D x D matrix or \code{NULL}.}

\item{D}{Expected number of dimensions.}

\item{label}{Character label for error messages.}
}
\value{
A valid D x D correlation matrix.
}
\description{
Checks dimensions, symmetry, unit diagonal, and positive semi-definiteness.
Returns a valid correlation matrix (or \code{diag(D)} when \code{NULL}).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3predict.R
\name{predict}
\alias{predict}
\alias{predict.MIRT}
\alias{predict.MGPCM}
\alias{predict.MGGUM}
\alias{predict.FCMIRT}
\alias{predict.FCDCM}
\alias{predict.FCGDINA}
\alias{predict.FCGGUM}
\alias{predict.TIRT}
\title{S3 Methods: predict}
\usage{
\method{predict}{MIRT}(object, newdata = NULL, type = c("probability", "response"), ...)

\method{predict}{MGPCM}(object, newdata = NULL, type = c("probability", "response"), ...)

\method{predict}{MGGUM}(object, newdata = NULL, type = c("probability", "response"), ...)

\method{predict}{FCMIRT}(object, newdata = NULL, type = c("probability", "response"), ...)

\method{predict}{FCDCM}(object, newdata = NULL, type = c("probability", "response"), ...)

\method{predict}{FCGDINA}(object, newdata = NULL, type = c("probability", "response"), ...)

\method{predict}{FCGGUM}(object, newdata = NULL, type = c("probability", "response"), ...)

\method{predict}{TIRT}(object, newdata = NULL, type = c("probability", "response"), ...)
}
\arguments{
\item{object}{A fitted model object of class \code{"MIRT"},
\code{"MGPCM"}, \code{"MGGUM"}, \code{"FCMIRT"}, \code{"FCDCM"},
\code{"FCGDINA"}, \code{"FCGGUM"}, or \code{"TIRT"}.}

\item{newdata}{An optional \eqn{M \times D} matrix of latent trait
values at which to compute predictions. If \code{NULL} (default),
predictions use the estimated \code{object$theta$est}.}

\item{type}{Character; \code{"probability"} (default) returns
response probabilities, \code{"response"} returns expected category
or simulated binary responses (random draws).}

\item{...}{Additional arguments (currently ignored).}
}
\value{
\describe{
\item{\code{type = "probability"}}{For binary models (MIRT, TIRT):
an \eqn{M \times I} matrix of endorsement probabilities. For
polytomous models (MGPCM, MGGUM): an \eqn{M \times \sum K_i}
matrix of stacked category probabilities. For forced-choice
models (FCMIRT, FCGGUM): an \eqn{M \times \sum K_b} matrix of
block-pattern probabilities. For FCDCM: an \eqn{M \times B}
matrix of marginal block-choice probabilities.}
\item{\code{type = "response"}}{For binary models: an
\eqn{M \times I} matrix of simulated 0/1 responses. For
polytomous models: an \eqn{M \times I} matrix of category
indices. For forced-choice models: an \eqn{M \times B} matrix
of chosen pattern indices.}
}
}
\description{
Predict response probabilities from fitted \pkg{ForceChoice} model
objects. Given a fitted model and optionally new person parameter
(latent trait) values, returns the model-implied response probabilities.
When \code{newdata} is not provided, predictions are made at the
estimated latent trait values for each person.
}
\section{Methods (by class)}{
\itemize{
\item \code{predict(MIRT)}: Predict response probabilities or simulated
responses from \code{MIRT} objects.

\item \code{predict(MGPCM)}: Predict response probabilities from \code{MGPCM} objects.

\item \code{predict(MGGUM)}: Predict response probabilities from \code{MGGUM} objects.

\item \code{predict(FCMIRT)}: Predict response probabilities from \code{FCMIRT} objects.

\item \code{predict(FCDCM)}: Predict response probabilities from \code{FCDCM} objects.

\item \code{predict(FCGDINA)}: Predict response probabilities from \code{FCGDINA} objects.
For \code{newdata}, provide either binary alpha profiles or class
probability rows.

\item \code{predict(FCGGUM)}: Predict response probabilities from \code{FCGGUM} objects.

\item \code{predict(TIRT)}: Predict response probabilities from \code{TIRT} objects.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3summary.R
\name{summary}
\alias{summary}
\alias{summary.good.of.fit}
\alias{summary.MIRT}
\alias{summary.MGPCM}
\alias{summary.MGGUM}
\alias{summary.FCMIRT}
\alias{summary.FCDCM}
\alias{summary.FCGDINA}
\alias{summary.FCGGUM}
\alias{summary.TIRT}
\title{S3 Methods: summary}
\usage{
\method{summary}{good.of.fit}(object, digits = 4, ...)

\method{summary}{MIRT}(object, digits = 4, ...)

\method{summary}{MGPCM}(object, digits = 4, ...)

\method{summary}{MGGUM}(object, digits = 4, ...)

\method{summary}{FCMIRT}(object, digits = 4, ...)

\method{summary}{FCDCM}(object, digits = 4, ...)

\method{summary}{FCGDINA}(object, digits = 4, ...)

\method{summary}{FCGGUM}(object, digits = 4, ...)

\method{summary}{TIRT}(object, digits = 4, ...)
}
\arguments{
\item{object}{An object of one of the following classes:
\itemize{
\item Fitted model objects: \code{"MIRT"}, \code{"MGPCM"},
\code{"MGGUM"}, \code{"FCMIRT"}, \code{"FCDCM"}, \code{"FCGGUM"},
\code{"FCGDINA"}, \code{"TIRT"}
\item Goodness-of-fit objects: \code{"good.of.fit"}
}}

\item{digits}{Number of decimal places for numeric output (default: 4).
Applied uniformly across all methods.}

\item{...}{Additional arguments passed to or from other methods (currently
ignored).}
}
\value{
Invisibly returns a structured list containing summary components.
The exact structure depends on the class of \code{object}. All returned
objects carry an appropriate S3 class (e.g., \code{"summary.MIRT"},
\code{"summary.good.of.fit"}) for use with corresponding \code{print}
methods.
}
\description{
Generates structured, comprehensive summaries of objects produced by the
\pkg{ForceChoice} package. This generic function dispatches to
class-specific methods that extract and organize key information including
model configurations, fit statistics, parameter estimates, and convergence
diagnostics. Designed for programmatic access and downstream reporting.
}
\details{
Each method returns a named list with class-specific components optimized
for structured access and formatted printing:

\describe{
\item{\strong{Fitted Model Objects}}{
Returns a \code{summary.<Class>} object with components:
\describe{
\item{\code{call}}{Original function call.}
\item{\code{model.info}}{List: \code{family}, \code{description},
\code{D}, \code{method}, and model-specific fields (\code{model},
\code{fc.type}, \code{dcm.type}, \code{I.states}, \code{N.block}).}
\item{\code{data.info}}{List: \code{N}, \code{I} or \code{B},
\code{response.type}.}
\item{\code{fit.stats}}{List: \code{LogLik}, \code{npar},
\code{AIC}, \code{BIC}.}
\item{\code{par.summary}}{Matrix: parameter summary (Min, Q1, Median,
Mean, Q3, Max) across items/statements/blocks for each parameter type.}
\item{\code{theta.summary}}{Matrix: person parameter summary per
dimension.}
\item{\code{Corr}}{Matrix: factor inter-trait correlation matrix
(where applicable).}
\item{\code{convergence}}{List: algorithm, batch counts, convergence
flags, latent grid length \code{L} (iStEM) or diagnostic note (Stan).}
\item{\code{digits}}{Numeric: precision used for formatting.}
}
Additional model-specific components:
\describe{
\item{\code{delta.summary}}{FCDCM only: higher-order delta parameters.}
\item{\code{classification}}{FCDCM only: observed attribute patterns,
mean maximum posterior.}
\item{\code{lambda.summary}}{TIRT only: factor loading summary.}
}
}

\item{\strong{Goodness-of-Fit Objects (\code{good.of.fit})}}{
Returns a \code{summary.good.of.fit} object with components:
\describe{
\item{\code{Sample_Size}, \code{Items}, \code{Response_Type},
\code{Parameters}, \code{Moments}, \code{LogLik},
\code{Deviance}}{Overview statistics.}
\item{\code{IC}}{Named vector: AIC, AICc, BIC, CAIC, SABIC, HQIC.}
\item{\code{M2}}{List: M2 statistic, df, p-value, RMSEA with CI.}
\item{\code{Residual_Fit}}{Named vector: SRMSR, RMSR, max
standardized residual.}
\item{\code{Comparative_Fit}}{Named vector: CFI, TLI, IFI.}
\item{\code{Pseudo_R2}}{Named vector: McFadden, Cox--Snell,
Nagelkerke, Aldrich--Nelson, Veall--Zimmermann.}
\item{\code{Local_Dependence}}{Named vector: Q3 mean, max, adjusted
max, P95.}
\item{\code{Classification}}{Named vector: posterior entropy,
max posterior.}
\item{\code{Null_Model}}{Named vector: null model LogLik, deviance,
M2, df.}
\item{\code{tables}}{List of data frames for formatted printing.}
}
}
}
}
\section{Methods (by class)}{
\itemize{
\item \code{summary(good.of.fit)}: Summary method for \code{good.of.fit} objects.
Extracts and structures all fit indices: information criteria,
limited-information M2, RMSEA, SRMSR, CFI/TLI/IFI, pseudo-\eqn{R^2},
local dependence (Yen's Q3), posterior classification, and null-model
diagnostics.

\item \code{summary(MIRT)}: Summary method for \code{MIRT} objects.
Multidimensional IRT (1PL--4PL): extracts binary-response model
configuration, fit statistics (LogLik, AIC, BIC), factor correlation
matrix, person and item parameter summaries, and convergence
diagnostics.

\item \code{summary(MGPCM)}: Summary method for \code{MGPCM} objects.
Multidimensional Generalized Partial Credit Model: polytomous
responses with category-specific step parameters.

\item \code{summary(MGGUM)}: Summary method for \code{MGGUM} objects.
Multidimensional Generalized Graded Unfolding Model: ideal-point
polytomous responses with discrimination (a), location (delta), and
threshold (tau) parameters.

\item \code{summary(FCMIRT)}: Summary method for \code{FCMIRT} objects.
Forced-Choice Multidimensional IRT: dominance model with sequential
ranking over item endorsement logits at the block level. Extracts
block configuration, FC type
(RANK/MOLE/PICK), and statement-level parameter summaries.

\item \code{summary(FCDCM)}: Summary method for \code{FCDCM} objects.
Forced-Choice Diagnostic Classification Model: higher-order trait with
exact marginalization over \eqn{2^D} attribute profiles. Extracts
DCM type (DINA/DINO), higher-order delta parameters, block eta
parameters, and posterior classification diagnostics.

\item \code{summary(FCGDINA)}: Summary method for \code{FCGDINA} objects.
Forced-Choice GDINA model: CDM item model (DINA/DINO/ACDM/GDINA) with
sequential forced-choice ranking over item endorsement logits.

\item \code{summary(FCGGUM)}: Summary method for \code{FCGGUM} objects.
Forced-Choice Generalized Graded Unfolding Model: ideal-point model
with sequential ranking over binary GGUM endorsement logits. Extracts
forced-choice block structure, unfolding parameters (a, delta, tau),
and convergence info.

\item \code{summary(TIRT)}: Summary method for \code{TIRT} objects.
Thurstonian IRT for Forced-Choice: pairwise probit comparison of latent
utility differences. Extracts statement loadings (lambda), uniquenesses
(psi2), pairwise gamma matrix, and factor correlations.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/istem-helpers.R
\name{resolve_D}
\alias{resolve_D}
\title{Resolve the number of latent dimensions}
\usage{
resolve_D(D, Q.matrix = NULL, block.items = NULL, I = NULL)
}
\arguments{
\item{D}{User-supplied D or NULL.}

\item{Q.matrix}{Optional Q-matrix.}

\item{block.items}{Deprecated internal argument; ignored.}

\item{I}{Deprecated internal argument; ignored.}
}
\value{
Integer D.
}
\description{
Uses an explicit \eqn{D}, or infers it from the number of columns in a
supplied Q-matrix. Latent dimensionality is never guessed from the number
of items or blocks.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/istem-helpers.R
\name{resolve_block_items}
\alias{resolve_block_items}
\title{Auto-detect block.items from forced-choice ranking data}
\usage{
resolve_block_items(block.items, data)
}
\arguments{
\item{block.items}{User-supplied block.items or NULL.}

\item{data}{The forced-choice data matrix.}
}
\value{
A list of integer vectors (one per block), or NULL if
auto-detection is not possible.
}
\description{
Parses ranking strings like \code{"2>1>3"} to extract the set of item
indices in each block.  Only works when data columns contain character
ranking strings.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.FCGDINA.R
\name{fit.FCGDINA}
\alias{fit.FCGDINA}
\title{Fit the Forced-Choice GDINA Model}
\usage{
fit.FCGDINA(
  data,
  Q.matrix,
  model = c("GDINA", "DINA", "DINO", "ACDM"),
  block.items = NULL,
  fc.type = NULL,
  method = c("EM", "stan", "iStEM"),
  control.model = NULL,
  control.method = NULL
)
}
\arguments{
\item{data}{An \eqn{N \times B} forced-choice data matrix.}

\item{Q.matrix}{An \eqn{I \times D} binary Q-matrix. Rows correspond to
statements, columns correspond to attributes, and every row must contain
at least one \code{1}.}

\item{model}{CDM item model: \code{"GDINA"}, \code{"DINA"},
\code{"DINO"}, or \code{"ACDM"}.}

\item{block.items}{A list of length \eqn{B}. Each element gives the global
statement indices in the corresponding forced-choice block. If
\code{NULL}, the block structure is parsed from full-ranking character
data when possible.}

\item{fc.type}{Forced-choice response type: \code{"RANK"}, \code{"MOLE"},
or \code{"PICK"}. A scalar is recycled to all blocks. If \code{NULL},
the type is inferred from character data when possible and otherwise
defaults to \code{"RANK"}.}

\item{method}{Estimation method. \code{"EM"} (default) runs a
deterministic posterior-weight EM algorithm, \code{"stan"} runs full
Bayesian MCMC, and \code{"iStEM"} runs improved stochastic EM.}

\item{control.model}{Optional list of model-level controls. For Stan,
\code{delta.mu} and \code{delta.sigma} set the normal prior
\eqn{\delta \sim N(\mu, \sigma)} for the free Stan FCGDINA logit-scale
delta effects (defaults: 0 and 1). \code{pi.prior.alpha} sets the
Dirichlet concentration for the structural parameters \eqn{\pi}
(default 1, uniform prior over the simplex). For EM and iStEM, Stan
priors are ignored; use \code{delta.lower} and \code{delta.upper} in
\code{control.method} instead.}

\item{control.method}{Optional list of method-specific controls.}
}
\value{
An object of class \code{"FCGDINA"}, a list containing:
\describe{
\item{\code{npar}}{Number of free CDM delta parameters.}
\item{\code{method}}{Estimation method actually used:
\code{"stan"}, \code{"iStEM"}, or \code{"EM"}.}
\item{\code{alpha}}{List with \code{est}, \code{se}, \code{Rhat}, and
\code{prob}. \code{est} is an \eqn{N \times D} matrix of posterior
attribute mastery probabilities; \code{prob} contains posterior
probabilities for all \eqn{2^D} attribute profiles.}
\item{\code{delta}}{List with \code{est}, \code{se}, and \code{Rhat}.
For Stan fits, each element is the probability-scale transform of
the identified minimum-norm logit coefficient vector. Absolute
statement endorsement probabilities are not identified by
forced-choice data alone.}
\item{\code{delta.identification}}{Stan-only rank, nullity, blockwise
rank diagnostics, and the identifying convention.}
\item{\code{delta.store}}{Stored posterior, iStEM, or bootstrap delta
draws when available.}
\item{\code{Q.matrix}, \code{block.items}, \code{patterns},
\code{patterns.total}, \code{design.matrix.list},
\code{alpha.patterns}, \code{fc.type}, \code{response}}{Processed
model and data structures used by prediction, fit indices, and
S3 methods.}
\item{\code{stan.obj}, \code{MCMC.obj}}{Stan fit and extracted posterior
draws for \code{method = "stan"}; otherwise \code{NULL}.}
\item{\code{iStEM}}{iStEM convergence diagnostics and chains for
\code{method = "iStEM"}; otherwise \code{NULL}.}
\item{\code{EM}}{EM convergence diagnostics, log-likelihood trace, and
bootstrap summary for \code{method = "EM"}; otherwise
\code{NULL}.}
\item{\code{logLik}}{Marginal log-likelihood with class \code{"logLik"}.}
\item{\code{call}, \code{arguments}}{Matched call and effective fitting
arguments.}
}
}
\description{
Fits a forced-choice cognitive diagnostic model (FCGDINA) to comparative
response data. The statement-level cognitive diagnosis component can be
DINA, DINO, ACDM, or GDINA, and the block-level response model transforms
statement endorsement probabilities into forced-choice probabilities for
full rankings (\code{"RANK"}), most-least choices (\code{"MOLE"}), or
best-only choices (\code{"PICK"}).
}
\section{Model Specification}{


Let \eqn{\boldsymbol{\alpha}_n \in \{0, 1\}^D} denote the latent attribute
profile for person \eqn{n}. For statement \eqn{i}, the binary Q-vector
\eqn{\mathbf{q}_i} selects the required attributes and the reduced
attribute pattern is mapped to a CDM design row \eqn{\mathbf{x}_{ic}}.
For Stan estimation, \eqn{\boldsymbol{\delta}_i} is represented directly
as the item-specific CDM effect vector on the logit scale: intercept,
attribute main effects, and interaction effects. The item-level endorsement
probability for attribute class \eqn{c} is
\deqn{
  p_{ic} = \mathrm{logit}^{-1}(\mathbf{x}_{ic}' \boldsymbol{\delta}_i),
}
so main effects and interactions may be negative while probabilities remain
in \eqn{(0, 1)}. Forced-choice observations identify only within-block
utility contrasts, not the absolute endorsement logits of individual
statements. The Stan implementation therefore removes the exact null space
of those contrasts and reports the unique minimum-norm coefficient
representative. The design matrix is generated according to \code{model}:
\code{"DINA"}, \code{"DINO"}, \code{"ACDM"}, or \code{"GDINA"}. EM and
iStEM retain the existing probability-scale CDM parameterisation.

For each forced-choice block, the statement endorsement probabilities are
converted to ranking-pattern probabilities by the same sequential
Luce--Plackett transformation used by \code{\link{fit.FCMIRT}} and
\code{\link{fit.FCGGUM}}. The likelihood marginalizes over all
\eqn{2^D} attribute profiles using a uniform latent-class prior:
\deqn{
  P(\mathbf{Y}_n) =
  \sum_{c=1}^{2^D} \pi_c
  \prod_{b=1}^{B} P(Y_{nb} \mid \boldsymbol{\alpha}_c,
  \boldsymbol{\delta}, \mathrm{block}_b).
}
}

\section{Data Coding}{


\code{data} must be an \eqn{N \times B} matrix, with one column per
forced-choice block. Entries may be character rankings such as
\code{"2>1>3"} or 1-based integer pattern indices. If integer pattern
indices are supplied, \code{block.items} must be provided because item
identities cannot be recovered from indices alone. For \code{"MOLE"} and
\code{"PICK"} character data, supplying \code{block.items} is recommended
and required whenever partial rankings do not identify all items in a
block.
}

\section{Estimation Controls}{


Common \code{control.method} entries are:
\describe{
\item{\code{seed}}{Integer random seed.}
\item{\code{cores}}{Number of cores used by Stan chains; ignored by the
deterministic EM updates.}
\item{\code{vis}}{Logical progress flag.}
\item{\code{delta.lower}, \code{delta.upper}}{Lower and upper bounds for
delta optimization in EM and iStEM. Defaults are -4 and 4.}
}

Stan-specific entries include \code{chains}, \code{iter}, \code{warmup},
\code{thin}, \code{init}, \code{algorithm}, and the usual Stan
\code{control} entries such as \code{adapt_delta}, \code{max_treedepth},
\code{stepsize}, and \code{metric}. Defaults are inherited from the common
ForceChoice Stan helper: 4 chains, 2000 iterations, and half the iterations
used as warmup. The default \code{init = 0} is valid for Stan FCGDINA
because it implies \eqn{p_{ic} = 0.5} under the logit link.

iStEM-specific entries include \code{M}, \code{B}, \code{burnin.maxitr},
\code{maxitr}, \code{eps1}, \code{eps2}, \code{frac1}, \code{frac2},
\code{optim.maxit}, and \code{estimate.se}.  Each iStEM update samples
discrete attribute classes from the current posterior and optimises
\eqn{\delta} parameters given the sampled class memberships.

EM-specific entries are:
\describe{
\item{\code{maxitr}}{Maximum EM iterations; default 500.}
\item{\code{minitr}}{Minimum EM iterations before convergence checks;
default 2.}
\item{\code{tol}}{Absolute log-likelihood change tolerance; default
\code{1e-6}.}
\item{\code{par.tol}}{Maximum absolute delta-parameter change tolerance;
default \code{1e-4}.}
\item{\code{optim.maxit}}{Maximum L-BFGS-B iterations for blockwise delta
updates; default 200.}
\item{\code{estimate.se}}{Logical; whether to estimate standard errors by
nonparametric bootstrap. Default \code{FALSE}.}
\item{\code{bootstrap}}{Number of bootstrap samples when
\code{estimate.se = TRUE}. Default 100; must be at least 2.}
}
}

\examples{
sim <- sim.data.FCGDINA(N.person = 20, N.block = 2, I.block = 2,
                        D = 2, model = "DINA", fc.type = "RANK")

fit <- fit.FCGDINA(
  sim$data,
  Q.matrix = sim$Q.matrix,
  block.items = sim$block.items,
  model = "DINA",
  fc.type = sim$fc.type,
  method = "EM",
  control.method = list(seed = 123, vis = FALSE,
                        maxitr = 2, estimate.se = FALSE)
)

coef(fit, type = "par")
head(fit$alpha$est)
logLik(fit)

\donttest{
# stan code, long time
fit.stan <- fit.FCGDINA(
  sim$data,
  Q.matrix = sim$Q.matrix,
  block.items = sim$block.items,
  model = "GDINA",
  method = "stan",
  control.method = list(seed = 123, cores = 2,
                        chains = 1, iter = 200, warmup = 100)
)
}

}
\references{
de la Torre, J. (2011). The generalized DINA model framework.
\emph{Psychometrika}, 76(2), 179--199.
\doi{10.1007/s11336-011-9207-7}

Templin, J., & Henson, R. A. (2006). Measurement of psychological disorders
using cognitive diagnosis models. \emph{Psychological Methods}, 11(3),
287--305. \doi{10.1037/1082-989X.11.3.287}
}
\seealso{
\code{\link{sim.data.FCGDINA}}, \code{\link{model.FCGDINA}},
\code{\link{get.fit.index.FCGDINA}}, \code{\link{fit.FCDCM}},
\code{\link{fit.FCMIRT}}, \code{\link{fit.FCGGUM}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{tirt_par_free_mask}
\alias{tirt_par_free_mask}
\title{Free-parameter mask for TIRT models}
\usage{
tirt_par_free_mask(Q.matrix, lambda.free, psi.free)
}
\arguments{
\item{Q.matrix}{I x D Q-matrix.}

\item{lambda.free}{Logical vector of length I.}

\item{psi.free}{Logical vector of length I.}
}
\value{
An I x (D+1) logical matrix.
}
\description{
Free-parameter mask for TIRT models
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.data.FCGDINA.R
\name{sim.data.FCGDINA}
\alias{sim.data.FCGDINA}
\title{Simulate Forced-Choice GDINA Data}
\usage{
sim.data.FCGDINA(
  N.person = 1000,
  N.block = 10,
  I.block = 2,
  D = 3,
  model = "GDINA",
  fc.type = "RANK",
  control = NULL
)
}
\arguments{
\item{N.person}{Number of persons.}

\item{N.block}{Number of forced-choice blocks.}

\item{I.block}{Number of items per block when \code{control$block.items} is
not supplied. Must be at least 2.}

\item{D}{Number of latent attributes. If \code{control$Q.matrix} is supplied,
\code{D} is reset to \code{ncol(control$Q.matrix)}.}

\item{model}{CDM item model: \code{"DINA"}, \code{"DINO"}, \code{"ACDM"}, or
\code{"GDINA"}.}

\item{fc.type}{Forced-choice response type: \code{"RANK"}, \code{"MOLE"}, or
\code{"PICK"}. A scalar value is recycled to all blocks; a vector of
length \eqn{B} may be supplied for mixed block formats.}

\item{control}{Optional named list controlling the data-generating process.
Supported entries are described below.}
}
\value{
An object of class \code{"data.FCGDINA"}, a list containing:
\describe{
\item{\code{data}}{\eqn{N \times B} character matrix of forced-choice
responses, such as \code{"1>3>2"} for ranking blocks.}
\item{\code{response}}{\eqn{N \times B} integer matrix of 1-based pattern
indices matching \code{patterns}.}
\item{\code{alpha}}{\eqn{N \times D} binary matrix of true latent
attribute profiles.}
\item{\code{gs}}{\eqn{I \times 2} matrix of item probability bounds
\code{P0} and \code{P1}.}
\item{\code{delta.list}}{List of true CDM delta vectors, one per
statement.}
\item{\code{design.matrix.list}}{List of CDM design matrices used to map
delta parameters to item endorsement probabilities.}
\item{\code{Q.matrix}}{\eqn{I \times D} Q-matrix.}
\item{\code{block.items}}{List of statement indices per forced-choice
block.}
\item{\code{model}}{CDM item model used for simulation.}
\item{\code{patterns}}{List of observed response-pattern matrices per
block.}
\item{\code{patterns.total}}{List of full-ranking pattern matrices per
block.}
\item{\code{prob.item}}{\eqn{N \times I} matrix of item-level endorsement
probabilities for sampled persons.}
\item{\code{prob.states}}{\eqn{I \times 2^D} matrix of item endorsement
probabilities for every attribute pattern.}
\item{\code{prob}}{\eqn{N \times \sum_b P_b} matrix of forced-choice
response-pattern probabilities.}
\item{\code{N.person}, \code{N.block}, \code{I.block}, \code{D},
\code{I.states}, \code{fc.type}}{Final data-generating dimensions
and response type.}
\item{\code{call}, \code{arguments}}{Matched call and effective
simulation arguments.}
}
}
\description{
Generates forced-choice response data from a cognitive diagnostic item
model. The item-level model may be DINA, DINO, ACDM, or GDINA. The resulting
statement endorsement probabilities are transformed into forced-choice
response probabilities for \code{"RANK"}, \code{"MOLE"}, or \code{"PICK"}
blocks using the same sequential Luce--Plackett mechanism used by
\code{\link{sim.data.FCMIRT}} and \code{\link{sim.data.FCGGUM}}.
}
\section{Data-Generating Process}{


The simulation proceeds in four steps:
\enumerate{
\item Generate or validate the Q-matrix and forced-choice block
structure.
\item Generate latent attribute profiles \eqn{\boldsymbol{\alpha}_n}.
\item Generate item-level CDM endorsement probabilities using guessing
and slipping-style bounds \code{P0} and \code{P1}; then recover the
corresponding CDM delta parameters from the design matrix.
\item Convert item endorsement probabilities into block-level
forced-choice pattern probabilities and sample observed responses.
}

Attribute profiles can be generated from a higher-order latent trait model,
a thresholded multivariate normal model, a uniform distribution over all
attribute patterns, or supplied directly through \code{control$alpha}.
}

\section{Control List}{


\describe{
\item{\code{Q.matrix}}{Optional \eqn{I \times D} binary Q-matrix. If
omitted, a Q-matrix is generated by \code{sim.data.Q.CDM()}.}
\item{\code{block.items}}{Optional list of item indices per block. If
omitted, blocks are formed sequentially using \code{N.block} and
\code{I.block}.}
\item{\code{single}}{Logical passed to \code{sim.data.Q.CDM()} when
\code{Q.matrix} is generated internally. Default \code{TRUE}.}
\item{\code{alpha}}{Optional \eqn{N \times D} binary matrix of true
attribute profiles. If supplied, latent attributes are not
generated.}
\item{\code{distribution}}{Latent attribute distribution when
\code{alpha} is not supplied. Options are \code{"horder"}
(default; alias \code{"higher.order"}), \code{"mvnorm"}, and
\code{"uniform"}.}
\item{\code{delta1}, \code{delta0}, \code{theta}}{Higher-order
discrimination, threshold, and person trait values used when
\code{distribution = "horder"}. Defaults are generated from
log-normal, normal, and standard normal distributions.}
\item{\code{Corr}, \code{threshold}}{Correlation matrix and thresholds
used when \code{distribution = "mvnorm"}.}
\item{\code{gs}}{Optional \eqn{I \times 2} matrix with columns
\code{P0} and \code{P1}; \code{P0} is the lower endorsement
probability and \code{P1} is the upper endorsement probability for
each statement. Defaults are sampled from \code{U(0, .2)} and
\code{U(.8, 1)}.}
\item{\code{mono.constraint}}{Logical; whether to enforce monotonic item
probabilities for GDINA item generation. Default \code{TRUE}.}
}
}

\examples{
set.seed(123)
sim <- sim.data.FCGDINA(N.person = 20, N.block = 3, I.block = 2,
                        D = 2, model = "DINA", fc.type = "RANK")

str(sim$data)
head(sim$response)
head(sim$alpha)
sim$Q.matrix

# Mixed response formats across blocks.
sim.mix <- sim.data.FCGDINA(
  N.person = 12, N.block = 3, I.block = 3, D = 2,
  model = "GDINA", fc.type = c("RANK", "MOLE", "PICK")
)
vapply(sim.mix$patterns, nrow, integer(1))

}
\references{
de la Torre, J. (2011). The generalized DINA model framework.
\emph{Psychometrika}, 76(2), 179--199.
\doi{10.1007/s11336-011-9207-7}
}
\seealso{
\code{\link{fit.FCGDINA}}, \code{\link{model.FCGDINA}},
\code{\link{sim.data.FCMIRT}}, \code{\link{sim.data.FCGGUM}},
\code{\link{sim.data.FCDCM}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3update.R
\name{update}
\alias{update}
\alias{update.MIRT}
\alias{update.MGPCM}
\alias{update.MGGUM}
\alias{update.FCMIRT}
\alias{update.FCDCM}
\alias{update.FCGDINA}
\alias{update.FCGGUM}
\alias{update.TIRT}
\title{S3 Methods: update}
\usage{
\method{update}{MIRT}(object, ...)

\method{update}{MGPCM}(object, ...)

\method{update}{MGGUM}(object, ...)

\method{update}{FCMIRT}(object, ...)

\method{update}{FCDCM}(object, ...)

\method{update}{FCGDINA}(object, ...)

\method{update}{FCGGUM}(object, ...)

\method{update}{TIRT}(object, ...)
}
\arguments{
\item{object}{An object of one of the following classes:
\itemize{
\item \code{"MIRT"} --- Multidimensional IRT model.
\item \code{"MGPCM"} --- Multidimensional Generalized Partial Credit Model.
\item \code{"MGGUM"} --- Multidimensional Generalized Graded Unfolding Model.
\item \code{"FCMIRT"} --- Forced-Choice Multidimensional IRT model.
\item \code{"FCDCM"} --- Forced-Choice Diagnostic Classification Model.
\item \code{"FCGDINA"} --- Forced-Choice GDINA model.
\item \code{"FCGGUM"} --- Forced-Choice Generalized Graded Unfolding Model.
\item \code{"TIRT"} --- Thurstonian IRT for Forced-Choice.
}}

\item{...}{Additional named arguments passed to override or extend the
original call. Valid arguments depend on the class of \code{object} and
correspond to the formal parameters of the matching \code{fit.*()}
function. Common overrides include \code{method}, \code{control.model},
\code{control.method}, and legacy \code{control}.}
}
\value{
An object of the same class as \code{object}, re-fitted using the
original arguments updated with any provided in \code{...}. All
unchanged parameters are preserved from the original call.
}
\description{
The \code{update} function provides a unified and convenient interface to
re-fit \pkg{ForceChoice} models with modified parameter settings while
preserving all other original configurations. It allows users to change
the estimation method, adjust control parameters, increase dimensionality,
or switch model specifications without re-specifying the entire call.
}
\details{
Internally, each method extracts the stored \code{arguments} list from
the input \code{object}, merges it with user-provided \code{...} using
\code{\link[utils]{modifyList}}, explicitly merges control lists from the
previous fit and the update call, filters to only the formal parameters of
the target \code{fit.*()} function, then re-invokes the fitting function
via \code{do.call}.

This ensures that:
\itemize{
\item Only explicitly overridden parameters are changed.
\item Default values from the original call remain intact.
\item Complex nested structures (e.g., \code{control.model},
\code{control.method}, and \code{control} lists) can be partially
updated.
\item Internal arguments (e.g., \code{cores}, \code{vis}, \code{seed})
are passed through to the method controls.
}
}
\section{Methods (by class)}{
\itemize{
\item \code{update(MIRT)}: Update method for \code{MIRT} objects.
Re-fits the MIRT model with modified arguments. Supports changing
\code{model} (e.g., m2pl -> m3pl), \code{method} (iStEM -> Stan),
\code{D}, \code{Q.matrix}, \code{control.model}, and
\code{control.method}.

\item \code{update(MGPCM)}: Update method for \code{MGPCM} objects.
Re-fits the MGPCM model with modified arguments.

\item \code{update(MGGUM)}: Update method for \code{MGGUM} objects.
Re-fits the MGGUM model with modified arguments.

\item \code{update(FCMIRT)}: Update method for \code{FCMIRT} objects.
Re-fits the FCMIRT model with modified arguments. Supports changing
\code{model}, \code{method}, \code{fc.type}, \code{block.items},
and control settings.

\item \code{update(FCDCM)}: Update method for \code{FCDCM} objects.
Re-fits the FCDCM model with modified arguments. Supports changing
\code{dcm.type}, \code{method}, \code{block.items}, and control
settings.

\item \code{update(FCGDINA)}: Update method for \code{FCGDINA} objects.
Re-fits the FCGDINA model with modified arguments.

\item \code{update(FCGGUM)}: Update method for \code{FCGGUM} objects.
Re-fits the FCGGUM model with modified arguments.

\item \code{update(TIRT)}: Update method for \code{TIRT} objects.
Re-fits the TIRT model with modified arguments. Supports changing
\code{fc.type}, \code{method}, \code{pairs.value}, and control
settings.

}}
\examples{
sim <- sim.data.MIRT(N = 20, I = 6, D = 2, model = "m2pl")
fit <- fit.MIRT(sim$response, model = "m2pl", D = 2, method = "iStEM",
                control.method = list(
                  vis = FALSE, seed = 123,
                  M = 2, B = 2, burnin.maxitr = 2,
                  maxitr = 3, eps1 = 10, eps2 = 10,
                  estimate.se = FALSE))

\donttest{
# stan code, long time
# Switch to Stan estimation with more chains
fit_stan <- update(fit, method = "stan",
                   control.method = list(chains = 1, iter = 200))
}

# Change model to 3PL
fit_3pl <- update(fit, model = "m3pl")

# Adjust prior hyperparameters
fit_prior <- update(fit, control.model = list(a.mu = 0.5, a.sigma = 0.5))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools-mcmc.R
\name{extract_corr_stan}
\alias{extract_corr_stan}
\title{Extract correlation posterior summary from Stan MCMC output}
\usage{
extract_corr_stan(MCMC.obj, stan.sum, D)
}
\arguments{
\item{MCMC.obj}{Output of \code{rstan::extract()}.}

\item{stan.sum}{Output of \code{rstan::summary()$summary}.}

\item{D}{Number of dimensions.}
}
\value{
A list with components \code{est}, \code{se}, and \code{Rhat}.
}
\description{
Extract correlation posterior summary from Stan MCMC output
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/istem-helpers.R
\name{resolve_fc_type}
\alias{resolve_fc_type}
\title{Auto-detect forced-choice response type from data}
\usage{
resolve_fc_type(fc.type, data, block.items = NULL)
}
\arguments{
\item{fc.type}{User-supplied fc.type or NULL.}

\item{data}{The forced-choice data matrix.}

\item{block.items}{Optional block-item list.}
}
\value{
Character vector of fc.type values (one per block).
}
\description{
When \code{fc.type} is \code{NULL}, inspects the ranking strings in the
data columns to infer whether they represent RANK, MOLE, or PICK.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools-mcmc.R
\name{setup_mcmc_env}
\alias{setup_mcmc_env}
\title{Setup single-threaded MCMC environment}
\usage{
setup_mcmc_env(cores)
}
\arguments{
\item{cores}{Desired number of parallel chains (one chain per core).}
}
\value{
A named list with old environment values and the validated
\code{cores} value.  Pass this to \code{restore_mcmc_env()} to restore
the previous thread-control state.
}
\description{
Ensures each MCMC chain runs strictly on a single OS thread by setting
environment variables that control Stan-internal threading and BLAS-level
parallelism.  The validated core count is returned inside the result list.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools-data.R
\name{get.response.from.data}
\alias{get.response.from.data}
\title{Convert FC Data Strings to an Integer Response (Pattern-Index) Matrix}
\usage{
get.response.from.data(data, block.items, fc.type = "RANK")
}
\arguments{
\item{data}{An \eqn{N \times B} character matrix or data frame.  Cells
encode within-block item orderings as global indices separated by
\code{">"} (e.g. \code{"3>1>2"} for RANK, \code{"3>2"} for MOLE,
\code{"3"} for PICK).}

\item{block.items}{A list of length \eqn{B} giving the global item
indices in each forced-choice block.}

\item{fc.type}{Character vector of length \eqn{B} (or a scalar
recycled to all blocks): \code{"RANK"}, \code{"MOLE"}, or
\code{"PICK"}.}
}
\value{
An \eqn{N \times B} integer matrix.  Each entry is a 1-based
pattern index (row number in the block's pattern matrix).  Column
names are \code{"block.1"}, \dots, \code{"block.B"}.
}
\description{
Parses human-readable forced-choice data strings into integer pattern
indices.  This is the generic converter used by \strong{FCGGUM},
\strong{FCMIRT}, and \strong{FCGDINA}.  Each data cell is matched
against the full set of permutation/observation patterns for that
block, and the resulting row index is stored in the response matrix.
}
\section{Pattern Matching}{

The conversion first generates all possible observation patterns for
each block via the internal permutation helper:
\itemize{
\item \strong{RANK} (\eqn{K} items): all \eqn{K!} full rankings,
ordered lexicographically by within-block position.
\item \strong{MOLE}: \eqn{K(K-1)} most-least observation patterns.
\item \strong{PICK}: \eqn{K} best-only patterns.
}
Each data cell string is parsed into an integer vector of global item
indices, which is then matched (as an ordered sequence) against the
pattern matrix.  The 1-based row index of the match becomes the
response value.  Pattern ordering is deterministic and reproducible.

A non-missing cell that cannot be matched to a valid pattern is returned as
\code{NA_integer_}.  This is useful for data-screening workflows: after
conversion, check \code{anyNA(response)} before passing the response matrix
to a fitting function.
}

\examples{
# 2 persons, 2 RANK blocks.
items <- list(1:2, 3:5)
dat <- data.frame(
  block.1 = c("1>2", "2>1"),
  block.2 = c("3>4>5", "5>4>3"),
  stringsAsFactors = FALSE
)
resp <- get.response.from.data(dat, items, fc.type = "RANK")
resp
get.data.from.response(resp, items, fc.type = "RANK")

# Mixed block formats: most-least plus pick.
dat.mixed <- data.frame(
  mole = c("1>3", "3>2"),
  pick = c("4", "6"),
  stringsAsFactors = FALSE
)
mixed.items <- list(1:3, 4:6)
get.response.from.data(
  dat.mixed, mixed.items, fc.type = c("MOLE", "PICK")
)
}
\seealso{
\code{\link{get.data.from.response}} for the reverse conversion.

\code{\link{generate_fc_permutation_patterns}} for pattern generation
details.

\code{\link{get.response.from.data.TIRT}} for the TIRT-specific
converter.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools-data.R
\name{get.data.from.response}
\alias{get.data.from.response}
\title{Convert an Integer Response (Pattern-Index) Matrix to FC Data Strings}
\usage{
get.data.from.response(response, block.items, fc.type = "RANK")
}
\arguments{
\item{response}{An \eqn{N \times B} integer matrix of pattern
indices.  Each entry is an integer between 1 and the number of
observed patterns for that block.}

\item{block.items}{A list of length \eqn{B} giving the global item
indices in each forced-choice block.}

\item{fc.type}{Character vector of length \eqn{B} (or a scalar
recycled to all blocks): \code{"RANK"}, \code{"MOLE"}, or
\code{"PICK"}.}
}
\value{
A data frame with \eqn{N} rows and \eqn{B} columns.  Column
names are \code{"block.1"}, \dots, \code{"block.B"}.  Cell values
are character strings: full rankings (\code{"2>5>8"}) for
\code{"RANK"}, \code{"best>worst"} pairs for \code{"MOLE"}, and
single integers for \code{"PICK"}.
}
\description{
Converts an integer matrix of pattern indices back to human-readable
forced-choice data strings.  This is the generic converter used by
\strong{FCGGUM}, \strong{FCMIRT}, and \strong{FCGDINA} where each
column of the response matrix is an index into the block's observed
permutation-pattern list.
}
\section{Pattern Generation and Numbering}{

For a block with \eqn{K} items, the set of possible response patterns
depends on the forced-choice type:
\describe{
\item{RANK}{All \eqn{K!} full permutations of the \eqn{K} items
are generated.  The patterns are listed in lexicographic order
of the within-block item positions.  For example, a 3-item
block with items \code{c(2,5,8)} produces \eqn{3! = 6} patterns:
\code{(2,5,8)}, \code{(2,8,5)}, \code{(5,2,8)}, \code{(5,8,2)},
\code{(8,2,5)}, \code{(8,5,2)}.  Pattern index 1 means the
first pattern was observed.}
\item{MOLE}{For a \eqn{K}-item block, \eqn{K(K-1)} most-least
observation patterns are possible: \eqn{K} choices for the most
preferred item times \eqn{K-1} choices for the least preferred
item.  Pattern \eqn{(a,b)} means item \eqn{a} was most
preferred and item \eqn{b} was least preferred.}
\item{PICK}{For a \eqn{K}-item block, exactly \eqn{K} patterns
exist: one for each possible most-preferred item.}
}
In all cases the pattern index (an integer from 1 to the number of
observed patterns) is the row number in the matrix returned by
the internal permutation helper for that block. The same deterministic
pattern construction is exposed by
\code{\link{generate_fc_permutation_patterns}}.

This format is a compact block-level encoding.  One response column equals
one forced-choice block, even when the block contains more than two items.
This is the representation expected by the generic forced-choice model
families FCGGUM, FCMIRT, and FCGDINA.  It should not be confused with the
TIRT pairwise format, where one block usually expands to several pair
columns.
}

\examples{
# 2 persons, 2 RANK blocks.
items <- list(1:2, 3:5)
resp <- cbind(c(1, 2), c(1, 6))
get.data.from.response(resp, items, fc.type = "RANK")

# Mixed block formats: first block is most-least, second block is pick.
mixed.items <- list(1:3, 4:6)
mixed.resp <- cbind(c(1, 6), c(1, 3))
get.data.from.response(
  mixed.resp, mixed.items, fc.type = c("MOLE", "PICK")
)
}
\seealso{
\code{\link{get.response.from.data}} for the reverse conversion.

\code{\link{generate_fc_permutation_patterns}} for pattern generation.

\code{\link{get.data.from.response.TIRT}} for the TIRT-specific
converter.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-utils.R
\name{make_item_prob_cdm}
\alias{make_item_prob_cdm}
\title{Build CDM item response probabilities for attribute mastery patterns}
\usage{
make_item_prob_cdm(patterns, model, p0, p1, mono.constraint = TRUE)
}
\arguments{
\item{patterns}{A binary matrix of attribute mastery patterns.}

\item{model}{One of \code{"DINA"}, \code{"DINO"}, \code{"ACDM"}, \code{"GDINA"}.}

\item{p0}{Lower-bound probability (guessing).}

\item{p1}{Upper-bound probability (1 - slipping).}

\item{mono.constraint}{Logical; if \code{TRUE} (default), enforce the
monotonicity constraint that mastering more attributes never decreases
the success probability.  Only applies to \code{model = "GDINA"}.}
}
\value{
A named numeric vector of probabilities, one per pattern row.
}
\description{
Computes the probability of endorsing an item for each attribute mastery
pattern under the specified CDM condensation rule (DINA, DINO, ACDM, or
GDINA).  \code{p0} is the lower-bound (guessing) probability and
\code{p1} is the upper-bound (1 - slipping) probability.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.data.FCGDINA.R
\name{model.FCGDINA}
\alias{model.FCGDINA}
\title{Compute FCGDINA forced-choice pattern probabilities}
\usage{
model.FCGDINA(
  alpha,
  delta.list,
  design.matrix.list,
  Q.matrix,
  patterns.total,
  patterns,
  prob.item = NULL
)
}
\arguments{
\item{alpha}{\eqn{N \times D} binary matrix of attribute profiles.}

\item{delta.list}{List of per-item delta parameter vectors. The length must
equal \code{nrow(Q.matrix)}.}

\item{design.matrix.list}{List of per-item CDM design matrices generated by
\code{\link{get_design_matrix_cdm}}.}

\item{Q.matrix}{\eqn{I \times D} binary Q-matrix.}

\item{patterns.total}{List of full-ranking pattern matrices.}

\item{patterns}{List of observed pattern matrices.}

\item{prob.item}{Optional precomputed item endorsement-probability matrix.
Supplying it avoids recomputation in simulation workflows.}
}
\value{
An \eqn{N \times \sum_b P_b} matrix of forced-choice response
probabilities, where \eqn{P_b} is the number of observable patterns in
block \eqn{b}.
}
\description{
Computes block-level forced-choice response probabilities from known
attribute profiles and item-level CDM delta parameters. This is the
deterministic probability engine used by \code{\link{sim.data.FCGDINA}} for
response generation and is useful for checking a fitted or simulated
FCGDINA parameterization.
}
\details{
Item endorsement probabilities are first computed by
\code{\link{compute_fcgdina_item_prob}}. For person \eqn{n} and statement
\eqn{i}, the probability is
\deqn{
  p_{ni} = \mathbf{x}_i(\boldsymbol{\alpha}_n)' \boldsymbol{\delta}_i,
}
where \eqn{\mathbf{x}_i(\boldsymbol{\alpha}_n)} is the relevant row of the
item CDM design matrix. The item probabilities are then converted to
block-level forced-choice pattern probabilities by
\code{\link{forced_choice_from_agree}}.
}
\examples{
sim <- sim.data.FCGDINA(N.person = 10, N.block = 2, I.block = 2,
                        D = 2, model = "DINA")
prob <- model.FCGDINA(
  alpha = sim$alpha,
  delta.list = sim$delta.list,
  design.matrix.list = sim$design.matrix.list,
  Q.matrix = sim$Q.matrix,
  patterns.total = sim$patterns.total,
  patterns = sim$patterns
)
stopifnot(all(abs(rowSums(prob) - length(sim$block.items)) < 1e-8))

}
\seealso{
\code{\link{sim.data.FCGDINA}},
\code{\link{compute_fcgdina_item_prob}},
\code{\link{forced_choice_from_agree}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-utils.R
\name{apply_rotation_promax}
\alias{apply_rotation_promax}
\title{Apply factor rotation to discrimination matrix and latent traits}
\usage{
apply_rotation_promax(a, theta, Corr, rotate, D, promax_m = 4)
}
\arguments{
\item{a}{\eqn{I \times D} discrimination (loading) matrix.}

\item{theta}{\eqn{N \times D} latent trait matrix.}

\item{Corr}{\eqn{D \times D} correlation matrix.}

\item{rotate}{Rotation method name (character).}

\item{D}{Number of dimensions.}

\item{promax_m}{Power parameter for \code{"promax"} (default 4).}
}
\value{
A named list with components \code{a}, \code{theta}, \code{Corr}.
}
\description{
Supports \code{"promax"} (via \code{stats::promax}) and GPArotation
methods (\code{"varimax"}, \code{"oblimin"}, etc.).  Transforms the
discrimination matrix \code{a}, the latent trait matrix \code{theta},
and the correlation matrix \code{Corr} accordingly.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3fitted.R
\name{fitted}
\alias{fitted}
\alias{fitted.MIRT}
\alias{fitted.MGPCM}
\alias{fitted.MGGUM}
\alias{fitted.FCMIRT}
\alias{fitted.FCDCM}
\alias{fitted.FCGDINA}
\alias{fitted.FCGGUM}
\alias{fitted.TIRT}
\title{S3 Methods: fitted}
\usage{
\method{fitted}{MIRT}(object, ...)

\method{fitted}{MGPCM}(object, ...)

\method{fitted}{MGGUM}(object, ...)

\method{fitted}{FCMIRT}(object, ...)

\method{fitted}{FCDCM}(object, ...)

\method{fitted}{FCGDINA}(object, ...)

\method{fitted}{FCGGUM}(object, ...)

\method{fitted}{TIRT}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{"MIRT"},
\code{"MGPCM"}, \code{"MGGUM"}, \code{"FCMIRT"}, \code{"FCDCM"},
\code{"FCGDINA"}, \code{"FCGGUM"}, or \code{"TIRT"}.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
A numeric matrix of model-implied probabilities. Dimensions:
\itemize{
\item Binary models (MIRT, TIRT): \eqn{N \times I}
\item Polytomous models (MGPCM, MGGUM): \eqn{N \times \sum K_i}
(stacked category probabilities)
\item Forced-choice models (FCMIRT, FCGDINA, FCGGUM): \eqn{N \times \sum K_b}
(block-pattern probabilities)
\item FCDCM: \eqn{N \times B} (marginal block-choice probabilities
after integrating over \eqn{2^D} attribute patterns)
}
}
\description{
Extract model-implied response probabilities from fitted
\pkg{ForceChoice} model objects. Returns the conditional probability of
each observed response given each person's estimated latent trait
\eqn{\hat{\boldsymbol{\theta}}_j}, evaluated via the model's response
function.
}
\details{
For binary-response models, each entry is the endorsement probability
\eqn{P(Y_{ij}=1 \mid \hat{\boldsymbol{\theta}}_j)}. For polytomous
models, probabilities are returned in stacked format (one column per
category). For forced-choice models, each entry is the probability of
the observed ranking pattern in the given block.
}
\section{Methods (by class)}{
\itemize{
\item \code{fitted(MIRT)}: Fitted probabilities for \code{MIRT} objects.
Returns \eqn{N \times I} matrix of endorsement probabilities.

\item \code{fitted(MGPCM)}: Fitted probabilities for \code{MGPCM} objects.
Returns stacked category probability matrix.

\item \code{fitted(MGGUM)}: Fitted probabilities for \code{MGGUM} objects.

\item \code{fitted(FCMIRT)}: Fitted probabilities for \code{FCMIRT} objects.
Returns block-pattern probability matrix.

\item \code{fitted(FCDCM)}: Fitted probabilities for \code{FCDCM} objects.
Returns \eqn{N \times B} matrix of marginal block-choice probabilities
after integrating over \eqn{2^D} attribute patterns.

\item \code{fitted(FCGDINA)}: Fitted probabilities for \code{FCGDINA} objects.
Returns block-pattern probability matrix after integrating over posterior
attribute classes.

\item \code{fitted(FCGGUM)}: Fitted probabilities for \code{FCGGUM} objects.

\item \code{fitted(TIRT)}: Fitted probabilities for \code{TIRT} objects.
Returns \eqn{N \times I_{pairs}} matrix of pairwise comparison
probabilities.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.data.FCMIRT.R
\name{model.FCMIRT}
\alias{model.FCMIRT}
\title{Compute FCMIRT Forced-Choice Pattern Probabilities}
\usage{
model.FCMIRT(theta, par, patterns.total, patterns)
}
\arguments{
\item{theta}{N x D matrix of latent trait values.}

\item{par}{I x (D + 3) MIRT item parameter matrix, where b is the
item difficulty and the item-level linear predictor is a' theta - b.}

\item{patterns.total}{List of full-ranking pattern matrices.}

\item{patterns}{List of observed pattern matrices.}
}
\value{
N x sum(pattern counts) probability matrix. For each block,
item endorsement probabilities are first computed by \code{model.MIRT},
transformed to logits, and then combined with a sequential
Luce/Plackett rule. MOLE and PICK probabilities are obtained by summing
compatible full-ranking probabilities and normalizing over the observed
block patterns.
}
\description{
Compute FCMIRT Forced-Choice Pattern Probabilities
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logLik.FCDCM.R, R/logLik.FCGDINA.R,
%   R/logLik.FCGGUM.R, R/logLik.FCMIRT.R, R/logLik.MGGUM.R, R/logLik.MGPCM.R,
%   R/logLik.MIRT.R, R/logLik.R, R/logLik.TIRT.R
\name{logLik.FCDCM}
\alias{logLik.FCDCM}
\alias{logLik.FCGDINA}
\alias{logLik.FCGGUM}
\alias{logLik.FCMIRT}
\alias{logLik.MGGUM}
\alias{logLik.MGPCM}
\alias{logLik.MIRT}
\alias{logLik-class}
\alias{logLik.TIRT}
\title{Extract Log-Likelihood from Fitted Models}
\usage{
\method{logLik}{FCDCM}(object, theta.low = NULL, theta.up = NULL, L = NULL, ...)

\method{logLik}{FCGDINA}(object, ...)

\method{logLik}{FCGGUM}(object, theta.low = NULL, theta.up = NULL, L = NULL, ...)

\method{logLik}{FCMIRT}(object, theta.low = NULL, theta.up = NULL, L = NULL, ...)

\method{logLik}{MGGUM}(object, theta.low = NULL, theta.up = NULL, L = NULL, ...)

\method{logLik}{MGPCM}(object, theta.low = NULL, theta.up = NULL, L = NULL, ...)

\method{logLik}{MIRT}(object, theta.low = NULL, theta.up = NULL, L = NULL, ...)

\method{logLik}{TIRT}(object, theta.low = NULL, theta.up = NULL, L = NULL, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{"MIRT"},
\code{"MGPCM"}, \code{"MGGUM"}, \code{"FCMIRT"}, \code{"FCDCM"},
\code{"FCGDINA"}, \code{"FCGGUM"}, or \code{"TIRT"}.}

\item{theta.low}{Numeric; lower bound for the quadrature grid on the
latent scale. Used by continuous-trait models; if \code{NULL}, the value
stored in the fitted object's controls is used, falling back to
\code{-6}.}

\item{theta.up}{Numeric; upper bound for the quadrature grid on the latent
scale. Used by continuous-trait models; if \code{NULL}, the value stored
in the fitted object's controls is used, falling back to \code{6}.}

\item{L}{Integer; number of quadrature points per dimension. If
\code{NULL}, the package chooses a dimension-dependent default.}

\item{...}{Additional arguments passed to the method. Currently ignored by
the implemented methods.}
}
\value{
An object of class \code{"logLik"} with attributes:
\itemize{
\item \code{nobs}: number of observations (\eqn{N})
\item \code{df}: number of free parameters
\item \code{theta.norm}: continuous-trait quadrature grid points, when
applicable
\item \code{prob}: model-implied response probabilities at grid points,
when applicable
\item \code{pi}: quadrature weights or FCGDINA class probabilities
\item \code{L}: quadrature grid size used, when applicable
\item \code{prob.class}: FCGDINA class-conditional forced-choice
probabilities, when applicable
}

An object of class \code{"logLik"}: a length-one numeric vector
containing the marginal log-likelihood, with at least attributes
\code{nobs} (sample size) and \code{df} (number of free parameters).
Continuous-trait methods also attach quadrature diagnostics such as
\code{theta.norm}, \code{prob}, \code{pi}, and \code{L}; FCGDINA attaches
latent-class probabilities and class-conditional response probabilities.
}
\description{
Extracts the marginal log-likelihood from a fitted ForceChoice model
object. For continuous-trait IRT models, the log-likelihood is computed
by numerically integrating the joint likelihood of the observed responses
over the latent trait distribution using Cartesian-product normal
quadrature. For FCGDINA, the marginal likelihood is obtained by summing
over the \eqn{2^D} discrete attribute profiles.

For a sample of \eqn{N} independent persons with responses
\eqn{\mathbf{Y}_j}, the marginal log-likelihood is:
\deqn{
  \ell = \sum_{j=1}^{N} \log \int
  P(\mathbf{Y}_j \mid \boldsymbol{\theta})\,
  \phi_D(\boldsymbol{\theta}; \mathbf{0}, \boldsymbol{\Sigma})\,
  d\boldsymbol{\theta},
}
where \eqn{\phi_D(\cdot)} is the \eqn{D}-variate normal density with
correlation matrix \eqn{\boldsymbol{\Sigma}}. The integral is approximated
on a Cartesian-product quadrature grid over \eqn{[-6, 6]^D}.

The continuous-trait quadrature weights are normalized normal densities
computed on the grid points, with the fitted correlation matrix used when
available. FCGDINA instead uses the estimated class-proportion vector
\eqn{\boldsymbol{\pi}} over attribute profiles.

S3 methods for extracting marginal log-likelihoods from fitted
\pkg{ForceChoice} model objects.
}
\details{
For MIRT, MGPCM, MGGUM, FCMIRT, FCGGUM, TIRT, and FCDCM objects, the
likelihood is evaluated by marginalizing over a normal quadrature grid
(and, for FCDCM, over attribute profiles conditional on the higher-order
trait). For FCGDINA objects, the likelihood is evaluated exactly over the
\eqn{2^D} latent attribute profiles.
}
\section{Functions}{
\itemize{
\item \code{logLik(FCDCM)}: FCDCM model: higher-order trait quadrature with exact
marginalization over discrete attribute profiles.

\item \code{logLik(FCGDINA)}: FCGDINA model: marginal forced-choice probabilities over
all \eqn{2^D} attribute profiles.

\item \code{logLik(FCGGUM)}: FCGGUM model: forced-choice unfolding block
probabilities with multivariate normal quadrature.

\item \code{logLik(FCMIRT)}: FCMIRT model: forced-choice block pattern
probabilities with multivariate normal quadrature.

\item \code{logLik(MGGUM)}: MGGUM model: polytomous unfolding responses with
per-item block-structured quadrature evaluation.

\item \code{logLik(MGPCM)}: MGPCM model: polytomous responses integrated over the
fitted multivariate normal latent distribution.

\item \code{logLik(MIRT)}: MIRT model: multivariate normal quadrature over
\eqn{D}-dimensional latent space with Gaussian correlation structure.

\item \code{logLik(TIRT)}: TIRT model: pairwise binary probit probabilities
with multivariate normal quadrature over traits.

}}
\examples{
sim <- sim.data.MIRT(N = 20, I = 6, D = 2, model = "m2pl")
fit <- fit.MIRT(
  sim$response, model = "m2pl", D = 2, method = "iStEM",
  control.method = list(
    vis = FALSE, seed = 123,
    M = 2, B = 2, burnin.maxitr = 2,
    maxitr = 3, eps1 = 10, eps2 = 10,
    estimate.se = FALSE)
)
ll <- logLik(fit, L = 9)
as.numeric(ll)
attr(ll, "df")

}
\seealso{
\code{\link{fit.MIRT}}, \code{\link{get.fit.index}},
\code{\link{good.of.fit}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{extract_rhat_vector}
\alias{extract_rhat_vector}
\title{Extract R-hat values for a vector parameter from a Stan summary}
\usage{
extract_rhat_vector(stan.sum, param_name, n)
}
\arguments{
\item{stan.sum}{Stan summary matrix.}

\item{param_name}{Parameter name prefix (e.g. \code{"delta1"}).}

\item{n}{Expected length of the output vector.}
}
\value{
A numeric vector of R-hat values (NA where not found).
}
\description{
Extract R-hat values for a vector parameter from a Stan summary
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-utils.R
\name{generate_theta_mvn}
\alias{generate_theta_mvn}
\title{Generate latent trait vectors from a multivariate normal distribution}
\usage{
generate_theta_mvn(N, D, Corr)
}
\arguments{
\item{N}{Number of persons.}

\item{D}{Number of latent dimensions.}

\item{Corr}{A \eqn{D \times D} correlation matrix.}
}
\value{
An \eqn{N \times D} numeric matrix of latent trait values.
}
\description{
When \code{D > 1}, uses \code{MASS::mvrnorm} with the supplied correlation
matrix.  When \code{D == 1}, uses standard normal draws.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.data.FCGGUM.R
\name{model.FCGGUM}
\alias{model.FCGGUM}
\title{Compute FCGGUM Forced-Choice Pattern Probabilities}
\usage{
model.FCGGUM(theta, par, patterns.total, patterns)
}
\arguments{
\item{theta}{Latent trait matrix.}

\item{par}{Binary GGUM item parameter matrix with columns
\code{a1..aD, delta1..deltaD, tau0, tau1}; \code{tau0} is fixed at 0
and \code{tau1 < 0}.}

\item{patterns.total}{Full ranking patterns.}

\item{patterns}{Observed ranking patterns.}
}
\value{
Block ranking probability matrix. The code first computes binary
GGUM endorsement probabilities from the distance-based MGGUM formula,
transforms them to logits, and then applies the sequential
Luce/Plackett rule. MOLE and PICK probabilities are sums over
compatible full rankings normalized over observed patterns.
}
\description{
Compute FCGGUM Forced-Choice Pattern Probabilities
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/istem-helpers.R
\name{model_type_to_int}
\alias{model_type_to_int}
\title{Map resolved model name to integer code}
\usage{
model_type_to_int(model)
}
\arguments{
\item{model}{Character string (canonical model name).}
}
\value{
Integer model type code.
}
\description{
Converts the canonical model string returned by \code{resolve_model_type()}
to the integer code expected by the Stan models (1 = 1PL, 2 = 2PL,
3 = 3PL, 4 = 4PL).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.data.MGPCM.R
\name{model.MGPCM}
\alias{model.MGPCM}
\title{Compute MGPCM Category Response Probabilities}
\usage{
model.MGPCM(theta, par)
}
\arguments{
\item{theta}{\eqn{N \times D} matrix of latent trait values.}

\item{par}{\eqn{I \times (D + \max K_i)} parameter matrix with
columns \code{a1..aD, d0, d1, ...}. Missing category columns for
shorter items should be \code{NA}; \code{d0} is typically fixed at 0.}
}
\value{
An \eqn{N \times \sum_i K_i} matrix of category probabilities
(stacked by item), where
\eqn{P(Y_{ij}=k) \propto \exp\{k\,\mathbf{a}_i'
  \boldsymbol{\theta}_j + d_{ik}\}}.
}
\description{
Compute MGPCM Category Response Probabilities
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3vcov.R
\name{deviance}
\alias{deviance}
\alias{deviance.MIRT}
\alias{deviance.MGPCM}
\alias{deviance.MGGUM}
\alias{deviance.FCMIRT}
\alias{deviance.FCDCM}
\alias{deviance.FCGDINA}
\alias{deviance.FCGGUM}
\alias{deviance.TIRT}
\title{S3 Methods: deviance}
\usage{
\method{deviance}{MIRT}(object, ...)

\method{deviance}{MGPCM}(object, ...)

\method{deviance}{MGGUM}(object, ...)

\method{deviance}{FCMIRT}(object, ...)

\method{deviance}{FCDCM}(object, ...)

\method{deviance}{FCGDINA}(object, ...)

\method{deviance}{FCGGUM}(object, ...)

\method{deviance}{TIRT}(object, ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
A numeric scalar: \eqn{-2 \log L}.
}
\description{
Returns the model deviance, defined as \eqn{-2 \ell} where \eqn{\ell}
is the marginal log-likelihood evaluated at the parameter estimates.
}
\section{Methods (by class)}{
\itemize{
\item \code{deviance(MIRT)}: Model deviance (\eqn{-2 \ell}) for \code{MIRT} objects.

\item \code{deviance(MGPCM)}: Model deviance for \code{MGPCM} objects.

\item \code{deviance(MGGUM)}: Model deviance for \code{MGGUM} objects.

\item \code{deviance(FCMIRT)}: Model deviance for \code{FCMIRT} objects.

\item \code{deviance(FCDCM)}: Model deviance for \code{FCDCM} objects.

\item \code{deviance(FCGDINA)}: Model deviance for \code{FCGDINA} objects.

\item \code{deviance(FCGGUM)}: Model deviance for \code{FCGGUM} objects.

\item \code{deviance(TIRT)}: Model deviance for \code{TIRT} objects.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools-mcmc.R
\name{build_stan_control}
\alias{build_stan_control}
\title{Build Stan sampling-control list from user-provided \code{control.method}}
\usage{
build_stan_control(algorithm, control.method)
}
\arguments{
\item{algorithm}{Character scalar: \code{"NUTS"}, \code{"HMC"}, or
\code{"Fixed_param"}.}

\item{control.method}{A named list of Stan method controls.}
}
\value{
A named list of Stan sampling-control parameters (possibly empty).
}
\description{
This helper extracts Stan sampler tuning knobs (e.g. \code{adapt_delta},
\code{max_treedepth}) from method-specific controls so they can be passed
safely to \code{rstan::sampling()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get.fit.index.FCDCM.R,
%   R/get.fit.index.FCGDINA.R, R/get.fit.index.FCGGUM.R,
%   R/get.fit.index.FCMIRT.R, R/get.fit.index.MGGUM.R, R/get.fit.index.MGPCM.R,
%   R/get.fit.index.MIRT.R, R/get.fit.index.R, R/get.fit.index.TIRT.R
\name{get.fit.index.FCDCM}
\alias{get.fit.index.FCDCM}
\alias{get.fit.index.FCGDINA}
\alias{get.fit.index.FCGGUM}
\alias{get.fit.index.FCMIRT}
\alias{get.fit.index.MGGUM}
\alias{get.fit.index.MGPCM}
\alias{get.fit.index.MIRT}
\alias{get.fit.index}
\alias{get.fit.index.TIRT}
\title{Goodness-of-Fit Indices for Fitted Models}
\usage{
\method{get.fit.index}{FCDCM}(object, ...)

\method{get.fit.index}{FCGDINA}(object, ...)

\method{get.fit.index}{FCGGUM}(object, ...)

\method{get.fit.index}{FCMIRT}(object, ...)

\method{get.fit.index}{MGGUM}(object, ...)

\method{get.fit.index}{MGPCM}(object, ...)

\method{get.fit.index}{MIRT}(object, ...)

\method{get.fit.index}{TIRT}(object, ...)
}
\arguments{
\item{object}{A fitted model object (class \code{"MIRT"}, \code{"MGPCM"},
\code{"MGGUM"}, \code{"FCMIRT"}, \code{"FCDCM"}, \code{"FCGGUM"}, or
\code{"TIRT"}).}

\item{...}{Additional arguments passed to \code{\link{good.of.fit}}.
Common options include \code{alpha} (tail probability for RMSEA CI;
default 0.05 gives a 90\% CI), \code{bivariate.groups} (exclude pairs
within the same group from bivariate moments), and \code{n.boot}
(bootstrap replicates).}
}
\value{
An object of class \code{"good.of.fit"}. Use
\code{\link{summary.good.of.fit}} and \code{\link{print.good.of.fit}}
for formatted output. Key components:
\itemize{
\item \code{LogLik}, \code{deviance}, \code{AIC}, \code{AICc},
\code{BIC}, \code{CAIC}, \code{SABIC}, \code{HQIC}
\item \code{M2.value}, \code{df}, \code{p.value}
\item \code{RMSEA2}, \code{RMSEA.lower}, \code{RMSEA.upper},
\code{RMSEA.p.close}
\item \code{SRMSR}
\item \code{CFI}, \code{TLI}, \code{IFI}
\item \code{McFadden.R2}, \code{CoxSnell.R2}, \code{Nagelkerke.R2}
\item \code{q3.mean}, \code{q3.abs.max}, \code{q3.adjusted.abs.max}
\item \code{posterior.entropy}, \code{posterior.entropy.normalized}
}
}
\description{
Generic function to compute a comprehensive suite of model-fit diagnostics
for fitted ForceChoice model objects. The function dispatches on the class
of the fitted model and returns an object of class \code{"good.of.fit"}
containing likelihood-based, limited-information, and residual-based
diagnostics.
}
\section{Methods (by class)}{
\itemize{
\item \code{get.fit.index(FCDCM)}: FCDCM model: binary block responses with
quadrature over higher-order \eqn{\theta} and exact marginalization
over \eqn{2^D} attribute patterns.

\item \code{get.fit.index(FCGDINA)}: FCGDINA model: forced-choice CDM with nominal
binary expansion; within-block pairs excluded.

\item \code{get.fit.index(FCGGUM)}: FCGGUM model: forced-choice unfolding with nominal
binary expansion; within-block pairs excluded.

\item \code{get.fit.index(FCMIRT)}: FCMIRT model: forced-choice ranking data expanded
to nominal binary indicators; within-block pairs excluded from bivariate
moments.

\item \code{get.fit.index(MGGUM)}: MGGUM model: polytomous unfolding responses with
M2* category-collapsed construction.

\item \code{get.fit.index(MGPCM)}: MGPCM model: polytomous responses with M2*
category-collapsed construction.

\item \code{get.fit.index(MIRT)}: MIRT model: binary responses with Gauss--Hermite
quadrature over \eqn{D}-dimensional latent space.

\item \code{get.fit.index(TIRT)}: TIRT model: pairwise binary responses with
within-block pairs excluded via \code{bivariate.groups}.

}}
\section{Limited-Information M2 Framework}{


Following Maydeu-Olivares and Joe (2005, 2006), the test statistic uses
first- and second-order marginal moments rather than the full contingency
table, making it suitable for sparse high-dimensional responses.

Let \eqn{\boldsymbol{\pi}(\boldsymbol{\psi})} be the vector of
\eqn{M} model-implied univariate and bivariate moments,
\eqn{\mathbf{p}} the observed moments, and
\eqn{\boldsymbol{\Delta} = \partial\boldsymbol{\pi}/\partial\boldsymbol{\psi}'}
the Jacobian. The M2 statistic is:
\deqn{
  M_2 = N\,\mathbf{e}'\hat{\mathbf{C}}_2\mathbf{e},
  \qquad \mathbf{e} = \mathbf{p} - \boldsymbol{\pi}(\hat{\boldsymbol{\psi}}),
}
where \eqn{\hat{\mathbf{C}}_2 = \boldsymbol{\Delta}_c
(\boldsymbol{\Delta}_c'\hat{\boldsymbol{\Xi}}_2\boldsymbol{\Delta}_c)^{-1}
\boldsymbol{\Delta}_c'} with \eqn{\boldsymbol{\Delta}_c} the orthogonal
complement of \eqn{\boldsymbol{\Delta}}. Under the null,
\eqn{M_2 \sim \chi^2_{M - p}} asymptotically.
}

\section{Fit Index Formulas}{


\strong{Information criteria} (Akaike 1974; Schwarz 1978; Bozdogan 1987;
Sclove 1987; Hannan & Quinn 1979):
\deqn{
  \begin{aligned}
  \text{AIC}   &= -2\ell + 2p, \\
  \text{AICc}  &= \text{AIC} + \frac{2p(p+1)}{N-p-1}, \\
  \text{BIC}   &= -2\ell + p\log N, \\
  \text{CAIC}  &= -2\ell + p(\log N + 1), \\
  \text{SABIC} &= -2\ell + p\log\!\bigl(\frac{N+2}{24}\bigr), \\
  \text{HQIC}  &= -2\ell + 2p\log(\log N).
  \end{aligned}
}

\strong{RMSEA} (Browne & Cudeck 1993):
\deqn{
  \text{RMSEA} = \sqrt{\max\!\left(0,\; \frac{M_2 - df}{N \cdot df}\right)},
  \qquad df = M - p.
}
Confidence intervals via non-central \eqn{\chi^2} inversion; close-fit
test \eqn{H_0: \text{RMSEA} \le 0.05}.

\strong{SRMSR:}
\deqn{
  \text{SRMSR} = \sqrt{\frac{1}{J}\sum_{i<j}(r_{ij}^{\text{obs}} - r_{ij}^{\text{model}})^2}.
}

\strong{CFI / TLI / IFI} (Bentler 1990; Tucker & Lewis 1973; Bollen 1989):
\deqn{
  \begin{aligned}
  \text{CFI} &= 1 - \frac{\max(M_2 - df, 0)}{\max(M_2^{\text{null}} - df^{\text{null}},\; M_2 - df,\; 0)}, \\[4pt]
  \text{TLI} &= \frac{M_2^{\text{null}}/df^{\text{null}} - M_2/df}{M_2^{\text{null}}/df^{\text{null}} - 1}, \\[4pt]
  \text{IFI} &= \frac{M_2^{\text{null}} - M_2}{M_2^{\text{null}} - df}.
  \end{aligned}
}

\strong{Pseudo-\eqn{R^2}:}
\deqn{
  \begin{aligned}
  R^2_{\text{McF}}     &= 1 - \ell/\ell_0, \\
  R^2_{\text{CS}}      &= 1 - \exp\!\bigl(\tfrac{2}{N}(\ell_0 - \ell)\bigr), \\
  R^2_{\text{N}}       &= R^2_{\text{CS}} / \bigl(1 - \exp(2\ell_0/N)\bigr), \\
  R^2_{\text{AN}}      &= G^2/(G^2 + N), \quad G^2 = D_0 - D.
  \end{aligned}
}

\strong{Yen's Q3} (Yen 1984):
\deqn{
  Q_{3,ik} = \text{Corr}(r_{ji}, r_{jk}), \qquad
  r_{ji} = Y_{ji} - E[Y_{ji} \mid \hat{\boldsymbol{\theta}}_j].
}

\strong{Posterior diagnostics} (Celeux & Soromenho 1996):
\deqn{
  E = -\frac{1}{N}\sum_{j=1}^{N}\sum_{c=1}^{Q}\hat{\tau}_{jc}\log\hat{\tau}_{jc},
  \qquad
  E_{\text{norm}} = 1 - E / \log Q.
}
}

\section{Heuristic Guidelines}{

\itemize{
\item RMSEA: \eqn{\le 0.05} close, \eqn{\le 0.08} reasonable
\item SRMSR: \eqn{\le 0.08} acceptable (Hu & Bentler 1999)
\item CFI / TLI: \eqn{\ge 0.95} good, \eqn{\ge 0.90} acceptable
\item Normalized entropy: \eqn{\ge 0.70} acceptable separation
}
}

\examples{
sim <- sim.data.MIRT(N = 20, I = 6, D = 2, model = "m2pl")
fit <- fit.MIRT(
  sim$response, model = "m2pl", D = 2, method = "iStEM",
  control.method = list(
    vis = FALSE, seed = 123,
    M = 2, B = 2, burnin.maxitr = 2,
    maxitr = 3, eps1 = 10, eps2 = 10,
    estimate.se = FALSE)
)
gof <- get.fit.index(fit)
summary(gof)

# Custom RMSEA confidence level (99\%)
gof99 <- get.fit.index(fit, alpha = 0.005)
summary(gof99)
}
\references{
Akaike, H. (1974). \emph{IEEE Trans. Autom. Control}, 19, 716--723.
Bozdogan, H. (1987). \emph{Psychometrika}, 52, 345--370.
Browne, M. W., & Cudeck, R. (1993). In Bollen & Long (Eds.), \emph{Testing
structural equation models} (pp. 136--162). Sage.
Celeux, G., & Soromenho, G. (1996). \emph{J. Classification}, 13, 195--212.
Hu, L. T., & Bentler, P. M. (1999). \emph{Struct. Equ. Modeling}, 6, 1--55.
Maydeu-Olivares, A., & Joe, H. (2005). \emph{JASA}, 100, 1009--1020.
Maydeu-Olivares, A., & Joe, H. (2006). \emph{Psychometrika}, 71, 713--732.
Schwarz, G. (1978). \emph{Ann. Statist.}, 6, 461--464.
Yen, W. M. (1984). \emph{Appl. Psychol. Meas.}, 8, 125--145.
}
\seealso{
\code{\link{good.of.fit}} for the underlying engine,
\code{\link{summary.good.of.fit}} for formatted summary tables.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.data.FCDCM.R
\name{sim.data.FCDCM}
\alias{sim.data.FCDCM}
\title{Simulate Data from the Forced-Choice DCM Model}
\usage{
sim.data.FCDCM(
  N.person = 1000,
  N.block = 10,
  D = 3,
  dcm.type = "DINA",
  control = NULL
)
}
\arguments{
\item{N.person}{Integer; number of persons (default: 1000).}

\item{N.block}{Integer; number of forced-choice block pairs (default: 10).}

\item{D}{Integer; number of attributes (default: 3, must be >= 2).}

\item{dcm.type}{Character; \code{"DINA"} (default) or \code{"DINO"}.}

\item{control}{Optional list with entries \code{Q.matrix}, \code{block.items},
\code{single}, \code{delta1}, \code{delta0},
\code{theta}, \code{alpha}, \code{par}.  If \code{block.items} is
omitted, statements are assembled into cross-Q-vector forced-choice
blocks with balanced first/second positions and diversified attribute
pairs.  \code{par} is a \eqn{B \times 2} matrix with columns
\code{eta0} and \code{etaAB}.}
}
\value{
An object of class \code{"data.FCDCM"} containing \code{data},
\code{response}, \code{theta}, \code{alpha}, \code{zeta}, \code{par},
\code{Q.matrix}, \code{block.items},
\code{prob}, and data-generating arguments.
}
\description{
Generates forced-choice paired-comparison data from a higher-order
cognitive diagnostic model (DINA/DINO). The continuous higher-order
trait governs attribute mastery probabilities via a logistic link;
two-statement blocks produce binary choices based on attribute
mastery patterns. In each block, selecting the first statement has
probability \eqn{\eta_0} when its condensed mastery is lower than the
second statement's, \eqn{0.5} when they are equal, and
\eqn{0.5+\eta_{AB}} when it is higher.
}
\examples{
set.seed(123)
sim <- sim.data.FCDCM(N.person = 20, N.block = 3, D = 2,
                      dcm.type = "DINA")
str(sim$data)
head(sim$response)
head(sim$alpha)

}
\seealso{
\code{\link{fit.FCDCM}}, \code{\link{model.FCDCM}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{extract_rhat_matrix}
\alias{extract_rhat_matrix}
\title{Extract R-hat values for a matrix parameter from a Stan summary}
\usage{
extract_rhat_matrix(stan.sum, param_name, nrow, ncol)
}
\arguments{
\item{stan.sum}{Stan summary matrix (rownames encode parameter indices).}

\item{param_name}{Parameter name prefix (e.g. \code{"theta"}).}

\item{nrow, ncol}{Dimensions of the output matrix.}
}
\value{
A numeric matrix of R-hat values.
}
\description{
Extract R-hat values for a matrix parameter from a Stan summary
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3coef.R
\name{coef}
\alias{coef}
\alias{coef.MIRT}
\alias{coef.MGPCM}
\alias{coef.MGGUM}
\alias{coef.FCMIRT}
\alias{coef.FCDCM}
\alias{coef.FCGDINA}
\alias{coef.FCGGUM}
\alias{coef.TIRT}
\title{S3 Methods: coef}
\usage{
\method{coef}{MIRT}(object, type = c("par", "theta", "Corr", "all"), ...)

\method{coef}{MGPCM}(object, type = c("par", "theta", "Corr", "all"), ...)

\method{coef}{MGGUM}(object, type = c("par", "theta", "Corr", "all"), ...)

\method{coef}{FCMIRT}(object, type = c("par", "theta", "Corr", "all"), ...)

\method{coef}{FCDCM}(object, type = c("par", "theta", "Corr", "all"), ...)

\method{coef}{FCGDINA}(object, type = c("par", "theta", "Corr", "all"), ...)

\method{coef}{FCGGUM}(object, type = c("par", "theta", "Corr", "all"), ...)

\method{coef}{TIRT}(object, type = c("par", "theta", "Corr", "all"), ...)
}
\arguments{
\item{object}{An object of one of the following classes:
\itemize{
\item \code{"MIRT"} --- Multidimensional IRT model.
\item \code{"MGPCM"} --- Multidimensional Generalized Partial Credit Model.
\item \code{"MGGUM"} --- Multidimensional Generalized Graded Unfolding Model.
\item \code{"FCMIRT"} --- Forced-Choice Multidimensional IRT model.
\item \code{"FCDCM"} --- Forced-Choice Diagnostic Classification Model.
\item \code{"FCGDINA"} --- Forced-Choice GDINA model.
\item \code{"FCGGUM"} --- Forced-Choice Generalized Graded Unfolding Model.
\item \code{"TIRT"} --- Thurstonian IRT for Forced-Choice.
}}

\item{type}{Character string specifying which parameters to extract:
\code{"par"} (default) for item/statement parameters, \code{"theta"}
for person trait estimates, \code{"Corr"} for factor correlations, or
\code{"all"} for a list with all three.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
\describe{
\item{\code{type = "par"}}{A matrix of item/statement/block parameter
estimates. Rows index items/statements/blocks and columns index
parameter types. Column names indicate the parameter (e.g.,
\code{"a1"}, \code{"b"}, \code{"d0"}, \code{"tau1"}, \code{"eta0"}).}
\item{\code{type = "theta"}}{An \eqn{N \times D} matrix of person
parameter (latent trait) estimates.}
\item{\code{type = "Corr"}}{A \eqn{D \times D} inter-trait correlation
matrix.}
\item{\code{type = "all"}}{A named list with components \code{par},
\code{theta}, and \code{Corr}.}
}
}
\description{
Extract estimated model coefficients (item, statement, or block
parameters) from fitted \pkg{ForceChoice} model objects. This provides a
standard R interface for retrieving parameter estimates, standard errors,
and convergence diagnostics in a consistent format across all eight model
families.
}
\details{
For \code{FCDCM} objects, \code{type = "par"} returns a list with
components \code{delta} (higher-order parameters, \eqn{D \times 2}) and
\code{par} (block \eqn{\eta} parameters, \eqn{B \times 2}), reflecting
the hierarchical structure of the model.
}
\section{Methods (by class)}{
\itemize{
\item \code{coef(MIRT)}: Extract coefficients from \code{MIRT} objects.
Returns \eqn{I \times (D+3)} matrix with columns a1..aD, b, c, d.

\item \code{coef(MGPCM)}: Extract coefficients from \code{MGPCM} objects.
Returns \eqn{I \times (D+K_{\max})} matrix with columns a1..aD, d0..d(K-1).

\item \code{coef(MGGUM)}: Extract coefficients from \code{MGGUM} objects.
Returns matrix with columns a1..aD, delta1..deltaD, tau0..tau(K-1).

\item \code{coef(FCMIRT)}: Extract coefficients from \code{FCMIRT} objects.
Returns statement parameter matrix with columns a1..aD, b, c, d.

\item \code{coef(FCDCM)}: Extract coefficients from \code{FCDCM} objects.
Returns a list with \code{delta} (\eqn{D \times 2} higher-order
parameters) and \code{par} (\eqn{B \times 2} block \eqn{\eta}
parameters). \code{type = "theta"} returns the higher-order trait,
and \code{type = "Corr"} returns a scalar 1.

\item \code{coef(FCGDINA)}: Extract coefficients from \code{FCGDINA} objects.
\code{type = "par"} returns CDM delta parameters,
\code{type = "theta"} returns posterior attribute mastery probabilities,
\code{type = "Corr"} returns \code{NA} (no continuous latent trait).

\item \code{coef(FCGGUM)}: Extract coefficients from \code{FCGGUM} objects.
Returns matrix with columns a1..aD, delta1..deltaD, tau0..tau(K-1).

\item \code{coef(TIRT)}: Extract coefficients from \code{TIRT} objects.
Returns \eqn{I \times (D+1)} matrix with columns lambda1..lambdaD, psi2.

}}
\examples{
sim <- sim.data.MIRT(N = 20, I = 6, D = 2, model = "m2pl")
fit <- fit.MIRT(
  sim$response, model = "m2pl", D = 2, method = "iStEM",
  control.method = list(
    vis = FALSE, seed = 123,
    M = 2, B = 2, burnin.maxitr = 2,
    maxitr = 3, eps1 = 10, eps2 = 10,
    estimate.se = FALSE)
)

coef(fit)                  # item parameter estimates
coef(fit, type = "theta")  # person trait estimates
coef(fit, type = "Corr")   # factor correlation matrix
coef(fit, type = "all")    # everything in a list

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.MIRT.R
\name{fit.MIRT}
\alias{fit.MIRT}
\title{Fit the Multidimensional Item Response Theory (MIRT) Model}
\usage{
fit.MIRT(
  data,
  model = "2PL",
  D = NULL,
  Q.matrix = NULL,
  method = c("iStEM", "stan"),
  control.model = NULL,
  control.method = NULL
)
}
\arguments{
\item{data}{An \eqn{N \times I} matrix of binary responses coded as
\code{0} (incorrect) and \code{1} (correct). Rows index persons, columns
index items. Missing values are not allowed.}

\item{model}{Character string specifying the model type. Accepts both
internal codes (\code{"m1pl"}, \code{"m2pl"}, \code{"m3pl"},
\code{"m4pl"}) and user-friendly aliases (\code{"Rasch"}, \code{"1PL"},
\code{"2PL"}, \code{"3PL"}, \code{"4PL"}). Case-insensitive.
Default is \code{"2PL"}.}

\item{D}{Integer; number of latent dimensions (\eqn{D \ge 1}). If
\code{NULL} (default), \code{D} is inferred from \code{Q.matrix}; if both
are \code{NULL}, an error is raised. When \code{D = 1}, the model reduces
to unidimensional IRT.}

\item{Q.matrix}{An optional \eqn{I \times D} binary matrix. Entry
\eqn{q_{id} = 1} indicates that the \eqn{d}-th dimension loads on item
\eqn{i}. For 2PL--4PL models, \eqn{q_{id} = 1} frees \eqn{a_{id}} and
\eqn{q_{id} = 0} fixes \eqn{a_{id} = 0}. For the current M1PL backend,
slopes are fixed to 1 and the Q-matrix does not zero out slope entries.
If \code{NULL} (default), a triangular identification pattern is used:
all items load on all dimensions except the last \eqn{D} items, which
follow a lower-triangular structure for rotational invariance resolution.}

\item{method}{Estimation method: \code{"iStEM"} (fast stochastic EM for
large data; default) or \code{"stan"} (full Bayesian HMC).}

\item{control.model}{A named list of model-level hyperparameters.
Supported entries:
\describe{
\item{\code{a.mu}, \code{a.sigma}}{Prior location and scale for
\eqn{\log a_{id}} (log-normal). Defaults: 0.25, 0.25.}
\item{\code{b.mu}, \code{b.sigma}}{Prior mean and SD for \eqn{b_i}
(normal). Defaults: 0, 1.}
\item{\code{c.mu}, \code{c.sigma}}{Uniform support bounds
\eqn{[c_{\min}, c_{\max}]} for \eqn{c_i}. Defaults: 0, 0.35.}
\item{\code{d.mu}, \code{d.sigma}}{Uniform support bounds
\eqn{[d_{\min}, d_{\max}]} for \eqn{d_i}. Defaults: 0.65, 1.}
\item{\code{theta.mu}}{Prior mean vector for
\eqn{\boldsymbol{\theta}_j}. Default: \code{rep(0, D)}.}
\item{\code{L}}{Theta grid size per dimension for marginal
log-likelihood computation and iStEM block Gibbs sampling.
Default adapts to \eqn{D}
(e.g., 61 for D = 1, 31 for D = 2, 15 for D = 3).}
\item{\code{theta.lower}, \code{theta.upper}}{Bounds for the
theta grid used by marginal log-likelihood computation and
iStEM block Gibbs sampling. Defaults: -6, 6.}
\item{\code{use.prior}}{Logical (iStEM only). If \code{FALSE}, the
item-prior penalty term is omitted from the item update,
recovering an approximate maximum-likelihood item step.
Default is \code{TRUE}.}
}}

\item{control.method}{A named list of method-specific tuning parameters.
Common entries (used by both Stan and iStEM):
\describe{
\item{\code{cores}}{Number of CPU cores for parallel chains
(Stan) or ignored (iStEM). Default: the number of
\code{chains}.}
\item{\code{vis}}{Logical; if \code{TRUE} (default), prints progress
information to the console.}
\item{\code{seed}}{Random seed for reproducibility.
Default: a random integer.}
}
Stan-specific entries:
\describe{
\item{\code{chains}}{Number of MCMC chains (default: 2).}
\item{\code{iter}}{Total iterations per chain (default: 5000).}
\item{\code{warmup}}{Warmup/burn-in iterations per chain
(default: \code{iter / 2}).}
\item{\code{thin}}{Thinning interval (default: 1).}
\item{\code{init}}{Initial values: \code{"random"} (default)
for uniform(-2, 2) initialization, or a list of initial
values per chain.}
\item{\code{algorithm}}{MCMC algorithm: \code{"HMC"} (default),
\code{"HMC"}, or \code{"Fixed_param"}.}
\item{\code{adapt_delta}}{Target average acceptance probability
(NUTS; default: 0.95).}
\item{\code{max_treedepth}}{Maximum tree depth (NUTS; default: 10).}
\item{\code{stepsize}}{Initial step size for the leapfrog integrator.
If not set, Stan determines an appropriate value automatically
during warmup. Only applicable when \code{algorithm = "HMC"}
or \code{"NUTS"}.}
\item{\code{int_time}}{Total integration time for each HMC leapfrog
trajectory. The number of steps is \code{int_time / stepsize}.
Only applicable when \code{algorithm = "HMC"}.}
\item{\code{metric}}{The mass matrix for HMC sampling. Can be a
unit vector (\code{"unit_e"}), a diagonal matrix
(\code{"diag_e"}), or a dense matrix (\code{"dense_e"}).
Stan defaults to \code{"diag_e"}.}
\item{\code{adapt_engaged}}{Logical; if \code{TRUE} (default), the
warmup adaptation is enabled. Set to \code{FALSE} to disable
step-size and mass-matrix adaptation during warmup.}
\item{\code{adapt_init_buffer}}{Number of initial warmup iterations
used for pure exploration before adaptation begins
(default: 25).}
\item{\code{adapt_term_buffer}}{Number of final warmup iterations
where adaptation is frozen so the sampler can converge to the
stationary distribution (default: 50).}
\item{\code{adapt_window}}{Number of warmup iterations between
adaptation updates. Larger values reduce the frequency of
adaptation (default: 25).}
}
iStEM-specific entries:
\describe{
\item{\code{M}}{Number of burn-in batches retained for convergence
diagnosis (default: 10). Must be at least 2. Larger values
improve the Geweke diagnostic stability but increase computation
during burn-in.}
\item{\code{B}}{Batch size: number of stochastic EM iterations per batch
(default: 20). Each iteration samples all persons'
\eqn{\boldsymbol{\theta}_j} blocks and updates all items.}
\item{\code{burnin.maxitr}}{Maximum number of burn-in batches
(default: 100). If convergence criteria are not met before this
limit, the algorithm proceeds with a warning. Increase this
value if convergence warnings appear consistently.}
\item{\code{maxitr}}{Maximum number of total batches including
post-burn-in iterations (default: 2000). The algorithm stops
when either the MC error criterion is met or this limit is
reached.}
\item{\code{eps1}}{Geweke z-score convergence threshold for the
burn-in phase (default: 1.5). The burn-in phase ends when
\eqn{\sum z^2 / K < \epsilon_1} or the MC error criterion
is satisfied. Lower values enforce stricter convergence
but prolong burn-in.}
\item{\code{eps2}}{Monte Carlo error tolerance for the final chain
(default: 0.4). The algorithm continues sampling until
\eqn{\max(d_k \cdot N) < \epsilon_2}, where \eqn{d_k} is
the batch-means variance for each parameter. Lower values
yield more precise estimates.}
\item{\code{frac1}, \code{frac2}}{Fractions used in the Geweke
convergence diagnostic (defaults: 0.1, 0.5). \code{frac1}
defines the proportion of the chain used for the early
segment; \code{frac2} defines the proportion for the late
segment. These follow standard practice from the
\pkg{coda} package.}
\item{\code{corr.optim.maxit}}{Maximum L-BFGS-B iterations for the
constrained unit-diagonal correlation update (default: 50).}
\item{\code{optim.maxit}}{Maximum L-BFGS-B iterations per item
during the item-parameter update step (default: 50).
Increase if item-level optimization warnings appear.}
\item{\code{fix.corr}}{Logical; if \code{TRUE}, the inter-trait
correlation matrix is fixed to the identity during estimation
(default: \code{FALSE}). Setting to \code{TRUE} enforces
orthogonal latent dimensions.}
\item{\code{estimate.se}}{Logical; if \code{TRUE} (default), standard
errors are computed as the batch-means-based standard deviation
of the final Monte Carlo chain. Set to \code{FALSE} to skip
SE computation (slightly faster).}
\item{\code{a.lower}, \code{a.upper}}{Bounds on each
\eqn{a_{id}} (defaults: 1e-4, 6). For 2PL--4PL models,
discrimination parameters are constrained to this interval
during L-BFGS-B optimization.}
\item{\code{b.lower}, \code{b.upper}}{Bounds on \eqn{b_i}
(defaults: -6, 6). Item difficulty/intercept parameters
are constrained to this interval during optimization.}
\item{\code{c.lower}, \code{c.upper}}{Bounds on \eqn{c_i} for
3PL/4PL models. Defaults are inherited from the prior support
bounds (\code{control.model$c.mu} = 0 and
\code{control.model$c.sigma} = 0.35, respectively).
Override these to impose tighter or wider bounds on the
lower-asymptote parameters during optimization.}
\item{\code{d.lower}, \code{d.upper}}{Bounds on \eqn{d_i} for
4PL models. Defaults are inherited from the prior support
bounds (\code{control.model$d.mu} = 0.65 and
\code{control.model$d.sigma} = 1, respectively).
Override these to constrain the upper-asymptote parameters
during optimization.}
}}
}
\value{
An object of class \code{"MIRT"} containing the following
components:
\describe{
\item{\code{npar}}{Integer; number of free parameters
(= freely estimated item parameters + free correlation elements).}
\item{\code{method}}{Character; \code{"stan"} or \code{"iStEM"}.}
\item{\code{theta}}{List with matrices \code{est}, \code{se},
\code{Rhat} (\eqn{N \times D}); person parameter estimates.}
\item{\code{par}}{List with matrices \code{est}, \code{se},
\code{Rhat}, \code{free} (\eqn{I \times (D+3)}); item parameter
arrays. Columns are \code{a1..aD, b, c, d}.}
\item{\code{Corr}}{List with matrices \code{est}, \code{se},
\code{Rhat} (\eqn{D \times D}); inter-trait correlation matrix.}
\item{\code{Q.matrix}}{The \eqn{I \times D} Q-matrix used.}
\item{\code{stan.obj}}{The \code{stanfit} object (Stan only;
\code{NULL} for iStEM).}
\item{\code{MCMC.obj}}{The list returned by \code{rstan::extract()}
(Stan only; \code{NULL} for iStEM).}
\item{\code{logLik}}{The marginal log-likelihood computed via
Gauss--Hermite-type quadrature (class \code{"logLik"}).}
\item{\code{call}}{The matched call.}
\item{\code{arguments}}{List of arguments used in fitting.}
\item{\code{iStEM}}{List of iStEM diagnostic quantities (iStEM only),
including burn-in size, convergence flags, and theta grid length.}
}
}
\description{
Fits a multidimensional extension of the 1PL, 2PL, 3PL, or 4PL item
response model to binary response data. Two estimation backends are
provided: full Bayesian inference via Hamiltonian Monte Carlo (Stan) and
a fast stochastic-EM algorithm (iStEM) that scales to large data sets.
}
\section{Model Specification}{


Let \eqn{Y_{ij} \in \{0, 1\}} denote the binary response of person
\eqn{j = 1, \dots, N} to item \eqn{i = 1, \dots, I}, and let
\eqn{\boldsymbol{\theta}_j = (\theta_{j1}, \dots, \theta_{jD})'}
denote the \eqn{D}-dimensional latent trait vector. The item response
function (IRF) for the four model variants is:

\strong{M1PL (Rasch / one-parameter logistic):}
\deqn{
  P(Y_{ij} = 1 \mid \boldsymbol{\theta}_j) =
  \frac{1}{1 + \exp\bigl[-\bigl(\sum_{d=1}^{D} a_{id}\,\theta_{jd} - b_i\bigr)\bigr]},
  \qquad a_{id} = 1
}
The M1PL implementation fixes the slope to 1 and is restricted to
\eqn{D = 1}; fixed unit slopes do not identify separate dimensions.

\strong{M2PL (two-parameter logistic):}
\deqn{
  P(Y_{ij} = 1 \mid \boldsymbol{\theta}_j) =
  \frac{1}{1 + \exp\bigl[-\bigl(\sum_{d=1}^{D} a_{id}\,\theta_{jd} - b_i\bigr)\bigr]},
  \qquad a_{id} > 0 \text{ if } q_{id} = 1,\; a_{id} = 0 \text{ otherwise}
}
The discrimination (slope) parameters \eqn{a_{id}} are freely estimated
subject to the Q-matrix pattern and a log-normal prior.

\strong{M3PL (three-parameter logistic):}
\deqn{
  P(Y_{ij} = 1 \mid \boldsymbol{\theta}_j) =
  c_i + (1 - c_i) \times
  \frac{1}{1 + \exp\bigl[-\bigl(\sum_{d=1}^{D} a_{id}\,\theta_{jd} - b_i\bigr)\bigr]}
}
where \eqn{c_i \in [0, 1)} is the lower-asymptote (pseudo-guessing) parameter.

\strong{M4PL (four-parameter logistic):}
\deqn{
  P(Y_{ij} = 1 \mid \boldsymbol{\theta}_j) =
  c_i + (d_i - c_i) \times
  \frac{1}{1 + \exp\bigl[-\bigl(\sum_{d=1}^{D} a_{id}\,\theta_{jd} - b_i\bigr)\bigr]}
}
where \eqn{c_i \in [0, 1)} is the lower asymptote and
\eqn{d_i \in (0, 1]} is the upper asymptote (1 - slippage).

In the unidimensional case (\eqn{D = 1}), these reduce to the standard
1PL--4PL models. For \eqn{D > 1}, the Q-matrix governs which dimensions
load on each item, enabling both exploratory (default triangular
identification) and confirmatory (user-specified Q-matrix) structures.

\strong{Prior distributions (Bayesian / MAP):}
\describe{
\item{\eqn{a_{id}} (free)}{Log-normal: \eqn{\log a_{id} \sim N(\mu_a, \sigma_a^2)}.
Defaults: \eqn{\mu_a = 0.25}, \eqn{\sigma_a = 0.25}.}
\item{\eqn{b_i}}{Normal: \eqn{b_i \sim N(\mu_b, \sigma_b^2)}.
Defaults: \eqn{\mu_b = 0}, \eqn{\sigma_b = 1}.}
\item{\eqn{c_i}}{Uniform: \eqn{c_i \sim U(c_{\min}, c_{\max})}.
Defaults: \eqn{c_{\min} = 0}, \eqn{c_{\max} = 0.35}.}
\item{\eqn{d_i}}{Uniform: \eqn{d_i \sim U(d_{\min}, d_{\max})}.
Defaults: \eqn{d_{\min} = 0.65}, \eqn{d_{\max} = 1}.}
\item{\eqn{\boldsymbol{\theta}_j}}{Multivariate normal:
\eqn{\boldsymbol{\theta}_j \sim N_D(\boldsymbol{\mu}_\theta, \boldsymbol{\Sigma})},
with \eqn{\boldsymbol{\mu}_\theta = \mathbf{0}} and
\eqn{\boldsymbol{\Sigma}} a correlation matrix.}
}
}

\section{Estimation Methods}{


\describe{
\item{\strong{Stan} (\code{method = "stan"}):}{
Full Bayesian inference via Hamiltonian Monte Carlo (NUTS/HMC).
The joint posterior of all parameters is explored using multiple
Markov chains. Output includes posterior means, standard deviations,
and \eqn{\hat{R}} convergence diagnostics for all parameters.
Marginal log-likelihood is approximated via Gauss--Hermite-type
quadrature over the latent space.}
\item{\strong{iStEM} (\code{method = "iStEM"}):}{
An improved Stochastic EM (iStEM) algorithm that
alternates between (a) sampling \eqn{\boldsymbol{\theta}_j} from its
finite-grid full conditional as one \eqn{D}-dimensional block using
the person's complete response likelihood and (b) maximizing the
complete-data posterior for item parameters via L-BFGS-B.
The inter-trait correlation matrix \eqn{\boldsymbol{\Sigma}} is estimated
by constrained unit-diagonal normal-likelihood optimization.
Convergence is monitored using Geweke (1992) convergence diagnostics
and batch-means Monte Carlo error estimates. Standard errors are
obtained from the final Monte Carlo chain.}
}
}

\examples{
# Simulate data from a 2-dimensional 2PL model
sim <- sim.data.MIRT(N = 20, I = 6, D = 2, model = "m2pl")

# Fit via iStEM (fast, large-data friendly)
fit_istem <- fit.MIRT(sim$response, model = "m2pl", D = 2,
                      method = "iStEM",
                      control.method = list(
                        vis = FALSE, seed = 123,
                        M = 2, B = 2, burnin.maxitr = 2,
                        maxitr = 3, eps1 = 10, eps2 = 10,
                        estimate.se = FALSE))

\donttest{
# stan code, long time
# Fit via Stan (full Bayesian inference, computationally heavier)
fit_stan <- fit.MIRT(sim$response, model = "m2pl", D = 2,
                     method = "stan",
                     control.method = list(
                       chains = 1, iter = 200, warmup = 100,
                       cores = 1, seed = 123))
}

# Extract results
print(fit_istem$par$est)       # item parameter estimates
print(fit_istem$theta$est)     # person trait estimates
print(fit_istem$Corr$est)      # factor correlation matrix

# Compute goodness-of-fit indices
gof <- get.fit.index(fit_istem)
summary(gof)

}
\references{
Reckase, M. D. (2009). \emph{Multidimensional Item Response Theory}.
Springer. \doi{10.1007/978-0-387-89976-3}

Birnbaum, A. (1968). Some latent trait models and their use in inferring
an examinee's ability. In F. M. Lord & M. R. Novick, \emph{Statistical
theories of mental test scores} (pp. 397--479). Addison-Wesley.

Barton, M. A., & Lord, F. M. (1981). An upper asymptote for the
three-parameter logistic item-response model. \emph{ETS Research Report
Series}, 1981(1), i--21.

Geweke, J. (1992). Evaluating the accuracy of sampling-based approaches
to the calculation of posterior moments. In J. M. Bernardo et al.
(Eds.), \emph{Bayesian Statistics 4} (pp. 169--193). Oxford University
Press.
}
\seealso{
\code{\link{good.of.fit}} for goodness-of-fit evaluation,
\code{\link{get.fit.index.MIRT}} for MIRT-specific fit indices,
\code{\link{sim.data.MIRT}} for simulating data from this model,
\code{\link{rotate}} for post-hoc rotation of MIRT solutions,
\code{\link{logLik.MIRT}} for marginal log-likelihood extraction.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-utils.R
\name{sample_fc_responses}
\alias{sample_fc_responses}
\title{Sample forced-choice responses from block-level pattern probabilities}
\usage{
sample_fc_responses(prob, patterns, N.person, N.block)
}
\arguments{
\item{prob}{\eqn{N \times \sum_B P_b} matrix of block-level pattern
probabilities (stacked by block).}

\item{patterns}{List of observed-pattern matrices (one per block).}

\item{N.person}{Number of persons.}

\item{N.block}{Number of blocks.}
}
\value{
A named list with \code{response} (\eqn{N \times B} integer
matrix of pattern indices) and \code{data} (\eqn{N \times B} character
matrix of ranking strings like \code{"1>3>2"}).
}
\description{
Uses inverse-CDF multinomial sampling (identical to the C++ implementation
in \code{forced_choice_from_agree}) to draw a discrete pattern index for
each person-block combination.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3vcov.R
\name{vcov}
\alias{vcov}
\alias{vcov.MIRT}
\alias{vcov.MGPCM}
\alias{vcov.MGGUM}
\alias{vcov.FCMIRT}
\alias{vcov.FCDCM}
\alias{vcov.FCGDINA}
\alias{vcov.FCGGUM}
\alias{vcov.TIRT}
\title{S3 Methods: vcov}
\usage{
\method{vcov}{MIRT}(object, ...)

\method{vcov}{MGPCM}(object, ...)

\method{vcov}{MGGUM}(object, ...)

\method{vcov}{FCMIRT}(object, ...)

\method{vcov}{FCDCM}(object, ...)

\method{vcov}{FCGDINA}(object, ...)

\method{vcov}{FCGGUM}(object, ...)

\method{vcov}{TIRT}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{"MIRT"},
\code{"MGPCM"}, \code{"MGGUM"}, \code{"FCMIRT"}, \code{"FCDCM"},
\code{"FCGDINA"}, \code{"FCGGUM"}, or \code{"TIRT"}.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
A square numeric matrix. Rows and columns correspond to the
free model parameters in the package's canonical parameter map.
}
\description{
Returns the variance--covariance matrix of the estimated model
parameters from a fitted \pkg{ForceChoice} model object. The package stores
marginal standard errors rather than a joint covariance estimate, so the
returned matrix is \eqn{\mathrm{diag}(\mathrm{SE}^2)} for every backend.
}
\details{
Standard errors reflect the active estimation backend. Off-diagonal
covariances are not retained in fitted objects and are therefore reported
as zero. Parameters whose standard errors were not estimated have
\code{NA} diagonal entries.
}
\section{Methods (by class)}{
\itemize{
\item \code{vcov(MIRT)}: Variance--covariance matrix of parameter estimates
for \code{MIRT} objects.

\item \code{vcov(MGPCM)}: Variance--covariance matrix for \code{MGPCM} objects.

\item \code{vcov(MGGUM)}: Variance--covariance matrix for \code{MGGUM} objects.

\item \code{vcov(FCMIRT)}: Variance--covariance matrix for \code{FCMIRT} objects.

\item \code{vcov(FCDCM)}: Variance--covariance matrix for \code{FCDCM} objects.

\item \code{vcov(FCGDINA)}: Variance--covariance matrix for \code{FCGDINA} objects.

\item \code{vcov(FCGGUM)}: Variance--covariance matrix for \code{FCGGUM} objects.

\item \code{vcov(TIRT)}: Variance--covariance matrix for \code{TIRT} objects.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-utils.R
\name{check_integer_scalar}
\alias{check_integer_scalar}
\title{Validate a scalar integer parameter}
\usage{
check_integer_scalar(x, name, min.val = 1L)
}
\arguments{
\item{x}{Value to check.}

\item{name}{Parameter name (used in the error message).}

\item{min.val}{Minimum allowed value (default 1L).}
}
\value{
\code{as.integer(x)} on success; raises an error otherwise.
}
\description{
Checks that \code{x} is a single finite value coercible to integer and
at least \code{min.val}.  Used by forced-choice \code{sim.data.*}
functions to validate \code{N.person}, \code{N.block}, \code{I.block},
and \code{D}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-utils.R
\name{pattern_key}
\alias{pattern_key}
\title{Convert binary matrix rows to string keys}
\usage{
pattern_key(x)
}
\arguments{
\item{x}{A numeric or integer matrix.}
}
\value{
A character vector of length \code{nrow(x)}.
}
\description{
Collapses each row of a binary matrix to a single string by pasting
the entries without separator (e.g. \code{c(0,1,0)} becomes \code{"010"}).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools-data.R
\name{get.data.from.response.TIRT}
\alias{get.data.from.response.TIRT}
\title{Convert TIRT Pairwise-Response Matrix to Forced-Choice Data Strings}
\usage{
get.data.from.response.TIRT(
  response,
  block.items,
  fc.type = NULL,
  pairs.value = NULL
)
}
\arguments{
\item{response}{An \eqn{N \times P} integer matrix of pairwise binary
responses.  Entries must be 1 (first item in the pair preferred) or 0
(second item in the pair preferred).}

\item{block.items}{A list of length \eqn{B} giving the consecutive item
labels in each forced-choice block.}

\item{fc.type}{Character vector of length \eqn{B} (or a scalar recycled
to all blocks): \code{"RANK"}, \code{"MOLE"}, or \code{"PICK"}.}

\item{pairs.value}{A list of length \eqn{N}, where each element is itself
a list of length \eqn{B} giving the pair-definition matrix for each
block.  Required when any block uses \code{"MOLE"} or \code{"PICK"}.
For all-\code{"RANK"} data this argument may be omitted because the pair
definitions are fixed by \code{block.items}.}
}
\value{
A data frame with \eqn{N} rows and \eqn{B} columns.  Column names
are \code{"block.1"}, \code{"block.2"}, etc.  Cell values are character
strings encoding the within-block ordering: full rankings for
\code{"RANK"}, \code{"best>worst"} for \code{"MOLE"}, and single
integers for \code{"PICK"}.
}
\description{
Converts a Thurstonian IRT pairwise-response matrix back to the
human-readable forced-choice strings used in \code{data}.  This is the
inverse of \code{\link{get.response.from.data.TIRT}}.
}
\details{
TIRT differs from the generic FC converters because the model is expressed
through pairwise comparisons.  The input \code{response} is therefore not a
block-level pattern-index matrix.  It is an \eqn{N \times P} binary matrix,
where each column is one observed item pair and each entry is coded:
\itemize{
\item 1: the first item in that pair was preferred;
\item 0: the second item in that pair was preferred.
}

The number of response columns contributed by a block with \eqn{K} items is
determined by \code{fc.type}:
\describe{
\item{RANK}{All \eqn{K(K-1)/2} unordered pairs are observed.  Pair columns
are generated in the order \eqn{(1,2), (1,3), \dots, (K-1,K)} using the
item labels in \code{block.items[[b]]}.}
\item{MOLE}{Only the pairs needed to identify the most- and least-preferred
items are retained.  The block contributes \eqn{2K - 3} response columns
per person.}
\item{PICK}{Only the pairs involving the picked item are retained.  The
block contributes \eqn{K - 1} response columns per person.}
}

For \code{"MOLE"} and \code{"PICK"}, the identities of the retained pairs
can vary by person.  The \code{pairs.value} object records those
person-specific pair definitions and is required to reconstruct data
strings without ambiguity.  It is normally copied directly from the
\code{pairs.value} component returned by
\code{\link{get.response.from.data.TIRT}}.

TIRT conversion assumes consecutive item labels across blocks, for example
\code{list(1:2, 3:5)}.  This is the layout returned by
\code{\link{sim.data.TIRT}}.
}
\examples{
# 2 RANK blocks: block 1 has items 1,2; block 2 has items 3,4,5.
items <- list(1:2, 3:5)
resp <- cbind(
  c(1, 0),            # block 1 pair: 1 vs 2
  c(1, 0),            # block 2 pair: 3 vs 4
  c(1, 0),            # block 2 pair: 3 vs 5
  c(1, 0)             # block 2 pair: 4 vs 5
)
get.data.from.response.TIRT(resp, items, fc.type = "RANK")
}
\seealso{
\code{\link{get.response.from.data.TIRT}} for the reverse conversion.

\code{\link{get.data.from.response}} for the generic (non-TIRT) converter
used by FCGGUM and FCMIRT.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools-data.R
\name{forced_choice_data_conversion}
\alias{forced_choice_data_conversion}
\title{Forced-Choice Data Conversion Formats}
\description{
Documents the three data objects used by the forced-choice conversion
helpers: human-readable \code{data}, block definitions
\code{block.items}, and model-ready \code{response} matrices.  This
overview is intended as the starting point before using
\code{\link{get.response.from.data}},
\code{\link{get.data.from.response}}, the TIRT-specific converters, or
the FCDCM-specific converters.
}
\details{
The package uses two layers of forced-choice representation:
\describe{
\item{\code{data}}{An \eqn{N \times B} matrix or data frame that is easy
to read and edit.  Rows are persons and columns are forced-choice
blocks.  A cell records the observed ordering with item labels separated
by \code{">"}.  For example, \code{"3>4>5"} means item 3 was ranked
above item 4, which was ranked above item 5.}
\item{\code{block.items}}{A list of length \eqn{B}.  Element \eqn{b}
gives the item labels appearing in column \eqn{b} of \code{data}.  This
object is the bridge between column order in the data file and item rows
in model parameter matrices such as \code{Q.matrix}.}
\item{\code{response}}{The compact numeric representation consumed by
model fitting code.  Its meaning depends on the model family.  For
FCGGUM, FCMIRT, and FCGDINA, \code{response} is an \eqn{N \times B}
matrix of 1-based pattern indices.  For TIRT, \code{response} is an
\eqn{N \times P} matrix of pairwise binary outcomes.  For FCDCM,
\code{response} is an \eqn{N \times B} binary matrix for two-item
blocks.}
}
}
\section{Choosing a Converter}{

\describe{
\item{FCGGUM, FCMIRT, and FCGDINA}{Use
\code{\link{get.response.from.data}} to convert character strings to
pattern indices, and \code{\link{get.data.from.response}} for the
inverse conversion.}
\item{TIRT}{Use \code{\link{get.response.from.data.TIRT}} and
\code{\link{get.data.from.response.TIRT}}.  TIRT expands each block to
pairwise comparisons, so its \code{response} columns are pairs rather
than block-level pattern numbers.}
\item{FCDCM}{Use \code{\link{get.response.from.data.FCDCM}} and
\code{\link{get.data.from.response.FCDCM}}.  Each block has exactly two
statements, so \code{response = 1} means the first statement in
\code{block.items[[b]]} was preferred and \code{response = 0} means the
second statement was preferred.}
\item{Recovering \code{block.items}}{Use
\code{\link{get.block.items.from.data}} for generic multi-item
forced-choice data and \code{\link{get.block.items.from.data.FCDCM}}
for FCDCM pair data.  Recovery is only possible for item labels that
actually appear in the observed strings.}
}
}

\section{Core Conventions}{

\itemize{
\item The \eqn{b}th element of \code{block.items} always corresponds to
the \eqn{b}th column of \code{data} or \code{response}.
\item \code{fc.type = "RANK"} expects a complete ranking such as
\code{"3>4>5"} for a three-item block.
\item \code{fc.type = "MOLE"} expects a most-least string such as
\code{"3>5"}: item 3 is most preferred and item 5 is least preferred.
\item \code{fc.type = "PICK"} expects a single most-preferred item such
as \code{"3"}.
\item Generic FC pattern-index responses are 1-based.  They are row
numbers in the deterministic pattern matrix generated internally; see
\code{\link{generate_fc_permutation_patterns}} for the public pattern
generator.
\item TIRT data should use consecutive item labels across blocks, for
example \code{list(1:2, 3:5, 6:8)}.  This is the format returned by
\code{\link{sim.data.TIRT}} and expected by the TIRT conversion
routines.
}
}

\examples{
# Generic FC conversion: response is a block-level pattern index.
block.items <- list(1:2, 3:5)
dat <- data.frame(
  block.1 = c("1>2", "2>1"),
  block.2 = c("3>4>5", "5>4>3"),
  stringsAsFactors = FALSE
)
resp <- get.response.from.data(dat, block.items, fc.type = "RANK")
resp
get.data.from.response(resp, block.items, fc.type = "RANK")

# TIRT conversion: response is pairwise 0/1 data.
tirt <- get.response.from.data.TIRT(dat, block.items, fc.type = "RANK")
tirt$response
get.data.from.response.TIRT(
  tirt$response, block.items, fc.type = "RANK",
  pairs.value = tirt$pairs.value
)

# FCDCM conversion: response is binary within each two-item block.
fcdcm.items <- list(c(1L, 3L), c(2L, 4L))
fcdcm.dat <- data.frame(
  block.1 = c("1>3", "3>1"),
  block.2 = c("2>4", "4>2"),
  stringsAsFactors = FALSE
)
fcdcm.resp <- get.response.from.data.FCDCM(fcdcm.dat, fcdcm.items)
fcdcm.resp$response
get.data.from.response.FCDCM(fcdcm.resp$response, fcdcm.items)

}
\seealso{
\code{\link{get.block.items.from.data}},
\code{\link{get.response.from.data}},
\code{\link{get.data.from.response}},
\code{\link{get.response.from.data.TIRT}},
\code{\link{get.data.from.response.TIRT}},
\code{\link{get.response.from.data.FCDCM}},
\code{\link{get.data.from.response.FCDCM}}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/istem-helpers.R
\name{resolve_model_type}
\alias{resolve_model_type}
\title{Resolve IRT model type with alias support}
\usage{
resolve_model_type(model)
}
\arguments{
\item{model}{Character scalar; the model identifier.}
}
\value{
Lower-case internal model code.
}
\description{
Accepts both internal codes (\code{"m1pl"}, \code{"m2pl"},
\code{"m3pl"}, \code{"m4pl"}) and user-friendly aliases
(\code{"Rasch"}, \code{"1PL"}, \code{"2PL"}, \code{"3PL"},
\code{"4PL"}).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3fitted.R
\name{residuals}
\alias{residuals}
\alias{residuals.MIRT}
\alias{residuals.MGPCM}
\alias{residuals.MGGUM}
\alias{residuals.FCMIRT}
\alias{residuals.FCDCM}
\alias{residuals.FCGDINA}
\alias{residuals.FCGGUM}
\alias{residuals.TIRT}
\title{S3 Methods: residuals}
\usage{
\method{residuals}{MIRT}(object, type = c("raw", "pearson", "deviance"), ...)

\method{residuals}{MGPCM}(object, type = c("raw", "pearson", "deviance"), ...)

\method{residuals}{MGGUM}(object, type = c("raw", "pearson", "deviance"), ...)

\method{residuals}{FCMIRT}(object, type = c("raw", "pearson", "deviance"), ...)

\method{residuals}{FCDCM}(object, type = c("raw", "pearson", "deviance"), ...)

\method{residuals}{FCGDINA}(object, type = c("raw", "pearson", "deviance"), ...)

\method{residuals}{FCGGUM}(object, type = c("raw", "pearson", "deviance"), ...)

\method{residuals}{TIRT}(object, type = c("raw", "pearson", "deviance"), ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{type}{Character; \code{"raw"} (default), \code{"pearson"}, or
\code{"deviance"}.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
A numeric matrix with the same dimensions as the corresponding
\code{fitted} output.
}
\description{
Compute residuals from fitted \pkg{ForceChoice} model objects.
Supported types are raw (response), Pearson, and deviance residuals.
}
\details{
Residual types follow standard GLM conventions for binary responses:
\itemize{
\item \strong{raw}: \eqn{e_{ij} = y_{ij} - \hat{p}_{ij}}
\item \strong{Pearson}:
\eqn{r_{ij} = e_{ij} / \sqrt{\hat{p}_{ij}(1-\hat{p}_{ij})}}
\item \strong{deviance}:
\eqn{d_{ij} = \mathrm{sign}(e_{ij}) \sqrt{2\left[y_{ij}\log\frac{y_{ij}}{\hat{p}_{ij}} + (1-y_{ij})\log\frac{1-y_{ij}}{1-\hat{p}_{ij}}\right]}}
}
For polytomous and forced-choice models, residuals are computed on a
category-indicator matrix. This treats ordered categories and nominal
forced-choice response patterns as multinomial outcomes rather than as
numeric scores.
}
\section{Methods (by class)}{
\itemize{
\item \code{residuals(MIRT)}: Residuals for \code{MIRT} objects.

\item \code{residuals(MGPCM)}: Residuals for \code{MGPCM} objects.

\item \code{residuals(MGGUM)}: Residuals for \code{MGGUM} objects.

\item \code{residuals(FCMIRT)}: Residuals for \code{FCMIRT} objects.

\item \code{residuals(FCDCM)}: Residuals for \code{FCDCM} objects.

\item \code{residuals(FCGDINA)}: Residuals for \code{FCGDINA} objects.

\item \code{residuals(FCGGUM)}: Residuals for \code{FCGGUM} objects.

\item \code{residuals(TIRT)}: Residuals for \code{TIRT} objects.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.MGGUM.R
\name{fit.MGGUM}
\alias{fit.MGGUM}
\title{Fit the Multidimensional Generalized Graded Unfolding Model (MGGUM)}
\usage{
fit.MGGUM(
  data,
  D = NULL,
  Q.matrix = NULL,
  length.poly = NULL,
  method = c("iStEM", "stan"),
  control.model = NULL,
  control.method = NULL
)
}
\arguments{
\item{data}{An \eqn{N \times I} matrix of integer responses
coded \eqn{0, 1, \dots, K_i-1}.}

\item{D}{Integer; number of latent dimensions (\eqn{D \ge 1}).
Default is \code{2}.}

\item{Q.matrix}{An optional \eqn{I \times D} matrix with entries
\eqn{-1, 0, 1}. See section \strong{Q-matrix sign convention}.
Default generates a random signed matrix.}

\item{length.poly}{Integer vector (length \eqn{I}) or scalar giving the
number of categories per item. If \code{NULL}, inferred from the
observed response maxima.}

\item{method}{Estimation method: \code{"iStEM"} (default) or
\code{"stan"}.}

\item{control.model}{A named list of model-level hyperparameters for the
GGUM unfolding model. Supported entries:
\describe{
\item{\code{a.mu}, \code{a.sigma}}{Prior location and scale for
\eqn{\log a_{id}} (log-normal). Defaults: 0, 0.5.
The log-normal prior ensures positivity of discrimination
parameters. The zero mean-log encourages slopes near 1
unless the data strongly indicate otherwise.}
\item{\code{delta.pos.mu}, \code{delta.pos.sigma}}{Prior mean and
SD for \eqn{\delta_{id}} when \eqn{q_{id} = 1} (positive-side
item locations; normal). Defaults: 1, 0.5. These priors
regularize item locations on the positive side of each
dimension.}
\item{\code{delta.neg.mu}, \code{delta.neg.sigma}}{Prior mean and
SD for \eqn{\delta_{id}} when \eqn{q_{id} = -1} (negative-side
item locations; normal). Defaults: -1, 0.5. These priors
regularize item locations on the negative side of each
dimension.}
\item{\code{tau.mu}, \code{tau.sigma}}{Location and scale for the
log-normal prior on positive threshold magnitudes
\eqn{-\tau_{ik}} for \eqn{k \ge 1}. Defaults: 0, 0.5.
The thresholds satisfy
\eqn{\tau_{i1} < \tau_{i2} < \dots < \tau_{i,K_i-1} < 0}
with \eqn{\tau_{i0} = 0} fixed for identification.}
\item{\code{theta.mu}}{Prior mean vector for
\eqn{\boldsymbol{\theta}_j}. Default: \code{rep(0, D)}.}
\item{\code{L}}{Theta grid size per dimension for marginal
log-likelihood computation and iStEM block Gibbs sampling.
Default adapts to \eqn{D}.}
\item{\code{theta.lower}, \code{theta.upper}}{Bounds for the
theta grid used by marginal log-likelihood computation and
iStEM block Gibbs sampling. Defaults: -6, 6.}
}}

\item{control.method}{A named list of method-specific tuning parameters.
Common entries (used by both Stan and iStEM):
\describe{
\item{\code{cores}}{Number of CPU cores for parallel chains
(Stan) or ignored (iStEM). Default: the number of
\code{chains}.}
\item{\code{vis}}{Logical; if \code{TRUE} (default), prints progress
information to the console.}
\item{\code{seed}}{Random seed for reproducibility.
Default: a random integer.}
}
Stan-specific entries:
\describe{
\item{\code{chains}}{Number of MCMC chains (default: 2).}
\item{\code{iter}}{Total iterations per chain (default: 5000).}
\item{\code{warmup}}{Warmup/burn-in iterations per chain
(default: \code{iter / 2}).}
\item{\code{thin}}{Thinning interval (default: 1).}
\item{\code{init}}{Initial values: \code{"random"} (default)
for uniform(-2, 2) initialization, or a list of initial
values per chain.}
\item{\code{algorithm}}{MCMC algorithm: \code{"HMC"} (default),
\code{"HMC"}, or \code{"Fixed_param"}.}
\item{\code{adapt_delta}}{Target average acceptance probability
(NUTS; default: 0.95). Values closer to 1 reduce step size
and improve sampling for difficult posteriors.}
\item{\code{max_treedepth}}{Maximum tree depth (NUTS; default: 10).
Increase if "max treedepth exceeded" warnings appear.}
\item{\code{stepsize}}{Initial step size for the leapfrog
integrator (auto-tuned by Stan if not set).}
\item{\code{int_time}}{Total integration time for HMC trajectories
(only when \code{algorithm = "HMC"}).}
\item{\code{metric}}{Mass matrix type: \code{"unit_e"},
\code{"diag_e"} (default), or \code{"dense_e"}.}
\item{\code{adapt_engaged}}{Logical; if \code{TRUE} (default),
warmup adaptation is enabled.}
\item{\code{adapt_init_buffer}, \code{adapt_term_buffer},
\code{adapt_window}}{Warmup adaptation scheduling parameters
(defaults: 25, 50, 25).}
}
iStEM-specific entries:
\describe{
\item{\code{M}}{Number of burn-in batches retained for Geweke
convergence diagnosis (default: 10; must be \eqn{\ge 2}).}
\item{\code{B}}{Batch size: MCMC iterations per batch (default: 20).}
\item{\code{burnin.maxitr}}{Maximum burn-in batches (default: 100).}
\item{\code{maxitr}}{Maximum total batches (default: 2000).}
\item{\code{eps1}}{Geweke z-score convergence threshold
(default: 1.5). The burn-in phase ends when
\eqn{\sum z^2 / K < \epsilon_1} or the MC error criterion
is satisfied.}
\item{\code{eps2}}{Monte Carlo error tolerance (default: 0.4).
The algorithm continues until
\eqn{max(d_k \cdot N) < \epsilon_2}.}
\item{\code{frac1}, \code{frac2}}{Fractions for the Geweke
diagnostic (defaults: 0.1, 0.5).}
\item{\code{corr.optim.maxit}}{Maximum L-BFGS-B iterations for the
constrained unit-diagonal correlation update (default: 50).}
\item{\code{optim.maxit}}{Maximum L-BFGS-B iterations per item
during the item-parameter update (default: 50). Increase
if item optimization does not converge within this limit.}
\item{\code{fix.corr}}{Logical; if \code{TRUE}, the inter-trait
correlation matrix is fixed to the identity (default:
\code{FALSE}).}
\item{\code{estimate.se}}{Logical; if \code{TRUE} (default),
standard errors are computed from the final Monte Carlo
chain.}
\item{\code{delta.lower}}{Lower bound on \eqn{|\delta_{id}|}
(default: \code{1e-4}). Ensures item locations are bounded
away from zero for active dimensions.}
\item{\code{tau.gap.lower}, \code{tau.gap.upper}}{Lower and upper
bounds on the positive gaps used to construct ordered
thresholds. Defaults are 1e-4 and 6.}
}}
}
\value{
An object of class \code{"MGGUM"} containing:
\describe{
\item{\code{npar}}{Number of free parameters.}
\item{\code{method}}{\code{"stan"} or \code{"iStEM"}.}
\item{\code{theta}}{List with \code{est}, \code{se}, \code{Rhat}
(\eqn{N \times D}).}
\item{\code{par}}{List with \code{est}, \code{se}, \code{Rhat},
\code{free} (\eqn{I \times (2D + K_{max})}).
Columns: \code{a1..aD, delta1..deltaD, tau0..tau_{K_{max}-1}}.}
\item{\code{Corr}}{List with \code{est}, \code{se}, \code{Rhat}
(\eqn{D \times D}).}
\item{\code{length.poly}}{Per-item category counts.}
\item{\code{logLik}}{Marginal log-likelihood (class \code{"logLik"}).}
}
}
\description{
Fits a multidimensional generalization of the generalized graded unfolding
model (GGUM) to polytomous response data. Unlike dominance (cumulative)
IRT models, the MGGUM belongs to the ideal-point (unfolding) model family
where the probability of endorsement is highest when the person and item
locations match, decreasing as the person moves away in either direction.
Two estimation backends are provided: Bayesian MCMC (Stan) and iStEM.
}
\section{Model Specification}{


Let \eqn{Y_{ij} \in \{0, 1, \dots, K_i - 1\}} be the categorical response
of person \eqn{j} to item \eqn{i}, with \eqn{K_i \ge 2} categories.
Let \eqn{\boldsymbol{\theta}_j} denote the \eqn{D}-dimensional latent
trait vector.

The category response probability implemented by \code{model.MGGUM},
iStEM, and Stan is the distance-based MGGUM form. Define
\deqn{
  r_{ij} =
  \left[
    \sum_{d=1}^{D} a_{id}^2
    \left(\theta_{jd} - \delta_{id}\right)^2
  \right]^{1/2},
  \qquad
  S_i = \sum_{d=1}^{D} a_{id}.
}
Let \eqn{\tau_{i0}=0} and
\eqn{\psi_{ik}=S_i\sum_{v=0}^{k}\tau_{iv}}. With
\eqn{M_i=2K_i-1}, the probability of category
\eqn{k = 0,\dots,K_i-1} is
\deqn{
  P(Y_{ij} = k \mid \boldsymbol{\theta}_j) =
  \frac{
    \exp\{k r_{ij}-\psi_{ik}\} +
    \exp\{(M_i-k)r_{ij}-\psi_{ik}\}
  }{
    \sum_{r=0}^{K_i-1}
    \bigl[
      \exp\{r r_{ij}-\psi_{ir}\} +
      \exp\{(M_i-r)r_{ij}-\psi_{ir}\}
    \bigr]
  }.
}
Here \eqn{a_{id} \ge 0} are discrimination parameters,
\eqn{\delta_{id}} are item-location parameters (signed positive when
\eqn{q_{id} = 1},
negative when \eqn{q_{id} = -1}), and threshold parameters \eqn{\tau_{ik}}
satisfying \eqn{\tau_{i0} = 0} and
\eqn{\tau_{i1} < \tau_{i2} < \dots < \tau_{i,K_i-1} < 0}.

\strong{Q-matrix sign convention:}
\describe{
\item{\eqn{q_{id} = 1}}{Active dimension; \eqn{\delta_{id} > 0}
(item located on the positive side of dimension \eqn{d}).}
\item{\eqn{q_{id} = -1}}{Active dimension; \eqn{\delta_{id} < 0}
(item located on the negative side of dimension \eqn{d}).}
\item{\eqn{q_{id} = 0}}{Inactive dimension;
\eqn{a_{id} = \delta_{id} = 0}.}
}

The sign of \eqn{q_{id}} controls the item-location side, not the sign of
the slope. Both \eqn{a_{id}} and \eqn{|\delta_{id}|} reflect the item's
relevance to dimension \eqn{d}.
}

\section{Estimation Methods}{


\describe{
\item{\strong{Stan} (\code{method = "stan"}):}{
Full Bayesian inference via HMC. All parameters are jointly sampled
from the posterior distribution.}
\item{\strong{iStEM} (\code{method = "iStEM"}):}{
Improved Stochastic EM with finite-grid block Gibbs person-sampling
and item optimization via L-BFGS-B.
The threshold parameters \eqn{\tau_{ik}} are constrained to maintain
monotonicity during optimization.}
}
}

\examples{
sim <- sim.data.MGGUM(N = 20, I = 6, D = 2, length.poly = 4)
fit <- fit.MGGUM(sim$response, D = 2, method = "iStEM",
                 control.method = list(
                   vis = FALSE, seed = 123,
                   M = 2, B = 2, burnin.maxitr = 2,
                   maxitr = 3, eps1 = 10, eps2 = 10,
                   estimate.se = FALSE))
head(fit$theta$est)
gof <- get.fit.index(fit)
summary(gof)

}
\references{
Roberts, J. S., Donoghue, J. R., & Laughlin, J. E. (2000). A general item
response theory model for unfolding unidimensional polytomous responses.
\emph{Applied Psychological Measurement}, 24(1), 3--32.
\doi{10.1177/01466216000241001}

Usami, S. (2011). Generalized graded unfolding model with structural
equation for subject parameters. \emph{Japanese Psychological Research},
53(3), 221--232.
}
\seealso{
\code{\link{sim.data.MGGUM}}, \code{\link{get.fit.index.MGGUM}},
\code{\link{logLik.MGGUM}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{mgpcm_par_free_mask}
\alias{mgpcm_par_free_mask}
\title{Free-parameter mask for MGPCM models}
\usage{
mgpcm_par_free_mask(Q.matrix, length.poly, max_poly = max(length.poly))
}
\arguments{
\item{Q.matrix}{I x D Q-matrix.}

\item{length.poly}{Integer vector of category counts per item.}

\item{max_poly}{Maximum number of categories (default \code{max(length.poly)}).}
}
\value{
An I x (D + max_poly) logical matrix.
}
\description{
Free-parameter mask for MGPCM models
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-utils.R
\name{get_precedent_patterns}
\alias{get_precedent_patterns}
\title{Find precedent (subset) patterns for monotonicity enforcement}
\usage{
get_precedent_patterns(patterns)
}
\arguments{
\item{patterns}{A binary matrix of attribute mastery patterns
(\eqn{L \times K}).}
}
\value{
A list of length \eqn{L}; the \eqn{l}-th element is an integer
vector of row indices corresponding to patterns that are proper
subsets of pattern \eqn{l}.
}
\description{
For each attribute mastery pattern, identifies which other patterns are
proper subsets (i.e., mastering a subset of the attributes).  Used to
enforce the GDINA monotonicity constraint: mastering more attributes
should never decrease the success probability.
}
\details{
Equivalent to \code{preloclist} in the GDINA R package.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.data.FCDCM.R
\name{model.FCDCM}
\alias{model.FCDCM}
\title{Compute FCDCM Block Response Probabilities}
\usage{
model.FCDCM(zeta, par, patterns)
}
\arguments{
\item{zeta}{Binary statement-level mastery status matrix.}

\item{par}{Block parameter matrix. A \eqn{B \times 2} matrix with columns
\code{eta0} and \code{etaAB}. Equal-condensation probabilities
(\eqn{\zeta_{A_b}=\zeta_{B_b}}) are fixed at 0.5.}

\item{patterns}{Block statement-pair matrix. Column 1 is the first
statement \eqn{A_b}; column 2 is the second statement \eqn{B_b}.}
}
\value{
Matrix of probabilities for choosing the first statement in each
block:
\eqn{\eta_{0b}} if \eqn{\zeta_{A_b}<\zeta_{B_b}},
\eqn{0.5} if equal, and
\eqn{0.5+\eta_{ABb}} if \eqn{\zeta_{A_b}>\zeta_{B_b}}.
}
\description{
Compute FCDCM Block Response Probabilities
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3print.R
\name{print}
\alias{print}
\alias{print.good.of.fit}
\alias{print.summary.good.of.fit}
\alias{print.MIRT}
\alias{print.summary.MIRT}
\alias{print.MGPCM}
\alias{print.summary.MGPCM}
\alias{print.MGGUM}
\alias{print.summary.MGGUM}
\alias{print.FCMIRT}
\alias{print.summary.FCMIRT}
\alias{print.FCDCM}
\alias{print.summary.FCDCM}
\alias{print.FCGDINA}
\alias{print.summary.FCGDINA}
\alias{print.FCGGUM}
\alias{print.summary.FCGGUM}
\alias{print.TIRT}
\alias{print.summary.TIRT}
\alias{print.data.MIRT}
\alias{print.data.MGPCM}
\alias{print.data.MGGUM}
\alias{print.data.FCMIRT}
\alias{print.data.FCDCM}
\alias{print.data.FCGDINA}
\alias{print.data.FCGGUM}
\alias{print.data.TIRT}
\title{S3 Methods: print}
\usage{
\method{print}{good.of.fit}(x, digits = 4, ...)

\method{print}{summary.good.of.fit}(x, ...)

\method{print}{MIRT}(x, ...)

\method{print}{summary.MIRT}(x, ...)

\method{print}{MGPCM}(x, ...)

\method{print}{summary.MGPCM}(x, ...)

\method{print}{MGGUM}(x, ...)

\method{print}{summary.MGGUM}(x, ...)

\method{print}{FCMIRT}(x, ...)

\method{print}{summary.FCMIRT}(x, ...)

\method{print}{FCDCM}(x, ...)

\method{print}{summary.FCDCM}(x, ...)

\method{print}{FCGDINA}(x, ...)

\method{print}{summary.FCGDINA}(x, ...)

\method{print}{FCGGUM}(x, ...)

\method{print}{summary.FCGGUM}(x, ...)

\method{print}{TIRT}(x, ...)

\method{print}{summary.TIRT}(x, ...)

\method{print}{data.MIRT}(x, ...)

\method{print}{data.MGPCM}(x, ...)

\method{print}{data.MGGUM}(x, ...)

\method{print}{data.FCMIRT}(x, ...)

\method{print}{data.FCDCM}(x, ...)

\method{print}{data.FCGDINA}(x, ...)

\method{print}{data.FCGGUM}(x, ...)

\method{print}{data.TIRT}(x, ...)
}
\arguments{
\item{x}{An object of one of the following classes:
\itemize{
\item Fitted model objects: \code{\link[ForceChoice]{fit.MIRT}}
(\code{"MIRT"}), \code{\link[ForceChoice]{fit.MGPCM}}
(\code{"MGPCM"}), \code{\link[ForceChoice]{fit.MGGUM}}
(\code{"MGGUM"}), \code{\link[ForceChoice]{fit.FCMIRT}}
(\code{"FCMIRT"}), \code{\link[ForceChoice]{fit.FCDCM}}
(\code{"FCDCM"}), \code{\link[ForceChoice]{fit.FCGGUM}}
(\code{"FCGGUM"}), \code{\link[ForceChoice]{fit.TIRT}}
(\code{"TIRT"})
\item Goodness-of-fit objects: \code{\link[ForceChoice]{good.of.fit}}
(\code{"good.of.fit"})
\item Summary objects: \code{summary.MIRT}, \code{summary.MGPCM},
\code{summary.MGGUM}, \code{summary.FCMIRT}, \code{summary.FCDCM},
\code{summary.FCGGUM}, \code{summary.TIRT},
\code{summary.good.of.fit}
}}

\item{digits}{Number of decimal places for numeric output (default: 4).
Used by \code{print.good.of.fit}.}

\item{...}{Additional arguments passed to methods (currently ignored in
most cases).}
}
\value{
Invisibly returns the input object \code{x}. No data is modified.
}
\description{
Provides user-friendly, formatted console output for objects generated by
the \pkg{ForceChoice} package. This generic function dispatches to
class-specific methods that display concise summaries of fitted models,
simulation datasets, goodness-of-fit indices, and rotated solutions.
Designed for interactive use and quick diagnostic inspection.
}
\details{
Each method produces a structured, human-readable summary optimized for
its object type:

\describe{
\item{\strong{Fitted Model Objects}}{
Invokes \code{summary()} internally and prints comprehensive output
including:
\itemize{
\item Model call and configuration (family, method, dimensions)
\item Data characteristics (N, I or B, response type)
\item Fit statistics (LogLik, npar, AIC, BIC)
\item Factor correlation matrix
\item Person parameter summary (N x D)
\item Item / statement / block parameter summary
\item For FCDCM: higher-order delta parameters and classification
diagnostics
\item For TIRT: loading (lambda) parameter summary
\item Convergence diagnostics (algorithm, batches, latent grid length L)
}
}

\item{\strong{Goodness-of-Fit Objects (\code{good.of.fit})}}{
Delegates to \code{summary.good.of.fit} and prints:
\itemize{
\item Overview table (sample size, items, parameters, moments)
\item Limited-information absolute fit (M2, RMSEA, SRMSR)
\item Incremental / comparative fit (CFI, TLI, IFI)
\item Likelihood and information criteria (AIC, BIC, SABIC, CAIC)
\item Pseudo-\eqn{R^2} measures
}
}
}
}
\section{Methods (by class)}{
\itemize{
\item \code{print(good.of.fit)}: Print method for \code{good.of.fit} objects.
Delegates to \code{summary.good.of.fit} and prints formatted tables.

\item \code{print(summary.good.of.fit)}: Print method for \code{summary.good.of.fit} objects.
Displays overview, absolute-fit, comparative-fit, likelihood/IC,
and pseudo-\eqn{R^2} tables.

\item \code{print(MIRT)}: Print method for \code{MIRT} objects.
Multidimensional IRT (1PL--4PL) model summary.

\item \code{print(summary.MIRT)}: Print method for \code{summary.MIRT} objects.
Shows model config, data info, fit stats, factor correlations,
person/item parameter summaries, and convergence.

\item \code{print(MGPCM)}: Print method for \code{MGPCM} objects.
Multidimensional Generalized Partial Credit Model summary.

\item \code{print(summary.MGPCM)}: Print method for \code{summary.MGPCM} objects.
Shows polytomous model config, fit stats, factor correlations,
person/item parameter summaries, and convergence.

\item \code{print(MGGUM)}: Print method for \code{MGGUM} objects.
Multidimensional Generalized Graded Unfolding Model summary.

\item \code{print(summary.MGGUM)}: Print method for \code{summary.MGGUM} objects.
Shows unfolding model config, fit stats, factor correlations,
person/item parameter summaries, and convergence.

\item \code{print(FCMIRT)}: Print method for \code{FCMIRT} objects.
Forced-Choice Multidimensional IRT model summary.

\item \code{print(summary.FCMIRT)}: Print method for \code{summary.FCMIRT} objects.
Shows forced-choice model config with block info, fit stats,
factor correlations, person/statement parameter summaries.

\item \code{print(FCDCM)}: Print method for \code{FCDCM} objects.
Forced-Choice Diagnostic Classification Model summary with
higher-order trait, attribute profiles, and classification diagnostics.

\item \code{print(summary.FCDCM)}: Print method for \code{summary.FCDCM} objects.
Shows higher-order delta parameters, block eta parameters,
attribute classification summary, and convergence.

\item \code{print(FCGDINA)}: Print method for \code{FCGDINA} objects.
Forced-Choice GDINA model summary.

\item \code{print(summary.FCGDINA)}: Print method for \code{summary.FCGDINA} objects.
Shows CDM type, forced-choice block info, fit stats, delta summaries,
posterior attribute probabilities, and convergence.

\item \code{print(FCGGUM)}: Print method for \code{FCGGUM} objects.
Forced-Choice Generalized Graded Unfolding Model summary.

\item \code{print(summary.FCGGUM)}: Print method for \code{summary.FCGGUM} objects.
Shows forced-choice unfolding model config, fit stats,
factor correlations, person/statement parameter summaries.

\item \code{print(TIRT)}: Print method for \code{TIRT} objects.
Thurstonian IRT for Forced-Choice model summary.

\item \code{print(summary.TIRT)}: Print method for \code{summary.TIRT} objects.
Shows pairwise-comparison model config, fit stats,
factor correlations, loading/person parameter summaries.

\item \code{print(data.MIRT)}: Print method for \code{data.MIRT} objects.

\item \code{print(data.MGPCM)}: Print method for \code{data.MGPCM} objects.

\item \code{print(data.MGGUM)}: Print method for \code{data.MGGUM} objects.

\item \code{print(data.FCMIRT)}: Print method for \code{data.FCMIRT} objects.

\item \code{print(data.FCDCM)}: Print method for \code{data.FCDCM} objects.

\item \code{print(data.FCGDINA)}: Print method for \code{data.FCGDINA} objects.

\item \code{print(data.FCGGUM)}: Print method for \code{data.FCGGUM} objects.

\item \code{print(data.TIRT)}: Print method for \code{data.TIRT} objects.

}}
\section{Output Conventions}{

\itemize{
\item Numeric values are rounded to the precision specified by
\code{digits} (default 4).
\item Parameter summary tables show Min, Q1, Median, Mean, Q3, Max
across items/statements for each parameter type.
\item Convergence diagnostics adapt to the estimation backend
(iStEM batch diagnostics vs. Stan HMC notes).
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{mirt_par_free_mask}
\alias{mirt_par_free_mask}
\title{Free-parameter mask for MIRT models}
\usage{
mirt_par_free_mask(model_type, Q.matrix)
}
\arguments{
\item{model_type}{Integer 2, 3, or 4.}

\item{Q.matrix}{I x D Q-matrix.}
}
\value{
An I x (D+3) logical matrix.
}
\description{
Free-parameter mask for MIRT models
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools-data.R
\name{get.block.items.from.data.FCDCM}
\alias{get.block.items.from.data.FCDCM}
\title{Extract Block Items from FCDCM Data}
\usage{
get.block.items.from.data.FCDCM(data)
}
\arguments{
\item{data}{An \eqn{N \times B} character matrix (or data frame) of
binary-preference strings.  Each cell must be of the form
\code{"a>b"} or \code{"b>a"} where \code{a} and \code{b} are global
statement indices.  Missing values are skipped.}
}
\value{
A list of length \eqn{B}.  Each element is a two-element integer
vector \code{c(a, b)} with \code{a < b}.
}
\description{
Recovers the two-item composition of each FCDCM block from a character
data matrix of \code{"a>b"} / \code{"b>a"} preference strings.  Because
every FCDCM block always compares exactly two statements, this function
only needs to inspect the first non-missing row of each column.
}
\details{
The returned item pair is sorted in ascending order.  For example, both
\code{"1>3"} and \code{"3>1"} imply \code{c(1L, 3L)} for that block.  This
sorted pair is then the default coding direction used by
\code{\link{get.response.from.data.FCDCM}}: response value 1 means the
first element of the pair was preferred, and response value 0 means the
second element was preferred.

This helper assumes the two compared statements are fixed within each
column.  If a column accidentally mixes different item pairs, only the first
parseable non-missing cell determines the returned pair, so such data should
be cleaned before conversion.
}
\examples{
dat <- data.frame(
  b1 = c("1>3", "3>1", "1>3"),
  b2 = c("2>4", "2>4", "4>2"),
  stringsAsFactors = FALSE
)
get.block.items.from.data.FCDCM(dat)
}
\seealso{
\code{\link{get.block.items.from.data}} for the generic multi-item
variant used by FCGGUM, FCMIRT, and TIRT.

\code{\link{get.response.from.data.FCDCM}} to convert FCDCM data strings
to a 0/1 response matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools-mcmc.R
\name{restore_mcmc_env}
\alias{restore_mcmc_env}
\title{Restore MCMC threading environment to its previous state}
\usage{
restore_mcmc_env(old)
}
\arguments{
\item{old}{A list returned by \code{setup_mcmc_env()}.}
}
\value{
Invisibly \code{NULL}.
}
\description{
Restore MCMC threading environment to its previous state
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools-data.R
\name{get.response.from.data.FCDCM}
\alias{get.response.from.data.FCDCM}
\title{Convert FCDCM Data Strings to a Binary Response Matrix}
\usage{
get.response.from.data.FCDCM(data, block.items)
}
\arguments{
\item{data}{An \eqn{N \times B} character matrix or data frame.  Each
cell must be of the form \code{"a>b"} or \code{"b>a"} where
\code{a} and \code{b} are the two item indices for that block.}

\item{block.items}{A list of length \eqn{B}.  Each element is an
integer vector of length 2 giving the two global statement indices
compared in that block, in the order used for the 0/1 coding.}
}
\value{
A named list with component \code{response}, an \eqn{N \times B}
integer matrix coded 0/1.
}
\description{
Parses FCDCM preference strings (\code{"a>b"} / \code{"b>a"}) into a
0/1 binary response matrix.  Each block compares exactly two items.
}
\details{
For each block, only two strings are valid: \code{"a>b"} and \code{"b>a"},
where \code{a = block.items[[b]][1]} and \code{b = block.items[[b]][2]}.
Leading and trailing whitespace is ignored.  Missing values or strings that
do not match the block-specific item pair produce an error, because FCDCM
fitting requires a complete binary response matrix.

If \code{block.items} was obtained from
\code{\link{get.block.items.from.data.FCDCM}}, the pair is sorted in
ascending order.  In that common workflow, response value 1 means the
smaller item index was preferred and response value 0 means the larger item
index was preferred.
}
\section{Response Coding}{

The output uses 0/1 coding: 1 indicates preference for the
first-named item in \code{block.items[[b]]}; 0 indicates preference
for the second-named item.  This coding is the native format expected
by the FCDCM model estimation functions.
}

\examples{
items <- list(c(1L, 3L), c(2L, 4L))
dat <- data.frame(
  b1 = c("1>3", "3>1", "1>3"),
  b2 = c("2>4", "2>4", "4>2"),
  stringsAsFactors = FALSE
)
get.response.from.data.FCDCM(dat, items)
get.data.from.response.FCDCM(
  get.response.from.data.FCDCM(dat, items)$response, items
)
}
\seealso{
\code{\link{get.data.from.response.FCDCM}} for the reverse conversion.

\code{\link{get.response.from.data}} for the generic multi-item
converter.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools-mcmc.R
\name{extract_theta_stan}
\alias{extract_theta_stan}
\title{Extract theta posterior summary from Stan MCMC output}
\usage{
extract_theta_stan(MCMC.obj, stan.sum, N, D, row_names = NULL)
}
\arguments{
\item{MCMC.obj}{Output of \code{rstan::extract()}.}

\item{stan.sum}{Output of \code{rstan::summary()$summary}.}

\item{N}{Number of persons.}

\item{D}{Number of dimensions.}

\item{row_names}{Row names for the output matrices (typically person IDs).}
}
\value{
A list with components \code{est}, \code{se}, and \code{Rhat}.
}
\description{
Extract theta posterior summary from Stan MCMC output
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{free_corr_npar}
\alias{free_corr_npar}
\title{Degrees of freedom for a correlation matrix}
\usage{
free_corr_npar(D, include_corr = TRUE)
}
\arguments{
\item{D}{Number of dimensions.}

\item{include_corr}{Logical; if \code{FALSE} returns 0.}
}
\value{
Integer, the number of free parameters in a D x D correlation matrix.
}
\description{
Degrees of freedom for a correlation matrix
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.FCMIRT.R
\name{fit.FCMIRT}
\alias{fit.FCMIRT}
\title{Fit the Forced-Choice Multidimensional IRT (FCMIRT) Model}
\usage{
fit.FCMIRT(
  data,
  model = "2PL",
  Q.matrix = NULL,
  block.items = NULL,
  D = NULL,
  fc.type = NULL,
  method = c("iStEM", "stan"),
  control.model = NULL,
  control.method = NULL
)
}
\arguments{
\item{data}{An \eqn{N \times B} forced-choice data object. Each entry is
either a ranking string (e.g., \code{"2>1>3"}) or a 1-based pattern
index into the block's ranking patterns. If pattern indices are used,
\code{block.items} must be supplied.}

\item{model}{MIRT model type: \code{"m1pl"}, \code{"m2pl"} (default),
\code{"m3pl"}, or \code{"m4pl"}.}

\item{Q.matrix}{An optional \eqn{I \times D} binary (0/1) matrix
indicating active item loadings.}

\item{block.items}{A list of length \eqn{B} where each element is an
integer vector of global item indices belonging to that block. If
\code{NULL} and \code{data} contains ranking strings, automatically
parsed from the data.}

\item{D}{Integer; number of latent dimensions. If \code{NULL}, inferred
from \code{Q.matrix}; if both are \code{NULL}, an error is raised.}

\item{fc.type}{Character vector (length 1 or \eqn{B}): \code{"RANK"}
(full ranking, default), \code{"MOLE"} (most-least), or \code{"PICK"}
(best only). Recycled to all blocks if scalar.}

\item{method}{Estimation method: \code{"iStEM"} (default) or
\code{"stan"}.}

\item{control.model}{A named list of model-level hyperparameters.
Supported entries:
\describe{
\item{\code{a.mu}, \code{a.sigma}}{Prior location and scale for
\eqn{\log a_{id}} (log-normal). Defaults: 0, 1.0.
\strong{Important:} The Stan default for \code{a.sigma}
is deliberately set to 1.0 (wider than the MIRT default of
0.25) because forced-choice blocks identify relative
utilities; tight common slope priors can spuriously
shrink within-block slopes toward equality, degrading
trait recovery. For iStEM, the prior defaults match the
standard MIRT values (\code{a.mu = 0.25}, \code{a.sigma = 0.25})
unless overridden.}
\item{\code{b.mu}, \code{b.sigma}}{Prior mean and SD for \eqn{b_i}
(normal). Defaults: 0, 1. Under the block zero-sum constraint
\eqn{\sum_{i \in \text{block } b} b_i = 0}, these priors
regularize the free difficulty parameters.}
\item{\code{c.mu}, \code{c.sigma}}{Uniform support bounds
\eqn{[c_{\min}, c_{\max}]} for lower-asymptote parameters
\eqn{c_i}. Defaults: 0, 0.35. Only used for 3PL/4PL models.}
\item{\code{d.mu}, \code{d.sigma}}{Uniform support bounds
\eqn{[d_{\min}, d_{\max}]} for upper-asymptote parameters
\eqn{d_i}. Defaults: 0.65, 1. Only used for 4PL models.}
\item{\code{theta.mu}}{Prior mean vector for
\eqn{\boldsymbol{\theta}_j}. Default: \code{rep(0, D)}.}
\item{\code{L}}{Theta grid size per dimension for marginal
log-likelihood computation and iStEM block Gibbs sampling.
Default adapts to \eqn{D}
(e.g., 61 for D = 1, 31 for D = 2, 15 for D = 3).}
\item{\code{theta.lower}, \code{theta.upper}}{Bounds for the
theta grid used by marginal log-likelihood computation and
iStEM block Gibbs sampling. Defaults: -6, 6.}
\item{\code{use.prior}}{Logical (iStEM only). If \code{FALSE}, the
item-prior penalty term is omitted from the item update,
recovering an approximate maximum-likelihood item step.
Default is \code{TRUE}.}
}}

\item{control.method}{A named list of method-specific tuning parameters.
Common entries (used by both Stan and iStEM):
\describe{
\item{\code{cores}}{Number of CPU cores for parallel chains
(Stan) or ignored (iStEM). Default: the number of
\code{chains}.}
\item{\code{vis}}{Logical; if \code{TRUE} (default), prints progress
information to the console.}
\item{\code{seed}}{Random seed for reproducibility.
Default: a random integer.}
}
Stan-specific entries:
\describe{
\item{\code{chains}}{Number of MCMC chains (default: 2).}
\item{\code{iter}}{Total iterations per chain (default: 5000).}
\item{\code{warmup}}{Warmup/burn-in iterations per chain
(default: \code{iter / 2}).}
\item{\code{thin}}{Thinning interval (default: 1).}
\item{\code{init}}{Initial values: \code{"random"} (default)
for uniform(-2, 2) initialization, or a list of initial
values per chain.}
\item{\code{algorithm}}{MCMC algorithm: \code{"HMC"} (default),
\code{"HMC"}, or \code{"Fixed_param"}.}
\item{\code{adapt_delta}}{Target average acceptance probability
(NUTS; default: 0.95). Forced-choice models often have
challenging posterior geometries; consider increasing to
0.9 or higher if divergent transitions appear.}
\item{\code{max_treedepth}}{Maximum tree depth (NUTS; default: 10).
Increase if "max treedepth exceeded" warnings appear.}
\item{\code{stepsize}}{Initial step size for the leapfrog
integrator (auto-tuned by Stan if not set).}
\item{\code{int_time}}{Total integration time for HMC trajectories
(only when \code{algorithm = "HMC"}).}
\item{\code{metric}}{Mass matrix type: \code{"unit_e"},
\code{"diag_e"} (default), or \code{"dense_e"}.}
\item{\code{adapt_engaged}}{Logical; if \code{TRUE} (default),
warmup adaptation is enabled.}
\item{\code{adapt_init_buffer}, \code{adapt_term_buffer},
\code{adapt_window}}{Warmup adaptation scheduling parameters
(defaults: 25, 50, 25).}
}
iStEM-specific entries:
\describe{
\item{\code{M}}{Number of burn-in batches retained for Geweke
convergence diagnosis (default: 10; must be \eqn{\ge 2}).}
\item{\code{B}}{Batch size: MCMC iterations per batch (default: 20).}
\item{\code{burnin.maxitr}}{Maximum burn-in batches (default: 100).}
\item{\code{maxitr}}{Maximum total batches (default: 2000).}
\item{\code{eps1}}{Geweke z-score convergence threshold for
burn-in (default: 1.5).}
\item{\code{eps2}}{Monte Carlo error tolerance for the final
chain (default: 0.4).}
\item{\code{frac1}, \code{frac2}}{Fractions for the Geweke
diagnostic (defaults: 0.1, 0.5).}
\item{\code{corr.optim.maxit}}{Maximum L-BFGS-B iterations for the
constrained unit-diagonal correlation update (default: 50).}
\item{\code{optim.maxit}}{Maximum L-BFGS-B iterations per item
(default: 50).}
\item{\code{fix.corr}}{Logical; fix correlations to identity
(default: \code{FALSE}).}
\item{\code{estimate.se}}{Logical; compute standard errors from
final MC chain (default: \code{TRUE}).}
\item{\code{a.lower}, \code{a.upper}}{Bounds on \eqn{a_{id}}
during optimization (defaults: 1e-4, 6).}
\item{\code{b.lower}, \code{b.upper}}{Bounds on \eqn{b_i}
during optimization (defaults: -6, 6).}
\item{\code{c.lower}, \code{c.upper}}{Bounds on \eqn{c_i} for
3PL/4PL models. Defaults from prior support.}
\item{\code{d.lower}, \code{d.upper}}{Bounds on \eqn{d_i} for
4PL models. Defaults from prior support.}
}}
}
\value{
An object of class \code{"FCMIRT"} containing:
\describe{
\item{\code{npar}}{Number of free parameters.}
\item{\code{method}}{\code{"stan"} or \code{"iStEM"}.}
\item{\code{theta}}{List with \code{est}, \code{se}, \code{Rhat}
(\eqn{N \times D}); person trait estimates.}
\item{\code{par}}{List with \code{est}, \code{se}, \code{Rhat},
\code{free} (\eqn{I \times (D+3)}); item parameters
\code{a1..aD, b, c, d}.}
\item{\code{Corr}}{List with \code{est}, \code{se}, \code{Rhat}
(\eqn{D \times D}).}
\item{\code{block.items}, \code{patterns}, \code{patterns.total},
\code{response}}{Block structure and response data.}
\item{\code{logLik}}{Marginal log-likelihood (class \code{"logLik"}).}
}
}
\description{
Fits a forced-choice variant of the multidimensional item response theory
model to comparative ranking data. In forced-choice (FC) formats,
respondents compare items within blocks rather than rating each item in
isolation. The FCMIRT model couples an item-level MIRT endorsement model
with a block-level ranking mechanism to recover latent trait estimates
from comparative responses.
}
\section{Model Specification}{


\strong{Item-level endorsement.}
Each statement \eqn{i = 1, \dots, I} is governed by a MIRT model
(1PL through 4PL). For a person with trait \eqn{\boldsymbol{\theta}_j},
the probability of endorsing statement \eqn{i} in isolation is:
\deqn{
  P_i(\boldsymbol{\theta}_j) =
  c_i + (d_i - c_i) \times
  \frac{1}{1 + \exp\bigl[-\bigl(\sum_{d=1}^{D} a_{id}\,\theta_{jd} - b_i\bigr)\bigr]}.
}

\strong{Block-level ranking.}
Let block \eqn{b = 1, \dots, B} consist of items
\eqn{\{i_1, i_2, \dots, i_{K_b}\}} and define the implemented
statement utility as the logit of the item-level endorsement probability,
\deqn{
  u_i(\boldsymbol{\theta}_j) =
  \log\left[
    \frac{P_i(\boldsymbol{\theta}_j)}
         {1-P_i(\boldsymbol{\theta}_j)}
  \right].
}
The probability that an examinee
produces the ranking \eqn{i_{(1)} \succ i_{(2)} \succ \dots \succ i_{(K_b)}}
(read as "\eqn{i_{(1)}} is preferred over \eqn{i_{(2)}} over ...") is:
\deqn{
  P\bigl(i_{(1)} \succ \dots \succ i_{(K_b)} \mid \boldsymbol{\theta}_j\bigr) =
  \prod_{m=1}^{K_b-1}
  \frac{
    \exp\{u_{i_{(m)}}(\boldsymbol{\theta}_j)\}
  }{
    \sum_{r=m}^{K_b} \exp\{u_{i_{(r)}}(\boldsymbol{\theta}_j)\}
  }.
}
The Stan code evaluates the same choice kernel as
\eqn{\log P_i + \sum_{h \ne i}\log(1-P_h)} within each remaining set,
which is algebraically equivalent to a softmax over
\eqn{\mathrm{logit}(P_i)}.

\strong{Partial rankings (MOLE / PICK).} For MOLE (most-least) data, only
the best and worst items in each block are identified; for PICK data,
only the best item. The probability of a partial ranking is obtained by
marginalizing (summing) the full-ranking probabilities over all
completions consistent with the partial constraint; the C++ probability
helper returns probabilities normalized over the observed RANK/MOLE/PICK
patterns for each block.

\strong{Identification constraint (2PLM-RANK).} Block intercepts
\eqn{b_i} are constrained to sum to zero within each block:
\deqn{
  \sum_{i \in \text{block } b} b_i = 0, \qquad b = 1, \dots, B,
}
so that only \eqn{K_b - 1} difficulties are freely estimated per block.
}

\section{Estimation Methods}{


\describe{
\item{\strong{Stan} (\code{method = "stan"}):}{
Full Bayesian inference via HMC. The forced-choice likelihood is
evaluated over the ranking pattern probabilities. By default,
\code{a.sigma} is set to \code{1.0} (wider prior) to avoid spurious
shrinkage of within-block slopes toward equality.}
\item{\strong{iStEM} (\code{method = "iStEM"}):}{
Improved Stochastic EM with finite-grid block Gibbs person-sampling
and item optimization (L-BFGS-B). Block-level identification
constraints are enforced during each item-parameter update.}
}
}

\examples{
# Simulate forced-choice data
sim <- sim.data.FCMIRT(N.person = 20, N.block = 3, I.block = 2,
                       D = 2, model = "m2pl", fc.type = "RANK")

# Fit via iStEM
fit <- fit.FCMIRT(sim$data, model = "m2pl",
                  Q.matrix = sim$Q.matrix,
                  block.items = sim$block.items,
                  D = 2, fc.type = "RANK",
                  method = "iStEM",
                  control.method = list(
                    vis = FALSE, seed = 123,
                    M = 2, B = 2, burnin.maxitr = 2,
                    maxitr = 3, eps1 = 10, eps2 = 10,
                    estimate.se = FALSE))

# Examine trait recovery
cor(fit$theta$est, sim$theta)

# Fit indices (nominal binary expansion)
gof <- get.fit.index(fit)
summary(gof)

}
\references{
Brown, A., & Maydeu-Olivares, A. (2011). Item response modeling of
forced-choice questionnaires. \emph{Educational and Psychological
Measurement}, 71(3), 460--502. \doi{10.1177/0013164410375112}

Luce, R. D. (1959). \emph{Individual choice behavior: A theoretical
analysis}. Wiley.

Plackett, R. L. (1975). The analysis of permutations. \emph{Journal of
the Royal Statistical Society: Series C}, 24(2), 193--202.
}
\seealso{
\code{\link{sim.data.FCMIRT}}, \code{\link{get.fit.index.FCMIRT}},
\code{\link{logLik.FCMIRT}}, \code{\link{fit.MIRT}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/good.of.fit.R
\name{good.of.fit}
\alias{good.of.fit}
\title{Goodness-of-Fit Indices for Latent Probability Models}
\usage{
good.of.fit(
  par.vec,
  loglik.fun,
  prob.fun,
  response,
  npar,
  pi,
  pi.fun = NULL,
  alpha = 0.05,
  response.type = c("auto", "binary", "polytomous", "nominal", "ordinary"),
  response.K = NULL,
  bivariate.groups = NULL,
  nominal.groups = NULL,
  exclusive.groups = NULL,
  n.boot = 1000L,
  ...
)
}
\arguments{
\item{par.vec}{Numeric vector of free model parameters.}

\item{loglik.fun}{Function returning the fitted model log-likelihood.}

\item{prob.fun}{Function returning conditional response probabilities at
the latent support points.}

\item{response}{An \eqn{N \times I} response matrix (binary, polytomous,
or nominal coding depending on \code{response.type}).}

\item{npar}{Integer; number of free parameters in the model.}

\item{pi}{Numeric vector of latent support weights (summing to 1).}

\item{pi.fun}{Optional function returning latent support weights given
\code{par.vec} and \code{...}. If \code{NULL}, \code{pi} is used as-is.}

\item{alpha}{Tail probability for the RMSEA confidence interval.
Default \code{0.05} yields a 90\% CI (two-sided \eqn{2\alpha}).
Use \code{0.025} for 95\% or \code{0.005} for 99\%.}

\item{response.type}{Character string specifying the response coding:
\code{"auto"} (default; detects from probability dimensions),
\code{"binary"} (each column is a binary indicator),
\code{"polytomous"} (stacked category probabilities, requires
\code{response.K}), or \code{"nominal"} (expanded binary indicators
with mutually exclusive categories within groups, requires
\code{nominal.groups}).}

\item{response.K}{Integer vector of length \eqn{I} giving the number of
categories for each response column. Required when
\code{response.type = "polytomous"}.}

\item{bivariate.groups}{Optional integer vector of length \eqn{I};
response column pairs within the same group are excluded from bivariate
limited-information moments. Useful for forced-choice or testlet
designs where within-block comparisons are structural zeros.}

\item{nominal.groups}{Integer vector of length matching the number of
expanded binary indicators for \code{response.type = "nominal"}.
Indicators sharing the same group value belong to the same original
nominal item/block.}

\item{exclusive.groups}{Deprecated alias for \code{nominal.groups}.}

\item{n.boot}{Integer; number of bootstrap replicates for bootstrapped
diagnostics. Default \code{1000}; set to \code{0} to skip.}

\item{...}{Additional arguments passed to \code{loglik.fun},
\code{prob.fun}, and \code{pi.fun}.}
}
\value{
An object of class \code{"good.of.fit"} (a list). Key components:
\describe{
\item{\code{LogLik}, \code{deviance}}{Log-likelihood and deviance.}
\item{\code{npar}, \code{N}, \code{nitems}}{Parameter count, sample
size, number of items/indicators.}
\item{\code{AIC}, \code{AICc}, \code{BIC}, \code{CAIC}, \code{SABIC},
\code{HQIC}}{Information criteria.}
\item{\code{M2.value}, \code{df}, \code{p.value}}{M2 statistic, degrees
of freedom, asymptotic p-value.}
\item{\code{RMSEA2}, \code{RMSEA.lower}, \code{RMSEA.upper},
\code{RMSEA.CI.level}, \code{RMSEA.p.close}}{RMSEA estimate,
confidence interval, interval level, close-fit test p-value.}
\item{\code{SRMSR}, \code{RMSR}}{Standardized and raw root mean
square residuals.}
\item{\code{CFI}, \code{TLI}, \code{IFI}}{Incremental fit indices.}
\item{\code{McFadden.R2}, \code{Nagelkerke.R2}, \code{CoxSnell.R2},
\code{AldrichNelson.R2}, \code{VeallZimmermann.R2}}{Pseudo-\eqn{R^2}
measures.}
\item{\code{residuals}, \code{residuals.standardized},
\code{correlation.residuals}}{Residual vectors.}
\item{\code{moments.observed}, \code{moments.predicted}}{First- and
second-order marginal moments.}
\item{\code{q3.mean}, \code{q3.abs.max}, \code{q3.adjusted.abs.max},
\code{q3.p95}}{Yen's Q3 local dependence statistics.}
\item{\code{mean.max.posterior}, \code{median.max.posterior},
\code{min.max.posterior}}{Posterior classification diagnostics.}
\item{\code{null.LogLik}, \code{null.M2.value}, \code{null.df}}{Null
(independence) model diagnostics.}
\item{\code{LRT}, \code{LRT.df}, \code{LRT.p}}{Likelihood ratio test
against the null model.}
}
}
\description{
Computes a comprehensive suite of model-fit diagnostics for probability
models used in IRT, CDM, latent class, and forced-choice applications.
Indices include likelihood-based information criteria, limited-information
goodness-of-fit statistics (M2, RMSEA, SRMSR), incremental/comparative fit
indices (CFI, TLI), pseudo-\eqn{R^2} measures, residual diagnostics,
local-dependence statistics (Yen's Q3), and posterior classification
diagnostics.
}
\section{Likelihood and Information Criteria}{


Let \eqn{\ell = \log L(\hat{\boldsymbol{\psi}})} be the maximized
log-likelihood, \eqn{N} the sample size, and \eqn{p} the number of free
parameters (\code{npar}).

\strong{Deviance:}
\deqn{D = -2\ell}

\strong{AIC} (Akaike, 1974):
\deqn{\text{AIC} = D + 2p}

\strong{AICc} (Hurvich & Tsai, 1989), small-sample correction:
\deqn{\text{AICc} = \text{AIC} + \frac{2p(p+1)}{N - p - 1},
      \quad N > p + 1}

\strong{BIC} (Schwarz, 1978):
\deqn{\text{BIC} = D + p \log N}

\strong{CAIC} (Bozdogan, 1987), consistent AIC:
\deqn{\text{CAIC} = D + p(\log N + 1)}

\strong{SABIC} (Sclove, 1987), sample-size adjusted BIC:
\deqn{\text{SABIC} = D + p \log\bigl(\frac{N + 2}{24}\bigr)}

\strong{HQIC} (Hannan & Quinn, 1979):
\deqn{\text{HQIC} = D + 2p \log(\log N), \quad N > e}
}

\section{Limited-Information M2 Family}{


The limited-information framework (Maydeu-Olivares & Joe, 2005, 2006) uses
first- and second-order marginal moments rather than the full
\eqn{I}-way contingency table. This is essential for sparse high-dimensional
categorical responses where full-information tests break down.

Let \eqn{\mathbf{m}(\boldsymbol{\psi})} be the vector of model-implied
univariate and bivariate moments (dimension \eqn{M}), and let
\eqn{\hat{\boldsymbol{\Xi}}_2} be their asymptotic covariance matrix
under the model.

\strong{M2 statistic:}
\deqn{
  M_2 = N\,
  \mathbf{e}'\hat{\mathbf{C}}_2\mathbf{e},
}
where \eqn{\mathbf{e} = \mathbf{p} - \boldsymbol{\pi}(\hat{\boldsymbol{\psi}})}
is the vector of marginal residuals, and
\eqn{\hat{\mathbf{C}}_2 = \boldsymbol{\Delta}_c(\boldsymbol{\Delta}_c'
\hat{\boldsymbol{\Xi}}_2 \boldsymbol{\Delta}_c)^{-1}\boldsymbol{\Delta}_c'}
with \eqn{\boldsymbol{\Delta}_c} the orthogonal complement of the Jacobian
matrix \eqn{\boldsymbol{\Delta} = \partial\boldsymbol{\pi}/\partial\boldsymbol{\psi}'}.

Under correct model specification, \eqn{M_2 \sim \chi^2_{M - p}}
asymptotically.

\strong{RMSEA (Root Mean Square Error of Approximation):}
\deqn{
  \text{RMSEA} = \sqrt{\max\!\left(0,\; \frac{M_2 - df}{N \cdot df}\right)},
  \qquad df = M - p
}
Confidence intervals are obtained via non-central \eqn{\chi^2} inversion.
The close-fit test evaluates \eqn{H_0: \text{RMSEA} \le 0.05}.

\strong{SRMSR (Standardized Root Mean Square Residual):}
\deqn{
  \text{SRMSR} = \sqrt{
    \frac{1}{J} \sum_{i < j} (r_{ij}^{\text{obs}} - r_{ij}^{\text{model}})^2
  },
}
where \eqn{r_{ij}} are the observed and model-implied correlation residuals,
summed over \eqn{J} unique off-diagonal pairs.

\strong{McDonald's NCI (Non-Centrality Index):}
\deqn{
  \text{NCI} = \exp\!\left(-\frac{\max(M_2 - df, 0)}{2N}\right)
}
}

\section{Incremental / Comparative Fit Indices}{


\strong{CFI (Comparative Fit Index):}
\deqn{
  \text{CFI} = 1 - \frac{\max(M_2 - df, 0)}
                          {\max(M_2^{\text{null}} - df^{\text{null}}, M_2 - df, 0)},
}
where the null (independence) model has \eqn{M_2^{\text{null}}} with
\eqn{df^{\text{null}}} degrees of freedom.

\strong{TLI (Tucker--Lewis Index, a.k.a. NNFI):}
\deqn{
  \text{TLI} = \frac{M_2^{\text{null}} / df^{\text{null}} - M_2 / df}
                    {M_2^{\text{null}} / df^{\text{null}} - 1}
}

\strong{IFI (Incremental Fit Index, a.k.a. BFI):}
\deqn{
  \text{IFI} = \frac{M_2^{\text{null}} - M_2}{M_2^{\text{null}} - df}
}
}

\section{Pseudo-\eqn{R^2} Measures}{


\strong{McFadden's \eqn{R^2}:}
\deqn{R^2_{\text{McF}} = 1 - \frac{\ell}{\ell_0}}

\strong{McFadden's adjusted \eqn{R^2}:}
\deqn{R^2_{\text{McF,adj}} = 1 - \frac{\ell - p}{\ell_0}}

\strong{Cox--Snell \eqn{R^2}:}
\deqn{R^2_{\text{CS}} = 1 - \exp\!\left(\frac{2}{N}(\ell_0 - \ell)\right)}

\strong{Nagelkerke \eqn{R^2}:}
\deqn{R^2_{\text{N}} = \frac{R^2_{\text{CS}}}
                                {1 - \exp(2\ell_0 / N)}}

\strong{Aldrich--Nelson \eqn{R^2}:}
\deqn{R^2_{\text{AN}} = \frac{G^2}{G^2 + N},
      \quad G^2 = D_0 - D}

\strong{Veall--Zimmermann \eqn{R^2}:}
\deqn{R^2_{\text{VZ}} = R^2_{\text{AN}} \times
      \frac{-2\ell_0 + N}{-2\ell_0}}

where \eqn{\ell_0} is the null-model (independence) log-likelihood and
\eqn{D_0} the corresponding deviance.
}

\section{Yen's Q3 Local Dependence}{


For each pair of items \eqn{(i, k)}, the residual correlation is
\deqn{
  Q_{3,ik} = \text{Corr}(r_{ji}, r_{jk}),
  \qquad r_{ji} = Y_{ji} - E[Y_{ji} \mid \hat{\boldsymbol{\theta}}_j],
}
where residuals are computed from posterior expected scores. Adjusted
Q3 values subtract the mean Q3 across all pairs to center the
distribution (Christensen, Makransky, & Horton, 2017).
}

\section{Heuristic Guidelines}{


The \code{print.summary.good.of.fit()} method reports descriptive
recommended ranges based on common conventions. These are heuristics,
not decision rules; fit cutoffs depend on model family, dimensionality,
category sparsity, local dependence, and sample size.
\itemize{
\item \strong{RMSEA}: \eqn{\le 0.05} close fit; \eqn{\le 0.08} reasonable;
\eqn{\le 0.10} mediocre (Browne & Cudeck, 1993)
\item \strong{SRMSR}: \eqn{\le 0.08} acceptable (Hu & Bentler, 1999)
\item \strong{CFI / TLI}: \eqn{\ge 0.95} good; \eqn{\ge 0.90} acceptable
}
}

\references{
Akaike, H. (1974). A new look at the statistical model identification.
\emph{IEEE Transactions on Automatic Control}, 19, 716--723.

Bozdogan, H. (1987). Model selection and Akaike's Information Criterion
(AIC): The general theory and its analytical extensions.
\emph{Psychometrika}, 52, 345--370. \doi{10.1007/BF02294361}

Browne, M. W., & Cudeck, R. (1993). Alternative ways of assessing model
fit. In K. A. Bollen & J. S. Long (Eds.), \emph{Testing structural
equation models} (pp. 136--162). Sage.

Christensen, K. B., Makransky, G., & Horton, M. C. (2017). Critical
values for Yen's Q3. \emph{Applied Psychological Measurement}, 41,
178--194. \doi{10.1177/0146621616677520}

Hannan, E. J., & Quinn, B. G. (1979). The determination of the order of
an autoregression. \emph{Journal of the Royal Statistical Society:
Series B}, 41, 190--195.

Hu, L. T., & Bentler, P. M. (1999). Cutoff criteria for fit indexes in
covariance structure analysis. \emph{Structural Equation Modeling}, 6,
1--55. \doi{10.1080/10705519909540118}

Hurvich, C. M., & Tsai, C. L. (1989). Regression and time series model
selection in small samples. \emph{Biometrika}, 76, 297--307.
\doi{10.1093/biomet/76.2.297}

MacCallum, R. C., Browne, M. W., & Sugawara, H. M. (1996). Power
analysis and determination of sample size for covariance structure
modeling. \emph{Psychological Methods}, 1, 130--149.

Maydeu-Olivares, A., & Joe, H. (2005). Limited- and full-information
estimation and goodness-of-fit testing in 2^n contingency tables: A
unified framework. \emph{Journal of the American Statistical
Association}, 100, 1009--1020.

Maydeu-Olivares, A., & Joe, H. (2006). Limited information
goodness-of-fit testing in multidimensional contingency tables.
\emph{Psychometrika}, 71, 713--732. \doi{10.1007/s11336-005-1295-9}

McFadden, D. (1974). Conditional logit analysis of qualitative choice
behavior. In P. Zarembka (Ed.), \emph{Frontiers in econometrics}
(pp. 105--142). Academic Press.

Schwarz, G. (1978). Estimating the dimension of a model. \emph{The Annals
of Statistics}, 6, 461--464. \doi{10.1214/aos/1176344136}

Sclove, S. L. (1987). Application of model-selection criteria to some
problems in multivariate analysis. \emph{Psychometrika}, 52, 333--343.

Yen, W. M. (1984). Effects of local item dependence on the fit and
equating performance of the three-parameter logistic model. \emph{Applied
Psychological Measurement}, 8, 125--145. \doi{10.1177/014662168400800201}
}
\seealso{
\code{\link{get.fit.index}} for model-specific wrappers,
\code{\link{summary.good.of.fit}} for formatted fit tables,
\code{\link{print.good.of.fit}} for the print method.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.data.MGGUM.R
\name{sim.data.MGGUM}
\alias{sim.data.MGGUM}
\title{Simulate Data from the Multidimensional Generalized Graded Unfolding Model}
\usage{
sim.data.MGGUM(
  N = 500,
  I = 20,
  D = 2,
  length.poly = 5,
  Q.matrix = NULL,
  Corr = NULL
)
}
\arguments{
\item{N}{Integer; number of examinees (default: 500).}

\item{I}{Integer; number of items (default: 20).}

\item{D}{Integer; number of latent dimensions (default: 2).}

\item{length.poly}{Integer vector or scalar indicating the number of
categories for each item. A scalar is recycled to length \eqn{I}.}

\item{Q.matrix}{Optional \eqn{I \times D} matrix with values -1, 0, or 1.
\itemize{
\item \code{1}: active dimension with positive-side \code{delta}
\item \code{-1}: active dimension with negative-side \code{delta}
\item \code{0}: \code{a} and \code{delta} fixed to 0 (inactive dimension)
}
The sign controls the item-location side, not the sign of discrimination.
If \code{NULL}, a default matrix of randomly signed active entries is used.}

\item{Corr}{Optional \eqn{D \times D} correlation matrix. If \code{NULL},
the identity matrix is used.}
}
\value{
An object of class \code{"data.MGGUM"}, a list containing:
\describe{
\item{\code{data}, \code{response}}{\eqn{N \times I} integer response
matrices with categories coded from 0 to \eqn{K_i - 1}.}
\item{\code{theta}}{\eqn{N \times D} matrix of true latent traits.}
\item{\code{par}}{\eqn{I \times (2D + \max_i K_i)} matrix of GGUM item
parameters: discrimination columns, location columns, and stacked
threshold columns.}
\item{\code{probability}}{\eqn{N \times \sum_i K_i} matrix of stacked
category probabilities.}
\item{\code{Q.matrix}, \code{length.poly}, \code{Corr}}{Design matrix,
category counts, and latent correlation matrix used to generate
the data.}
\item{\code{N}, \code{I}, \code{D}, \code{call}}{Data-generating
metadata.}
}
}
\description{
Generates polytomous responses from the same distance-based MGGUM
probability function used by \code{\link{model.MGGUM}} and
\code{\link{fit.MGGUM}}. For each item, active discriminations are drawn
from \eqn{U(0.5, 2)}, active locations follow the sign of the Q-matrix,
\eqn{\tau_{i0}=0}, and the remaining \eqn{\tau} values are ordered
negative thresholds.
}
\section{Data Generation Process}{

\enumerate{
\item Draw latent traits from
\eqn{N_D(\mathbf{0}, \boldsymbol{\Sigma})}.
\item Generate discrimination, location, and threshold parameters under
the sign constraints encoded by \code{Q.matrix}.
\item Compute stacked category probabilities with
\code{\link{model.MGGUM}}.
\item Sample one ordinal response per person and item. A category can be
absent in a finite sample; \code{length.poly} retains the intended
support.
}
}

\examples{
set.seed(123)
sim <- sim.data.MGGUM(N = 20, I = 5, D = 2, length.poly = 4)
str(sim$response)
dim(sim$par)
table(sim$response[, 1])

}
\seealso{
\code{\link{fit.MGGUM}}, \code{\link{model.MGGUM}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.TIRT.R
\name{fit.TIRT}
\alias{fit.TIRT}
\title{Fit the Thurstonian Item Response Theory (TIRT) Model}
\usage{
fit.TIRT(
  data,
  Q.matrix,
  block.items = NULL,
  fc.type = NULL,
  method = c("iStEM", "stan"),
  control.model = NULL,
  control.method = NULL
)
}
\arguments{
\item{data}{An \eqn{N \times B} forced-choice data matrix with ranking
strings (e.g., \code{"2>1>3"}).}

\item{Q.matrix}{An \eqn{I \times D} matrix with entries \eqn{-1, 0, 1}.
Each row must have exactly one non-zero entry that assigns the statement
to a specific dimension with a sign indicating the scoring direction.}

\item{block.items}{A list of length \eqn{B} giving item indices in each
block. Parsed from \code{data} if \code{NULL}.}

\item{fc.type}{Forced-choice response type: \code{"RANK"} (default),
\code{"MOLE"}, or \code{"PICK"}.}

\item{method}{Estimation method: \code{"iStEM"} (default) or
\code{"stan"}.}

\item{control.model}{A named list of model-level hyperparameters for the
Thurstonian IRT model. Supported entries:
\describe{
\item{\code{lambda.alpha}, \code{lambda.beta}}{Lower and upper bounds
for free loading magnitudes \eqn{\lambda_i}. Defaults: 0.40,
0.90. Stan uses
\eqn{\lambda_i \sim U(\lambda_{\min}, \lambda_{\max})}, where
these two controls supply \eqn{\lambda_{\min}} and
\eqn{\lambda_{\max}};
iStEM uses the same values as optimization bounds unless
\code{lambda.lower} or \code{lambda.upper} is supplied in
\code{control.method}.}
\item{\code{psi.mu}, \code{psi.sigma}}{Prior mean and SD for
uniqueness standard deviations \eqn{\psi_i} (normal, truncated
to positive values). Defaults: 0.80, 0.30. The prior
\eqn{\psi_i \sim N_+(0.80, 0.30^2)} reflects the expectation
that uniqueness SDs are moderate in size. Note that the model
parameterizes the uniqueness variance \eqn{\psi_i^2}, so
the prior on the SD scale is transformed accordingly.}
\item{\code{gamma.mu}, \code{gamma.sigma}}{Prior mean and SD for
pairwise intercepts \eqn{\gamma_{ik}} (normal). Defaults: 0,
0.8. The skew-symmetry constraint \eqn{\gamma_{ik} = -\gamma_{ki}}
is automatically enforced. Unobserved pairwise comparisons
(in MOLE/PICK designs) have their \eqn{\gamma} parameters fixed
to \code{gamma.mu}.}
\item{\code{theta.mu}}{Mean vector for the iStEM normal kernel for
\eqn{\boldsymbol{\theta}_j}. Default: \code{rep(0, D)}. The
current Stan model accepts this value in its data list but
centers \eqn{\boldsymbol{\theta}_j} at zero.}
\item{\code{L}}{Theta grid size per dimension for marginal
log-likelihood computation and iStEM block Gibbs sampling.
Default adapts to \eqn{D}.}
\item{\code{theta.lower}, \code{theta.upper}}{Bounds for the
theta grid used by marginal log-likelihood computation and
iStEM block Gibbs sampling. Defaults: -6, 6.}
}}

\item{control.method}{A named list of method-specific tuning parameters.
Common entries (used by both Stan and iStEM):
\describe{
\item{\code{cores}}{Number of CPU cores for parallel chains
(Stan) or ignored (iStEM). Default: the number of
\code{chains}.}
\item{\code{vis}}{Logical; if \code{TRUE} (default), prints progress
information to the console.}
\item{\code{seed}}{Random seed for reproducibility.
Default: a random integer.}
}
Stan-specific entries:
\describe{
\item{\code{chains}}{Number of MCMC chains (default: 2).}
\item{\code{iter}}{Total iterations per chain (default: 5000).}
\item{\code{warmup}}{Warmup/burn-in iterations per chain
(default: \code{iter / 2}).}
\item{\code{thin}}{Thinning interval (default: 1).}
\item{\code{init}}{Initial values: \code{"random"} (default)
for uniform(-2, 2) initialization, or a list of initial
values per chain.}
\item{\code{algorithm}}{MCMC algorithm: \code{"HMC"} (default),
\code{"HMC"}, or \code{"Fixed_param"}.}
\item{\code{adapt_delta}}{Target average acceptance probability
(NUTS; default: 0.95). The TIRT model's pairwise probit
likelihood typically mixes well; however, for high-dimensional
models (\eqn{D > 5}) or models with many fixed parameters,
consider increasing to 0.9.}
\item{\code{max_treedepth}}{Maximum tree depth (NUTS; default: 10).
Increase if "max treedepth exceeded" warnings appear.}
\item{\code{stepsize}}{Initial step size for the leapfrog
integrator (auto-tuned by Stan if not set).}
\item{\code{int_time}}{Total integration time for HMC trajectories
(only when \code{algorithm = "HMC"}).}
\item{\code{metric}}{Mass matrix type: \code{"unit_e"},
\code{"diag_e"} (default), or \code{"dense_e"}.}
\item{\code{adapt_engaged}}{Logical; if \code{TRUE} (default),
warmup adaptation is enabled.}
\item{\code{adapt_init_buffer}, \code{adapt_term_buffer},
\code{adapt_window}}{Warmup adaptation scheduling parameters
(defaults: 25, 50, 25).}
}
iStEM-specific entries:
\describe{
\item{\code{M}}{Number of burn-in batches retained for Geweke
convergence diagnosis (default: 10; must be \eqn{\ge 2}).}
\item{\code{B}}{Batch size: MCMC iterations per batch (default: 20).}
\item{\code{burnin.maxitr}}{Maximum burn-in batches (default: 100).}
\item{\code{maxitr}}{Maximum total batches (default: 2000).}
\item{\code{eps1}}{Geweke z-score convergence threshold for
burn-in (default: 1.5).}
\item{\code{eps2}}{Monte Carlo error tolerance for the final
chain (default: 0.4).}
\item{\code{frac1}, \code{frac2}}{Fractions for the Geweke
diagnostic (defaults: 0.1, 0.5).}
\item{\code{corr.optim.maxit}}{Maximum L-BFGS-B iterations for the
constrained unit-diagonal correlation update (default: 50).}
\item{\code{optim.maxit}}{Maximum L-BFGS-B iterations per
parameter block during structural parameter updates
(default: 50). Applies to the optimization of \eqn{\lambda},
\eqn{\psi^2}, and \eqn{\gamma} parameters.}
\item{\code{fix.corr}}{Logical; if \code{TRUE}, the inter-trait
correlation matrix is fixed to the identity (default:
\code{FALSE}). Useful when orthogonal dimensions are
theoretically expected.}
\item{\code{estimate.se}}{Logical; if \code{TRUE} (default),
standard errors are computed from the final Monte Carlo
chain.}
\item{\code{lambda.lower}, \code{lambda.upper}}{Bounds on free
loading magnitudes during iStEM optimization. Defaults are
\code{lambda.alpha} and \code{lambda.beta} from
\code{control.model}.}
\item{\code{psi.lower}, \code{psi.upper}}{Bounds on uniqueness
standard deviations \eqn{\psi_i} during iStEM optimization
(defaults: 1e-4 and 5). The reported \code{par} matrix stores
\eqn{\psi_i^2}.}
\item{\code{gamma.lower}, \code{gamma.upper}}{Bounds on pairwise
intercepts \eqn{\gamma_{ik}} during iStEM optimization
(defaults: -6 and 6).}
}}
}
\value{
An object of class \code{"TIRT"} containing:
\describe{
\item{\code{npar}}{Number of free parameters.}
\item{\code{method}}{\code{"stan"} or \code{"iStEM"}.}
\item{\code{theta}}{List with \code{est}, \code{se}, \code{Rhat}
(\eqn{N \times D}).}
\item{\code{par}}{List with \code{est}, \code{se}, \code{Rhat},
\code{free} (\eqn{I \times (D + 1)}). Columns include the
loading \eqn{\lambda_i} on the design dimension and
\eqn{\psi_i^2}.}
\item{\code{Corr}}{List with \code{est}, \code{se}, \code{Rhat}
(\eqn{D \times D}).}
\item{\code{gamma.matrix}}{List with \code{est}, \code{se}, \code{Rhat}
(\eqn{I \times I}); pairwise intercept matrix.}
\item{\code{pairs.matrix}, \code{pairs.value}, \code{response},
\code{response.total}}{Pairwise and response data structures.}
\item{\code{logLik}}{Marginal log-likelihood (class \code{"logLik"}).}
}
}
\description{
Fits the Thurstonian IRT model for forced-choice data. Unlike the
FCMIRT/FCGGUM models that operate on the item-level endorsement
probabilities, the TIRT model works directly at the level of paired
comparisons: each pairwise comparison is treated as a binary outcome
governed by a latent difference process with a probit link.
}
\section{Model Specification}{


\strong{Pairwise utility difference.}
For a pair of statements \eqn{(i, k)} in block \eqn{b}, the latent
comparative judgment is:
\deqn{
  y_{j,ik}^* =
  -\gamma_{ik} +
  \lambda_i \sum_{d=1}^{D} q_{id}\,\theta_{jd} -
  \lambda_k \sum_{d=1}^{D} q_{kd}\,\theta_{jd} +
  \varepsilon_{j,i} - \varepsilon_{j,k},
}
where:
\itemize{
\item \eqn{\gamma_{ik}} is the pairwise intercept (unfolding threshold
parameter). Skew-symmetry requires \eqn{\gamma_{ik} = -\gamma_{ki}}.
\item \eqn{\lambda_i > 0} is the loading magnitude of statement
\eqn{i}; the sign of the statement direction is carried by
\eqn{q_{id}}.
\item \eqn{q_{id} \in \{-1, 0, 1\}} is the Q-matrix entry; each row
has exactly one non-zero entry.
\item \eqn{\varepsilon_{j,i} \sim N(0, \psi_i^2)} is the uniqueness
(error) of statement \eqn{i}.
}

\strong{Binary pairwise response.} The observed response is
\deqn{
  Y_{j,ik} = \begin{cases}
    1, & \text{if } y_{j,ik}^* \ge 0 \quad (\text{prefer } i \text{ over } k), \\
    0, & \text{otherwise}.
  \end{cases}
}

Under the normality of errors, the probability is:
\deqn{
  P(Y_{j,ik} = 1 \mid \boldsymbol{\theta}_j) =
  \Phi\!\left(
    \frac{
      -\gamma_{ik} + \lambda_i\,\mathbf{q}_i'\boldsymbol{\theta}_j -
      \lambda_k\,\mathbf{q}_k'\boldsymbol{\theta}_j
    }{
      \sqrt{\psi_i^2 + \psi_k^2}
    }
  \right),
}
where \eqn{\Phi(\cdot)} is the standard normal CDF.

\strong{Identification constraints.} Several standard constraints are
applied:
\itemize{
\item \strong{Case A} (\eqn{I_{\text{block}} = 2, D > 2}): All
\eqn{\psi_i^2 = 0.5} are fixed.
\item \strong{Case B} (\eqn{D = 2, I_{\text{block}} = 2}): The first
block's \eqn{\lambda_i} are fixed to 0.80, all
\eqn{\psi_i^2 = 0.5} are fixed.
\item \strong{General case}: The last item in each block has
\eqn{\psi_i^2 = 1.0} fixed. Pairwise \eqn{\gamma_{ik}}
parameters for comparisons that are never observed under the
supplied RANK/MOLE/PICK design are fixed to \code{gamma.mu}.
}

\strong{Correlation structure.} The inter-trait correlation matrix
\eqn{\boldsymbol{\Sigma}} is estimated as in the MIRT model. In the Stan
file, \eqn{\boldsymbol{\theta}_j} is centered at zero; the iStEM backend
uses \code{control.model$theta.mu} as the mean in the normal kernel.
}

\section{Estimation Methods}{


\describe{
\item{\strong{Stan} (\code{method = "stan"}):}{
Full Bayesian inference via HMC. The probit likelihood is evaluated
over all pairwise comparisons.}
\item{\strong{iStEM} (\code{method = "iStEM"}):}{
Improved Stochastic EM. Person sampling uses finite-grid block Gibbs;
structural parameters (\eqn{\lambda}, \eqn{\psi^2}, \eqn{\gamma})
updated via optimization or closed-form steps.}
}
}

\examples{
sim <- sim.data.TIRT(N.person = 20, N.block = 3, I.block = 2,
                     D = 2, fc.type = "RANK")
fit <- fit.TIRT(sim$data, Q.matrix = sim$Q.matrix,
                block.items = sim$block.items,
                fc.type = "RANK", method = "iStEM",
                control.method = list(
                  vis = FALSE, seed = 123,
                  M = 2, B = 2, burnin.maxitr = 2,
                  maxitr = 3, eps1 = 10, eps2 = 10,
                  estimate.se = FALSE))
head(fit$theta$est)
cor(fit$theta$est, sim$theta)
gof <- get.fit.index(fit)
summary(gof)

}
\references{
Brown, A., & Maydeu-Olivares, A. (2011). Item response modeling of
forced-choice questionnaires. \emph{Educational and Psychological
Measurement}, 71(3), 460--502. \doi{10.1177/0013164410375112}

Maydeu-Olivares, A., & Brown, A. (2010). Item response modeling of
paired comparison and ranking data. \emph{Multivariate Behavioral
Research}, 45(6), 935--974.

Thurstone, L. L. (1927). A law of comparative judgment.
\emph{Psychological Review}, 34(4), 273--286.
}
\seealso{
\code{\link{sim.data.TIRT}}, \code{\link{get.fit.index.TIRT}},
\code{\link{logLik.TIRT}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{mggum_par_free_mask}
\alias{mggum_par_free_mask}
\title{Free-parameter mask for MGGUM models}
\usage{
mggum_par_free_mask(Q.matrix, length.poly, max_poly = max(length.poly))
}
\arguments{
\item{Q.matrix}{I x D Q-matrix.}

\item{length.poly}{Integer vector of category counts per item.}

\item{max_poly}{Maximum number of categories.}
}
\value{
An I x (2D + max_poly) logical matrix.
}
\description{
Free-parameter mask for MGGUM models
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.data.TIRT.R
\name{sim.data.TIRT}
\alias{sim.data.TIRT}
\title{Simulate Data from the Thurstonian IRT (TIRT) Model}
\usage{
sim.data.TIRT(
  N.person = 1000,
  N.block = 10,
  I.block = 2,
  D = 3,
  fc.type = "RANK",
  control = NULL
)
}
\arguments{
\item{N.person}{Integer; number of persons (default: 1000).}

\item{N.block}{Integer; number of blocks (default: 10).}

\item{I.block}{Integer; items per block (default: 2).}

\item{D}{Integer; number of latent dimensions (default: 3, >= 2).}

\item{fc.type}{Character; \code{"RANK"} (default), \code{"MOLE"}, or
\code{"PICK"}.}

\item{control}{Optional list with entries \code{Q.matrix}, \code{block.items},
\code{lambda}, \code{psi2}, \code{gamma.matrix}, \code{gamma.item},
\code{Corr}.}
}
\value{
An object of class \code{"data.TIRT"} containing \code{data},
\code{response}, \code{theta}, \code{par}, \code{Q.matrix},
\code{block.items}, \code{Corr}, \code{gamma.matrix}, \code{lambda},
\code{psi2}, \code{pairs.value}, and data-generating arguments.
}
\description{
Generates forced-choice pairwise comparison data under the Thurstonian
IRT framework. Each statement loads on exactly one latent dimension;
pairwise latent differences with probit link determine binary outcomes.
Supports RANK, MOLE, and PICK response types. For each pair \eqn{(i,k)},
the simulated comparison uses
\eqn{-\gamma_{ik} + \lambda_i q_i'\theta_j -
\lambda_k q_k'\theta_j + \epsilon_i - \epsilon_k}, with
\eqn{\epsilon_i \sim N(0,\psi_i^2)}.
}
\examples{
set.seed(123)
sim <- sim.data.TIRT(N.person = 20, N.block = 3, I.block = 2,
                     D = 2, fc.type = "RANK")
str(sim$data)
head(sim$response)
dim(sim$Q.matrix)

}
\seealso{
\code{\link{fit.TIRT}}, \code{\link{model.TIRT}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-utils.R
\name{generate_fc_permutation_patterns}
\alias{generate_fc_permutation_patterns}
\title{Generate forced-choice permutation patterns}
\usage{
generate_fc_permutation_patterns(block.items, fc.type)
}
\arguments{
\item{block.items}{List of integer vectors (item indices per block).}

\item{fc.type}{Character vector of length \code{length(block.items)}
with values \code{"RANK"}, \code{"MOLE"}, or \code{"PICK"}.}
}
\value{
A named list with components \code{patterns.total} (list of
full-ranking matrices) and \code{patterns} (list of observed-pattern
matrices).
}
\description{
Produces the full-ranking patterns (\code{patterns.total}) and the
observed patterns (\code{patterns}) for each block, based on the
forced-choice response type.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.data.FCGDINA.R
\name{compute_fcgdina_item_prob}
\alias{compute_fcgdina_item_prob}
\title{Compute FCGDINA item-level endorsement probabilities}
\usage{
compute_fcgdina_item_prob(
  alpha,
  delta.list,
  design.matrix.list,
  Q.matrix,
  gs = NULL,
  eps = 1e-12,
  link = c("identity", "logit")
)
}
\arguments{
\item{alpha}{\eqn{N \times D} binary matrix of attribute profiles.}

\item{delta.list}{List of per-item delta parameter vectors.}

\item{design.matrix.list}{List of per-item design matrices.}

\item{Q.matrix}{\eqn{I \times D} binary Q-matrix.}

\item{gs}{Optional \eqn{I \times 2} matrix of lower and upper probability
bounds. If supplied, computed probabilities are clamped to
\code{gs[i, 1]} and \code{gs[i, 2]} for item \eqn{i}.}

\item{eps}{Numeric probability clamp used when \code{gs = NULL}. Default
\code{1e-12}.}

\item{link}{Item probability link. \code{"identity"} uses
\eqn{p = X\delta}; \code{"logit"} uses
\eqn{p = \mathrm{logit}^{-1}(X\delta)}. The latter is used by the Stan
FCGDINA backend.}
}
\value{
An \eqn{N \times I} matrix of item endorsement probabilities.
}
\description{
Computes statement endorsement probabilities for supplied attribute
profiles under an FCGDINA item-level CDM parameterization.
}
\details{
The function maps each full attribute pattern in \code{alpha} to the
reduced pattern required by each item-specific Q-vector, evaluates the
corresponding design row, and multiplies by that item's delta vector.
This function does not sample responses.
}
\seealso{
\code{\link{model.FCGDINA}}, \code{\link{sim.data.FCGDINA}}
}
\keyword{internal}
